package org.eso.ohs.phase2.apps.masktracker;

import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import org.eso.ohs.core.utilities.FileIOUtils;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.persistence.Config;
/**
 * Abstract superclass for Mask actions, eg manufacturing, insertion
 * and discarding.
 * 
 * <p>
  * $Id: MaskActionGenerator.java,v 1.18 2007/10/11 11:42:56 pnunes Exp $
 * <br>
 * @author Bob Kemp
 */
abstract public class MaskActionGenerator {

    private static String obrepDbName_ =
        Config.getCfg().getDbName(ObservationBlock.getSuffix());

    /*   
    * Generate a mask order
    * @param maxNumOBs 
    * @return The order as a  string
    */

    abstract public String generateOrder(int maxNumOBs);

    /**
     * Receive a report 
     * @param report Receive a report 
     */

    abstract public void receiveReport(String report);

    
    private boolean singleSelect_ = false;

    public boolean isSingleSelect() {
        return singleSelect_;
    }
    public void setSingleSelect(boolean singleSelect) {
        singleSelect_ = singleSelect;
    }

    public int getIndexString(String name) {
        int maskNo = -1;
        if (name.startsWith("INS.ADP")) {
            //j++:
            int startIndex = -1;
            int endIndex = -1;
            for (int k = 0; k < name.length(); k++) {
                if (Character.isDigit(name.charAt(k))) {
                    startIndex = k;
                }
            }
            endIndex = name.length();
            String idxStr = name.substring(startIndex, endIndex);
            //int maskNo = -1;
            System.out.println("Idx String " + idxStr);
            maskNo = new Integer(idxStr).intValue();
        }
        return maskNo;
    }

    protected class SelectionDialog extends JDialog {

        private JOptionPane optionPane;

        private long[] obIdList;

        private String value_;

        private JButton okButton_ = new JButton("Yes");

        private JButton cancelButton_ = new JButton("No");


        private JButton[] buttonArray =
            { okButton_, cancelButton_};

        private int value = JOptionPane.CANCEL_OPTION;

        private String report_;
        
        
        public SelectionDialog(
            JFrame queueView,
            String question,
            String title,
            boolean modal,
            String report,
            String childTitle) {
            super(queueView, title, modal);
            optionPane =
                new JOptionPane(
                    question,
                    JOptionPane.QUESTION_MESSAGE,
                    JOptionPane.OK_CANCEL_OPTION,
                    null,
                    buttonArray,
                    "Yes");
            JTextArea textArea = new JTextArea(20, 45);
            this.getContentPane().add(new JScrollPane(textArea), "Center");                    
            textArea.setText(report);
            textArea.setCaretPosition(0);                                        
            this.getContentPane().add(optionPane, "South");
            this.pack();
            this.setResizable(false);
            //optionPane.addPropertyChangeListener(this); 
            //optionPane.addActionListener(this);            
            //this.obIdList = obIdList;
            if (queueView.getState() == JFrame.NORMAL) {
                setLocationRelativeTo(queueView);
            } else {
                Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
                setLocation(
                    (screen.width - getSize().width) / 2,
                    (screen.height - getSize().height) / 2);
            }
            okButton_.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    value = JOptionPane.OK_OPTION;
                    setVisible(false);
                }
            });
            cancelButton_.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    value = JOptionPane.CANCEL_OPTION;
                    setVisible(false);
                }
            });
        }

        public int getValue() {
            return value;
        }
    }
    
    /**
     * Save the report to disk before it gets sent.
     * @param filename
     * @param data
     */

    protected void saveFile(String filename,String data) {
        String directory = AppConfig.getAppConfig().getPafDirectory();
        File dir = new File(directory);
        String saveFilename =
            AppConfig.getAppConfig().getPafDirectory()
                + File.separator
                + filename;

        if (dir.canWrite()) {
            try {
                FileIOUtils.stringToFile(data, saveFilename);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

    }
}
