package org.eso.ohs.phase2.apps.masktracker;

import java.io.IOException;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.text.SimpleDateFormat;
import java.util.Vector;

import javax.swing.JOptionPane;

import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.gui.widgets.OutOfMemoryReport;
import org.eso.ohs.core.gui.widgets.TextDisplayWidget;
import org.eso.ohs.core.utilities.ConnectionPool;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.TextUtils;
import org.eso.ohs.dfs.Mask;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.instruments.Instrument;
import org.eso.ohs.instruments.InstrumentList;
import org.eso.ohs.instruments.MaskInstrument;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.MaskEvent;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.dbase.phase2.MaskDBIO;
import org.eso.ohs.persistence.dbase.phase2.MaskQueueManager;
import org.eso.ohs.persistence.dbase.phase2.MaskReport;
import org.eso.ohs.persistence.dbase.phase2.VIMOSReportImp;
import org.eso.ohs.persistence.dbase.phase2.MaskDBIO.IdInfo;
import org.eso.ohs.phase2.apps.ot.gui.QueueView;
import org.eso.ohs.phase2.icdVcs.IcdVcsCfg;


/**
 * Listener for script events from the BOB interface
 * This is a singleton refernce class
 * <p>
 * $Id: MaskInsertion.java,v 1.24 2007/10/11 11:43:12 pnunes Exp $
 * <br>
 * @author Bob Kemp
 */
public class MaskInsertion extends MaskActionGenerator {

    private static String obrepDbName_ =
        Config.getCfg().getDbName(ObservationBlock.getSuffix());
    /**
     * Reference to the only instanced of this clss
     */

    private static MaskInsertion instance_ = null;

    /**
     * ISO date format
     */

    SimpleDateFormat isoTimeFormat_;

    /**
     * Private Mask Insertion construction to force the use of 
     * getInstance 
     */

    private static TextDisplayWidget log_ = new TextDisplayWidget("Report");

    private MaskInsertion() {
    }

    /**
     * Get an instance of the MaskInsertion class
     * @return MaskInsertion instance
     */

    public static MaskInsertion getInstance() {
        if (instance_ == null) {
            instance_ = new MaskInsertion();
        }

        return instance_;
    }
    private int maskManExecCount = 0;
    private int maskInsertExecCount = 0;
    private Vector<IdInfo> cannotIns_ = new Vector<IdInfo>();

    private Vector<String> maskIdVec_ = new Vector<String>();

    /**
     * Generate an order for Mask Insertion
     * @param maxNumOBs Maximum number of OB's
     * @return A Mask Insetion order
     *
     */

    public String generateOrder(int maxNumOBs) {
        String pafText = "";
        int maxMaskCount = 0;
        try {
            IcdVcsCfg cfg = IcdVcsCfg.getCfg();
            long[] obIdList = cfg.getSelections(maxNumOBs);
            QueueView queueView =
                MTViewManager.getInstance().getSelectedQueueView();
            if (queueView == null)
                return "";

            //Updater.propagateMaskInfo();
            //Object message, int messageType, int optionType,
            String question =
                "Are you sure you to send an insertion order for "
                    + queueView.getQueue().getName()
                    + " ?";

            SelectionDialog d =
                new SelectionDialog(
                    queueView,
                    question,
                    "Mask Insertion",
                    true,
                    getMaskReport(obIdList),
                    "Insertion Report Statistics");
            d.show();
            int value = d.getValue();
            if (value == JOptionPane.OK_OPTION
                || value == JOptionPane.CANCEL_OPTION) {
                d.setVisible(false);
                d.dispose();
            }
            if (value == JOptionPane.CANCEL_OPTION)
                return "";
            MaskInstrument inst =
                (MaskInstrument) InstrumentList.getInstance().getInstrument(
                    "VIMOS");
            maxMaskCount = inst.getMaxMaskCount();
            pafText = getOrderText(obIdList);
            MaskQueueManager.setMaskRowStatus(
                maskIdVec_,
                Mask.PENDING_INSERTION,
                ((MaskInstrument) inst).getDummyNaid());

        } catch (ICCabinetFullException e) {
            e.printStackTrace();
            JOptionPane.showMessageDialog(
                null,
                e.getMessage(),
                "Too many masks Error",
                JOptionPane.WARNING_MESSAGE);
        } catch (OutOfMemoryError e) {
            e.printStackTrace();
            OutOfMemoryReport.showMessage();
            throw e;
        } catch (Exception e) {
            e.printStackTrace();
        }
        saveFile(TextUtils.currentISODateTime() + ".mio",pafText);            
        return pafText;
    }

    public String getOrderText(long[] idList)
        throws SQLException, ICCabinetFullException {

        StringBuffer paf = new StringBuffer();
        if ((idList != null) && (idList.length > 0)) {
            paf.append(ReportHeaders.generateInsertionOrderHeader());
            SimpleDateFormat isoTimeFormat_;
            paf.append(getMaskIdsString(idList));
            paf.append("\n");
            paf.append("TPL.FILE.DIRNAME   \"\"");
            paf.append("\n");
        }
        StringBuffer obd = new StringBuffer();
        obd.append(ReportHeaders.generateObdHeader());
        String body = ReportHeaders.getObdBody("MMU_Insertion_Order");
        obd.append(body);
        obd.append(
            ReportHeaders.generateMaskTemplate("VIMOS_mmu_Insertion_order"));
        String pafStr = paf.toString();
        if (pafStr.length() > 0)
            obd.append(
                "SEQ.MIO" + "\t\t\"" + TextUtils.escapedString(pafStr) + "\"");
        return obd.toString();
        //return "";
    }

    public String reportString() {
        StringBuffer naidReport = new StringBuffer();

        if (cannotIns_.size() > 0) {
            naidReport.append("The following Masks could not be inserted\n");
            naidReport.append(
                "Mask Status must be MaskStored and have a valid mask id \n");
            for (int i = 0; i < cannotIns_.size(); i++) {
                naidReport.append(cannotIns_.elementAt(i).toString());
                naidReport.append("\n");
            }
        }
        return naidReport.toString();
    }
    public String getMaskReport(long[] idList)
        throws SQLException, ICCabinetFullException {
        Instrument inst = InstrumentList.getInstance().getInstrument("VIMOS");
        int maskInserted = MaskDBIO.getNumMasksForStatus(Mask.MASK_INSERTED);
        int pendingInsertion =
            MaskDBIO.getNumMasksForStatus(Mask.PENDING_INSERTION);
        int maskCount = maskInserted + pendingInsertion;
        int maxMaskCount = ((MaskInstrument) inst).getMaxMaskCount();
        StringBuffer paf = new StringBuffer();
        IdInfo[] maskIds = getMaskIds(idList);
        for (int j = 0; j < maskIds.length; j++) {
            String status =
                TextUtils.padString(
                    Mask.getDisplayStatusValue(maskIds[j].getMaskStatus()),
                    12);
            paf.append(status);
            paf.append(" ");
            paf.append("INS.MASK" + maskIds[j].getAdpIndex() + ".ID");
            paf.append("\t");
            paf.append(maskIds[j].getMaskId());
            paf.append("\t");
            paf.append(maskIds[j].getMaskName());
            paf.append("\n");
        }
        if (maskManExecCount == 0) {
            if (maskInsertExecCount == 0)
                paf.append(
                    "Blank insertion report cabinets will be unloaded\n");
            else {
                paf.append("No masks will be inserted\n");
                paf.append(
                    maskInsertExecCount
                        + " masks already inserted will stay in the instrument cabinets\n");
            }
        } else {
            paf.append(
                maskManExecCount
                    + " masks will be inserted into the instrument cabinets\n");
            if (maskInsertExecCount > 0)
                paf.append(
                    maskInsertExecCount
                        + " masks already inserted will stay in the instrument cabinets\n");
        }
        return paf.toString();
    }

    public IdInfo[] getMaskIds(long[] idList)
        throws SQLException, ICCabinetFullException {
        StringBuffer paf = new StringBuffer();
        Instrument inst = InstrumentList.getInstance().getInstrument("VIMOS");
        int maskInserted = MaskDBIO.getNumMasksForStatus(Mask.MASK_INSERTED);
        int pendingInsertion =
            MaskDBIO.getNumMasksForStatus(Mask.PENDING_INSERTION);
        int maskCount = maskInserted + pendingInsertion;
        int maxMaskCount = ((MaskInstrument) inst).getMaxMaskCount();
        maskManExecCount = 0;
        maskInsertExecCount = 0;
        cannotIns_.removeAllElements();
        maskIdVec_.removeAllElements();
        Vector<IdInfo> vecMaskIdsMan = new Vector<IdInfo>();
        Vector<IdInfo> vecMaskIdsInsert = new Vector<IdInfo>();
        Vector<IdInfo> vecMaskIds = new Vector<IdInfo>();
        for (int i = 0; i < idList.length; i++) {
            if (!inst.hasMasks()) {
                continue;
            }
            System.out.println("Calling Mask Ids");
            IdInfo[] tmpMaskIds = MaskDBIO.getMaskIdsForOB(idList[i]);
            for (int j = 0; j < tmpMaskIds.length; j++) {
                String status = tmpMaskIds[j].getMaskStatus();
                System.out.println("IN Temp Mask Loop");
                if (!tmpMaskIds[j].getMaskId().equals("-99")
                    && (status.equals(Mask.MASK_MANUFACTURED)
                        || status.equals(Mask.MASK_INSERTED))) {
                    System.out.println("Mask ID" + tmpMaskIds[j].getMaskId());
                    if (!vecMaskIds.contains(tmpMaskIds[j]))
                        vecMaskIds.addElement(tmpMaskIds[j]);
                    if (status.equals(Mask.MASK_MANUFACTURED)) {
                        if (!vecMaskIdsMan.contains(tmpMaskIds[j])) {
                            vecMaskIdsMan.addElement(tmpMaskIds[j]);
                            maskManExecCount++;
                        }
                    }
                    if (status.equals(Mask.MASK_INSERTED)) {
                        if (!vecMaskIdsInsert.contains(tmpMaskIds[j])) {
                            vecMaskIdsInsert.addElement(tmpMaskIds[j]);
                            maskInsertExecCount++;
                        }
                    }
                } else {
                    cannotIns_.addElement(tmpMaskIds[j]);
                }
            }

        }
        IdInfo[] maskIds = {
        };
        System.out.println("Mask Man Exec Count " + maskManExecCount);
        System.out.println("Mask Man Exec Count " + maskInsertExecCount);
        System.out.println("Mask Inserted " + maskInserted);
        System.out.println("Max Mask Count " + maxMaskCount);
        String messageStr =
            "MaskStored = "
                + maskManExecCount
                + " MaskInserted = "
                + maskInsertExecCount
                + " Pending Insertion = "
                + pendingInsertion;

        if ((maskManExecCount + maskInsertExecCount + pendingInsertion)
            > maxMaskCount) {
            throw new ICCabinetFullException(
                "The total number of masks which are MaskInserted and MaskStored in \n"
                    + " this insertion order plus the total numner of masks PendingInsertion \n "
                    + " exceed the instrument cabinet space available\n "
                    + messageStr
                    + "\n"
                    + "The total space in the ICs are "
                    + maxMaskCount);

        } else {
            maskIds = new IdInfo[vecMaskIds.size()];
            vecMaskIds.copyInto(maskIds);
        }
        return maskIds;
    }

    public String getMaskIdsString(long[] idList)
        throws SQLException, ICCabinetFullException {
        int insert = 0;
        int man = 0;
        StringBuffer paf = new StringBuffer();
        IdInfo[] maskIds = getMaskIds(idList);
        for (int j = 0; j < maskIds.length; j++) {
            if (maskIdVec_.contains(maskIds[j].getMaskId()))
                continue;
            else
                maskIdVec_.addElement(maskIds[j].getMaskId());
            paf.append("INS.MASK" + maskIds[j].getAdpIndex() + ".ID");
            paf.append("\t\t\"");
            paf.append(maskIds[j].getMaskId());
            paf.append("\"");
            paf.append("\n");
        }
        return paf.toString();
    }

    // XXX This should go into a separate interface class in
    //     order to keep mask code distinct from the actual SQL.

    /**
     * Receive a mask Insertion report from BOB
     * @param report Mask Insertion Report
     */

    public void receiveReport(String report) {
        ConnectionPool pool = ConnectionPool.getPool();
        Connection conn = null;
        Statement stmt = null;
        Vector obIds = new Vector();
        try {
            MaskReport maskReport = new VIMOSReportImp();
            maskReport.receiveInsertionReport(report);
            MaskQueueManager.fireMaskPropertyChange(
                new MaskEvent("MaskStatus", Mask.MASK_INSERTED));
        } catch (SQLException e) {
            ErrorMessages.sqlError(e, null);
            e.printStackTrace();
        } catch (ObjectIOException e) {
            ErrorMessages.announceIOError(null, e);
            e.printStackTrace();
        } catch (ObjectNotFoundException e) {
            ErrorMessages.announceNoObject(null, e);
            e.printStackTrace();

        } catch (IOException e) {
            ErrorMessages.announceIOError(null, e);
            e.printStackTrace();
        }
    }

    /**
     * The requested object was not found on the external medium.
     * 
     * <p>
     * $Id: MaskInsertion.java,v 1.24 2007/10/11 11:43:12 pnunes Exp $
     */

    public class ICCabinetFullException extends Exception {

        /**
         * Construct an exception without explanatory message (not recommended)
         */
        public ICCabinetFullException() {
            super();
        }

        /**
         * Construct an exception with accompanying explanatory text.
         * 
         * @param msg   message giving a reason for the exception.
         */
        public ICCabinetFullException(String msg) {
            super(msg);
        }

    }

}
