package org.eso.ohs.phase2.apps.masktracker.actions;

import java.awt.event.ActionEvent;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.eso.ohs.core.dbb.client.DbbView;
import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.baseaction.IncrementalAction;
import org.eso.ohs.core.gui.baseaction.IncrementalActionAutomaton;
import org.eso.ohs.core.utilities.ObjUtils;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.Mask;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectNotFoundException;

/**
 * Action class to delete a business object in the chosen directory.
 * 
 * <p>
 * $Id: DeleteMaskAction.java,v 1.18 2007/10/11 11:43:19 pnunes Exp $
 * <p>
 * @author Bob Kemp
 */
public class DeleteMaskAction extends ActionSuperclass {

    /** Threshold for naming individual objects in messages to user */
    private final static int DELETION_THRESHOLD = 1;

 

    /** view of the folder in which the OB is to be deleted */
    DbbView dbbView_;



    public Long [] getAllSelected() {
        int []   selectedRows  = dbbView_.getSelectedRows();     
        Long[]  selectedOBsId = new Long[selectedRows.length];
        for (int i=0; i<selectedRows.length;i++) {
            Long value = new Long(dbbView_.getValueAt(selectedRows[i],0).toString());          
            selectedOBsId[i] = value;
        }   
        return selectedOBsId;
    }

    /**
     * Constructs a delete BusinessObject action.
     * 
     * @param device      medium containing the parent directory.
     * @param folderView  GUI view of (summary information in) the folder.
     */
    public DeleteMaskAction(DbbView dbbView) {
	super((JComponent)dbbView, "Delete");
	dbbView_ = dbbView;
    }

    /**
     * Event handler for the delete BusinessObject action.
     * 
     * @param ev       user-initiated ActionEvent to delete an OD.
     */
    public void actionPerformedImpl(ActionEvent ev) {
        Long [] idLongs = getAllSelected();
   
        int option = JOptionPane.showConfirmDialog(
                                                   dbbView_.getTopLevelAncestor(),
                                                   "Are you sure you want to delete selected masks ?",
                                                   "Delete",JOptionPane.OK_CANCEL_OPTION,
                                                   JOptionPane.QUESTION_MESSAGE,
                                                   null
	    );

        switch (option) {
	    case JOptionPane.CLOSED_OPTION : return;
	    case JOptionPane.CANCEL_OPTION : return;
        }

        new IncrementalActionAutomaton(
	    new IncrementalDeletion(idLongs),
	    (JFrame)dbbView_.getTopLevelAncestor(),
	    "Deleting..."
	);

    }


    /**
     * Utility class to delete a sequence of OBs, one at a time.
     * The user is also given the opportunity to stop the process using
     * a dialog box.
     */
    public class IncrementalDeletion implements IncrementalAction  {
	private ObjectManager objMgr = ObjectManager.getObjectManager();

        private Long [] idLongs_;

	public IncrementalDeletion(
						   Long [] idLongs) {
	    idLongs_ = idLongs;
	}

	public int getNumStepsRemaining(int counter) {
	    return ObjUtils.howManyNotNull(idLongs_, counter);
	}

	public void startOfAction() { }

	public void endOfAction() { }

	public void performStep(int counter) {
	    if (idLongs_[counter] != null) {
				// Main body of the "loop" + fall thru
	        doDeletion(idLongs_[counter]);
	    }
	}

	public void doDeletion(Long idLong) {
	    try {
                ObjectManager objMgr = ObjectManager.getObjectManager();    
	        objMgr.deleteBusObj(Media.DBASE,Config.getCfg().tableToUniqueId(idLong.longValue(),Mask.class),Mask.class);
	    }
            catch(ObjectIOException  e) {
	      //ErrorMessages.announceObjectIOException();
	    }
            catch(ObjectNotFoundException e2) {
	      //ErrorMessages.announceObjectNotFoundException()
            }
	}
    }

}


