package org.eso.ohs.phase2.apps.ot.actions;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.util.Vector;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.eso.ohs.core.dbb.sql.DbbQueryTerminated;
import org.eso.ohs.core.dbb.sql.DbbQueryTerminatedListener;
import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.baseaction.IncrementalAction;
import org.eso.ohs.core.gui.baseaction.IncrementalActionAutomaton;
import org.eso.ohs.core.gui.widgets.TextDisplayWidget;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.TextUtils;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.evm.EvmExecutorImpl;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectContainer;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.phase2.apps.ot.dbb.OTDbbView;
import org.eso.ohs.phase2.apps.ot.gui.ExecutionSequenceBView;

/**
 * Action class to  generate the execution list
 * on the selected BusinessObjects.
 *
 * <p>
<<<<<<< ExecutionSequenceAction.java
 * @version $Revision: 1.21 $
=======
 * @version $Revision: 1.21 $
>>>>>>> 1.3.2.4
 * <br>
 */
public class ExecutionSequenceAction extends ActionSuperclass {

  
    private ObjectContainer folderView_;
    private boolean finished_ = false;
    private static TextDisplayWidget log_ = new TextDisplayWidget("Execution Sequence Report");
    private final static String HEADER = ("EXECUTION MESSAGE LOG "
            + TextUtils.currentISODateTime() + "(UT)\n"
            + "==================================================\n");
    private OTDbbView view_;

    private ExecQueryTerminatedListener listener_;
 
 
    public ExecutionSequenceAction(ObjectContainer folderView,JComponent comp, String name) {
    super(comp,name);
        folderView_ = folderView;
        view_ = ExecutionSequenceBView.getInstance().getView();
        listener_ = new ExecQueryTerminatedListener(this);
        view_.getDbbView().addQueryTerminatedListener(listener_);
    }

    public void actionPerformedImpl(ActionEvent ev) {         
        this.setEnabled(false);
        Long [] idLongs = folderView_.getAllSelected();

        if (idLongs == null || idLongs.length == 0) {
            JOptionPane.showMessageDialog(
                folderView_.getTopLevelAncestor(),
                new Object[] {
                  "There are no selected Business Objects ",                  "Which object should be reported?",
                },
                "Nothing selected",
                JOptionPane.ERROR_MESSAGE
            );
            return;             // nothing selected?
        }
        new IncrementalActionAutomaton(
            new IncrementalExecution(idLongs),
            (JFrame)folderView_.getTopLevelAncestor(), "Adding to Execution Sequence...");
    }

    public void unsubscribe() {
         view_.getDbbView().removeQueryTerminatedListener(listener_);
    }


    private class ExecQueryTerminatedListener implements DbbQueryTerminatedListener {

        private ExecutionSequenceAction action_;

        public ExecQueryTerminatedListener(ExecutionSequenceAction action) {
            action_ = action;
        }

        public void queryTerminatedAction(DbbQueryTerminated ev) {
            action_.setEnabled(true);
        }
    }                                                                                                       

    public class IncrementalExecution implements IncrementalAction {

        private ObjectManager objMgr = ObjectManager.getObjectManager();
        private Long[] idLongs_;
        int numDuplicated_ = 0;
        StringBuffer buf_;
        StringBuffer ioErrorBuffer_;
        StringBuffer objNotFoundBuffer_;
        StringBuffer verifyBuffer_;
        Vector<Long> setOBIDs_ = new Vector<Long>();
        //ExecutionSequenceBView execView_;

        public IncrementalExecution(Long[] idLongs) {
            idLongs_  = idLongs;
        }

        public int getNumStepsRemaining(int counter) {
            int stepRemaining = 0;
            if (!finished_) {
                stepRemaining = idLongs_.length - counter;
            }
            return stepRemaining;
        }

        public void startOfAction() {
            numDuplicated_ = 0;
            buf_ = new StringBuffer();
            ioErrorBuffer_ = new StringBuffer();
            objNotFoundBuffer_ = new StringBuffer();
            verifyBuffer_ = new StringBuffer();
            setOBIDs_.removeAllElements();
        }

        public void endOfAction() {
            /** After going thru the complete list of OBs
             *  Call method to add the OBs to ExecutionSequenceBView
             */

            Long[] appendIds = new Long[setOBIDs_.size()];
            long [] prmAppendIds = new long[setOBIDs_.size()];
            setOBIDs_.copyInto(appendIds);
            for(int i=0;i<appendIds.length;i++) {
                prmAppendIds[i] = appendIds[i].longValue();
            }
            ExecutionSequenceBView.getInstance().appendOBs(prmAppendIds);
            /** Print report of OBs that failed to be added to Execution Sequence */
            if(numDuplicated_ > 0 || 
               objNotFoundBuffer_.length() > 0 ||
               ioErrorBuffer_.length() > 0 || 
               verifyBuffer_.length() > 0) {
                 log_.clearTextArea();
                 StringBuffer report = new StringBuffer();
                 if(numDuplicated_ > 0) {
                     report.append("\n\nThe following OBs/CBs have already been " +
                     "added to the Execution Sequence:\n\n" +
                     buf_.toString() + "\n\n");
                 }
                 if(ioErrorBuffer_.length() > 0) {
                     report.append("\n\nThe following OBs/CBs have corrupted or missing data \n"
                            + "thus they cannot be added to the Execution Sequence:\n\n"
                            + ioErrorBuffer_.toString()
                           );
                 }
                 if(objNotFoundBuffer_.length() > 0) {
                     report.append("\n\nThe following OBS/CBS no longer exist in the database \n"
                           + "thus they cannot be added to the Execution Sequence:\n\n"
                           + objNotFoundBuffer_.toString());

                 }
                 if(verifyBuffer_.length() > 0) {
                     report.append("\n\nThe following OBs/CBs failed verification \n"
                           + "thus they cannot be added to the Execution Sequence:\n\n"
                           + verifyBuffer_.toString());


                 }
                 if(report.length() > 0) {
                     log_.setTextArea(HEADER + report.toString());
                     log_.setSize(new Dimension(700,300));
                     log_.setVisible(true);
                 }
            }
        }

        public void performStep(int counter) {
            if (!finished_) {
                doAppendOBs(idLongs_[counter].longValue());
            }
        }


        public void doAppendOBs(long objMgrObIds) {
            long obId = Config.getCfg().uniqueToTableId(objMgrObIds);     

            if (view_.containsOB(obId) == true) {
                numDuplicated_++;
                if(numDuplicated_ >=2 )
                    buf_.append("\n");
                buf_.append(obId + " ");

            }
            else {
                CalibrationBlock cb = null;
                try {
                    Class c = Config.getCfg().getClassFromId(objMgrObIds);
                    ObjectManager objMgr = ObjectManager.getObjectManager();
                    objMgr.refreshBusObjRecursive(Media.DBASE,objMgrObIds,c);
                    cb = (CalibrationBlock)objMgr.getBusObj(Media.DBASE,objMgrObIds,c);                 
                    if(cb.verify().length() > 0) {                         
                         verifyBuffer_.append(obId);
                         verifyBuffer_.append(cb.verify());
                         return;
                    }

      

                    // Perform the Stage2Verification
                    boolean verifyPassed = true;
                    String stage2VerifErrors = new String();
                    StringBuffer header = new StringBuffer();
                    StringBuffer footer = new StringBuffer();
                    StringBuffer report = new StringBuffer();
                    if (EvmExecutorImpl.evmConfigured()) {
                        EvmExecutorImpl evm = EvmExecutorImpl.getInstance();
                        if (!evm.isInitialized()) {
                            verifyPassed = false;
                        }
                        else {
                            verifyPassed = evm.verify(cb,report, header, footer);
                            //Format report message which includes 
                            //info, warnings, and errors for report 
                            // cmavros 2002/03/04 1690-OT
                            stage2VerifErrors = "\n" + report.toString().trim() + "\n" ;
                        }
                    }

                    if(!verifyPassed){
                        verifyBuffer_.append("OB/CB ");
                        verifyBuffer_.append(obId);
                        verifyBuffer_.append(stage2VerifErrors.toString());
                        return;
                    }
                    Long tmpOBId = new Long(objMgrObIds);
                    if(!setOBIDs_.contains(tmpOBId))
                        setOBIDs_.addElement(tmpOBId);
                }
                catch (ObjectNotFoundException e) {
                    objNotFoundBuffer_.append(obId+": "+e.getMessage());
                    objNotFoundBuffer_.append("\n");
                }
                catch (ObjectIOException e) {
                    ioErrorBuffer_.append(obId + ": ");
                    ioErrorBuffer_.append(e.getMessage());
                    ioErrorBuffer_.append("\n");
                    return;
                }
           }
       }

   }

}
