package org.eso.ohs.phase2.apps.ot.actions;

import java.awt.event.ActionEvent;
import java.sql.SQLException;

import javax.swing.AbstractAction;
import javax.swing.JOptionPane;
import javax.swing.JTable;

import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.phase2.apps.ot.Persistence;
import org.eso.ohs.phase2.apps.ot.gui.CommentDialog;
import org.eso.ohs.phase2.apps.ot.gui.QueueItemsBView;

public class OpenCommentsAction extends AbstractAction {

    /** Standard log channel */
    private static Logger stdlog_ =
        Logger.getLogger(OpenCommentsAction.class);

    private QueueItemsBView queueItemView_;

    private static CommentDialog dialog_ = new CommentDialog();

    private JTable table_;

    public OpenCommentsAction(QueueItemsBView queueItemView) {
        super("Comment...");
        queueItemView_ = queueItemView;
        table_ = queueItemView_.getView().getJTable();
    }

    /**Comments a set of OBs*/
    private void commentOBs() {
        long[] OBIDs = queueItemView_.getView().getAllSelectedOBIds();
        int j = 0;
        try {

            ObjectManager objMgr = ObjectManager.getObjectManager();
            Class c =
                queueItemView_.getView().getClassForId(
                    Config.getCfg().uniqueToTableId(OBIDs[0]));
            stdlog_.debug("Class " + c);
            if (OBIDs.length > 1) {
                for (int i = 1; i < OBIDs.length; i++) {
                    if (!c.equals(Config.getCfg().getClassFromId(OBIDs[i]))) {
                        String message =
                            "The list of OB's/CB's you wish to comment contain \n"
                                + "Calibration Blocks and Observation Blocks \n"
                                + "Please select only OB's or CB's ";
                        JOptionPane.showMessageDialog(
                            table_.getTopLevelAncestor(),
                            message,
                            "ERROR",
                            JOptionPane.ERROR_MESSAGE);
                        return;
                    }
                }
            }
            CalibrationBlock cb =
                (CalibrationBlock) objMgr.getBusObj(Media.DBASE, OBIDs[0], c);

            for (int i = 0; i < CalibrationBlock.getStatusValues().size(); i++) {
                if (CalibrationBlock.getStatusCode(i).equals(cb.getStatus())) {
                    dialog_.setStatus(CalibrationBlock.markLabels[i]);
                    break;
                }
            }
            if (c.equals(ObservationBlock.class))
                dialog_.setQCVisible(true);
            else
                dialog_.setQCVisible(false);
            dialog_.setObsComment(cb.getObsComments());
            dialog_.setUserComment(cb.getUserComments());
            dialog_.setInstComment(cb.getInstrumentComments());
              if (cb instanceof ObservationBlock)
                dialog_.setQcGrade(((ObservationBlock) cb).getObsGrade());
            dialog_.setVisible(true);
            int returnValue = dialog_.getUserChoice();
            if (returnValue == JOptionPane.CANCEL_OPTION)
                return;
            Persistence persistence = Persistence.getInstance();
            persistence.commentOBs(
                OBIDs,
                dialog_.getStatus(),
                dialog_.getObsComment(),
                dialog_.getQcGrade());
        } catch (ObjectIOException ex) {
            ErrorMessages.announceIOError(table_, ex);
        } catch (ObjectNotFoundException ex) {
            ErrorMessages.announceNoObject(table_, ex);
        } catch (SQLException e) {
            String message = "Error: " + e.getMessage();
            JOptionPane.showMessageDialog(
                table_.getTopLevelAncestor(),
                message,
                "ERROR",
                JOptionPane.ERROR_MESSAGE);
            stdlog_.error(e);
        }
    }

    public void actionPerformed(ActionEvent ev) {
        commentOBs();
    }

}
