package org.eso.ohs.phase2.apps.ot.actions;

import java.awt.event.ActionEvent;

import javax.swing.JOptionPane;
import javax.swing.JTable;

import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.baseaction.IncrementalAction;
import org.eso.ohs.core.gui.baseaction.IncrementalActionAutomaton;
import org.eso.ohs.core.utilities.ObjUtils;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.BusinessObject;
import org.eso.ohs.dfs.Summary;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.phase2.apps.ot.gui.OTMainView;
import org.eso.ohs.phase2.apps.ot.gui.OTViewManager;
import org.eso.ohs.phase2.apps.ot.gui.QueueSummaryModel;
import org.eso.ohs.phase2.apps.ot.gui.SummaryTableEvent;

/**
 * Action class to Open a queue 
 * <p>
 * $Id: OpenQueueAction.java,v 1.11 2006/04/25 16:33:12 pnunes Exp $
 * <br>
 * Author: Bob Kemp
 */
public class OpenQueueAction extends ActionSuperclass {

    private static Logger stdlog_ =
        Logger.getLogger(OpenQueueAction.class);

    private JTable queueTable_;

    private JTable openQueueTable_;

    private OTMainView otMainView_;
    
    private Class dispClass_;

    public OpenQueueAction(OTMainView otMView, String name) {
        super(otMView, name);
        queueTable_ = otMView.getQueueTable();
        openQueueTable_ = otMView.getOpenQueueTable();
        otMainView_ = otMView;
        dispClass_ = otMainView_.getDisplayedObjectClass();
    }

    private long[] getSelectedQueueIds() {
        int[] rows = queueTable_.getSelectedRows();
        long[] queueIds = new long[rows.length];
        QueueSummaryModel model = (QueueSummaryModel) queueTable_.getModel();
        for (int i = 0; i < rows.length; i++) {
            queueIds[i] = model.getQueueId(rows[i]);
        }
        return queueIds;
    }

    public void actionPerformedImpl(ActionEvent ev) {
        QueueSummaryModel model = (QueueSummaryModel) queueTable_.getModel();
        long[] queueIds = getSelectedQueueIds();
        int result = JOptionPane.YES_OPTION;
        switch (result) {
            case JOptionPane.YES_OPTION :
                new IncrementalActionAutomaton(
                    new IncrementalOpenQueue(queueIds),
                    otMainView_,
                    "Opening....");
            default :
                return;
        }
    }

    /**
        * Utility class to open a series of queues, one at a time.
        * The user is also given the opportunity to stop the process using
        * a dialog box.
        */
    public class IncrementalOpenQueue implements IncrementalAction {

        /** array of ids of objects to be duplicated */
        private long[] queueIds_;

        private int objCount_;

        
        /**
         * Incrementally open the queues fro the given ID.
         * @param queueIds
         */
        public IncrementalOpenQueue(long[] queueIds) {
            queueIds_ = queueIds;
            objCount_ = queueIds.length;
            stdlog_.debug(
                "IncrementalOpenQueue: Afer constructing Incremental Open Queue");
        }

        /**
         * Number of steps in the action that are still remaining to be done.
         * 
         * @return number of steps remaining (eg objects to acted upon)
         */
        public int getNumStepsRemaining(int counter) {
            Long[] queueObjs = new Long[queueIds_.length];
            for (int i = 0; i < queueIds_.length; i++) {
                queueObjs[i] = new Long(queueIds_[i]);
            }
            return ObjUtils.howManyNotNull(queueObjs, counter);
        }

        /**
         * Method to perform one incremental step of a larger action.
         * 
         * @param stepNo	Counter to indicate the n'th step of a larger action
         */

        public void performStep(int counter) {
            QueueSummaryModel model =
                (QueueSummaryModel) queueTable_.getModel();
            int row = model.getRowFromQueueId(queueIds_[counter]);
            Summary summ = model.getObjectAt(row);
            doOpen(queueIds_[counter],summ.getObjectClass());
         
            stdlog_.debug(
                "performStep: Queue ID counter " + queueIds_[counter]);
         
            stdlog_.debug(
                "performStep: Summ = " + summ.getPropertyValue("QueueId"));
            otMainView_.fireObjectTableEvent(
                summ,
                SummaryTableEvent.OBJECT_ADDED);
            otMainView_.fireObjectTableEvent(
                summ,
                SummaryTableEvent.OBJECT_CHANGED);
        }

        /**
         * Hook for initialisation of action object.
         */
        public void startOfAction() {
        }

        /**
         * Hook for cleanup of action object.
         */
        public void endOfAction() {

        }


        /**
          * Utility method to duplicate a given business object
          * 
          * @param id        id of business object to duplicate
          */
        public void doOpen(long queueId,Class c) {
            ObjectManager objMgr = ObjectManager.getObjectManager();
            if (!objMgr.isInRegistry(queueId)) {
                //if the queue has not been already opened
                try {
                    //Class c = otMainView_.getDisplayedObjectClass();
                    BusinessObject busObj =
                        (BusinessObject) objMgr.getBusObj(
                            Media.DBASE,
                            queueId,
                            c);
                    busObj.setIsNew(false);
                    OTViewManager otViewMgr = OTViewManager.getOTViewMgr();
                    otViewMgr.showObject(busObj,c);
                } catch (ObjectNotFoundException ex) {
                    String message =
                        " An error occured while loading queue "
                            + queueId
                            + ": "
                            + ex.getMessage();
                    JOptionPane.showMessageDialog(
                        otMainView_,
                        message,
                        "I/O Error",
                        JOptionPane.ERROR_MESSAGE);
                    ex.printStackTrace();

                } catch (ObjectIOException ex) {
                    String message =
                        " An error occured while loading queue "
                            + queueId
                            + ": "
                            + ex.getMessage();
                    JOptionPane.showMessageDialog(
                        otMainView_,
                        message,
                        "I/O Error",
                        JOptionPane.ERROR_MESSAGE);
                    ex.printStackTrace();
                }
            } else {
                String message = "Queue " + queueId + " is already open";
                JOptionPane.showMessageDialog(
                    otMainView_,
                    message,
                    "WARNING",
                    JOptionPane.WARNING_MESSAGE);
            }
        }

    }
}
