package org.eso.ohs.phase2.apps.ot.actions;

import java.awt.event.ActionEvent;
import java.sql.SQLException;

import javax.swing.JOptionPane;
import javax.swing.JTable;

import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.baseaction.IncrementalAction;
import org.eso.ohs.core.gui.baseaction.IncrementalActionAutomaton;
import org.eso.ohs.core.utilities.ObjUtils;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.Queue;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.phase2.apps.ot.gui.OTMainView;
import org.eso.ohs.phase2.apps.ot.gui.OTViewManager;
import org.eso.ohs.phase2.apps.ot.gui.QueueSummaryModel;
import org.eso.ohs.phase2.apps.ot.gui.QueueView;


/**
 * Action class refresh a queue
 *
 * <p>
 * $Id: RefreshQueueAction.java,v 1.9 2005/02/11 10:31:49 tcanavan Exp $
 * <br>
 * Author: cmavros
 */
public class RefreshQueueAction extends ActionSuperclass {

   /** Standard log channel */
   private static Logger stdlog_ =
       Logger.getLogger( RefreshQueueAction.class );

    private JTable table_;

    private QueueSummaryModel queueModel_;

    private OTMainView view_;

    IncrementalActionAutomaton actionAutomaton_;

    public RefreshQueueAction(OTMainView view,JTable table, String name) {
	super(view,name);
	view_ = view;
	table_ = table;
	queueModel_ = (QueueSummaryModel)table_.getModel();
    }

    private long [] getSelectedQueueIds() {
	int [] rows = table_.getSelectedRows();
	long [] queueIds = new long[rows.length];
	QueueSummaryModel model = (QueueSummaryModel)table_.getModel();
	for(int i=0;i<rows.length;i++) {
             queueIds[i] = model.getQueueId(rows[i]);
	}
        return queueIds;
    }

    public void actionPerformedImpl(ActionEvent ev) {
	stdlog_.debug("RefreshQueueAction Event " + ev);
        QueueSummaryModel model = (QueueSummaryModel)table_.getModel();
	int [] rows = table_.getSelectedRows();
	long [] queueIds = getSelectedQueueIds();

	actionAutomaton_ = new IncrementalActionAutomaton(
                    new IncrementalRefreshQueue(queueIds,rows), view_,
                    "Refreshing...."
		);

    }


    /**
     * Utility class to duplicate a sequence of OBs, one at a time.
     * The user is also given the opportunity to stop the process using
     * a dialog box.
     */
    public class IncrementalRefreshQueue implements IncrementalAction {

	private long [] queueIds_;
	private int [] rows_;
	private int objCount_;
	private int currrentRow_;


	/**
	 * Constructor.
	 * 
	 * @param frm	    window on which to base any dialog boxes.
	 * @param device    media containing the objects to be duplicated
	 * @param deepCopy  should we duplicate attached objects?
	 * @param dir	    directory containing the ObsBlocks
	 * @param objType   type of object to be duplicated
	 * @param idLongs   array of ids of objects to be duplicated
	 */
	public IncrementalRefreshQueue(long [] queueIds,int [] rows) {	   
	    queueIds_  = queueIds;         
	    objCount_ = queueIds.length;	
            rows_ = rows;
	}

       /**
        * Number of steps in the action that are still remaining to be done.
        * 
        * @return number of steps remaining (eg objects to acted upon)
        */
        public int getNumStepsRemaining(int counter) {
            Long [] queueObjs = new Long[queueIds_.length];
	    for(int i=0;i<queueIds_.length;i++) {
                queueObjs[i] = new Long(queueIds_[i]);
            }
            return ObjUtils.howManyNotNull(queueObjs, counter);
        }

        /**
         * Method to perform one incremental step of a larger action.
         * 
         * @param stepNo	Counter to indicate the n'th step of a larger action
         */
        public void performStep(int counter) {           
	    doRefresh(queueIds_[counter]);                          
        }

        /**
         * Hook for initialisation of action object.
         */
        public void startOfAction() { }

        /**
         * Hook for cleanup of action object.
         */
        public void endOfAction() { }


	/**
	 * Utility method to duplicate a given business object
	 * 
	 * @param id	    id of business object to duplicate
	 */
	public void doRefresh(long id) {
            OTViewManager manager = OTViewManager.getOTViewMgr();
            QueueView queueManager = manager.getQueueView(id);
 	    try {
                ObjectManager objMgr = ObjectManager.getObjectManager();
                /*if(queueManager != null) {
	           OTViewManager otMgr = OTViewManager.getOTViewMgr();
                   int index = manager.getQueueViewManagers().indexOf(queueManager);
      	           queueManager.closeQueueWithoutSaving();
                   if(actionAutomaton_.getProgressDialog()!=null){
                     actionAutomaton_.getProgressDialog().setVisible(true);
                   }
               }*/
                Queue queue = null;
                boolean inRegistry = true; 
                if(!objMgr.isInRegistry(id)) {
                    inRegistry = false;	            
                }
                Class c = Config.getCfg().getClassFromQueueId(id);
                queue = (Queue)objMgr.getBusObj(Media.DBASE,id,c);
                objMgr.refreshBusObj(Media.DBASE,id,c);
	        queue.setSize(queue.getSize());
                queue.fireNameEvent();
                if(queueManager != null)
                    queueManager.setQueue(queue);
                queue.setUpdate(false);
                if(!inRegistry)
	            objMgr.forgetBusObj(Media.DBASE,id,c);

           }
	   catch (ObjectNotFoundException ex) {
	       JOptionPane.showMessageDialog(null,
	      	 "An error occured while loading the Queue",
	    	 "I/O Error", JOptionPane.WARNING_MESSAGE);
			ex.printStackTrace();
	   }
	   catch (ObjectIOException ex) {
	     JOptionPane.showMessageDialog(null,
	    	"An error occured while loading the Queue",
	    	 "I/O Error", JOptionPane.WARNING_MESSAGE);
	    ex.printStackTrace();
           }
           catch(SQLException ex) {
                 JOptionPane.showMessageDialog(null,
	    	"A SQL  occured while loading the Queue",
	    	 "SQL Error", JOptionPane.WARNING_MESSAGE);
	    ex.printStackTrace();
           }
        }
    }
}
