package org.eso.ohs.phase2.apps.ot.actions;

import java.awt.event.ActionEvent;
import java.lang.reflect.InvocationTargetException;
import java.sql.SQLException;

import javax.swing.JComponent;
import javax.swing.JOptionPane;

import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.MaskQueue;
import org.eso.ohs.dfs.Queue;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.phase2.apps.ot.gui.OTViewManager;
import org.eso.ohs.phase2.apps.ot.gui.QueueContainer;
import org.eso.ohs.phase2.apps.ot.gui.QueueView;

/**
 * Action class to save a queue as another name and aabandon the 
 * edits of the old one.
 * <p>
 *
 * $Id: SaveAsQueueAction.java,v 1.15 2006/02/14 15:08:25 pnunes Exp $
 * <br>
 * Author: Bob Kemp
 */
public class SaveAsQueueAction extends ActionSuperclass {
	
	/** Standard log channel */
	private static Logger stdlog_ =
		Logger.getLogger( SaveAsQueueAction.class );
	
	
	private QueueContainer queueContainer_;
	
		
	
	
	public SaveAsQueueAction(QueueContainer queueContainer ,JComponent comp,String name) {
		super(comp,name);
		queueContainer_ = queueContainer;
	}
	
	
	
	public void actionPerformedImpl(ActionEvent ev) {
		String inputValue =  JOptionPane.showInputDialog( "Please enter a new queue name" );
		if(inputValue != null) {
			try {
				ObjectManager objMgr = ObjectManager.getObjectManager();							
					
				long [] queueIds = queueContainer_.getSelectedOpenQueueIds();
				//Queue queue= (Queue)objMgr.getBusObj(Media.DBASE,
				//					Config.getCfg().tableToUniqueId(queueIds[0],Queue.class),
				//				Queue.class);
				
				System.out.println("Queue Ids " + queueIds[0]);
				Class c = Config.getCfg().getClassFromQueueId(queueIds[0]);
				Queue oldQueue= (Queue)objMgr.getBusObj(Media.DBASE,
						queueIds[0],
						c);
												
				
				Queue newQueue;
				if(oldQueue instanceof MaskQueue)
					newQueue  = new MaskQueue();
				else
					newQueue = new Queue();
				objMgr.register(Media.DBASE,objMgr.getRoot(Media.DBASE),newQueue); 
				oldQueue.copyProperties(oldQueue.simpleProperties(),newQueue);              
				newQueue.setName(inputValue);              
				objMgr.putBusObj(newQueue);
				newQueue.setUpdate(false);
				
				c = Config.getCfg().getClassFromQueueId(oldQueue.getId());
				objMgr.refreshBusObj(Media.DBASE,newQueue.getId(),c);
				//objMgr.refreshBusObj(Media.DBASE,oldQueue.getId(),c);
				newQueue.setUpdate(false);
				// Force setSize to fire an event that allows the summary model to update
				newQueue.setSize(oldQueue.getSize());
				//stdlog_.debug("Old Queue " + oldQueue.getSize());
				OTViewManager otMgr = OTViewManager.getOTViewMgr();
				QueueView mgr = otMgr.getQueueView(oldQueue.getId());              
				objMgr.forgetBusObj(Media.DBASE,oldQueue.getId(),c);
				mgr.setQueue(newQueue);
				mgr.getQueue().addPropertyChangeListener(mgr);
				newQueue.setIsNew(false);
				mgr.updateTitle(); 
				
			}
			catch(SQLException e) {
				e.printStackTrace();
				stdlog_.error(e);
			}
			catch(ObjectIOException e) {
				e.printStackTrace();
				ErrorMessages.announceIOError(null,e);
				stdlog_.error(e);
			} 
			catch(ObjectNotFoundException e) {
				e.printStackTrace();
				ErrorMessages.announceNoObject(null,e);
			}
			catch(InvocationTargetException e) {
				stdlog_.error(e);
			}
			catch(IllegalAccessException e) {
				stdlog_.error(e);
			}
			catch(NoSuchMethodException e) {
				stdlog_.error(e);
			}
		}                 
	}
}

