package org.eso.ohs.phase2.apps.ot.dbb;

// $Id: OTDbbView.java,v 1.25 2006/04/25 16:35:11 pnunes Exp $

import java.awt.Container;
import java.awt.Rectangle;
import java.sql.SQLException;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.Vector;

import javax.swing.CellEditor;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.event.EventListenerList;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.client.DbbTable;
import org.eso.ohs.core.dbb.client.DbbTableSortEvent;
import org.eso.ohs.core.dbb.client.DbbView;
import org.eso.ohs.core.dbb.server.DbbSqlEngine;
import org.eso.ohs.core.dbb.sql.DbbQueryStarted;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.BusinessObject;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectContainer;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.ObjectSelectionEvent;
import org.eso.ohs.persistence.ObjectSelectionListener;
import org.eso.ohs.persistence.ObjectSelector;

/**
 * 
 * @author Giorgio Passera ESO 15-Sep-2000
 */

public class OTDbbView implements ObjectContainer, ObjectSelector {

    private static final long serialVersionUID = 1L;

    static public final String OB_ID = "OB ID";

    static public final String TYPE = "Type";

    public static final String STATUS = "Status";

    public static final String QUEUE_ID = "QueueID";

    public static final String ORANG_RANK = "Rank";

    public static final String RUN_ID = "Run Id";

    public static final String ORANG_RANK_HIDE = "RankHide";

    private static Logger stdlog_ = Logger.getLogger(OTDbbView.class);

    private DbbView dbbView;

    private DbbTable summaryTable_;

    private EventListenerList listenerList_ = new EventListenerList();

    // ----------------------------------------
    // Constructor
    // ----------------------------------------

    /**
     * Build a database browser view.
     * 
     * @param sqlEngine
     *            Reference to the SQL engine which will do all the real work.
     */
    public OTDbbView() throws SQLException {

        /* Call the DbbView constructor. */
        DbbSqlEngine sqlEngine = BrowserConfig.getEngine();
        dbbView = new DbbView(sqlEngine);
        summaryTable_ = dbbView.getTable();

        // stdlog_.debug("Afer constructing Engine");
        // if (!sortButton)
        // hideSortButton();
        // hideCountButton();
        // hideStopButton();
        // if (!clearButton)
        // hideResetButton();
        // if (!queryButton)
        // hideQueryButton();
    }

    public JTable getJTable() {
        return dbbView.getTable().getTable();
    }

    public void createDefaultListListener() {
        dbbView.addListSelectionListener(new ListListener(summaryTable_
                .getTable()));
    }

    public void sqlStartAction() {
        dbbView.sqlStartAction(new DbbQueryStarted(this));
    }

    public void tableSortAction(DbbTableSortEvent event) {
        dbbView.tableSortAction(event);
    }

    /**
     * Removes a set of rows
     * 
     * @param rows
     *            specifies a set of row's indeces to be removed
     */
    public void removeRows(int[] rows) {
        summaryTable_.removeRows(rows);
        dbbView.updateDisplayedRowCount();
    }

    /**
     * MovesUp a set of rows
     * 
     * @param rows
     *            specifies a set of row's indeces to be moved up
     */
    public void moveUpRows(int[] rows) {
        summaryTable_.moveUpRows(rows);
    }

    /**
     * MovesUp a set of rows
     * 
     * @param rows
     *            specifies a set of row's indeces to be moved up
     */
    public void moveToTop(int[] rows) {
        summaryTable_.moveToTop(rows);
    }

    public void setValueAt(Object object, int row, String column) {
        int colIndex = summaryTable_.getColIndex(column);
        summaryTable_.getModel().setValueAt(object, row, colIndex);
    }

    /**
     * MovesUp a set of rows
     * 
     * @param rows
     *            specifies a set of row's indeces to be moved up
     */
    public void moveDownRows(int[] rows) {
        summaryTable_.moveDownRows(rows);
    }

    public Class getClassForId(long id) throws NullPointerException {
        int rowCount = summaryTable_.getRows();
        for (int i = 0; i < rowCount; i++) {
            Integer rowid = (Integer) dbbView.getValueAt(i, OB_ID);
            // stdlog_.debug("Row Id in table " + rowid.longValue());
            String type = (String) dbbView.getValueAt(i, TYPE);
            if (rowid.longValue() == id) {
                if (type.equalsIgnoreCase("O"))
                    return ObservationBlock.class;
                if (type.equalsIgnoreCase("C"))
                    return CalibrationBlock.class;
            }
        }
        return null;
    }

    public void selectAllObjects() {
        ListSelectionModel lsm = summaryTable_.getTable().getSelectionModel();
        CellEditor ed = summaryTable_.getTable().getCellEditor();
        if (ed != null) {
            ed.stopCellEditing();
        }
        lsm.addSelectionInterval(0, summaryTable_.getModel().getRowCount() - 1);
    }

    public Container getTopLevelAncestor() {
        return ( summaryTable_.getTable().getTopLevelAncestor());
    }

    public Long[] getAllSelected() {
        int[] selectedRows = dbbView.getSelectedRows();

        Long[] selectedOBsId = new Long[selectedRows.length];
        for (int i = 0; i < selectedRows.length; i++) {
            Integer value = (Integer) dbbView
                    .getValueAt(selectedRows[i], OB_ID);
            String type = (String) dbbView.getValueAt(selectedRows[i], TYPE);
            if (type.equals("O")) {
                selectedOBsId[i] = new Long(Config.getCfg().tableToUniqueId(
                        value.longValue(), ObservationBlock.class));
            } else {
                selectedOBsId[i] = new Long(Config.getCfg().tableToUniqueId(
                        value.longValue(), CalibrationBlock.class));
            }
        }
        return selectedOBsId;
    }
    
    public long[] getAllSelectedOBIds() {
        int[] selectedRows = dbbView.getSelectedRows();
        long[] selectedOBsId = new long[selectedRows.length];
        for (int i = 0; i < selectedRows.length; i++) {
            Integer value = (Integer) dbbView
                    .getValueAt(selectedRows[i], OB_ID);
            String type = (String) dbbView.getValueAt(selectedRows[i], TYPE);
            if (type.equals(CalibrationBlock.OBSBLK_TYPE)) {
                selectedOBsId[i] = Config.getCfg().tableToUniqueId(
                        value.longValue(), ObservationBlock.class);
            } else {
                selectedOBsId[i] = Config.getCfg().tableToUniqueId(
                        value.longValue(), CalibrationBlock.class);
            }
        }
        return selectedOBsId;
    }


    public Long getSelected() {
        Class objTypeSelected;
        int row = dbbView.getSelectedRow();

        if (row == -1)
            return new Long(row);
        Integer id = (Integer) dbbView.getValueAt(row, OB_ID);
        String type = (String) dbbView.getValueAt(row, TYPE);
        if (type.equalsIgnoreCase(CalibrationBlock.OBSBLK_TYPE))
            objTypeSelected = ObservationBlock.class;
        else
            objTypeSelected = CalibrationBlock.class;

        Long dbid = new Long(Config.getCfg().tableToUniqueId(id.longValue(),
                objTypeSelected));

        return dbid;
    }

    public Media getMedia() {
        return Media.DBASE;
    }

    public Long[] getAll() {
        int selectedRows = dbbView.getRows();
        Long[] selectedOBsId = new Long[selectedRows];
        for (int i = 0; i < selectedRows; i++) {
            Integer value = (Integer) dbbView.getValueAt(i, OB_ID);
            String type = (String) dbbView.getValueAt(i, TYPE);
            if (type.equals("O")) {
                selectedOBsId[i] = new Long(Config.getCfg().tableToUniqueId(
                        value.longValue(), ObservationBlock.class));
            } else {
                selectedOBsId[i] = new Long(Config.getCfg().tableToUniqueId(
                        value.longValue(), CalibrationBlock.class));
            }
        }
        return selectedOBsId;
    }

    public long[] getAllOBIds() {
        int rows = dbbView.getRows();
        long[] selectedOBsId = new long[rows];
        for (int i = 0; i < rows; i++) {
            Integer value = (Integer) dbbView.getValueAt(i, OB_ID);
            String type = (String) dbbView.getValueAt(i, TYPE);
            if (type.equals("O")) {
                selectedOBsId[i] = Config.getCfg().tableToUniqueId(
                        value.longValue(), ObservationBlock.class);
            } else {
                selectedOBsId[i] = Config.getCfg().tableToUniqueId(
                        value.longValue(), CalibrationBlock.class);
            }
        }
        return selectedOBsId;
    }

    /**
     * @see org.eso.ohs.persistence.ObjectSelector#getDisplayedObject()
     */
    public BusinessObject getDisplayedObject() {
        Long selected = getSelected();
        if (selected.longValue() == -1)
            return null;
        ObjectManager objMgr = ObjectManager.getObjectManager();
        Class c = Config.getCfg().getClassFromId(selected.longValue());
        CalibrationBlock cb;
        try {
            cb = (CalibrationBlock) objMgr.getBusObj(Media.DBASE, selected
                    .longValue(), c);
        } catch (ObjectNotFoundException e) {
            ErrorMessages.announceNoObject(summaryTable_.getTable(), e);
            return null;
        } catch (ObjectIOException e) {
            ErrorMessages.announceIOError(summaryTable_.getTable(), e);
            return null;
        }
        return cb;
    }
    
    /**
     * Given a database ob/cb id return the type 
     * @param OBId
     * @return String
     */

    public String getObType(long OBId) {
        int row = getRowID(OBId);
        return (String) dbbView.getValueAt(row,TYPE);
    }
    
    
    public void refreshViewsStatus(long obId, String status) {
        int[] rows = dbbView.getRowsFromColumnName(new Long(obId),OB_ID);
        for (int i = 0; i < rows.length; i++) {
            setValueAt(status,rows[i],STATUS);
        }
    }

    
    /**
     *Test if the View contains a specific obId
     *@param obId specifies the ID of the OB to be checked 
     *@return true if a specific obId is contained into the view
     */

     public boolean containsOB(long obId) {
         boolean found = false;
         for (int i = 0; i < dbbView.getRows(); i++) {
             Integer value = (Integer) dbbView.getValueAt(i, OB_ID);
             if (obId == value.longValue()) {
                 found = true;
                 break;
             }
         }
         return found;
     }



    public Class getDisplayedObjectClass() {
        Long selected = getSelected();
        if (selected.longValue() == -1)
            return null;

        Class c = Config.getCfg().getClassFromId(selected.longValue());
        return c;
    }

    protected void fireObjectSelectionEvent(Media device, long id,
            Class objType, boolean forceShow, boolean forceToFront, int rowCount) {

        Object[] listeners = listenerList_.getListenerList();
        ObjectSelectionListener lnr = null;
        Class visibleObjType = getDisplayedObjectClass();
        ObjectSelectionEvent selnEvt = new ObjectSelectionEvent(this, device,
                id, visibleObjType, forceShow, forceToFront, rowCount);

        for (int i = 0; i < listeners.length; i += 2) {
            lnr = (ObjectSelectionListener) listeners[i + 1];
            lnr.objectSelected(selnEvt);
        }
    }

    /**
     * Adds a new listener to be notified of selection changes.
     * 
     * @param lnr
     *            the listener
     */
    public void addObjectSelectionListener(ObjectSelectionListener lnr) {
        listenerList_.add(ObjectSelectionListener.class, lnr);
    }

    /**
     * Removes a new listener, which will no longer be notified of selection
     * changes.
     * 
     * @param lnr
     *            the listener
     */
    public void removeObjectSelectionListener(ObjectSelectionListener lnr) {
        listenerList_.remove(ObjectSelectionListener.class, lnr);
    }

    public class ListListener implements ListSelectionListener {

        JTable summaryTable_;

        public ListListener() {
            summaryTable_ = null;
        }

        public ListListener(JTable summaryTable) {
            summaryTable_ = summaryTable;
        }

        public void valueChanged(ListSelectionEvent ev) {
            stdlog_.debug("List Selection Event " + ev);
            if (ev.getValueIsAdjusting()) {
                return;
            }
            ListSelectionModel lsm = (ListSelectionModel) ev.getSource();
            int selectedRow = lsm.getMinSelectionIndex();
            Long idObj = getSelected();

            long id = ObjectSelectionEvent.NO_OBJECT;

            if (!lsm.isSelectionEmpty()) {
                if (selectedRow >= 0
                        && selectedRow < summaryTable_.getModel().getRowCount()) {
                    id = idObj.longValue();
                }
                fireObjectSelectionEvent(Media.DBASE, id, Config
                        .getCfg()
                        .getClassFromId(id), false, true, summaryTable_
                        .getModel()
                        .getRowCount());
            }
            if (id != ObjectSelectionEvent.NO_OBJECT)
                fireObjectSelectionEvent(Media.DBASE, id, Config
                        .getCfg()
                        .getClassFromId(id), false, true, summaryTable_
                        .getModel()
                        .getRowCount());
            else
                fireObjectSelectionEvent(Media.DBASE, id,
                        ObservationBlock.class, false, true, summaryTable_
                                .getModel()
                                .getRowCount());
        }
    }

    /** Returns the ID of an OB of a specific row */
    // TODO Check if transformation of ID is neccessary
    public long getOBID(int row) {

        Integer value = (Integer) dbbView.getValueAt(row, OB_ID);
        String type = (String) dbbView.getValueAt(row, TYPE);
        if (type.equals(CalibrationBlock.OBSBLK_TYPE))
            return Config.getCfg().tableToUniqueId(value.longValue(),
                    ObservationBlock.class);

        return Config.getCfg().tableToUniqueId(value.longValue(),
                CalibrationBlock.class);
    }

    public int getRowID(long OBId) {
        for (int i = 0; i < dbbView.getRows(); i++) {
            if (OBId == getOBID(i))
                return i;
        }
        return -1;
    }

    public int[] getRowIDs(long[] OBIds) {
        Vector<Integer> resultV = new Vector<Integer>();
        for (int i = 0; i < OBIds.length; i++) {
            int row = getRowID(OBIds[i]);
            if (row >= 0)
                resultV.addElement(new Integer(row));
        }
        int[] returnSet = new int[resultV.size()];
        for (int j = 0; j < resultV.size(); j++) {
            Integer tmp = resultV.elementAt(j);
            returnSet[j] = tmp.intValue();
        }
        return returnSet;
    }

    /**
     * After OB's have been movedUp,toTop,etc. make the OB's that have moved the
     * selected items.
     * 
     * @param selOBIDs
     */

    private void setInitialOBRowSelection(long[] selOBIDs) {
        DbbTable table = dbbView.getTable();
        table.clearSelection();
        int[] newRows = getDuplicateRows(getRowIDs(selOBIDs));
        int i = 0;
        while (i < newRows.length) {
            table.getTable().addRowSelectionInterval(newRows[i], newRows[i]);
            i++;
        }    
        JTable jTable = getJTable();
        Rectangle r = jTable.getCellRect(newRows[i-1], 1, true);
        jTable.scrollRectToVisible(r);
    }

    /**
     * Return the index of all rows that contain the same OBID of a specific row
     * 
     * @param row
     *            specified the row
     */
    public int[] getDuplicatedRows(int row) {
        Vector<Integer> v = new Vector<Integer>();
       
        Integer obId = (Integer)getDbbView().getValueAt(row, OB_ID);
        // scans all the list for duplicate
        int rows = getDbbView().getRows();
        for (int i = 0; i < rows; i++) {
            Integer nextObId = (Integer)getDbbView().getValueAt(i, OB_ID);            
            if (obId.equals(nextObId)) {
                v.addElement(new Integer(i));
            }
        }
        int[] returnRows = new int[v.size()];
        for (int i = 0; i < v.size(); i++) {
            returnRows[i] = v.elementAt(i).intValue();
        }
        return returnRows;
    }

    private int[] getDuplicateRows(int[] rows) {
        LinkedHashSet<Integer> obList = new LinkedHashSet<Integer>();
        // add all the ob items
        for (int i = 0; i < rows.length; i++){
            Integer obId = (Integer)getDbbView().getValueAt(rows[i], OB_ID);       
            obList.add(obId);
        }       
        // scan the row list searching for duplicate obs
        Vector<Integer> v = new Vector<Integer>();
        int numRows = getDbbView().getRows();
        for (int i = 0; i < numRows; i++) {
            Integer nextObId = (Integer)getDbbView().getValueAt(i, OB_ID);            
            if (obList.contains(nextObId)) {
                v.addElement(new Integer(i));
            }
        }
        // converts the vector to an int array
        int[] returnRowList = new int[v.size()];
        for (int i = 0; i < v.size(); i++) {
            Integer row = v.elementAt(i);
            returnRowList[i] = row.intValue();
        }
        return returnRowList;
    }

    /** Returns a set of IDs of OBs of specific rows */
    public long[] getOBIDs(int[] rows) {
        LinkedHashSet<Long> setObIds = new LinkedHashSet<Long>();
        for (int i = 0; i < rows.length; i++) {
          //  Integer obId = (Integer) dbbView.getValueAt(rows[i], OB_ID);
            long obMgrId = getOBID(rows[i]);
            setObIds.add(new Long(obMgrId));
        }
        long[] obIDs = new long[setObIds.size()];
        Iterator<Long> cursor = setObIds.iterator();
        int i = 0;
        while (cursor.hasNext()) {
            Long obgMgrId = cursor.next();        
            obIDs[i] = obgMgrId.longValue();
            i++;
        }
        return obIDs;
    }

    /**
     * Moves down a set of rows
     * 
     * @param rows
     *            specifies the row's index to be moved down
     */
    public void moveDownOBs(int[] rows,int destRow) {
        long[] selOBIDs = getOBIDs(rows);
        int[] destRows = {};
        DbbTable table = dbbView.getTable();
        destRows = getDuplicatedRows(destRow);
        int index = (rows[rows.length - 1]) + (destRows.length);
        table.moveDownRows(rows, index);
        setInitialOBRowSelection(selOBIDs);
    }

    /**
     * Move Up rows at the level of the gui
     * 
     * @return boolean
     */

    public void moveUpOBs(int[] rows, int destRow) {
        long[] selOBIDs = getOBIDs(rows);
        int[] destRows = {};
        DbbTable table = dbbView.getTable();
        destRows = getDuplicatedRows(destRow);
        int[] moveRows = getDuplicateRows(rows);
        int index = (rows[0] - destRows.length);
        table.moveUpRows(moveRows,index);
        setInitialOBRowSelection(selOBIDs);
    }
    
    /**
     * Move Up rows at the level of the gui
     * 
     * @return boolean
     */

    public void moveUpOBsToTop(int[] rows) {
        long[] selOBIDs = getOBIDs(rows);
        int[] destRows = {};
        DbbTable table = dbbView.getTable();
        int[] moveRows = getDuplicateRows(rows);
        table.moveUpRows(moveRows,0);
        setInitialOBRowSelection(selOBIDs);
    }
    
    

    public void removeSelectedOBs(long [] selectedOBIDs) {        
        int[] rowIDs = getRowIDs(selectedOBIDs);
        removeRows(rowIDs);
    }

    public DbbView getDbbView() {
        return dbbView;
    }

    public void setDbbView(DbbView view) {
        this.dbbView = view;
    }

}