package org.eso.ohs.phase2.apps.ot.gui;

import java.awt.Component;
import java.awt.Container;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import org.eso.ohs.core.gui.models.LimitedCharsModel;
import org.eso.ohs.dfs.CalibrationBlock;

/** The CommentDialog dailog is used to modify the following OB property
 *  observer comments , status , obs grade
 *
 *  @version $Revision: 1.9 $
 *  @author  G.Passera, DMD/ESO, <gpassera@eso.org>
 */
public class CommentDialog extends JDialog {

    private JLabel qcGradeLabel = null;
    private JComboBox statusComboBox = new JComboBox(CalibrationBlock.markLabels);
     private JComboBox qcGradeComboBox;
     private JTextArea obsCommentsTextField =
         new JTextArea(new LimitedCharsModel(), "", 5, 10);
     private JTextArea userCommentsTextField =
         new JTextArea(new LimitedCharsModel(), "", 3, 10);
     private JTextArea instCommentsTextField =
         new JTextArea(new LimitedCharsModel(), "", 3, 10);

    private JButton cancelButton_ = new JButton("Cancel");
    private JButton okButton_ = new JButton("OK");

    private JPanel contentPanel = new JPanel();

    private int returnValue_ = JOptionPane.CANCEL_OPTION;

    /** Default constructor*/
    public CommentDialog() {
        super((java.awt.Frame) null, "Set Observer Parameters", true);

        Vector<String> vec = new Vector<String>();
        vec.addElement(new String(" "));
        for (int i = 0; i < CalibrationBlock.QC_GRADE_VALUES.length; i++) {
            vec.addElement(CalibrationBlock.QC_GRADE_VALUES[i]);
        }
        qcGradeComboBox = new JComboBox(vec);

        //builds the panel layout
        JPanel content = new JPanel();
        content.setLayout(new GridBagLayout());
        Container c = content;
        JScrollPane pane = new JScrollPane(obsCommentsTextField);
         JLabel obsCommentsLabel = new JLabel("Observer comments");
         JLabel statusLabel = new JLabel("Status");
        qcGradeLabel = new JLabel("QC grade");
         JLabel userCommentLabel = new JLabel("User comment");
         JLabel instCommentLabel = new JLabel("Instrument comment");
        addComponent(c,statusLabel,statusComboBox,1,0);
        addComponent(c,qcGradeLabel,qcGradeComboBox,1,1);
         addComponent(c,obsCommentsLabel,pane,3,2);

         pane = new JScrollPane(userCommentsTextField);
         userCommentsTextField.setEnabled(false);
         userCommentsTextField.setWrapStyleWord(true);
         userCommentsTextField.setLineWrap(true);
         addComponent(c,userCommentLabel,pane,3,6);
         pane = new JScrollPane(instCommentsTextField); 
         instCommentsTextField.setEnabled(false);
         instCommentsTextField.setLineWrap(true);
         instCommentsTextField.setWrapStyleWord(true);
         addComponent(c,instCommentLabel,pane,3,9);
        
        GridBagAdder.add(
            c,
            okButton_,
            0,
            16,
            1,
            1,
            0,
            0,
            GridBagConstraints.BOTH,
            GridBagConstraints.CENTER);

        GridBagAdder.add(
            c,
            cancelButton_,
            3,
            16,
            1,
            1,
            0,
            0,
            GridBagConstraints.VERTICAL,
            GridBagConstraints.EAST);

        getContentPane().add(c);

        okButton_.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                returnValue_ = JOptionPane.OK_OPTION;
                setVisible(false);
            }
        });
        cancelButton_.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                returnValue_ = JOptionPane.CANCEL_OPTION;
                setVisible(false);
            }
        });
        this.pack();
    }

    /**
     * @param c void
     * @param i 
     * @param i 
     */
    private void addComponent(Container c,Component label,Component widget, int numRowsWidget, int startRowPosition)  {
        
        GridBagAdder.add(
            c,
            label,
            0,
            startRowPosition,
            2,
            1,
            0,
            0,
            GridBagConstraints.HORIZONTAL,
            GridBagConstraints.CENTER);
        GridBagAdder.add(
                c,
                widget,
                2,
                startRowPosition,
                2,
                numRowsWidget,
                1,
                0,
                GridBagConstraints.HORIZONTAL,
                GridBagConstraints.CENTER);
    }

    public int getUserChoice() {
        return returnValue_;
    }

    public void setQCVisible(boolean visible) {
        qcGradeLabel.setVisible(visible);
        qcGradeComboBox.setVisible(visible);
    }

    /**Returns the selected status*/
    public String getStatus() {
        return CalibrationBlock.getStatusCode(statusComboBox.getSelectedIndex());
    }

    /**Returns  the selected QCGrade*/
    public String getQcGrade() {
        return (String) qcGradeComboBox.getSelectedItem();
    }

    /**Retruns the observer comment*/
    public String getObsComment() {
        if (obsCommentsTextField.getText() != null)
            return obsCommentsTextField.getText();
        else
            return " ";
    }

    /**Returns the selected status*/
    public void setStatus(String status) {
        statusComboBox.setSelectedItem(status);
    }

    /**Returns  the selected QCGrade*/
    public void setQcGrade(String grade) {
        if(grade != null && grade.length() > 0)
           qcGradeComboBox.setSelectedItem(grade);
        else   
           qcGradeComboBox.setSelectedIndex(0);   
    }

    /**Retruns the observer comment*/
    public void setObsComment(String comments) {
        obsCommentsTextField.setText(comments);
    }

    public void setUserComment(String comments) {
        userCommentsTextField.setText(comments);
        
    }

    public void setInstComment(String comments) {
        instCommentsTextField.setText(comments);
        
    }

}
