package org.eso.ohs.phase2.apps.ot.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.InetAddress;
import java.net.MalformedURLException;
import java.net.UnknownHostException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.sql.SQLException;

import javax.swing.AbstractButton;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.JToolBar;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.client.DbaseDec;
import org.eso.ohs.core.dbb.client.DbaseRA;
import org.eso.ohs.core.dbb.client.DbbArrowButton;
import org.eso.ohs.core.dbb.client.DbbExecutionTime;
import org.eso.ohs.core.dbb.client.DbbPanel;
import org.eso.ohs.core.dbb.client.DbbTextField;
import org.eso.ohs.core.dbb.client.DbbView;
import org.eso.ohs.core.dbb.client.DisplayTableAsTextAction;
import org.eso.ohs.core.dbb.client.MoonDistance;
import org.eso.ohs.core.dbb.server.DbbSqlEngine;
import org.eso.ohs.core.dbb.sql.DbbSqlChunk;
import org.eso.ohs.core.dbb.sql.DbbSqlOperator;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.utilities.MsgManager;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.CommentType;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.OBEvent;
import org.eso.ohs.persistence.OBListener;
import org.eso.ohs.persistence.dbase.phase2.QueueManager;
import org.eso.ohs.phase2.actions.ShowFindingChartsAction;
import org.eso.ohs.phase2.apps.ot.actions.DeselectAllAction;
import org.eso.ohs.phase2.apps.ot.actions.DisplayOBAction;
import org.eso.ohs.phase2.apps.ot.actions.MoveDownRowsAction;
import org.eso.ohs.phase2.apps.ot.actions.MoveUpRowsAction;
import org.eso.ohs.phase2.apps.ot.actions.OBHistoryAction;
import org.eso.ohs.phase2.apps.ot.dbb.BrowserConfig;
import org.eso.ohs.phase2.apps.ot.dbb.OTDbbView;

/**
 * The QueueItemsBView class is a DB search engine used for displaying queues of
 * OBs according to a user-definable set of properties
 * 
 * @version $Revision: 1.44.4.1 $
 * @author Gorgio Passera DMD/ESO <gpassera@eso.org> <br>
 *         ChangeLog: cmavros 01/07/02 - Add local method addDefaultColumns()
 */
public class ExecutionSequenceBView extends JPanel implements TableMenuEnable, TreeViewUpdate,OBListener  {

   
    private static final long serialVersionUID = 1L;

    public static final String cvsID_ = "$Id: ExecutionSequenceBView.java,v 1.44.4.1 2007/10/05 11:19:32 pnunes Exp $";

    /** Standard log channel */
    private static Logger stdlog_ = Logger
            .getLogger(ExecutionSequenceBView.class);

    private JButton moveToTopButton;

    private JButton removeAllButton;

    private JButton displayTextButton;

    private static ExecutionSequenceBView execSequence_ = new ExecutionSequenceBView(new OBTreeView());

    private OBTreeView tree_;
   
   
    private String hostName = null;

    private OTDbbView view_;

    private DbbView dbbView_;

    private TableListSelectionListener listListener_;

    private TreeViewClearer treeViewClearer_;

    private DeselectAllAction deselectAllAction;

    private MoveUpRowsAction moveUpAction;

    private MoveDownRowsAction moveDownAction;

    private DisplayOBAction displayAction;

    private OBHistoryAction obHistoryAction;

    private ShowFindingChartsAction findingChartAction;

    private AbstractButton removeButton;

    private JButton moveUpButton;

    private AbstractButton moveDownButton;
    
    
 

    /**
     * Default constructor <br>
     * ChangeLog: Gorgio Passera <DMD/ESO> <gpassera@eso.org>
     * @throws SQLException 
     * 
     */
    private ExecutionSequenceBView(OBTreeView obTreeView) {
        tree_ = obTreeView;       
        try {
            view_ = new OTDbbView();
            dbbView_ = view_.getDbbView();
            buildColumns();
            buildRows();
            buildSort();
            layoutBrowser();
            listListener_ = new TableListSelectionListener(this);
            dbbView_.addListSelectionListener(listListener_);
            treeViewClearer_ = new TreeViewClearer(this);
            dbbView_.addListSelectionListener(treeViewClearer_);
            dbbView_.getTable().removeSortListener();
            QueueManager.addOBListener(this);
            getDbbView().hideColumn(OTDbbView.TYPE);
        } catch (SQLException e) {
            MsgManager.errDialog("Unable to initialise Execution Sequence",e);
        } catch (MalformedURLException e) {
            MsgManager.errDialog("Unable to initialise Execution Sequence",e);
        } catch (RemoteException e) {
            MsgManager.errDialog("Unable to initialise Execution Sequence",e);
        } catch (NotBoundException e) {
            MsgManager.errDialog("Unable to initialise Execution Sequence",e);
        }                       
    }
    
    public void sqlStartAction() {
        view_.sqlStartAction();       
    }


    public void layoutBrowser() {
        setLayout(new BorderLayout());        
        setSize(new Dimension(650, 600));
        /* JSplitPane */
        JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, dbbView_,
                tree_);

        splitPane.setDividerLocation(300);
        JToolBar toolBar = new JToolBar();
        add(splitPane, BorderLayout.CENTER);
        add(splitPane, BorderLayout.CENTER);
        add(toolBar, BorderLayout.SOUTH);
        add(new JLabel("Execution Sequence"), BorderLayout.NORTH);
        addToolBarItems(toolBar);

    }

    public static ExecutionSequenceBView getInstance() {
        return execSequence_;
    }

    public void obPropertyChange(OBEvent ev) {
        long[] objMgrObIds = ev.getOBsId();
        if (ev.getValue().equals("Delete")) {  
            if (objMgrObIds != null && objMgrObIds.length > 0)
                removeOBs(objMgrObIds);
        } else {
            for (int i = 0; i < objMgrObIds.length; i++) {
                long obId = Config.getCfg().uniqueToTableId(objMgrObIds[i]);   
                if (view_.containsOB(obId)) {
                    view_.refreshViewsStatus(obId, ev.getValue());
                }
            }
        }
    }

    public void removeOBs(long[] selectedOBIDs) {
        int[] rowIDs = view_.getRowIDs(selectedOBIDs);
        view_.removeRows(rowIDs);
    }

    protected void addToolBarItems(JToolBar toolBar) {
        createActions();
        toolBar.setFloatable(false);
        removeAllButton = new JButton("Remove All");
        toolBar.add(removeAllButton);

        removeAllButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                try {
                    QueueManager.deleteOBFromOTExecSequence(view_.getAllOBIds());
                } catch (ObjectIOException e) {
                    ErrorMessages.announceIOError(dbbView_.getTable().getTable(),
                            e);
                }
            }

        });

        removeButton = new JButton("Remove");
        removeButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                try {
                    QueueManager
                            .deleteOBFromOTExecSequence(view_.getAllSelectedOBIds());
                } catch (ObjectIOException e) {
                    ErrorMessages.announceIOError(dbbView_.getTable().getTable(),
                            e);
                }
                // removeOBs();
            }

        });

        toolBar.add(removeButton);

        moveUpButton = new JButton("Move Up");
        moveUpButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ev) {
                try {
                    moveUpRows();
                    QueueManager.updateExecOBsSequences(view_.getAllOBIds());
                } catch (ObjectIOException e) {
                    ErrorMessages.announceIOError(dbbView_.getTable().getTable(),
                            e);
                }
            }
        });

        moveDownButton = new JButton("Move Down");
        moveDownButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ev) {
                try {
                    moveDownRows();
                    QueueManager.updateExecOBsSequences(view_.getAllOBIds());
                } catch (ObjectIOException e) {
                    ErrorMessages.announceIOError(dbbView_.getTable().getTable(),
                            e);
                }
            }

        });

        moveToTopButton = new JButton("Move To Top");
        moveToTopButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ev) {
                try {
                    moveRowsToTop();
                    QueueManager.updateExecOBsSequences(view_.getAllOBIds());
                } catch (ObjectIOException e) {
                    ErrorMessages.announceIOError(dbbView_.getTable().getTable(),
                            e);
                }
            }
        });

        displayTextButton = new JButton();

        toolBar.add(moveUpButton);
        toolBar.add(moveDownButton);
        toolBar.add(moveToTopButton);
        toolBar.add(displayAction);
        toolBar.add(obHistoryAction);
        toolBar.add(findingChartAction);
        toolBar.add(displayTextButton);
        setEnabled(false);
    }

    public void setDisplayTextAction(JFrame frame) {
        displayTextButton.setAction(new DisplayTableAsTextAction(dbbView_,
                "Execution Sequence", "Execution Sequence", frame));
        displayTextButton.setText("Text View");
    }

    public void setEnabled(boolean enabled) {
        removeAllButton.setEnabled(enabled);
        removeButton.setEnabled(enabled);
        moveUpButton.setEnabled(enabled);
        moveDownButton.setEnabled(enabled);
        moveToTopButton.setEnabled(enabled);
        displayAction.setEnabled(enabled);
        obHistoryAction.setEnabled(enabled);
        findingChartAction.setEnabled(enabled);
        displayTextButton.setEnabled(enabled);
    }

    /**
     * Return a set of OBsID that belong to the execution sequence
     * 
     * @param maxNumOBs
     *            specifies the number of ObsId requested
     * @return a set of OBsID that belong to the execution sequence or null
     */
    public long[] getOBsIdFromExecutionSequence(int maxNumOBs) {
        try {

            if (maxNumOBs > dbbView_.getRows()) {
                maxNumOBs = dbbView_.getRows();
            }
            long[] returnList = new long[maxNumOBs];
            for (int i = 0; i < maxNumOBs; i++) {
                returnList[i] = view_.getOBID(i);
            }
            return returnList;
        } catch (NumberFormatException e) {
            stdlog_.error(e);
            return null;
        }
    }

    public boolean isOBSelected() {
        int[] selectedRows = dbbView_.getSelectedRows();
        return (selectedRows.length != 0);
    }

    /**
     * Build DB browser view: SELECT panel
     * 
     * <br>
     * ChangeLog: Gorgio Passera
     * 
     * @exception SQLException
     *                JDBC exception occurs
     * @exception RemoteException
     *                RMI exception occurs
     * @exception MalformedURLException
     *                malformed url i.e. servername//port
     * @exception NotBoundException
     *                client could not bind to server
     */
    private void buildColumns()
            throws NotBoundException, MalformedURLException, SQLException,
            RemoteException {
        String label_name;
        String column_name;
        column_name = "ob_id";
        label_name = "OB ID";
        DbbSqlEngine engine = dbbView_.getSqlEngine();
        engine.addDefaultColumnsChunk(new DbbSqlChunk(BrowserConfig.oes, column_name,
                label_name));
        DbbPanel panel = new DbbPanel("Selected Columns", 9);
        dbbView_.setColumnsPanel(panel, "North", false);
        // Type
        column_name = "type";
        label_name = "Type";
        engine.addDefaultColumnsChunk(new DbbSqlChunk(BrowserConfig.ob, column_name,
                label_name));

        // Status
        column_name = "status";
        label_name = "Status";
        engine.addDefaultColumnsChunk(new DbbSqlChunk(BrowserConfig.ob, column_name,
                label_name));

        // OB Name
        label_name = "OB Name";
        column_name = "item_name";
        engine.addDefaultColumnsChunk(new DbbSqlChunk(BrowserConfig.ob, column_name,
                label_name));
        
        
        column_name = "userComments";
        label_name = "OB Comment";
        engine.addDefaultColumnsChunk(new DbbSqlChunk(BrowserConfig.ob, column_name,
                label_name,CommentType.class));
        
        column_name = "inst_comments";
        label_name = "Inst Comment";
        engine.addDefaultColumnsChunk(new DbbSqlChunk(BrowserConfig.ob, column_name,
                label_name,CommentType.class));

        

        

        label_name = "Instrument";
        column_name = "instrument";

        engine.addDefaultColumnsChunk(new DbbSqlChunk(BrowserConfig.od, column_name,
                label_name));

        // ProgID
        label_name = "ProgID";
        column_name = "progid";
        engine.addDefaultColumnsChunk(new DbbSqlChunk(BrowserConfig.sr, column_name,
                label_name, String.class));

        // PI (surname)
        label_name = "LastName";
        column_name = "last_name";
        engine.addDefaultColumnsChunk(new DbbSqlChunk(BrowserConfig.profiles, column_name,
                label_name));

        // RA
        label_name = "RA";
        column_name = "ra";
        engine.addDefaultColumnsChunk(new DbbSqlChunk(BrowserConfig.tp, column_name,
                label_name, DbaseRA.class));

        // Dec
        label_name = "Dec";
        column_name = "dec";
        engine.addDefaultColumnsChunk(new DbbSqlChunk(BrowserConfig.tp, column_name,
                label_name, DbaseDec.class));

        // Exec Time
        label_name = "ExecTime";
        column_name = "execution_time";
        engine.addDefaultColumnsChunk(new DbbSqlChunk(BrowserConfig.ob, column_name,
                label_name, DbbExecutionTime.class));

        // Rank Class
        label_name = "RankClass";
        column_name = "rank_class";
        engine.addDefaultColumnsChunk(new DbbSqlChunk(BrowserConfig.sr, column_name,
                label_name, String.class));

        // UsrP
        label_name = "UsrP";
        column_name = "userPriority";
        engine.addDefaultColumnsChunk(new DbbSqlChunk(BrowserConfig.ob, column_name,
                label_name));

        // Seeing
        label_name = "Seeing";
        column_name = "seeing";
        engine.addDefaultColumnsChunk(new DbbSqlChunk(BrowserConfig.cs, column_name,
                label_name));

        // Airmass
        label_name = "Airmass";
        column_name = "airmass";
        engine.addDefaultColumnsChunk(new DbbSqlChunk(BrowserConfig.cs, column_name,
                label_name));

        // Sky Tran
        label_name = "SkyTran";
        column_name = "sky_transparency";
        engine.addDefaultColumnsChunk(new DbbSqlChunk(BrowserConfig.cs, column_name,
                label_name));

        // FLI
        label_name = "FLI";
        column_name = "fli";
        engine.addDefaultColumnsChunk(new DbbSqlChunk(BrowserConfig.cs, column_name,
                label_name));

        // MoonDis
        label_name = "MoonDis";
        column_name = "moon_distance";
        engine.addDefaultColumnsChunk(new DbbSqlChunk(BrowserConfig.cs, column_name,
                label_name, MoonDistance.class));

        // Strehl
        label_name = "Strehl";
        column_name = "strehl_ratio";
        engine.addDefaultColumnsChunk(new DbbSqlChunk(BrowserConfig.cs, column_name,
                label_name));

        // label_name = "Finding Charts";
        // column_name = "fc_name";
        // engine.addDefaultColumnsChunk(
        // new DbbSqlChunk(finds_view, column_name, label_name));

        // label_name = "hostname";
        // column_name = "hostname";
        // engine.addDefaultColumnsChunk(
        // new DbbSqlChunk(BrowserConfig.oes, column_name, label_name));

        
        
        
        
    }

    /** Build DB browser view: rows selection panel */
    public void buildRows() {

        DbbPanel panel = new DbbPanel("Selection Criteria", 5);
        DbbSqlChunk c;

        String label_name;
        String column_name;

        DbbSqlOperator op_equal = null;

        try {
            op_equal = new DbbSqlOperator("=");
        } catch (Exception e) {
            stdlog_.error(e);
        }

        // Queue ID
        /*
         * label_name = "Queue ID"; column_name = "sch_id";
         * 
         * c = new DbbSqlChunk( es_table, column_name, label_name, op_equal,
         * Integer.class); queueIDTF = new DbbTextField(label_name, c);
         * panel.addWidget(queueIDTF);
         * 
         * queueIDTF.set(new
         * Long(Config.getCfg().uniqueToTableId(queue.getId())));
         * queueIDTF.setEnabled(false);
         */

        label_name = "hostname";
        column_name = "hostname";
        c = new DbbSqlChunk(BrowserConfig.oes, column_name, label_name, op_equal,
                String.class);
        DbbTextField hostTextField = new DbbTextField(label_name, c,false);
        panel.addWidget(hostTextField);
        dbbView_.setWherePanel(panel, "North", false);
        
        hostTextField.set(getHostName());
        hostTextField.setEnabled(false);
    }

    /**
     * Method to append OBs that pass verification test to the Execution
     * Sequence View. Called by endOfAction method in
     * ot.actions.ExecutionSequenceAction
     * 
     * @param setOBIDs
     *            list of OBs to be added to the Execution Sequence View <br>
     *            ChangeLog: cmavros 12/03/02 1709/OT
     */
    public boolean appendOBs(long[] appendIds) {
        JTable table = dbbView_.getTable().getTable();
        stdlog_.debug("Column Count " + table.getColumnCount());
        String hostName = getHostName();
        try {
            QueueManager.appendOBsToExecSequence(appendIds, dbbView_.getRows(),
                    hostName);
            view_.sqlStartAction();
        } catch (ObjectIOException e) {
            ErrorMessages.announceIOError(table, e);
        }
        return true;
    }

    /**
     * @param hostName
     * @return String
     */
    private String getHostName() {
        if (hostName == null) {
            try {
              hostName = getHostNameNODot();
                stdlog_.debug("IP hostname: " + hostName);
            } catch (UnknownHostException e) {
                JTable table1 = dbbView_.getTable().getTable();
                String[] text = new String[1];
                text[0] = "Cannot resolve IP host name.";
                ErrorMessages.announceError(table1, text, e);
            }
        }
        return hostName;
    }

    /**
     * @throws UnknownHostException void
     */
    public static  String getHostNameNODot() throws UnknownHostException {
        String hostName = InetAddress.getLocalHost().getHostName().trim();
        int indexOfDot = hostName.indexOf(".");
        if (indexOfDot>= 0) {
            hostName= hostName.substring(0,indexOfDot);
        }
        return hostName;
    }

    
//TODO Removw type column as in QueueView.    

//    private class QueryTerminatedListener implements DbbQueryTerminatedListener {
//        public void queryTerminatedAction(DbbQueryTerminated ev) {
//            // now we have a full table we can remove the typ
//            // column from the view
//            JTable table = dbbView_.getTable().getTable();
//            TableColumn typeColumn = table.getColumn("Type");
//            table.removeColumn(typeColumn);
//        }
//
//    }

    /** Build DB browser view: sort selection panel */
    private void buildSort() {

        DbbPanel panel = new DbbPanel("Sorting Criteria", 1);
        DbbSqlChunk c;
        DbbArrowButton w;
        String column_name;
        String label_name;

        // sequence
        label_name = "sequence";
        column_name = "sequence";

        c = new DbbSqlChunk(BrowserConfig.oes, column_name, label_name);
        w = new DbbArrowButton(label_name, c, 1);

        panel.addWidget(w);
        w.actionPerformed(new ActionEvent(this, 0, null));

        dbbView_.setSortPanel(panel, "West", false);
    }

    
    private void createActions() {

        displayAction = new DisplayOBAction(view_, tree_);

        obHistoryAction = new OBHistoryAction(view_);
        findingChartAction = new ShowFindingChartsAction(Media.DBASE, view_,
                view_.getJTable(), false);
    }



    public void updateView(CalibrationBlock object) {
        // TODO Auto-generated method stub
        
    }
  
    public void moveUpRows() {
        int[] rows = dbbView_.getSelectedRows();
        if ((rows.length > 0) && (rows[0] - 1 >= 0)) {
            view_.moveUpOBs( rows, rows[0] - 1);

        }
    } 
    public void moveRowsToTop() {
        int[] rows = dbbView_.getSelectedRows();
        if (rows.length > 0) {
            view_.moveUpOBsToTop(rows);
        }
    }
    
    public void moveDownRows() {
        int[] rows = dbbView_.getSelectedRows();
        if ((rows.length > 0)
                && (rows[rows.length - 1] + 1 < dbbView_
                        .getTable()
                        .getRowCount())) {
            int destRow = rows[rows.length - 1] + 1;        
            view_.moveDownOBs(rows, destRow);
        }
    }


    public DbbView getDbbView() {
        return dbbView_;
    }


    public OTDbbView getView() {
        return view_;
    }


    
}
