package org.eso.ohs.phase2.apps.ot.gui;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JMenu;

import org.eso.ohs.core.gui.utilities.MenuUtils;
import org.eso.ohs.core.utilities.OHSKeys;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectContainer;
import org.eso.ohs.phase2.actions.ExpTimeReportAction;
import org.eso.ohs.phase2.actions.MissingTemplatesReport;
import org.eso.ohs.phase2.actions.OBReportAction;
import org.eso.ohs.phase2.actions.OrangVisiplotAction;
import org.eso.ohs.phase2.actions.PrintFindingCharts;
import org.eso.ohs.phase2.actions.ReadmeStatusHistoryAction;
import org.eso.ohs.phase2.actions.SaveFCAction;
import org.eso.ohs.phase2.actions.ShowAcceptedReadmeAction;
import org.eso.ohs.phase2.actions.ShowPAFDescrAction;
import org.eso.ohs.phase2.actions.ShowReadmeFileHistoryAction;
import org.eso.ohs.phase2.actions.Stage2VerifyBOAction;
import org.eso.ohs.phase2.actions.USGEditAction;
import org.eso.ohs.phase2.actions.ViewEphFileAction;
import org.eso.ohs.phase2.actions.VisiplotAction;
import org.eso.ohs.phase2.apps.ot.actions.ChileNotesReadmeAction;
import org.eso.ohs.phase2.apps.ot.actions.ConfigReportAction;
import org.eso.ohs.phase2.apps.ot.actions.JustifyRankingAction;
import org.eso.ohs.phase2.apps.ot.actions.OrangShortGUIStartAction;
import org.eso.ohs.phase2.apps.ot.actions.OrangStandardGUIStartAction;
import org.eso.ohs.phase2.apps.ot.actions.OrangStartAction;
import org.eso.ohs.phase2.apps.ot.wizard.OrangEngine;
import org.eso.ohs.phase2.orang.OBScheduleInfo;
import org.eso.ohs.phase2.visiplot.VisiPlotOptions;

/**
 * Class MenuCreator
 *
 * Creates the menus for the various panel of the OT.
 *
 * @version $Revision: 1.15.4.1 $
 * @author ddorigo, Jan 18, 2006
 */

// $Id: MenuCreator.java,v 1.15.4.1 2007/01/11 14:09:53 pnunes Exp $
public class MenuCreator {

    private JMenu expMenu;    
    private JFrame frame;
    private JMenu printFCMenu;
    private JMenu saveFCMenu;
    private JMenu obMenu;
    private JMenu verifyMenu;
    private JMenu configReportMenu;
    private JMenu missingTplMenu;
    private JMenu showPAFDescriptionMenu;
    private JMenu visiplotMenu;
    private JMenu orangMenu;
    private JMenu readmeMenu;
    private JMenu justifyRankingMenu;   // amchavan, 01-Mar-2005   

    private ViewEphFileAction ephViewAction;
    
    private OrangVisiplotAction visiplotActionSelected;
    
    private OrangVisiplotAction visiplotActionAll;
        
    private JustifyRankingAction justifyRankingActionSelected;
    
    private JustifyRankingAction justifyRankingActionAll; 


    /**
     * @param frame the menu creator will attach the menus to this class
     */
    public MenuCreator(JFrame frame) {
        this.frame = frame;
    }

    /**
     * @param container  contains all the objects managed by the actions
     * @param comp 
     */
    public void createFCPrintMenu(ObjectContainer container, JComponent comp) {
        String menuName = "Finding Charts";
        if (printFCMenu == null)
            printFCMenu = new JMenu("Print");
        printFCMenu.add(
            new PrintFindingCharts(Media.DBASE, container,comp, "All", true));
        printFCMenu.add(
            new PrintFindingCharts(Media.DBASE, container,comp, "Selected", false));
        MenuUtils.addSubMenu(frame,menuName, printFCMenu);
        String saveMenu = "Save As jpg";
        if (saveFCMenu == null)
            saveFCMenu = new JMenu(saveMenu);            
        saveFCMenu.add(new SaveFCAction(Media.DBASE, container,comp, "All",true));        
        saveFCMenu.add(new SaveFCAction(Media.DBASE, container,comp, "Selected", false));    
        MenuUtils.addSubMenu(frame,menuName,saveFCMenu);
    }
    
    /**
     * @param container  contains all the objects managed by the actions
     */
    public void createReportsMenu(ObjectContainer container,JComponent comp) {
        String menuName = "Reports";
        if (expMenu == null)
            expMenu = new JMenu("Execution Time");
        expMenu.add(new ExpTimeReportAction(container,comp, true, "All"));
        expMenu.add(new ExpTimeReportAction(container,comp, false, "Selected"));

        if (obMenu == null)
            obMenu = new JMenu("ObsBlocks Breakdown");
        obMenu.add(new OBReportAction(container,comp, true, "All"));
        obMenu.add(new OBReportAction(container,comp, false, "Selected"));

        if (verifyMenu == null)
            verifyMenu = new JMenu("Verify");
        verifyMenu.add(new Stage2VerifyBOAction(container,comp, true, "All"));
        verifyMenu.add(new Stage2VerifyBOAction(container,comp, false, "Selected"));
        if (configReportMenu == null)
            configReportMenu = new JMenu("Config Reports");
        configReportMenu.add(new ConfigReportAction(container,comp, true, "All"));
        configReportMenu.add(
            new ConfigReportAction(container,comp, false, "Selected"));

        // amchavan, 21-Oct-2001
        if (missingTplMenu == null)
            missingTplMenu = new JMenu("Missing Templates");
        missingTplMenu.add(new MissingTemplatesReport(container,comp, true, "All"));
        missingTplMenu.add(
            new MissingTemplatesReport(container,comp, false, "Selected"));

        if (showPAFDescriptionMenu == null)
            showPAFDescriptionMenu = new JMenu("Display in OBD (PAF) format");
        showPAFDescriptionMenu.add(
            new ShowPAFDescrAction(container,comp, true, "All"));
        showPAFDescriptionMenu.add(
            new ShowPAFDescrAction(container,comp, false, "Selected"));

        createVisiplotMenu(container,comp);

        MenuUtils.addSubMenu(frame,menuName, expMenu);
        MenuUtils.addSubMenu(frame,menuName, obMenu);
        MenuUtils.addSubMenu(frame,menuName, configReportMenu);
        MenuUtils.addSubMenu(frame,menuName, verifyMenu);
        MenuUtils.addSubMenu(frame,menuName, missingTplMenu); // amchavan, 21-Oct-2001: 
        MenuUtils.addSubMenu(frame,menuName, showPAFDescriptionMenu);
        MenuUtils.addSubMenu(frame,menuName, visiplotMenu);
    }    

    
    public void createOrangReportsMenu(ObjectContainer container,JComponent comp,VisiPlotOptions currentSettings,OBScheduleInfo[] obs) {
		String menuName = "Reports";
    	createStandardReports(container, comp);
        createOrangVisiplotMenu(container,comp,currentSettings,obs);
        MenuUtils.addSubMenu(frame,menuName, visiplotMenu);
    }


	private void createStandardReports(ObjectContainer container, JComponent comp) {
		String menuName = "Reports";
        if (expMenu == null)
            expMenu = new JMenu("Execution Time");
        expMenu.add(new ExpTimeReportAction(container,comp, true, "All"));
        expMenu.add(new ExpTimeReportAction(container,comp, false, "Selected"));

        if (obMenu == null)
            obMenu = new JMenu("ObsBlocks Breakdown");
        obMenu.add(new OBReportAction(container,comp, true, "All"));
        obMenu.add(new OBReportAction(container,comp, false, "Selected"));

        if (verifyMenu == null)
            verifyMenu = new JMenu("Verify");
        verifyMenu.add(new Stage2VerifyBOAction(container,comp, true, "All"));
        verifyMenu.add(new Stage2VerifyBOAction(container,comp, false, "Selected"));
        if (configReportMenu == null)
            configReportMenu = new JMenu("Config Reports");
        configReportMenu.add(new ConfigReportAction(container,comp, true, "All"));
        configReportMenu.add(
            new ConfigReportAction(container,comp, false, "Selected"));

        // amchavan, 21-Oct-2001
        if (missingTplMenu == null)
            missingTplMenu = new JMenu("Missing Templates");
        missingTplMenu.add(new MissingTemplatesReport(container,comp, true, "All"));
        missingTplMenu.add(
            new MissingTemplatesReport(container,comp, false, "Selected"));

        if (showPAFDescriptionMenu == null)
            showPAFDescriptionMenu = new JMenu("Display in OBD (PAF) format");
        showPAFDescriptionMenu.add(
            new ShowPAFDescrAction(container,comp, true, "All"));
        showPAFDescriptionMenu.add(
            new ShowPAFDescrAction(container,comp, false, "Selected"));
        MenuUtils.addSubMenu(frame,menuName, expMenu);
        MenuUtils.addSubMenu(frame,menuName, obMenu);
        MenuUtils.addSubMenu(frame,menuName, configReportMenu);
        MenuUtils.addSubMenu(frame,menuName, verifyMenu);
        MenuUtils.addSubMenu(frame,menuName, missingTplMenu); // amchavan, 21-Oct-2001: 
        MenuUtils.addSubMenu(frame,menuName, showPAFDescriptionMenu);        
	}
    
    /**
     * @param container  contains all the objects managed by the actions
     */
    public void createOrangMenu(Orang orang) {
        String menuName = OT.ORANG_MENU;
           orangMenu = MenuUtils.getMenu(frame,menuName);
           orangMenu.add(new OrangStartAction(frame,OT.WIZARD,orang));
           orangMenu.add(new OrangStandardGUIStartAction(frame,OT.PANEL,orang));
           orangMenu.add(new OrangShortGUIStartAction(frame,OT.SMALL,orang));
        
    }
    /**
     * @param container contains all the objects managed by the actions
     */
    public void createVisiplotMenu(ObjectContainer container,JComponent comp) {
        if (visiplotMenu == null)
            visiplotMenu = new JMenu("Visibility Plots");
        visiplotMenu.add(new VisiplotAction(container,comp, true, "All"));
        visiplotMenu.add(new VisiplotAction(container,comp, false, "Selected"));
    }
    
    
    private void createOrangVisiplotMenu(ObjectContainer container,JComponent comp,VisiPlotOptions currentSettings,OBScheduleInfo [] obs) {
        if (visiplotMenu == null) {
            visiplotMenu = new JMenu("Visibility Plots");
            visiplotActionAll = new OrangVisiplotAction(container, true, "All observation blocks (Orang)",
                    currentSettings,obs);
            visiplotActionSelected = new OrangVisiplotAction(container, false,
                    "Selected observation blocks (Orang)", currentSettings,obs);
            
            visiplotMenu.add(visiplotActionAll);
            visiplotMenu.add(visiplotActionSelected);
        } 
        MenuUtils.addSubMenu(frame,"Reports", visiplotMenu);
        
    }

    
    public void reinitFCMenu(ObjectContainer container,JComponent comp) {
        if (printFCMenu != null)
            MenuUtils.removeMenuItems(printFCMenu);
        if (saveFCMenu != null) 
            MenuUtils.removeMenuItems(saveFCMenu);        	
        createFCPrintMenu(container, comp);
    }
    
    
    public void reinitReportsMenu(ObjectContainer container,JComponent comp) {
        if (expMenu != null)
            MenuUtils.removeMenuItems(expMenu);
        if (obMenu != null)
            MenuUtils.removeMenuItems(obMenu);
        if (verifyMenu != null)
            MenuUtils.removeMenuItems(verifyMenu);
        if (configReportMenu != null)
            MenuUtils.removeMenuItems(configReportMenu);
        if (missingTplMenu != null)
            MenuUtils.removeMenuItems(missingTplMenu);
        if (showPAFDescriptionMenu != null)
            MenuUtils.removeMenuItems(showPAFDescriptionMenu);
        if (visiplotMenu != null)
            MenuUtils.removeMenuItems(visiplotMenu);
        createReportsMenu(container,comp);
    }

    
    

    /**
     * @param container  contains all the objects managed by the actions
     * @param comp 

     */
    public void createReadmeMenu(ObjectContainer container, JComponent comp) {
        String menuName = "Readme";
        if (readmeMenu == null)
            readmeMenu = MenuUtils.getMenu(frame, menuName);
        if(AppConfig.getAppConfig().getBoolean(OHSKeys.USGEnabledKey)) {
            readmeMenu.add(new ShowReadmeFileHistoryAction(container,frame,"View Current Readme"));
            readmeMenu.add(new ShowAcceptedReadmeAction(container,frame,"View Readme At The Mountain"));
            readmeMenu.add(new ReadmeStatusHistoryAction(container));
            readmeMenu.add(new USGEditAction(container));
        } else {
            readmeMenu.add(new ShowAcceptedReadmeAction(container,frame,"View Readme"));
            readmeMenu.add(new ChileNotesReadmeAction(container,frame,"Readme Notes"));
            //readmeMenu.add(new USGEditAction(container));
        }
    }


    
    /**
     * Creates a menu for ephemeris
     * @param view view to which the menu is attached.
     */
    public void createEphFileMenu(ObjectContainer view,JComponent comp) {
        final String menuName = "Ephemeris Files";
        ephViewAction = new ViewEphFileAction(view,comp, Media.DBASE);
        JMenu ephMenu = MenuUtils.getMenu(frame,menuName);
        ephMenu.add(ephViewAction);
    }
    
    
    
    public  void overrideVisiplotMenu(ObjectContainer container, VisiPlotOptions currentSettings, OBScheduleInfo [] obs) {
    	
    	MenuUtils.removeMenuItems(visiplotMenu);    	
    	        
        visiplotMenu.add(new OrangVisiplotAction(container, true, "All observation blocks (Orang)",
                currentSettings,obs));
        visiplotMenu.add(new OrangVisiplotAction(container, false,
                "Selected observation blocks (Orang)", currentSettings,obs));
        
        MenuUtils.addSubMenu(frame,"Reports", visiplotMenu);
    }
    
    // amchavan, 01-Mar-2006: added
    public void addJustifyRankingMenu( ObjectContainer container, 
                                       JComponent comp, 
                                       OrangEngine engine ) {
        if( justifyRankingMenu == null ) {
            justifyRankingMenu = new JMenu( "Justify ranking" );
            
            justifyRankingActionAll = new JustifyRankingAction( container, comp, true, 
                    "All", engine );
            
            justifyRankingActionSelected = new JustifyRankingAction( container, comp, false, 
                    "Selected", engine );
            
            justifyRankingMenu.add(justifyRankingActionAll);
            justifyRankingMenu.add(justifyRankingActionSelected);
            
            MenuUtils.addSubMenu( frame, "Reports", justifyRankingMenu );
        }
    }

	public OrangVisiplotAction getVisiplotActionAll() {
		return visiplotActionAll;
	}
	
	public OrangVisiplotAction getVisiplotActionSelected() {
		return visiplotActionSelected;
	}
	
	public JustifyRankingAction getJustifyRankingActionSelected() {
		return justifyRankingActionSelected;
	}

	public void setJustifyRankingActionSelected(
			JustifyRankingAction justifyRankingActionSelected) {
		this.justifyRankingActionSelected = justifyRankingActionSelected;
	}

	public JustifyRankingAction getJustifyRankingActionAll() {
		return justifyRankingActionAll;
	}

	public void setJustifyRankingActionAll(
			JustifyRankingAction justifyRankingActionAll) {
		this.justifyRankingActionAll = justifyRankingActionAll;
	}    
}
