package org.eso.ohs.phase2.apps.ot.gui;

import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.Collections;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JMenu;
import javax.swing.JPanel;
import javax.swing.border.Border;

import org.eso.ohs.core.dbb.client.DbbTable;
import org.eso.ohs.core.dbb.client.DbbView;
import org.eso.ohs.core.dbb.sql.DbbQueryTerminated;
import org.eso.ohs.core.dbb.sql.DbbQueryTerminatedListener;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.Queue;
import org.eso.ohs.dfs.QueueItem;
import org.eso.ohs.persistence.ObjectContainer;
import org.eso.ohs.phase2.actions.OrangVisiplotAction;
import org.eso.ohs.phase2.apps.ot.dbb.OTDbbView;
import org.eso.ohs.phase2.apps.ot.wizard.OrangEngine;
import org.eso.ohs.phase2.orang.OBScheduleInfo;
import org.eso.ohs.phase2.orang.OrangModel;
import org.eso.ohs.phase2.visiplot.VisiPlotOptions;

import com.jeta.forms.components.image.ImageComponent;

// $Id: QueueOrangView.java,v 1.25 2006/04/26 07:48:49 pnunes Exp $
public class QueueOrangView extends QueueView implements
        DbbQueryTerminatedListener {

    /**
     * Visiplot Options set by ORANG
     */
    private VisiPlotOptions currentSettings_;

    /*
     * ORANG Computed OB
     */
    private OBScheduleInfo[] obs_;

    private OrangModel model;

    private OrangEngine engine;

    private JMenu visiplotMenu;

    public QueueOrangView(OBScheduleInfo[] obs, OrangModel model,
            OrangEngine engine) {
        super();
        obs_ = obs;
        this.model = model;
        this.engine = engine;
    }

    private ImageIcon loadImage(String imageName) {
        String imagePath = "org/eso/ohs/icons/" + imageName;
        try {

            ClassLoader classloader = getClass().getClassLoader();
            java.net.URL url = classloader.getResource(imagePath);
            if (url != null) {
                ImageIcon icon = new ImageIcon(url);
                return icon;
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        throw new IllegalArgumentException("Unable to load image: " + imagePath);
    }

    protected JPanel getTitlePanel() {
        JPanel newPanel = new JPanel();
        BoxLayout box = new BoxLayout(newPanel, BoxLayout.X_AXIS);
        newPanel.setLayout(box);
        ImageComponent orange = new ImageComponent();
        Border empty = BorderFactory.createEmptyBorder(2, 2, 0, 5);
        orange.setBorder(empty);
        orange.setIcon(loadImage("orangeSmall.gif"));
        orange.setName("orange");
        newPanel.add(orange);
        newPanel.add(getQueuePropertiesView());
        return newPanel;
    }

    public void initQueueItemsBView() {
        obTreeView_ = new OBTreeView();
        queueItemsView_ = QueueItemsBView.createNotObservableWizardQueue(
                queue_, obTreeView_);
    }

    public void initQueueItemsRankedBView() {
        obTreeView_ = new OBTreeView();
        queueItemsView_ = QueueItemsBView.createObservableWizardQueue(queue_,
                obTreeView_);

        // add rank incremental action
        DbbView actualOrangView = queueItemsView_.getView().getDbbView();
        actualOrangView.addQueryTerminatedListener(this);

    }

    /**
     * @param newQueue
     *            void
     */
    public void initQueueView(Queue newQueue, VisiPlotOptions settings) {
        currentSettings_ = settings;
        newQueue(newQueue);
        setQueuePropertiesView(new QueuePropertiesView(newQueue));
        initQueueItemsBView();
        menuItems = new QueueViewMenuItems(this, getQueueItemsView().getView(),
                getQueueItemsView().getView().getJTable(), getQueueItemsView());
        menuItems.buildCompleteMainMenu();
        buildMenu(queueItemsView_.getView().getJTable());

        createView();
        setTitle("Queue view: ORANG ");
        if (actionMenuDefault_ != null) {
            actionMenuDefault_.actionPerformedImpl(new ActionEvent(
                    actionMenuDefault_, 0, ""));
        }

        menuItems.overrideVisiplotMenu(getQueueItemsView().getView(), settings,
                obs_);
    }

    protected void createVisiplotMenu(ObjectContainer container) {
        visiplotMenu = new JMenu("Visibility Plots");
        visiplotMenu.add(new OrangVisiplotAction(container, true,
                "All observation blocks", currentSettings_, obs_));
        visiplotMenu.add(new OrangVisiplotAction(container, false,
                "Selected observation blocks", currentSettings_, obs_));
    }

    /**
     * Builds a menu bar
     */
    protected void buildMenu(JComponent comp) {
        try {
            menuItems.buildFileMenu();
            menuItems.buildEditMenu();
            menuItems.buildOBsMenuNoMove();
            menuItems.addSomeOrangQueueViewMenus(queueItemsView_.getView(),
                    comp);
        } catch (ObjectIOException e) {
            stdlog_.error(e);
        }
    }

    public boolean closeQueue() {
        OTViewManager otMgr = OTViewManager.getOTViewMgr();
        int index = otMgr.getQueueViews().indexOf(this);
        updateQueue();
        if (index != -1) {
            forgetQueue(index);
        }
        return true;
    }

    public void initRankedQueueView(Queue newQueue, VisiPlotOptions settings) {
        currentSettings_ = settings;
        newQueue(newQueue);
        setQueuePropertiesView(new QueuePropertiesView(newQueue));
        initQueueItemsRankedBView();
        menuItems = new QueueViewMenuItems(this, getQueueItemsView().getView(),
                getQueueItemsView().getView().getJTable(), getQueueItemsView());
        menuItems.buildCompleteMainMenu();
        buildMenu(queueItemsView_.getView().getJTable());
        createView();
        setTitle("Queue view: ORANG ");
        if (actionMenuDefault_ != null) {
            actionMenuDefault_.actionPerformedImpl(new ActionEvent(
                    actionMenuDefault_, 0, ""));
        }

        menuItems.overrideVisiplotMenu(getQueueItemsView().getView(), settings,
                obs_);
        menuItems.addJustifyRankingMenu(getQueueItemsView().getView(),
                getQueueItemsView().getView().getJTable(), engine);
    }

    public void queryTerminatedAction(DbbQueryTerminated ev) {

        stdlog_.debug("query terminated action");
        DbbView view = queueItemsView_.getView().getDbbView();
        addRankColumn(view);

    }


	public static void addRankColumn(DbbView view) {
        DbbTable table = view.getTable();
        // stdlog_.debug("Init calculateSubtotal");

        int rows = view.getRows();

        Integer[] rankCounter = new Integer[rows];
        ArrayList<RankInfo> rankStringList = new ArrayList<RankInfo>();

        // get the rank string column
        for (int i = 0; i < rows; i++) {
            String currentRankString = (String) view.getValueAt(i,
                    OTDbbView.ORANG_RANK);
            stdlog_.debug("Rank: " + currentRankString + " index " + i);
            RankInfo rankInfo = new RankInfo(currentRankString, i);
            rankStringList.add(rankInfo);
        }

        // sort according to rank string
        Collections.sort(rankStringList);

        // set the order of rank int
        if (rankStringList.size() > 0) {
            RankInfo previousRank = rankStringList.get(0);           
            previousRank.setRankCounter(new Integer(1));        
            for (int i = 1; i < rankStringList.size(); i++) {
                RankInfo currentOrderedRank = rankStringList.get(i);
                if (previousRank.getRankString().equals(
                        currentOrderedRank.getRankString())) {
                    currentOrderedRank.setRankCounter(previousRank
                            .getRankCounter());
                } else {
                    currentOrderedRank.setRankCounter(new Integer(previousRank
                            .getRankCounter().intValue()+1));
                }
              previousRank = currentOrderedRank;
            }
        }
        // resort according to original
        Collections.sort(rankStringList, new TableIndexComparator());

        // convert to array
        for (int i = 0; i < rankStringList.size(); i++) {
            RankInfo currentOrderedRank = rankStringList.get(i);
            rankCounter[i] = currentOrderedRank.getRankCounter();
            stdlog_.debug("Rank: " + currentOrderedRank.getRankDescription()
                    + " rank id " + rankCounter[i] + " index "
                    + currentOrderedRank.getPosition());

        }

        table.addColumn("Rank Score", rankCounter, Integer.class, 0);

    }

    public void appendQueueItems(QueueItem[] selectedOBIDs) {
        if (getQueueItemsView().appendQueueItems(selectedOBIDs)) {

        }
    }
}
