package org.eso.ohs.phase2.apps.ot.gui;

import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.FileWriter;
import java.io.PrintWriter;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.client.DelConfigAction;
import org.eso.ohs.core.dbb.client.SaveConfigAction;
import org.eso.ohs.core.gui.utilities.MenuUtils;
import org.eso.ohs.core.utilities.OHSConfig;
import org.eso.ohs.core.utilities.OHSKeys;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectContainer;
import org.eso.ohs.phase2.actions.ExportBOAction;
import org.eso.ohs.phase2.actions.OrangVisiplotAction;
import org.eso.ohs.phase2.actions.ReloadOBAction;
import org.eso.ohs.phase2.apps.ot.actions.CloseQueueAction;
import org.eso.ohs.phase2.apps.ot.actions.ReloadQueueAction;
import org.eso.ohs.phase2.apps.ot.actions.SaveAsQueueAction;
import org.eso.ohs.phase2.apps.ot.actions.SaveQueueAction;
import org.eso.ohs.phase2.apps.ot.actions.SelectAllAction;
import org.eso.ohs.phase2.apps.ot.wizard.OrangEngine;
import org.eso.ohs.phase2.orang.OBScheduleInfo;
import org.eso.ohs.phase2.visiplot.VisiPlotOptions;

/**
 * It contains menu items for queue views and orang GUI
 * 
 * @author pnunes 24.02.2006
 */
public class QueueViewMenuItems {

	
	protected static Logger stdlog_ = Logger.getLogger(QueueViewMenuItems.class);	
	
	private JMenuBar menuBar;	
	private JMenu fileMenu;
    private JMenu OBsMenu;
    private JMenu editMenu;
    private JMenu optionsMenu_;
    private JMenu reload;
    private JMenu printFCMenu;
    private JMenuItem selectAllMenuItem;
    private JMenuItem removeMenuItem;
    private JMenuItem moveUpMenuItem;
    private JMenuItem moveDownMenuItem;

    // OBsMenuItems
    private JMenuItem displayMenuItem;
    private JMenuItem commentMenuItem;

    // private JMenu markSubMenu;
    // private JMenuItem reloadQueueItemsMenuItem;
    private JMenuItem queueForExecutionMenuItem;

    // <br>ChangeLog: amchavan, 28-May-2002: commented out
    // private JMenuItem queueMaskBrowserMenuItem;

    // file menu items
    private JMenuItem reloadQueueMenuItem;
    private JMenuItem closeQueueMenuItem;
    
    private VisiPlotOptions currentSettings;
    private OBScheduleInfo [] obs;

    // <br>ChangeLog: amchavan, 28-May-2002: commented out
    // private JMenuItem maskQueueViewMenuItem;
    private JMenuItem saveQueueMenuItem;
    private JMenuItem saveAsQueueMenuItem;
    private JMenuItem summaryQueueMenuItem;

    // private JMenuItem dumpQueueMenuItem;
    private JMenuItem verifyQueueMenuItem;
    
    String userConfigFile_ = null;
    private static final String CONFIG_KEYWORD = "QUEUEVIEW.CONFIG.CONFIGMENU";
    
    protected final String DEFAULT_REQUEST = "Save As Default";
    
    // references to parents
    private JComponent component;    
    private ObjectContainer container;    
    
	private QueueItemsBView queueItemsView;
	private JFrame parentFrame;

	private MenuCreator otMenu;
    
    
    
    /**
     * @param parentFrame
     * @param container
     * @param component
     * @param queueItemsView
     */
    public QueueViewMenuItems(JFrame parentFrame, ObjectContainer container, JComponent component, QueueItemsBView queueItemsView) {
    	this.parentFrame = parentFrame;
		this.container = container;
		this.component = component;
		this.queueItemsView = queueItemsView;				
	}
    
    /**
     * @param parentFrame
     * @param container
     * @param component
     * @param queueItemsView
     */
    public QueueViewMenuItems(JFrame parentFrame, ObjectContainer container, JComponent component, QueueItemsBView queueItemsView,
    		VisiPlotOptions visiplotOptions,OBScheduleInfo[] obs) {
    	this.parentFrame = parentFrame;
		this.container = container;
		this.component = component;
		this.queueItemsView = queueItemsView;
		this.currentSettings = visiplotOptions;
		this.obs = obs;
	}



	/**
     * build the main menu
     */
    public void buildCompleteMainMenu() {
        menuBar = new JMenuBar();
        fileMenu = new JMenu("Queue");
        menuBar.add(fileMenu);

        editMenu = new JMenu("Edit");
        menuBar.add(editMenu);

        OBsMenu = new JMenu("OBs");
        menuBar.add(OBsMenu);
        parentFrame.setJMenuBar(menuBar);
    }
    
    
                   
    
    /**
     * @param comp
     * @throws ObjectIOException
     */
    protected void addSomeMenus(JComponent comp) throws ObjectIOException {
        otMenu = new MenuCreator(parentFrame);
        if (!AppConfig.getAppConfig().getBoolean(OHSKeys.MaskTrackerEnabledKey)) {
            otMenu.createReadmeMenu(container,comp);
            otMenu.createEphFileMenu(container,comp);
        }
        otMenu.createOrangReportsMenu(container,component,currentSettings,obs);
        otMenu.createFCPrintMenu(container,comp);
        if (optionsSetup()) {
            optionsMenu_ = new JMenu("Options");
            buildOptionsMenu() ;
            menuBar.add(optionsMenu_);
            parentFrame.setJMenuBar(menuBar);
        }

        
    }
    
    protected void addSomeOrangQueueViewMenus(ObjectContainer container,JComponent comp) throws ObjectIOException {
        otMenu = new MenuCreator(parentFrame);
        if (!AppConfig.getAppConfig().getBoolean(OHSKeys.MaskTrackerEnabledKey)) {
            otMenu.createReadmeMenu(container,comp);
            otMenu.createEphFileMenu(container,comp);
        }
        otMenu.createReportsMenu(container,comp);
        otMenu.createFCPrintMenu(container,comp);
        if (optionsSetup()) {
            optionsMenu_ = new JMenu("Options");
            buildOptionsMenu() ;
            menuBar.add(optionsMenu_);
            parentFrame.setJMenuBar(menuBar);
        }

        
    }

    
    
    
    /**
     * @param comp
     * @throws ObjectIOException
     */
    protected void addSomeObsMenus(JComponent comp) throws ObjectIOException {
        otMenu = new MenuCreator(parentFrame);
        if (!AppConfig.getAppConfig().getBoolean(OHSKeys.MaskTrackerEnabledKey)) {
            otMenu.createReadmeMenu(container,comp);
            otMenu.createEphFileMenu(container,comp);
        }
        otMenu.createOrangReportsMenu(container,component,currentSettings,obs);
        otMenu.createFCPrintMenu(container,comp);
        if (optionsSetup()) {
            optionsMenu_ = new JMenu("Options");
            buildOptionsObsMenu();
            menuBar.add(optionsMenu_);
            parentFrame.setJMenuBar(menuBar);
        }

        
    }
    
    public JMenuBar getMenuBar() {
		return menuBar;
	}




	/**
     * Builds the file menu
     */
    public void buildFileMenu() {
        
        reloadQueueMenuItem = fileMenu
                .add(new ReloadQueueAction(queueItemsView,parentFrame, "Revert"));
        reloadQueueMenuItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_R, ActionEvent.CTRL_MASK));
        closeQueueMenuItem = fileMenu.add(new CloseQueueAction(queueItemsView,component, "Close"));
        fileMenu.addSeparator();
        saveQueueMenuItem = fileMenu.add(new SaveQueueAction(queueItemsView,component, "Save"));
        if (!AppConfig.getAppConfig().getBoolean(OHSKeys.MaskTrackerEnabledKey)) {                    
              saveAsQueueMenuItem = fileMenu.add(new SaveAsQueueAction(queueItemsView,component,"Save As"));
        }
        fileMenu.add(new ExportBOAction(Media.DBASE, container,component, "Export...", true, true));
        fileMenu.addSeparator();
        
        fileMenu.addSeparator();
        
        // <br>ChangeLog: amchavan, 28-May-2002: commented out
        // maskQueueViewMenuItem = fileMenu.add(new MaskQueueBrowserAction());
        // fileMenu.add(maskQueueViewMenuItem);
        closeQueueMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_W,
                ActionEvent.CTRL_MASK));
        closeQueueMenuItem.setMnemonic(KeyEvent.VK_W + KeyEvent.CTRL_MASK);

        saveQueueMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S,
                ActionEvent.CTRL_MASK));
        saveQueueMenuItem.setMnemonic(KeyEvent.VK_S + KeyEvent.CTRL_MASK);
    }

    /**
     * Create and populate the "Edit" menu.
     */
    public void buildEditMenu() {
    	selectAllMenuItem = editMenu.add(new SelectAllAction(queueItemsView.getDbbView().getTable().getTable()));
        selectAllMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A,
                InputEvent.CTRL_MASK, false));
    }

    /**
     * Builds a menu bar
     */
    public void buildMenu(JComponent comp) {
        try {
            buildFileMenu();
            buildEditMenu();
            buildOBsMenu();
            otMenu = new MenuCreator(parentFrame);
            if (!AppConfig.getAppConfig().getBoolean(OHSKeys.MaskTrackerEnabledKey)) {
                otMenu.createReadmeMenu(container,comp);
                otMenu.createEphFileMenu(container,comp);
            }
            otMenu.createReportsMenu(container,comp);
            otMenu.createFCPrintMenu(container,comp);
            if (optionsSetup()) {
                optionsMenu_ = new JMenu("Options");
                buildOptionsMenu();
                menuBar.add(optionsMenu_);
                parentFrame.setJMenuBar(menuBar);
            }    

        } catch (ObjectIOException e) {
            stdlog_.error(e);
        }
    }
    
    
    /**
     * Builds a menu bar
     */
    public void buildOrangMenu(JComponent comp) {
        try {
            menuBar = new JMenuBar();
            editMenu = new JMenu("Edit");
            menuBar.add(editMenu);
            OBsMenu = new JMenu("OBs");
            menuBar.add(OBsMenu);
            parentFrame.setJMenuBar(menuBar);
            buildEditMenu();
            buildOrangOBsMenu();
            addSomeMenus(comp);            
        } catch (ObjectIOException e) {
            stdlog_.error(e);
        }
    }
    
    
    /**
     * Builds a menu bar
     */
    public void buildObservableMenu(JComponent comp) {
        try {
            menuBar = new JMenuBar();
            editMenu = new JMenu("Edit");
            menuBar.add(editMenu);
            OBsMenu = new JMenu("OBs");
            menuBar.add(OBsMenu);
            parentFrame.setJMenuBar(menuBar);
            buildEditMenu();
            buildOrangOBsMenu();
            addSomeObsMenus(comp);
        } catch (ObjectIOException e) {
            stdlog_.error(e);
        }
    }
    
    public void updateVisiplotActions(OBScheduleInfo [] obScheduleInfo,VisiPlotOptions visiplotOptions) {
    	OrangVisiplotAction allAction = otMenu.getVisiplotActionAll();
    	allAction.setObs(obScheduleInfo);
       	allAction.setCurrentSettings(visiplotOptions);    	        
    	
    	OrangVisiplotAction selectedAction = otMenu.getVisiplotActionSelected();
    	selectedAction.setObs(obScheduleInfo);    	
    	selectedAction.setCurrentSettings(visiplotOptions);
    }
    
    public void updateJustifyActions(OrangEngine engine) {
    	otMenu.getJustifyRankingActionAll().setEngine(engine);
    	otMenu.getJustifyRankingActionSelected().setEngine(engine);
    }


    
    /**
     * 
     * Create and populate the "Options" menu in the database browser view.
     * 
     * @exception ObjectIOException
     *                object IO Exception
     */
    private void buildOptionsMenu() throws ObjectIOException {

        SaveConfigAction confAction = new SaveConfigAction(parentFrame, optionsMenu_,
                null, QueueItemsBView.colPanel_, QueueItemsBView.sortPanel_,
                CONFIG_KEYWORD, userConfigFile_, "Save settings", true);
        optionsMenu_.add(confAction);

        confAction = new SaveConfigAction(parentFrame, optionsMenu_, null,
                QueueItemsBView.colPanel_, QueueItemsBView.sortPanel_,
                CONFIG_KEYWORD, userConfigFile_, DEFAULT_REQUEST, true);
        optionsMenu_.add(confAction);

        DelConfigAction delAction = new DelConfigAction(parentFrame, optionsMenu_,
                CONFIG_KEYWORD, userConfigFile_, "Delete Settings", true);
        delAction.setMenuMap(confAction.getMenuMap());

        optionsMenu_.add(delAction);
        optionsMenu_.addSeparator();
        confAction.addMenus(CONFIG_KEYWORD);
        confAction.applyDefault("QUEUEVIEW.CONFIG.MENUDEFAULT");
        menuBar.add(optionsMenu_);
    }
    
    /**
     * 
     * Create and populate the "Options" menu in the database browser view.
     * 
     * @exception ObjectIOException
     *                object IO Exception
     */
    private void buildOptionsObsMenu() throws ObjectIOException {

        SaveConfigAction confAction = new SaveConfigAction(parentFrame, optionsMenu_,
                null, QueueItemsBView.colObsPanel_, QueueItemsBView.colObsPanel_,
                CONFIG_KEYWORD, userConfigFile_, "Save settings", true);
        optionsMenu_.add(confAction);

        confAction = new SaveConfigAction(parentFrame, optionsMenu_, null,
                QueueItemsBView.colObsPanel_, QueueItemsBView.sortObsPanel_,
                CONFIG_KEYWORD, userConfigFile_, DEFAULT_REQUEST, true);
        optionsMenu_.add(confAction);

        DelConfigAction delAction = new DelConfigAction(parentFrame, optionsMenu_,
                CONFIG_KEYWORD, userConfigFile_, "Delete Settings", true);
        delAction.setMenuMap(confAction.getMenuMap());

        optionsMenu_.add(delAction);
        optionsMenu_.addSeparator();
        confAction.addMenus(CONFIG_KEYWORD);
        confAction.applyDefault("QUEUEVIEW.CONFIG.MENUDEFAULT");
        menuBar.add(optionsMenu_);
    }
    
    /**
     * void
     */
    private void addLastItems() {
        queueForExecutionMenuItem = OBsMenu.add(queueItemsView.getExecutionAction());
        queueForExecutionMenuItem.setText("Execution Sequence");

        /**
         * Moved submenu item from OB menu to Reports menu to get consistent
         * with OT Main Menu 1510/OT CMM 07/02/02 showPAFDescriptionMenuItem =
         * OBsMenu.add(queueItemsView_.getShowPAFDescriptionAction());
         */

        OBsMenu.add(queueItemsView.getExportAction());
        if (AppConfig.getAppConfig().getBoolean(OHSKeys.MaskTrackerEnabledKey)) {
            OBsMenu.add(queueItemsView.getDumpAdpAdmDataAction());
        }
        OBsMenu.add(queueItemsView.getObHistoryAction());


        queueForExecutionMenuItem.setAccelerator(KeyStroke.getKeyStroke(
                KeyEvent.VK_E, ActionEvent.CTRL_MASK));
        queueForExecutionMenuItem.setMnemonic(KeyEvent.VK_E
                + KeyEvent.CTRL_MASK);
    }

    /**
     * void
     */
    private void addFirstItems() {
        addReloadMenu();
        if (!AppConfig.getAppConfig().getBoolean(OHSKeys.MaskTrackerEnabledKey)) {
            displayMenuItem = OBsMenu.add(queueItemsView.getDisplayAction());
        }
        if (!AppConfig.getAppConfig().getBoolean(OHSKeys.MaskTrackerEnabledKey)) {
            commentMenuItem = OBsMenu.add(queueItemsView.getOpenCommentsAction());
        }
        removeMenuItem = OBsMenu.add(queueItemsView.getRemoveAction());        
        removeMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X,
                ActionEvent.CTRL_MASK));
        removeMenuItem.setMnemonic(KeyEvent.VK_X + KeyEvent.CTRL_MASK);
    }


	private void addReloadMenu() {
		reload = new JMenu("Reload OB");
        
        reload.add(new ReloadOBAction(container,component, true,
                "All"));
        reload.add(new ReloadOBAction(container,component, false,
                "Selected"));
        OBsMenu.add(reload);
	}

    /**
     * void
     */
    private void addMoveItems() {
        moveUpMenuItem = OBsMenu.add(queueItemsView.getMoveUpAction());
        moveDownMenuItem = OBsMenu.add(queueItemsView.getMoveDownAction());
        moveUpMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_P,
                ActionEvent.CTRL_MASK));
        moveUpMenuItem.setMnemonic(KeyEvent.VK_P + KeyEvent.CTRL_MASK);

        moveDownMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N,
                ActionEvent.CTRL_MASK));
        moveDownMenuItem.setMnemonic(KeyEvent.VK_N + KeyEvent.CTRL_MASK);
    }
    
    
    protected void buildOBsMenu() {
        addFirstItems();
        addMoveItems();
        addLastItems();

    }
    
    protected void buildOrangOBsMenu() {  
        addReloadMenu();
        commentMenuItem = OBsMenu.add(queueItemsView.getOpenCommentsAction());
        addLastItems();
    }


    protected void buildOBsMenuNoMove() {
        addFirstItems();
        addLastItems();

    }
    
    
    public void reinitOBsMenu() {
        MenuUtils.removeMenuItems(OBsMenu);
        MenuUtils.removeMenuItems(reload);
        buildOBsMenu();
    }
    
    /**
     * Read the config file and the neccessary keyword for creating the
     * "Options" menu in the database browser view.
     * 
     * @return true if no exceptions caught
     */
    protected boolean optionsSetup() {
        OHSConfig config = OHSConfig.getInstance();
        AppConfig appConfig = AppConfig.getAppConfig();
        try {
            String optionDir = appConfig.getOptionDir();
            // stdlog_.info ("Options Dir: " + optionDir);
            if (optionDir != null && optionDir.length() > 0) {
                String fileName = config.getValue(getOptionsKeyword());
                if (fileName != null && fileName.length() > 0) {
                    userConfigFile_ = optionDir + File.separator + fileName;
                    config.addParamfile(userConfigFile_);
                } else {
                    return false;
                }
            } else {
                return false;
            }
        } catch (Exception e) {
            stdlog_.info("File: " + userConfigFile_ + " does not exist.");
            try {
                PrintWriter out = new PrintWriter(new FileWriter(
                        userConfigFile_));
                out.println("");
                out.close();
                stdlog_.info("File: " + userConfigFile_ + " created.");
                config.addParamfile(userConfigFile_);
            } catch (java.io.FileNotFoundException e1) {
                return false;
            } catch (org.eso.ohs.core.utilities.NotASCIIException e2) {
                return false;
            } catch (java.io.IOException e3) {
                return false;
            }
        }
        return true;
    }
    
    protected String getOptionsKeyword() {
        return CONFIG_KEYWORD;
    }
       
    
    /**
     * Hack to hide the un-necessary menus in the orang GUI
     * 
     * @param fileMenu
     * @param OBsMenu
     * @param editMenu
     * @param optionsMenu
     * @param reload
     */
    public void hideMenus(boolean fileMenu, boolean OBsMenu, boolean editMenu,
			boolean optionsMenu, boolean reload) {

    	
    	this.fileMenu.setVisible(fileMenu);
    	this.OBsMenu.setVisible(OBsMenu);
    	this.editMenu.setVisible(editMenu);
    	this.optionsMenu_.setVisible(optionsMenu);
    	this.reload.setVisible(reload);
	}
    
    public  void overrideVisiplotMenu(ObjectContainer container,VisiPlotOptions currentSettings, OBScheduleInfo [] obs) { 
        otMenu.overrideVisiplotMenu(container,currentSettings,obs);
    }
    
    // amchavan, 01-Mar-2006: added
    public void addJustifyRankingMenu( ObjectContainer container, 
                                       JComponent comp, 
                                       OrangEngine engine ) { 
        otMenu.addJustifyRankingMenu( container, comp, engine );
    }
    
  

	public MenuCreator getOtMenu() {
		return otMenu;
	}

	public void setOtMenu(MenuCreator otMenu) {
		this.otMenu = otMenu;
	}

	public QueueItemsBView getQueueItemsView() {
		return queueItemsView;
	}

	public void setQueueItemsView(QueueItemsBView queueItemsView) {
		this.queueItemsView = queueItemsView;
	}

	public ObjectContainer getContainer() {
		return container;
	}

	public void setContainer(ObjectContainer container) {
		this.container = container;
	}

    
}
