package org.eso.ohs.phase2.apps.ot.wizard;

import java.text.ParseException;
import java.util.LinkedList;

import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.apache.commons.collections.CollectionUtils;
import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.baseaction.DescIncrementalActionAutomaton;
import org.eso.ohs.core.gui.baseaction.DescriptiveIncrementalAction;
import org.eso.ohs.core.gui.widgets.ProgressModel;
import org.eso.ohs.core.gui.widgets.ProgressModelListener;
import org.eso.ohs.core.utilities.Convert;
import org.eso.ohs.core.utilities.OHSConfig;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.ObTargetConstraintInfo;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.Queue;
import org.eso.ohs.dfs.QueueItem;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.OBEvent;
import org.eso.ohs.persistence.OBListener;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.dbase.phase1.Phase1SelectStmt;
import org.eso.ohs.persistence.dbase.phase2.Phase2SelectStmt;
import org.eso.ohs.phase2.apps.ot.Persistence;
import org.eso.ohs.phase2.apps.ot.gui.OTViewManager;
import org.eso.ohs.phase2.orang.OBInfoLoader;
import org.eso.ohs.phase2.orang.OBRankingEngine;
import org.eso.ohs.phase2.orang.OBScheduleInfo;
import org.eso.ohs.phase2.orang.ObDataContainer;
import org.eso.ohs.phase2.orang.ObWeatherFilter;
import org.eso.ohs.phase2.orang.OrangCache;
import org.eso.ohs.phase2.orang.OrangCacheContainer;
import org.eso.ohs.phase2.orang.OrangModel;
import org.eso.ohs.phase2.orang.OrangScriptException;
import org.eso.ohs.phase2.orang.OrangScriptingBox;
import org.eso.ohs.phase2.timeline.EarlyGreedy;
import org.eso.ohs.phase2.timeline.Timeline;
import org.eso.ohs.phase2.timeline.TimelineView;
import org.eso.ohs.phase2.visibility.VisibilityJCalc;
import org.eso.ohs.phase2.visibility.VisicalcOptions;
import org.eso.ohs.phase2.visibility.PCFPlotterFilter.ValueRejectedException;
import org.eso.ohs.phase2.visiplot.TimerLog;
import org.eso.ohs.phase2.visiplot.VisiPlotOptions;

/**
 * This class implements all functionality needed to run the ORANG, either via
 * the wizard or another interface.
 * 
 * Methods of this class take all info about the current state of the OB ranking
 * Engine from an {@link org.eso.ohs.phase2.orang.OrangModel}
 * 
 * @version $Revision: 1.54 $
 * @author pnunes, ddorigo
 */


// $Id: OrangEngine.java,v 1.54 2006/04/26 07:49:45 pnunes Exp $
public class OrangEngine implements OBListener, OrangCache {

    
	private static final String OB_STATUS_WRONG = "NoStatus";

	private static final String COMPUTING_MSG = "Computing...";
	private static final String LOAD_OBS_MSG = "Loading OBs...";
	private static final String COMPUTE_WEATHER_MSG = "Filtering by weather...";
	private static final String COMPUTE_VISIBILITY_MSG = "Computing visibility...";
	private static final String RANK_MSG = "Ranking OBs...";
	private static final String FINISH_MSG = "Finishing...";
    
    /**
     * Logger
     */
    protected static Logger stdlog_ = Logger.getLogger(OrangEngine.class);

    private LinkedList<ObDataContainer> obList = new LinkedList<ObDataContainer>();
    private LinkedList<ObDataContainer> obVisicalcInputList = new LinkedList<ObDataContainer>();
    private LinkedList<OBScheduleInfo> obVisicalcOutList = new LinkedList<OBScheduleInfo>();
    private LinkedList<OBScheduleInfo> obRanked = new LinkedList<OBScheduleInfo>();
    
    
    private Queue queueObservable = null;
    private Queue queueNotObservable = null;

    
    
    /** Filters by weather conditions */
    private ObWeatherFilter obWeatherFilter = null;
    
    /**  Loads the observation blocks  */
    private OBInfoLoader obLoader;
        
    /**
     * Calculates visibility 
     */
    private VisibilityJCalc visibilityCalc;
    
    
    /**
     * Scripting engine for ranking the OBs 
     */
    private OBRankingEngine rankingEngine;
    
    
    /*
     * Progress listener
     */
    private ProgressModelListener progressListener;
    
    /** Widget to display the current timeline */
    private TimelineView timelineView = null;

    /** Top-level window to display the current timeline */
    private JFrame timelineFrame;

    /**
     * initialise the objet member.
     */
    public OrangEngine() {
        init();
        OrangCacheContainer.setOrangCache(this);
    }
    
    //public OrangEngine getInstance()
    

    /**
     * reinitialize the engine
     */
    private void init() {
    	Phase1SelectStmt.definePhase1TableDb("opc70", "dbo");
        Phase2SelectStmt.definePhase2TableDb("obrep2", "dbo");
        setObList(new ObDataContainer[0]);
        
        setObVisicalcInputList(new ObDataContainer[0]);
        setObVisicalcOutList(new OBScheduleInfo[0]);
        setObRanked(new OBScheduleInfo[0]);
        obLoader = new OBInfoLoader();
        obWeatherFilter = new ObWeatherFilter(new ObDataContainer[0]);
        visibilityCalc = new VisibilityJCalc();
        rankingEngine = OrangScriptingBox.getInstance();
        queueObservable = new Queue();
        queueNotObservable = new Queue();
    }

    /**
     * Loads an OB list from the database
     * 
     * @param model
     * @throws ObjectNotFoundException
     * @throws ObjectIOException
     */
    public void loadObList(OrangModel model)
            throws ObjectNotFoundException, ObjectIOException {
        
    	
    	
        int queueSelectionType = model.getObSelectionType();
        if (queueSelectionType == OrangModel.OB_SELECTION_INSTRUMENT) {
            String[] queueList = model.getQueuesInstruments();

            stdlog_.debug("Loading Instrument OBs.");
            setObList(obLoader.loadQueues(queueList, model.getUser(), model
                    .isSuperUser()));
            
        } else if (queueSelectionType == OrangModel.OB_SELECTION_TELESCOPE) {
            String[] queueList = model.getQueuesTelescopes();

            stdlog_.debug("Loading Telescope OBs.");
            setObList(obLoader.loadQueues(queueList, model.getUser(), model
                    .isSuperUser()));
        } else if (queueSelectionType == OrangModel.OB_SELECTION_QUEUES) {
        	stdlog_.debug("Loading Queue OBs.");
        	
            setObList(obLoader.loadQueues(model.getSelectedQueues(), model
                    .getUser(), model.isSuperUser()));
        } else {
            throw new UnsupportedOperationException("operation "
                    + queueSelectionType + " in selection not supported");
        }
    }

    /**
     * Filter OBs by weather conditions
     * 
     * @param model the weather condition passed by the model
     */
    public void filterObByWeather(OrangModel model) {
    	stdlog_.debug("Filtering OB by weather");
    	obWeatherFilter = new ObWeatherFilter(getObList());        
        setObVisicalcInputList(getObWeatherFilter().filterObArray(
                model.getWeatherConstraint(),model.getStartDate()));
    }

    /**
     * Computes visibilities
     * 
     * @param model
     * @throws ObjectNotFoundException
     * @throws ObjectIOException
     * @throws ValueRejectedException
     */
    public void filterObByConstraints(OrangModel model)
            throws ObjectNotFoundException, ObjectIOException,
            ValueRejectedException {
        
    	stdlog_.debug("Computing OB Visibilities");
    	
        VisicalcOptions visiCalc = model.getVisiCalcOptions();

        setObVisicalcOutList(visibilityCalc.calc(getObVisicalcInputList(),
                model.getStepInterval(), model.getStartDate(), model
                        .getEndDate(), visiCalc, model.getLatitude(), model
                        .getLongitude()));

    }

    /**
     * Compute and show a timeline from the current list of ranked OBS.
     * 
     * @param model
     *            We take the current ORANG settings form here.
     */
    // amchavan & pnunes: added, 17-Oct-2005
    public void showTimeline(OrangModel model) {
        // Compute a timeline using the early-greedy algorithm
        Timeline timeline = new EarlyGreedy(getObRanked(),
                model.getStartDate(), model.getEndDate());
        System.out.print(timeline);

        if (timelineView == null) {
            timelineView = new TimelineView(timeline);
            timelineFrame = new JFrame("Early-greedy timeline");
            timelineFrame.setContentPane(timelineView);
            timelineFrame.pack();
        } else {
            timelineView.setTimeline(timeline);
        }
        timelineFrame.setVisible(true);
    }

    /**
     * computes the ranking for each observable OB
     * 
     * @param model
     * @throws OrangScriptException
     */
    public void applyAlgorithm(OrangModel model) throws OrangScriptException {
        String algorithm = model.getRankingAlgorithm();
        
        stdlog_.debug("Ranking OBs");
        
        rankingEngine.setOBsForRanking(getObservableOBs());
        rankingEngine.setModel(model);
        rankingEngine.rank(algorithm);
        setObRanked(rankingEngine.getRankedOBs());
    }

    /**
     * Creates and pops-up the orang queue views
     * @param model
     */
    public void showRankedObs(OrangModel model) {
    	try {
    		boolean supressEvents = false;
    		String orangQueueName = OHSConfig.getInstance().getValue("ORANG.OBSERVABLE.NAME");
    		String orangNotObsQueueName = OHSConfig.getInstance().getValue("ORANG.NOTOBSERVABLE.NAME");
    		Queue queueObservableOld = queueObservable;
    		Queue queueNotObservableOld = queueNotObservable;
    		queueObservable = createNewQueue(model, orangQueueName,supressEvents);
    		queueNotObservable = createNewQueue(model, orangNotObsQueueName,supressEvents);
    		
    		OBScheduleInfo[] obNotObservable = getNonObservableOBs();
    		QueueItem[] notObsQueueItems= buildNotObservQueueItems(obNotObservable,queueNotObservable);
    		
    		OBScheduleInfo[] obObservable = getObservableOBs();
    		QueueItem[] obsQueueItems= buildObservQueueItems(obObservable,queueObservable);          
    		    		 
    		VisiPlotOptions settings = generateSettings(model);
    		
    		OTViewManager otViewMgr = OTViewManager.getOTViewMgr();
    		otViewMgr.forceQueueClose(queueObservableOld);
    		otViewMgr.forceQueueClose(queueNotObservableOld);
    		otViewMgr.showOrangNotObservableQueue(queueNotObservable, settings,model,obNotObservable,notObsQueueItems,this);
    		otViewMgr.showOrangObservableQueue(queueObservable, settings,model,obObservable,obsQueueItems,this);
    		
    	} catch (ObjectIOException e) {
    		JOptionPane.showMessageDialog(null,
    				"An error occured while registering the queue", "Error",
    				JOptionPane.WARNING_MESSAGE);
    		e.printStackTrace();
    	}
    }

    /**
     * Creates a new orang queue
     * 
     * @param model
     * @param otViewMgr
     * @param orangQueueName
     * @param queueObservable2
     * @throws ObjectIOException
     *             void
     */
    private Queue createNewQueue(OrangModel model, String orangQueueName,boolean supressEvent)
            throws ObjectIOException {
        Queue currentQueue = saveQueue(orangQueueName, model.getStartISODate(),
                model.getEndISODate(),supressEvent);
        return currentQueue;
    }

    /**
     * Creates a VisiPlotOptions object based on an OrangModel
     * @param model
     * @return VisiPlotOptions
     */
    public VisiPlotOptions generateSettings(OrangModel model) {
        VisiPlotOptions settings = new VisiPlotOptions();        

        settings.getVisicalcOptions().setInterval(model.getStepInterval());
        settings.getVisicalcOptions().setStartTime(model.getStartDate());
        settings.getVisicalcOptions().setEndTime(model.getEndDate());
        
        // if the moon is disregarded, the FLI and moon distance are irrelevant
        settings.getVisicalcOptions().setCalcMoon(model.isMoonAngleConstrain());
        if (model.isMoonAngleConstrain()) {
        	settings.getVisicalcOptions().setCalcFli(model.isFliConstrain());
        	settings.getVisicalcOptions().setCalcMoonDist(model.isMoonDistanceConstrain());
        } else {
        	settings.getVisicalcOptions().setCalcFli(false);
        	settings.getVisicalcOptions().setCalcMoonDist(false);
        }
        
        settings.getVisicalcOptions().setCalcSun(model.isSunAngleConstrain());        
        settings.getVisicalcOptions().setCalcTimeInterval(model.isTimeIntervalConstrain());
        settings.getVisicalcOptions().setCalcSiderealTimeInterval(model.isSidTimeIntervalConstrain());
        settings.getVisicalcOptions().setSunConstraint(model.getSunAngleDegree());
        settings.getVisicalcOptions().setCalcZenithInvisibility(model.isTelescopeZenithConstraint());

        
        return settings;
    }

    /**
     * Creates a queueItem array of non-observable OBs
     * 
     * @param queueItemObservableVector
     * @param queueItemNotObservableVector
     * @param queueObservable
     * @param queueNotObservable
     *            void
     */
    private QueueItem[] buildNotObservQueueItems(
            OBScheduleInfo[] queueItemNotObservable,
            Queue queueNotObservable) {
        QueueItem[] queueItemResult = new QueueItem [queueItemNotObservable.length];
        for (int i = 0; i < queueItemNotObservable.length; i++) {
            OBScheduleInfo currentOb =queueItemNotObservable[i];
                QueueItem item = new QueueItem(currentOb
                        .getOb()
                        .getObInfo()
                        .getId(), queueNotObservable.getId(), i,
                        ObservationBlock.OBSBLK_TYPE);             
                queueItemResult[i] = item;
            }
         return queueItemResult;
    }
    
    
    /**
     * 
     * Creates a queueItem array of observable OBs
     * @param queueItemObservableVector
     * @param queueItemNotObservableVector
     * @param queueObservable
     * @param queueNotObservable
     *            void
     */
    private QueueItem[] buildObservQueueItems(
            OBScheduleInfo[] queueItemObservable,
            Queue queueObservable) {
        QueueItem[] queueItemResult = new QueueItem [queueItemObservable.length];
         
        for (int i = 0; i < queueItemObservable.length; i++) {
            OBScheduleInfo currentOb = queueItemObservable[i];
                QueueItem item = new QueueItem(currentOb
                        .getOb()
                        .getObInfo()
                        .getId(), queueObservable.getId(), i,
                        ObservationBlock.OBSBLK_TYPE, currentOb
                                .getRankingScore());
                
                queueItemResult[i] = item;
           
        }
        return queueItemResult;
    }

    /**
     * Saves a queue to the database
     * TODO refactor this out
     * @param date2
     * @param date
     * @param objMgr
     * @return
     * @throws ObjectIOException
     *             Queue
     */
    private Queue saveQueue(String name, String startISODate, String endISODate,boolean supressEvent)
            throws ObjectIOException {
        Persistence objMgr = Persistence.getInstance();
        // Set the objectManager not to fiere event back to the GUI.
        // Thus it will not add the Queue to the Open List of queues on the
        // OTMainView. This obviously must be reset at the end of this method 
        // as the ObjectManaget is a singleton.
        
        objMgr.setSupressEvents(supressEvent);
        
        Queue queue = (Queue) objMgr.createAndRegister(Media.DBASE, objMgr
                .getRoot(Media.DBASE), Queue.class);
        try {
            long startLong = Convert.ISOdateToLong(startISODate);
            long endLong = Convert.ISOdateToLong(endISODate);

            queue.setIsNew(true);
            queue.setDuration(endLong - startLong);
            queue.setStartTime(startLong);
        } catch (ParseException e) {
            e.printStackTrace();
        }

        queue.setName(name);
        queue.setTelescope("UT1");
        objMgr.setSupressEvents(false);
        return queue;
    }


    /**
     * Re-initializes the engine 
     */
    public void clear() {
        init();
    }

    /**
     * Returns a string about loaded OBs
     * @return
     */
    public String getLoadedInfo() {

        String res = "0 Observation Blocks loaded";

        if (getObList() != null) {
            res = getObList().length + " Observation Blocks loaded";
        }

        return (res);

    }

    /**
     * Returns a string about OB weather filtering
     * @return
     */
    public String getWeatherInfo() {
        String res = "0 Observation Blocks considered";

        res = getObVisicalcInputList().length
                + " Observation Blocks considered. "
                + (getObList().length - getObVisicalcInputList().length)
                + " filtered out.";
        return (res);
    }

    /**
     * Returns a string about visibility
     * @return
     */
    public String getVisibilityInfo() {
        String res = "0 Observation Blocks considered";

        int observable = 0;
        int not_observable = 0;
        OBScheduleInfo[] currentVisicalc = getObVisicalcOutList();
        for (int i = 0; i < currentVisicalc.length; i++) {
            OBScheduleInfo ob = currentVisicalc[i];
            if (ob.isObservable()) {
                observable++;
            } else {
                not_observable++;
            }

        }

        res = observable + " Observation Blocks considered. " + not_observable
                + " filtered out.";

        return (res);
    }

    /**
     * Returns a string about ranked OBs
     * @return
     */
    public String getRankedInfo() {
        String res = "0 Observation Blocks ranked";

        int observable = 0;
        OBScheduleInfo[] currentVisicalc = getObVisicalcOutList();
        for (int i = 0; i < currentVisicalc.length; i++) {
             OBScheduleInfo ob = currentVisicalc[i];
            if (ob.isObservable()) {
                observable++;
            }

        }

        res = observable + " Observation Blocks ranked. ";

        return (res);
    }

    /**
     * @param obList
     *            The obList to set.
     */
    public void setObList(ObDataContainer[] obList) {
        this.obList = new LinkedList<ObDataContainer>();      
        CollectionUtils.addAll(this.obList, obList);
    }

    /**
     * @return Returns the obList.
     */
    public ObDataContainer[] getObList() {
        return obList.toArray(new ObDataContainer[obList.size()]);
    }
    
    /**
     * @return Returns the obList.
     */
    public ObDataContainer getObFromId(ObDataContainer partialOb) {
        int index = obList.indexOf(partialOb);
        ObDataContainer ob = partialOb;
        if (index >= 0) {
            ob =  obList.get(index);
        }    
        return ob;
    }

    

    /**
     * @return Returns the obWeatherFilter.
     */
    private ObWeatherFilter getObWeatherFilter() {
        return obWeatherFilter;
    }

    /**
     * @param obVisicalcInputList
     *            The obVisicalcInputList to set.
     */
    private void setObVisicalcInputList(ObDataContainer[] obVisicalcInputList) {
        this.obVisicalcInputList = new LinkedList<ObDataContainer>();
        CollectionUtils.addAll(this.obVisicalcInputList, obVisicalcInputList);
    }

    /**
     * @return Returns the obVisicalcInputList.
     */
    private ObDataContainer[] getObVisicalcInputList() {
        return obVisicalcInputList
                .toArray(new ObDataContainer[obVisicalcInputList.size()]);
    }

    /**
     * @param obVisicalcOutList
     *            The obVisicalcOutList to set.
     */
    private void setObVisicalcOutList(OBScheduleInfo[] obVisicalcOutList) {
        this.obVisicalcOutList = new LinkedList<OBScheduleInfo>();
        CollectionUtils.addAll(this.obVisicalcOutList, obVisicalcOutList);
    }

    /**
     * @return Returns the obVisicalcOutList.
     */
    public OBScheduleInfo[] getObVisicalcOutList() {
        return  obVisicalcOutList
                .toArray(new OBScheduleInfo[obVisicalcOutList.size()]);
    }

    /**
     * @param obRanked
     *            The obRanked to set.
     */
    private void setObRanked(OBScheduleInfo[] obRanked) {
        this.obRanked = new LinkedList<OBScheduleInfo>();
        CollectionUtils.addAll(this.obRanked, obRanked);
    }

    /**
     * @return Returns the obRanked.
     */
    public OBScheduleInfo[] getObRanked() {
        return  obRanked.toArray(new OBScheduleInfo[obRanked
                .size()]);
    }

    /**
     * Returns an array of observable OBs
     * 
     * @return the observable obs
     */
    private OBScheduleInfo[] getObservableOBs() {
        OBScheduleInfo[] obs=getObVisicalcOutList();
        LinkedList<OBScheduleInfo> observable = new LinkedList<OBScheduleInfo>();

        for (int i = 0; i < obs.length; i++) {
            OBScheduleInfo ob = obs[i];
            if (ob.isObservable()) {
                observable.add(ob);
            }
        }
        OBScheduleInfo[] res =  observable
                .toArray(new OBScheduleInfo[observable.size()]);
        return (res);

    }

    /**
     * Returns an array of non-observable OBs from the first list of obs
     * 
     * @return the list of not observable
     * 
     */
    public OBScheduleInfo[] getNonObservableOBs() {
        ObDataContainer[] allObs = getObList();
        LinkedList<OBScheduleInfo> notObservable = new LinkedList<OBScheduleInfo>();

        for (int i = 0; i < allObs.length; i++) {
            OBScheduleInfo ob = new OBScheduleInfo();
            ob.setOb(allObs[i]);
            int currentObIndex = obVisicalcOutList.indexOf(ob);
            if (currentObIndex < 0) { // element not in visicalc means not observable
                notObservable.add(ob);
            } else { // must verify element is not observable after visibility calc
               OBScheduleInfo visicalcOb = obVisicalcOutList.get(currentObIndex);
               if (!visicalcOb.isObservable()) {
                   notObservable.add(ob);
               }
            }
        }
        OBScheduleInfo[] res =  notObservable.toArray(new OBScheduleInfo[notObservable.size()]);
        return (res);

    }

    /**
     * if a status change event and the new status of the ob is executed or
     * completed, it removes the ob from all the orang lists
     * 
     * @see org.eso.ohs.persistence.OBListener#obPropertyChange(org.eso.ohs.persistence.OBEvent)
     */
    public synchronized void obPropertyChange(OBEvent event) {
        // if delete event don't do anything. Check only when is a status change
        // event
        if (!event.getValue().equals(DELETE_EVENT)) {
            long[] OBIDs = event.getOBsId();
            for (int i = 0; i < OBIDs.length; i++) {

                String status = event.getValue();   
                long dbId = Config.getCfg().uniqueToTableId(OBIDs[i]);
                if (status != null
                        && !status.equals(OB_STATUS_WRONG)
                        && (status.equals(ObservationBlock.EXECUTED) || status
                                .equals(ObservationBlock.COMPLETED))) {
                    ObTargetConstraintInfo currentOb = new ObTargetConstraintInfo();
                    currentOb.setDbaseId(dbId);
                    ObDataContainer currentObContainer = new ObDataContainer(
                            currentOb);
                    int objPosition = obList.indexOf(currentObContainer);
                    if (objPosition >= 0) {
                        obList.remove(objPosition);
                    }

                    objPosition = obVisicalcInputList
                            .indexOf(currentObContainer);
                    if (objPosition >= 0) {
                        obVisicalcInputList.remove(objPosition);
                    }

                    OBScheduleInfo currentSchedOb = new OBScheduleInfo();
                    currentSchedOb.setOb(currentObContainer);
                    objPosition = obVisicalcOutList.indexOf(currentSchedOb);
                    if (objPosition >= 0) {
                        obVisicalcOutList.remove(objPosition);
                    }
                    objPosition = obRanked.indexOf(currentSchedOb);
                    if (objPosition >= 0) {
                        obRanked.remove(objPosition);
                    }
                }
            }
        }
    }

	public ProgressModelListener getProgressListener() {
		return progressListener;
	}

	public void setProgressListener(ProgressModelListener progressListener) {
		this.progressListener = progressListener;
	}
	
	public ProgressModel [] getLoadOBProgressModel() {
		ProgressModel [] model = new ProgressModel[1];
		model[0] = obLoader;
		return(model);
	}
	
	
	public ProgressModel [] getProgressModels() {
		ProgressModel [] models = new ProgressModel[4];
		models[0] = obLoader;
		models[1] = obWeatherFilter;
		models[2] = visibilityCalc;
		models[3] = rankingEngine;
		
		return(models);
		
	}
	
	
	/**
	 * It ranks the OBs, computing what's needed depending on changes on the model
	 * 
	 * @param orangModel
	 * @param orangGUI
	 * @throws ObjectNotFoundException
	 * @throws ObjectIOException
	 * @throws ValueRejectedException
	 * @throws OrangScriptException
	 */
	public void computeAll(OrangModel orangModel, OrangGUI orangGUI) {

				
		if (orangModel.obSelectionChanged()) {			
						
			new DescIncrementalActionAutomaton(new IncrementalCompute(orangGUI,
					orangModel, IncrementalCompute.ALL), orangGUI,
					"Computing...");
		} else {
			if (orangModel.weatherChanged()) {
				
				new DescIncrementalActionAutomaton(new IncrementalCompute(orangGUI,orangModel,IncrementalCompute.FROM_WEATHER),orangGUI,"Computing...");
			} else {
				if ( orangModel.visibilityConstraintChanged()) {
				
					new DescIncrementalActionAutomaton(new IncrementalCompute(orangGUI,orangModel,IncrementalCompute.FROM_VISIBILITIES),orangGUI,"Computing...");
				} else {
				
					if (orangModel.rankAlgorithmChanged()) {
						new DescIncrementalActionAutomaton(new IncrementalCompute(orangGUI,orangModel,IncrementalCompute.FROM_RANK),orangGUI,"Computing...");
					} 
				}
			}
		}
	}
	
	/**
     * Creates and pops-up the orang queue views
     * @param model
     */
    public void updateGUI(OrangModel model,OrangGUI gui) {
    	try {
    		
    		createOrangQueues(model);
    		gui.update(queueObservable,queueNotObservable);
    		
    	} catch (ObjectIOException e) {
    		JOptionPane.showMessageDialog(null,
    				"An error occured while registering the queue", "Error",
    				JOptionPane.WARNING_MESSAGE);
    		e.printStackTrace();
    	}
    }

	public void createOrangQueues(OrangModel model) throws ObjectIOException {
		String orangQueueName = OHSConfig.getInstance().getValue("ORANG.OBSERVABLE.NAME");
		String orangNotObsQueueName = OHSConfig.getInstance().getValue("ORANG.NOTOBSERVABLE.NAME");
		
		Persistence objMgr = Persistence.getInstance();
		if(!objMgr.isInRegistry(queueObservable.getId())) {    		
		     queueObservable = createNewQueue(model, orangQueueName,true);
		}    
		if(!objMgr.isInRegistry(queueNotObservable.getId())) {    		            
		     queueNotObservable = createNewQueue(model, orangNotObsQueueName,true);
		}         		
		OBScheduleInfo[] obNotObservable = getNonObservableOBs();
		QueueItem[] notObsQueueItems= buildNotObservQueueItems(obNotObservable,queueNotObservable);
		
		OBScheduleInfo[] obObservable = getObservableOBs();
		QueueItem[] obsQueueItems= buildObservQueueItems(obObservable,queueObservable);          
		    		    		
		
		queueObservable.setQueueList(obsQueueItems);
		queueNotObservable.setQueueList(notObsQueueItems);
	}

	public Queue getQueueNotObservable() {
		return queueNotObservable;
	}

	public Queue getQueueObservable() {
		return queueObservable;
	}

	
	
	/**
	 * This class implements the computation in an incremental way
	 * @author pnunes 01.03.05
	 */
	protected class IncrementalCompute implements DescriptiveIncrementalAction {
	
		
		private static final int ALL = 4;
		private static final int FROM_WEATHER = 3;
		private static final int FROM_VISIBILITIES = 2;
		private static final int FROM_RANK = 1;
		
		
		
		// 	incremental action implementation		
		private boolean loadedOBs;
		private boolean filteredWeather;
		private boolean computedVisibilities;
		private boolean rankedOBs;
		private boolean finished;
		private OrangGUI orangGUI;
		private OrangModel orangModel;
		private int totalSteps;
		private String[] messages;
		private TimerLog timer;
		
		public IncrementalCompute( OrangGUI oranggui, OrangModel model, int totalSteps) {
			orangGUI = oranggui;
			orangModel = model;
			this.totalSteps = totalSteps;			
			
			setLoadedOBs(false);
			setFilteredWeather(false);
			setComputedVisibilities(false);
			setRankedOBs(false);
			setFinished(false);
				
			
			messages = new String [] { LOAD_OBS_MSG , COMPUTE_WEATHER_MSG , COMPUTE_VISIBILITY_MSG , RANK_MSG , FINISH_MSG};			
			timer = new TimerLog(messages);
			
			if ( totalSteps == FROM_WEATHER) {
				messages = new String [] {  COMPUTE_WEATHER_MSG , COMPUTE_VISIBILITY_MSG , RANK_MSG ,FINISH_MSG};
				setLoadedOBs(true);				
			} else if ( totalSteps == FROM_VISIBILITIES) {
				messages = new String [] {   COMPUTE_VISIBILITY_MSG , RANK_MSG,FINISH_MSG };
				setLoadedOBs(true);
				setFilteredWeather(true);
			} else if ( totalSteps == FROM_RANK) {
				messages = new String [] {  RANK_MSG ,FINISH_MSG};
				setLoadedOBs(true);
				setFilteredWeather(true);
				setComputedVisibilities(true);
			} 
			
			
		}
		
		public int getNumStepsRemaining(int counter) {
			return(totalSteps - counter+1);
		}
		
		public void performStep(int stepNo) {
			try {			
				if (!isLoadedOBs()) {	
					timer.calcTimer(stepNo);
					loadObList(orangModel);		
					setLoadedOBs(true);
					timer.calcTimer(stepNo);
					
				} else {
					if (!isFilteredWeather()) {
						timer.calcTimer(stepNo);
						filterObByWeather(orangModel);
						setFilteredWeather(true);
						timer.calcTimer(stepNo);
					} else {					
						if (!isComputedVisibilities()) {
							timer.calcTimer(stepNo);
							filterObByConstraints(orangModel);
							setComputedVisibilities(true);
							timer.calcTimer(stepNo);
						}	else {
								if (!isRankedOBs()) {
									timer.calcTimer(stepNo);
									applyAlgorithm(orangModel);
									setRankedOBs(true);
									timer.calcTimer(stepNo);
								} else {
									if (!isFinished()) {
										timer.calcTimer(stepNo);
										updateGUI(orangModel,orangGUI);
										setFinished(true);
										timer.calcTimer(stepNo);
									}
								}
							}
						}
					}
				
				
			} catch (ObjectNotFoundException e) {
				JOptionPane.showMessageDialog(orangGUI, "Could not load the observation blocks: "+ e.toString(),
						"Error", JOptionPane.ERROR_MESSAGE);
				e.printStackTrace();
			} catch (ObjectIOException e) {
				JOptionPane.showMessageDialog(orangGUI, "Could not load the observation blocks: "+ e.toString(),
						"Error", JOptionPane.ERROR_MESSAGE);
				e.printStackTrace();
			} catch (ValueRejectedException e) {
				JOptionPane.showMessageDialog(orangGUI, "Unexpected error while computing visibilities: "+ e.toString(),
						"Error", JOptionPane.ERROR_MESSAGE);
				e.printStackTrace();
			} catch (OrangScriptException e) {
				JOptionPane.showMessageDialog(orangGUI, "Could not rank the observation blocks: "+ e.toString(),
						"Error", JOptionPane.ERROR_MESSAGE);
				e.printStackTrace();
			} 
			
		}
		
		
		public void startOfAction() {
		}
		
		public void endOfAction() {			
			System.out.println(getLoadedInfo());
			System.out.println(getWeatherInfo());
			System.out.println(getVisibilityInfo());
			System.out.println("Ranking with " + orangModel.getRankingAlgorithm());
			System.out.println(getRankedInfo());			
			System.out.println(timer.reportResult());
			
		}

		private boolean isComputedVisibilities() {
			return computedVisibilities;
		}

		private void setComputedVisibilities(boolean computedVisibilities) {
			this.computedVisibilities = computedVisibilities;
		}

		private boolean isFilteredWeather() {
			return filteredWeather;
		}

		private void setFilteredWeather(boolean filteredWeather) {
			this.filteredWeather = filteredWeather;
		}

		private boolean isLoadedOBs() {
			return loadedOBs;
		}

		private void setLoadedOBs(boolean loadedOBs) {
			this.loadedOBs = loadedOBs;
		}

		private boolean isRankedOBs() {
			return rankedOBs;
		}

		private void setRankedOBs(boolean rankedOBs) {
			this.rankedOBs = rankedOBs;
		}
		
		private boolean isFinished() {
			return finished;
		}
		
		private void setFinished(boolean finished) {
			this.finished = finished;
		}

		public String getStatus(int stepNo) {			
			if (stepNo < messages.length-1) {
					return(messages[stepNo]);
				
			}
			else
				return("Finished.");
		}
	

	}

}
