package org.eso.ohs.phase2.apps.ot.wizard;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.client.DbbView;
import org.eso.ohs.core.dbb.sql.DbbQueryTerminated;
import org.eso.ohs.core.dbb.sql.DbbQueryTerminatedListener;
import org.eso.ohs.dfs.Queue;
import org.eso.ohs.dfs.QueueItem;
import org.eso.ohs.phase2.apps.ot.actions.OrangComputeAction;
import org.eso.ohs.phase2.apps.ot.actions.OrangTimelineAction;
import org.eso.ohs.phase2.apps.ot.dbb.OTDbbView;
import org.eso.ohs.phase2.apps.ot.gui.OBTreeView;
import org.eso.ohs.phase2.apps.ot.gui.QueueItemsBView;
import org.eso.ohs.phase2.apps.ot.gui.QueueOrangView;
import org.eso.ohs.phase2.apps.ot.gui.QueueViewMenuItems;
import org.eso.ohs.phase2.orang.OrangModel;

/**
 * @author pnunes 26.01.2005
 * 
 * This is the standard GUI for Orang
 *
 */
public class OrangGUI extends JFrame implements DbbQueryTerminatedListener{
	
	private static Logger stdlog_ = Logger.getLogger(OrangGUI.class);	
	
	//private static final int NONOBS_PANEL_IDX = 2;	
	//private static final int OBS_PANEL_IDX = 1;
	
	private static final int STD_NONOBS_PANEL_IDX = 1;
	private static final int STD_OBS_PANEL_IDX = 0;
	
	private static final int SHORT_NONOBS_PANEL_IDX = 2;
	private static final int SHORT_OBS_PANEL_IDX = 1;

	private static final String NON_OBS_OB_TAB_TITLE = "Non observable OB";
	private static final String OBS_OB_TAB_TITLE = "Observable OB";
	private static final String SETTINGS_TAB_TITLE = "Settings";
	
	private static final int STANDARD_MODE = 1;
	private static final int SHORT_MODE = 2;
	
	private OBSelectPanel obSelectPanel;
	private AlgorithmChooserPanel algorithmChooserPanel;
	private OBVisibilityPanel oBVisibilityPanel;
	private OBWeatherPanel oBWeatherPanel;
	private TimelinePanel timelinePanel;	
	private OrangModel model;
	private OrangEngine engine;
	private JButton computeButton;
	private JButton showTimelineButton;
	private QueueItemsBView obsView;
	private QueueItemsBView nonObsView;
	private OBTreeView obsTreeView;
	private OBTreeView nonObsTreeView;
	
	private JTabbedPane tabbedPanel;
	private QueueViewMenuItems menuItemsObserv;
	private QueueViewMenuItems menuItemsNotObserv;	
	
	private String currentObsTabTitle;
	private String currentNotObsTabTitle;
	
	private int mode;
	
	
	
	/**
	 * @param model
	 * @param engine
	 * @param shortMode
	 */
	public OrangGUI ( OrangModel model, OrangEngine engine ) {		
		this.engine = engine;
		this.model = model;	
		this.setTitle("ORANG");
		initPanels();		
		
	}
	
	public void showStandard() {
		if ( mode != STANDARD_MODE ) {
			mode = STANDARD_MODE;
			buildPanels(true);
			createMenus();
			pack();
		}
		
		setVisible(true);
		
	}
	
	public void showSmall() {
		if (mode != SHORT_MODE) {
			mode = SHORT_MODE;
			buildPanels(false);
			createMenus();
			pack();
		}
		setVisible(true);
	}
	
	
	/**
	 * Initializes the orang panels
	 */
	private void initPanels() {
		obSelectPanel = new OBSelectPanel(model.getWidgetsData());
		obSelectPanel.initData();		
		oBWeatherPanel = new OBWeatherPanel();
		oBWeatherPanel.initData(model.getWidgetsData());
		oBVisibilityPanel = new OBVisibilityPanel();		
		algorithmChooserPanel = new AlgorithmChooserPanel();
		algorithmChooserPanel.initData(model.getWidgetsData());
		timelinePanel = new TimelinePanel();
		timelinePanel.initData();
	}


	/**
	 * builds the panels
	 */
	private void buildPanels(boolean standard) {	
		
		setContentPane(new JPanel());
		getContentPane().setLayout(new BorderLayout());				
		JPanel titlePanel = OrangGUICommon.getOrangTitlePanel();	
		
		getContentPane().add(titlePanel, BorderLayout.NORTH);
		
		if (standard) 
			getContentPane().add(initStandardMainPanel(),BorderLayout.CENTER);
		 else
			getContentPane().add(initShortMainPanel(),BorderLayout.CENTER);

	}


	/**
	 * Initializes the two menu instances, each one referring to one  
	 */
	public void createMenus() {
        OTDbbView oview = obsView.getView();
        JTable otable = obsView.getView().getJTable();
                
		menuItemsObserv = new QueueViewMenuItems(this,oview,otable,obsView,engine.generateSettings(model),engine.getObVisicalcOutList());
        menuItemsObserv.buildObservableMenu(otable);		        
        menuItemsObserv.addJustifyRankingMenu( oview, otable, engine );
        
        OTDbbView noview = nonObsView.getView();
        JTable notable = noview.getJTable();
        menuItemsNotObserv = new QueueViewMenuItems(this,noview,notable,nonObsView,engine.generateSettings(model),engine.getObVisicalcOutList());
        menuItemsNotObserv.buildOrangMenu(notable);

        updateMenu(tabbedPanel.getSelectedIndex());
	}
	
	
	/**
	 * Initializes the two menu instances, each one referring to one  
	 */
	public void updateMenus() {
                
		
		menuItemsObserv.updateVisiplotActions(engine.getObRanked(),engine.generateSettings(model));
		menuItemsNotObserv.updateVisiplotActions(engine.getNonObservableOBs(),engine.generateSettings(model));

        updateMenu(tabbedPanel.getSelectedIndex());
	}


	
	/**
	 * Initializes settings panel
	 * @return JPanel
	 */
	private JPanel getSettingsPanel() {
		JPanel optionPanel = new JPanel(new GridBagLayout());		
		
		GridBagConstraints gridBagConstraints;
			
		// OB selection Panel														
		JPanel dummy = new JPanel(new BorderLayout());
		dummy.setBorder(new TitledBorder(null, "OBs to consider", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, TitledBorder.DEFAULT_POSITION, new Font("Tahoma", 1, 11)));
		dummy.add(obSelectPanel,BorderLayout.WEST);
						
		gridBagConstraints = OrangGUICommon.getGridBagConstraints(0,0,GridBagConstraints.NORTHWEST,0,0,0,0);
		gridBagConstraints.fill = GridBagConstraints.BOTH;		
		optionPanel.add(dummy, gridBagConstraints);
		
		
		// Weather Panel														
		JPanel dummy2 = new JPanel(new BorderLayout());
		dummy2.setBorder(new TitledBorder(null, "Current weather conditions", TitledBorder.DEFAULT_JUSTIFICATION, TitledBorder.DEFAULT_POSITION, new Font("Tahoma", 1, 11)));
		dummy2.add(oBWeatherPanel,BorderLayout.WEST);
		gridBagConstraints = OrangGUICommon.getGridBagConstraints(0,1,GridBagConstraints.NORTHWEST,0,0,0,0);
		gridBagConstraints.fill = GridBagConstraints.BOTH;		
		optionPanel.add(dummy2, gridBagConstraints);
		
		
		// visibility						
		JPanel dummy3 = new JPanel(new BorderLayout());
		dummy3.setBorder(new TitledBorder(null, "Visibility constraints", TitledBorder.DEFAULT_JUSTIFICATION, TitledBorder.DEFAULT_POSITION, new Font("Tahoma", 1, 11)));						
		dummy3.add(oBVisibilityPanel,BorderLayout.WEST);
		gridBagConstraints = OrangGUICommon.getGridBagConstraints(0,2,GridBagConstraints.NORTHWEST,0,0,0,0);				
		gridBagConstraints.fill = GridBagConstraints.BOTH;
		optionPanel.add(dummy3, gridBagConstraints);
		
		
		// ranking algorithm		
		JPanel dummy4 = new JPanel(new BorderLayout());													
		dummy4.setBorder(new TitledBorder(null, "Ranking algoritm", TitledBorder.DEFAULT_JUSTIFICATION, TitledBorder.DEFAULT_POSITION, new Font("Tahoma", 1, 11)));
		dummy4.add(algorithmChooserPanel,BorderLayout.WEST);	
		
										
		// compute button
		computeButton = new JButton("Compute");
		computeButton.addActionListener(new OrangComputeAction(this,"Compute",model,engine));
				
		gridBagConstraints = OrangGUICommon.getGridBagConstraints(0,3,GridBagConstraints.NORTHWEST,0,0,0,0);
		gridBagConstraints.fill = GridBagConstraints.BOTH;
		dummy4.add(computeButton, BorderLayout.EAST);		
		optionPanel.add(dummy4, gridBagConstraints);
		
		
		
		JPanel result = new JPanel(new BorderLayout());
		result.add(optionPanel,BorderLayout.NORTH);
		
		JPanel result2 = new JPanel(new BorderLayout());
		result2.add(result,BorderLayout.WEST);
		return(result2);
		
	}
		
	/**
	 * Creates the main panel with the setting panel on the left side 
	 * and a tabbed panel on the right, containing the two queues
	 * @return
	 */
	private JComponent initStandardMainPanel() {
		
		JPanel mainPanel = new JPanel();
        mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.X_AXIS));
		JPanel leftPanel = getSettingsPanel();
          JScrollPane leftPanelScroll = new JScrollPane(leftPanel);
          
		tabbedPanel = new JTabbedPane();
	
		JPanel obsQueue = getObservablePanel(new QueueItem[] {});
    
      
       // setPreferredSize(new Dimension(450, 110));	
		
		if (getCurrentObsTabTitle() == null) {
			setCurrentObsTabTitle(OBS_OB_TAB_TITLE);
		}
		
		if (getCurrentNotObsTabTitle() == null) {
			setCurrentNotObsTabTitle(NON_OBS_OB_TAB_TITLE);
		}
        JPanel notObsQueue = getNonObservableView(new QueueItem[] {});
        JScrollPane notObsQueueScroll = new JScrollPane(notObsQueue);
        JScrollPane obsQueueScroll = new JScrollPane(obsQueue);
        
		tabbedPanel.addTab(getCurrentObsTabTitle(),obsQueueScroll );		
		tabbedPanel.addTab(getCurrentNotObsTabTitle(),notObsQueueScroll);
		
		tabbedPanel.addChangeListener(new ChangeListener() {
	        // This method is called whenever the selected tab changes
	        public void stateChanged(ChangeEvent evt) {
	            JTabbedPane pane = (JTabbedPane)evt.getSource();
	    
	            // Get current tab
	            int sel = pane.getSelectedIndex();
	            
	            updateMenu(sel);
	        } 
		});
		
          
		mainPanel.add(leftPanelScroll);
        mainPanel.add(Box.createRigidArea(new Dimension(5,0)));
		mainPanel.add(tabbedPanel);
		
		//return(queuesPanel);
		return(mainPanel);
		
	}
	
	
	/**
	 * Creates the main panel with tabbed panel, containing the settings and the two queues tabs
	 * @return
	 */
	private JComponent initShortMainPanel() {
		
		JPanel mainPanel = new JPanel(new BorderLayout());
		JPanel settingsPanel = getSettingsPanel();
           JScrollPane settingsPanelScroll = new JScrollPane(settingsPanel);
              
		tabbedPanel = new JTabbedPane();
	
		JPanel obsQueue = getObservablePanel(new QueueItem[] {});
		JPanel notObsQueue = getNonObservableView(new QueueItem[] {});
        JScrollPane notObsQueueScroll = new JScrollPane(notObsQueue);
        JScrollPane obsQueueScroll = new JScrollPane(obsQueue);
        
		if (getCurrentObsTabTitle() == null) {
			setCurrentObsTabTitle(OBS_OB_TAB_TITLE);
		}
		
		if (getCurrentNotObsTabTitle() == null) {
			setCurrentNotObsTabTitle(NON_OBS_OB_TAB_TITLE);
		}
		
		tabbedPanel.addTab(SETTINGS_TAB_TITLE,settingsPanelScroll);
		tabbedPanel.addTab(getCurrentObsTabTitle(),obsQueueScroll );		
		tabbedPanel.addTab(getCurrentNotObsTabTitle(),notObsQueueScroll);
		
		tabbedPanel.addChangeListener(new ChangeListener() {
	        // This method is called whenever the selected tab changes
	        public void stateChanged(ChangeEvent evt) {
	            JTabbedPane pane = (JTabbedPane)evt.getSource();
	    
	            // Get current tab
	            int sel = pane.getSelectedIndex();
	            
	            updateMenu(sel);
	        } 
		});
		
		
		mainPanel.add(tabbedPanel);
		
		return(mainPanel);
		
	}
	
	
	
	
	/**
	 * Update the GUI with two new queues
	 * 
	 * @param obsQueue
	 * @param nonObs
	 */
	public void update(Queue obsQueue, Queue nonObs) {
		obsView.setQueue(obsQueue);
		nonObsView.setQueue(nonObs);
		obsView.setDbbQueueId();
		nonObsView.setDbbQueueId();
		updateObservablePanel(obsQueue);
		updateNonObservablePanel(nonObs);				
		updateMenus();		
		//pack();
	}
	
	/**
	 * Updates the observable panel with the new queue and sets its title
	 * 
	 * @param queue
	 * @param items
	 */
	private void updateObservablePanel(Queue obsQueue) {						
	    //obsView.deleteQueueItems();	
		obsView.deleteAndAppendQueueItems(obsQueue.getQueueList()); 
		//obsView.getView().sqlStartAction();
		setCurrentObsTabTitle(OBS_OB_TAB_TITLE + " (" + obsQueue.getQueueList().length + ")");		
		tabbedPanel.setTitleAt(getObservablePanelIndex(), getCurrentObsTabTitle() );

	}
	
	
	
	
	/**
	 * Updates the non-observable panel with the new queue and sets its title
	 * 
	 * @param queue
	 * @param queueItems
	 */
	private void updateNonObservablePanel( Queue nonObs) {		
	    nonObsView.deleteAndAppendQueueItems(nonObs.getQueueList());
		//nonObsView.getView().sqlStartAction();		
		setCurrentNotObsTabTitle(NON_OBS_OB_TAB_TITLE + " (" + nonObs.getQueueList().length + ")");
		tabbedPanel.setTitleAt(getNonObservablePanelIndex(), getCurrentNotObsTabTitle());
		
	}
	
	
	/**
	 * @param queue
	 * @param items
	 * @return
	 */
	private JPanel getObservablePanel(QueueItem [] items) {
		JPanel obsQueue = new JPanel(new BorderLayout());
		obsQueue.add(getObservableView(items),BorderLayout.NORTH);
				
		JPanel timeline = new JPanel();		
		timeline.setBorder(new TitledBorder(null, "Timeline algorithm", TitledBorder.DEFAULT_JUSTIFICATION, TitledBorder.DEFAULT_POSITION, new Font("Tahoma", 1, 11)));				
		timeline.add(timelinePanel);	
		showTimelineButton = new JButton();
		showTimelineButton.setText("Show");
		showTimelineButton.addActionListener(new OrangTimelineAction(this,"Show",model,engine));			
		timeline.add(showTimelineButton);
		
		obsQueue.add(timeline,BorderLayout.WEST);
		
		return(obsQueue);
	}
	
	
	
	/**
	 * Creates and returns a view of observable OBs
	 * 
	 * @param queue
	 * @param items
	 * @return
	 */
	private JPanel getObservableView(QueueItem [] items) {
		JPanel content = new JPanel();
		content.setLayout(new BorderLayout());
        obsTreeView = new OBTreeView();
        
        //if (obsView != null) {
		//	obsView.makeComponentsAavailableForGC();		
	//		obsView.getView().getDbbView().removeQueryTerminatedListener(this);
		//}
        if(obsView==null ) {
            obsView = QueueItemsBView.createObservableQueue(engine.getQueueObservable(),obsTreeView);
            obsView.addJustifyAction(engine);
            obsView.getView().getDbbView().addQueryTerminatedListener(this);
        }
        
        content.add(obsView.getOuterPanel(), BorderLayout.CENTER);                      
        return(content);                				

	}
	
	
	/**
	 * Creates and returns a view of non observable OBs 
	 * 
	 * @param queue
	 * @param items
	 * @return
	 */
	private JPanel getNonObservableView(QueueItem [] items) {
		JPanel content = new JPanel();
        content.setLayout(new BorderLayout());
        
//        if (nonObsView != null) 
//			nonObsView.makeComponentsAavailableForGC();
		        
        nonObsTreeView = new OBTreeView();
        if(nonObsView == null) {
            nonObsView = QueueItemsBView.createNotObservableQueue(engine.getQueueNotObservable(),nonObsTreeView);        	
        }
        content.add(nonObsView.getOuterPanel(), BorderLayout.CENTER);        
        return(content);
	}
	
	
	
	/**
	 * Updates the model with the GUI's contents 
	 */
	public void updateModel() {
						
		obSelectPanel.updateModel(model);
		oBWeatherPanel.updateModel(model);
		oBVisibilityPanel.updateModel(model);
		algorithmChooserPanel.updateModel(model);
		timelinePanel.updateModel(model);
		
	}
	
	/* 
	 * Validates the user input
	 */
	public boolean isValid() {
		
		return(obSelectPanel.isReady() &&
				oBWeatherPanel.isReady() &&
				oBVisibilityPanel.isReady() &&
				algorithmChooserPanel.isReady() 
				 );
	}
	
	public boolean validTimeline() {
		return(timelinePanel.isReady());
	}
	
	/**
	 * Returns validation message assigned when calling isValid() method 
	 * @return
	 */
	public String getValidationString() {
		return ( obSelectPanel.getValidationString() +
				oBWeatherPanel.getValidationString() +
				oBVisibilityPanel.getValidationString() +
				algorithmChooserPanel.getValidationString() +
				timelinePanel.getValidationString() );
	}
	
	private void updateMenu(int sel) {		
		if (isObservablePanelActive(sel)) {
			setJMenuBar(menuItemsObserv.getMenuBar());
			stdlog_.debug("Changed to observable Tab");			
		} else {			
			setJMenuBar(menuItemsNotObserv.getMenuBar());
			stdlog_.debug("Changed to non-observable Tab");
			
		}
		
	}


	public void queryTerminatedAction(DbbQueryTerminated ev) {
		stdlog_.debug("query terminated action");
        DbbView view = obsView.getView().getDbbView();
        QueueOrangView.addRankColumn(view);
		
	}
	
	
	private int getObservablePanelIndex() {
		if (mode == SHORT_MODE) {
			return(SHORT_OBS_PANEL_IDX);			
		} else {
			return(STD_OBS_PANEL_IDX);
		}
	}
	
	private int getNonObservablePanelIndex() {
		if (mode == SHORT_MODE) {
			return(SHORT_NONOBS_PANEL_IDX);			
		} else {
			return(STD_NONOBS_PANEL_IDX);
		}
	}
	
	
	private boolean isObservablePanelActive(int selectedIndex) {
		if (mode == SHORT_MODE) {
			if ( selectedIndex == SHORT_OBS_PANEL_IDX ) 
				return(true);	
			else
				return(false);
		} else {
			if ( selectedIndex == STD_OBS_PANEL_IDX )
				return(true);
			else
				return(false);
			
		}
	}

	private String getCurrentNotObsTabTitle() {
		return currentNotObsTabTitle;
	}

	private void setCurrentNotObsTabTitle(String currentNotObsTabTitle) {
		this.currentNotObsTabTitle = currentNotObsTabTitle;
	}

	private String getCurrentObsTabTitle() {
		return currentObsTabTitle;
	}

	private void setCurrentObsTabTitle(String currentObsTabTitle) {
		this.currentObsTabTitle = currentObsTabTitle;
	}
	

	
}
