/*
 * Created by amchavan on Jul 14, 2005
 */
package org.eso.ohs.phase2.apps.ot.wizard;

import java.awt.Component;
import java.awt.Cursor;

import javax.swing.JOptionPane;

import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.widgets.OHSWizard;
import org.eso.ohs.core.gui.widgets.OHSWizardPage;
import org.eso.ohs.core.utilities.MsgManager;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.phase2.orang.OrangModel;
import org.eso.ohs.phase2.orang.OrangModelDefault;
import org.eso.ohs.phase2.orang.OrangScriptException;
import org.eso.ohs.phase2.orang.OrangWidgetsData;
import org.eso.ohs.phase2.visibility.PCFPlotterFilter.ValueRejectedException;

// $Id: OrangWizard.java,v 1.21 2006/03/03 12:20:52 pnunes Exp $
public class OrangWizard extends OHSWizard  {
  
    private static Logger stdlog_ = Logger.getLogger(OrangWizard.class);

    private static final long serialVersionUID = 1L;

    private OBSelectPage selectorPage;
    private OBWeatherPage weatherPage;
    private OBVisibilityPage constrainPage;
    private AlgorithmChooserPage algorithmPage;
    private OrangEngine orang;
    private FinishPage finishedPage;
    private TimelinePage timelinePage;
    private OrangWidgetsData defaultData;
    private OrangModel model;
    
    
    public OrangWizard(OrangEngine orangEngine) {
    	
        orang = orangEngine;        
        defaultData = new OrangWidgetsData();
        selectorPage = new OBSelectPage(this, defaultData);
        weatherPage = new OBWeatherPage(this, defaultData);
        constrainPage = new OBVisibilityPage(this, defaultData);
        algorithmPage = new AlgorithmChooserPage(this, defaultData);
        timelinePage = new TimelinePage(this, defaultData);
        finishedPage = new FinishPage(this, defaultData);
        model = new OrangModelDefault(defaultData);

        addPage(selectorPage);
        addPage(weatherPage);
        addPage(constrainPage);
        addPage(algorithmPage);
        addPage(timelinePage);
        addPage(finishedPage);
    }

    /* (non-Javadoc)
     * @see org.eso.ohs.core.gui.widgets.OHSWizard#leavingPage(org.eso.ohs.core.gui.widgets.OHSWizardPage, boolean)
     */
    protected void leavingPage(OHSWizardPage from, boolean usingNext) {
        if (usingNext) {
            try {

            	// sets the cursor
            	((Component)getTopLevelAncestor()).setCursor(new Cursor(Cursor.WAIT_CURSOR));
            	
                if (from instanceof OBSelectPage) {
                	selectorPage.updateModel(model);
                	
                	if (model.obSelectionChanged()) {
                		orang.loadObList(model);
                		weatherPage.setInfo(orang.getLoadedInfo());
                	}
                	
                	
                } else if (from instanceof OBWeatherPage) {
                	weatherPage.updateModel(model);
                	
                	if (model.obSelectionChanged() || model.weatherChanged() ) {
                		orang.filterObByWeather(model);
                		constrainPage.setInfo(orang.getWeatherInfo());
                	}
                } else if (from instanceof OBVisibilityPage) {
                	constrainPage.updateModel(model);
                	
                	if (model.obSelectionChanged() || model.weatherChanged()
							|| model.visibilityConstraintChanged()) {
                		orang.filterObByConstraints(model);
                		algorithmPage.setInfo(orang.getVisibilityInfo());
                	}
                    
                 } else if (from instanceof AlgorithmChooserPage) {
                	algorithmPage.updateModel(model);
                	if (model.obSelectionChanged() || model.weatherChanged()
							|| model.visibilityConstraintChanged()
							|| model.rankAlgorithmChanged()) {
                	
                		orang.applyAlgorithm(model);
                		orang.showRankedObs(model);
                		timelinePage.setInfo(orang.getRankedInfo());
                    
                	}
                    
                } else if (from instanceof TimelinePage) {
                	timelinePage.updateModel(model);
                    orang.showTimeline(model);
                    finishedPage.setInfo(orang.getRankedInfo());
                    
                }
            } catch (ObjectNotFoundException e) {
                MsgManager.errDialog("Object I/O Error", e);
                e.printStackTrace();
            } catch (ObjectIOException e) {
                MsgManager.errDialog("Object I/O Error", e);
                e.printStackTrace();
            } catch (ValueRejectedException e) {
            	MsgManager.errDialog("Unexpected error", e);
                e.printStackTrace();
            } catch (OrangScriptException e) {
                JOptionPane.showMessageDialog(algorithmPage
                        .getTopLevelAncestor(), e.getMessage(),
                        "ORANG Script Error", JOptionPane.ERROR_MESSAGE);

                e.printStackTrace();
                return;
            } finally {
            
            	// 	sets the cursor
            	((Component)getTopLevelAncestor()).setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
            }
        }
    }
    
	

    /* (non-Javadoc)
     * @see uic.widgets.UICWizard#leavingWizard(boolean)
     */
    protected void leavingWizard(boolean successful) {
        orang.clear();
        this.setVisible(false);
    }

  

}
