package org.eso.ohs.phase2.apps.ot;

import java.io.File;
import java.io.IOException;
import java.text.ParseException;
import java.util.Date;

import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.eso.ohs.core.utilities.Convert;
import org.eso.ohs.core.utilities.NotASCIIException;
import org.eso.ohs.core.utilities.OHSKeys;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.ObTargetConstraintInfo;
import org.eso.ohs.dfs.RunProperties;
import org.eso.ohs.dfs.SchedRun;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.EVMStorageMgr;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.StorageManager;
import org.eso.ohs.persistence.dbase.phase1.OpcDbaseMgr;
import org.eso.ohs.persistence.dbase.phase2.DbaseMgr;
import org.eso.ohs.persistence.dbase.phase2.OTDbaseMgr;
import org.eso.ohs.phase2.apps.ot.wizard.OrangEngine;
import org.eso.ohs.phase2.orang.ObDataContainer;
import org.eso.ohs.phase2.orang.OrangModel;
import org.eso.ohs.phase2.orang.OrangModelDefault;
import org.eso.ohs.phase2.orang.OrangScriptException;
import org.eso.ohs.phase2.orang.OrangScriptingBox;
import org.eso.ohs.phase2.orang.OrangWidgetsData;
import org.eso.ohs.phase2.visibility.PCFPlotterFilter.ValueRejectedException;
import org.eso.ohs.phase2.visiplot.TimerLog;

public class OrangPerformance extends TestCase {
		    
    private String configFile_ = "site.cf";
	private String configDir_ = "config"+  File.separator ;
	
    private String dbConfigPath = "config" + File.separator + "ot-db.cf.UNITTESTS";
    
	private String rankAlgorithm = "ParanalPriority";
	
	protected void setUp() throws Exception {				
		
		setupDB();
		String scriptDir = AppConfig.getAppConfig().getValue(OrangScriptingBox.scriptDirectory);				
		OrangScriptingBox.getInstance().setup(this.configDir_);
		
		
		
	}

	private void setupDB() throws IOException, NotASCIIException, ObjectIOException {
		AppConfig.setAppConfig(new AppConfig());
        AppConfig.getAppConfig().setStandardOHSKeys("");
        AppConfig.getAppConfig().setValue(OHSKeys.IPVersioningKey, "n");
        AppConfig.getAppConfig().addParamfile(configDir_+configFile_);
        ObjectManager objMgr;
        DbaseMgr stMgr;
        File f = new File(dbConfigPath);
        System.out.println(f.getAbsolutePath());
        try {
        AppConfig.getAppConfig().setupDbDirectConfig("",new File(dbConfigPath));
        } catch(IOException e) {
            e.printStackTrace();
            throw new IOException(e.getMessage());
        }
        
        Config.setCfg(new TestConfig());
        // InstrCfg.setCfg(new TestInstrCfg());
       

        stMgr = new DbaseMgr();
        stMgr.setAccountId(52004);
        ObjectManager.startup();
        objMgr = ObjectManager.getObjectManager();
        //objMgr.setStorageManager(Media.DBASE, stMgr);
        
        StorageManager  opcMgr = new OpcDbaseMgr();
        EVMStorageMgr evmMgr = new EVMStorageMgr(stMgr,opcMgr);        
        objMgr.setStorageManager(Media.DBASE,evmMgr);
        
        Persistence.startup();
        Persistence per = Persistence.getInstance();
        per.setStorageManager(Media.PHASE1_DB, new OpcDbaseMgr());
        per.setStorageManager(Media.PHASE2_DB, new OTDbaseMgr());
	}

	protected void tearDown() throws Exception {
		// TODO Auto-generated method stub
		super.tearDown();
	}
	
	private ObDataContainer  [] createOBs( int num ) {
		
		ObDataContainer  [] res = new ObDataContainer[num];

		
		String [] classes = {"A","B","C"}; 
						
		for (int i = 1; i <= num; i++) {
			ObTargetConstraintInfo obInfo = new ObTargetConstraintInfo();
					
			
			
			obInfo.setRa(Convert.HHMMSSToMilliarcsec("05:25:01.700"));
			obInfo.setDec(Convert.DDMMSSToMilliarcsec("-10:19:44.000"));
			//obInfo.setRa(Convert.HHMMSSToMilliarcsec((i%24)+":00:00"));
			//obInfo.setDec(Convert.DDMMSSToMilliarcsec((i%360)+":00:00"));
			
			
			obInfo.setExecutionTime(60);
			obInfo.setFractionalLunarIllumination((float)(i/1000.));
			obInfo.setMoonAngularDistance(180);			
			obInfo.setAirmass(2.5f);
			
			
			ObDataContainer  container = new ObDataContainer(obInfo);
			
			SchedRun schedRun = new SchedRun();
			schedRun.setRankClass(classes[i%3]);
			container.setSchedRun(schedRun);
		
			
			RunProperties runProp = new RunProperties();
			runProp.setCarryOver(true);
			runProp.setChilean(false);
			runProp.setPreImaging(true);
			runProp.setTimeCritical(false);
			runProp.setType(4);
			
			container.setRunProperties(runProp);
															
			res[i-1] = container;    		
			
			
		}
		return(res);
	}
	
	
	public void testPerformance() throws ObjectNotFoundException, ObjectIOException, ValueRejectedException, OrangScriptException, ParseException {
		
						
		String [] steps = {"Weather","Visibility","Ranking"};
		TimerLog timer = new TimerLog(steps);
					
		String [] scripts = {rankAlgorithm};
		OrangWidgetsDataTest data = new OrangWidgetsDataTest(scripts);
		OrangModel model = new OrangModelDefault(data);

		Date startTime = Convert.ISODateToDate("2006-04-12T12:00:00");		
		Date endTime = Convert.ISODateToDate("2006-04-13T12:00:00");

		System.out.println("--------------------------------");
		System.out.println("Start of benchmark." );
		System.out.println("Hours of computation: 24" );
		System.out.println("---------------------------------");
		
		
		// set the options
		model.setWindEnabled(false);
		model.setSeeingEnabled(false);
		model.setSkyTranspEnabled(false);
		model.setMoonAngleConstrain(false);
		model.setFliConstrain(true);
		model.setMoonDistanceConstrain(true);
		model.setStartDate(startTime);
		model.setEndDate(endTime);
		model.setTimeIntervalConstrain(false);
		model.setSidTimeConstrain(false);
	
		model.setDuration("10");
		model.setRankingAlgorithm(rankAlgorithm);
		
		//create the OBs
		ObDataContainer [] obs = createOBs(1000);
		
		
		int [] numObs = {1 , 5 , 10 , 50 , 100 , 500 , 1000 , 5000};
		
		for (int i = 0; i < numObs.length; i++) {
			
			System.out.println("------------------------");
			System.out.println("Number OBS: " + numObs[i]);
			System.out.println("------------------------");
			
			
			obs = createOBs(numObs[i]);
			OrangEngine engine = new OrangEngine();
			engine.setObList(obs);
			
			timer.setCalculate(true);		
			timer.calcTimer(0);
			engine.filterObByWeather(model);
			System.out.println("Weather: " + engine.getWeatherInfo());
			timer.calcTimer(0);
			
			timer.calcTimer(1);
			engine.filterObByConstraints(model);
			System.out.println("Visibility: " + engine.getVisibilityInfo());
			timer.calcTimer(1);
			
			
			timer.calcTimer(2);
			engine.applyAlgorithm(model);
			System.out.println("Ranking: " + engine.getRankedInfo());
			timer.calcTimer(2);		
			
			
			System.out.println(timer.reportResult());		
		}
		
		
	}
	
	

	
	public static TestSuite suite() {
        TestSuite suite = new TestSuite();
        suite.addTest(new TestSuite(OrangPerformance.class));
        return suite;
    }
	

	protected class OrangWidgetsDataTest extends OrangWidgetsData {
		
		
		private String [] scripts;
		
		public OrangWidgetsDataTest (String [] scripts) {			
			this.scripts = scripts;
		}
		
		public String[] getAlgorithmList() throws OrangScriptException {
			return( scripts );
		}
		
		public String getCurrentDate() {			
			return super.getCurrentDate();
		}
		
		public String[] getDurationRangeList() {
			return( new String[] {"10","30","60"} );
		}
		
		public String[] getInstruments() {
			return( new String[] {"VIMOS"} );
		}
		
		public String[] getQueueInst() {
			return new String[0];
		}
		
		public String[] getQueueInst(String inst) {
			return new String[0];
		}
		
		public String[] getQueues() {
			return new String[0];
		}
		
		public String[] getQueueTel(String tel) {
			return new String[0];
		}
		
		public String[] getSkyTranspList() {
			return new String[0];
		}
		
		public String getStartISODate() {
			return("");
		}
		
		public String getStepInterval() {
			return("10");
		}
		
		public String[] getStepRangeList() {
			return new String[0];
		}
		
		public String[] getTelescopes() {
			return new String[0];
		}
	}
	

	
}

