package org.eso.ohs.phase2.apps.ot;

import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.AccountInfo;
import org.eso.ohs.dfs.BusinessObject;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.ConstraintSet;
import org.eso.ohs.dfs.Mask;
import org.eso.ohs.dfs.ObTargetConstraintInfo;
import org.eso.ohs.dfs.ObjectFactory;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.ObservationDescription;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.dfs.Queue;
import org.eso.ohs.dfs.RunProperties;
import org.eso.ohs.dfs.SchedRun;
import org.eso.ohs.dfs.Target;
import org.eso.ohs.dfs.TimeInterval;
import org.eso.ohs.p2pp.Folder;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.DirectoryNode;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectManager;

/**
 * Config class for unit tests
 * 
 * <p>
 * $Id: TestConfig.java,v 1.1.2.2 2007/10/11 08:24:45 pnunes Exp $<br>
 * 
 * @author Tim Canavan
 * @author Bob Kemp
 */
public class TestConfig extends Config {

    public final static int DEFAULT_USER_ID = 52004;

    public TestConfig() {
    	AccountInfo info = new AccountInfo();
    	info.setAccountId(DEFAULT_USER_ID);
    	info.setUsername(DEFAULT_USER_ID + "");
    	info.setPasswd("");
    	info.setEngineering(true);
    	info.setOperations(false);    	    	
        setAccount(info);                
    }
    

    public int getAccountId() {
        int userName = 52004;
        return userName;
    }

    public String getDbUserName() {
        return AppConfig.getAppConfig().getDbUserName();
    }

    public String getDbPassword() {
        return AppConfig.getAppConfig().getDbPassword();
    }

    public String getDbName(String arg) {
       return AppConfig.getAppConfig().getDbName(arg);
    }

    public String getUrl(String arg) {
        return AppConfig.getAppConfig().getUrl(arg);
    }

    public boolean isUserLevel(Class cls) {
        return true;
    }

    public boolean isOperationsModeId(int uid) {
        return false;
    }

    public boolean isStandardModeId(int uid) {
        return true;
    }

    public boolean isEngineeringModeId(int uid) {
        return false;
    }

    public boolean isVisitorModeId(int uid) {
        return false;
    }

    public boolean isCloneModeId() {
        return false;
    }

    public boolean isSuperUserEntitled() {
        return false;
    }

    /**
     * What is the first period for which IP versioning info is expected in the
     * data. 0 indicates no start period, no version info is expected.
     */
    public int getStartPeriodForIPVersioning() {
        int rc = 0;
        return rc;
    }

    public void askForAuthentication(Media device) {
    }

    public String[] folderColumnProperties(Class cls) {
        return new String[] {};
    }

    /**
     * Creates a BusinessObject, registers it with the ObjectManager and, if
     * it's a CalBlk or ObsBlk, sets the ObservingRun property.
     * 
     * @param device
     *            Media on which the object is to be stored
     * @param dir
     *            Directory in which the object is to be stored
     * @param objType
     *            Type of object to be created
     */
    public BusinessObject createAndRegister(Media device, DirectoryNode dir,
            Class objType) throws ObjectIOException {
        BusinessObject bo = ObjectFactory.create(objType);
        ObjectManager objMgr = ObjectManager.getObjectManager();

        objMgr.recursiveRegister(device, dir, bo);

        return bo;
    }

    public void initBusObj(Media device, DirectoryNode node, BusinessObject sobj)
            throws ObjectIOException {
    }

    public void initPersistenceObjInfo() {
        ObjConfigPersistence ocp;

        ocp = new ObjConfigPersistence(ConstraintSet.class, new String[] { // Dbase
                                                                           // columns
                                                                           // for
                                                                           // Summary
                                                                           // info
                "cs_id", "item_name", "airmass", "moon_distance", "seeing" },
                "constraint_sets", "cs", 4);
        objCfgPersistence.put(ocp.key, ocp);

        ocp = new ObjConfigPersistence(Target.class, new String[] { // Dbase
                                                                    // columns
                                                                    // for
                                                                    // Summary
                                                                    // info
                "tp_id", "item_name", "ra", "dec", "equinox", "epoch" },
                "target_packages", "tp", 6);
        objCfgPersistence.put(ocp.key, ocp);

        ocp = new ObjConfigPersistence(ObservationBlock.class, new String[] { // Dbase
                                                                              // columns
                                                                              // for
                                                                              // Summary
                                                                              // info
                "obs_blocks.ob_id", "obs_blocks.item_name", "status",
                        "target_packages.item_name",
                        "obs_descriptions.item_name" },
                "obs_blocks, target_packages, obs_descriptions", "ob", 3);
        objCfgPersistence.put(ocp.key, ocp);

        ocp = new ObjConfigPersistence(CalibrationBlock.class, new String[] { // Dbase
                                                                              // columns
                                                                              // for
                                                                              // Summary
                                                                              // info
                "obs_blocks.ob_id", "obs_blocks.item_name", "status",
                        "obs_descriptions.item_name" },
                "obs_blocks, obs_descriptions", "cb", 7);
        objCfgPersistence.put(ocp.key, ocp);

        ocp = new ObjConfigPersistence(ObservationDescription.class,
                new String[] { // Dbase columns for Summary info
                "od_id", "item_name", "instrument" }, "obs_descriptions", "od",
                2);
        objCfgPersistence.put(ocp.key, ocp);

        ocp = new ObjConfigPersistence(ObservingRun.class, new String[] { // Dbase
                                                                          // columns
                                                                          // for
                                                                          // Summary
                                                                          // info
                "obs_runs.id", "obs_programmes.title" },
                "obs_runs, sched_rep, obs_programmes, eso_users, proposed",
                "or", 5);
        objCfgPersistence.put(ocp.key, ocp);

        ocp = new ObjConfigPersistence(Folder.class, new String[] { // Dbase
                                                                    // columns
                                                                    // for
                                                                    // Summary
                                                                    // info
                }, "", "sum", 1);
        objCfgPersistence.put(ocp.key, ocp);

        ocp = new ObjConfigPersistence(DirectoryNode.class, new String[] { // Dbase
                                                                           // columns
                                                                           // for
                                                                           // Summary
                                                                           // info
                }, "", "dir", 0);
        objCfgPersistence.put(ocp.key, ocp);

        ocp = new ObjConfigPersistence(Mask.class, new String[] { // Dbase
                                                                  // columns for
                                                                  // Summary
                                                                  // info
                "masks.ob_id", "masks.ts_index", "masks.param_index",
                        "masks.mask_naid", "masks.mask_id",
                        "masks.mask_slot_num" }, "masks", "msk", 8);
        objCfgPersistence.put(ocp.key, ocp);
        
        
    	ocp = new ObjConfigPersistence(
    		    ObTargetConstraintInfo.class,
    		    new String[] {},
    		    "",
    		    "",
    		    10
    		);
    		
    	objCfgPersistence.put(ocp.key, ocp);
    	
    	
    	ocp = new ObjConfigPersistence(
    		    SchedRun.class,
    		    new String[] {},
    		    "",
    		    "",
    		    11
    		);
    		
    	objCfgPersistence.put(ocp.key, ocp);
    	
    	
    	ocp = new ObjConfigPersistence(
    		    TimeInterval.class,
    		    new String[] {},
    		    "",
    		    "",
    		    15
    		);
    		
    	objCfgPersistence.put(ocp.key, ocp);
    	
    	
    	ocp = new ObjConfigPersistence(
    		    RunProperties.class,
    		    new String[] {},
    		    "",
    		    "",
    		    16
    		);
    		
    	objCfgPersistence.put(ocp.key, ocp);
    	
    	
    	ocp = new ObjConfigPersistence(
    		    Queue.class,
    		    new String[] {},
    		    "",
    		    "",
    		    20
    		);
    		
    	objCfgPersistence.put(ocp.key, ocp);
    	

    }

    public Class getClassForFileSuffix(String suffix) {

        if (suffix.equals("tp")) {
            return Target.class;
        }

        if (suffix.equals("ob")) {
            return ObservationBlock.class;
        }

        if (suffix.equals("od")) {
            return ObservationDescription.class;
        }

        if (suffix.equals("cs")) {
            return ConstraintSet.class;
        }

        if (suffix.equals("or")) {
            return ObservingRun.class;
        }

        return null;
    }


}

