package org.eso.ohs.phase2.apps.p2pp;

import java.awt.Component;
import java.awt.Cursor;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.util.Vector;

import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JTable;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;

import org.eso.ohs.core.dbb.client.DismissAction;
import org.eso.ohs.core.gui.utilities.ImageUtils;
import org.eso.ohs.core.gui.widgets.CutPasteUtility;
import org.eso.ohs.dfs.BusinessObject;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.ConstraintSet;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.ObservationDescription;
import org.eso.ohs.dfs.Target;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectContainer;
import org.eso.ohs.persistence.ObjectSelector;
import org.eso.ohs.phase2.actions.OBDiffAction;
import org.eso.ohs.phase2.actions.ShowPAFDescrAction;
import org.eso.ohs.phase2.actions.Stage2VerifyBOAction;
import org.eso.ohs.phase2.apps.p2pp.actions.CopyBOAction;
import org.eso.ohs.phase2.apps.p2pp.actions.CopyPropertyAction;
import org.eso.ohs.phase2.apps.p2pp.actions.P2PPExpTimeReportAction;
import org.eso.ohs.phase2.apps.p2pp.actions.P2PPOBReportAction;
import org.eso.ohs.phase2.apps.p2pp.actions.QuitAction;
/**
 * Common superclass for all JP2PP windows with menus
 *
 * <p>
 * @version $Id: MenuGroups.java,v 1.28.4.1 2006/10/13 14:46:28 gdonino Exp $
 * <br>
 * @author Tim Canavan
 * <br>ChangeLog: cmavros 2002/02/11, 1510/OT Add OBD Format
 * <br>ChangeLog: tcanavan 2002/02/25, Initialisation of the PafTextDisplayWidget
 */

public class MenuGroups extends JFrame {

    /** title string for menus */
    public static final String FILEMENU_NAME = "File";
    public static final String EDITMENU_NAME = "Edit";
    public static final String FOLDERMENU_NAME = "Folder";
    public static final String OBSBLKSMENU_NAME = "ObsBlocks";
    public static final String PREFERENCESMENU_NAME = "Preferences";

    /** vector of window instances that have not been disposed of */
    private static Vector<MenuGroups> wdwVec_ = new Vector<MenuGroups>();

    protected Action opTemplatesAction_;
    protected Action acqTemplateAction_;
    protected Action targetAction_;
    protected Action constraintAction_;
    protected Action commentsAction_;
    protected Action commentsinstAction_;
    protected Action copyFCAction_;
    protected Action copyTIAction_;
    protected Action copySTTIAction_;
    protected JMenuItem obDiffMenu_;



    protected JMenuItem syncSelected_;
    protected JMenuItem syncFolder_;
    protected JMenuItem selectExpMenuItem_;
    protected JMenuItem selectObsBlockMenuItem_;
    protected JMenuItem selectVerifyMenuItem_;

    //protected JMenuItem selectVisiplotMenuItem_;
    protected JMenu showPAFDescriptionMenu;
    protected JMenuItem showPAFDescriptionMenuItem_;
    
    


    /**
     * Default constructor
     */
    public MenuGroups() {}


    /**
     * Constructor
     *
     * @param title     initial title of window
     */
    public MenuGroups(String title) {
        super(title);
    }


    /**
     * XXX This is a hack to make menus work properly under HP-UX JDK1.1.8
     * Without this kludge the first three menus are empty when using
     * the JIT, presumably because the system is somehow not notified
     * about the added elements
     */
    public void addNotify() {
        super.addNotify();
        // XXX Kludge for HP-UX JDK 1.1.8 with JIT enabled
        setJMenuBar(getJMenuBar());
    }


    protected void addClosableWindow() {
        // add active window to list and add listener for disposals
        wdwVec_.addElement(this);

    }


    protected void addTargetMenu(String menuName, ObjectSelector objSelector) {
        targetAction_ =
            new CopyBOAction(objSelector, this, Target.class, "Copy Target");
        targetAction_.setEnabled(false);
        addMenuItem(menuName, targetAction_);
    }


    protected void addCopyBOMenu(String menuName, ObjectSelector objSelector) {

        opTemplatesAction_ = new CopyBOAction(objSelector,
            this, ObservationDescription.class,
            "Copy " + ObservationDescription.getDisplayName());
            
        copyFCAction_ = new CopyPropertyAction(objSelector,
            this,"FindingCharts","Copy " + "Finding charts");
            
        copyTIAction_ = new CopyPropertyAction(objSelector,
            this,"TimeIntervals","Copy " + "Time Intervals");
            
        copySTTIAction_ = new CopyPropertyAction(objSelector,
            this,"STTimeIntervals","Copy " + "Sideral Time Intervals");
            

        acqTemplateAction_ = new CopyPropertyAction(objSelector,
            this, "TemplateSignature", "Copy Acquisition Template");

        constraintAction_ = new CopyBOAction(objSelector,
            this, ConstraintSet.class, "Copy Constraint Set");

        commentsAction_ = new CopyPropertyAction(objSelector,
            this,"UserComments", "Copy Comments");
        
        commentsinstAction_ = new CopyPropertyAction(objSelector,
                this,"InstrumentComments", "Copy Instrument Specific Comments");

        
            
            
            

        opTemplatesAction_.setEnabled(false);
        acqTemplateAction_.setEnabled(false);
        constraintAction_.setEnabled(false);
        copyFCAction_.setEnabled(false);
        copyTIAction_.setEnabled(false);
        copySTTIAction_.setEnabled(false);
        commentsinstAction_.setEnabled(false);
        commentsAction_.setEnabled(false);

        addMenuItem(menuName, opTemplatesAction_);
        addMenuItem(menuName, acqTemplateAction_);
        addTargetMenu(menuName, objSelector);
        addMenuItem(menuName, constraintAction_);
        addMenuItem(menuName, copyFCAction_);
        addMenuItem(menuName, copyTIAction_);
        addMenuItem(menuName, copySTTIAction_);
        addMenuItem(menuName, commentsAction_);
        addMenuItem(menuName, commentsinstAction_);

    }
    
   

    protected void createReportsMenu(ObjectContainer container,JTable table, Media device) {

        JMenu expMenu = new JMenu("Execution Time");
        expMenu.add(new P2PPExpTimeReportAction(container,table, true, "All"));
        selectExpMenuItem_= expMenu.add(
            new P2PPExpTimeReportAction(container,table,false, "Selected"));
        selectExpMenuItem_.setEnabled(false);


        JMenu obMenu = new JMenu("ObsBlocks Breakdown");
        obMenu.add(new P2PPOBReportAction(container,table, true, "All"));
        selectObsBlockMenuItem_= obMenu.add(
            new P2PPOBReportAction(container,table,false,"Selected"));
        selectObsBlockMenuItem_.setEnabled(false);

        JMenu verifyMenu = new JMenu("Verify");
        verifyMenu.add(new Stage2VerifyBOAction(container,table, true, "All"));
        selectVerifyMenuItem_= verifyMenu.add(
            new Stage2VerifyBOAction(container,table,false, "Selected"));
        selectVerifyMenuItem_.setEnabled(false);

        showPAFDescriptionMenu = new JMenu("Display in OBD (PAF) format");
        showPAFDescriptionMenu.add(
            new ShowPAFDescrAction(container,table, true, "All"));
        showPAFDescriptionMenuItem_ = showPAFDescriptionMenu.add(
            new ShowPAFDescrAction(container,table,false, "Selected"));
        showPAFDescriptionMenuItem_.setEnabled(false);

        if (!Config.getCfg().isInEngineeringMode()
                && !Config.getCfg().isDebugMode()) {
            showPAFDescriptionMenu.setEnabled(false);
        }
        String menuName = "Reports";
        obDiffMenu_ = new JMenuItem();
        Action act = new OBDiffAction(container,"OB Difference"); 
        obDiffMenu_.setAction(act);
        addSubMenu(menuName,obDiffMenu_);
        addSubMenu(menuName, expMenu);
        addSubMenu(menuName, obMenu);
        addSubMenu(menuName, verifyMenu);
        addSubMenu(menuName, showPAFDescriptionMenu);
    }
        protected void createReportsMenuLocal(ObjectContainer container,Media device) {

            JMenu expMenu = new JMenu("Execution Time");
            expMenu.add(new P2PPExpTimeReportAction(container, true, "All"));
            selectExpMenuItem_= expMenu.add(
                new P2PPExpTimeReportAction(container,false, "Selected"));
            selectExpMenuItem_.setEnabled(false);


            JMenu obMenu = new JMenu("ObsBlocks Breakdown");
            obMenu.add(new P2PPOBReportAction(container, true, "All"));
            selectObsBlockMenuItem_= obMenu.add(
                new P2PPOBReportAction(container,false,"Selected"));
            selectObsBlockMenuItem_.setEnabled(false);

            JMenu verifyMenu = new JMenu("Verify");
            verifyMenu.add(new Stage2VerifyBOAction(container,true, "All"));
            selectVerifyMenuItem_= verifyMenu.add(
                new Stage2VerifyBOAction(container,false, "Selected"));
            selectVerifyMenuItem_.setEnabled(false);

            showPAFDescriptionMenu = new JMenu("Display in OBD (PAF) format");
            showPAFDescriptionMenu.add(
                new ShowPAFDescrAction(container, true, "All"));
            showPAFDescriptionMenuItem_ = showPAFDescriptionMenu.add(
                new ShowPAFDescrAction(container,false, "Selected"));
            showPAFDescriptionMenuItem_.setEnabled(false);

            if (!Config.getCfg().isInEngineeringMode()
                    && !Config.getCfg().isDebugMode()) {
                showPAFDescriptionMenu.setEnabled(false);
            }


        /*
        JMenu visiplotMenu = new JMenu("Visibility Plots");
        visiplotMenu.add( new VisiplotAction(container, true, "All"));
        selectVisiplotMenuItem_= visiplotMenu.add(
            new VisiplotAction(container, false, "Selected"));
        selectVisiplotMenuItem_.setEnabled(false);
        */

        String menuName = "Reports";
        obDiffMenu_ = new JMenuItem();
        Action act = new OBDiffAction(container,"OB Difference"); 
        obDiffMenu_.setAction(act);
        addSubMenu(menuName,obDiffMenu_);
        addSubMenu(menuName, expMenu);
        addSubMenu(menuName, obMenu);
        addSubMenu(menuName, verifyMenu);
        addSubMenu(menuName, showPAFDescriptionMenu);
        //addSubMenu(menuName, visiplotMenu);
    }


    public void setEnabledSynchroniseMenu(BusinessObject bo) {

        boolean enableOpTempls   = false;
        boolean enableTargets    = false;
        boolean enableConstraint = false;
        boolean enableComments   = false;
        boolean enableInstComments   = false;
        boolean enableAcqTempl   = false;
        boolean enableFC = false;
        boolean enableTI = false;
        boolean enableSTTI = false;

        

        if (bo instanceof CalibrationBlock) {
            ObservationDescription od = ((CalibrationBlock)bo).getOd();
            enableOpTempls = (od != null);
            enableComments=true;
            if (bo instanceof ObservationBlock) {
                ObservationBlock ob = (ObservationBlock)bo;
                enableAcqTempl   = (ob.getTemplateSignature() != null);
                enableTargets    = true;
                enableConstraint = true;
                if(ob.getFindingCharts().length > 0 && ob.getId() < 0) {
                    enableFC=true;
                }
                if(ob.getTimeIntervals().length > 0 && ob.getId() < 0) {
                    enableTI=true;
                }
                if(ob.getSTTimeIntervals().length > 0 && ob.getId() < 0) {
                    enableSTTI=true;
                }
                
                if(ob.getInstrumentComments()!= null && ob.getInstrumentComments().length()>0) {
                    enableInstComments=true;
                }

                if(ob.getUserComments()!= null && ob.getUserComments().length()>0) {
                    enableComments=true;
                }

                
            }

        } else if (bo instanceof Target) {
            enableTargets = true;
        }

        if (opTemplatesAction_ != null)
            opTemplatesAction_.setEnabled(enableOpTempls);
        if (acqTemplateAction_ != null)
            acqTemplateAction_.setEnabled(enableAcqTempl);
        if (targetAction_ != null)
            targetAction_.setEnabled(enableTargets);
        if (constraintAction_ != null)
            constraintAction_.setEnabled(enableConstraint);
        if (commentsAction_ != null)
            commentsAction_.setEnabled(enableComments);
        if (commentsinstAction_ != null)
            commentsinstAction_.setEnabled(enableInstComments);
        if (copyFCAction_ != null)
            copyFCAction_.setEnabled(enableFC);
        if (copyTIAction_ != null)
            copyTIAction_.setEnabled(enableTI);    
        if (copySTTIAction_ != null)
            copySTTIAction_.setEnabled(enableSTTI);    
            
                
    }


    public void setEnabledSelectedReports(boolean enabled) {
        selectExpMenuItem_.setEnabled(enabled);
        selectObsBlockMenuItem_.setEnabled(enabled);
        selectVerifyMenuItem_.setEnabled(enabled);
        if (Config.getCfg().isInEngineeringMode() || Config.getCfg().isDebugMode()) {
            showPAFDescriptionMenuItem_.setEnabled(enabled);
        } else {
            showPAFDescriptionMenu.setEnabled(false);
        }
        //selectVisiplotMenuItem_.setEnabled(enabled);
    }


    public void setEnabledReports(boolean enabled) {
        getMenu("Reports").setEnabled(enabled);
    }


    public void dispose() {
        super.dispose();
        wdwVec_.removeElement(MenuGroups.this);
    }


    public static MenuGroups[] getWindowList() {
        MenuGroups[] mgrp = new MenuGroups[wdwVec_.size()];
        wdwVec_.copyInto(mgrp);
        return mgrp;
    }


    public void addToToolbar(Action act, String imgName,
                             String toolTip, JToolBar toolbar) {
        JButton btn = toolbar.add(act);
        act.putValue(Action.NAME, "");
        act.putValue(Action.SMALL_ICON, ImageUtils.getIcon(imgName));
        btn.setText(toolTip);
        btn.setToolTipText(toolTip);
    }

    
    public void addToToolbar(Action act, String imgName,
            String text,String toolTip, JToolBar toolbar) {
        JButton btn = toolbar.add(act);
        act.putValue(Action.NAME, "");
        act.putValue(Action.SMALL_ICON, ImageUtils.getIcon(imgName));
        btn.setText(text);
        btn.setToolTipText(toolTip);
    }

    /**
     * Retrieves or creates the menu with the given name in the menubar
     *
     * @param menuName  name of the menu
     */
    protected JMenu getMenu(String menuName) {

        JMenuBar mb = getJMenuBar();
        int numMenus = 0;
        JMenu m = null;
        boolean found = false;

        if (mb == null) {
            mb = new JMenuBar();
            setJMenuBar(mb);
        }

        numMenus = mb.getMenuCount();

        for (int i=0;i<numMenus;i++) {
            m = mb.getMenu(i);
            if (m.getText().equals(menuName)) {
                found=true;
                break;
            }
        }
        if (!found) {
            m = new JMenu(menuName);
            mb.add(m);
        }
        return m;
    }


    /**
     * Adds a action to the named menu
     *
     * @param menuName  name of the menu
     * @param action    action to be added
     */
    public JMenuItem addMenuItem(String menuName, Action action) {
        JMenu m = getMenu(menuName);
        JMenuItem item = m.add(action);
        return item;
    }


    /**
     * Adds a gui component to the named menu
     *
     * @param menuName  name of the menu
     * @param action    action to be added
     */
    public void addMenuComponent(String menuName, Component comp) {
        JMenu m = getMenu(menuName);
        m.add(comp);
    }


    /**
     * Adds a action to the named menu
     *
     * @param menuName  name of the menu
     * @param subMenu   subMenu to be added
     */
    public JMenuItem addSubMenu(String menuName, JMenuItem subMenu) {
        JMenu m = getMenu(menuName);
        JMenuItem mnItem = m.add(subMenu);
        return mnItem;
    }


    /**
     * Adds cut/copy/paste actions to the Edit menu
     */
    public void addCutCopyPaste() {
        String menuName = EDITMENU_NAME;
        JMenu m = getMenu(menuName);
        JMenuItem mnItem;

        Action cutAction = CutPasteUtility.getCutAction();
        cutAction.putValue(Action.NAME, "Cut");
        mnItem = addMenuItem(menuName, cutAction);
        mnItem.setAccelerator(KeyStroke.getKeyStroke(
            KeyEvent.VK_X, KeyEvent.CTRL_MASK));

        Action copyAction = CutPasteUtility.getCopyAction();
        copyAction.putValue(Action.NAME, "Copy");
        mnItem = addMenuItem(menuName, copyAction);
        mnItem.setAccelerator(KeyStroke.getKeyStroke(
            KeyEvent.VK_C, KeyEvent.CTRL_MASK));

        Action pasteAction = CutPasteUtility.getPasteAction();
        pasteAction.putValue(Action.NAME, "Paste");
        mnItem = addMenuItem(menuName, pasteAction);
        mnItem.setAccelerator(KeyStroke.getKeyStroke(
            KeyEvent.VK_V, KeyEvent.CTRL_MASK));
    }


    /**
     * Adds a quit action to the File menu
     */
    public void addExit() {
        String menuName = FILEMENU_NAME;
        JMenu menu = getMenu(menuName);
        JMenuItem mnItem = addMenuItem(menuName, new QuitAction(this));
        mnItem.setAccelerator(KeyStroke.getKeyStroke(
            KeyEvent.VK_Q, InputEvent.CTRL_MASK, false));
    }


    /**
     * Adds a close action to the File menu
     */
    public void addClose() {
        String menuName = FILEMENU_NAME;
        JMenu menu = getMenu(menuName);
        JMenuItem mnItem = addMenuItem(menuName, new DismissAction(this));
        mnItem.setAccelerator(KeyStroke.getKeyStroke(
            KeyEvent.VK_W, InputEvent.CTRL_MASK, false));
    }


    /**
     * Adds a separator line to the specified menu
     */
    public void addSeparator(String menuName) {
        JMenu menu = getMenu(menuName);
        menu.addSeparator();
    }


    public void normalCursor() {
        JComponent glassPane = (JComponent)getGlassPane();
        glassPane.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        glassPane.setVisible(false);
    }

}

