package org.eso.ohs.phase2.apps.p2pp;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.HeadlessException;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.swing.ButtonGroup;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.ScrollPaneConstants;
import javax.swing.border.EmptyBorder;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.text.JTextComponent;

import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.models.FloatRangeModel;
import org.eso.ohs.core.gui.models.IntRangeModel;
import org.eso.ohs.core.gui.models.LimitedCharsModel;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.dfs.Readme;
import org.eso.ohs.instruments.Instrument;
import org.eso.ohs.instruments.InstrumentConstraintConfig;
import org.eso.ohs.instruments.InstrumentConstraintMetaData;
import org.eso.ohs.instruments.InstrumentList;
import org.eso.ohs.instruments.ReadmeAttribute;
import org.eso.ohs.instruments.ReadmeAttributesFactory;
import org.eso.ohs.instruments.ReadmeKeywordList;
import org.eso.ohs.p2pp.Folder;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.ObjectSelectionEvent;
import org.eso.ohs.persistence.ObjectSelectionListener;
import org.eso.ohs.phase2.apps.p2pp.views.P2PPComponent;

/**
 * @author tcanavan
 *
 * To change this generated comment e dit the template variable "typecomment":
 * Window>Preferences>Java>Templates. To enable and disable the creation of type
 * comments go to Window>Preferences>Java>Code Generation.
 */
public class ReadmeEditView
    extends JFrame
    implements ObjectSelectionListener, PropertyChangeListener, TreeSelectionListener {

    private Logger stdlog_ = Logger.getLogger(ReadmeEditView.class);

    private ObservingRun obsRun_;

    private JScrollPane pane_;
    
    private MainPanel mainPanel_ = new MainPanel();
    
    private JPanel northPanel = new JPanel();
    
    private JPanel southPanel = new JPanel();
    
    private JPanel centerPanel = new JPanel();

    private JTabbedPane commentsPane_ = new JTabbedPane();

    private Vector<JComponent> components_ = new Vector<JComponent>();

    private static ReadmeEditView readmeEditView;

    public static ReadmeEditView getInstance() {
        if (readmeEditView == null) {
            readmeEditView = new ReadmeEditView();
        }
        return readmeEditView;
    }

    /**
     * Constructor for JPEGViewer.
     * @throws HeadlessException
     */
    private ReadmeEditView() throws HeadlessException {
        super();
        //getContentPane().setLayout(new BorderLayout());
//        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
//        int maxWidth = (screenSize.width / 2);
//        int maxHeight = ((screenSize.height / 4) * 3);
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent ev) {
                setVisible(false);
            }
        });
        mainPanel_.setLayout(new BorderLayout());
        pane_ = new JScrollPane(mainPanel_);        
        getContentPane().add(pane_);

        Insets insets = new Insets(4,4,4,4);
        
        northPanel.setLayout(new GridBagLayout());
        northPanel.setBorder(new EmptyBorder(insets));
        centerPanel.setLayout(new GridBagLayout());
        centerPanel.setBorder(new EmptyBorder(insets));
        southPanel.setLayout(new GridBagLayout());
        southPanel.setBorder(new EmptyBorder(insets));
        
        
        mainPanel_.add(BorderLayout.NORTH,northPanel);
        mainPanel_.add(BorderLayout.CENTER,centerPanel);
        mainPanel_.add(BorderLayout.SOUTH,southPanel);
        
        
        JMenuBar mb = new JMenuBar();
        String menuName = "File";
        JMenu menu = new JMenu(menuName);
        JMenuItem close = new JMenuItem("Close");
        close.setAccelerator(
            KeyStroke.getKeyStroke(KeyEvent.VK_W, InputEvent.CTRL_MASK, false));
        close.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                setVisible(false);
            }
        });
        menu.add(close);
        mb.add(menu);
        setJMenuBar(mb);
    }

    public void init(ObservingRun obsRun)
        throws IOException, ObjectNotFoundException, ObjectIOException {
        obsRun_ = obsRun;
        Readme readme = null;

        if (obsRun_ != null)
            readme = obsRun.getReadme();
        if (obsRun == null || !obsRun_.isServiceMode() || readme == null) {
            removeFocusListenes();
            northPanel.removeAll();
            southPanel.removeAll();
            centerPanel.removeAll();
            commentsPane_.removeAll();
            repaint();
            return;
        }
        if (obsRun.getIPVersion() != readme.getIpVersion()) {
            refreshReadme(obsRun, readme); 
        }

        if (readme.getRedmeAttributes().length == 0) {
            Instrument inst =
                InstrumentList.getInstance().getInstrument(
                    obsRun.getInstCode(),
                    obsRun.getIPVersion());
            readme.setRedmeAttributes(
                ReadmeAttributesFactory.createReadmeAttributes(inst));
        }
        removeFocusListenes();
        commentsPane_.removeAll();
        commentsPane_.setTabPlacement(JTabbedPane.LEFT);
        northPanel.removeAll();
        southPanel.removeAll();
        centerPanel.removeAll();
        ReadmeAttribute[] readmeAttributes = readme.getRedmeAttributes();
        int j = 10;
        Dimension d = new Dimension(120, 20);
        GridBagConstraints gbc = new GridBagConstraints();
        addSummaryInfo(obsRun, gbc);
        gbc.gridwidth = 1;
        gbc.gridheight = 1;
        gbc.fill = gbc.fill = GridBagConstraints.HORIZONTAL;

        for (int i = 0; i < readmeAttributes.length; i++) {
            InstrumentConstraintMetaData metaData =
                readmeAttributes[i].getReadmeConfig().getMetaData();
            stdlog_.debug("Meta Data Base Type " + metaData.getBaseType());
            InstrumentConstraintConfig config =
                readmeAttributes[i].getReadmeConfig();
            if (config.getSuperType().equals("largetext")) {
                addTextArea(readmeAttributes[i]);
            }
        }
        gbc.gridwidth = 5;
        gbc.weightx=1;
        centerPanel.add(commentsPane_, gbc);
        gbc.weightx=0;
        for (int i = 0; i < readmeAttributes.length; i++) {
            InstrumentConstraintMetaData metaData =
                readmeAttributes[i].getReadmeConfig().getMetaData();
            InstrumentConstraintConfig config =
                readmeAttributes[i].getReadmeConfig();
            stdlog_.debug("Meta Data Base Type " + config.getSuperType());
            if (config.getSuperType().equals("normaltext")
                || config.getSuperType().equals("integer")
                || config.getSuperType().equals("float")
                || config.getSuperType().equals("emailAddressType")) {
                addJTextField(readmeAttributes[i], gbc);
            }
            if (config.getSuperType().equals("BooleanType")) {
                addCheckBox(readmeAttributes[i], gbc);
            }
            if (config.getSuperType().equals("ThreeStateBooleanType")) {
                add3StateRadio(readmeAttributes[i], gbc);
            }
            if (config.getSuperType().equals("FourStateBooleanType")) {
                add4StateRadio(readmeAttributes[i], gbc);
            }
            if (config.getSuperType().equals("multilist")) {
                addMultiListSelect(readmeAttributes[i], gbc);
            }

        }
        if (readme.isCheckedIn()) {
            setEnabled(false);
        } else {
            setEnabled(true);
        }
        repaint();
        setVisible(true);
        pack();
    }

    /**
     * @param obsRun
     * @param readme
     */
    public void  refreshReadme(ObservingRun obsRun, Readme readme) {
        Instrument inst = InstrumentList.getInstance().getInstrument(obsRun.getInstCode(),obsRun.getIPVersion());
        boolean isEquals = ReadmeAttributesFactory.isEqualsMeta(
        inst,
        readme.getRedmeAttributes());
 
        ReadmeAttribute [] newReadmeAttributes =  ReadmeAttributesFactory.refreshReadmeAttributes(
            inst,
            readme.getRedmeAttributes());
         stdlog_.debug("Get Readme Attributes " + readme.getRedmeAttributes().length); 
         readme.setRedmeAttributes(newReadmeAttributes);
         stdlog_.debug("New Readme Attributes " + newReadmeAttributes.length); 
         stdlog_.debug("Get Readme Attributes " + readme.getRedmeAttributes().length); 
         readme.setIpVersion(obsRun.getIPVersion());
        if(!isEquals) {
            JOptionPane.showMessageDialog(
                this,
                new Object[] {
                    "This Readme has been updated to use",
                    "the Instrument Package specified in the Observing Run." },
                "Update to new Instrument Package",
                JOptionPane.INFORMATION_MESSAGE);
  
        }
         // force firing of event for autosaver to save. 
         obsRun_.setReadme(readme);
    }

    public void setEnabled(boolean enabled) {
        for (int i = 0; i < components_.size(); i++) {
            JComponent comp = components_.elementAt(i);
            comp.setEnabled(enabled);
        }
    }

    private void addSummaryInfo(ObservingRun obsRun, GridBagConstraints gbc) {
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.anchor = GridBagConstraints.WEST;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        //gbc.fill = GridBagConstraints.BOTH;
        gbc.gridheight = 1;
        gbc.gridwidth = 1;
        JLabel label = new JLabel("ESO Programme ID : ");
        northPanel.add(label, gbc);
        gbc.gridx++;
        gbc.weightx=1;
        northPanel.add(new JLabel(obsRun.getProgId()), gbc);
        gbc.gridx = 0;
        gbc.gridy++;
        gbc.weightx=0;
        label = new JLabel("Instrument : ");
        northPanel.add(label, gbc);
        gbc.gridx++;
        gbc.weightx=1;        
        northPanel.add(new JLabel(obsRun.getInstCode()), gbc);
        gbc.gridx = 0;
        gbc.gridy++;
        gbc.weightx=0;        
        label = new JLabel("Opc Priority Class : ");
        northPanel.add(label, gbc);
        gbc.gridx++;
        gbc.weightx=1;
        String rankClass=obsRun.getRankClass();
        if(rankClass == null || rankClass.length() <= 0)
            rankClass = "N/A";
        label = new JLabel(rankClass);
        northPanel.add(label, gbc);
        gbc.gridy++;
        gbc.gridx = 0;
        gbc.weightx=0;
        label = new JLabel("Opc Approved Execution Time (Hours) : ");
        northPanel.add(label, gbc);        
        gbc.gridx++;
        gbc.weightx=1;
        label = new JLabel(String.valueOf(obsRun.getDecimalHours()));
        northPanel.add(label, gbc);
        gbc.gridx = 0;
        gbc.gridy++;
        gbc.weightx=0;
        label = new JLabel("Proposal Type : ");
        northPanel.add(label, gbc);
        gbc.gridx++;
        gbc.weightx=1;
        label = new JLabel(String.valueOf(obsRun.getProgType()));
        northPanel.add(label, gbc);
        gbc.gridx = 0;
        gbc.gridy++;
        gbc.weightx=0;
        label = new JLabel("Principal Investigator Name : ");
        northPanel.add(label, gbc);
        gbc.gridx++;
        gbc.weightx=1;
        label = new JLabel(String.valueOf(obsRun.getPIName()));
        northPanel.add(label, gbc);
        gbc.gridy++;
        gbc.gridx = 0;
        gbc.weightx=0;

    }

    private void setModel(
        ReadmeAttribute readmeAttribute,
        JTextComponent comp) {
        InstrumentConstraintConfig config = readmeAttribute.getReadmeConfig();
        InstrumentConstraintMetaData metaData = config.getMetaData();
        if (config.getSuperType().equals("integer")) {
            comp.setDocument(
                new IntRangeModel(
                    metaData.getPattern(),
                    (int) metaData.getMinInclusive(),
                    (int) metaData.getMaxInclusive()));

        }
        if (config.getSuperType().equals("float")) {
            comp.setDocument(
                new FloatRangeModel(
                    metaData.getPattern(),
                    (float) metaData.getMinInclusive(),
                    (float) metaData.getMaxInclusive()));

        }
        if(config.getSuperType().equals("emailAddressType")) {
            stdlog_.debug("!!!!! Setting pattern " + metaData.getMaxLength() + " " + metaData.getPattern());
            comp.setDocument(new LimitedCharsModel(metaData.getMaxLength()));

        }
        if (config.getSuperType().equals("normaltext")
            || config.getSuperType().equals("largetext")) {
            comp.setDocument(new LimitedCharsModel(metaData.getMaxLength()));
        }
    }

    private void addJTextField(
        ReadmeAttribute readmeAttribute,
        GridBagConstraints gbc) {
        String labelText = readmeAttribute.getReadmeConfig().getLabel();
        JLabel label = new JLabel(labelText);
        gbc.gridwidth = 1;
        southPanel.add(label, gbc);
        gbc.gridwidth = 5;
        gbc.gridx++;
        gbc.weightx=1;
        JTextField textField = new MyJTextField();
        setModel(readmeAttribute, textField);
        textField.setName(readmeAttribute.getReadmeConfig().getName());
        String value = readmeAttribute.getValue();
        if (value != null)
            textField.setText(readmeAttribute.getValue());
        setToolTip(readmeAttribute, textField);
        JScrollPane pane = new JScrollPane(textField);
        southPanel.add(pane, gbc);
        addFocusListenerMPanel(textField);
        gbc.gridy++;
        gbc.gridx = 0;
        gbc.weightx=0;

    }

    private void setToolTip(
        ReadmeAttribute readmeAttribute,
        JComponent textField) {
        String toolTip = readmeAttribute.getReadmeConfig().getToolTip();
        if(toolTip.length() > 0)
             textField.setToolTipText(toolTip);
    }

    private void addTextArea(ReadmeAttribute readmeAttribute) {
        String labelText = readmeAttribute.getReadmeConfig().getLabel();
        JTextArea textArea = new MyJTextArea(15,70);
        textArea.setWrapStyleWord(true);
        textArea.setLineWrap(true);
        textArea.setName(readmeAttribute.getReadmeConfig().getName());
        setToolTip(readmeAttribute, textArea);
        JScrollPane pane = new JScrollPane(textArea);
        addFocusListenerMPanel(textArea);
        commentsPane_.add(labelText,pane);
        commentsPane_.setToolTipTextAt(commentsPane_.getComponentCount()-1,readmeAttribute.getToolTipText());
        setModel(readmeAttribute, textArea);
        textArea.setText(readmeAttribute.getValue());
    }

    private void addMultiListSelect(
        ReadmeAttribute readmeAttribute,
        GridBagConstraints gbc) {
        String labelText = readmeAttribute.getReadmeConfig().getLabel();
        JLabel label = new JLabel(labelText);
        southPanel.add(label, gbc);

        gbc.gridwidth = 2;
        gbc.gridx = 0;
        southPanel.add(label, gbc);
        gbc.gridx++;
        JTextField textField = new JTextField(30);
        textField.setEnabled(false);
        JButton btn = new JButton();
        if (!readmeAttribute.hasValue())
            btn.setText("Select Values");
        else
            btn.setText(readmeAttribute.getValue());
        btn.addActionListener(
            new EditableListActionListener(
                (ReadmeKeywordList) readmeAttribute,
                btn));
        gbc.weightx=1;
                
        southPanel.add(btn, gbc);
        gbc.weightx=0;
        gbc.gridy++;
        gbc.gridx = 0;
        addFocusListenerMPanel(btn);
    }

    private void add3StateRadio(
        ReadmeAttribute readmeAttribute,
        GridBagConstraints gbc) {
        String labelText = readmeAttribute.getReadmeConfig().getLabel();
        JLabel label = new JLabel(labelText);
        gbc.gridwidth = 3;
        gbc.gridx = 0;
        southPanel.add(label, gbc);
        JRadioButton button1 = new MyRadioButton("YES");
        JRadioButton button2 = new MyRadioButton("N/A");
        if (readmeAttribute.getValue().equals("YES")) {
            button1.setSelected(true);
        }
        if (readmeAttribute.getValue().equals("N/A")) {
            button2.setSelected(true);
        }
        ButtonGroup buttonGroup = new ButtonGroup();
        buttonGroup.add(button1);
        buttonGroup.add(button2);
        gbc.weightx=1;

        gbc.gridx = 4;
        gbc.gridwidth = 1;
        southPanel.add(button1, gbc);
        gbc.gridx++;
        southPanel.add(button2, gbc);
        gbc.gridx = 0;
        gbc.gridy++;
        gbc.weightx=0;
        button1.setName(readmeAttribute.getReadmeConfig().getName());
        setToolTip(readmeAttribute,button1);
        button2.setName(readmeAttribute.getReadmeConfig().getName());
        setToolTip(readmeAttribute,button2);
        addFocusListenerMPanel(button1);
        addFocusListenerMPanel(button2);

    }

    private void addCheckBox(
        ReadmeAttribute readmeAttribute,
        GridBagConstraints gbc) {
        String labelText = readmeAttribute.getReadmeConfig().getLabel();
        JLabel label = new JLabel(labelText);
        gbc.gridwidth = 3;
        gbc.gridx = 0;
        southPanel.add(label, gbc);
        gbc.gridx++;
        gbc.weightx=1;

        JCheckBox chkBox = new MyJCheckBox(true);
        if (readmeAttribute.getValue().equals("false")) {
            chkBox.setSelected(false);
        }
        addFocusListenerMPanel(chkBox);
        chkBox.setName(readmeAttribute.getName());
        chkBox.setToolTipText(readmeAttribute.getReadmeConfig().getToolTip());
        setToolTip(readmeAttribute,chkBox);

        southPanel.add(chkBox, gbc);
        gbc.gridx = 0;
        gbc.gridy++;
        gbc.weightx=0;

    }

    private void add4StateRadio(
        ReadmeAttribute readmeAttribute,
        GridBagConstraints gbc) {
        String labelText = readmeAttribute.getReadmeConfig().getLabel();
        JLabel label = new JLabel(labelText);
        gbc.gridwidth = 4;
        southPanel.add(label, gbc);
        JRadioButton button1 = new MyRadioButton("YES");
        JRadioButton button2 = new MyRadioButton("NO");
        JRadioButton button3 = new MyRadioButton("N/A");
        if (readmeAttribute.getValue().equals("YES")) {
            button1.setSelected(true);
        }
        if (readmeAttribute.getValue().equals("NO")) {
            button2.setSelected(true);
        }
        if (readmeAttribute.getValue().equals("N/A")) {
            button3.setSelected(true);
        }
        ButtonGroup buttonGroup = new ButtonGroup();
        buttonGroup.add(button1);
        buttonGroup.add(button2);
        buttonGroup.add(button3);
        gbc.weightx=1;

        gbc.gridx = 4;
        gbc.gridwidth = 1;
        southPanel.add(button1, gbc);
        gbc.gridx++;
        southPanel.add(button2, gbc);
        gbc.gridx++;
        southPanel.add(button3, gbc);
        gbc.gridx = 0;
        gbc.gridy++;
        button1.setName(readmeAttribute.getReadmeConfig().getName());
        button2.setName(readmeAttribute.getReadmeConfig().getName());
        button3.setName(readmeAttribute.getReadmeConfig().getName());
        setToolTip(readmeAttribute,button1);
        setToolTip(readmeAttribute,button2);
        setToolTip(readmeAttribute,button3);
        addFocusListenerMPanel(button1);
        addFocusListenerMPanel(button2);
        addFocusListenerMPanel(button3);
        gbc.weightx=0;

    }

    private void addFocusListenerMPanel(JComponent comp) {
        comp.addFocusListener(mainPanel_);
        components_.addElement(comp);
    }

    private void removeFocusListenes() {
        for (int i = 0; i < components_.size(); i++) {
            JComponent comp = components_.elementAt(i);
            comp.removeFocusListener(mainPanel_);
        }
    }

    /**
     * @see org.eso.ohs.persistence.ObjectSelectionListener#objectSelected(org.eso.ohs.apps.ObjectSelectionEvent)
     */
    public void objectSelected(ObjectSelectionEvent ev) {
        if (!isVisible())
            return;

    }
    public void reinit() {
        setTitle("No OB Selected ");
    }

    /* (non-Javadoc)
     * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
     */
    public void propertyChange(PropertyChangeEvent arg0) {

    }

    /* (non-Javadoc)
     * @see javax.swing.event.TreeSelectionListener#valueChanged(javax.swing.event.TreeSelectionEvent)
     */
    public void valueChanged(TreeSelectionEvent arg0) {
        MainWindow mdw = MainWindow.getPrincipalWindow();
        if (mdw == null)
            return;
        HierarchyView hView = mdw.getHierarchyView();
        try {
            Folder folder = hView.getSelectedFolder();
            if (folder != null) {
                ObservingRun obsRun = folder.findObsRun();
                if (obsRun_ == null) {
                    obsRun_ = obsRun;
                }
            }
        } catch (ObjectIOException e) {
            ErrorMessages.announceIOError(this,e);
        }
    }

    //    if (config.isProperty()) {
    //        Class cl = obsRun_.getReadme().getClass();
    //        Method getMeth = null;
    //        try {
    //            getMeth = cl.getMethod("is" + name, new Class[] {
    //            });
    //        } catch (SecurityException e) {
    //            // TODO Auto-generated catch block
    //            e.printStackTrace();
    //        } catch (NoSuchMethodException e) {
    //            // TODO Auto-generated catch block
    //            e.printStackTrace();
    //        }
    //        try {
    //            Method setMeth =
    //                cl.getMethod(
    //                    "set" + name,
    //                    new Class[] { getMeth.getReturnType()});
    //            try {
    //                setMeth.invoke(obsRun_.getReadme(),new Object[] {readmeAttribute[i].getRawValue()});
    //            } catch (IllegalArgumentException e2) {
    //                // TODO Auto-generated catch block
    //                e2.printStackTrace();
    //            } catch (IllegalAccessException e2) {
    //                // TODO Auto-generated catch block
    //                e2.printStackTrace();
    //            } catch (InvocationTargetException e2) {
    //                // TODO Auto-generated catch block
    //                e2.printStackTrace();
    //            }        
    //        } catch (SecurityException e1) {
    //            // TODO Auto-generated catch block
    //            e1.printStackTrace();
    //        } catch (NoSuchMethodException e1) {
    //            // TODO Auto-generated catch block
    //            e1.printStackTrace();
    //        }
    //                        
    //    }

    private class MainPanel extends JPanel implements FocusListener {

        /* (non-Javadoc)
         * @see java.awt.event.FocusListener#focusGained(java.awt.event.FocusEvent)
         */
        public void focusGained(FocusEvent arg0) {
            // TODO Auto-generated method stub

        }

        /* (non-Javadoc)
         * @see java.awt.event.FocusListener#focusLost(java.awt.event.FocusEvent)
         */
        public void focusLost(FocusEvent arg0) {
            stdlog_.debug("Focus Lost " + arg0);
            if(arg0.getComponent() instanceof P2PPComponent) {
                P2PPComponent comp = (P2PPComponent) arg0.getComponent();
                Readme readme = obsRun_.getReadme();
                ReadmeAttribute[] readmeAttribute = readme.getRedmeAttributes();
                for (int i = 0; i < readmeAttribute.length; i++) {
                    InstrumentConstraintConfig config =
                        readmeAttribute[i].getReadmeConfig();
                    String name = config.getName();
                    if (((JComponent) comp).getName().equals(name)) {
                        readmeAttribute[i].setValue(comp.getValue());
                    }
                }
                obsRun_.setReadme(readme);
            }
        }
    }

    /**
     * @author tcanavan
     *
     * To change the template for this generated type comment go to
     * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
     */
    private class KeywordListDialog extends JDialog {

        /** JList for source items */
        private JList srcList_;

        /** JList for destination items */
        private JList destList_;

        /** Source model for source list control */
        private DefaultListModel srcModel_;

        /** Destination model for destination list control */
        private DefaultListModel destModel_;
        //         /** Textfield for keyword date entry */
        //         private JTextField itemText_;

        /** Ok button for commiting selection */
        private JButton ok_;

        /** Cancel button for canceling dialog box */
        private JButton cancel_;

        /** Button that addItem to destination list */
        private JButton addItem_;

        /** Button that removed item from destination list */
        private JButton removeItem_;

        /** Panel to add controls to */
        private JPanel panel_;
        //         /** Button that adds keyword to source list */
        //         private JButton addKeyword_;

        /**
         * We store here a copy of the source values before editing
         * begins; we then use it to restore the original source
         * values if Cancel is clicked.
         *
         * <br>ChangeLog: A M Chavan, 24-Jan-01, SPR 1235.
         */
        private Object[] originalValues_;

        /** Layout manager for panel in dialog */
        GridBagConstraints c;

        private String value_;

        /**
         * Create a KeywordListDialog
         * @param f Frame for dialog
         * @param title Title of dialog
         * @param srcValues source values for model in source JList
         * @param destValues dest source values for model in destination JList
         */
        public KeywordListDialog(
            Frame f,
            String title,
            Object[] srcValues,
            Object[] destValues) {
            super(f, title, true);
            panel_ = new JPanel();
            panel_.setLayout(new GridBagLayout());
            c = new GridBagConstraints();
            c.weightx = 100;
            c.weighty = 100;
            c.gridx = GridBagConstraints.RELATIVE;
            c.gridy = 0;
            c.gridwidth = 1;
            c.gridheight = 5;
            c.fill = GridBagConstraints.BOTH;
            initLists(srcValues, destValues);
            srcList_.setPrototypeCellValue("mmmmMmmmmMmmmmM");
            destList_.setPrototypeCellValue("mmmmMmmmmMmmmmM");
            c.gridheight = 1;
            c.gridwidth = 0;
            c.weightx = 0;
            c.weighty = 0;
            c.fill = GridBagConstraints.NONE;
            c.anchor = GridBagConstraints.NORTH;
            c.gridy = GridBagConstraints.RELATIVE;
            ok_ = new JButton("    OK     ");
            cancel_ = new JButton("   Cancel  ");
            addItem_ = new JButton("    Add    ");
            removeItem_ = new JButton("   Delete  ");
            // addKeyword_  = new JButton("Add Keyword");
            panel_.add(ok_, c);
            panel_.add(cancel_, c);
            panel_.add(addItem_, c);
            panel_.add(removeItem_, c);
            // panel_.add(addKeyword_,c);
            c.fill = GridBagConstraints.BOTH;
            c.gridy = 6;
            c.gridx = 0;
            c.gridwidth = 5;
            c.weightx = 100;
            c.weighty = 100;
            c.anchor = GridBagConstraints.SOUTH;
            // itemLabel_ = new JLabel("Add keyword ");
            // panel_.add(itemLabel_,c);
            // c.gridx++;
            // itemText_ = new JTextField();
            // panel_.add(itemText_,c);
            setContentPane(panel_);
            addActionListeners();
            pack();

            if (f != null) {
                setLocation(f.getLocation());
            }
        }

        /**
         * Initialise the source and destination lists
         * @param srcValues used to creare model for source list
         * @param destValues used to ceate model for destination list
         */
        private void initLists(Object[] srcValues, Object[] destValues) {

            srcModel_ = new DefaultListModel();
            if (srcValues != null) {
                for (int i = 0; i < srcValues.length; i++)
                    srcModel_.addElement(srcValues[i]);
            }
            srcList_ = new JList(srcModel_);

            initDestinationList(destValues);

            final JScrollPane srcScrollPane =
                new JScrollPane(
                    srcList_,
                    ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
                    ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
            panel_.add(srcScrollPane, c);

            final JScrollPane destScrollPane =
                new JScrollPane(
                    destList_,
                    ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
                    ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
            panel_.add(destScrollPane, c);
        }

        /**
         * Set value to be displated in destination list (currently
         * selected keywords). Here we also set the originalValues_
         * field, will be used by the Cancel action.
         *
         * @param destValues List of values to be displayed in the
         *                   destination list
         *
         * <br>ChangeLog: A M Chavan, 24-Jan-01, SPR 1235.
         */
        public void initDestinationList(Object[] destValues) {

            destModel_ = new DefaultListModel();

            if (destValues != null) {

                originalValues_ = new Object[destValues.length];
                for (int i = 0; i < destValues.length; i++) {

                    destModel_.addElement(destValues[i]);
                    originalValues_[i] = destValues[i];
                }
            }

            destList_ = new JList(destModel_);
        }

        /**
         * Add action listeners for this dialog
         */
        private void addActionListeners() {
            addItem_.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    if (srcList_.getSelectedValue() != null) {
                        int srcIndex = srcList_.getSelectedIndex();
                        Object obj = srcModel_.elementAt(srcIndex);
                        if (!destModel_.contains(obj))
                            destModel_.addElement(
                                srcModel_.elementAt(srcIndex));
                    }
                }
            });

            removeItem_.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent evt) {

                    if (destList_.getSelectedValue() != null) {
                        destModel_.removeElementAt(
                            destList_.getSelectedIndex());
                    }
                }
            });

            ok_.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    setVisible(false);
                    dispose();
                }
            });

            cancel_.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent evt) {

                    /* Cancel behaves same as OK, but
                       first restores the values we initially
                       stored in originalValues_.
                       A M Chavan, 24-Jan-01, SPR 1235
                    */

                    initDestinationList(originalValues_);
                    setVisible(false);
                    dispose();
                }
            });
        }

        /**
         * Return value to be displated in grid
         * @return value to be displayed
         */
        public String getValue() {
            StringBuffer value = new StringBuffer();
            for (int i = 0; i < destModel_.getSize(); i++) {
                String item = (String) destModel_.getElementAt(i);
                value.append(item);
                if (i < destModel_.getSize() - 1)
                    value.append(":");
            }
            return value.toString();
        }

        public String[] getSelectedValues() {
            //Object [] array = destModel_.toArray();
            String[] returnArray = new String[destModel_.size()];
            destModel_.copyInto(returnArray);
            return returnArray;
        }
    }

    private class MyJTextArea extends JTextArea implements P2PPComponent {

        /**
         * @param i
         * @param j
         */
        public MyJTextArea(int i, int j) {
            super(i, j);
            // TODO Auto-generated constructor stub
        }

        /* (non-Javadoc)
         * @see org.eso.ohs.phase2.apps.p2pp.views.P2PPComponent#getValue()
         */
        public String getValue() {
            // TODO Auto-generated method stub
            return this.getText();
        }

    }

    private class MyJTextField extends JTextField implements P2PPComponent {

        /**
         * @param i
         * @param j
         */
        public MyJTextField() {
            super();

        }

        /* (non-Javadoc)
         * @see org.eso.ohs.phase2.apps.p2pp.views.P2PPComponent#getValue()
         */
        public String getValue() {

            return this.getText();
        }

    }

    private class MyRadioButton extends JRadioButton implements P2PPComponent {

        private String value;

        /**
         * @param i
         * @param j
         */
        public MyRadioButton(String value) {
            super(value);
            this.value = value;
            // TODO Auto-generated constructor stub
        }

        /* (non-Javadoc)
         * @see org.eso.ohs.phase2.apps.p2pp.views.P2PPComponent#getValue()
         */
        public String getValue() {
            // TODO Auto-generated method stub
            return value;

        }

    }

    private class MyJCheckBox extends JCheckBox implements P2PPComponent {

        private String value;

        /**
         * @param i
         * @param j
         */
        public MyJCheckBox(boolean state) {
            super(null, null, state);
            // TODO Auto-generated constructor stub
        }

        /* (non-Javadoc)
         * @see org.eso.ohs.phase2.apps.p2pp.views.P2PPComponent#getValue()
         */
        public String getValue() {
            // TODO Auto-generated method stub
            if (isSelected())
                return "true";
            return "false";

        }

    }

    private class EditableListActionListener implements ActionListener {

        private ReadmeKeywordList keywordList;

        private JButton btn_;

        public EditableListActionListener(
            ReadmeKeywordList keyList,
            JButton btn) {
            this.keywordList = keyList;
            btn_ = btn;
        }

        /**
         * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
         */
        public void actionPerformed(ActionEvent e) {
            InstrumentConstraintMetaData instMeta =
                keywordList.getReadmeConfig().getMetaData();
            String[] strArray = null;
            if (keywordList.hasValue()) {
                StringTokenizer tok =
                    new StringTokenizer(keywordList.getValue(), ":");
                strArray = new String[tok.countTokens()];
                int i = 0;
                while (tok.hasMoreTokens()) {
                    strArray[i] = tok.nextToken();
                    i++;
                }
            } else {
                strArray = new String[0];
            }

            KeywordListDialog dialog =
                new KeywordListDialog(
                    ReadmeEditView.this,
                    "List Selection",
                    instMeta.getAllowedValues(),
                    strArray);
            dialog.setVisible(true);
            keywordList.setValue(dialog.getValue());
            if (dialog.getValue().length() <= 0)
                btn_.setText("Select Values");
            else
                btn_.setText(dialog.getValue());
            // force firing property change event
            obsRun_.setReadme(obsRun_.getReadme());
        }
    }
}
