package org.eso.ohs.phase2.apps.p2pp.actions;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Hashtable;

import javax.swing.JOptionPane;

import org.apache.log4j.Category;
import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.baseaction.IncrementalAction;
import org.eso.ohs.core.gui.widgets.TextDisplayWidget;
import org.eso.ohs.core.utilities.ObjUtils;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.TextUtils;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.FindingChart;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.ObservationDescription;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.evm.EVMException;
import org.eso.ohs.evm.EvmExecutorImpl;
import org.eso.ohs.instruments.TemplateVersionInfo;
import org.eso.ohs.p2pp.Folder;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.persistence.DirectoryNode;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.OHSApplication;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.phase2.apps.p2pp.FolderGridView;
import org.eso.ohs.phase2.apps.p2pp.MenuGroups;
import org.eso.ohs.phase2.apps.p2pp.P2PP;
import org.eso.ohs.phase2.apps.p2pp.Persistence;

/**
 * Action class to check-in the selected OB.
 * 
 * <p>
 * <br>
 * @author Bob Kemp
 * <br>ChangeLog: cmavros 22Apr02 1195/P2PP Validation of OBs through P2PP AppServer.
 * <br>ChangeLog: cmavros 28JUNE02 Correct the validation of OBs through P2PP AppServer.
 */
public class CheckInAction extends ActionSuperclass {

    private static Category stdlog_ = Category.getInstance(CheckInAction.class);

    /** FolderGridView for the folder containing the selected OB */
    private FolderGridView folderView_;

    private String actionName_ = null;

    /**
     * Constructs a check-in action.
     */
    public CheckInAction(FolderGridView folderView, String actionName) {
        super(folderView, actionName);

        folderView_ = folderView;
        actionName_ = actionName;
    }

    /**
     * Event handler that is called when the menu item, toolbar item,
     * etc is selected.
     * 
     * @param ev    event giving more information
     */
    public void actionPerformedImpl(ActionEvent ev) {
        Persistence objMgr = Persistence.getInstance();
        Long[] idLongs = null;
        CalibrationBlock[] cbs = null;
        CalibrationBlock cb = null;
        ObservingRun or = folderView_.getFolder().findObsRun();

        boolean onlyCheckEVMs = true;

        // What kind of object are we checking in?
        Class objType = folderView_.getDisplayedObjectClass();

        idLongs = folderView_.getAllSelected();
        cbs = new CalibrationBlock[idLongs.length];

        if (actionName_.equals("Check-in")) {
            onlyCheckEVMs = false;

            if(AppConfig.getAppConfig().canCheckInVMOBs(or.getInstCode()) 
             && or.getObsMode().equalsIgnoreCase(ObservingRun.OBSMODE_VISITOR)) {
                int option =
                    JOptionPane.showConfirmDialog(
                        folderView_.getTopLevelAncestor(),
                        new Object[] {
                            "Are you sure you want to check-in the selected objects?",
                            "Beware ! You are checking in " +
                            or.getInstCode()  + " Visitor mode OBs which ",
                            "once checked-in you will not be able to check-out ",
                            "If you need to modify an OB follow the instructions below:-  ",
                            "- duplicate the OB, apply your changes, and check-in the new OB ",
                            "- contact usg-help@eso.org for the cancellation of your first OB (indicating the related OB name and DbaseId) " },
                        "Continue with Check-In?",
                        JOptionPane.OK_CANCEL_OPTION);
                switch (option) {
                    case JOptionPane.CLOSED_OPTION :
                        return;
                    case JOptionPane.CANCEL_OPTION :
                        return;
                }

            } else {
                int option =
                    JOptionPane.showConfirmDialog(
                        folderView_.getTopLevelAncestor(),
                        new Object[] { "Are you sure you want to check-in the selected objects?" },
                        "Continue with Check-In?",
                        JOptionPane.OK_CANCEL_OPTION);
                switch (option) {
                    case JOptionPane.CLOSED_OPTION :
                        return;
                    case JOptionPane.CANCEL_OPTION :
                        return;
                }

            }
        }

        if (!or.getObsMode().equalsIgnoreCase(ObservingRun.OBSMODE_STANDARD)
            && !(or.getObsMode().equalsIgnoreCase(ObservingRun.OBSMODE_VISITOR)
                && AppConfig.getAppConfig().canCheckInVMOBs(or.getInstCode()))) {
            JOptionPane.showMessageDialog(
                folderView_.getTopLevelAncestor(),
                new Object[] {
                    "Only objects for service-mode Observing Runs",
                    "can be checked-in." },
                "Cannot Check-In",
                JOptionPane.ERROR_MESSAGE);
            return;
        }

        Checker chk = new Checker(folderView_, objType);
        Folder folder = folderView_.getFolder().findRootFolder();
        DirectoryNode node = folder.getNode();

        try {
            for (int j = 0; j < idLongs.length; j++) {
                cb =
                    (CalibrationBlock) objMgr.getBusObj(
                        Media.LOCAL,
                        idLongs[j].longValue(),
                        objType);
                if (chk.check(cb)) {
                    cbs[j] = cb;
                }
            }

        } catch (ObjectNotFoundException e) {
            announceNoObject(e);
            return;
        } catch (ObjectIOException e) {
            announceIOError(e);
            return;
        }

        chk.reportErrors();
        int numErrs = chk.getErrorCount();

        if (cbs.length > numErrs) {
            if (!OHSApplication.ensureAuthenticated()) {
                return;
            }

            new P2PPIncrementalActionAutomaton(
                new IncrementalCheckIn(cbs, folderView_, onlyCheckEVMs),
                (MenuGroups) folderView_.getTopLevelAncestor(),
                onlyCheckEVMs ? "Verifying with server..." : "Checking in...");

        } else { // There are no selected OBs now!
            JOptionPane.showMessageDialog(
                folderView_.getTopLevelAncestor(),
                new Object[] {
                    "There are no selected business objects",
                    "after invalid objects have been excluded.",
                    " ",
                    "None of the business objects could be checked-in." },
                "Nothing selected",
                JOptionPane.ERROR_MESSAGE);
            return;
        }

    }

    private static class Checker {

        private FolderGridView folderView_;
        private Class objType_;

        private Hashtable<String, String> badInstruments = new Hashtable<String, String>();

        private int verificationErrors_ = 0;
        private int alreadyCheckedIn_ = 0;
        private int invalidState_ = 0;
        private int instrFilesOutOfDate_ = 0;
        private int instrFilesMissing_ = 0;
        private int objIoErrors_ = 0;
        private int ioErrors_ = 0;
        private ObjectIOException lastObjIOException_ = null;

        public Checker(FolderGridView folderView, Class objType) {
            folderView_ = folderView;
            objType_ = objType;
        }

        /*
         * Note (from Nick Kornweibel):
         *
         * If a template is not found, it is added to the error list with a prefixed
         * "- " string. This way the client can differentiate between an out of date template
         * and a missing template. This prefix is not defined anywhere - it is hard coded into
         * the client and server code. It really needs to be handled as an exception, which would
         * require changing the RMI interface.
         * The server code for this is: RmiStorgaeManager.java
         */

        public boolean check(CalibrationBlock cb) {
            Persistence objMgr = Persistence.getInstance();
            ObservationDescription od = cb.getOd();
            String errs = cb.verify();

            if (!errs.equals("")) {
                verificationErrors_++;
                folderView_.deselectObject(objType_, cb.getId());
                return false;
            }

            if (cb.isCheckedIn()) {
                alreadyCheckedIn_++;
                folderView_.deselectObject(objType_, cb.getId());
                return false;
            }

            if (!CalibrationBlock.PARTIALLY_DEFINED.equals(cb.getStatus())) {
                invalidState_++;
                folderView_.deselectObject(objType_, cb.getId());
                return false;
            }

            try {
                TemplateVersionInfo[] tmplVersions =
                    P2PP.getTemplateSignatureVersions(
                        od.getInstrument(),
                        od.getSignatures());

                String[] versionErrs =
                    objMgr.verifyTemplateSignatures(Media.DBASE, tmplVersions);
                for (int x = 0; x < versionErrs.length; x++) {
                    /*
                     * See note in method header for meaning of "- " prefix.
                     */
                    if (versionErrs[x].startsWith("- ")) {
                        instrFilesMissing_++;
                    } else {
                        instrFilesOutOfDate_++;
                    }
                }

                if (versionErrs.length > 0) {
                    badInstruments.put(od.getInstrument(), od.getInstrument());
                    folderView_.deselectObject(objType_, cb.getId());
                }
            } catch (ObjectIOException e) {
                stdlog_.error(e);
                objIoErrors_++;
                lastObjIOException_ = e;
                folderView_.deselectObject(objType_, cb.getId());
            } catch (IOException e) {
                stdlog_.error(e);
                ioErrors_++;
                folderView_.deselectObject(objType_, cb.getId());
            }

            return true;
        }

        public void reportErrors() {
            // CB can only be checked-in if TSF files are readable,
            //  so that we can check the versions are compatible.
            if (objIoErrors_ > 0) {
                String msg1;
                String msg2;
                String[] excInfo = {
                };

                if (ioErrors_ > 1) {
                    msg1 = objIoErrors_ + " of the objects";
                    msg2 = "They will be ignored.";
                } else {
                    msg1 = "one object";
                    msg2 = "It will be ignored.";
                }

                if (lastObjIOException_ != null) {
                    excInfo =
                        new String[] {
                            " ",
                            "The last error message was: ",
                            " ",
                            lastObjIOException_.getMessage()};
                }

                JOptionPane.showMessageDialog(
                    folderView_.getTopLevelAncestor(),
                    new Object[] {
                        "Errors occurred while accessing " + msg1,
                        msg2,
                        excInfo },
                    "Error",
                    JOptionPane.ERROR_MESSAGE);
            }

            // CB can only be checked-in if TSF files are right version.
            if (instrFilesOutOfDate_ > 0 || instrFilesMissing_ > 0) {
                Enumeration<String> enum0 = badInstruments.keys();
                StringBuffer sb = new StringBuffer();

                while (enum0.hasMoreElements()) {
                    sb.append("    ");
                    sb.append(enum0.nextElement());
                    sb.append("\n");
                }

                Object[] msg = null;
                if (instrFilesMissing_ > 0 && instrFilesOutOfDate_ > 0) {
                    msg =
                        new Object[] {
                            "Instrument files were not found on the server for ",
                            sb.toString(),
                            "The instrument files on your local machine for",
                            sb.toString(),
                            "are out of date.",
                            "Please replace the instrument package",
                            "before you check in this observation block" };
                } else if (instrFilesMissing_ > 0) {
                    msg =
                        new Object[] {
                            "Instrument files were not found on the server for ",
                            sb.toString(),
                            "Please replace the instrument package",
                            "before you check in this observation block" };
                } else if (instrFilesOutOfDate_ > 0) {
                    msg =
                        new Object[] {
                            "The instrument files on your local machine for",
                            sb.toString(),
                            "are out of date.",
                            "Please replace the instrument package",
                            "before you check in this observation block" };
                }
                JOptionPane.showMessageDialog(
                    folderView_.getTopLevelAncestor(),
                    msg,
                    "Error",
                    JOptionPane.ERROR_MESSAGE);
            }

            // An _OB_ can only be checked-in once!
            if (alreadyCheckedIn_ > 0) {
                String msg2;
                String msg1;

                if (alreadyCheckedIn_ > 1) {
                    msg1 = alreadyCheckedIn_ + " objects have";
                    msg2 = "They will be ignored.";
                } else {
                    msg1 = "One object has";
                    msg2 = "It will be ignored.";
                }

                JOptionPane.showMessageDialog(
                    folderView_.getTopLevelAncestor(),
                    new Object[] {
                        msg1 + " already been checked in.",
                        msg2,
                        " ",
                        "An Observation Block can only be checked in once." },
                    "Error",
                    JOptionPane.ERROR_MESSAGE);
            }

            // CB can only be checked-in if TSF files are readable,
            //  so that we can check the versions are compatible.
            if (ioErrors_ > 0) {
                String msg2;
                String msg1;

                if (ioErrors_ > 1) {
                    msg1 = ioErrors_ + " objects have";
                    msg2 = "They will be ignored.";
                } else {
                    msg1 = "One object has";
                    msg2 = "It will be ignored.";
                }

                JOptionPane.showMessageDialog(
                    folderView_.getTopLevelAncestor(),
                    new Object[] {
                        msg1 + " inaccessible instrument files.",
                        msg2,
                        " ",
                        "An object can only be checked in once ",
                        "its instrument files have been checked." },
                    "Error",
                    JOptionPane.ERROR_MESSAGE);
            }

            // CB can only be checked-in if state is "partially defined"
            if (invalidState_ > 0) {
                String msg2;
                String msg1;

                if (invalidState_ > 1) {
                    msg1 = invalidState_ + " Observation Blocks have";
                    msg2 = "They will be ignored.";
                } else {
                    msg1 = "One Observation Block has";
                    msg2 = "It will be ignored.";
                }

                JOptionPane.showMessageDialog(
                    folderView_.getTopLevelAncestor(),
                    new Object[] {
                        msg1 + " are not partially defined.",
                        msg2,
                        " ",
                        "An Observation Block can only be checked in if.",
                        "it is in the 'Partially Defined' state." },
                    "Error",
                    JOptionPane.ERROR_MESSAGE);
            }

            if (verificationErrors_ > 0) {
                String msg =
                    verificationErrors_ > 1
                        ? "There are "
                            + verificationErrors_
                            + " verification errors"
                        : "There is one verification error";

                JOptionPane.showMessageDialog(
                    folderView_.getTopLevelAncestor(),
                    new Object[] {
                        msg,
                        "All errors must be corrected before external verification or checking-in.",
                        " ",
                        "Use 'Verify' for more information." },
                    "Error",
                    JOptionPane.ERROR_MESSAGE);
            }
        }

        public int getErrorCount() {
            return verificationErrors_
                + alreadyCheckedIn_
                + invalidState_
                + instrFilesOutOfDate_
                + instrFilesMissing_
                + objIoErrors_
                + ioErrors_;
        }

    }

    /**
     * Utility class to check in a sequence of OBs, one at a time.
     * The user is also given the opportunity to stop the process using
     * a dialog box.
     *
     * This class also does Verification (EVM scripts) if the
     * verification is enabled and the class is requested to perform
     * verification. While perhaps not the ideal place to be
     * performing verification, it is done here so as to utilize the
     * IncrementalActionAutomaton (progress bar), displayed for
     * checking in OBs.
     *
     * <br>ChangeLog: nkornwei
     */
    public static class IncrementalCheckIn implements IncrementalAction {

        /** cached reference to Persistence */
        private Persistence objMgr = Persistence.getInstance();
        private FolderGridView fView_;

        /** array of OBs to be checked-in */
        private CalibrationBlock[] bos_;

        /** members used for Verification. */
        protected StringBuffer msgLog_ = null;
        protected StringBuffer passedMsg_ = null;
        protected StringBuffer reportHeader_ = null;
        protected static TextDisplayWidget reportLog_ = null;
        protected boolean performEvmsOnly_ = true;
        protected int numErrs_ = 0;
        protected int numGood_ = 0;
        private ObjectIOException lastObjIOException_ = null;

        /**
         * A list of finding charts in a given set of OB's
         * which are already checked in . So no need to
         * send to the AppServer again
         */

        private Hashtable<Long, Long> isCheckedIn = new Hashtable<Long, Long>();

        //private Hashtable alreadyCheckedIn = new Hashtable

        public IncrementalCheckIn(
            CalibrationBlock[] cbs,
            FolderGridView fv,
            boolean performEvmsOnly) {
            bos_ = cbs;
            fView_ = fv;
            performEvmsOnly_ = performEvmsOnly;
        }

        public int getNumStepsRemaining(int counter) {
            return ObjUtils.howManyNotNull(bos_, counter);
        }

        public void startOfAction() {
            msgLog_ = new StringBuffer();
            passedMsg_ = new StringBuffer();
            numErrs_ = 0;
        }

        public void endOfAction() {
            if (numErrs_ > 0) {
                showFinalMessage();
            } else if (performEvmsOnly_ && numGood_ > 0) {
                JOptionPane.showMessageDialog(
                    fView_.getTopLevelAncestor(),
                    new Object[] {
                        "Verification Passed for the following OB(s): ",
                        passedMsg_.toString()},
                    "External Verification",
                    JOptionPane.INFORMATION_MESSAGE);
            }
        }

        /**
         * Method to check-in the n'th element of cbs, where n = counter.
         * 
         * @param counter   counter to show which element of cbs to check in.
         * @return	    true if the checking-in should continue
         *		    and false otherwise.
         */
        public void performStep(int counter) {
            if (bos_[counter] != null) {
                // Main body of the "loop" + fall thru
                try {
                    if (performEvmsOnly_) {
                        stdlog_.debug("Check EVMs on appserver.");
                        objMgr.verifyObject(bos_[counter]);
                        //No exception thrown must be a good OB
                        numGood_++;
                        passedMsg_.append("\n     " + bos_[counter].getName());

                    } else {
                        stdlog_.debug("Check-in action");

                        if (EvmExecutorImpl.evmConfigured()) {
                            EvmExecutorImpl evm = EvmExecutorImpl.getInstance();
                            //Check that the evm is initialized
                            if (evm.isInitialized()) {
                                try {
                                    evm.calculateETCS(bos_[counter]);
                                } catch (EVMException e) {
                                    bos_[counter].setExecutionTime(0);
                                    // If no exec time calcilation the OB can 
                                    // still be checkedin...
                                }
                            }
                        }
                        objMgr.checkIn(bos_[counter], isCheckedIn);
                        if (bos_[counter] instanceof ObservationBlock) {
                            ObservationBlock ob =
                                (ObservationBlock) bos_[counter];
                            FindingChart[] fcCharts = ob.getFindingCharts();
                            for (int i = 0; i < fcCharts.length; i++) {
                                isCheckedIn.put(
                                    new Long(fcCharts[i].getCheckSum()),
                                    new Long(fcCharts[i].getCheckSum()));
                            }
                        }
                        bos_[counter].setExecTimeUpToDate(true);

                    }
                } catch (ObjectIOException e) {
                    numErrs_++;
                    lastObjIOException_ = e;
                    msgLog_.append(lastObjIOException_.getMessage());
                } catch (IOException e) {
                    numErrs_++;
                } catch (ObjectNotFoundException e) {
                    //TODO
                }
            }
        }

        /**
         * show a dialog box with information
         */

        protected void showFinalMessage() {
            if (JOptionPane.YES_OPTION
                == JOptionPane.showConfirmDialog(
                    fView_.getTopLevelAncestor(),
                    new Object[] {
                        ""
                            + numErrs_
                            + " error(s) were detected during verification,",
                        "objects that fail verification cannot be checked-in,",
                        "\nWould you like to view the verification log?" },
                    "Verification errors",
                    JOptionPane.YES_NO_OPTION,
                    JOptionPane.ERROR_MESSAGE)) {

                if (reportLog_ == null) {
                    reportLog_ =
                        new TextDisplayWidget("Verification Message Log");
                }
                reportLog_.clearTextArea();
                reportHeader_ = new StringBuffer();
                reportHeader_.append(
                    "VERIFICATION MESSAGE LOG "
                        + TextUtils.currentISODateTime()
                        + "(UT)\n");
                reportHeader_.append(
                    "==================================================\n");
                reportLog_.setTextArea(reportHeader_.toString());
                reportLog_.setTextArea(msgLog_.toString());
                if (numGood_ > 0) {
                    reportLog_.setTextArea(
                        "\n............................................");
                    reportLog_.setTextArea(
                        "\nVerification passed for the following OB(s):");
                    reportLog_.setTextArea(passedMsg_.toString());
                }
                reportLog_.setSize(new Dimension(700, 300));
                reportLog_.setVisible(true);
            }
        }

    } // end of class IncrementalCheckIn.
}
