package org.eso.ohs.phase2.apps.p2pp.actions;

import java.awt.Component;
import java.awt.event.ActionEvent;

import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JTextField;

import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.utilities.ImageUtils;
import org.eso.ohs.core.gui.widgets.SimpleInputDialog;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.p2pp.Folder;
import org.eso.ohs.persistence.DirectoryNode;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.phase2.apps.p2pp.FolderFactory;
import org.eso.ohs.phase2.apps.p2pp.HierarchyView;
import org.eso.ohs.phase2.apps.p2pp.Persistence;

/**
 * Action class to create a subdirectory of the selected directory.
 *
 * <p>
 * $Id: CreateDirectoryAction.java,v 1.12 2006/04/27 13:25:00 pnunes Exp $
 * <br>
 * @author Bob Kemp
 */
public class CreateDirectoryAction extends ActionSuperclass {
    
    
    private static Logger stdlog_ =
        Logger.getLogger(CreateDirectoryAction.class);   

    /** medium on which the directory is to be created */
    Media device_;

    /** Window on which to base any dialog boxes, eg for errors */
    HierarchyView hView_;

    private JFrame parentFrame_;

    /**
     * Constructs a create directory action.
     * 
     * @param device   medium containing the parent directory.
     * @param path     path in HierarchyView tree to the selected directory.
     */
    public CreateDirectoryAction(
        JFrame parentFrame,
        Media device,
        HierarchyView hView) {
        super(hView, "New");

        device_ = device;
        hView_ = hView;
        parentFrame_ = parentFrame;
    }

    /**
     * Event handler for the create directory action, eg from a menu item.
     * 
     * @param ev       user-initiated ActionEvent to create a directory.
     */
    public void actionPerformedImpl(ActionEvent ev) {
        Persistence objMgr = Persistence.getInstance();
        Folder folder;

        try {
            folder = hView_.getSelectedFolder();

            if (folder == null) {
                JOptionPane.showMessageDialog(
                    parentFrame_,
                    new Object[] {
                        "There is no selected folder.",
                        "Where should the new folder be created?",
                        },
                    "Nothing selected",
                    JOptionPane.ERROR_MESSAGE);
                return;
            }

            DirectoryNode parent = folder.getNode();
            long id = objMgr.createDirectory(device_, parent);
            DirectoryNode child = parent.getDirectory(id);

            boolean renamed =
                createDirectoryViaDialog(
                    parentFrame_,
                    hView_,
                    FolderFactory.getFolder(device_, child));

            if (!renamed) {
                stdlog_.debug("Delete directory " + child);
                objMgr.deleteDirectory(device_, child);
            }
        } catch (ObjectIOException e) {
            announceIOError(e);
        }
    }

    public boolean createDirectoryViaDialog(
        JFrame parentFrame,
        JComponent comp,
        Folder folder) {
        Persistence objMgr = Persistence.getInstance();
        String oldName = folder.getNode().getDescription();
        JTextField newNameTF = new JTextField();

        newNameTF.setText(oldName);

        SimpleInputDialog dlg =
            new SimpleInputDialog(
                parentFrame_,
                "Enter new name",
                "Enter the new name for the folder: " + oldName,
                new String[] { "New name:" },
                new Component[] { newNameTF });

        dlg.setVisible(true);

        if (dlg.getUserChoice() == JOptionPane.OK_OPTION) {
            if (newNameTF.getText() != null
                && newNameTF.getText().equalsIgnoreCase(
                    "where did it come from")) {
                JOptionPane pane =
                    new JOptionPane(
                        "",
                        JOptionPane.INFORMATION_MESSAGE,
                        JOptionPane.DEFAULT_OPTION,
                        ImageUtils.getIcon("linked.gif"));
                JDialog dialog = pane.createDialog(parentFrame_, "Welcome");
                dialog.setVisible(true);
            }
            objMgr.renameDirectory(
                folder.getMedia(),
                folder.getNode(),
                newNameTF.getText());
            return true;
        } else {
            return false; // Cancelled or closed window?
        }
    }
}
