package org.eso.ohs.phase2.apps.p2pp.actions;

import java.awt.event.ActionEvent;

import javax.swing.JOptionPane;

import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.baseaction.IncrementalAction;
import org.eso.ohs.core.utilities.ObjUtils;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.BusinessObject;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.ConstraintSet;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.instruments.ParamFileParameter;
import org.eso.ohs.instruments.Parameter;
import org.eso.ohs.instruments.TemplateSignature;
import org.eso.ohs.p2pp.Folder;
import org.eso.ohs.persistence.DirectoryNode;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.phase2.apps.p2pp.FolderGridView;
import org.eso.ohs.phase2.apps.p2pp.MainWindow;
import org.eso.ohs.phase2.apps.p2pp.MenuGroups;
import org.eso.ohs.phase2.apps.p2pp.P2PP;
import org.eso.ohs.phase2.apps.p2pp.Persistence;
/**
 * Action class to make copies of the selected business objects.
 * 
 * <p>
 *
 * $Id: DuplicateBOAction.java,v 1.10 2006/04/27 13:25:00 pnunes Exp $
 *
 */

public class DuplicateBOAction extends ActionSuperclass {
    /** view of the folder in which the OB is to be created */
    FolderGridView folderView_;

    /**
     * Constructs a "copy BusinessObject" action.
     * 
     * @param folderView  GUI view of (summary information in) the folder.
     */
    public DuplicateBOAction(FolderGridView folderView) {
    super(folderView, "Duplicate");

    folderView_ = folderView;
    }

    /**
     * Event handler for the create BusinessObject action.
     * 
     * @param ev       user-initiated ActionEvent to create an OD.
     */
    public void actionPerformedImpl(ActionEvent ev) {
    Folder folder = folderView_.getFolder();
    Class objType = folderView_.getDisplayedObjectClass();
    Long[] idLongs = folderView_.getAllSelected();
    Object[] options = { "Yes", "No", "Cancel" };

    if (idLongs.length == 0) {
        JOptionPane.showMessageDialog(
        folderView_.getTopLevelAncestor(),
        new Object[] {
          "There are no selected observation blocks.",
          "Which objects should be duplicated?",
        },
        "Nothing selected",
        JOptionPane.ERROR_MESSAGE
        );
        return;		// nothing selected?
    }



    if (objType.equals(CalibrationBlock.class)
       || objType.equals(ObservationBlock.class)) {
        new P2PPIncrementalActionAutomaton(
            new IncrementalDuplication(
            folder.getMedia(),			
            folder.getNode(),
            objType,
            idLongs
            ),
            (MenuGroups)folderView_.getTopLevelAncestor(),
            "Duplicating..."
        );
            // Cancel button or closed window
    }
    }


    /**
     * Utility class to duplicate a sequence of OBs, one at a time.
     * The user is also given the opportunity to stop the process using
     * a dialog box.
     */
    public class IncrementalDuplication implements IncrementalAction {

       /** cached reference to Persistence */
       private Persistence objMgr_ = Persistence.getInstance();
   
       /** array of ids of objects to be duplicated */
       private Long[] idLongs_;
   
       /** Type of business object to be duplicated */
       private Class objType_;
   
       /** Media containing the objects to be duplicated */
       private Media device_;
   
       /** For ObsBlocks, should we duplicate attached ODs and Targets */
       private boolean deepCopy_;
   
       /** Directory containing the ObsBlocks */
       private DirectoryNode dir_;
   
       /** Did the copy succeed? */
       private boolean failedCopy_;
   
       /**
        * Constructor.
        * 
        * @param frm	    window on which to base any dialog boxes.
        * @param device    media containing the objects to be duplicated
        * @param deepCopy  should we duplicate attached objects?
        * @param dir	    directory containing the ObsBlocks
        * @param objType   type of object to be duplicated
        * @param idLongs   array of ids of objects to be duplicated
        */
       public IncrementalDuplication(Media device,
                       DirectoryNode dir, Class objType,
                       Long[] idLongs) {
           device_   = device;	    
           dir_      = dir;
           objType_  = objType;
           idLongs_  = idLongs;
       }
   
       public int getNumStepsRemaining(int counter) {
           return ObjUtils.howManyNotNull(idLongs_, counter);
       }
   
       public void startOfAction() { }
   
       public void endOfAction() { }
   
       /**
        * Method to duplicate the n'th element of obs, where n = counter.
        * 
        * @param counter   counter to show which element of obs to duplicate.
        * @return	    true if the duplicating should continue
        *		    and false otherwise.
        */
       public void performStep(int counter) {
           if (idLongs_[counter] != null) {
                   // Main body of the "loop" + fall thru
           doDuplication(idLongs_[counter].longValue());
           }
       }
   
       /**
        * Utility method to duplicate a given business object
        * 
        * @param id	    id of business object to duplicate
        */
       public void doDuplication(long id) {
           BusinessObject bo;
   
           try {
               bo = (BusinessObject)objMgr_.getBusObj(
                           device_,
                           id,
                           objType_
                       );
   
               failedCopy_ = false;	
               BusinessObject boCopy;
               Media device = objMgr_.getMediaFor(bo);
               DirectoryNode dir = objMgr_.getDirectoryFor(bo);	       
               boCopy = objMgr_.deepCopy(device,dir,bo);
               if(boCopy instanceof CalibrationBlock){
                   P2PP.makeNameUnique(boCopy);
               }
              /*
               * if this is an OB, then get a reference to the acquisition
               * template.
               * Get the list of paramteres associated with this acquisition
               * template and get the first ParamFile, if there is one at all.
               * Update the Target according to the ParamFile.
               */
               if(boCopy instanceof ObservationBlock){
                   ObservationBlock ob = (ObservationBlock) boCopy;
                   TemplateSignature tsig = null;
                   if((tsig = ob.getTemplateSignature()) != null){
                       Parameter plist[] = tsig.getParamList();
                       //ob_.getTarget().freeLocks();
                       for(int i = 0; i < plist.length; i++){
                           if(plist[i].getType().equals("paramfile")) {
                   
                               ParamFileParameter pfp = (ParamFileParameter) plist[i];
                               try {
                                   ob.getTarget().updateFromParamFileParameter( pfp );
                               }
                               catch( Exception e ) {
                           
                                   String name = pfp.getParameterName();
                                   String msg = 
                                   "Error while duplicating paramfile\n" +
                                   "attached to parameter " + name + 
                                   ":\n" + e.getMessage() + 
                                   "\nTarget info was not updated.";
                                   JOptionPane.
                                   showMessageDialog( MainWindow.getPrincipalWindow(),
                                              msg,
                                              "Error in paramfile",
                                              JOptionPane.ERROR_MESSAGE );
                               }
                                   
                               try {
                                   ConstraintSet cs = ob.getConstraintSet();
                                       if (cs != null) {
                                           cs.updateFromParamFileParameter( pfp );
                                       }
                               } catch( Exception e ) {
                                       
                                   String name = pfp.getParameterName();
                                   String msg = 
                                   "Error while duplicating paramfile\n" +
                                   "attached to parameter " + name + 
                                   ":\n" + e.getMessage() + 
                                   "\nConstraint Set info was not updated.";                                  
                                   System.out.println( msg );
                                   JOptionPane.
                                   showMessageDialog( MainWindow.getPrincipalWindow(),
                                              msg,
                                              "Error in paramfile",
                                              JOptionPane.ERROR_MESSAGE );
                               }
                               break;
                           }
                       }
                   }
                   //FindingChartSession.incFcCount(ob);
                   
               }
           }              
           catch (ObjectNotFoundException e) {             
               announceNoObject(e);
           }
	   catch (ObjectIOException e) {
               announceIOError(e);
           }
   
       }
    }
}

