package org.eso.ohs.phase2.apps.p2pp.actions;

import java.awt.event.ActionEvent;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.gui.widgets.OverwriteOptionPane;
import org.eso.ohs.core.gui.widgets.SwingFileChooser;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.dfs.Readme;
import org.eso.ohs.instruments.Instrument;
import org.eso.ohs.instruments.InstrumentList;
import org.eso.ohs.p2pp.Folder;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.persistence.ExportReadme;
import org.eso.ohs.phase2.apps.p2pp.HierarchyView;

/**
 * Action class to export a readme file.
 * 
 * <p>
 * $Id: ExportReadmeAction.java,v 1.14 2006/04/27 13:25:00 pnunes Exp $
 *  
 */

public class ExportReadmeAction extends ActionSuperclass {

   
    private HierarchyView hView_;

    private int returnValue_;
    

    private static SwingFileChooser fileChooser_ = new SwingFileChooser();
    static {
        File f1 = new File(AppConfig.getAppConfig().getExpDir());
        fileChooser_.setCurrentDirectory(new File(f1.getParent()));
        fileChooser_.setSelectedFile(f1);
    }

    /**
     * Constructs a create BusinessObject action.
     * 
     * @param device
     *            medium containing the parent directory.
     * @param folderView
     *            GUI view of (summary information in) the folder.
     */
    public ExportReadmeAction(HierarchyView hView) {
        super(hView, "Export Readme");
        hView_ = hView;
    }

    /**
     * Event handler for the create BusinessObject action.
     * 
     * @param ev
     *            user-initiated ActionEvent to create an OD.
     */
    public void actionPerformedImpl(ActionEvent ev) {
        ObservingRun obsRun=null;
        fileChooser_.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        int response = fileChooser_
                .showDialog(hView_, "Select Export Directory ");
        if (response == JOptionPane.CLOSED_OPTION
                || response == JOptionPane.CANCEL_OPTION)
            return;
        Folder folder;
        try {
            folder = hView_.getSelectedFolder();
            obsRun = folder.findObsRun();

        } catch (ObjectIOException e2) {
            ErrorMessages.announceIOError(hView_,e2);
        }
        if (response == JFileChooser.APPROVE_OPTION) {
            File f = fileChooser_.getSelectedFile();
            if (!f.canWrite()) {
                JOptionPane
                        .showMessageDialog(
                                null,
                                new Object[] { "No write permissions to export to this directory" },
                                "No write permisssions",
                                JOptionPane.ERROR_MESSAGE);
            } else {
                try {
                    if(checkExport(obsRun))
                        doExport(obsRun,f.getAbsolutePath());
                } catch(FileNotFoundException e) {
                    e.printStackTrace();
                    ErrorMessages.announceIOError(hView_,e);
                } catch(IOException e1) {
                    ErrorMessages.announceIOError(hView_,e1);
                }
            }
        }

    }

    /**
     * Utility method to export a readme file.
     * 
     * @param id
     *            id of business object to duplicate
     * @throws IOException
     * @throws FileNotFoundException
     */
    public void doExport(ObservingRun obsRun,String path) throws FileNotFoundException, IOException {
        Instrument inst =
            InstrumentList.getInstance().getInstrument(
                obsRun.getInstCode(),
                obsRun.getIPVersion());
        Readme readme = obsRun.getReadme();
        if (readme == null) {           
            readme = Readme.createReadme(inst);
            // fire event.
            obsRun.setReadme(readme);
        }
        String progId = obsRun.getProgId().trim();
        String replaceFirstBracket = progId.replace('(','_');
        String replaceSecondBracket = replaceFirstBracket.replace(')','_');
        
        File file = new File(path + File.separator + replaceSecondBracket  + "." + "rdx");
        if (file.exists()) {
            returnValue_ = JOptionPane.showConfirmDialog(
                    hView_.getTopLevelAncestor(),
                    "Do you wish to overwrite previously exported file " + replaceSecondBracket + ".rdx file"
                    );
        }
        if (!file.exists() || returnValue_ == OverwriteOptionPane.YES_OPTION) {
            ExportReadme exportRdm = new ExportReadme();
            exportRdm.export(obsRun,file);
        }   
    }
    
    public boolean checkExport(ObservingRun obsRun) {
        Instrument inst =
            InstrumentList.getInstance().getInstrument(
                obsRun.getInstCode(),
                obsRun.getIPVersion());
        if(!obsRun.getObsMode().equalsIgnoreCase(ObservingRun.OBSMODE_STANDARD)) {
            JOptionPane.showMessageDialog(
                    hView_,
                    new Object[] {
                        "The readme file option only applies to Service mode Observing Runs",
                        },
                    "Readme Available",
                    JOptionPane.ERROR_MESSAGE);
                return false;
            
        }
        if(!inst.hasReadmeData()) {
            JOptionPane.showMessageDialog(
                hView_,
                new Object[] {
                    "There is no readme configuration available for this observing run.",
                    "Try download refresh observing runs",
                    "If this still does not work then contact usd-help@eso.org",
                    },
                "Readme Available",
                JOptionPane.ERROR_MESSAGE);
            return false;
        }
        return true;
    }
}