package org.eso.ohs.phase2.apps.p2pp.actions;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.baseaction.IncrementalAction;
import org.eso.ohs.core.gui.widgets.ExtensionFileFilter;
import org.eso.ohs.core.gui.widgets.OverwriteOptionPane;
import org.eso.ohs.core.gui.widgets.SwingFileChooser;
import org.eso.ohs.core.gui.widgets.TextDisplayWidget;
import org.eso.ohs.core.utilities.ImportPafFile;
import org.eso.ohs.core.utilities.ObjUtils;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.PafFile;
import org.eso.ohs.core.utilities.PafFileSyntaxException;
import org.eso.ohs.core.utilities.TextUtils;
import org.eso.ohs.dfs.BusinessObject;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.instruments.InstrumentList;
import org.eso.ohs.p2pp.Folder;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.persistence.DirectoryNode;
import org.eso.ohs.persistence.Import;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectContainer;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.phase2.apps.p2pp.FolderGridView;
import org.eso.ohs.phase2.apps.p2pp.MenuGroups;
/**
 * Action class to create a business object in the chosen directory.
 * 
 * <p>
 * $Id: ImportBOAction.java,v 1.17 2006/04/27 13:25:00 pnunes Exp $
 * @author cmavros 05JUNE02 [1766/P2PP] Do not import 
 *        files with blank name fields.
 */

public class ImportBOAction extends ActionSuperclass {

    /** Standard log channel */
    private static Logger stdlog_ =
        Logger.getLogger(ImportBOAction.class);

    /** medium on which the business object is to be created */
    Media device_;

    /** view of the folder in which the OB is to be created */
    ObjectContainer folderView_;


    /**
     * A hashtable of strings that contain any waring information about the
     * import of an individual file
     *
     */

    private Hashtable<String, String> warningList_ = new Hashtable<String, String>();

    /**
     * The number of successfully imported files
     */

    private int successImports_ = 0;

    private int numLines = 0;

    private boolean truncateReport = false;

    /**
     * List of filenames that were not imported
     */

    private Vector<String> failedImports_ = new Vector<String>();

    /**
     * Status window which displays reports on imports.     
     */

    private static TextDisplayWidget log_ =
        new TextDisplayWidget("Import Report");

    /**  
     * Static reference to file chooser dialog
     */

    private static SwingFileChooser chooser =
        new SwingFileChooser(AppConfig.getAppConfig().getExpDir());

    /**
     * Static refrences to file chooser filters
     */

    private static ExtensionFileFilter obFilter =
        new ExtensionFileFilter("obx", "ObsBlock Import Files");

    static {
        File f1 = new File(AppConfig.getAppConfig().getExpDir());
        chooser.setCurrentDirectory(f1);
        chooser.setSelectedFile(f1);
    }

    /**
     * Constructs a create BusinessObject action.
     * 
     * @param device      medium containing the parent directory.
     * @param folderView  GUI view of (summary information in) the folder.
     */

    public ImportBOAction(Media device, ObjectContainer folderView) {
        super((JComponent) folderView, "Import");
        device_ = device;
        folderView_ = folderView;
    }

    /**
     * Event handler for the create BusinessObject action.
     * 
     * @param ev       user-initiated ActionEvent to create an OD.
     */

    public void actionPerformedImpl(ActionEvent ev) {
        Class type = ((FolderGridView) folderView_).getDisplayedObjectClass();
        Folder folder = ((FolderGridView) folderView_).getFolder();
        String ext = BusinessObject.getSuffix(type) + "x";

        chooser.addChoosableFileFilter(obFilter);

        if (ext.equals("obx"))
            chooser.setFileFilter(obFilter);

        // Java1.4 port chooser.clearSelection();      
        chooser.cancelSelection();
        chooser.setMultiSelectionEnabled(true);
        int response = chooser.showDialog(null, "Select Import files");
        if (response == JOptionPane.CLOSED_OPTION
            || response == JOptionPane.CANCEL_OPTION)
            return;

        File[] files = chooser.getSelectedFiles();

        if (response == JFileChooser.APPROVE_OPTION) {

            new P2PPIncrementalActionAutomaton(
                new IncrementalImport(
                    Media.LOCAL,
                    files,
                    chooser.getCurrentDirectory().getAbsolutePath(),
                    folder.getDirNode(),
                    folder.findObsRun()),
                (MenuGroups) folderView_.getTopLevelAncestor(),
                "Importing...");

        }

    }

    /**
     * Return the status buffer all the files that have been imported
     * @return status buffer for files that are imported.
     */

    public String getStatusBuffer() {
        StringBuffer buf = new StringBuffer();
        buf.append(
            "IMPORT MESSAGE LOG "
                + TextUtils.currentISODateTime()
                + "(UT)\n\n");
        buf.append("Successfully imported: " + successImports_ + "\n");
        buf.append("Failed imports: " + failedImports_.size() + "\n");
        buf.append("\n");
        for (int i = 0; i < failedImports_.size(); i++) {
            buf.append(failedImports_.elementAt(i));
        }
        Enumeration<String> enum0 = warningList_.keys();
        while (enum0.hasMoreElements()) {
            String filename = enum0.nextElement();
            buf.append(
                "\n\n"
                    + filename
                    + " was successfully imported but has the following warnings\n");
            String warning = warningList_.get(filename);
            buf.append(warning);
        }
        if (truncateReport) {
            buf.append("Report > 300 lines further messages supressed\n");
        }
        return buf.toString();
    }

    /**
     * Display the contents of the status buffer in the log windows
     */

    public void displayStatusBuffer() {
        String display = getStatusBuffer();
        log_.clearTextArea();
        if (display.length() > 0) {
            log_.setTextArea(display);
            log_.setSize(new Dimension(700, 300));
            log_.setVisible(true);
        }
    }

    /**
     * Utility class to export a sequence of Business Objects, one at a time.
     * The user is also given the opportunity to stop the process using
     * a dialog box.
     */

    public class IncrementalImport implements IncrementalAction {


        /** Directory containing the ObsBlocks */
        private String dir_;

        /** Directory node the business objects will end up in*/

        private DirectoryNode node_;

        /**
         * List of file handles to import
         */

        private File[] files_;

        /**
         * Observing run for files to be imported
         */

        private ObservingRun obsRun_;

        /**
         * Reference to import class
         */

        private Import import_;

        int returnValue_;

        /**
         * Constructor.
         * 
         * @param frm	    window on which to base any dialog boxes.
         * @param device    media containing the objects to be duplicated
         * @param dir	    directory containing the ObsBlocks
         * @param objType   type of object to be duplicated
         * @param idLongs   array of ids of objects to be duplicated
         */

        public IncrementalImport(
            Media device,
            File[] files,
            String dir,
            DirectoryNode node,
            ObservingRun obsRun) {

            device_ = device;
            files_ = files;
            dir_ = dir;
            node_ = node;
            obsRun_ = obsRun;

            //import_ = new Import();	       
        }

        public int getNumStepsRemaining(int counter) {
            return ObjUtils.howManyNotNull(files_, counter);
        }

        public void startOfAction() {
            failedImports_.removeAllElements();
            warningList_.clear();
            successImports_ = 0;
            numLines = 2;
            returnValue_ = -1;
            truncateReport = false;
        }

        public void endOfAction() {
            displayStatusBuffer();
        }

        /**
         * Method to duplicate the n'th element of obs, where n = counter.
         * 
         * @param counter   counter to show which element of obs to check in.
         * @return	    true if the duplicating should continue
         *		    and false otherwise.
         */
        public void performStep(int counter) {
            boolean imported = true;
            import_ = new Import();
            if (files_[counter] != null) {
                // Main body of the "loop" + fall thru
                try {
                    String filename = files_[counter].getAbsolutePath();
                    File f = new File(filename);
                    if (!f.canRead()) {
                        if (files_.length == 1) {
                            JOptionPane.showMessageDialog(
                                folderView_.getTopLevelAncestor(),
                                "You do not have read permissions on file "
                                    + f.getName(),
                                "Read Error",
                                JOptionPane.WARNING_MESSAGE);
                        }
                        failedImports_.addElement(
                            "\n\nImport failed : "
                                + f.getName()
                                + "\nNo read permissions\n");
                        numLines++;
                        return;
                    }

                    String msg = Import.checkLinks(dir_, filename);

                    if (filename.endsWith(".obx") && msg.length() > 0) {
                        if (files_.length == 1) {
                            JOptionPane.showMessageDialog(
                                folderView_.getTopLevelAncestor(),
                                msg,
                                "File error",
                                JOptionPane.WARNING_MESSAGE);
                        }
                        failedImports_.addElement("\n\nImport failed : " + msg);
                        numLines++;
                        return;
                    }

                    if (filename.endsWith(".obx")
                        && !Import.isValidOBType(filename)) {
                        if (files_.length == 1) {
                            JOptionPane.showMessageDialog(
                                folderView_.getTopLevelAncestor(),
                                "Invalid <type> : value should equal C or O ",
                                "File error",
                                JOptionPane.WARNING_MESSAGE);
                        }
                        failedImports_.addElement(
                            "\n\nImport failed : "
                                + "Invalid <type> : value should equal C or O\n");
                        numLines++;
                        return;
                    }
                    if (!Import.checkImport(dir_, filename)) {
                        if (files_.length == 1) {
                            JOptionPane.showMessageDialog(
                                folderView_.getTopLevelAncestor(),
                                "This is not an export file",
                                "Invalid file format",
                                JOptionPane.WARNING_MESSAGE);
                        }
                        failedImports_.addElement(
                            "\n\nImport failed : "
                                + f.getName()
                                + "\nThis is not an export file\n");
                        numLines++;
                        return;
                    }
                    if (!InstrumentList
                        .getInstance()
                        .instrumentExists(obsRun_.getInstCode())) {
                        if (files_.length == 1) {
                            JOptionPane.showMessageDialog(
                                folderView_.getTopLevelAncestor(),
                                "Cannot import into an observing run which has "
                                    + "no associated instrument package installed ",
                                "Instrument package not installed",
                                JOptionPane.WARNING_MESSAGE);
                        }
                        failedImports_.addElement(
                            "\n\nImport failed : "
                                + f.getName()
                                + "\nInstrument package nor installed for"
                                + " selected observing run\n");
                        numLines++;
                        return;
                    }

                    if (!Import.checkInstrument(dir_, filename, obsRun_)) {
                        if (files_.length == 1) {
                            JOptionPane.showMessageDialog(
                                folderView_.getTopLevelAncestor(),
                                "Exported file refers to a different Instrument than the observing run",
                                "Invalid Instrunent Reference",
                                JOptionPane.WARNING_MESSAGE);
                        }
                        failedImports_.addElement(
                            "\n\nImport failed : "
                                + f.getName()
                                + "\nExported file refers to a different"
                                + " Instrument than the observing run\n");
                        numLines++;
                        return;
                    }

                    if (!import_.checkNames(dir_, filename, obsRun_)) {
                        if (files_.length == 1) {
                            JOptionPane.showMessageDialog(
                                folderView_.getTopLevelAncestor(),
                                "Blank name fields detected.",
                                "Blank",
                                JOptionPane.WARNING_MESSAGE);
                        }
                        StringBuffer buf = new StringBuffer();
                        buf.append(import_.getNameMissing());
                        failedImports_.addElement(
                            "\n\nImport failed : "
                                + f.getName()
                                + "\nThe following fields are blank:"
                                + buf.toString()
                                + "\n");
                        numLines++;
                        return;
                    }

                    doImport(files_[counter]);

                    if (import_.hasWarning()) {
                        StringTokenizer warningToken =
                            new StringTokenizer(
                                import_.toWarningString(),
                                "\n");
                        numLines = numLines + warningToken.countTokens() + 1;
                        stdlog_.debug(
                            "CLASS ImportBOAction - performStep: Num lines "
                                + numLines);
                        if (numLines <= 300)
                            warningList_.put(
                                files_[counter].getName(),
                                import_.toWarningString());
                        else
                            truncateReport = true;
                    }

                } catch (FileNotFoundException e) {
                    announceIOError(e);
                } catch (IOException e) {
                    announceIOError(e);
                }
            } else {
                displayStatusBuffer();
            }
        }

        /**
         * Utility method to duplicate a given business object
         * 
         * @param id	    id of business object to duplicate
         */
        public void doImport(File f) {
            try {
                PafFile paf = new ImportPafFile(f.getAbsolutePath());
                stdlog_.debug(
                    "CLASS ImportBOAction - doImport: OB Name - trying to get name");
                String name = paf.readLine("name");
                stdlog_.debug(
                    "CLASS ImportBOAction - doImport: OB Name " + name);
                String type = Import.getOBType(f.getAbsolutePath());
                BusinessObject obj =
                    ((FolderGridView) folderView_).getBOByName(name,type);
                import_ = new Import();
                if (obj == null) {
                    successImports_++;
                    import_.importBO(dir_, f.getAbsolutePath(), node_, obsRun_);
                } else {
                    if (obj.isCheckedIn())
                        return;
                        
                    if (returnValue_
                        != OverwriteOptionPane.YES_TO_ALL_OPTION) {
                        returnValue_ =
                            OverwriteOptionPane.showOptionDialog(
                                (JFrame) folderView_.getTopLevelAncestor(),
                                "Do you wish to overwrite " + name + " ?",
                                "Overwrite");
                    }
                    if (returnValue_ == OverwriteOptionPane.YES_OPTION
                        || returnValue_
                            == OverwriteOptionPane.YES_TO_ALL_OPTION) {
                        successImports_++;
                        if (obj instanceof ObservationBlock)
                            import_.overwriteOB(
                                dir_,
                                f.getAbsolutePath(),
                                node_,
                                obsRun_,
                                (ObservationBlock) obj);
                        else
                            import_.overwriteCB(
                                dir_,
                                f.getAbsolutePath(),
                                node_,
                                obsRun_,
                                (CalibrationBlock) obj);
                    }
                }
            } catch (ObjectIOException e) {
                announceIOError(e);
            } catch (NoSuchMethodException e) {
                stdlog_.error(e);
            } catch (IllegalAccessException e) {
                stdlog_.error(e);
            } catch (InvocationTargetException e) {
                stdlog_.error(e);
            } catch (ObjectNotFoundException e) {
                stdlog_.error(e);
            } catch (IOException e) {
                announceIOError(e);
            } catch (PafFileSyntaxException e) {
                announceIOError(e);                
            }

        }
    }
}
