package org.eso.ohs.phase2.apps.p2pp.actions;

import java.awt.event.ActionEvent;
import java.io.File;
import java.io.IOException;

import javax.swing.JOptionPane;

import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.baseaction.IncrementalAction;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.gui.widgets.ExtensionFileFilter;
import org.eso.ohs.core.gui.widgets.SwingFileChooser;
import org.eso.ohs.core.gui.widgets.TextDisplayWidget;
import org.eso.ohs.core.utilities.ObjUtils;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.p2pp.Folder;
import org.eso.ohs.persistence.DirectoryNode;
import org.eso.ohs.persistence.ImportSurvey;
import org.eso.ohs.persistence.ImportSurveyException;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.phase2.apps.p2pp.FolderGridView;
import org.eso.ohs.phase2.apps.p2pp.MenuGroups;
import org.eso.ohs.phase2.apps.p2pp.Persistence;

/**
 * Action to import a survey definition file from the SADT
 * 
 * <p>
 * 
 */

public class ImportSurveyAction extends ActionSuperclass {

	/** Standard log channel */
	private static Logger stdlog_ = Logger.getLogger(ImportSurveyAction.class);

	private FolderGridView folderView_;

	private ImportSurvey importer_;

	private static TextDisplayWidget log_ = new TextDisplayWidget(
			"Import Survey Report");

	/**
	 * Static reference to file chooser dialog
	 */

	private static SwingFileChooser chooser = new SwingFileChooser();

	/**
	 * Static refrences to file chooser filters
	 */

	private static ExtensionFileFilter xmlFilter = new ExtensionFileFilter(
			"xml", "Survey Definition Files");

	/**
	 * Constructs a ImportTargetLiust action
	 * 
	 * @param folderView
	 * 
	 * 
	 */

	public ImportSurveyAction(FolderGridView folderView) {
		super(folderView, "Import Survey definition file");
		folderView_ = folderView;
		importer_ = new ImportSurvey();
	}

	/**
	 * Event handler import a list of targets from a catalog.
	 * 
	 * @param ev
	 *            user-initiated ActionEvent to create an OD.
	 */

	public void actionPerformedImpl(ActionEvent ev) {
		Class type = folderView_.getDisplayedObjectClass();
		Folder folder = folderView_.getFolder();

		Long[] idLongs = folderView_.getAllSelected();

		if (idLongs == null || idLongs.length == 0) {
			JOptionPane.showMessageDialog(folderView_.getTopLevelAncestor(),
					new Object[] { "There are no selected observation blocks.",
							"Which observation blocks should be updated?", },
					"Nothing selected", JOptionPane.ERROR_MESSAGE);
			return; // nothing selected?
		}

		if (idLongs.length > 1) {
			String msg = "Please note that only the first selected observation block will be\n"
					+ "used to create new OB from the target list file.\n\n"
					+ "Are you sure you want to continue?\n";

			int option = JOptionPane.showConfirmDialog(folderView_.getTopLevelAncestor(), new Object[] {
					"There are " + idLongs.length
							+ " observation blocks selected.", msg },
					"Continue with Import?", JOptionPane.OK_CANCEL_OPTION);
			switch (option) {
			case JOptionPane.CLOSED_OPTION:
				return;
			case JOptionPane.CANCEL_OPTION:
				return;
			}
		}
		
	    Persistence objMgr = Persistence.getInstance();

	    ObservationBlock ob=null;
		try {
			ob = (ObservationBlock)objMgr.getBusObj(
						Media.LOCAL,
						idLongs[0].longValue(),
						ObservationBlock.class
					    );
		} catch (ObjectNotFoundException e1) {
			ErrorMessages.announceNoObject(getWdwComponent(),e1);
			return;
		} catch (ObjectIOException e1) {
			announceIOError(e1);
			return;
		}
	    if (objMgr.isAllowedToModify(ob)) {
			chooser.cancelSelection();
			chooser.setMultiSelectionEnabled(false);
			int response = chooser.showDialog(null, "Select Survey definition Files");
			if (response == JOptionPane.CLOSED_OPTION
					|| response == JOptionPane.CANCEL_OPTION)
				return;

			File f = chooser.getSelectedFile();
			// Parse file and create a list of targets. Assign
			// this to the targets variable below and and pass it to
			// Incremental action.

			importer_= new ImportSurvey();

			ObservationBlock [] blocks = new ObservationBlock[0];
			try {
				blocks = importer_.importSADTFile(f,ob);
				new P2PPIncrementalActionAutomaton(new IncrementalImport(blocks,
						ob,
						folder.getDirNode(), folder.findObsRun()),
						(MenuGroups) folderView_.getTopLevelAncestor(),
						"Importing...");


			} catch (IOException e) {

				JOptionPane.showMessageDialog(folderView_.getTopLevelAncestor(),
						new Object[] { e.getMessage() }, "IO File error",
						JOptionPane.ERROR_MESSAGE);

				e.printStackTrace();
			} catch (ImportSurveyException e) {
				JOptionPane.showMessageDialog(folderView_.getTopLevelAncestor(),
						new Object[] { e.toString() }, "Import error",
						JOptionPane.ERROR_MESSAGE);
				e.printStackTrace();
			}
	    } else {
	    	 JOptionPane.showMessageDialog(
	    			 folderView_.getTopLevelAncestor(),
	 			    new Object[] {
	 			      "The observation block is locked",
	 			      "and may not be updated with target data."
	 			    },
	 			    "OB is locked",
	 			    JOptionPane.ERROR_MESSAGE
	 		        );
	 		        return;
	    }
         

	}

	/**
	 * Utility class to impoer and create a target list
	 */

	public class IncrementalImport implements IncrementalAction {

		/** Directory node the business objects will end up in */

		private DirectoryNode node_;

		/**
		 * List of OB to import
		 */

		private ObservationBlock[] obs_;
		
        private ObservationBlock parentOB_;
		 
		
		/**
		 * Observing ffr the new ob
		 */

		private ObservingRun obsRun_;

		/**
		 * Constructor
		 * 
		 * @param targets
		 * @param node
		 * @param obsRun
		 */
		public IncrementalImport(ObservationBlock [] obs,ObservationBlock ob, DirectoryNode node,
				ObservingRun obsRun) {
			obs_ = obs;
			node_ = node;
			obsRun_ = obsRun;
			parentOB_ = ob;
		}

		public int getNumStepsRemaining(int counter) {
			return ObjUtils.howManyNotNull(obs_, counter);
		}

		public void startOfAction() {
		}

		public void endOfAction() {
		}

		/**
		 * Method to create and save a new OB 
		 * 
		 * @param counter
		 *            counter to show which element of obs to check in.
		 * @return true if the duplicating should continue and false otherwise.
		 */
		public void performStep(int counter) {
			if (obs_[counter] != null) {
				// Main body of the "loop" + fall thru
				Persistence objMgr = Persistence.getInstance();
				try {
					
					if (counter == 0) {
						
						
						importer_.copyImportedValues( obs_[counter],  parentOB_ );
						
						
					} else {
						if (counter > 0) {
							ObservationBlock ob = (ObservationBlock)objMgr.deepCopy(
									Media.LOCAL,
									node_,
									obs_[counter]
							);
							
							importer_.copyImportedValues( obs_[counter],  ob );
						}
					}


				} catch (ObjectIOException e) {
					announceIOError(e);
				}

			}
		}

	}
}
