package org.eso.ohs.phase2.apps.p2pp.actions;

import java.awt.event.ActionEvent;

import javax.swing.JFrame;
import javax.swing.JOptionPane;

import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.p2pp.Folder;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.phase2.apps.p2pp.HierarchyView;
import org.eso.ohs.phase2.apps.p2pp.MainWindow;
/**
 * Action class to open a folder view of the selected directory.
 *
 * <p>
 * $Id: OpenFolderAction.java,v 1.7 2006/04/27 13:25:00 pnunes Exp $
 * <br>
 * @author Bob Kemp
 * CHANGELOG cmavros 25JUNE02 Rename from 'Open' to 'Open New Window'
 */

public class OpenFolderAction extends ActionSuperclass {
    /** medium on which the directory is to be created */
    Media device_;

    HierarchyView hView_;


    private JFrame parentFrame_;

    /**
     * Constructs an open directory action.
     * 
     * @param device   medium containing the directory.
     * @param path     path in HierarchyView tree to the selected directory.
     */
    public OpenFolderAction(JFrame parentFrame,Media device, HierarchyView hView) {
	super(hView, "Open New Window");

	device_ = device;
	hView_  = hView;
	parentFrame_ = parentFrame;
    }

    /**
     * Event handler for the open directory action, eg from a menu item.
     * 
     * @param ev       user-initiated ActionEvent to view a directory.
     */
    public void actionPerformedImpl(ActionEvent ev) {
	HierarchyView newHView;
	MainWindow mwin;
	Folder folder;

	try {
	    mwin = new MainWindow(device_);
	    newHView = mwin.getHierarchyView();

	    folder = hView_.getSelectedFolder();
    
	    if (folder == null) {
		JOptionPane.showMessageDialog(
		    parentFrame_,
		    new Object[] {
		      "There is no selected folder.",
		      "Which folder should be displayed?",
		    },
		    "Nothing selected",
		    JOptionPane.ERROR_MESSAGE
		);
		return;
	    }

	    newHView.setSelectedFolder(folder);
	    mwin.setVisible(true);
	}
	catch (ObjectIOException e) {
	    announceIOError(e);
	}
    }

}

