package org.eso.ohs.phase2.apps.p2pp.actions;

import java.awt.event.ActionEvent;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.SwingUtilities;

import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectContainer;
import org.eso.ohs.persistence.StorageManager;
import org.eso.ohs.phase2.apps.p2pp.P2PP;
import org.eso.ohs.phase2.apps.p2pp.Persistence;

/**
 * Action class to display the detailed contents of a BusinessObject.
 *
 * <p>
 * $Id: ReloginAction.java,v 1.8.6.1 2007/10/05 11:15:53 pnunes Exp $
 * <br>
 * @author Bob Kemp
 */

public class ReloginAction extends ActionSuperclass {

	private ObjectContainer folderView_;

	/**
	 * Constructs a ReloginAction action.
	 */
	public ReloginAction(ObjectContainer folderView) {
		super((JComponent) folderView, "Login as ...");

		folderView_ = folderView;
	}

	/**
	 * Event handler for the ReloginAction action.
	 * 
	 * @param ev       user-initiated ActionEvent to quit.
	 */
	public void actionPerformedImpl(ActionEvent evt) {
		Persistence objMgr = Persistence.getInstance();
		StorageManager dbaseMgr = objMgr.getStorageManager(Media.DBASE);

		try {
			int oldUserId = Config.getCfg().getAccountId();
			final int status =
				P2PP.login(
					(JFrame) folderView_.getTopLevelAncestor(),
					dbaseMgr);

			if (status >= 0) {
				SwingUtilities.invokeLater(new RestartWindows(oldUserId, status));
			}
		} catch (ObjectIOException e) {
			ErrorMessages.announceIOError((JComponent) folderView_, e);
		}
	}
}
