package org.eso.ohs.phase2.apps.p2pp.views;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.sql.SQLException;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.TableModel;

import org.apache.log4j.Logger;
import org.eso.ohs.core.docview.gui.DocView;
import org.eso.ohs.core.gui.models.BoNameModel;
import org.eso.ohs.core.gui.models.LimitedCharsModel;
import org.eso.ohs.core.gui.widgets.ErrorMessages;
import org.eso.ohs.core.gui.widgets.ToolTipJList;
import org.eso.ohs.core.utilities.OHSConfig;
import org.eso.ohs.core.utilities.OHSKeys;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.BusinessObject;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.ObservationDescription;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.evm.EVMException;
import org.eso.ohs.evm.EvmExecutorImpl;
import org.eso.ohs.instruments.Instrument;
import org.eso.ohs.instruments.InstrumentList;
import org.eso.ohs.instruments.TemplateSignature;
import org.eso.ohs.instruments.TemplateSignatureFactory;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.OHSApplication;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.dbase.phase2.OBHistory;
import org.eso.ohs.phase2.apps.p2pp.Persistence;

/**
 * This is a specialised JPanel with subcomponents linked to the
 * given CalibrationBlock object's fields.  When this component is
 * displayed, changes to it will be reflected in the CalibrationBlock
 * object.  Likewise changes to the CalibrationBlock object will be
 * reflected in the displayed fields.  Some fields of an CalibrationBlock
 * may not be visible.  Some fields may not be modifiable.
 *
 * <p>
 *
 * $Id: CBView.java,v 1.26.4.2 2007/10/05 11:16:11 pnunes Exp $
 * <br>ChangeLog: cmavros added 17-10-2001
 * Add Textfield for Execution Time
 * <br>ChangeLog: cmavros added 22-10-01
 * Add method and calls method to calculate Execution Time
 * Add Value Change calculates the Execution Time
 * when the user changes a value.
 * Add method and calls method to calculate Execution Time
 * <br>ChangeLog: cmavros added 20-11-01
 * move Execution Time calculation to utilities XTime
 *
 */

public class CBView extends ObjectView implements PropertyChangeListener {

    private static String[] userPriority_ =
        { "1", "2", "3", "4", "5", "6", "7", "8", "9", "10" };

    public static JTextField executionTextField_;

    /** Standard log channel */
    private static Logger stdlog_ = Logger.getLogger(CBView.class);

    /** observation block currently displayed, or null */
    CalibrationBlock currentCB_;

    /* GUI component for the Name field */
    private DocView fieldName_;

    /* GUI component for the UserPriority field */
    private DocView fieldUserPriority_;

    /* GUI component for the Status field */
    private DocView fieldStatus_;

    /* GUI component for the UserComments field */
    private DocView fieldUserComments_;

    /* Button to allow the acquisition template to be edited */
    private DocView editTemplates_;

    /** Table that contains TemplateModel for CBView*/
    private CBTemplateTable templateTable_;

    /** TemplateModel for CBView*/
    private ODGridModel templateModel_;

    /* GUI component for the OD name field */
    private DocView fieldODName_;

    /* GUI component for the Execution Time field */
    /* cmavros added 16-10-2001 */
    private DocView fieldExecutionTime_;

    /* GUI component for the OD name field */
    private ObservationDescription currentOD_;

    /** Label for name of Observation Block */
    private JLabel nameLabel_;

    private JLabel execTimeLabel_ = new JLabel("Execution Time");

    /** Reference to the currently selected instrument*/
    private Instrument inst_;

    /** Button to allow the user to update the object on external media */

    private JList instrumentTypes_ = new JList();

    private ToolTipJList templateNames_;

    private JButton duplicateTemplate_ = new JButton("Duplicate");

    private JButton removeTemplate_ = new JButton("Remove");

    private JButton addTemplate_ = new JButton("Add");

    private JButton execTimeButton_ = new JButton("Recalc ExecTime");

    private JButton save_ = new JButton("Save");

    private void setBorder(JPanel panel, Color color) {
        //        panel.setBorder(BorderFactory.createLineBorder(color));
    }

    /**
     * Construct the JPanel and add the subcomponents and listeners, etc.
     * Some fields of a CalibrationBlock may not be visible.
     * Some fields may not be modifiable.
     *
     * <p> Initially, no CB is displayed.
     */
    public CBView() {
        DocView dView;
        JCheckBox ckBox;
        JTextField textf;
        JTextArea texta;
        JLabel lab;

        setLayout(new BorderLayout());

        GridLayout outerGrid = new GridLayout(1, 0);
        JPanel outerPanel = new JPanel(outerGrid);
        add(outerPanel, BorderLayout.NORTH);

        JPanel col1 = new JPanel(new GridLayout(0, 1));
        JPanel col2 = new JPanel(new BorderLayout());
        outerPanel.add(col1);
        outerPanel.add(col2);
        JPanel outerNmStatus = new JPanel(new BorderLayout());
        col1.add(outerNmStatus);
        JPanel leftCol1 = new JPanel();
        JPanel rightCol1 = new JPanel();
        leftCol1.setLayout(new GridLayout(0, 1));
        rightCol1.setLayout(new GridLayout(0, 1));

        JPanel leftCol2 = new JPanel();
        JPanel rightCol2 = new JPanel();
        leftCol2.setLayout(new BorderLayout());
        rightCol2.setLayout(new BorderLayout());
        col2.add(leftCol2, BorderLayout.WEST);
        col2.add(rightCol2, BorderLayout.CENTER);
        outerNmStatus.add(leftCol1, BorderLayout.WEST);
        outerNmStatus.add(rightCol1, BorderLayout.CENTER);

        //panel.add(lockedLabel_, gbc);

        nameLabel_ = new JLabel("Name: ");
        nameLabel_.setHorizontalAlignment(JLabel.RIGHT);
        leftCol1.add(nameLabel_);

        textf = new JTextField(Config.getCfg().getMaxNameLength());
        textf.setDocument(new BoNameModel());
        dView = new DocView(textf);
        rightCol1.add(dView);
        fieldName_ = dView;

        lab = new JLabel("Status: ");
        lab.setHorizontalAlignment(JLabel.RIGHT);
        leftCol1.add(lab);
        textf = new JTextField();
        textf.setEditable(false);
        textf.setOpaque(false);
        dView = new DocView(textf);
        rightCol1.add(dView);
        fieldStatus_ = dView;

        /* Execution Time */
        /* cmavros added 17-10-2001 */
        //		lab = new JLabel(" Execution Time: ");
        execTimeLabel_.setHorizontalAlignment(JLabel.RIGHT);
        leftCol1.add(execTimeLabel_);
        executionTextField_ = new JTextField();
        executionTextField_.setEditable(false);
        executionTextField_.setOpaque(false);
        fieldExecutionTime_ = new DocView(executionTextField_);
        rightCol1.add(fieldExecutionTime_);

        lab = new JLabel("User Priority: ");
        lab.setHorizontalAlignment(JLabel.RIGHT);
        leftCol1.add(lab);
        fieldUserPriority_ = new DocView(new JComboBox(userPriority_));
        rightCol1.add(fieldUserPriority_);

        lab = new JLabel("OD Name: ");
        lab.setHorizontalAlignment(JLabel.RIGHT);
        leftCol1.add(lab);
        textf = new JTextField(Config.getCfg().getMaxNameLength());
        textf.setDocument(new BoNameModel());
        fieldODName_ = new DocView(textf);
        rightCol1.add(fieldODName_);

        JPanel instTitlePanel = new JPanel(new GridLayout(0, 1));
        lab = new JLabel("Template Type");
        instTitlePanel.add(lab);
        leftCol2.add(instTitlePanel, BorderLayout.NORTH);
        setBorder(leftCol2, Color.blue);
        setBorder(rightCol2, Color.white);
        //topCol2.add(lab);
        //templateNames_ = new ToolTipJList();
        // set a fixed prototypical size of list cell.
        instrumentTypes_.setPrototypeCellValue("mmmmMmmmmMmmmmM");

        final JScrollPane instScrollPane = new JScrollPane(instrumentTypes_);

        leftCol2.add(instScrollPane, BorderLayout.CENTER);

        JPanel titlePanel = new JPanel(new GridLayout(0, 1));
        lab = new JLabel("Template");
        titlePanel.add(lab);
        rightCol2.add(titlePanel, BorderLayout.NORTH);

        //topCol2.add(lab);
        templateNames_ = new ToolTipJList();
        // set a fixed prototypical size of list cell.
        templateNames_.setPrototypeCellValue("mmmmMmmmmMmmmmM");

        final JScrollPane nameScrollPane = new JScrollPane(templateNames_);

        rightCol2.add(nameScrollPane, BorderLayout.CENTER);

        JPanel commentsBorder = new JPanel(new BorderLayout());

        lab = new JLabel("User Comments: ");
        lab.setHorizontalAlignment(JLabel.LEFT);
        commentsBorder.add(lab, BorderLayout.NORTH);

        texta = new JTextArea();
        texta.setDocument(new LimitedCharsModel());
        fieldUserComments_ = new DocView(texta);
        commentsBorder.add(
            new JScrollPane(fieldUserComments_),
            BorderLayout.CENTER);

        col1.add(commentsBorder, BorderLayout.CENTER);

        //if(Config.isInOperationsMode()) {
        Box buttonPanel = Box.createVerticalBox();
        buttonPanel.add(addTemplate_);
        buttonPanel.add(removeTemplate_);
        buttonPanel.add(duplicateTemplate_);
        buttonPanel.add(execTimeButton_);
        buttonPanel.add(save_);
        buttonPanel.add(Box.createVerticalGlue());
        col2.add(buttonPanel, BorderLayout.EAST);
        //}

        templateTable_ = new CBTemplateTable();
        templateTable_.setIsInScrollPane(true);
        templateTable_.setPreferredScrollableViewportSize(
            new Dimension(100, 100));
        add(new JScrollPane(templateTable_));

        save_.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                Persistence objMgr = Persistence.getInstance();
                try {
                    objMgr.putBusObj(currentOD_);
                    objMgr.putBusObj(currentCB_);
                    try {
                        OBHistory.addEvent(
                            Config.getCfg().uniqueToTableId(currentCB_.getId()),
                            null,
                            OHSApplication.getShortDesc(),
                            new Integer(Config.getCfg().getAccountId())
                                .toString(),
                            CalibrationBlock.EDITED_BY_USG);
                    } catch (SQLException e) {
                        ErrorMessages.announceIOError(templateTable_, e);
                    }

                } catch (ObjectIOException e) {
                    ErrorMessages.announceIOError(templateTable_, e);
                } catch (ObjectNotFoundException e) {
                    ErrorMessages.announceIOError(templateTable_, e);
                }

            }
        });

        addActionListeners();
        addListListeners();
        setObject(null);
    }

    private void initLists() {
        ObservingRun obr = currentCB_.getObsRun();
        if (obr == null)
            return;
        inst_ =
            InstrumentList.getInstance().getInstrument(
                obr.getInstCode(),
                obr.getIPVersion());
        instrumentTypes_.setListData(Instrument.instrumentTypes);

        instrumentTypes_.setSelectedIndex(0);
        if (inst_ != null) {
            templateNames_.setListData(
                inst_.getTemplateNames(
                    (String) instrumentTypes_.getSelectedValue()));
            templateNames_.setCellRenderer(new TemplateListCellRenderer(inst_));
        }
    }

    private void addTemplate() {
        try {
            Object templateName = templateNames_.getSelectedValue();
            TemplateSignature ts =
                TemplateSignatureFactory.getTemplate(
                    inst_,
                    (String) templateName);
            int column = 0;
            if (templateModel_.getColumnCount() != 0) {
                templateTable_.stopCellEditing();
                column = templateTable_.getSelectedColumn();
            }
            templateModel_.insertColumn(column, ts);
            if (templateModel_.getTemplateSignatures() != null)
                currentOD_.setSignatures(
                    templateModel_.getTemplateSignatures());
            templateTable_.setRowColumn(0, templateModel_.getCurrColumn());
            templateTable_.requestFocus();
            removeTemplate_.setEnabled(true);
            duplicateTemplate_.setEnabled(true);
        } catch (IOException e) {
            ErrorMessages.announceIOError(templateTable_, e);
        }
    }

    private void addActionListeners() {
        //if(Config.isInOperationsMode()) {
        save_.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                ObjectManager objMgr = ObjectManager.getObjectManager();
                try {
                    objMgr.putBusObj(currentOD_);
                    objMgr.putBusObj(currentCB_);
                } catch (ObjectIOException e) {
                } catch (ObjectNotFoundException e) {
                }

            }
        });
        //}

        addTemplate_.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                addTemplate();
                //templateTable_.setRowColumn(0,templateModel_.getCurrColumn());
            }
        });

        removeTemplate_.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                //try{
                templateTable_.stopCellEditing();
                int column = templateTable_.getSelectedColumn();
                TemplateSignature[] array;
                templateModel_.removeColumn(column);
                if (templateModel_.getColumnCount() > 0) {
                    array = templateModel_.getTemplateSignatures();
                } else {
                    array = new TemplateSignature[0];
                }
                if (array != null)
                    currentOD_.setSignatures(array);
                if (templateModel_.getCurrColumn() == -1) {
                    removeTemplate_.setText("Delete");
                    duplicateTemplate_.setText("Duplicate");
                    removeTemplate_.setEnabled(false);
                    duplicateTemplate_.setEnabled(false);
                } else {
                    templateTable_.setRowColumn(
                        0,
                        templateModel_.getCurrColumn());
                    templateTable_.requestFocus();
                }
                // }
                //catch(java.lang.ClassCastException e) {
                //     e.printStackTrace();
                // }
            }
        });

        duplicateTemplate_.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                if (templateModel_.getColumnCount() != 0) {
                    templateTable_.stopCellEditing();
                    int column = templateTable_.getSelectedColumn();
                    Object item = templateModel_.getDataItem(column);
                    TemplateSignature ts = null;
                    if (item instanceof TemplateSignature) {
                        if (!((TemplateSignature) item)
                            .getTemplateType()
                            .equalsIgnoreCase("acquisition")) {
                            ts =
                                new TemplateSignature((TemplateSignature) item);
                            templateModel_.insertColumn(column, ts);
                            currentOD_.setSignatures(
                                templateModel_.getTemplateSignatures());
                            templateTable_.setRowColumn(
                                0,
                                templateModel_.getCurrColumn());
                            templateTable_.requestFocus();
                        }
                    }
                }
            }
        });

        execTimeButton_.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                try {
                    if (EvmExecutorImpl.evmConfigured()) {
                        EvmExecutorImpl evm = EvmExecutorImpl.getInstance();
                        //Check that the evm is initialized
                        if (evm.isInitialized()) {
                            try {
                                if (!inst_.hasExecTimeCalc()) {
                                    displayNotFoundError();
                                    return;
                                }
                                evm.calculateETCS(currentCB_);
                                currentCB_.setExecTimeUpToDate(true);
                                AcquisitionModel model =
                                    (AcquisitionModel) templateTable_
                                        .getModel();
                                model.fireTableDataChanged();

                            } catch (EVMException ex) {
                                displayExecError();
                            }

                        }
                    }
                } catch (Exception e) {
                }
            }
        });

    }

    private void displayExecError() {
        JOptionPane.showMessageDialog(
            this,
            "Exception in Exectime script",
            "Exception in Exectime script",
            JOptionPane.ERROR_MESSAGE);
    }

    private void displayNotFoundError() {
        JOptionPane.showMessageDialog(
            this,
            "Exec Time Calculator not available for this instrument",
            "Execution Time Calculatar not found",
            JOptionPane.ERROR_MESSAGE);
    }

    private void addListListeners() {
        ListSelectionListener lsl = new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent ev) {
                if (ev.getValueIsAdjusting()) {
                    return;
                }
                String type = (String) instrumentTypes_.getSelectedValue();
                int index;
                if (ev.getSource() == instrumentTypes_) {
                    type = (String) instrumentTypes_.getSelectedValue();
                    addTemplate_.setEnabled(false);
                } else if (ev.getSource() == templateNames_) {
                    index = templateNames_.getSelectedIndex();
                    if (index >= 0)
                        addTemplate_.setEnabled(true);
                    else
                        addTemplate_.setEnabled(false);
                    if (type.equals("acquisition"))
                        addTemplate_.setToolTipText(
                            "Add "
                                + type
                                + " template to first column of grid");
                    else
                        addTemplate_.setToolTipText(
                            "Add " + type + " template to the grid");
                }
                if (type != null && ev.getSource() != templateNames_) {
                    if (inst_ == null)
                        return; // XXX
                    templateNames_.setListData(inst_.getTemplateNames(type));
                }
            }
        };
        instrumentTypes_.addListSelectionListener(lsl);
        templateNames_.addListSelectionListener(lsl);
        templateNames_.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent evt) {
                if (evt.getClickCount() == 2)
                    addTemplate();
            }
        });
    }

    /**
     * Construct the JPanel and add the subcomponents and listeners, etc.
     * Some fields of an CalibrationBlock may not be visible.
     * Some fields may not be modifiable.
     *
     * @param ob the CalibrationBlock object to link to.
     */
    public CBView(CalibrationBlock ob) {
        this();
        setObject(ob);
    }

    /**
     * Returns a reference to the currently displayed CB.
     *
     * @return reference to an CalibrationBlock.
     */
    public BusinessObject getObject() {
        return currentCB_;
    }

    /**
     * Sets a new value of the current CB.
     * This method is included to satisfy the ObjectView
     * interface, which in turn allows for generalisation of
     * other parts of the system.
     *
     * @param newValue reference to a BusinessObject.
     */
    public void setObject(BusinessObject newValue) {
        setObject((CalibrationBlock) newValue);
    }

    /**
     * Enable or disable the fields of this panel and any
     * nested panels.  The "locked" icon is displayed
     * next to the Name field if the fields are locked.
     *
     * @param enable	True if the fields are to be enabled, false otherwise.
     */
    public void setEnabled(boolean enable) {
        super.setEnabled(enable);

        nameLabel_.setIcon(enable ? null : locked_);

        fieldName_.setEnabled(enable);
        fieldUserComments_.setEnabled(enable);
        fieldUserPriority_.setEnabled(enable);
        templateNames_.setEnabled(enable);
        fieldODName_.setEnabled(enable);
        removeTemplate_.setEnabled(enable);
        duplicateTemplate_.setEnabled(enable);
        if (!enable || currentCB_ == null || currentCB_.verify().length() > 0) {
                execTimeButton_.setEnabled(false);
        } else {
                execTimeButton_.setEnabled(true);
        }
    }

    /**
     * Allows the user to explicitly save the displayed object
     * to external media by making the "Save" button visible.
     */
    public void putInEditMode() {
        setEnabled(true);
        fieldODName_.setEnabled(true);
        templateNames_.setEnabled(false);
        save_.setVisible(true);
        save_.setEnabled(true);
        //templateModel_.setEnabled(true);
    }

    /**
     * Handle a change in the property value for the associated
     * BusinessObject.  Enable or disable changes to the fields,
     * according to whether they are allowed to be modified.
     * Update nested ObjectView objects.
     *
     * @param ev    Property Change Event
     */
    public void propertyChange(PropertyChangeEvent ev) {
        setEnabled(isAllowedToModify(currentCB_));
        addTemplate_.setEnabled(isAllowedToModify(currentCB_));
        if (isAllowedToModify(currentCB_)) {
            addTemplate_.setEnabled(!templateNames_.isSelectionEmpty());
        } else
            addTemplate_.setEnabled(false);
        if (Config.getCfg().isInOperationsMode())
            putInEditMode();
        System.out.println("EV Property Name " + ev.getPropertyName());
        if (ev.getPropertyName() != null) {
            if (ev.getPropertyName().equals("Template")) {
                currentCB_.setExecTimeUpToDate(false);
                execTimeLabel_.setText("* Execution Time");
            }
            if (ev.getPropertyName().equals("ExecutionTime")) {
                execTimeLabel_.setText("Execution Time");
            }
        }

    }

    /**
     * Updates the DocView components to refer to the correct CB.
     *
     * @param newCB	    CalibrationBlock object to be displayed.
     */
    public void setObject(CalibrationBlock newCB) {

        if (currentCB_ != null) {
            currentCB_.removePropertyChangeListener(this);
        }

        currentCB_ = newCB;
        boolean canModify = isAllowedToModify(currentCB_);
        boolean forceODModelReload = false;
        if (currentCB_ != null) {
            /*
             * MIPS Project - if not single IP mode and CB is editable:
             * If the Observing Run IP version differs from the IP version in the
             * ObservationDescription of the currentOB_, and the CB is modifyable,
             * then it must be converted to the new IP version.
             */
            if (canModify) {
                ObservingRun obr = currentCB_.getObsRun();
                if (obr != null) {
                    boolean singleIPMode =
                        OHSConfig.getInstance().getBoolean(
                            OHSKeys.IPVersioningSingleIPModeEnabled);
                    float requiredIPVersion;
                    if (singleIPMode) {
                        // in single IP mode the required IP version
                        // is driven by the Instrument in the InstrumentList
                        // not from the Observing Run
                        Instrument inst =
                            InstrumentList.getInstance().getInstrument(
                                obr.getInstCode(),
                                obr.getIPVersion());
                        requiredIPVersion = inst.getVersionNumber();
                    } else {
                        // in multiple IP mode the required IP version
                        // is driven by the Observing Run
                        requiredIPVersion = obr.getIPVersion();
                    }
                    float obIPVersion = currentCB_.fetchIPVersionFromOd();
                    if (requiredIPVersion != obIPVersion) {
                        // The OB must be converted
                        stdlog_.debug(
                            "MIPS This CB is v"
                                + obIPVersion
                                + " but must be converted to v"
                                + requiredIPVersion);
                        try {
                            convertOB(requiredIPVersion);
                        } catch (IOException e) {
                            stdlog_.debug("MIPS CB CONVERSION FAILED");
                            e.printStackTrace();
                        }
                        if (!singleIPMode
                            && OHSConfig.getInstance().getBoolean(
                                OHSKeys.ShowConversionDialog)) {
                            JOptionPane.showMessageDialog(
                                this,
                                new Object[] {
                                    "This Calibration Block has been updated to use",
                                    "the Instrument Package specified in the Observing Run." },
                                "Update to new Instrument Package",
                                JOptionPane.INFORMATION_MESSAGE);
                        }
                        stdlog_.debug("MIPS CB CONVERSION EXECUTED");
                        forceODModelReload = true;
                    }
                }
            }
            currentOD_ = currentCB_.getOd();
        }

        fieldName_.setObjectProperty(currentCB_, "Name");

        fieldStatus_.setObjectProperty(currentCB_, "Status");

        fieldUserComments_.setObjectProperty(currentCB_, "UserComments");

        fieldUserPriority_.setObjectProperty(currentCB_, "UserPriority");

        /* cmavros added 17-10-2001 */
        fieldExecutionTime_.setObjectProperty(currentCB_, "ExecutionTime");

        fieldODName_.setObjectProperty(currentOD_, "Name");

        if (currentCB_ != null) {
            initLists();
            currentCB_.addPropertyChangeListener(this);
            if (!currentCB_.isExecTimeUpToDate()
                || currentCB_.getExecutionTime() <= 0) {
                execTimeLabel_.setText("* Execution Time");
            } else {
                execTimeLabel_.setText("Execution Time");
            }

        }
        if (currentOD_ != null) {
            templateModel_ =
                ODGridModel.getCachedODModel(
                    this,
                    currentOD_,
                    forceODModelReload);
            templateTable_.setModel(templateModel_);
            if (templateModel_.getColumnCount() > 0) {
                templateTable_.setRowColumn(
                    0,
                    templateModel_.getColumnCount() - 1);
            } else {
                removeTemplate_.setText("Delete");
                duplicateTemplate_.setText("Duplicate");
                removeTemplate_.setEnabled(false);
                duplicateTemplate_.setEnabled(false);
            }
        }

        fieldName_.requestFocus();

        setEnabled(isAllowedToModify(currentCB_));
        if (Config.getCfg().isInOperationsMode())
            putInEditMode();
        else {
            save_.setVisible(false);
            save_.setEnabled(false);
        }
    }

    /**
     * MIPS Project:
     * Convert the currentOB_ to the specified ip version
     * (instrument stays the same, of course!).
     * Method assumes that currentOB_ is non-null.
     * @throws IOException if the conversion (template refresh) failed.
     */
    private void convertOB(float obrIPVersion) throws IOException {
        currentCB_.convertIPVersion(obrIPVersion);
        // refresh templates (forcing a full refresh) and fire the autosaver.
        currentCB_.refreshTemplates(true, true);
    }

    private class CBTemplateTable extends ODTemplateTable {

        /**
         * Create a reference to a JTable
         * @param model initialise this table with this model
         */

        CBTemplateTable(TableModel model) {
            super(model);
        }

        /**
         * Create a table using the default model
         */

        CBTemplateTable() {
            super();
        }

        public void setModel(TableModel model) {
            super.setModel(model);
        }

        public void columnSelectionChanged(ListSelectionEvent ev) {
            super.columnSelectionChanged(ev);
            if (ev.getValueIsAdjusting())
                return;
            boolean canModify = isAllowedToModify(currentCB_);
            int column = getSelectedColumn();
            int columnCount = templateModel_.getColumnCount();
            if (!canModify || Config.getCfg().isInOperationsMode()) {
                return;
            }
            if (columnCount == 0) {
                removeTemplate_.setText("Delete");
                duplicateTemplate_.setText("Duplicate");
                removeTemplate_.setEnabled(false);
                duplicateTemplate_.setEnabled(false);
                return;
            }
            // Java1.4 port if(column >= columnCount)
            if (column >= columnCount || column < 0)
                return;
            removeTemplate_.setEnabled(true);
            duplicateTemplate_.setEnabled(true);
            if (templateModel_.isLabel(column)) {
                duplicateTemplate_.setEnabled(false);
                column++;
            }
            removeTemplate_.setText("Delete Col : " + (column + 1));
            duplicateTemplate_.setText("Duplicate Col : " + (column + 1));
        }
    }
}
