package org.eso.ohs.phase2.apps.p2pp.views;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Container;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Enumeration;
import java.util.Hashtable;

import javax.swing.Action;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JPanel;

import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.SystemStatusEvent;
import org.eso.ohs.core.utilities.SystemStatusListener;
import org.eso.ohs.dfs.BusinessObject;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.ObservationDescription;
import org.eso.ohs.dfs.VisitorAdaptor;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.ObjectSelectionEvent;
import org.eso.ohs.persistence.ObjectSelector;
import org.eso.ohs.phase2.actions.ShowFindingChartsAction;
import org.eso.ohs.phase2.apps.p2pp.MenuGroups;
import org.eso.ohs.phase2.apps.p2pp.P2PP;
import org.eso.ohs.phase2.apps.p2pp.Persistence;
import org.eso.ohs.phase2.apps.p2pp.actions.AttachFindingChartsMultiOBSel;
import org.eso.ohs.phase2.apps.p2pp.actions.ClearFindingCharts;
import org.eso.ohs.phase2.apps.p2pp.actions.DetatchFindingCharts;

/**
 * Displays one of a selection of ObjectView components.
 * <p>
 * $Id: ViewContainer.java,v 1.21 2006/04/27 13:25:00 pnunes Exp $
 * <br>
 * @author Bob Kemp
 */

public class ViewContainer
    extends MenuGroups
    implements ObjectSelector, SystemStatusListener {

    /** argument for bringViewToTheFront, to produce a blank screen */
    private final static String EMPTY_VIEW = "null";

    /** layout manager to be used. */
    private CardLayout layout_ = new CardLayout();

    /** table of known views that may be selected */
    private Hashtable<String, DocHolder> knownView_ = new Hashtable<String, DocHolder>();

    /** Tag for the ObjectView that is currently "at the front". */
    private String tagForVisiblePane_ = EMPTY_VIEW;

    /** Visitor method for generating the window's title */
    private final static TitleVisitor titleVisitor_ = new TitleVisitor();

    private JMenu copyMenu_;

    private JMenu fcMenu_;

    private Action attachFCAction_;

    private Action clearFCAction_;

    private Action detachFCAction_;

    private Action showFindingChartAction_;

    /**
     * Container to use for adding objects, etc.
     * A convenience to make it easier to switch between
     * subclassing JFrame and JPanel (at compile-time, of course :-).
     */
    private Container contentPane_;

    /**
     * Construct a viewer for ObjectView components.
     */
    public ViewContainer() {
        addClose();
        addCutCopyPaste();

        addCopyBOMenu("Synchronise", this);

        addFindingChartMenu("FindingCharts", this);

        contentPane_ = getContentPane();
        contentPane_.setLayout(layout_);

        addObjectViewImpl(
            new DocHolder(new EmptyView(), EMPTY_VIEW),
            EMPTY_VIEW);

        bringViewToTheFront(tagForVisiblePane_);
        setLocation(50, 50);

        P2PP.addSystemStatusListener(this);
    }

    public void setEnabledFCMenu(BusinessObject bo) {
        boolean opMode = Config.getCfg().isInOperationsMode();
        if(bo == null) {
            fcMenu_.setEnabled(false);
            return;
        }
        if (opMode) {
            // User has switched from operations normal mode
            // to opeations mode so we muse reinit actions.
            createOpModeActions();
            if(bo instanceof ObservationBlock)
                fcMenu_.setEnabled(true);
            else 
                fcMenu_.setEnabled(false);                      
        } else if (!opMode && bo.getDbaseId() != 0 && bo.getId() > 0) {
            // User cannot attach finding charts from
            // view invoked within ESO Database view
            fcMenu_.setEnabled(false);

        } else if (bo instanceof ObservationBlock) {
            // Enabled FC menu if OB
            fcMenu_.setEnabled(true);
        } else {
            // disable FC menu if CB
            fcMenu_.setEnabled(false);
        }

    }

    /**
     * Method addFindingChartMenu.
     * @param menuNAme
     * @param viewContainer
     */
    private void addFindingChartMenu(
        String menuName,
        ViewContainer viewContainer) {
        if (Config.getCfg().isInOperationsMode()) {
            createOpModeActions();
        } else {
            createLocalActions();
        }
        clearFCAction_ = new ClearFindingCharts(this);

        addMenuItem(menuName, attachFCAction_);
        addMenuItem(menuName, clearFCAction_);
        addSeparator(menuName);
        addMenuItem(menuName, detachFCAction_);
        addMenuItem(menuName, showFindingChartAction_);
        fcMenu_ = getMenu(menuName);

    }

    private void createLocalActions() {
        detachFCAction_ = new DetatchFindingCharts(this, this, Media.LOCAL);
        attachFCAction_ = new AttachFindingChartsMultiOBSel(this,this,Media.LOCAL);
        showFindingChartAction_ =
            new ShowFindingChartsAction(Media.LOCAL, this, this, true);
    }

    private void createOpModeActions() {
        attachFCAction_ = new AttachFindingChartsMultiOBSel(this,this, Media.DBASE);
        detachFCAction_ = new DetatchFindingCharts(this, this, Media.DBASE);
        showFindingChartAction_ =
            new ShowFindingChartsAction(Media.DBASE, this, this, false);

    }
    
    

    /**
     * Generate a tag corresponding to a particular BusinessObject class.
     * 
     * @param cls   Class for which the tag is to be generated.
     * 
     * @return	    String corresponding to the Class, or null if the
     * 		    parameter doesn't correspond to a BusinessObject subclass.
     */
    private String getTagForObjectClass(Class cls) {
        return BusinessObject.getSuffix(cls);

    }

    public JFrame getTopLevelAncestor() {
        return this;
    }

    /**
     * Add a new ObjectView that can be displayed.
     * 
     * @param view  view to be added.
     * @param objType   type of the view to be added.
     */
    private void addDocHolder(DocHolder holder) {
        addObjectViewImpl(holder, holder.getTag());
    }

    /**
     * Add a new ObjectView that can be displayed.
     * 
     * @param view  view to be added.
     * @param objType   type of the view to be added.
     */
    public void addObjectView(ObjectView view, Class<? extends BusinessObject> objType) {
        String tag = getTagForObjectClass(objType);
        DocHolder holder = new DocHolder(view, tag);

        addObjectViewImpl(holder, tag);
    }

    /**
     * Add a new ObjectView that can be displayed.
     * 
     * @param view  view to be added.
     * @param tag   type of the view to be added.
     */
    private void addObjectViewImpl(DocHolder holder, String tag) {
        contentPane_.add(holder, tag);
        knownView_.put(tag, holder);
        holder.viewAddedHook(this);
        pack();
    }

    /**
     * Remove a new ObjectView from the list that can be displayed.
     * 
     * @param cls	Class for the ObjectView to be removed
     */
    private void removeDocHolder(Class<? extends BusinessObject> cls) {
        String tag = getTagForObjectClass(cls);
        DocHolder holder = knownView_.remove(tag);

        if (holder != null) {
            contentPane_.remove(holder);
            holder.viewRemovedHook(this);
        }
    }

    /**
     * Moves an ObjectView object from another ViewContainer into this one.
     * 
     * @param objType       type of object in the ObjectView to be moved
     * @param from	    ViewContainer originally containing ObjectView
     */
    public void moveViewFrom(Class<? extends BusinessObject> objType, ViewContainer from) {
        String tag = getTagForObjectClass(objType);
        DocHolder comp = from.knownView_.get(tag);

        from.removeDocHolder(objType);
        addDocHolder(comp);
        setDisplayedObjClass(objType);

        from.setVisible(false);
    }

    /**
     * Makes this window visible and alerts ObjectViews contained in it
     * that their visibility has changed.
     * 
     * @param visible	    true if the window is to be displayed,
     *			    false if it is not.
     */
    public void setVisible(boolean visible) {
        super.setVisible(visible);

        Enumeration<String> enum0 = knownView_.keys();
        DocHolder holder;

        while (enum0.hasMoreElements()) {
            holder = knownView_.get(enum0.nextElement());
            holder.visibilityChange(this, visible);
        }
    }

    /**
     * Listener for changes in the overall system status.
     * Specifically, when the user logs in under a different account,
     * the ObjectViews contained in this ViewContainer are cleared.
     * Thus the user cannot be mistakenly show objects for the previous
     * account.
     * 
     * @param ev	    Event describing how the system status has changed
     */
    public void systemStatusChange(SystemStatusEvent ev) {
        if (ev.getType() == SystemStatusEvent.RELOGIN) {
            Enumeration<String> enum0= knownView_.keys();
            DocHolder holder;

            while (enum0.hasMoreElements()) {
                holder = knownView_.get(enum0.nextElement());
                holder.setDisplayedObject(this, null);
            }
        }
    }

    /**
     * Brings a new ObjectView object to the front within this ViewContainer.
     * 
     * @param tag	Tag for the ObjectView to be brought forward.
     */
    private void bringViewToTheFront(String tag) {
        DocHolder newHolder = knownView_.get(tag);

        if (newHolder != null) {
            DocHolder oldHolder =
                knownView_.get(tagForVisiblePane_);
            if (oldHolder != null) {
                oldHolder.orderingChange(this, false);
            }

            tagForVisiblePane_ = tag;
            layout_.show(contentPane_, tag);

            newHolder.orderingChange(this, true);
        }
    }

    /**
     * Brings the ObjectView for the specified object type to the
     * front in this Viewvontainer.
     * 
     * @param objType	Class for an ObjectView in this ViewContainer
     */
    public void setDisplayedObjClass(Class<? extends BusinessObject> objType) {
        String tag = getTagForObjectClass(objType);

        bringViewToTheFront(tag);
    }

    public boolean isAnObjectDisplayed() {
        DocHolder holder = knownView_.get(tagForVisiblePane_);

        if (holder != null) {
            ObjectView view = holder.getObjectView();

            if (view != null) {
                return (view.getObject() != null);
            }
        }

        return false;
    }

    /**
     * Make this ViewContainer visible if it has an object to display.
     */
    public void restoreView() {
        DocHolder holder = knownView_.get(tagForVisiblePane_);

        if (holder != null) {
            ObjectView view = holder.getObjectView();

            if (view != null && view.getObject() != null) {
                setVisible(true);
            }
        }
    }

    public BusinessObject getDisplayedObject() {
        if (isAnObjectDisplayed()) {
            DocHolder holder = knownView_.get(tagForVisiblePane_);
            if (holder != null) {
                ObjectView view = holder.getObjectView();
                if (view != null) {
                    return view.getObject();
                }
            }
        }
        return null;
    }

    /**
     * Sets the object displayed in the corresponding ObjectView.
     * Note that the ObjectView is not brought to the front.
     * 
     * @param bo	BusinessObject to be displayed.
     */
    public void setDisplayedObject(BusinessObject bo) {
        // Set window contents to display the right object.
        String tag = getTagForObjectClass(bo.getClass());
        Object holder = knownView_.get(tag);

        if (holder instanceof DocHolder) {
            ((DocHolder) holder).setDisplayedObject(this, bo);

        }
    }
    
    


    /**
     * Enable or disable the ObjectView objects contained in this ViewContainer.
     * 
     * @param enable	True if ObjectViews are to be enabled, false otherwise
     */
    public void setEnabled(boolean enable) {
        Enumeration<String> enum0 = knownView_.keys();

        while (enum0.hasMoreElements()) {
            Object obj = knownView_.get(enum0.nextElement());

            if (obj instanceof DocHolder) {
                ((DocHolder) obj).setEnabled(enable);
            }
        }
    }

    /**
     * Sets an object in the corresponding ObjectView
     * and optionally brings it to the front, thus making it visible
     * if the window itself is visible.
     * 
     * @param device		Media on which the object is stored
     * @param id		Id of the object to be displayed
     * @param objType		Type of object to be displayed
     * @param bringToTheFront	should it be brought to the front
     *				    or just updated.
     * @exception   ObjectIOException
     *				if an error occurred while reading the object
     * @exception   ObjectNotFoundException
     *				if the object was not found on the media
     */
    public void display(
        Media device,
        long id,
        Class objType,
        boolean bringToTheFront)
        throws ObjectIOException, ObjectNotFoundException {

        Persistence objMgr = Persistence.getInstance();
        String tag = EMPTY_VIEW;
        BusinessObject bo;

        if (id != ObjectSelectionEvent.NO_OBJECT) {
            tag = getTagForObjectClass(objType);
        }

        if (bringToTheFront) {
            bringViewToTheFront(tag);
        }

        if (id != ObjectSelectionEvent.NO_OBJECT) {
            bo = (BusinessObject) objMgr.getBusObj(device, id, objType);
            setDisplayedObject(bo);

        }
    }

    /** An empty ObjectView, displaying just an empty screen */
    public class EmptyView extends ObjectView {
        public BusinessObject getObject() {
            return null;
        }

        public void setObject(BusinessObject bo) {
            // do nothing
        }
    }

    /**
     * DocHolders act as intermediaries between a ViewContainer
     * and an ObjectView.  They update the window title to reflect
     * the displayed contents of the ObjectView and lazily update
     * the object displayed within the ObjectView to avoid unnecessary
     * work.
     */
    private class DocHolder extends JPanel implements PropertyChangeListener {

        /** ObjectView contained in this DocHolder */
        private ObjectView objView_;

        /** Tag within the ViewContainer for this DocHolder */
        private String tag_;

        /** Is the containing window visible? */
        private boolean isVisible_;

        /**
         * Is this DocHolder "at the front"?
         * In other words, not covered by others in the CardLayout.
         */
        private boolean isInFront_;

        /** Object that is to be displayed in the ObjectView */
        private BusinessObject object_;

        /**
         * Constructor.
         * 
         * @param objv	    ObjectView to be displayed.
         * @param tag	    Tag for this DocHolder
         */
        public DocHolder(ObjectView objv, String tag) {
            objView_ = objv;
            tag_ = tag;

            setLayout(new BorderLayout());
            add(objv, BorderLayout.CENTER);
        }

        /**
         * Returns the tag for this DocHolder
         * 
         * @return	    Tag for this object
         */
        public String getTag() {
            return tag_;
        }

        /**
         * Returns the ObjectView for this DocHolder
         * 
         * @return	    ObjectView for this DocHolder
         */
        public ObjectView getObjectView() {
            return objView_;
        }

        public boolean isInFront() {
            return isInFront_;
        }

        public void visibilityChange(ViewContainer vc, boolean isVisible) {
            isVisible_ = isVisible;
            sync(vc);
        }

        /**
         * Notifies the DocHolder that it has been "brought to the front"
         * within the ViewContainer or "moved behind" another ObjectView.
         * 
         * @param   vc	    	Containing ViewContainer for this DocHolder
         * @param   toFront	True if ObjectView is being brought to front,
         *			false if it's being moved behind something.
         */
        public void orderingChange(ViewContainer vc, boolean toFront) {
            isInFront_ = toFront;
            sync(vc);
        }

        /**
         * Notifies the DocHolder of a change in its visibility.
         * 
         * @param   vc	    	Containing ViewContainer for this DocHolder
         * @param   isVisible	True if window is now visible, false otherwise
         */
        public BusinessObject getDisplayedObject() {
            return object_;
        }

        /**
         * Sets the object to be displayed by the ObjectView within this
         * DocHolder.  However, setDisplayedObject() may not immediately be
         * called on the ObjectView, if it is not currently visible to the user.
         * 
         * @param   vc	    	Containing ViewContainer for this DocHolder
         * @param   isVisible	True if window is now visible, false otherwise
         */
        public void setDisplayedObject(ViewContainer vc, BusinessObject obj) {
            if (object_ != null) {
                object_.removePropertyChangeListener(this);
            }
            object_ = obj;
            if (object_ != null) {
                object_.addPropertyChangeListener(this);
            }
            sync(vc);
        }

        /**
         * Notifies the DocHolder that it has been added to the
         * ViewContainer.  The nested ObjectView is, in turn, notified.
         * 
         * @param   vc	    	new ViewContainer for this DocHolder
         */
        public void viewAddedHook(ViewContainer vc) {
            if (objView_ != null) {
                objView_.viewAddedHook(vc);
                sync(vc);
            }
        }

        /**
         * Notifies the DocHolder that it has been removed from the
         * ViewContainer.  The nested ObjectView is, in turn, notified.
         * 
         * @param   vc	    	old ViewContainer for this DocHolder
         */
        public void viewRemovedHook(ViewContainer vc) {
            if (objView_ != null) {
                objView_.viewRemovedHook(vc);
            }
        }

        /**
         * Notifies the DocHolder that it has been removed from the
         * ViewContainer.  The nested ObjectView is, in turn, notified.
         * 
         * @param   vc	    	old ViewContainer for this DocHolder
         */
        private void sync(ViewContainer vc) {

            if (isInFront_) {
                vc.setTitle(makeTitle());
                if (isVisible_) {
                    setEnabledSynchroniseMenu(object_);
                    setEnabledFCMenu(object_);
                    objView_.setObject(object_);
                }
            }
        }

        public void propertyChange(PropertyChangeEvent ev) {
            if (isInFront_) {
                setEnabledSynchroniseMenu(object_);
                setEnabledFCMenu(object_);
                setTitle(makeTitle());

            }
        }

        /**
        * Create a title for the ViewContainer based on the current object.
        * 
        * @return	Generated title for the window
        */
        private String makeTitle() {
            Persistence objMgr = Persistence.getInstance();

            if (object_ == null || !objMgr.isInRegistry(object_.getId())) {
                return "No object";
            }

            Media device = objMgr.getMediaFor(object_);
            String name = object_.getClass().getName();
            String where = device.getName();
            String title = new String();

            if (where != "local")
                title += where + ": ";

            title += BusinessObject.getDisplayName(object_.getClass())
                + ": "
                + object_.getName();

            String instrument = (String) object_.accept(titleVisitor_, null);
            if (instrument != null) {
                title += ": " + instrument;
            }
            return title;
        }

    }

    /** Utility class to generate a title based on the object and its type */
    private static class TitleVisitor extends VisitorAdaptor {

        public Object visit(CalibrationBlock cb, Object arg) {
            return cb.getObsRun().getInstCode();
        }

        public Object visit(ObservationBlock ob, Object arg) {
            return ob.getObsRun().getInstCode();
        }

        public Object visit(ObservationDescription od, Object arg) {
            return od.getInstrument();
        }

        public Object commonVisitor(BusinessObject bo, Object arg) {
            return null;
        }

    }

}
