#!/bin/csh -f

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# COPYRIGHT (c) 2003 European Southern Observatory
# LICENSE
#
# NAME:      checkAllDfs.sh
# TYPE:      Shell script
# PROJECT:   VLT Data Flow System
# AUTHOR:    KH - ESO/SDD/SED/SEG
# DESCRIPTION:
# USAGE:     checkAllDfs.sh YYYY-MM-DD (one mandatory argument= date of
#                                       the directory where to check logs
#                                       under /data/msg/) 
# EXAMPLES:  
#
# $Id
#
# who            when            what
#------------------------------------------------------------------------------
# K. Haggouchi   01/04/07        created
#------------------------------------------------------------------------------

set OS=`uname -s`
switch ($OS)
  case HP-UX:
    set RSH='remsh'
    breaksw
  case Linux:
    set RSH='/usr/bin/rsh'
    breaksw
  default:
    echo "Unknown OS=$OS"
    exit 1
endsw

set dateToCheck=$1

set bon =  "[1m"
set boff = "[0m"
set noglob =  1

echo "**************************************************************************"
echo "*                                                                        *"
echo "* This tool performs a number of checks on DFS logs on all DFS machines. *"
echo "*                                                                        *"
echo "**************************************************************************"
echo ""

set dateToCheck=$1
if ( "${dateToCheck}" == "" ) then
   echo "*** ERROR *** The mandatory argument is missing: must be the date of"
   echo "          *** the directory where to check logs under /data/msg/"
   echo "          *** Example:"
   echo "          ***           checkAllDfs.sh 2007-04-01"
   echo "          *** will check all DFS logs located under /data/msg/2007-04-01"
   echo "          *** on all OlasWSs and all PipelineWSs, and all logs located under"
   echo "          *** /data-utN/msg/2007-04-01/ on all UserWSs"
   echo "          *** Try again with a valid date (format: YYYY-MM-DD) !"
   exit 1
endif

if ( "${dateToCheck}" == "-help" ) then
   echo "Usage: checkAllDfs.sh YYYY-MM-DD"
   echo "Example: checkAllDfs.sh 2007-04-01"
   echo "         will check all DFS logs located under /data/msg/2007-04-01"
   echo "         on all OlasWSs and all PipelineWSs, and all logs located under"
   echo "         /data-utN/msg/2007-04-01/ on all UserWSs"
   exit 1
endif

echo "-------------------------------------------------------------------------------"
echo "- OlasWSs:                                                                    -"
echo "  Look for ERRORs in the logs related to data transportation                  -"
echo "  (Olas processes: DHS, FrameIngest, ngamsIngest and VCSOLAC)                 -"
echo "  for the given date                                                          -"
echo "-------------------------------------------------------------------------------"
loop.sh OlasLSs "grep -H -i error /data/msg/DHS*.${dateToCheck} /data/msg/FrameIngest*.${dateToCheck} /data/msg/ngamsIngest*.${dateToCheck} /data/msg/VCSOLAC*.${dateToCheck}"

echo ""
echo "-------------------------------------------------------------------------------"
echo "- PipelineWSs:                                                                -"
echo "  Look for ERRORs in the logs related to data transportation                  -"
echo "  (Olas processes: DhsSubscribe, DHS, and VCSOLAC)                            -"
echo "  for the given date                                                          -"
echo "-------------------------------------------------------------------------------"
loop.sh PipelineLSs "grep -H -i error /data/msg/DhsSubscribe*.${dateToCheck} /data/msg/DHS*.${dateToCheck} /data/msg/VCSOLAC*.${dateToCheck} "

echo ""
echo "-------------------------------------------------------------------------------"
echo "- PipelineWSs:                                                                -"
echo "  Look for ERRORs in the log related to the creation of Reduction Blocks      -"
echo "  (DataOrganiser)                                                             -"
echo "  for the given date                                                          -"
echo "-------------------------------------------------------------------------------"
# Beware: the DataOrganiser.log of today does not contain the date in its name
set currentDate=`date --iso-8601`
if ( "${dateToCheck}" == "${currentDate}" ) then
   loop.sh PipelineLSs "grep -H -i error /data/msg/DataOrganiser.log"
else
   loop.sh PipelineLSs "grep -H -i error /data/msg/DataOrganiser.log.${dateToCheck}"
endif

echo ""
echo "-------------------------------------------------------------------------------"
echo "- PipelineWSs:                                                                -"
echo "  Look for ERRORs in the logs related to the processing of Reduction Blocks   -"
echo "  (RBS, Midas, rblog, r*.log)                                                 -"
echo "  for the given date                                                          -"
echo "-------------------------------------------------------------------------------"
loop.sh PipelineLSs "grep -H ERROR /data/msg/RBS*.${dateToCheck} /data/msg/Midas.${dateToCheck} /data/msg/*.${dateToCheck}*.rblog /data/msg/r.*${dateToCheck}*.log | grep -v INFO"

echo ""
echo "-------------------------------------------------------------------------------"
echo "- UserWSs:                                                                    -"
echo "  Look for ERRORs in the logs related to data transportation                  -"
echo "  (Olas processes: DhsSubscribe-RAW and DhsSubscribe-REDUCED)                 -"
echo "  for the given date                                                          -"
echo "-------------------------------------------------------------------------------"
echo "$bon w2p2off $boff"
$RSH w2p2off "grep -H -i error /data/msg/DhsSubscribe*.${dateToCheck}"

echo "$bon wg5off $boff"
$RSH wg5off "grep -H -i error /data/msg/DhsSubscribe*.${dateToCheck}"

echo "$bon w3p6off $boff"
$RSH w3p6off "grep -H -i error /data/msg/DhsSubscribe*.${dateToCheck}"


echo ""
echo "-------------------------------------------------------------------------------"
echo "- UserWSs:                                                                    -"
echo "  Look for ERRORs in the logs related to adding MOON parameters               -"
echo "  (FTU: fitsTranslatelog)                                                     -"
echo "  for the given date                                                          -"
echo "-------------------------------------------------------------------------------"
echo "$bon w2p2off $boff"
$RSH w2p2off "grep -H -i error /data/msg/fitsTranslatelog.${dateToCheck} | grep -H -v INFO | grep -H -v WARNING"

echo "$bon wg5off $boff"
$RSH wg5off "grep -H -i error /data/msg/fitsTranslatelog.${dateToCheck} | grep -H -v INFO | grep -H -v WARNING"

echo "$bon w3p6off $boff"
$RSH w3p6off "grep -H -i error /data/msg/fitsTranslatelog.${dateToCheck} | grep -H -v INFO | grep -H -v WARNING"

