#!/bin/bash 

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# COPYRIGHT (c) 2003 European Southern Observatory
# LICENSE
#
# NAME:      checkcalibDBtar.sh
# TYPE:      Bourne Shell script
# PROJECT:   VLT Data Flow System
# AUTHOR:    KH - ESO/SDD/SED/SEG
# DESCRIPTION:
# USAGE:     checkcalibDBtar.sh <inst>-calibdb-X.Y.Z.tar.gz
# EXAMPLES:
#
# $Id
#
# who            when            what
#------------------------------------------------------------------------------
# C. Guirao       06/02/08        modified to be used with ipip
# K. Haggouchi    06/02/08        created
#------------------------------------------------------------------------------
 
#  check that the FITS and TFITS files provided in the input tar file 
#  are given under a <inst>/cal/
#  directory (return 0), and not a <inst>/<mode>/cal/ one (return 1)


tarFile=$1
# check mandatory argument
if [ "${tarFile}" = "" ]; then
   exit 1
fi

#
# For each file in the tar.gz file that finishes with "fits", remove the
# first directory and the filename, the rest should be "cal" otherwise error.
#
tar tzf ${tarFile} | egrep "fits$" | sed -e 's/^[^/]*\///' -e 's/\/[^/]*$//' | while read subdir
do
  # echo ${subdir}
  if [ "${subdir}" = "cal" ] ; then
    continue
  else
    basename_tarFile=`basename ${tarFile}`
    echo ""
    echo "ERROR: ${basename_tarFile} is not flat." 
    echo "       It contains fits files other than in \"cal\" subdir." 
    echo "       Fits files in subdir \"${subdir}\" are not valid. Exit."
    exit 1
  fi
done
exit $?
