#!/usr/bin/env bash 
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# COPYRIGHT (c) 1996 European Southern Observatory
# LICENSE
#
# PROJECT:   VLT Data Flow System
# AUTHOR:    Carlos Guirao - ESO/DMD/DPG
#
# USAGE: To be executed by root in a Bourne Shell as follows
#        	./hostreport.sh > /tmp/hostreport.`hostname`.`date +'%C%y-%m-%d'` 2>&1
#
#	 In combination with loops command, use it as follow:
#
#		~flowmgr/dfs/bin/loop.sh PARANAL '/root/bin/hostreport.sh > /tmp/hostreport.`hostname`.`date +'%C%y-%m-%d'` 2>&1'
#
# REMARK: When changing this file DO NOT FORGET to update the WEB page:
#		webcheck -o -w seg-hostreport-man.html
#		webcheck -i -w seg-hostreport-man.html
#
# $Revision: 1.14 $
#
# SDC	11.02.2005	adapted for Linux:
#			updated PATH
#			use sfdisk instead of diskinfo on Linux
#			use df instead of bdf on Linux
#			don't use HP-specific commands on Linux
#			use rpm instead of swlist on Linux
#
#------------------------------------------------------------------------------

PATH=/usr/sbin:/usr/bin:/opt/ansic/bin:/usr/ccs/bin:/usr/contrib/bin:/opt/nettladm/bin:/opt/pd/bin:/usr/bin/X11:/usr/contrib/bin/X11:/opt/upgrade/bin:/opt/langtools/bin:/opt/fortran/bin:/opt/imake/bin:/bin:/sbin
export PATH

#
# Execute as $1 account the command $2. Silently
#
execute_silently()
{
  if su - $1 -c true 2>/dev/null; then
    su - $1 -c "(TERM=nodumb ; . .bashrc ; $2 2>&1)"
  fi
}
#
# Execute as $1 account the command $2 
#
execute()
{
  if su - $1 -c true 2>/dev/null; then
    echo "======================================================================="
    echo "Executing: $2 as $1 account"
    echo "======================================================================="
    su - $1 -c "(TERM=nodumb ; . .bashrc ; $2 2>&1)"
  fi
}

check_asto_db()
{

rm -f /tmp/check_asto_db.sh
#
# First create the script /tmp/check_asto_db.sh
#

cat >/tmp/check_asto_db.sh << EOF  
#!/bin/sh 

SYBASE=/opt/sybase
DB_USER=asto
DB_PWD=astotest

echo "======================================================================="
echo "Check the static tables of database asto"
echo "======================================================================="
touch /tmp/tmp.bcp

echo "Content of asto..bookDevice"
\$SYBASE/bin/bcp asto..bookDevice out /tmp/tmp.bcp -c -S\$DSQUERY -U\$DB_USER -P\$DB_PWD >/dev/null 2>&1
cat /tmp/tmp.bcp
echo "end of asto..bookDevice"

echo ""
echo "Content of asto..devices"
\$SYBASE/bin/bcp asto..devices out /tmp/tmp.bcp -c -S\$DSQUERY -U\$DB_USER -P\$DB_PWD >/dev/null 2>&1
cat /tmp/tmp.bcp
echo "end of asto..devices"

echo ""
echo "Content of asto..logicalDevices"
\$SYBASE/bin/bcp asto..logicalDevices out /tmp/tmp.bcp -c -S\$DSQUERY -U\$DB_USER -P\$DB_PWD >/dev/null 2>&1
cat /tmp/tmp.bcp
echo "end of asto..logicalDevices"

echo ""
echo "Content of asto..mediaTypes"
\$SYBASE/bin/bcp asto..mediaTypes out /tmp/tmp.bcp -c -S\$DSQUERY -U\$DB_USER -P\$DB_PWD >/dev/null 2>&1
cat /tmp/tmp.bcp
echo "end of asto..mediaTypes"

echo ""
echo "Content of asto..media"
\$SYBASE/bin/bcp asto..media out /tmp/tmp.bcp -c -S\$DSQUERY -U\$DB_USER -P\$DB_PWD >/dev/null 2>&1

cat /tmp/tmp.bcp
echo "end of asto..media"

echo ""
echo "Content of asto..media_stages"
\$SYBASE/bin/bcp asto..media_stages out /tmp/tmp.bcp -c -S\$DSQUERY -U\$DB_USER -P\$DB_PWD >/dev/null 2>&1
cat /tmp/tmp.bcp
echo "end of asto..media_stages"

rm -f /tmp/tmp.bcp
EOF

#
# Make the script executable
#
chmod +x /tmp/check_asto_db.sh

#
# Execute the script as 'archeso
#
execute archeso /tmp/check_asto_db.sh

#
# Finally remove the script
#
rm -rf /tmp/check_asto_db.sh
}

check_file_exist ()
{
  if [ -f $1 ]; then
    return 0
  else
    return 1
  fi
}

find_vlt_boot_release ()
{
  if ! check_file_exist /vlt/System/vltsw_bootstrapping_release ; then
    echo "Bootstrap file /vlt/System/vltsw_bootstrapping_release does not exist"
    return 1
  fi
  VLTSW_RELEASE=`egrep -v '^#|^[        ]*$' /vlt/System/vltsw_bootstrapping_release`
  if [ -z "$VLTSW_RELEASE" ]; then
    echo "is empty. (ERROR)"
    echo "Bootstrap VLTSW_RELEASE is not known"
    return 1
  else
    echo "Bootstrap VLTSW_RELEASE is $VLTSW_RELEASE"
    return 0
  fi
}

thisScript=$0

echo "BEGIN REPORT: \$Revision: 1.14 $"

case `uname` in
  'HP-UX' ) command_list=( \
    hostname \
    date \
    uptime \
    model \
    'grep `hostname` /etc/hosts' \
    'uname -a' \
    /root/bin/myparms \
    exportfs \
    bdf \
    'ioscan -knf' \
    'swlist -l bundle' \
    'swlist -l product' \
    'y2koscheck -v' \
    setboot \
    'vgdisplay -v vg00' \
    'which xdialog' \
    'ps -ef' ) ;;
  'Linux' ) command_list=( \
    hostname \
    date \
    uptime \
    "grep `hostname` /etc/hosts" \
    "sfdisk -l" \
    exportfs \
    df \
    "cat /proc/cpuinfo" \
    "cat /proc/meminfo" \
    "uname -a" \
    "rpm -qa | sort" \
    "chkconfig --list" \
    "ps -ef" ) ;;
esac

for command in "${command_list[@]}"
do
  echo "====================================================================="
  echo "Executing: $command"
  echo "====================================================================="
  eval $command
done

echo "====================================================================="
echo "Executing: find_vlt_boot_release"
echo "====================================================================="
find_vlt_boot_release

echo "====================================================================="
echo "Executing: grep \"Revision: \" $thisScript"
echo "====================================================================="
grep \"Revision: \" $thisScript

for file in \
/etc/fstab  \
/etc/pfs_fstab  \
/etc/services  \
/etc/syslog.conf \
/etc/passwd  \
/etc/group  \
/stand/system \
/var/adm/cron/cron.allow \
/var/spool/cron/crontabs/root \
/var/spool/cron/crontabs/pipeline \
/var/spool/cron/crontabs/archeso \
/var/spool/cron/root \
/var/spool/cron/pipeline \
/var/spool/cron/archeso \
/etc/resolv.conf \
/etc/rc.log \
/etc/ld.so.conf \
/etc/ntp.conf \
/etc/rc.config.d/netconf \
/etc/atm/atm_0/el100 \
/usr/server/etc/sudoers \
/opt/sybase/interfaces \
/etc/pecs/releases/000/etc/locality/apps-all.env \
/etc/pecs/releases/000/etc/locality/misc-all.env \
/etc/pecs/releases/000/etc/locality/misc-all.ali \
/etc/pecs/releases/000/etc/locality/apps-`hostname`.env \
~flowmgr/dfs/admin/dataflowUser.sh \
/vlt/$VLTSW_RELEASE/NOCCS/vltMakeInstall.log \
/vlt/$VLTSW_RELEASE/CCSLite/vltMakeInstall.log
do
  if [ -f $file ]; then
    echo "====================================================================="
    echo "Executing: cat $file"
    echo "====================================================================="
    eval cat $file
  fi
done

for dir in \
/ \
/opt \
/etc/init.d \
/sbin/init.d \
/sbin/rc2.d \
/sbin/rc3.d \
/usr/server/bin \
/usr/server/etc \
/vlt \
/home \
/diska \
/diskb \
/diskc \
/diskd \
/data \
~flowmgr \
~flowmgr/dfsSrc \
/vltdata/ENVIRONMENTS \
~instmgr/instruments
do
  if [ -d $dir ]; then
    echo "====================================================================="
    echo "Executing: cd $dir ;  ls -l"
    echo "====================================================================="
    eval "cd $dir ; ls -l"
  fi
done

for dir in \
/diska \
/diskb \
/diskc \
/diskd \
~flowmgr
do
  if [ -d $dir ]; then
    echo "====================================================================="
    echo "Executing: cd $dir ;  du -s -k *"
    echo "====================================================================="
    eval "cd $dir ; du -s -k *"
  fi
done

if [ `uname` = 'HP-UX' ]; then
  echo "====================================================================="
  echo "Executing: diskinfo <disks devices>"
  echo "====================================================================="
  for disk in `ioscan -k -d sdisk -f -n | grep '/dev/dsk' | awk '{print $2}'`
  do
    diskinfo $disk 2>&1
    echo ""
  done
fi

#
# Load the DFS environment to know in which machine we are
#
if [ -f ~flowmgr/dfs/admin/dataflowUser.sh ]; then
  . ~flowmgr/dfs/admin/dataflowUser.sh 2>/dev/null
fi

#
# if we are in the $AstoWS machine
#
if [ "$AstoWS" = `hostname` ]; then
  execute archeso env
  execute archeso 'cat .rhosts'
  execute archeso 'cat .dbrc'
  execute archeso 'cat .pecs/apps-all.env'
  execute archeso 'cat .pecs/misc-all.env'
  execute archeso 'cat .pecs/misc-all.ali'
  execute archeso 'ls -l /media'
  execute archeso 'cd /media ; find . -type d -exec ls -ld {} \;'
  execute archeso 'du -s -k *'
  for file in astoControl.config dataSubscriberAsto.config \
	      libAd.config libAm.config writeControl.config mkisofs.config
  do
    echo "======================================================================="
    echo "Executing: cat ~archeso/astoconf/$file"
    echo "======================================================================="
    cat ~archeso/astoconf/$file
  done
  check_asto_db
fi

# if we are in the $PipelineWS machine
#
if [ "$PipelineWS" = `hostname` -o "$UserWS" = `hostname` ]; then
  execute quality env
  execute quality 'df .'
  execute quality 'cd MSQL2 ; find . -type d -exec ls -ld {} \;'
  execute quality 'cd /data/calibDB ; find . -type d -exec ls -ld {} \;'
  execute quality 'cat .pecs/apps-all.env'
  execute quality 'cat .pecs/misc-all.env'
  execute quality 'cat .pecs/misc-all.ali'
  execute quality 'ls -lR INS_ROOT'
  execute quality 'ls -lR INS_ROOT_DO'
  execute quality 'du -s -k *'
  execute quality 'ls -l pipeline-deliveries/'
  execute quality 'ls -l pipelines/'
  execute pipeline 'cat pipeline.config'
  execute pipeline env
  execute pipeline 'df .'
  execute pipeline 'cat .bashrc'
  execute pipeline 'cat .rhosts'
  execute pipeline 'cat .dbrc'
  execute pipeline 'cat .pecs/apps-all.env'
  execute pipeline 'cat .pecs/misc-all.env'
  execute pipeline 'cat .pecs/misc-all.ali'
  execute pipeline 'cat .eclipse-rc'
  execute pipeline 'cat .gasganorc'
  execute pipeline 'cat .esorex/esorex.rc'
  execute pipeline 'ls -lR'
  execute pipeline 'du -s -k *'
  for dir in /$MIDASHOME/$MIDVERS/pipeline /tcal /cal
  do
    echo "====================================================================="
    echo "Executing: cd $dir ; find . -type d"
    echo "====================================================================="
    eval "cd $dir; find . -type d" 
  done
fi

#
# if we are in the $UserWS machine
#
if [ "$UserWS" = `hostname` ]; then
  execute archeso env
  execute archeso 'df .'
  execute archeso 'cat .bashrc'
  execute archeso 'cat .rhosts'
  execute archeso 'du -s -k *'
  for account in astro astro1 astro2 astro3 astro4 
  do 
    execute $account env
    execute $account 'df .'
    execute $account 'cat .dataSubscriberConfig'
    execute $account 'cat .dbrc'
    execute $account 'ls -l .pecs'
    execute $account 'cat .pecs/apps-all.env'
    execute $account 'cat .pecs/misc-all.env'
    execute $account 'cat .pecs/misc-all.ali'
  done
fi

#
# if we are in the $OhsWS machine
#
if [ "$OhsWS" = `hostname` ]; then
  for account in visitor service instmgr
  do
    execute $account env
    execute $account 'cat .p2pp.cf'
    execute $account 'cat .ot.cf'
    execute $account 'cat .pecs/apps-all.env'
    execute $account 'cat .pecs/misc-all.env'
    execute $account 'cat .pecs/misc-all.ali'
    execute $account 'which jp2pp jot jre'
    execute $account 'du -s -k *'
  done
  echo "======================================================================="
  echo "Executing: ls -l ~flowmgr/dfs/dataflowJava/config"
  ls -l ~flowmgr/dfs/dataflowJava/config
  echo "======================================================================="
  for file in p2pp-db.cf ot-db.cf fcv-db.cf mt-db.cf site.cf
  do
    echo "======================================================================="
    echo "Executing: cat ~flowmgr/dfs/dataflowJava/config/$file"
    echo "======================================================================="
    cat ~flowmgr/dfs/dataflowJava/config/$file
  done
fi

#
# if we are in the $OlasWS machine
#
if [ "$OlasWS" = `hostname` ]; then
  execute archeso env
  execute archeso 'cat .bashrc'
  execute archeso 'cat .rhosts'
  execute archeso 'cat .pecs/apps-all.env'
  execute archeso 'cat .pecs/misc-all.env'
  execute archeso 'cat .pecs/misc-all.ali'
  execute archeso 'cat .dbrc'
  execute archeso 'du -s -k *'
fi

echo "END REPORT"
exit 0
