#!/bin/sh
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# COPYRIGHT (c) 2008 European Southern Observatory
# LICENSE
#
# PROJECT:   VLT Data Flow System
# AUTHOR:    Carlos Guirao - ESO/SDD/SDE
# DESCRIPTION: Script to roll-back previous DFS release
#
#------------------------------------------------------------------------------


#
# Check how the echo command behaves
#
set_echoes()
{
  if [ "`echo -n a`" = a ]; then
    # BSD echo
    NNN=-n
    CCC=
  else
    # USG echo
    NNN=
    CCC='\c'
  fi
  readonly NNN CCC
}


#
#  A safe way to perform echo without sending a new-line
#
necho()
{
  if [ "$ECHO" = "1" ]; then
    echo $NNN "$*" $CCC
  fi
}

myecho()
{
  if [ "$ECHO" = "1" ]; then
    echo "$*"
  fi
}

error()
{
#  standard error message
   echo "***ERROR  ! ***" 
}


check_is_a_link ()
{
  if [ -h "$1" ]; then
    myecho "$1 is a soft link. (OK)"
    return 0
  else
    myecho "$1 is not a soft link. (ERROR)"
    return 1
  fi
}

check_directory_exist ()
{
  necho "Directory $1 "
  if [ -d $1 ]; then
    myecho "exists. (OK)"
    return 0
  else
    myecho "does not exist. (ERROR)"
    return 1
  fi
}

#
#  Get a yes/no answer from the user; $1 is the prompt, $2 the default
#
ask_yn()
{
  prompt="$1"
  def="${2:-y}"           # yes is the default if not supplied
  while true; do
    necho "$prompt [$def]: "
    read answer
    answer="${answer:-$def}"
    case "$answer" in
      [yY])
        answer=y
        break
        ;;
      [nN])
        answer=n
        break
        ;;
      *)
        error Please enter "'y'" or "'n'".
        ;;
    esac
  done
  [ $answer = y ]
}

#
# MAIN, Main, main
cmd=`basename $0`
dirname=`dirname $0`

set_echoes
#
# ECHO=1 : functions necho() and myecho() enable (to print to terminal)
# ECHO=0 : functions necho() and myecho() disable (do not print anything)
#
ECHO=0

if [ `whoami` != "flowmgr" ]; then
  echo "This script is only meant to be executed by flowmgr"
  exit 1
fi

cd

if ! check_is_a_link dfs.old ; then
  echo "dfs.old is not a soft link"
  exit 1
fi
if ! check_directory_exist dfs.old ; then
  echo "dfs.old is not a directory or it does not exist"
  exit 1
fi
if ! check_is_a_link dfs ; then
  echo "dfs is not a soft link"
  exit 1
fi
if ! check_directory_exist dfs ; then
  echo "dfs is not a directory or it does not exist"
  exit 1
fi

ECHO=1
while :
do
  OLD_DFS=`ls -l dfs.old | awk -F '>' '{printf $2}'`
  DFS=`ls -l dfs | awk -F '>' '{printf $2}'`

  echo ""
  echo "Current settings:"
  echo "      dfs     points to $DFS"
  echo "      dfs.old points to $OLD_DFS"

  echo ""
  echo "Ready to roll-back, i.e:"
  echo "      dfs     points to $OLD_DFS"
  echo "      dfs.old points to $DFS"

  echo ""
  if ask_yn "Do you want to roll-back [y/n] (n)?" n ; then
    echo ""
    echo "Rolling-back then:"
    echo  rm -f dfs.tmp
    rm -f dfs.tmp
    echo mv dfs dfs.tmp
    mv dfs dfs.tmp
    echo mv dfs.old dfs
    mv dfs.old dfs
    echo mv dfs.tmp dfs.old
    mv dfs.tmp dfs.old
    break
  else
    echo ""
    echo "No roll-back will be executed"
    break
  fi
done

OLD_DFS=`ls -l dfs.old | awk -F '>' '{printf $2}'`
DFS=`ls -l dfs | awk -F '>' '{printf $2}'`
echo ""
echo "Final settings:"
echo "      dfs     points to $DFS"
echo "      dfs.old points to $OLD_DFS"
echo ""
echo "Bye"
exit 0
