/// -*-c++-*-

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Christophe Moins- ESO/SDD/DFS
// SUBSYSTEM: arcsys - Archive Systems
//
// $Revision: 1.2 $
// $Log: ARSystemService.h,v $
// Revision 1.2  2007/10/25 09:07:09  cmoins
// adding of a constructor whose parameters are the database server name,
// the user name and password, the database name
//  CVS: ----------------------------------------------------------------------
//
// Revision 1.1  2007/10/12 12:29:35  cmoins
// service access layer to ARSystem
//
// 
//------------------------------------------------------------------------------

#ifndef ARSYSTEM_SERVICE_H
#define ARSYSTEM_SERVICE_H

#include <string>
#include "PANL2NightObs.h"
#include "PANL2NightObsDAOSyb.h"
#include "ServiceConnectionSyb.h"

/// Data Access Service layer to the ARSystem database
class ARSystemService
{
public:

  /// \brief Constructor
  /// \param connAlias       data repository connection keyword
  ARSystemService(std::string connAlias);

  /// \brief Constructor
  /**
   * \param dbServer DB server name
   * \param dbUser   DB user name
   * \param dbPasswd DB password
   * \param dbName   DB name
   */
  ARSystemService(std::string dbServer,
		  std::string dbUser,
		  std::string dbPasswd,
		  std::string dbName);

  /// \brief Destructor
  ~ARSystemService(){};

  /// \brief finds the observation conditions associated to an observation block
  /**
   * \param obsId observation block id
   * \param rnightObsSet set of observation night conditions
   * \return the number of night condition records if successful, -1 otherwise
   */
  int findPANL2NightObsByObsId(int obsId,
                               PANL2NightObs::Container& rnightObsSet) const;

private :

  ///  DAO instance to observation night conditions
  PANL2NightObsDAOSyb nightObsDAO_; 
  /// Database connection handler
  mutable ServiceConnectionSyb dsConnection_;
};
#endif // ARSYSTEM_SERVICE_H
