// -*-c++-*-
#ifndef _ArcDbGen_h_
#define _ArcDbGen_h_

/*
 * E.S.O. - VLT project/ESO Archive
 *
 * $Id: ArcDbGen.h,v 1.4 2005/10/17 13:18:59 nferguso Exp $
 *
 * ArcDbGen.h - class definitions for class ArcDbGen, a generic class 
 * representing an application that read/write from/to the database
 *
 * who                  when         what
 * -------------------  -----------  --------------------------------------
 * Elisabetta Angeloni  26 Jan 1998  Created
 * Neil Ferguson        20 Oct 2004  Converted to DfsStdLib/SybConn
 * Neil Ferguson        20 Sep 2005  Added dbAlias_ and related changes 
 */

#include <stdarg.h>
#include <ArcDbConnect.h>
#include <sybconnection.h>

// Generic class internal to the ORO library used for storing the data
// for database connection and error messages
// All the *Rep classes derives from this one.
class ArcDbGen {

protected:
    //
    mutable SybaseConnection myDbase_;
    
    //
    DfsCString dbAlias_;
    
    //
    DfsCString errorMsg_;
    
    //
    int dbErrorFlag_;
    
    //
    int debugSql_;
    
    // Default constructor: it supplies a connection to the database.
    // The default values must be read from a configuration file
    ArcDbGen(DfsCString dbAlias);
    
    // Constructor that supplies the connection keywords.
    ArcDbGen(DfsCString dbUser, DfsCString dbPasswd, 
             DfsCString dbServer, DfsCString dbName);
    
    // Constructor that does not make any connection 
    // but requires a valid database object
    ArcDbGen(SybaseConnection& database);
    
    // Init error code and message
    void initError();
    
public:

    // Set the error message in a private field using the printf format
    void setErrorMsg(const char* fmt, ...);
    
    // Set the error database error code (TRUE or FALSE)
    void setErrorFlag(const int code);
    
    //  Return the database error message
    inline DfsCString getErrorMsg() {return errorMsg_;}
    
    //  Return the database error code (TRUE or FALSE)
    int retSybaseCode() const {return dbErrorFlag_;}

    //  Return the database status
    DfsBoolean isValid();

    //  Return a handler to the database object
    SybaseConnection getDbase() const {return myDbase_;}
    
    //  Return a session
    SybaseConnection getSession() const {return myDbase_;}
    
    //  Return the name of the server which was used when connecting to the DB
    DfsCString getDbServer() {return myDbase_.getDb().c_str();}

    //  Return the name of the user which was used when connecting to the DB
    DfsCString getDbUser() {return myDbase_.getUsername().c_str();}
    
    //  Return the password of the user which was used when connecting to the DB
    DfsCString getDbPasswd() {return myDbase_.getPassword().c_str();}

	//  Return the alias which was used when connecting to the DB
	DfsCString getDbAlias() {return dbAlias_;}

}; // end of definition of class ArcDbGen

#endif /* _ArcDbGen_h_ */
