// -*-c++-*-
#ifndef _ArcMeteo_h_
#define _ArcMeteo_h_

/*
 * E.S.O. - VLT project/ESO Archive
 *
 * $Id: ArcMeteo.h,v 1.2 2005/07/22 18:51:20 nferguso Exp $
 *
 * ArcMeteo.h - class definitions for class ArcMeteo, a class 
 * representing an application that write in the
 * table meteo_<site> of ambient repository database
 *
 * who                  when         what
 * -------------------  -----------  ----------------------------------------
 * Elisabetta Angeloni  08 Jul 1998  Created
 * Neil Ferguson        28 Oct 2004  Converted to DfsStdLib/SybConn
 */

#include <ArcTime.h>

const int ARC_METEO_NULL = -0x7fffffff; // MAXINT

/**
 * This class is used for storing the seeing data retrieved 
 * from the PAF files in the database table meteo_<site>; 
 */
class ArcMeteo {

private:
    
    int date_;
    int interval_;
    
    int temp1_;
    int temp2_;
    int temp3_;
    int tempdew1_;
    int tempdew2_;
    
    int rhum1_;
    int rhum2_;
   
    int pres_;
    int presqnh_;
    
    int winddir1_;
    int winddir2_;
    int windspeed1_;
    int windspeed2_;
    int windspeedu_;
    int windspeedv_;
    int windspeedw_;
    
    int dusts1_;
    int dusts2_;
    int dustl1_;
    int dustl2_;

public:

/**@name Constructors
 */
//@{
    /// Default constructor
    ArcMeteo();
    
//@}

    void clear();
    DfsBoolean isValid() const;
    DfsCString getSQLInsertData() const; 
    
/**@name Members access (Write mode)
 */
//@{

    /// Set the value of the column meteo_<site>.start_date
    // it expects to receive the value of GEN.AMBI.DATE keyword
    inline void AMBI_DATE(DfsCString date) {date_ = -stringToUnixTime80(date);}

    /// Set the value of the column meteo_<site>.interval
    // it expects to receive the value of GEN.AMBI.INTERVAL keyword
    inline void AMBI_INTERVAL(double d) {interval_ = int(rint(d));}

    /// Set the value of the column meteo_<site>.t1
    // it expects to receive the value of GEN.AMBI.TEMP1 keyword
    inline void AMBI_TEMP1(double d) {temp1_ = int(rint(d * 100.));}

    /// Set the value of the column meteo_<site>.t2
    // it expects to receive the value of GEN.AMBI.TEMP2 keyword
    inline void AMBI_TEMP2(double d) {temp2_ = int(rint(d * 100.));}

    /// Set the value of the column meteo_<site>.t3
    // it expects to receive the value of GEN.AMBI.TEMP3 keyword
    inline void AMBI_TEMP3(double d) {temp3_ = int(rint(d * 100.));}

    /// Set the value of the column meteo_<site>.td1
    // it expects to receive the value of GEN.AMBI.TEMPDEW1 keyword
    inline void AMBI_TEMPDEW1(double d) {tempdew1_ = int(rint(d * 100.));}

    /// Set the value of the column meteo_<site>.td2
    // it expects to receive the value of GEN.AMBI.TEMPDEW2 keyword
    inline void AMBI_TEMPDEW2(double d) {tempdew2_ = int(rint(d * 100.));}

    /// Set the value of the column meteo_<site>.rh1
    // it expects to receive the value of GEN.AMBI.RHUM1 keyword
    inline void AMBI_RHUM1(double d) {rhum1_ = int(rint(d));}
    
    /// Set the value of the column meteo_<site>.rh21
    // it expects to receive the value of GEN.AMBI.RHUM2 keyword
    inline void AMBI_RHUM2(double d) {rhum2_ = int(rint(d));}
    
    /// Set the value of the column meteo_<site>.pr
    // it expects to receive the value of GEN.AMBI.PRES keyword
    inline void AMBI_PRES(double d) {pres_ = int(rint(d * 100.));}

    /// Set the value of the column meteo_<site>.prq
    // it expects to receive the value of GEN.AMBI.PRESQNH keyword
    inline void AMBI_PRESQNH(double d) {presqnh_ = int(rint(d * 100.));}

    /// Set the value of the column meteo_<site>.wd1
    // it expects to receive the value of GEN.AMBI.WINDDIR1 keyword
    inline void AMBI_WIND_DIR1(double d) {winddir1_ = int(rint(d));}
    
    /// Set the value of the column meteo_<site>.wd2
    // it expects to receive the value of GEN.AMBI.WIND.DIR2 keyword
    inline void AMBI_WIND_DIR2(double d) {winddir2_ = int(rint(d));}
    
    /// Set the value of the column meteo_<site>.ws1
    // it expects to receive the value of GEN.AMBI.WIND.SPEED1 keyword
    inline void AMBI_WIND_SPEED1(double d) {windspeed1_ = int(rint(d * 100.));}

    /// Set the value of the column meteo_<site>.ws1
    // it expects to receive the value of GEN.AMBI.WIND.SPEED2 keyword
    inline void AMBI_WIND_SPEED2(double d) {windspeed2_ = int(rint(d * 100.));}

    /// Set the value of the column meteo_<site>.wsu
    // it expects to receive the value of GEN.AMBI.WIND.SPEEDU keyword
    inline void AMBI_WIND_SPEEDU(double d) {windspeedu_ = int(rint(d * 100.));}

    /// Set the value of the column meteo_<site>.wsv
    // it expects to receive the value of GEN.AMBI.WIND.SPEEDV keyword
    inline void AMBI_WIND_SPEEDV(double d) {windspeedv_ = int(rint(d * 100.));}

    /// Set the value of the column meteo_<site>.wsw
    // it expects to receive the value of GEN.AMBI.WIND.SPEEDW keyword
    inline void AMBI_WIND_SPEEDW(double d) {windspeedw_ = int(rint(d * 100.));}

    /// Set the value of the column meteo_<site>.dus1
    // it expects to receive the value of GEN.AMBI.DUSTS1 keyword
    inline void AMBI_DUSTS1(int i) {dusts1_ = i;}
    
    /// Set the value of the column meteo_<site>.dus2
    // it expects to receive the value of GEN.AMBI.DUSTS2 keyword
    inline void AMBI_DUSTS2(int i) {dusts2_ = i;}
    
    /// Set the value of the column meteo_<site>.dul1
    // it expects to receive the value of GEN.AMBI.DUSTL1 keyword
    inline void AMBI_DUSTL1(int i) {dustl1_ = i;}
    
    /// Set the value of the column meteo_<site>.dul2
    // it expects to receive the value of GEN.AMBI.DUSTL2 keyword
    inline void AMBI_DUSTL2(int i) {dustl2_ = i;}
    
//@}

}; // end of definition of class ArcTemplate

inline ArcMeteo::ArcMeteo() 
    : date_(0), 
      interval_(ARC_METEO_NULL),
      temp1_(ARC_METEO_NULL),
      temp2_(ARC_METEO_NULL),
      temp3_(ARC_METEO_NULL),
      tempdew1_(ARC_METEO_NULL),
      tempdew2_(ARC_METEO_NULL),
      rhum1_(ARC_METEO_NULL),
      rhum2_(ARC_METEO_NULL),
      pres_(ARC_METEO_NULL),
      presqnh_(ARC_METEO_NULL),
      winddir1_(ARC_METEO_NULL),
      winddir2_(ARC_METEO_NULL),
      windspeed1_(ARC_METEO_NULL),
      windspeed2_(ARC_METEO_NULL),
      windspeedu_(ARC_METEO_NULL),
      windspeedv_(ARC_METEO_NULL),
      windspeedw_(ARC_METEO_NULL),
      dusts1_(ARC_METEO_NULL),
      dusts2_(ARC_METEO_NULL),
      dustl1_(ARC_METEO_NULL),
      dustl2_(ARC_METEO_NULL)
{ ; }

inline void ArcMeteo::clear()
{ 
    date_       = 0; 
    interval_   = ARC_METEO_NULL;
    temp1_      = ARC_METEO_NULL;
    temp2_      = ARC_METEO_NULL;
    temp3_      = ARC_METEO_NULL;
    tempdew1_   = ARC_METEO_NULL;
    tempdew2_   = ARC_METEO_NULL;
    rhum1_      = ARC_METEO_NULL;
    rhum2_      = ARC_METEO_NULL;
    pres_       = ARC_METEO_NULL;
    presqnh_    = ARC_METEO_NULL;
    winddir1_   = ARC_METEO_NULL;
    winddir2_   = ARC_METEO_NULL;
    windspeed1_ = ARC_METEO_NULL;
    windspeed2_ = ARC_METEO_NULL;
    windspeedu_ = ARC_METEO_NULL;
    windspeedv_ = ARC_METEO_NULL;
    windspeedw_ = ARC_METEO_NULL;
    dusts1_     = ARC_METEO_NULL;
    dusts2_     = ARC_METEO_NULL;
    dustl1_     = ARC_METEO_NULL;
    dustl2_     = ARC_METEO_NULL;
}

inline DfsBoolean ArcMeteo::isValid() const
{ 
    if(date_ == 0)
	return FALSE;
    else
	return TRUE; 
}

inline DfsCString ArcMeteo::getSQLInsertData() const 
{
    // The start date is always present. 
    DfsCString columns("(start_date");
    DfsCString values(") VALUES (" + DfsNumConvert::asString(date_)); 

    if(interval_ != ARC_METEO_NULL)
    {
        columns += ", interval";
        values  += ", " + DfsNumConvert::asString(interval_);
    }

    if(temp1_ != ARC_METEO_NULL)
    {
        columns += ", t1";
        values  += ", " + DfsNumConvert::asString(temp1_);
    }

    if(temp2_ != ARC_METEO_NULL)
    {
        columns += ", t2";
        values  += ", " + DfsNumConvert::asString(temp2_);
    }

    if(temp3_ != ARC_METEO_NULL)
    {
        columns += ", t3";
        values  += ", " + DfsNumConvert::asString(temp3_);
    }

    if(tempdew1_ != ARC_METEO_NULL)
    {
        columns += ", td1";
        values  += ", " + DfsNumConvert::asString(tempdew1_);
    }

    if(tempdew2_ != ARC_METEO_NULL)
    {
        columns += ", td2";
        values  += ", " + DfsNumConvert::asString(tempdew2_);
    }

    if(rhum1_ != ARC_METEO_NULL)
    {
        columns += ", rh1";
        values  += ", " + DfsNumConvert::asString(rhum1_);
    }

    if(rhum2_ != ARC_METEO_NULL)
    {
        columns += ", rh2";
        values  += ", " + DfsNumConvert::asString(rhum2_);
    }

    if(pres_ != ARC_METEO_NULL)
    {
        columns += ", pr";
        values  += ", " + DfsNumConvert::asString(pres_);
    }

    if(presqnh_ != ARC_METEO_NULL)
    {
        columns += ", prq";
        values  += ", " + DfsNumConvert::asString(presqnh_);
    }

    if(winddir1_ != ARC_METEO_NULL)
    {
        columns += ", wd1";
        values  += ", " + DfsNumConvert::asString(winddir1_);
    }

    if(winddir2_ != ARC_METEO_NULL)
    {
        columns += ", wd2";
        values  += ", " + DfsNumConvert::asString(winddir2_);
    }

    if(windspeed1_ != ARC_METEO_NULL)
    {
        columns += ", ws1";
        values  += ", " + DfsNumConvert::asString(windspeed1_);
    }

    if(windspeed2_ != ARC_METEO_NULL)
    {
        columns += ", ws2";
        values  += ", " + DfsNumConvert::asString(windspeed2_);
    }

    if(windspeedu_ != ARC_METEO_NULL)
    {
        columns += ", wsu";
        values  += ", " + DfsNumConvert::asString(windspeedu_);
    }

    if(windspeedv_ != ARC_METEO_NULL)
    {
        columns += ", wsv";
        values  += ", " + DfsNumConvert::asString(windspeedv_);
    }

    if(windspeedw_ != ARC_METEO_NULL)
    {
        columns += ", wsw";
        values  += ", " + DfsNumConvert::asString(windspeedw_);
    }

    if(dusts1_ != ARC_METEO_NULL)
    {
        columns += ", dus1";
        values  += ", " + DfsNumConvert::asString(dusts1_);
    }

    if(dusts2_ != ARC_METEO_NULL)
    {
        columns += ", dus2";
        values  += ", " + DfsNumConvert::asString(dusts2_);
    }

    if(dustl1_ != ARC_METEO_NULL)
    {
        columns += ", dul1";
        values  += ", " + DfsNumConvert::asString(dustl1_);
    }

    if(dustl2_ != ARC_METEO_NULL)
    {
        columns += ", dul2";
        values  += ", " + DfsNumConvert::asString(dustl2_);
    }

    return(columns + values + ")"); 
}

#endif /* _ArcMeteo_h_ */
