// -*-c++-*-
#ifndef _ArcObLog_h_
#define _ArcObLog_h_

/*
 * E.S.O. - VLT project/ESO Archive
 *
 * $Id: ArcObLog.h,v 1.3 2005/08/18 17:03:27 nferguso Exp $
 *
 * ArcObLog.h - class definitions for class ArcObLog, a class 
 * for storing data in the ob_event_log table of obrep 
 *
 * who                  when         what
 * -------------------  -----------  ----------------------------------------
 * Elisabetta Angeloni  26 Jan 1998  Created
 * Neil Ferguson        01 Nov 2004  Converted to DfsStdLib/SybConn
 */

/*
 * This class is used by the ArcObRep class for logging events in the table
 ob_event_log of the obrep database.
*/
class ArcObLog {
    
private:

    // Observation Block ID (column "ob_id")
    int obsId_;

    // Time stamp (column "event_timestamp")
    int timestamp_;

    // Event to be recorded (column "event_type")
    DfsCString event_;

    // Information 1 (column "event_info1")
    DfsCString info1_;

    // Information 2 (column "event_info2")
    DfsCString info2_;

public:

/* 
 * Default constructor.
 */
    ArcObLog(int obsId=0);

    // Members access (Write mode)
    
    // Set the value to be stored in the column ob_event_log.event
    inline void event(DfsCString value) {event_ = value;}
   
    // Set the value to be stored in the column ob_event_log.info1
    inline void info1(DfsCString value) {info1_ = value;}
   
    // Set the value to be stored in the column ob_event_log.event
    inline void info2(DfsCString value) {info2_ = value;}

    /// @brief Returns a string in the format "(column_names) VALUES 
    /// (column_values)" holding this ArcObLog object's contents, 
    /// ready for use as part of an SQL INSERT command. 
    DfsCString getSQLInsertData() const;
   
}; // end of definition of class ArcObLog

inline ArcObLog::ArcObLog(int obsId) 
    : obsId_(obsId), 
      event_(""), 
      info1_(""),
      info2_("")
{
    // Init the date-time fields with the current UT time
    struct timeval tv;
    struct timezone tz;
    gettimeofday(&tv, &tz);

    timestamp_ = tv.tv_sec;
}

// Returns a string in the format "(column_names) VALUES (column_values)" 
// holding this ArcObLog object's contents, ready for use as part of an 
// SQL INSERT command. 
inline DfsCString ArcObLog::getSQLInsertData() const 
{
    DfsCString columns("(ob_id");
    DfsCString values(") VALUES (" + DfsNumConvert::asString(obsId_)); 

    columns += ", event_timestamp";
    values  += ", " + DfsNumConvert::asString(timestamp_);
    
    columns += ", event_type";
    values  += ", " + SybUtils::quoteForSQL(event_.asSTLString());
    
    columns += ", event_info1";
    values  += ", " + SybUtils::quoteForSQL(info1_.asSTLString());
    
    columns += ", event_info2";
    values  += ", " + SybUtils::quoteForSQL(info2_.asSTLString());
    
    return(columns + values + ")"); 
}

#endif /* _ArcObLog_h_ */
