// -*-c++-*-
#ifndef _ArcObsBlock_h_
#define _ArcObsBlock_h_

/*
 * E.S.O. - VLT project/ESO Archive
 *
 * $Id: ArcObsBlock.h,v 1.3 2005/08/18 17:03:27 nferguso Exp $
 *
 * ArcObsBlock.h - class definitions for class ArcObsBlock, a class 
 * representing an application that read/write from/to the
 * tables obs_blocks and constraint_sets of OB repository database
 *
 * who                  when         what
 * -------------------  -----------  ----------------------------------------
 * Elisabetta Angeloni  26 Jan 1998  Created
 * Neil Ferguson        01 Nov 2004  Converted to DfsStdLib/SybConn
 */

#include "DfsCString.h"
#include "DfsTValSlist.h"
#include <sybconnection.h>
#include <ArcQCParam.h>
#include <ArcTime.h>

/**
 * This class is used by the ArcObRep class for storing the data retrieved or
 * to be recorded in the database tables obs_blocks and constraint_sets; 
 * this class contains only methods 
 * for accessing the class's members, none of them access the database fields.
 */
class ArcObsBlock {

private:

    // *** OB parameters (table "obs_block")
    
    // Observation Block ID (column "ob_id")
    int obsId_;
    
    // programme ID (column "run_id")
    DfsCString progId_;
    
    // Template Signature ID (column "ts_id")
    //int templateSigId_;
    
    // User-assigned name of the OB (column "item_name")
    DfsCString obsName_;
    
    // Information related to OB execution in service mode (column "obsComments")
    DfsCString obsComments_;
    
    // OB-related information (column "userComments")
    DfsCString userComments_;
    
    // Observing mode (column "obsMode")
    //DfsCString obsMode_;
    
    // Status of the Observation Block (column "status")
    DfsCString obsStatus_;
    
    // (column "calibrationReq")
    DfsCString calibrationReq_;
    
    // (column "reductionRule")
    //DfsCString reductionRule_;
    
    // Observation Type  (column "type")
    DfsCString obsType_;
    
    // Constraint Set ID (column "cs_id")
    int constraintSetId_;
    
    // Last modification date (column "last_mod_date")
    unsigned long modifDate_;
    
    // *** User Constraint parameters (table "constraint_sets")
    
    /// Seeing or Image quality (column "seeing")
    float seeing_;
    
    /// Sky Transparency (column "sky_transparency")
    DfsCString skyTransparency_;
    
    /// Seeing or Image quality (column "airmass")
    float airMass_;
    
    /// Max Fractional Lunar Illumination (column "fli")
    float fli_;
    
    /// Min allowed angular distance (column "moon_distance")
    int moonDistance_;
    
    DfsBoolean operator== (const ArcObsBlock& t) const;
    
public:
    
    // Used by ArcObRep class
    DfsCString selectItemList(const DfsCString& obs_blocks, 
                              const DfsCString& constr_sets); 
    
/**@name Constructors
 */
//@{
    /// Default constructor
    ArcObsBlock();
    
//@}


/**@name Members access (Read mode)
*/
//@{

    /// Get the value of the column obs_blocks.id
    // Get the value of private member obsId_ 
    inline int getObsId() {return obsId_;}
    
    /// Get the value of the column obs_blocks.run_id
    // Get the value of private member progId_
    inline const DfsCString& getProgId() {return progId_;}
    
    /// Get the value of the column obs_blocks.item_name
    // Get the value of private member obsName_
    inline const DfsCString& getObsName() {return obsName_;}
    
    /// Get the value of the column obs_blocks.obsComments
    // Get the value of private member obsComments_
    inline const DfsCString& getObsComments() {return obsComments_;}
    
    /// Get the value of the column obs_blocks.userComments
    // Get the value of private member userComments_
    inline const DfsCString& getUserComments() {return userComments_;}
    
    /// Get the value of the column obs_blocks.status
    // Get the value of private member obsStatus_
    inline DfsCString getObsStatus() {return obsStatus_;}
    
    /// Get the value of the column obs_blocks.calibrationReq
    // Get the value of private member calibrationReq_
    inline const DfsCString& getCalibrationReq() {return calibrationReq_;}
    
    /// Get the value of the column obs_blocks.type
    // Get the value of the private member obsType_
    inline DfsCString getObsType() {return obsType_;}
    
    /// Get the value of the column obs_blocks.cs_id
    // Get the value of private member constraintSetId_
    inline int getConstraintSetId() {return constraintSetId_;}
    
    /// Get the value of the column obs_blocks.last_mod_date
    // Get the value of the private member modifDate_
    inline MJD_DATE getModifDate() {return UnixTimeToMjd(modifDate_);}

    /// Get the value of the column constraint_sets.<param> 
    // where <param> is of double type (SEEING, AIR_MASS, FLI, MOON_DISTANCE, EXPTIME)
    inline int getParam(const enum QCParamType QCParamType, double& paramValue);
    
    /// Get the value of the column constraint_sets.<param> 
    // where <param> is of double type (SEEING, AIR_MASS, FLI, MOON_DISTANCE, EXPTIME)
    inline int getParam(const enum QCParamType QCParamType, DfsCString& paramValue);
    
//@}

/**@name Members access (Write mode)
*/
//@{

    /// Set the value of the column obs_blocks.id
    // Set the value of private member obsId_
    inline void obsId(int i) {obsId_ = i;}

    /// Set the value of the column obs_blocks.run_id
    // Set the value of private member progId_
    inline void progId(DfsCString id) {progId_ = id;}
    
    /// Set the value of the column obs_blocks.item_name
    // Set the value of private member obsName_
    inline void obsName(DfsCString obsName) {obsName_ = obsName;}
    
    /// Set the value of the column obs_blocks.obsComments
    // Set the value of private member obsComments_
    inline void obsComments(DfsCString obsComments) {obsComments_ = obsComments;}
    
    /// Set the value of the column obs_blocks.userComments
    // Set the value of private member userComments_
    inline void userComments(DfsCString userComments) {userComments_ = userComments;}
    
    /// Set the value of the column obs_blocks.status
    // Set the value of private member obsStatus_
    inline void obsStatus(DfsCString status) {obsStatus_ = status;}
    
    /// Set the value of the column obs_blocks.calibrationReq
    // Set the value of private member calibrationReq_
    inline void calibrationReq(DfsCString req) {calibrationReq_ = req;}
    
    /// Set the value of the column obs_blocks.type
    // Set the value of the private member obsType_
    inline void obsType(DfsCString type) {obsType_ = type;}
    
    /// Set the value of the column obs_blocks.cs_id
    // Set the value of private member constraintSetId_
    inline void constraintSetId(int i) {constraintSetId_ = i;}
    
    /// Set the value of the column obs_blocks.last_mod_date
    // Set the value of the private member modifDate_
    inline void modifDate(unsigned long date) {modifDate_ = date;}
    
    /// Set the value of the column constraint_sets.<param> 
    // where <param> is of double type (SEEING, AIR_MASS, FLI, MOON_DISTANCE, EXPTIME)
    int param(const enum QCParamType QCParamType, const double paramValue); 
    
    /// Set the value of the column constraint_sets.<param> 
    // where <param> is of string type (SKY_TRANSPARENCY)
    int param(const enum QCParamType QCParamType, const DfsCString paramValue); 
    
    /// @brief Set private member values based on the current row of the 
    /// given SybaseResultSet. 
    void storeSQLQueryResult(SybaseResultSet& results); 
//@}
    
    /// Return whether or not the current OB contains a valid constraint set
    DfsBoolean constraintSetIsNull(); 
    
/**
****************************************************************************
 * This method prints out the content of the ArcObsBlock class
 
@memo
 Print out the content of the ArcObsBlock class
@param
output stream (default is standard output)
*/

    std::ostream& print(std::ostream& o) const;

}; // end of definition of class ArcObsBlock


/*
 * Constructor
*/

inline ArcObsBlock::ArcObsBlock() 
    :  obsId_(0), 
       progId_(""),
       //templateSigId_(0),
       obsName_(""),
       obsComments_(""),
       userComments_(""),
       //obsMode_(""),
       obsStatus_(""),
       calibrationReq_(""),
       //reductionRule_(""),
       obsType_(""),
       constraintSetId_(-1),
       modifDate_(0),
       seeing_(0),
       skyTransparency_(""),
       airMass_(0),
       fli_(0),
       moonDistance_(0)
{;}

/*
 * Build the select item list, from the obs_blocks and constraint_sets tables.
 * WARNING: The selection order must match the reading order shown below, 
 *          in storeSQLQueryResult(). 
 */

inline DfsCString ArcObsBlock::selectItemList(const DfsCString& obs_blocks,
                                              const DfsCString& constr_sets)
{
    // Build the following select with outer join: 
    // SELECT obs_blocks.*, constraint_sets.* 
    // FROM obs_blocks, constr_sets 

    DfsCString items(
        obs_blocks + ".ob_id, "
        + obs_blocks + ".run_id, "
        + obs_blocks + ".item_name, "
        + obs_blocks + ".obsComments, "
        + obs_blocks + ".userComments, "
        + obs_blocks + ".status, "
        + obs_blocks + ".calibrationReq, "
        + obs_blocks + ".type, "
        + obs_blocks + ".cs_id, "
        + obs_blocks + ".last_mod_date, "
        + constr_sets + ".seeing, "
        + constr_sets + ".sky_transparency, "
        + constr_sets + ".airmass, "
        + constr_sets + ".fli, "
        + constr_sets + ".moon_distance");
    
    // Since this database does not support SQL-92 syntax,
    // the joining condition must be expressed as part of the 
    // WHERE clause. 
    // See the collection of find() methods
    
    // WARNING: The selection order must match the reading order shown below, 
    //          in storeSQLQueryResult(). 
    
    return items;
}

// Set private member values based on the current row of the given 
// SybaseResultSet. 
inline void ArcObsBlock::storeSQLQueryResult(SybaseResultSet& results)
{
    // Read new data. 
    // WARNING: the order of reading must match the order of selection 
    //          defined above, in selectItemList(). 
    int i = 0; 
    
    obsId_           = results.getLongValue(++i); 
    progId_          = results.getStringValue(++i); 
    obsName_         = results.getStringValue(++i); 
    obsComments_     = results.getStringValue(++i); 
    userComments_    = results.getStringValue(++i); 
    obsStatus_       = results.getStringValue(++i); 
    calibrationReq_  = results.getStringValue(++i); 
    obsType_         = results.getStringValue(++i); 
    constraintSetId_ = results.getLongValue(++i); 
    modifDate_       = results.getLongValue(++i); 
    seeing_          = results.getDoubleValue(++i); 
    skyTransparency_ = results.getStringValue(++i); 
    airMass_         = results.getDoubleValue(++i); 
    fli_             = results.getDoubleValue(++i); 
    moonDistance_    = results.getLongValue(++i); 
}

/*
 * Get the value read from constraint set table of those
 * parameters that are supposed to be of floating-point type
 * These parameteres are stored as integer in the database
 * So a conversion in double is done (divided by 1000)
 */
int ArcObsBlock::getParam(const enum QCParamType QCParamType, 
                                 double& paramValue) 
{
    int retCode = 0;
    switch(QCParamType)
    {
    case SEEING:
	paramValue = seeing_;
	break;
	
    case AIR_MASS:
	paramValue = airMass_;
	break;
	
    case FLI:
	paramValue = fli_;
	break;
	
    case MOON_DISTANCE:
	paramValue = moonDistance_;
	break;
	
    default:
	retCode = -1;
	break;
    }
    
    return retCode;
}

/*
 * Get the value read from constraint set table of those
 * parameters that are supposed to be of string type  
 */
inline int ArcObsBlock::getParam(const enum QCParamType QCParamType, 
                                 DfsCString& paramValue) 
{
    int retCode = 0;
    paramValue = "";
    switch(QCParamType)
    {
    case SKY_TRANSPARENCY:
	paramValue = skyTransparency_;
	break;
	
    default:
	retCode = -1;
	break;
    }
    
    return retCode;
}

/*
 * Set the value of those
 * parameters that are supposed to be of floating-point type  
 * These parameteres are stored as integer in the database
 * So a conversion in double is done (multiplying by 1000)  
 */

inline int ArcObsBlock::param(const enum QCParamType QCParamType, 
                              const double paramValue) 
{
    int retCode = 0;
    int intParamValue = int(paramValue*1000);
    switch(QCParamType)
    {
    case SEEING:
	seeing_ = intParamValue;
	break;
	
    case AIR_MASS:
	airMass_ = intParamValue;
	break;
	
    case FLI:
	fli_ = intParamValue;
	break;
	
    case MOON_DISTANCE:
	moonDistance_ = intParamValue;
	break;
	
    default:
	retCode = -1;
	break;
    }
    return retCode;
}

/*
 * Set the value of those
 * parameters that are supposed to be of string type  
 */
inline int ArcObsBlock::param(const enum QCParamType QCParamType, const DfsCString paramValue) 
{
    int retCode = 0;
    switch(QCParamType)
    {
    case SKY_TRANSPARENCY:
	skyTransparency_ = paramValue;
	break;
	
    default:
	retCode = -1;
	break;
    }
   return retCode;
}

inline DfsBoolean ArcObsBlock::operator== (const ArcObsBlock& t) const
{
    return obsId_ == t.obsId_;
}

inline DfsBoolean ArcObsBlock::constraintSetIsNull() 
{
    if(constraintSetId_ <= 0)
	return TRUE;
    else
	return FALSE;
}

inline std::ostream& ArcObsBlock::print(std::ostream& aStream) const
{
    using std::endl;

    TimeStamp dateToPrint(UnixTimeToMjd(modifDate_));
    aStream << "OBSERVATION BLOCK "                   << endl
            << "ob_id: "          << obsId_           << endl
            << "run_id: "         << progId_          << endl
            << "item_name: "      << obsName_         << endl
            << "obsComments: "    << obsComments_     << endl
            << "userComments: "   << userComments_    << endl
            << "status: "         << obsStatus_       << endl
            << "calibrationReq: " << calibrationReq_  << endl
            << "type: "           << obsType_         << endl
	        << "cs_id: "          << constraintSetId_ << endl
            << "last_mod_date: "  << modifDate_       << endl
            << "last_mod_date: "  << dateToPrint.asString("%x %X") << endl
            << "seeing: "         << seeing_          << endl
            << "sky_transparency: " << skyTransparency_ << endl
            << "airmass: "        << airMass_         << endl
            << "fli: "            << fli_             << endl
            << "moon_distance: "  << moonDistance_    << endl << endl;
    return aStream;
}

/**
****************************************************************************
 * It is used to create an single linked list of ArcObsBlock instances
 * An instance of ArcOBList can be accessed like an array, 
 * by using the operator[]. 
 * The call to the entries() function will return the number
 * of ArcObsBlock instances actually in the list.
 * NOTE: Since instances of ArcObsBlock in the DfsTValSlist instance
 * were created on the heap, they must be destroyed manually.
 * The clear() member function of DfsTValSlist
 * takes care of that task.
@memo
 It is used to create an instance of DfsTValSlist 
 that is a list of ArcObsBlock instances
@see
DfsTValSlist template class
*/

typedef DfsTValSlist<ArcObsBlock> ArcOBList;

#endif /* _ArcObsBlock_h_ */
