// -*-c++-*-
#ifndef _ArcQCReport_h_
#define _ArcQCReport_h_

/*
 * E.S.O. - VLT project/ESO Archive
 *
 * $Id: ArcQCReport.h,v 1.3 2005/08/18 17:03:27 nferguso Exp $
 *
 * ArcQCReport.h - class definitions for class ArcQCReport, a class 
 * representing an application that read/write from/to the
 * tables qc_reports and qcr_parameters of QC report repository database
 *
 * who                  when         what
 * -------------------  -----------  ----------------------------------------
 * Elisabetta Angeloni  26 Jan 1998  Created
 * Neil Ferguson        23 Nov 2004  Converted to DfsStdLib/SybConn
 */

#include "DfsCString.h"
#include "DfsTValSlist.h"
#include <ArcTime.h>
#include <ArcQCParam.h>

/**
 * This class is used by the ArcQcrRep class for storing the data retrieved or
 * to be recorded in the database table qc_reports ; 
 * this class contains only methods 
 * for accessing the class's members, none of them access the database fields.
 */
class ArcQCReport {

private:
    
    // Observation Block ID (column "ob_id")
    int obsId_;
    
    // Creation date (column "created")
    TimeStamp creationDate_;
    
    // Last modification date (column "updated")
    TimeStamp updateDate_;
    
    // Information related to data product (column "qcr_com")
    DfsCString qcrComment_;
    
    // Status of the QC report (column "qcr_st")
    DfsCString qcrStatus_;
    
    // Data product ID (column "dp_id")
    DfsCString dpId_;
    
    /// DO category (column "do_cat")
    DfsCString DOCat_;
    
    /// Information related to data product (column "dp_com")
    DfsCString dpComment_;
    
    // Status of the data product (column "dp_st")
    DfsCString dpStatus_;
    
    // *** Template signature fields
    
    // Template signature name (column "tp_name")
    DfsCString tpName_;
    
    // Information related to the template signature (column "tp_com")
    DfsCString tpComment_;
    
    // Status of the template signature (column "tp_st")
    DfsCString tpStatus_;
    
    DfsBoolean operator== (const ArcQCReport& t) const;
    
public:
    
    class ArcQCParam QCReportParam_;

/**@name Constructors
 */
//@{
/**
****************************************************************************
* Default constructor.
Init the date-time fields with the current UT time
*/
 
    ArcQCReport();

//@}

    // Used by the ArcQcrRep class
    DfsCString selectItemList(const DfsCString& qc_reports,
                              const DfsCString& qcr_param); 


/**@name Members access (Read mode)
 */
//@{

    /// Get the value to be stored in the column qc_reports.ob_id
    // Get the value of private member obsId_ 
    inline int getObsId() {return obsId_;}

    /// Get the value to be stored in the column qc_reports.dp_id
    // Get the value of private member dpId_
    inline const DfsCString& getDpId() {return dpId_;}

    /// Get the value to be stored in the column qc_reports.created
    // Get the value of private member creationDate_ 
    inline MJD_DATE getCreationDate() {return creationDate_.mjd();}
    // inline const TimeStamp getCreationDate() {return creationDate_;}

    /// Get the value to be stored in the column qc_reports.updated
    // Get the value of private member updateDate_ 
    inline MJD_DATE getUpdateDate() {return updateDate_.mjd();}
    // inline const TimeStamp getUpdateDate() {return updateDate_;}

    /// Get the value to be stored in the column qc_reports.qcr_com
    // Get the value of private member qcrComment_ 
    inline const DfsCString getQcrComment() {return qcrComment_;}

    /// Get the value to be stored in the column qc_reports.qcr_st
    // Get the value of private member qcrStatus_ 
    inline const DfsCString getQcrStatus() {return qcrStatus_;}
    
    /// Get the value to be stored in the DO category (column "do_cat")
    inline const DfsCString getDOCat() {return DOCat_;}

    /// Get the value to be stored in the Information related to data product (column "dp_com")
    inline const DfsCString getDpComment() {return dpComment_;}

    /// Get the value to be stored in the Status of the data product (column "dp_st")
    inline const DfsCString getDpStatus() {return dpStatus_;}


    /// get Template signature name (column "tp_name")
    inline const DfsCString getTpName() {return tpName_;}

    /// Get Information related to the template signature (column "tp_com")
    inline const DfsCString getTpComment() {return tpComment_;}

    /// Get Status of the template signature (column "tp_st")
    inline const DfsCString getTpStatus() {return tpStatus_;}

    /// Get the measured value of the column qcr_param.<param> 
    // where <param> is of double type (SEEING, AIR_MASS, FLI, MOON_DISTANCE)
    inline int getParam(const enum QCParamType QCParamType, double& paramValue);
   
    /// Get the value of the column qcr_param.<param> 
    // where <param> is of string type (SKY_TRANSPARENCY)
    inline int getParam(const enum QCParamType QCParamType, DfsCString& paramValue);
   
    /// Get the target value of the column qcr_param.<param>_tgt 
    // where <param> is of double type (SEEING, AIR_MASS, FLI, MOON_DISTANCE)
    inline int getParamTarget(const enum QCParamType QCParamType, double& paramValue);
   
    /// Get the target value of the column qcr_param.<param>_tgt
    // where <param> is of string type (SKY_TRANSPARENCY)
    inline int getParamTarget(const enum QCParamType QCParamType, DfsCString& paramValue);
   
    /// Get the min value of the column qcr_param.<param>_min 
    // where <param> is of double type (SEEING, AIR_MASS, FLI, MOON_DISTANCE)
    inline int getParamMin(const enum QCParamType QCParamType, double& paramValue);
   
    /// Get the min value of the column qcr_param.<param>_min
    // where <param> is of string type (SKY_TRANSPARENCY)
    inline int getParamMin(const enum QCParamType QCParamType, DfsCString& paramValue);
   
    /// Get the max value of the column qcr_param.<param>_max 
    // where <param> is of double type (SEEING, AIR_MASS, FLI, MOON_DISTANCE)
    inline int getParamMax(const enum QCParamType QCParamType, double& paramValue);
   
    /// Get the max value of the column qcr_param.<param>_max
    // where <param> is of string type (SKY_TRANSPARENCY)
    inline int getParamMax(const enum QCParamType QCParamType, DfsCString& paramValue);
   
    /// Get the status value of the column qcr_param.<param>_st 
    inline int getParamStatus(const enum QCParamType QCParamType, DfsCString& paramValue);
   
    /// Get the comment value of the column qcr_param.<param>_com 
    inline int getParamComment(const enum QCParamType QCParamType, DfsCString& paramValue);
      
    /// @brief Returns a string in the format "(column_names) VALUES 
    /// (column_values)" holding this ArcQCReport object's contents, 
    /// ready for use as part of an SQL INSERT command. 
    DfsCString getSQLInsertData() const; 

//@}

/**@name Members access (Write mode)
 */
//@{

    /// Set the value to be stored in the column qc_reports.ob_id
    inline void obsId(int i) {obsId_ = i;}

    /// Set the value to be stored in the column qc_reports.dp_id
    inline void dpId(DfsCString value) {dpId_ = value; QCReportParam_.dpId_ = value;}

    /// Set the value to be stored in the column qc_reports.created
    // Two methods are supported
    inline void creationDate(MJD_DATE date) {creationDate_ = TimeStamp(date);}
    inline void creationDate(TimeStamp date) {creationDate_ = date;}

    /// Set the value to be stored in the column qc_reports.updated
    // Two methods are supported
    inline void updateDate(MJD_DATE date) {updateDate_ = TimeStamp(date);}
    inline void updateDate(TimeStamp date) {updateDate_ = date;}
    
    /// Set the value to be stored in the column qc_reports.qcr_com
    inline void qcrComment(DfsCString value) {qcrComment_ = value;}
    
    /// Set the value to be stored in the column qc_reports.qcr_st
    inline void qcrStatus(DfsCString value) {qcrStatus_ = value;}
    
    /// Set the value to be stored in the DO category (column "qc_reports.do_cat")
    inline void DOCat(DfsCString value) {DOCat_ = value;}
    
    /// Set the value of the Information related to data product (column "qc_reports.dp_com")
    inline void dpComment(DfsCString value) {dpComment_ = value;}
    
    /// Set the value of the Status of the data product (column "qc_reports.dp_st")
    inline void dpStatus(DfsCString value) {dpStatus_ = value;}
    
    /// Template signature name (column "qc_reports.tp_name")
    inline void tpName(DfsCString value) {tpName_ = value;}
    
    /// Information related to the template signature (column "qc_reports.tp_com")
    inline void tpComment(DfsCString value) {tpComment_ = value;}
    
    /// Status of the template signature (column "qc_reports.tp_st")
    inline void tpStatus(DfsCString value) {tpStatus_ = value;}
    
    /// Set a param value of double type (column "qcr_param.<param>")
    // where <param> is of double type (SEEING, AIR_MASS, FLI, MOON_DISTANCE)
    inline int param(const enum QCParamType QCParamType, const double paramValue);
    
    /// Set a param value of string type (column "qcr_param.<param>")
    // where <param> is of string type (SKY_TRANSPARENCY)
    inline int param(const enum QCParamType QCParamType, const DfsCString paramValue);
    
    /// Set a param Target value of double type (column "qcr_param.<param>_tgt")
    // where <param> is of double type (SEEING, AIR_MASS, FLI, MOON_DISTANCE)
    inline int paramTarget(const enum QCParamType QCParamType, const double paramValue);
    
    /// Set a param Target value of string type (column "qcr_param.<param>_tgt")
    // where <param> is of string type (SKY_TRANSPARENCY)
    inline int paramTarget(const enum QCParamType QCParamType, const DfsCString paramValue);
    
    /// Set a param Min value of double type (column "qcr_param.<param>_min")
    // where <param> is of double type (SEEING, AIR_MASS, FLI, MOON_DISTANCE)
    inline int paramMin(const enum QCParamType QCParamType, const double paramValue);
    
    /// Set a param Min value of string type (column "qcr_param.<param>_min")
     // where <param> is of string type (SKY_TRANSPARENCY)
    inline int paramMin(const enum QCParamType QCParamType, const DfsCString paramValue);
    
    /// Set a param Max value of double type (column "qcr_param.<param>_max")
    // where <param> is of double type (SEEING, AIR_MASS, FLI, MOON_DISTANCE)
    inline int paramMax(const enum QCParamType QCParamType, const double paramValue);
    
    /// Set a param Max value of string type (column "qcr_param.<param>_max")
    // where <param> is of string type (SKY_TRANSPARENCY)
    inline int paramMax(const enum QCParamType QCParamType, const DfsCString paramValue);
    
    /// Set a param Status value (column "qcr_param.<param>_st")
    inline int paramStatus(const enum QCParamType QCParamType, const DfsCString paramValue);
    
    /// Set a param Comment value (column "qcr_param.<param>_com")
    inline int paramComment(const enum QCParamType QCParamType, const DfsCString paramValue);
    
    /// @brief Set private member values based on the current row of the 
    /// given SybaseResultSet. 
    void storeSQLQueryResult(SybaseResultSet& results); 

//@}

/**
****************************************************************************
* This method prints out the content of the ArcQCReport class

@memo
Print out the content of the ArcQCReport class
@param
output stream (default is standard output)
*/
    
    std::ostream& print(std::ostream& o) const;

}; // end of definition of class ArcQCReport


/*
 * Constructor
 */
inline ArcQCReport::ArcQCReport() 
    : obsId_(0), 
      dpId_(""), 
      qcrComment_(""),
      qcrStatus_(""),
      DOCat_(""),
      dpStatus_(""),
      dpComment_(""),
      tpName_(""),
      tpStatus_(""),
      tpComment_("")
{
    // This block does not need to do anything - TimeStamps initialise 
    // to the current UT time automatically. 
}

inline DfsBoolean ArcQCReport::operator== (const ArcQCReport& t) const
{
    return dpId_ == t.dpId_;
}

/*
 * Build the select item list from the qc_reports and qcr_param tables. 
 * WARNING: The selection order must match the reading order shown below, 
 *          in storeSQLQueryResult(). 
 */
inline DfsCString ArcQCReport::selectItemList(const DfsCString& qc_reports,
                                              const DfsCString& qcr_param) 
{
    // Build the following select :
    // SELECT qc_reports.*
    // FROM qc_reports
    DfsCString items(
    	qc_reports + ".dp_id, " 
    	+ qc_reports + ".ob_id, " 
    	+ qc_reports + ".created, " 
    	+ qc_reports + ".updated, " 
    	+ qc_reports + ".qcr_st, " 
    	+ qc_reports + ".qcr_com, " 
    	+ qc_reports + ".do_cat, " 
    	+ qc_reports + ".dp_st, " 
    	+ qc_reports + ".dp_com, " 
    	+ qc_reports + ".tp_name, " 
    	+ qc_reports + ".tp_st, " 
    	+ qc_reports + ".tp_com, " 
    	+ qcr_param + ".qcr_dp_id, " 
    	+ qcr_param + ".seeing, " 
    	+ qcr_param + ".seeing_tgt, " 
    	+ qcr_param + ".seeing_min, " 
    	+ qcr_param + ".seeing_max, " 
    	+ qcr_param + ".seeing_st, " 
    	+ qcr_param + ".seeing_com, " 
    	+ qcr_param + ".sky_transparency, " 
    	+ qcr_param + ".sky_transparency_tgt, " 
    	+ qcr_param + ".sky_transparency_min, " 
    	+ qcr_param + ".sky_transparency_max, " 
    	+ qcr_param + ".sky_transparency_st, " 
    	+ qcr_param + ".sky_transparency_com, " 
    	+ qcr_param + ".air_mass, " 
    	+ qcr_param + ".air_mass_tgt, " 
    	+ qcr_param + ".air_mass_min, " 
    	+ qcr_param + ".air_mass_max, " 
    	+ qcr_param + ".air_mass_st, " 
    	+ qcr_param + ".air_mass_com, " 
    	+ qcr_param + ".fli, " 
    	+ qcr_param + ".fli_tgt, " 
    	+ qcr_param + ".fli_min, " 
    	+ qcr_param + ".fli_max, " 
    	+ qcr_param + ".fli_st, " 
    	+ qcr_param + ".fli_com, " 
    	+ qcr_param + ".moon_distance, " 
    	+ qcr_param + ".moon_distance_tgt, " 
    	+ qcr_param + ".moon_distance_min, " 
    	+ qcr_param + ".moon_distance_max, " 
    	+ qcr_param + ".moon_distance_st, " 
    	+ qcr_param + ".moon_distance_com"); 

    // WARNING: The selection order must match the reading order shown below, 
    //          in storeSQLQueryResult(). 
    
    return items;
}

// Set private member values based on the current row of the given 
// SybaseResultSet. 
inline void ArcQCReport::storeSQLQueryResult(SybaseResultSet& results)
{
    // Read new data. 
    // WARNING: the order of reading must match the order of selection 
    //          defined above, in selectItemList(). 
    int i = 0; 
    
    dpId_ = results.getStringValue(++i); 
    obsId_ = results.getLongValue(++i); 
    creationDate_ = UnixTimeToMjd(results.getDateAsTimeTValue(++i));
    updateDate_ = UnixTimeToMjd(results.getDateAsTimeTValue(++i));
    qcrStatus_ = results.getStringValue(++i); 
    qcrComment_ = results.getStringValue(++i); 
    DOCat_ = results.getStringValue(++i); 
    dpStatus_ = results.getStringValue(++i); 
    dpComment_ = results.getStringValue(++i); 
    tpName_ = results.getStringValue(++i); 
    tpStatus_ = results.getStringValue(++i); 
    tpComment_ = results.getStringValue(++i); 
   
    // Read QC parameters
    QCReportParam_.dpId_ = results.getStringValue(++i); 
    
    if(results.isNull(++i)) 
        QCReportParam_.seeing_ = NULL_INT_QC_PARAM;
    else
        QCReportParam_.seeing_ = results.getLongValue(i); 
    
    if(results.isNull(++i)) 
        QCReportParam_.seeingTarget_ = NULL_INT_QC_PARAM;
    else
        QCReportParam_.seeingTarget_ = results.getLongValue(i); 
    
    if(results.isNull(++i)) 
        QCReportParam_.seeingMin_ = NULL_INT_QC_PARAM;
    else
        QCReportParam_.seeingMin_ = results.getLongValue(i); 
    
    if(results.isNull(++i)) 
        QCReportParam_.seeingMax_ = NULL_INT_QC_PARAM;
    else
        QCReportParam_.seeingMax_ = results.getLongValue(i); 
    
    QCReportParam_.seeingStatus_ = results.getStringValue(++i); 
    QCReportParam_.seeingComment_ = results.getStringValue(++i); 
    
    QCReportParam_.skyTransparency_ = results.getStringValue(++i); 
    QCReportParam_.skyTransparencyTarget_ = results.getStringValue(++i); 
    QCReportParam_.skyTransparencyMin_ = results.getStringValue(++i); 
    QCReportParam_.skyTransparencyMax_ = results.getStringValue(++i); 
    QCReportParam_.skyTransparencyStatus_ = results.getStringValue(++i); 
    QCReportParam_.skyTransparencyComment_ = results.getStringValue(++i); 

    if(results.isNull(++i)) 
        QCReportParam_.airMass_ = NULL_INT_QC_PARAM;
    else
        QCReportParam_.airMass_ = results.getLongValue(i); 
    
    if(results.isNull(++i)) 
        QCReportParam_.airMassTarget_ = NULL_INT_QC_PARAM;
    else
        QCReportParam_.airMassTarget_ = results.getLongValue(i); 
    
    if(results.isNull(++i)) 
        QCReportParam_.airMassMin_ = NULL_INT_QC_PARAM;
    else
        QCReportParam_.airMassMin_ = results.getLongValue(i); 
    
    if(results.isNull(++i)) 
        QCReportParam_.airMassMax_ = NULL_INT_QC_PARAM;
    else
        QCReportParam_.airMassMax_ = results.getLongValue(i); 
    
    QCReportParam_.airMassStatus_ = results.getStringValue(++i); 
    QCReportParam_.airMassComment_ = results.getStringValue(++i); 
    
    if(results.isNull(++i)) 
        QCReportParam_.fli_ = NULL_INT_QC_PARAM;
    else
        QCReportParam_.fli_ = results.getLongValue(i); 
    
    if(results.isNull(++i)) 
        QCReportParam_.fliTarget_ = NULL_INT_QC_PARAM;
    else
        QCReportParam_.fliTarget_ = results.getLongValue(i); 
    
    if(results.isNull(++i)) 
        QCReportParam_.fliMin_ = NULL_INT_QC_PARAM;
    else
        QCReportParam_.fliMin_ = results.getLongValue(i); 
    
    if(results.isNull(++i)) 
        QCReportParam_.fliMax_ = NULL_INT_QC_PARAM;
    else
        QCReportParam_.fliMax_ = results.getLongValue(i); 
    
    QCReportParam_.fliStatus_ = results.getStringValue(++i); 
    QCReportParam_.fliComment_ = results.getStringValue(++i); 
    
    if(results.isNull(++i)) 
        QCReportParam_.moonDistance_ = NULL_INT_QC_PARAM;
    else
        QCReportParam_.moonDistance_ = results.getLongValue(i); 
    
    if(results.isNull(++i)) 
        QCReportParam_.moonDistanceTarget_ = NULL_INT_QC_PARAM;
    else
        QCReportParam_.moonDistanceTarget_ = results.getLongValue(i); 
    
    if(results.isNull(++i)) 
        QCReportParam_.moonDistanceMin_ = NULL_INT_QC_PARAM;
    else
        QCReportParam_.moonDistanceMin_ = results.getLongValue(i); 
    
    if(results.isNull(++i)) 
        QCReportParam_.moonDistanceMax_ = NULL_INT_QC_PARAM;
    else
        QCReportParam_.moonDistanceMax_ = results.getLongValue(i); 
    
    QCReportParam_.moonDistanceStatus_ = results.getStringValue(++i); 
    QCReportParam_.moonDistanceComment_ = results.getStringValue(++i); 
}

// Returns a string in the format "(column_names) VALUES (column_values)" 
// holding this ArcQCReport object's contents, ready for use as part of an 
// SQL INSERT command. 
inline DfsCString ArcQCReport::getSQLInsertData() const 
{
    DfsCString columns("(dp_id");
    DfsCString values(") VALUES (" + SybUtils::quoteForSQL(dpId_.asSTLString())); 

    columns += ", ob_id";
    values  += ", " + DfsNumConvert::asString(obsId_);

    columns += ", created";
    values  += ", \'" + creationDate_.asString("%x ") 
               + creationDate_.getTime() + "\'";

    columns += ", updated";
    values  += ", \'" + updateDate_.asString("%x ") 
               + updateDate_.getTime() + "\'";

    columns += ", qcr_st";
    values  += ", " + SybUtils::quoteForSQL(qcrStatus_.asSTLString());

    columns += ", qcr_com";
    values  += ", " + SybUtils::quoteForSQL(qcrComment_.asSTLString());

    columns += ", do_cat";
    values  += ", " + SybUtils::quoteForSQL(DOCat_.asSTLString());

    columns += ", dp_st";
    values  += ", " + SybUtils::quoteForSQL(dpStatus_.asSTLString());

    columns += ", dp_com";
    values  += ", " + SybUtils::quoteForSQL(dpComment_.asSTLString());

    columns += ", tp_name";
    values  += ", " + SybUtils::quoteForSQL(tpName_.asSTLString());

    columns += ", tp_st";
    values  += ", " + SybUtils::quoteForSQL(tpStatus_.asSTLString());

    columns += ", tp_com";
    values  += ", " + SybUtils::quoteForSQL(tpComment_.asSTLString());

    return(columns + values + ")"); 
}

inline std::ostream& ArcQCReport::print(std::ostream& aStream) const
{
    using std::endl; 

    aStream << "QC REPORT "                             << endl
            << "Data product id: "      << dpId_        << endl
            << "OB id: "                << obsId_       << endl
            << "Creation Date: " 
                << creationDate_.asString("%x %X") << endl
            << "Last mod Date: "        
                << updateDate_.asString("%x %X") << endl
            << "QC Report comment: "    << qcrComment_  << endl
            << "QC Report status: "     << qcrStatus_   << endl
            << "DO Category: "          << DOCat_       << endl
            << "Data product comment: " << dpComment_   << endl
            << "Data product status: "  << dpStatus_    << endl
            << "Template name : "       << tpName_      << endl
            << "Template comment: "     << tpComment_   << endl
            << "Template status: "      << tpStatus_    << endl << endl;
   
    QCReportParam_.print(aStream);

    return aStream;
}

/*
 * Get the value read from qcr_param table of those
 * parameters that are supposed to be of floating-point type  
 * These parameteres are stored as integer in the database
 * So a conversion in double is done (divided by 1000)  
 */
inline int ArcQCReport::getParam(const enum QCParamType QCParamType, double& paramValue) 
{
    int retCode = 0;
    int intParamValue = NULL_INT_QC_PARAM;
    switch(QCParamType)
    {
    case SEEING:
	intParamValue = QCReportParam_.seeing_;
	break;
           
    case AIR_MASS:
	intParamValue = QCReportParam_.airMass_;
	break;
           
    case FLI:
	intParamValue = QCReportParam_.fli_;
	break;
           
    case MOON_DISTANCE:
	intParamValue = QCReportParam_.moonDistance_;
	break;
           
    default:
	retCode = 1;
	break;
    }
     
    if(intParamValue != NULL_INT_QC_PARAM)
        paramValue = double(intParamValue)/1000.;
    else
        paramValue = double(intParamValue);
        
    return retCode;
   
}

/*
 * Get the TARGET value read from qcr_param table of those
 * parameters that are supposed to be of floating-point type  
 * These parameteres are stored as integer in the database
 * So a conversion in double is done (divided by 1000)  
 */
inline int ArcQCReport::getParamTarget(const enum QCParamType QCParamType, double& paramValue) 
{
    int retCode = 0;
    int intParamValue = NULL_INT_QC_PARAM;
    switch(QCParamType)
    {
    case SEEING:
	intParamValue = QCReportParam_.seeingTarget_;
	break;
           
    case AIR_MASS:
	intParamValue = QCReportParam_.airMassTarget_;
	break;
           
    case FLI:
	intParamValue = QCReportParam_.fliTarget_;
	break;
           
    case MOON_DISTANCE:
	intParamValue = QCReportParam_.moonDistanceTarget_;
	break;
           
    default:
	retCode = -1;
	break;
    }
     
    if(intParamValue != NULL_INT_QC_PARAM)
        paramValue = double(intParamValue)/1000.;
    else
        paramValue = double(intParamValue);
        
    return retCode;
   
}

/*
 * Get the MAX value read from qcr_param table of those
 * parameters that are supposed to be of floating-point type  
 * These parameteres are stored as integer in the database
 * So a conversion in double is done (divided by 1000)  
 */
inline int ArcQCReport::getParamMax(const enum QCParamType QCParamType, double& paramValue) 
{
    int retCode = 0;
    int intParamValue = NULL_INT_QC_PARAM;
    switch(QCParamType)
    {
    case SEEING:
	intParamValue = QCReportParam_.seeingMax_;
	break;
           
    case AIR_MASS:
	intParamValue = QCReportParam_.airMassMax_;
	break;
           
    case FLI:
	intParamValue = QCReportParam_.fliMax_;
	break;
           
    case MOON_DISTANCE:
	intParamValue = QCReportParam_.moonDistanceMax_;
	break;
           
    default:
	retCode = -1;
	break;
    }
     
    if(intParamValue != NULL_INT_QC_PARAM)
        paramValue = double(intParamValue)/1000.;
    else
        paramValue = double(intParamValue);
        
    return retCode;
   
}

/*
 * Get the MIN value read from qcr_param table of those
 * parameters that are supposed to be of floating-point type  
 * These parameteres are stored as integer in the database
 * So a conversion in double is done (divided by 1000)  
 */
inline int ArcQCReport::getParamMin(const enum QCParamType QCParamType, double& paramValue) 
{
    int retCode = 0;
    int intParamValue = NULL_INT_QC_PARAM;
    switch(QCParamType)
    {
    case SEEING:
	intParamValue = QCReportParam_.seeingMin_;
	break;
           
    case AIR_MASS:
	intParamValue = QCReportParam_.airMassMin_;
	break;
           
    case FLI:
	intParamValue = QCReportParam_.fliMin_;
	break;
           
    case MOON_DISTANCE:
	intParamValue = QCReportParam_.moonDistanceMin_;
	break;
           
    default:
	retCode = -1;
	break;
    }
     
    if(intParamValue != NULL_INT_QC_PARAM)
        paramValue = double(intParamValue)/1000.;
    else
        paramValue = double(intParamValue);
        
    return retCode;
   
}


/*
 * Get the value read from qcr_param table of those
 * parameters that are supposed to be of string type  
 */
inline int ArcQCReport::getParam(const enum QCParamType QCParamType, DfsCString& paramValue) 
{
    int retCode = 0;
    paramValue = "";
    switch(QCParamType)
    {
    case SKY_TRANSPARENCY:
	paramValue = QCReportParam_.skyTransparency_;
	break;
           
    default:
	retCode = -1;
	break;
    }

    return retCode;
   
}
 
/*
 * Get the TARGET value read from qcr_param table of those
 * parameters that are supposed to be of string type  
 */
inline int ArcQCReport::getParamTarget(const enum QCParamType QCParamType, DfsCString& paramValue) 
{
    int retCode = 0;
    paramValue = "";
    switch(QCParamType)
    {
    case SKY_TRANSPARENCY:
	paramValue = QCReportParam_.skyTransparencyTarget_;
	break;
           
    default:
	retCode = -1;
	break;
    }

    return retCode;
   
}

/*
 * Get the MAX value read from qcr_param table of those
 * parameters that are supposed to be of string type  
 */
inline int ArcQCReport::getParamMax(const enum QCParamType QCParamType, DfsCString& paramValue) 
{
    int retCode = 0;
    paramValue = "";
    switch(QCParamType)
    {
    case SKY_TRANSPARENCY:
	paramValue = QCReportParam_.skyTransparencyMax_;
	break;
           
    default:
	retCode = -1;
	break;
    }

    return retCode;
   
}

/*
 * Get the MIN value read from qcr_param table of those
 * parameters that are supposed to be of string type  
 */
inline int ArcQCReport::getParamMin(const enum QCParamType QCParamType, DfsCString& paramValue) 
{
    int retCode = 0;
    paramValue = "";
    switch(QCParamType)
    {
    case SKY_TRANSPARENCY:
	paramValue = QCReportParam_.skyTransparencyMin_;
	break;
           
    default:
	retCode = -1;
	break;
    }

    return retCode;
   
}

/*
 * Get the STATUS value read from qcr_param table
 * The status is of string type for all the parameters
 */
inline int ArcQCReport::getParamStatus(const enum QCParamType QCParamType, DfsCString& paramValue) 
{
    int retCode = 0;
    paramValue = "";
    switch(QCParamType)
    {
    case SEEING:
	paramValue = QCReportParam_.seeingStatus_;
	break;
           
    case AIR_MASS:
	paramValue = QCReportParam_.airMassStatus_;
	break;
           
    case FLI:
	paramValue = QCReportParam_.fliStatus_;
	break;
           
    case MOON_DISTANCE:
	paramValue = QCReportParam_.moonDistanceStatus_;
	break;
           
    case SKY_TRANSPARENCY:
	paramValue = QCReportParam_.skyTransparencyStatus_;
	break;
           
    default:
	retCode = -1;
	break;
    }

    return retCode;
   
}

/*
 * Get the COMMENT value read from qcr_param table
 * The status is of string type for all the parameters
 */
inline int ArcQCReport::getParamComment(const enum QCParamType QCParamType, DfsCString& paramValue) 
{
    int retCode = 0;
    paramValue = "";
    switch(QCParamType)
    {
    case SEEING:
	paramValue = QCReportParam_.seeingComment_;
	break;
           
    case AIR_MASS:
	paramValue = QCReportParam_.airMassComment_;
	break;
           
    case FLI:
	paramValue = QCReportParam_.fliComment_;
	break;
           
    case MOON_DISTANCE:
	paramValue = QCReportParam_.moonDistanceComment_;
	break;
           
    case SKY_TRANSPARENCY:
	paramValue = QCReportParam_.skyTransparencyComment_;
	break;
           
    default:
	retCode = -1;
	break;
    }

    return retCode;
   
}

/*
 * Set the value of those
 * parameters that are supposed to be of floating-point type  
 * These parameteres are stored as integer in the database
 * So a conversion in double is done (multiplying by 1000)  
 */
inline int ArcQCReport::param(const enum QCParamType QCParamType, const double paramValue) 
{
    int retCode = 0;
    int intParamValue = int(paramValue*1000);
    switch(QCParamType)
    {
    case SEEING:
	QCReportParam_.seeing_ = intParamValue;
	break;
           
    case AIR_MASS:
	QCReportParam_.airMass_ = intParamValue;
	break;
           
    case FLI:
	QCReportParam_.fli_ = intParamValue;
	break;
           
    case MOON_DISTANCE:
	QCReportParam_.moonDistance_ = intParamValue;
	break;
           
    default:
	retCode = -1;
	break;
    }
     
    return retCode;
   
}

/*
 * Set the value of those
 * parameters that are supposed to be of string type  
 */
inline int ArcQCReport::param(const enum QCParamType QCParamType, const DfsCString paramValue) 
{
    int retCode = 0;
    switch(QCParamType)
    {
    case SKY_TRANSPARENCY:
	QCReportParam_.skyTransparency_ = paramValue;
	break;
           
    default:
	retCode = -1;
	break;
    }
     
    return retCode;
   
}


/*
 * Set the TARGET value of those
 * parameters that are supposed to be of floating-point type  
 * These parameters are stored as integer in the database
 * So a conversion in double is done (multiplying by 1000)  
 */
inline int ArcQCReport::paramTarget(const enum QCParamType QCParamType, const double paramValue) 
{
    int retCode = 0;
    int intParamValue = int(paramValue*1000);
    switch(QCParamType)
    {
    case SEEING:
	QCReportParam_.seeingTarget_ = intParamValue;
	break;
           
    case AIR_MASS:
	QCReportParam_.airMassTarget_ = intParamValue;
	break;
           
    case FLI:
	QCReportParam_.fliTarget_ = intParamValue;
	break;
           
    case MOON_DISTANCE:
	QCReportParam_.moonDistanceTarget_ = intParamValue;
	break;
           
    default:
	retCode = -1;
	break;
    }
     
    return retCode;
   
}

/*
 * Set the TARGET value of those
 * parameters that are supposed to be of string type  
 */
inline int ArcQCReport::paramTarget(const enum QCParamType QCParamType, const DfsCString paramValue) 
{
    int retCode = 0;
    switch(QCParamType)
    {
    case SKY_TRANSPARENCY:
	QCReportParam_.skyTransparencyTarget_ = paramValue;
	break;
           
    default:
	retCode = -1;
	break;
    }
     
    return retCode;
   
}

/*
 * Set the MIN value of those
 * parameters that are supposed to be of floating-point type  
 * These parameteres are stored as integer in the database
 * So a conversion in double is done (multiplying by 1000)  
 */
inline int ArcQCReport::paramMin(const enum QCParamType QCParamType, const double paramValue) 
{
    int retCode = 0;
    int intParamValue = int(paramValue*1000);
    switch(QCParamType)
    {
    case SEEING:
	QCReportParam_.seeingMin_ = intParamValue;
	break;
           
    case AIR_MASS:
	QCReportParam_.airMassMin_ = intParamValue;
	break;
           
    case FLI:
	QCReportParam_.fliMin_ = intParamValue;
	break;
           
    case MOON_DISTANCE:
	QCReportParam_.moonDistanceMin_ = intParamValue;
	break;
           
    default:
	retCode = -1;
	break;
    }
     
    return retCode;
   
}

/*
 * Set the MIN value of those
 * parameters that are supposed to be of string type  
 */
inline int ArcQCReport::paramMin(const enum QCParamType QCParamType, const DfsCString paramValue) 
{
    int retCode = 0;
    switch(QCParamType)
    {
    case SKY_TRANSPARENCY:
	QCReportParam_.skyTransparencyMin_ = paramValue;
	break;
           
    default:
	retCode = -1;
	break;
    }
     
    return retCode;
   
}

/*
 * Set the MAX value of those
 * parameters that are supposed to be of floating-point type  
 * These parameteres are stored as integer in the database
 * So a conversion in double is done (multiplying by 1000)  
 */
inline int ArcQCReport::paramMax(const enum QCParamType QCParamType, const double paramValue) 
{
    int retCode = 0;
    int intParamValue = int(paramValue*1000);
    switch(QCParamType)
    {
    case SEEING:
	QCReportParam_.seeingMax_ = intParamValue;
	break;
           
    case AIR_MASS:
	QCReportParam_.airMassMax_ = intParamValue;
	break;
           
    case FLI:
	QCReportParam_.fliMax_ = intParamValue;
	break;
           
    case MOON_DISTANCE:
	QCReportParam_.moonDistanceMax_ = intParamValue;
	break;
           
    default:
	retCode = -1;
	break;
    }
     
    return retCode;
   
}

/*
 * Set the MAX value of those
 * parameters that are supposed to be of string type  
 */
inline int ArcQCReport::paramMax(const enum QCParamType QCParamType, const DfsCString paramValue) 
{
    int retCode = 0;
    switch(QCParamType)
    {
    case SKY_TRANSPARENCY:
	QCReportParam_.skyTransparencyMax_ = paramValue;
	break;
           
    default:
	retCode = -1;
	break;
    }
     
    return retCode;
   
}

/*
 * Set the STATUS value 
 */
inline int ArcQCReport::paramStatus(const enum QCParamType QCParamType, const DfsCString paramValue) 
{
    int retCode = 0;
    switch(QCParamType)
    {
    case SEEING:
	QCReportParam_.seeingStatus_ = paramValue;
	break;
           
    case AIR_MASS:
	QCReportParam_.airMassStatus_ = paramValue;
	break;
           
    case FLI:
	QCReportParam_.fliStatus_ = paramValue;
	break;
           
    case MOON_DISTANCE:
	QCReportParam_.moonDistanceStatus_ = paramValue;
	break;
           
    case SKY_TRANSPARENCY:
	QCReportParam_.skyTransparencyStatus_ = paramValue;
	break;
           
    default:
	retCode = -1;
	break;
    }

    return retCode;
   
}

/*
 * Set the COMMENT value 
 */
inline int ArcQCReport::paramComment(const enum QCParamType QCParamType, const DfsCString paramValue) 
{
    int retCode = 0;
    switch(QCParamType)
    {
    case SEEING:
	QCReportParam_.seeingComment_ = paramValue;
	break;
           
    case AIR_MASS:
	QCReportParam_.airMassComment_ = paramValue;
	break;
           
    case FLI:
	QCReportParam_.fliComment_ = paramValue;
	break;
           
    case MOON_DISTANCE:
	QCReportParam_.moonDistanceComment_ = paramValue;
	break;
           
    case SKY_TRANSPARENCY:
	QCReportParam_.skyTransparencyComment_ = paramValue;
	break;
           
    default:
	retCode = -1;
	break;
    }

    return retCode;
}

/**
****************************************************************************
* It is used to create an instance of DfsTValSlist 
* that is a single linked list of ArcQCReport instances
* An instance of ArcQCReportList can be accessed like an array, 
* by using the operator[]. 
* The call to the entries() function will return the number
* of ArcQCReport instances actually in the list.
* NOTE: Since instances of ArcQCReport in the DfsTValSlist instance
* were created on the heap, they must be destroyed manually.
* The clear() member function of DfsTValSlist
* takes care of that task.
@memo
It is used to create an instance of DfsTValSlist 
that is a list of ArcQCReport instances
@see
DfsTValSlist template class
*/

typedef DfsTValSlist<ArcQCReport> ArcQCReportList;

#endif /* _ArcQCReport_h_ */
