// -*-c++-*-
#ifndef _ArcSeeing_h_
#define _ArcSeeing_h_

/*
 * E.S.O. - VLT project/ESO Archive
 *
 * $Id: ArcSeeing.h,v 1.2 2005/07/22 18:51:20 nferguso Exp $
 *
 * ArcSeeing.h - class definitions for class ArcSeeing, a class 
 * representing an application that write in the
 * table seeing_<site> of ambient repository database
 *
 * who                  when         what
 * -------------------  -----------  ----------------------------------------
 * Elisabetta Angeloni  07 Jul 1998  Created
 * Neil Ferguson        28 Oct 2004  Converted to DfsStdLib/SybConn
 */

#include <ArcTime.h>
#include "DfsNumConvert.h"

const int ARC_SEEING_NULL = -0x7fffffff; // MAXINT

/**
 * This class is used for storing the seeing data retrieved 
 * from the PAF files in the database table seeing_<site>; 
 */
class ArcSeeing {

private:

  // 
    int date_;

  //
    int interval_;
    
    int fwhm_;
    
    int ra_;
    
    int dec_;
    
    int airmass_;
  
    int tau_;
    int rfl_;
    int tet_;
  

public:

/**@name Constructors
*/
//@{
    /// Default constructor
    ArcSeeing();

//@}

    void clear();
    DfsBoolean isValid() const;
    DfsCString getSQLInsertData() const;
/**@name Members access (Write mode)
*/
//@{

    /// Set the value of the column seeing_<site>.start_date
    // it expects to receive the value of GEN.AMBI.DATE keyword
    inline void AMBI_DATE(DfsCString date) {date_ = -stringToUnixTime80(date);}

    /// Set the value of the column seeing_<site>.interval
    // it expects to receive the value of GEN.AMBI.INTERVAL keyword
    inline void AMBI_INTERVAL(double d) {interval_ = int(rint(d));}

    /// Set the value of the column seeing_<site>.fwhm
    // it expects to receive the value of GEN.AMBI.FWHM keyword
    inline void AMBI_FWHM(double d) {fwhm_ = int(rint(d * 100.));}

    /// Set the value of the column seeing_<site>.ra
    // it expects to receive the value of GEN.AMBI.DIMM.RA keyword
    inline void AMBI_DIMM_RA(double d) {ra_ = int(rint(d * 3600000.));}

    /// Set the value of the column seeing_<site>.dec
    // it expects to receive the value of GEN.AMBI.DIMM.DEC keyword
    inline void AMBI_DIMM_DEC(double d) {dec_ = int(rint(d * 3600000.));}

    /// Set the value of the column seeing_<site>.airmass
    // it expects to receive the value of GEN.AMBI.DIMM.AIRMASS keyword
    inline void AMBI_DIMM_AIRMASS(double d) {airmass_ = int(rint(d * 100.));}

    /// Set the value of the column seeing_<site>.tau
    // it expects to receive the value of GEN.AMBI.TAU keyword
    inline void AMBI_TAU(double d) {tau_ = int(rint(d * 1000000.));}

    /// Set the value of the column seeing_<site>.rfl
    // it expects to receive the value of GEN.AMBI.RFL keyword
    inline void AMBI_RFL(double d) {rfl_ = int(rint(d * 100000.));}

    /// Set the value of the column seeing_<site>.tet
    // it expects to receive the value of GEN.AMBI.TET keyword
    inline void AMBI_TET(double d) {tet_ = int(rint(d * 100.));}

//@}

}; // end of definition of class ArcTemplate


inline ArcSeeing::ArcSeeing() 
  : date_(0), 
  interval_(ARC_SEEING_NULL),
  fwhm_(ARC_SEEING_NULL),
  ra_(ARC_SEEING_NULL),
  dec_(ARC_SEEING_NULL),
  airmass_(ARC_SEEING_NULL),
  tau_(ARC_SEEING_NULL),
  rfl_(ARC_SEEING_NULL),
  tet_(ARC_SEEING_NULL)
  
{ ; }

inline void ArcSeeing::clear()
{ 
   date_     = 0; 
   interval_ = ARC_SEEING_NULL;
   fwhm_     = ARC_SEEING_NULL;
   ra_       = ARC_SEEING_NULL;
   dec_      = ARC_SEEING_NULL;
   airmass_  = ARC_SEEING_NULL;
   tau_      = ARC_SEEING_NULL;
   rfl_      = ARC_SEEING_NULL;
   tet_      = ARC_SEEING_NULL;
}

inline DfsBoolean ArcSeeing::isValid() const
{ 
   if(date_ == 0 || fwhm_ == ARC_SEEING_NULL)
      return FALSE;
   else
      return TRUE; 
}

inline DfsCString ArcSeeing::getSQLInsertData() const 
{
    // The start date is always present. 
    DfsCString columns("(start_date");
    DfsCString values(") VALUES (" + DfsNumConvert::asString(date_)); 

    if(interval_ != ARC_SEEING_NULL)
    {
        columns += ", interval";
        values  += ", " + DfsNumConvert::asString(interval_);
    }

    if(fwhm_ != ARC_SEEING_NULL)
    {
        columns += ", fwhm";
        values  += ", " + DfsNumConvert::asString(fwhm_);
    }

    if(ra_ != ARC_SEEING_NULL)
    {
        columns += ", ra";
        values  += ", " + DfsNumConvert::asString(ra_);
    }

    if(dec_ != ARC_SEEING_NULL)
    {
        columns += ", dec";
        values  += ", " + DfsNumConvert::asString(dec_);
    }

    if(airmass_ != ARC_SEEING_NULL)
    {
        columns += ", airmass";
        values  += ", " + DfsNumConvert::asString(airmass_);
    }

    if(tau_ != ARC_SEEING_NULL)
    {
        columns += ", tau";
        values  += ", " + DfsNumConvert::asString(tau_);
    }

    if(rfl_ != ARC_SEEING_NULL)
    {
        columns += ", rfl";
        values  += ", " + DfsNumConvert::asString(rfl_);
    }

    if(tet_ != ARC_SEEING_NULL)
    {
        columns += ", tet";
        values  += ", " + DfsNumConvert::asString(tet_);
    }

    return(columns + values + ")"); 
}


#endif /* _ArcSeeing_h_ */

