// -*-c++-*-

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Christophe Moins- ESO/DMD/DFS
// SUBSYSTEM: arcsys - Archive Systems
//
// $Revision: 1.1 $
// $Log: ConstraintSetDAOSyb.h,v $
// Revision 1.1  2007/09/19 15:32:56  cmoins
// implementation of a DAO  class to handle constraint sets in the database
//
// 
//------------------------------------------------------------------------------

#ifndef CONSTRAINT_SET_DAO_SYB_H
#define CONSTRAINT_SET_DAO_SYB_H

#include "ConstraintSet.h" 

class ServiceConnectionSyb;

/// Sybase Data Access Object to ConstraintSet objects
class ConstraintSetDAOSyb
{
public:
 
  /// \brief Constructor
  /// \param constraintSetTable  ConstraintSet table
  /// \param obsBlockTable       Observation block table
  ConstraintSetDAOSyb( std::string constraintSetTable,
		       std::string observationBlockTable = "NONE");

  /// \brief Retrieves a set of constraints associated to an observation block
  /**
   * \param obsId observation block Id
   * \param constraintSets constraint sets of the observation block
   * \param rconnectionHandle database connection handle
   * \return the number of retrieved objects, or -1 in case of error
   */
  int findByObsId(int obsId, 
		  ConstraintSet::Container& rconstraintSets,
		  const ServiceConnectionSyb& rconnectionHandle) const;
  
private:

  /// Field Names
  static const std::string CS_ID_FIELD; 
  static const std::string USER_ID_FIELD; 
  static const std::string ITEM_NAME_FIELD; 
  static const std::string ITEM_DESCRIPTION_FIELD; 
  static const std::string SEEING_FIELD; 
  static const std::string SKY_TRANSPARENCY_FIELD; 
  static const std::string AIR_MASS_FIELD; 
  static const std::string FLI_FIELD; 
  static const std::string MOON_DISTANCE_FIELD; 
  static const std::string LOCAL_STORAGE_ID_FIELD; 
  static const std::string STREHL_RATIO_FIELD; 
  static const std::string BASELINE_ID_FIELD; 

  /// table storing constraintSet
  std::string constraintSetTable_;
  /// table storing observation blocks used for the join
  std::string obsBlockTable_;

};

#endif // CONSTRAINT_SET_DAO_SYB_H
