//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//COPYRIGHT (c) 1996 European Southern Observatory
//LICENSE
//
//PROJECT:   VLT On-line Data Flow
//AUTHOR:    Michael Wiedmer - ESO/DMD/DPG
//SUBSYSTEM: System Core
//
//$Revision: 1.1 $
//$Log: CorPersistent.h,v $
//Revision 1.1  2004/09/06 14:35:21  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/06/23 12:47:09  cguirao
//First check in
//
//Revision 1.2  1998/06/30 08:49:50  dataflow
//Removing $NAME
//
//Revision 1.1  1997/04/10 08:05:44  dfadmin
//Initial revision
//
//Revision 1.1  1996/11/25 09:35:08  mwiedmer
//first release
//
//-----------------------------------------------------------------------------

#ifndef COR_PERSISTENT_H
#define COR_PERSISTENT_H

// includes from this library
// includes from other ESO libraries
#include <CorBase.h>
#include <CorRunTimeType.h>
// includes from other libraries
// system includes

// typedef, extern, enums
// variables: consts, statics, exported variables (declared extern elsewhere)
// local forward function declarations
class CorUnitTest;
class CorStream;


// helper macro
#define DECLARE_PERSISTENT(className) \
    DECLARE_DYNCREATE(className) \
    friend CorStream& operator>>(CorStream& aStream, className* &anObject);

#define IMPLEMENT_PERSISTENT(className, baseClassName, aVersion) \
    void className::construct(void* aSpace) \
    { \
      new(aSpace) className;\
    } \
    IMPLEMENT_RUNTIMETYPE(className, baseClassName, aVersion, \
                          className::construct) \
    CorStream& operator>>(CorStream& aStream, className*& anObject) \
    { \
      anObject = (className*) aStream.readObject(RUNTIME_TYPE(className)); \
      return aStream; \
    }


/// A simple persistent class 
class CorPersistent : public CorBase {
  DECLARE_DYNAMIC(CorPersistent);
  /**
  {\footnotesize $Id: CorPersistent.h,v 1.1 2004/09/06 14:35:21 cguirao Exp $ } \\

  {\large SYNOPSIS} \\
  \begin{verbatim}
    #include <CorPersistent.h>
  
    CorPersistent myObject;
  \end{verbatim}

  {\large DESCRIPTION} \\
  This text serves as an example for the documentation of a class definition

  {\large CAUTIONS} \\
  This class does not do any pointer swivelling yet. It is only needed for
  other classes as a base, so that the CorStream can actually stream those
  objects.

  {\large EXAMPLES} \\
  <optional> \\

  {\large SEE ALSO} \\
  <optional> \\
  \Ref{CorBase}\\
  */

public:

  /// 
  CorPersistent();
  /** {\large Purpose} \\
  The default c'tor.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */
    

  /// 
  CorPersistent(
    /** {\large Purpose} \\
    The copy c'tor creates a new instance from a reference.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const CorPersistent& aRef
    /** the reference instance */
  );
    

  /// 
  CorPersistent& operator=(
    /** {\large Purpose} \\
    The assignment operator makes this instance look like another one.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    A reference to this instance.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const CorPersistent& aRef
    /** the reference instance */
  );
    

  ///
  virtual ~CorPersistent();
  /** {\large Purpose} \\
  The d'tor.
  
  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  
  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  ///
  CorBool isStreamable() const;
  /** {\large Purpose} \\
  Query if this class can be streamed.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function retutns corTRUE if an this class can be streamed, ie. it
  is not an abstract class.

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


  ///
  CorBool isKindOf(
    /** {\large Purpose} \\
    Query if this instance is of the same type as the supplied type.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Returns}\\
    This function retutns corTRUE if the types match.

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    const CorRunTimeType* aType
    /** type of other class */
  ) const;


  ///
  virtual void stream(
    /** {\large Purpose} \\
    Stream this isntance through a file.

    {\large Preconditions} \\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Exceptions}\\
    \begin{itemize}
    \item Itemize this section if necessary. Remove itemization otherwise.
    \end{itemize}

    {\large Arguments}\\
    */
    ///
    CorStream& aFile
    /** file to stream through */
  );


  ///
  virtual CorPersistent* makeClone() const;
  /** {\large Purpose} \\
  Make a clone of this instance.

  {\large Preconditions} \\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}

  {\large Returns}\\
  This function retutns a pointer to a newly created instance that has the same
  properties as this one.

  {\large Exceptions}\\
  \begin{itemize}
  \item Itemize this section if necessary. Remove itemization otherwise.
  \end{itemize}
  */


protected:


private:
  friend void utest_CorPersistent(CorUnitTest&);  // the unit test function

};

#endif // COR_PERSISTENT_H
