/// -*-c++-*-

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Christophe Moins- ESO/SDD/DFS
// SUBSYSTEM: arcsys - Archive Systems
//
// $Revision: 1.5 $
// $Log: DbcmService.h,v $
// Revision 1.5  2008/02/15 11:11:45  cmoins
// adding of a default constructor
//
// Revision 1.4  2008/01/09 13:51:45  cmoins
// use of dbrc file to retrieve database connection parameters
//
// Revision 1.3  2008/01/08 12:48:20  cmoins
// database parameters become configurable
//
// Revision 1.2  2008/01/04 16:13:14  cmoins
// replacement of database name by database owner name
//
// Revision 1.1  2007/12/14 16:40:22  cmoins
// first version
//
// Revision 1.1  2007/12/07 12:57:20  cmoins
// first version of the header on the fly correction tool
//
// 
//------------------------------------------------------------------------------

#ifndef DBCM_SERVICE_H
#define DBCM_SERVICE_H

#include <string>
#include "KeywordRep.h"
#include "KeywordRepDAO.h"
 
namespace headerOnTheFly {

  /// Data Access Service layer to the ARSystem database
  class DbcmService
  {
  public:

    /// Default constructor
    DbcmService(){}

    /// \brief Constructor
    /**
     * \param dbServer  DB server name
     * \param dbUser    DB user name
     * \param dbPasswd  DB password
     */
    DbcmService(std::string dbServer,
		std::string dbUser,
		std::string dbPasswd);

    /// \brief Destructor
    ~DbcmService(){};

    /// \brief finds keyword data associated to a data product id
    /**
     * \param dpId data product identifier
     * \param rkeywordRepSet set of keyword data associated to the dp id.
     * \return the number of retrieved items if successful, -1 otherwise
     */
    int findKeywordByDpId(std::string dpId,
			  KeywordRep::ResultSet& rkeywordRepSet) const;

  private :

    /// DAO to get Keyword data
    KeywordRepDAO keywordRepDAO_;
    /// Name of the database server
    std::string dbServer_;
    /// Name of the database user
    std::string dbUser_;
    /// Password to connect to the db server
    std::string dbPasswd_;
  };
}
#endif // DBCM_SERVICE_H
