// -*-c++-*-

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) 2003 European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System - RogueWave->dfsStdLib port
// AUTHOR:    Neil Ferguson - ESO/DMD/DFS
// SUBSYSTEM: dfsStdLib RWBoundsErr replacement
//
// $Revision: 1.4 $
// $Log: DfsBoundsErr.h,v $
// Revision 1.4  2004/08/17 15:54:22  nferguso
// Documentation updates.
//
// Revision 1.3  2004/07/29 14:10:54  nferguso
// Added comment noting that the what() method is inherited from DfsXmsg
// (which inherits it from std::exception).
//
// Revision 1.2  2003/12/19 15:03:02  nferguso
//
// String-handling classes can be built and used, although some things remain
// to be done. Most significantly:
//
// - methods working with iostreams are not fully compliant with RogueWave
// - STL string reserve() memory management is incomplete
//
// No non-string RogueWave classes have been replaced yet.
//
// Revision 1.1.1.1  2003/11/28 14:21:16  nferguso
// First Check-In
//
// 
//------------------------------------------------------------------------------

#ifndef DFS_BOUNDS_ERR_H
#define DFS_BOUNDS_ERR_H

// Includes from this library.
#include <DfsXmsg.h>

/// The DfsBoundsErr class is a partial replacement of the RWBoundsErr class. 
/** Only methods required by existing DFS code have been reimplemented, as 
 * new DFS code should be implemented using STL capabilities. 
 *
 * NOTE: The RogueWave RWBoundsErr has a default assignment operator, which 
 * should have been disabled, as it is unsafe. The DfsBoundsErr implementation 
 * allows the default assignment operator to work safely and correctly. 
 *
 * As DfsBoundsErr methods should correspond exactly to the original RogueWave 
 * API, this documentation is sparse. Consult the RogueWave documentation for 
 * more details. 
 */ 
class DfsBoundsErr : public DfsXmsg
{
public:
    /// @brief Constructs a new DfsBoundsErr exception with the given 
    /// @a reason string. 
    DfsBoundsErr(const char* reason) :
        DfsXmsg(reason)
    {
        // This constructor does nothing beyond base class initialisation. 
    }

    // Destructor is default (and virtual, courtesy of DfsXmsg). 

    // The why() method is inherited from DfsXmsg. 

    // The what() method is inherited from DfsXmsg. 

protected:
    // DfsBoundsErr uses the stlString member from DfsXmsg. 

private:
    
};

#endif // DFS_BOUNDS_ERR_H
