// -*-c++-*-

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) 2004 European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System - RogueWave->dfsStdLib port
// AUTHOR:    Neil Ferguson - ESO/DMD/DFS
// SUBSYSTEM: dfsStdLib RWLocale replacement
//
// $Revision: 1.8 $
// $Log: DfsNumConvert.h,v $
// Revision 1.8  2005/07/22 18:48:49  nferguso
// Ported to DfsStdLib / SybConn.
//
// Revision 1.7  2004/09/01 13:51:58  nferguso
// Documentation tweaks.
//
// Revision 1.6  2004/08/17 15:54:22  nferguso
// Documentation updates.
//
// Revision 1.5  2004/08/10 15:56:38  nferguso
// Removed redundant #include directives.
//
// Revision 1.4  2004/08/10 11:27:06  nferguso
// Documented floating point precision limitations which apply to the
// asString(double, int, DfsBoolean) method.
//
// Revision 1.3  2004/04/29 16:16:52  nferguso
// - DfsCString crossreferences added to asString() documentation.
// - Detailed namespace description improved.
//
// Revision 1.2  2004/03/26 15:07:39  nferguso
// Copyright years updated.
//
// Revision 1.1  2004/03/26 09:43:29  nferguso
// First Check-In
//
// 
//------------------------------------------------------------------------------

#ifndef DFS_NUMCONVERT_H
#define DFS_NUMCONVERT_H

// Includes from this library
#include <DfsCString.h>

/// @brief The functions in the DfsNumConvert namespace (include 
/// DfsNumConvert.h) are reimplementations of RWLocale methods required 
/// by the DFS. 
/** The name was changed from RWLocale to DfsNumConvert, so that it would 
 * more accurately reflect the functionality used by the DFS. Only methods 
 * required by existing DFS code have been reimplemented, as new DFS code 
 * should be implemented using STL capabilities. 
 *
 * <B>WARNING:</B> These functions only support the global locale. 
 *
 * As DfsNumConvert functions should correspond exactly to the equivalent 
 * RWLocale methods, this documentation is sparse. Consult the RogueWave 
 * RWLocale documentation for more details. 
 */ 
namespace DfsNumConvert
{
    /// Converts @a number to a DfsCString. 
    DfsCString asString(int number);

    /// Converts @a number to a DfsCString. 
    DfsCString asString(long number);

    /// Converts @a number to a DfsCString. 
    /** The DfsCString returned by this method always contains the number 
     * written in decimal form, as opposed to scientific notation. As a 
     * result, the limited precision of @c double values can cause problems. 
     *
     * <B>WARNING:</B> If any of the following conditions occur, the output 
     * string may contain digits with unexpected values: 
     *
     * - the number requires more digits before the decimal point than double 
     *   precision can provide. 
     * - the number of digits specified by @a precision is larger than double 
     *   precision can provide. 
     *
     * Two problematic examples from a system where @c double values have 64 
     * bits of storage and 15 digits of precision: 
     * 
     * - 1.0E25 with @a precision set to 0 becomes 
     *   "10000000000000000905969664". 
     * - 1.0E-1 with @a precision set to 25 becomes "0.10000000000000001". 
     * 
     * <B>WARNING:</B> for portability reasons, @a precision must never be 
     * negative. Specifying a negative value will produce erroneous results 
     * on HP-UX, due to a defect in the HP C library. 
     */
    DfsCString asString(double number, 
                        int precision = 6, 
                        DfsBoolean showpoint = false);

    /// @brief Converts @a sourceString to a @c double, placing the value in 
    /// @a number. 
    DfsBoolean stringToNum(const DfsCString& sourceString, double* number); 

    /// @brief Converts @a sourceString to a @c long, placing the value in 
    /// @a number. 
    DfsBoolean stringToNum(const DfsCString& sourceString, long* number); 
}

#endif  // DFS_NUMCONVERT_H
