// -*-c++-*-

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) 2004 European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System - RogueWave->dfsStdLib port
// AUTHOR:    Neil Ferguson - ESO/DMD/DFS
// SUBSYSTEM: dfsStdLib utility code
// CI NUMBER: n/a
//
// $Revision: 1.3 $
// $Log: DfsUtils.h,v $
// Revision 1.3  2004/08/17 15:54:22  nferguso
// Documentation updates.
//
// Revision 1.2  2004/03/26 15:07:40  nferguso
// Copyright years updated.
//
// Revision 1.1  2004/02/20 10:17:09  nferguso
// First Check-In:
// - checkIndex utility function.
//
// 
//------------------------------------------------------------------------------

#ifndef DFS_UTILS_H
#define DFS_UTILS_H

// Includes from this library
#include <DfsBoundsErr.h>

namespace DfsStd
{
    /// Throws a DfsBoundsErr exception if the given @a index is out of range. 
    /** A DfsBoundsErr exception will be thrown if @a index is not within the 
     * range [0, @a length). Otherwise, this function returns normally. 
     * 
     * @param index       The index value to check. 
     * @param length      The container/array length that the index is 
     *                    being checked against. 
     * @param specialNPOS @c True if @c DFS_NPOS has special significance, 
     *                    @c false otherwise. 
     */ 
    void checkIndex(size_t index, 
                    size_t length, 
                    bool specialNPOS = false) throw(DfsBoundsErr); 
}

#endif // DFS_UTILS_H
