/// -*-c++-*-

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Christophe Moins- ESO/SDD/DFS
// SUBSYSTEM: arcsys - Archive Systems
//
// $Revision: 1.7 $
// $Log: FitsHandler.h,v $
// Revision 1.7  2008/05/08 11:41:05  cmoins
// comments modification + removal of useless constant
//
// Revision 1.6  2008/03/03 15:27:33  cmoins
// Changing of the command line tool interface
//
// Revision 1.5  2008/02/19 16:57:38  cmoins
// initialization of fptr_ data member to null to avoid
// core dump in th destructor
//
// Revision 1.4  2008/02/15 11:12:23  cmoins
// adding of a help method + option to handle only primary extensions
//
// Revision 1.3  2008/01/09 13:51:45  cmoins
// use of dbrc file to retrieve database connection parameters
//
// Revision 1.2  2008/01/08 12:48:20  cmoins
// database parameters become configurable
//
// Revision 1.1  2007/12/14 16:40:22  cmoins
// first version
//
// Revision 1.1  2007/12/14 15:58:20  cmoins
// first "operation version" of the header of the fly tool
//
//
//
// 
//------------------------------------------------------------------------------


#ifndef FITS_HANDLER_H
#define FITS_HANDLER_H

#include <string>
#include "FrameHandler.h"
#include "fitsio.h"
#include "KeywordRep.h"

namespace headerOnTheFly {

  /**
   *  Implementation of a fits file handler. Its main task will be to 
   *  substitute records inside the header of all the extensions of a fits file
   *  by database records.
   */ 
  class FitsHandler : public  FrameHandler
  {
  public:

    /// Default Constructor 
    FitsHandler() {fptr_ = 0;}

    /**
     * Constructor
     * Opens the fits file to be handled
     * \param fitsFileName name of the fits file to handle
     * \param dbServer database server
     * \param dbUser database user
     * \param dbPasswd user passwd
     * \param onlyPrimExt only primary extension handling flag
     */
    FitsHandler(std::string fitsFileName,
		std::string dbServer,
		std::string dbUser,
		std::string dbPasswd,
		bool onlyPrimExt);

    /**
     * Destructor 
     * Closes the fits file to be handle
     */
    ~FitsHandler();

    /**
     * Substitutes the records of all extensions headers by a set of record
     * coming from the database. It assumes that the database records are 
     * sorted according to their extension and keyword identifiers
     * returns 0 if successful, -1 otherwise
     */
    virtual int substituteExtHeader() ;

    /**
     * returns the successful message of the frame
     */
    virtual std::string getSuccessMessage();

    /**
     * returns the error message of the frame
     */
    virtual std::string getErrorMessage();

  private :

    /// id of the fits file to handle
    std::string fitsFileId_;
    /// pointer to the fits file content
    fitsfile *fptr_;
    /// flag that indicates that only the primary header should be replaced
    bool onlyPrimary_;
  };
}
#endif // FITS_HANDLER_H
