// -*-c++-*-

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Christophe Moins- ESO/SDD/DFS
// SUBSYSTEM: arcsys 
//
// $Revision: 1.3 $
// $Log: Media.h,v $
// Revision 1.3  2007/05/24 11:33:16  cmoins
// adding of mediaAction_ field
//
// Revision 1.2  2007/04/26 12:51:13  cmoins
// doxygen comments modification
//
// Revision 1.1  2007/04/25 15:08:57  cmoins
// creation of a structure implementing a media request
//
// 
//------------------------------------------------------------------------------

#ifndef MEDIA_H
#define MEDIA_H

#include <string>
#include "MediaAction.h"

using namespace std;

/// Implementation of  a media request
/** 
 * Stores information about a request to store data on a media support.
 * Its identifier will be generated automatically. Its name will be given
 * by the user.
 */
struct Media
{ 

  /// Default constructor
  Media()
  {}

  /// Constructor setting the media name, type, number of copies and host.
  Media(string mediaRequestName,
	string mediaType,
	int numCopies,
	string host):
    mediaRequestName_(mediaRequestName),
    mediaType_(mediaType),
    numCopies_(numCopies),
    host_(host)
  {}

  /// Media request identifier automatically generated
  int mediaRequestId_; 
  /// name of the media request
  string mediaRequestName_;  
  string sourceId_;
  /// date the request was performed
  string dateRequested_;   
  /// date the request was fulfilled
  string dateCompleted_;   
  /// indicates if it is a user or archive media
  string mediaRequestType_; 
  /// Type of the media data will be stored onto
  string mediaType_;  
  /// Number of copies that shall be created
  int numCopies_;  
  /// format of the media support name
  string unitNameFormat_; 
  /// "" if being created, "I" in progress, "D" done, "E" error
  char status_;  
  /// Host on which the media request was created
  string host_;
  int incremental_;

  /// action associated to the media request
  MediaAction mediaAction_;
};
#endif /* MEDIA_H */

