// -*-c++-*-

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Christophe Moins- ESO/SDD/DFS
// SUBSYSTEM: arcsys 
//
// $Revision: 1.5 $
// $Log: MediaFile.h,v $
// Revision 1.5  2007/07/25 07:27:40  cmoins
// doxygen comments correction
//
// Revision 1.4  2007/05/21 12:51:01  cmoins
// adding of pconstMediaFile
//
// Revision 1.3  2007/05/15 15:30:08  cmoins
// removal of program id data member
//
// Revision 1.2  2007/04/26 12:51:36  cmoins
// doxygen comments modification
//
// Revision 1.1  2007/04/25 15:10:08  cmoins
// creation of a structure implementing a file associated to a media request
//
// 
//------------------------------------------------------------------------------

#ifndef MEDIA_FILE_H
#define MEDIA_FILE_H

#include <string>
#include <vector>

using namespace std;

/// Implementation of  a file that is or will be stored on a media support
struct MediaFile
{  
  /// set of media files
  typedef vector<MediaFile> ResultSet;
  typedef ResultSet::iterator pMediaFile;
  typedef ResultSet::const_iterator pconstMediaFile;

  /// Constructor using the file identifier
  MediaFile(string fileId):
    fileId_(fileId)
  {}

  /// Constructor setting the main data of a file on a media support
  MediaFile(string fileId,
	    string fileName,
	    int    fileSize,
	    int    uncompFileSize,
	    char   compressionFlag,
	    char   status):
    fileId_(fileId),
    fileName_(fileName),
    fileSize_(fileSize),
    uncompFileSize_(uncompFileSize),
    compressionFlag_(compressionFlag),
    status_(status)
  {}

  /// File identifier
  string fileId_;                
  /// Name of the file on a magnetic disk
  string fileName_;        
  /// id of the media request the file belongs to
  int    mediaRequestId_;  
  /// Name of the media unit the file will go on
  string mediaUnitName_; 
  /// date of the entry in the data repository
  string dateNotified_;    
  string sourceId_; 
  /// size of the file
  int    fileSize_;        
  /// size of the uncompressed file
  int    uncompFileSize_;  
  /// indicated if the file is to be compressed
  char   compressionFlag_; 
  /// "" if created, "R"etrieved, "U"navailable,"E"rror, "D"one
  char   status_; 
};
#endif /* MEDIA_FILE_H */

