// -*-c++-*-

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Jens Knudstrup - ESO/DMD/DFS
// SUBSYSTEM: DFS Archive System
//
// $Name: arcsys-1_2_8 $
// $Revision: 1.13 $
// $Log: Ngas2LocalCopyTask.h,v $
// Revision 1.13  2007/12/07 16:44:43  mpadovan
// The class is now friend of TestNgas2LocalCopyTask (for unit test only).
//
// Revision 1.12  2006/08/11 14:59:47  jknudstr
// -
//
// Revision 1.11  2006/08/08 15:29:22  jknudstr
// -
//
//-----------------------------------------------------------------------------

#ifndef NGAS2LOCALCOPYTASK_H
#define NGAS2LOCALCOPYTASK_H


// External packages includes
#include "ace/Mutex.h"

// Local includes
#include <GenTask.h>


/// Class that implements the NGAS to Local Copy Task.
class Ngas2LocalCopyTask: public GenTask
{
#ifdef UNIT_TEST
  friend class TestNgas2LocalCopyTask;
#endif

  public:
    /// Constructor registering common instance of the Replication Agent Class
    /// and the output message queue to use for this task.
    Ngas2LocalCopyTask(RepAgent&     repAgent,
		       MsgQueue&     msgOutputQueue);

    /// Destructor method.
    ~Ngas2LocalCopyTask(void);

    /// ACE_Task Server method.
    int svc(void);

    /// Method that implements the business logic of the initialization.
    void initBusiness(void);

    /// Method that implements the business logic of the server loop.
    void srvBusiness(void);

  private: 
    NgamsClient    ngasClient_;
    string         targetDir_;
    string         ngasTmpDir_;
    bool           uniqueName_;
};



#endif // NGAS2LOCALCOPYTASK_H

// EOF

