// -*-c++-*-

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Jens Knudstrup - ESO/DMD/DFS
// SUBSYSTEM: DFS Archive System
//
// $Name: arcsys-1_2_8 $
// $Revision: 1.14 $
// $Log: Ngas2TlIngTask.h,v $
// Revision 1.14  2007/06/01 07:23:45  jknudstr
// Changes in connection with testing/finalizing of SA->PA Rep Agent.
//
// Revision 1.13  2007/01/11 15:35:10  jknudstr
// -
//
// Revision 1.12  2006/10/30 08:56:51  jknudstr
// -
//
// Revision 1.11  2006/08/08 15:29:22  jknudstr
// -
//
//-----------------------------------------------------------------------------

#ifndef NGAS2TLINGTASK_H
#define NGAS2TLINGTASK_H


// Local includes
#include <GenTask.h>


/// Class that implements the Replication Queue Monitoring Task.
/// This kind of task may run in several instances.
class Ngas2TlIngTask: public GenTask
{
  public:
    /// Constructor registering common instance of the Replication Agent Class
    /// and the output message queue to use for this task.
    Ngas2TlIngTask(RepAgent&     repAgent,
		   MsgQueue&     msgOutputQueue);

    /// Destructor method.
    ~Ngas2TlIngTask(void);

    /// ACE_Task server method.
    int svc(void);

    /// Method implementing the business logic of the initialization.
    void initBusiness(void);
    
    /// Method implementing the business logic of this server process.
    void srvBusiness(void);

    /// Restore the previous state from the DB information.
    void restore(void);

  private: 
    // TODO: Check if necessary to have one instance of ProTrieve/NgamsClient
    // per instance of Ngas2TlIngTask.
    ProTrieve           proTrieve_;
    NgamsClient         ngasClient_;

    static string       saContext_;
    static string       ngasTmpDir_;
};


#endif // NGAS2TLINGTASK_H

// EOF

