// -*-c++-*-

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Christophe Moins - ESO/DMD/DFS
// SUBSYSTEM: arcsys - Archive Systems
// 
// $Revision: 1.7 $
// $Log: NgasFile.h,v $
// Revision 1.7  2007/11/12 14:38:11  cmoins
// adding of equality operator
//
// Revision 1.6  2007/01/08 13:12:15  cmoins
// ngas file iterator type creation
//
// Revision 1.5  2006/12/15 15:21:13  cmoins
// DAO & Service design modification
//
// Revision 1.4  2006/12/14 13:43:57  cmoins
// cvs heading adding
//
//
// Ngas file business object
// 
//------------------------------------------------------------------------------


#ifndef NGAS_FILE_H
#define NGAS_FILE_H

#include <string>
#include <ctime>
#include <vector>

/// Ngas File business object
/**
 * This structure represents a file handled by ngas
 * in the context of the ESO archive systems
 */

struct NgasFile
{
  typedef unsigned char NgasFileStatus ; 
  typedef std::vector<NgasFile> ResultSet;
  typedef ResultSet::iterator pNgasFile;

  bool operator == (const NgasFile& rfile)
  {
    return ((fileId_ == rfile.fileId_) &&
	    (fileName_ == rfile.fileName_) &&
            (fileVersion_ == rfile.fileVersion_) &&
            (diskId_ == rfile.diskId_) &&
            (format_ == rfile.format_) &&
            (fileSize_ == rfile.fileSize_) &&
            (uncompressedFileSize_ == rfile.uncompressedFileSize_) &&
            (compression_ == rfile.compression_) &&
            (ingestionDate_.tv_sec == rfile.ingestionDate_.tv_sec) &&
            (ingestionDate_.tv_usec == rfile.ingestionDate_.tv_usec) &&
            (ignore_ == rfile.ignore_) &&
            (checksum_ == rfile.checksum_) &&
            (checksumPlugin_ == rfile.checksumPlugin_) &&
            (fileStatus_ == rfile.fileStatus_) &&
	    (creationDate_.tv_sec == rfile.creationDate_.tv_sec) &&
	    (creationDate_.tv_usec == rfile.creationDate_.tv_usec));
  }
  std::string fileId_;
  std::string fileName_;
  int fileVersion_;
  std::string diskId_;
  std::string format_;
  int fileSize_;
  int uncompressedFileSize_;
  std::string compression_;
  struct timeval ingestionDate_;
  int ignore_;
  std::string checksum_;
  std::string checksumPlugin_; 
  unsigned char fileStatus_;
  struct timeval creationDate_;
};
#endif  // NGAS_FILE_H
