/// -*-c++-*-

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Christophe Moins- ESO/SDD/DFS
// SUBSYSTEM: arcsys - Archive Systems
//
// $Revision: 1.1 $
// $Log: ObRep2Service.h,v $
// Revision 1.1  2007/09/19 15:33:54  cmoins
// implementation of a service layer to handle data in the ObRep2 database
//
// 
//------------------------------------------------------------------------------

#ifndef OBREP2_SERVICE_H
#define OBREP2_SERVICE_H

#include <string>
#include "ConstraintSet.h"
#include "ConstraintSetDAOSyb.h"

/// Data Access Service layer to the ObRep2 database
class ObRep2Service
{
public:

 
  /// \brief Constructor
  /// \param connAlias data repository connection keyword
  ObRep2Service(std::string connAlias);

  /// \brief Destructor
  ~ObRep2Service(){};

  /// \brief finds the sets of constraints associated to an observation block
  /**
   * \param obsId observation block Id
   * \param constraintSets sets of constraints associated to obsId
   * \return the number of constraint sets if successful, -1 otherwise
   */
  int findConstraintSetByObsId(int obsId,
			       ConstraintSet::Container& rconstraintSets) const;

private :

  /// constraint set table name
  static const std::string CONSTRAINT_SETS_TABLE;       

  /// observation block table
  static const std::string OBS_BLOCKS_TABLE;

  /// data repository connection keyword
  std::string connAlias_;  

  /// Constraint set DAO class
  ConstraintSetDAOSyb constraintSetDAO_; 

};
#endif // OBREP2_SERVICE_H
