// -*-c++-*-

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Christophe Moins- ESO/SDD/DFS
// SUBSYSTEM: arcsys - Archive Systems
//
// $Revision: 1.3 $
// $Log: ObsBlockFactory.h,v $
// Revision 1.3  2007/05/04 12:19:37  cmoins
// adding of ObservationService and StorageService template parameters
//
// Revision 1.2  2007/04/26 12:57:32  cmoins
// modification of doxygen comments
//
// Revision 1.1  2007/04/25 15:57:41  cmoins
//
// 
//------------------------------------------------------------------------------

#ifndef OBS_BLOCK_FACTORY_H
#define OBS_BLOCK_FACTORY_H

#include <string>
#include "ObsBlock.h"
#include "GenObsBlock.h"
#include "CalibObsBlock.h"

using namespace std;

/// Observation Block class factory.
/**
 * This class implements a factory that instantiates observation
   blocks according to an identifier given by the user.
 */
template <class ObservationService,class StorageService>
class ObsBlockFactory
{
public:

  /// \brief Constructor
  ObsBlockFactory(){}
 
  ~ObsBlockFactory(){}

  /**
   *  Instantiates an observation block according to the 
   *  identifier given by the user
   * \param obsBlockCId  obs block class identifier
   * \param connAlias   data repository connection keyword
   * \param obsId       observation block identifier
   * \return a pointer on the created instance if successful, NULL otherwise
   */
  static 
  ObsBlock<StorageService>* getObsBlock(string obsBlockCId,
					string connAlias,
					int obsId);

  /// general observation block class id
  static const string GenObsBlockCId; 
  /// calibration block class id
  static const string CalibObsBlockCId; 

};

// observation program identifiers creation
template <class ObservationService,class StorageService>
const string ObsBlockFactory<ObservationService,StorageService>
::GenObsBlockCId = "GEN_OBS_BLOCK";

template <class ObservationService,class StorageService>
const string ObsBlockFactory<ObservationService,StorageService>
::CalibObsBlockCId = "CALIB_OBS_BLOCK";


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Instantiates an observation block according to the identifier given by
// the user.
//------------------------------------------------------------------------------
template <class ObservationService,class StorageService>
ObsBlock<StorageService>*
ObsBlockFactory<ObservationService,StorageService>
::getObsBlock(string obsBlockCId,
	      string connAlias,
	      int obsId)
{ 
  ObsBlock<StorageService>* pobsBlock;

  if (obsBlockCId == GenObsBlockCId)
    {
      pobsBlock = 
	new GenObsBlock<ObservationService,StorageService>(obsId,
							   connAlias);
    }
  else if (obsBlockCId == CalibObsBlockCId)
    {
      pobsBlock = 
	new CalibObsBlock<ObservationService,StorageService>(obsId,
							     connAlias);
    }
  else
    { 
      pobsBlock = NULL;   
    }
  return pobsBlock;
}

#endif // OBS_BLOCK_FACTORY_H
