// -*-c++-*-

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Christophe Moins- ESO/SDD/DFS
// SUBSYSTEM: arcsys - Archive Systems
//
// $Revision: 1.44 $
// $Log: ObsProgPackage.h,v $
// Revision 1.44  2009/07/06 15:33:29  vforchi
// Implemented tickets:
// DFS04048
// DFS07185
// DFS07175
//
// Revision 1.43  2009/06/26 11:25:16  vforchi
// Handle reduced files produced by HARPS and FEROS.
// Fixed tickets:
// DFS07175
// DFS07185
// DFS07195
//
// Revision 1.42  2009/06/15 12:30:10  vforchi
// Removed daemon mode, added reduced data handling
//
// Revision 1.41  2009/05/19 07:20:59  vforchi
// added a check to prevent multiple instancies
//
// Revision 1.40  2009/05/08 08:19:51  vforchi
// - Implemented retrieval of remote files with wildcard and per directory
// - Workaround to allow astoControl to queue non-completed packages
//
// Revision 1.39  2008/01/27 18:54:40  cmoins
// modification of the media type list parsing
//
// Revision 1.38  2008/01/16 15:45:19  cmoins
// ticket dfs04221 : suspend and restore operations behave in a symetrical way
//
// Revision 1.37  2008/01/16 11:05:34  cmoins
// reshuffling of error messages
//
// Revision 1.36  2008/01/02 12:53:59  cmoins
// A vector of ObsProg* is handled instead of a set
//
// Revision 1.35  2007/08/17 07:39:33  cmoins
// help content modification for remote files adding
//
// Revision 1.34  2007/08/16 14:09:13  cmoins
// DFS04236 : a message is added when suspending or restoring a file
// if the file is not present on the machine.
//
// Revision 1.33  2007/08/16 11:16:05  cmoins
// adding of defaults for some parameters in the help display
//
// Revision 1.32  2007/08/16 09:17:08  cmoins
// DFS04226 : an exception is thrown if no media type is available in the database
// so that useless messages are not displayed if the database connection is not available.
//
// Revision 1.31  2007/08/16 08:22:35  cmoins
// DFS04220 : when suspending or restoring an additional file, the file name
// may contain ADDITIONAL in case a fetched file and an additional file
// have the same name.
//
// Revision 1.30  2007/08/16 07:45:11  cmoins
// bug correction : the package id was not correct in the database
// when adding a new file
//
// Revision 1.29  2007/08/14 16:35:25  cmoins
// DFS04210 : adding of files to a package made possible when fetching  simultaneously
// data for it for the first time.
//
// Revision 1.28  2007/08/14 14:46:58  cmoins
// DFS04210 : all the tables are cleaned up even if the data storage
// was successful when a "end of operation" is requested by the user.
//
// Revision 1.27  2007/08/14 10:02:38  cmoins
// DFS04059 : warnings are issued if the end time or the verbose level
// have been modified
//
// Revision 1.26  2007/08/14 07:40:36  cmoins
// DFS04052 : <media type> added after -m option in  help display, an exception
// is thrown if the logging facility configuration failed, all messages related
// to a failed configuration are sent to the standard output, command line is
// stored in the log file, permission on log file is checked.
//
// Revision 1.25  2007/08/01 16:31:52  cmoins
// delete files functionality replaced by suspend files one,
// adding of restore and list package content functionalities
//
// Revision 1.24  2007/07/25 12:02:32  cmoins
// use of VERSION symbolic constant in config.h to display the version
//
// Revision 1.23  2007/07/17 16:24:20  cmoins
// display of logging info messages if verbose level wrong or end time not specified
//
// Revision 1.22  2007/07/17 16:07:14  cmoins
// improvement of help display
//
// Revision 1.21  2007/07/16 08:24:29  cmoins
// display of missing parameters
//
// Revision 1.20  2007/07/12 12:37:48  cmoins
// remove of files based on their names and not their files id
//
// Revision 1.19  2007/07/12 09:13:17  cmoins
// change of add files and remove files operation : files become program
// arguments and are not options any more.
//
// Revision 1.18  2007/07/12 08:17:08  cmoins
// an error message is issued if a media insertion is not possible
//
// Revision 1.17  2007/07/11 15:24:46  cmoins
// display of missing parameters
//
// Revision 1.16  2007/07/10 12:24:00  cmoins
// removal operation goes on even if files are missing
//
// Revision 1.15  2007/07/10 08:58:14  cmoins
// the file redundancy is handled by looking at the list
// of files already stored in a package and not by keeping
// a track of the previously used instruments
//
// Revision 1.14  2007/07/09 07:41:57  cmoins
// DFS04091 : strange chars display correction
//
// Revision 1.13  2007/07/05 13:57:48  cmoins
// DFS04061 : adding of a remove file feature
//
// Revision 1.12  2007/07/05 09:20:57  cmoins
// DFS04059 : tighter checking of configuration parameters
//
// Revision 1.11  2007/07/04 14:34:17  cmoins
// DFS04054 : copy of additional files whise name contains a coln is possible
//
// Revision 1.10  2007/07/04 12:01:50  cmoins
// more meaningful messages at the end of the adding and fetching operations
//
// Revision 1.9  2007/07/03 12:51:19  cmoins
// DFS04047 and DFS04048 handling : help and version option adding + clearer information display
//
// Revision 1.8  2007/06/14 13:01:40  cmoins
// run id becomes a non optional argument
//
// Revision 1.7  2007/06/13 07:36:49  cmoins
// replacement of MANUAL by ADDITIONAL
//
// Revision 1.6  2007/05/24 11:40:39  cmoins
// previously retrieved files are taken into account when retrieving package files
//
// Revision 1.5  2007/05/21 12:53:33  cmoins
// configuration modification, help display adding
//
// Revision 1.4  2007/05/15 15:33:43  cmoins
// program id is not used any more to build media files
//
// Revision 1.3  2007/05/15 10:15:03  cmoins
// modification of logging messages
//
// Revision 1.2  2007/05/04 12:29:29  cmoins
// taking in account of ObsProg and ObsBlock new interfaces
//
// Revision 1.1  2007/04/25 15:57:41  cmoins
//
// 
//------------------------------------------------------------------------------
#ifndef OBS_PROG_PACKAGE_H
#define OBS_PROG_PACKAGE_H

#include <string>
#include <iostream>
#include <fstream>
#include <iomanip>
#include <time.h>
#include <DfsLog.h>
#include <DfsLogConfig.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sys/utsname.h>
#include <boost/filesystem/operations.hpp>
#include <boost/filesystem/convenience.hpp>
#include <boost/assign/list_inserter.hpp>
#include <boost/algorithm/string.hpp>
#include "DfsProperties.h"
#include "FileCopyHandler.h"
#include "TimeStamp.h"
#include "ObsProg.h"
#include "ObsProgFactory.h"
#include "ObsBlock.h"
#include "ObsBlockFactory.h"
#include "RawDataProduct.h"
#include "FileCopy.h"
#include "Media.h"
#include "MediaFile.h"
#include "MediaType.h"
#include "PackageConfig.h"
#include "DPPackerDef.h"
#include "config.h"
#include "RemoteFileListRetriever.h"
using namespace std;
namespace fs = boost::filesystem;
namespace as = boost::assign;

/// Observation files package Template class 
/**
 * Template package class containing observation programs, observation blocks
 * It gives the possibility to retrieve all the files associated to them and to 
 * store them in a staging area. It also gives the possibility to include other 
 * type of files during package creation or to add them to an existing package.
 * The template parameters are the following 
 * ObservationService : Observation service template class that deals 
 *                      with observation products. It allows to retrieve
 *                      all the files of the package.
 * Storage Service : storage service template class that deals with
 *                   creating a media request associated to a package
 *                   and inserting it in a data repository along with the files
 *                   of the package. It is aimed to keep a track of the media
 *                   associated to a package and of its content.
 */
template <class ObservationService,class StorageService>
class ObsProgPackage
{
public:

#if defined UNIT_TEST
  friend class TestObsProgPackage;
#endif

  /// Set of packages
  typedef set<ObsProgPackage<ObservationService,StorageService> > ResultSet;
  typedef typename ResultSet::iterator pObsProgPackage;
  typedef typename ResultSet::const_iterator pconstObsProgPackage;

  /// Default constructor  
  ObsProgPackage()
  {}

  /// Constructor setting the configuration data member
  /// \param packageConfig package configuration object
  ObsProgPackage(const PackageConfig& packageConfig):
    packageConfig_(packageConfig)
  {}

  /// Constructor needed for the find operation
  /// \param packageName : name of the package to be retrieved
  ObsProgPackage(string packageName):
    packageName_(packageName)
  {}

  /// Destructor
  ~ObsProgPackage()
  {
    typename ObsProgStorSer::pObsProg pobsProg;
    for(pobsProg = progSet_.begin(); pobsProg != progSet_.end();pobsProg++)
      {
	if (*pobsProg)
	  delete *pobsProg;
      }
  }

  /// \brief copy constructor
  /// \param robsProgPackage : package to be copied
  ObsProgPackage(const ObsProgPackage<ObservationService,StorageService>& 
		 robsProgPackage);
 
  /// \brief assignment operator
  /// \param robsProgPackage : package to be assigned
  ObsProgPackage& operator = (const ObsProgPackage<ObservationService,
			      StorageService>& robsProgPackage);

  /// \brief operator < used for the set container
  bool operator < (const ObsProgPackage<ObservationService,StorageService>& 
		   robsProgPackage)  const
  {
    return (packageName_ < robsProgPackage.packageName_);
  }

  /// \brief operator == used for the set container
  bool operator == (const ObsProgPackage<ObservationService,StorageService>& 
		    robsProgPackage)  const
  {
    return (packageName_ == robsProgPackage.packageName_);
  }

  /**
   * Reads the the configuration data member to know all the options 
   * linked to the retrieval and storage of files.
   * \return 0 if the configuration successful, -1 otherwise
   */
  int configure();


  /**
   * According to the configuration of the package, it will retrieve package
   * data or add new files to it
   * \return 0 if the operation if successful, -1 otherwise
   */
  int execute();

  /**
   * Stores the content of a package in the staging area and updates
   * the database before the current ongoing operation is completed.
   * \return 0 if the operation was successful, -1 otherwise
   */
  int terminate();     

private :

  /// Set of  available operation ids
  enum { FETCH_DATA, ADD_FILE, SUSPEND_FILE, RESTORE_FILE, LIST_CONTENT, NO_OP }; 
  typedef ObsProg<StorageService> ObsProgStorSer;
  typedef ObsBlock<StorageService> ObsBlockStorSer;
 
  /**
   * Writes the content of the package in the staging area and updates the 
   * media request data repository to keep a track of it along with its content
   * \return 0 if the operation was successful, -1 otherwise
   */
  int storePackage();     

  /**
   * Fetches all the observation program files according to the package
   * configuration, updates the media request data repository and copies all
   *  the files in the staging area.
   * \return 0 if the operation was successful, -1 otherwise
   */
  int fetchData();

  /**
   * Adds a local or remote file or directory to an existing package and
   * stores it in its staging area. The syntax of a file name shall be :
   * - hostname:filepath for a remote file or
   * - filepath for a local file
   * \return 0 if the operation was successful, -1 otherwise
   */
  int addFile();

  /**
   * Suspends a file  from a package by updating its state in the database
   * \return 0 if the operation was successful, -1 otherwise
   */
  int suspendFile();

  /**
   * Restores a file in a package by updating its state in the database
   * \return 0 if the operation was successful, -1 otherwise
   */
  int restoreFile();

  /**
   * Lists the content of the package
   * \return 0 if the operation was successful, -1 otherwise
   */
  int listContent();

  /**
   * Inserts inside the media request data repository the content of the package
   * \param rmedia media request in the data repository associated to
   *        the package. The content of the package will be associated to it.
   * \return 0 if the operation was successful, -1 otherwise
   */
  int updatePackageDB(Media& rmedia);

  /** 
   * Inserts a file or the content of a local directory inside the set of 
   * additional files to be stored in the package if not already available
   * 
   * \param rcurrentPath path of the directory to be inserted
   * \param rmediaFileSet set of files already included in the package
   */
  int insertLocalDirectory(const fs::path& rcurrentPath,
			   const MediaFile::ResultSet& rmediaFileSet);

  int insertRemoteDirectory(const string host, const string path, 
			    const MediaFile::ResultSet& rmediaFileSet);

  /** 
   * Finds the set of files to be handle inside a package from a 
   * file name or directory 
   * 
   * \param rcurrentPath directory or file to be handled
   * \param rmediaFileSet set of files already included in the package
   */
  int findFilesToHandle(const fs::path& rcurrentPath,
			const MediaFile::ResultSet& rmediaFileSet);

  /**
   * configures the system facilities by setting the verbose level and
   * the output file
   * \param verboseLevel logging verbose level
   * \param loggingFile logging file
   */
  int loggingConfigure(int verboseLevel,
		       string loggingFile);
 
  /// package configuration object
  PackageConfig packageConfig_;  
  /// Operation to be performed (fetch or add)
  int operationToBePerformed_;   
  /// Alias associated to connection parameter 
  string connAlias_; 
  /// start time offset for calib files
  double startTimeOffset_;
  /// end time offset for calib files  
  double endTimeOffset_;  
  /// Identifier of the package = media request id
  int    packageId_; 
  /// name of the package
  string packageName_;
  /// Type of media on which the package will be stored
  string mediaType_;  
  /// start time of the data retrieval
  double startTime_; 
  /// end time of the data retrieval
  double endTime_; 
  /// package staging area
  string stagingArea_;
  /// daemon period in sec
  int    wait_;  
  /// number of copies
  int numCopies_;
  /// database server name
  string srvName_;
  /// database user name
  string userName_;
  /// database user password 
  string passwd_; 

  /// remote locations of reduced files, in the form host:/dir
  vector<string> reducedFilesLocations_; 

  /// set of programs
  typename ObsProgStorSer::ResultSet progSet_;      
 
  /// set of additional files
  RawDataProduct::ResultSet dataFileSet_;

  /// media creation request associated to the package
  Media mediaRequest_;  
};

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// copy constructor
//------------------------------------------------------------------------------
template <class ObservationService,class StorageService>
ObsProgPackage<ObservationService,StorageService>::
ObsProgPackage(const ObsProgPackage<ObservationService,StorageService>& 
	       robsProgPackage)
{
  *this = robsProgPackage;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// assignment operator
//------------------------------------------------------------------------------
template <class ObservationService,class StorageService>
ObsProgPackage<ObservationService,StorageService>&
ObsProgPackage<ObservationService,StorageService>::
operator=(const ObsProgPackage<ObservationService,StorageService>& 
	  robsProgPackage)
{
  if (this != &robsProgPackage)
    {
      // current data cancellation
      typename ObsProgStorSer::pObsProg pobsProg;
      for(pobsProg = progSet_.begin();pobsProg != progSet_.end();pobsProg++)
	{
	  if (*pobsProg)
	    delete *pobsProg;
	}

      for(pobsProg = robsProgPackage.progSet_.begin();
	  pobsProg != robsProgPackage.progSet_.end();pobsProg++)
	{
	  if (*pobsProg)
	    progSet_.push_back((*pobsProg)->clone());
	}

      packageConfig_   = robsProgPackage.packageConfig_;
      startTimeOffset_ = robsProgPackage.startTimeOffset_;
      endTimeOffset_   = robsProgPackage.endTimeOffset_;
      packageId_       = robsProgPackage.packageId_;
      packageName_     = robsProgPackage.packageName_;
      mediaType_       = robsProgPackage.mediaType_;
      startTime_       = robsProgPackage.startTime_;
      endTime_         = robsProgPackage.endTime_;
      stagingArea_     = robsProgPackage.stagingArea_;
      wait_            = robsProgPackage.wait_;
      numCopies_       = robsProgPackage.numCopies;
      connAlias_       = robsProgPackage.connAlias_;
      dataFileSet_     = robsProgPackage.dataFileSet_;

    }
  return (*this);
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// reads the the configuration object to know all the options 
// linked to the retrieval and storage of files.
//------------------------------------------------------------------------------
template <class ObservationService,class StorageService>
int
ObsProgPackage<ObservationService,StorageService>::
configure()
{
  const string PACKAGE_NAME_OPTION    = "pid";  
  const string MEDIA_TYPE_OPTION      = "media_id";     
  const string COPIES_NB_OPTION       = "num_copies";
  const string ADD_FILE_OPTION        = "add";
  const string ADDITIONAL_FILE_OPTION = "file";
  const string SUSPEND_FILE_OPTION     = "suspend";
  const string RESTORE_FILE_OPTION     = "restore";
  const string START_TIME_OPTION      = "starttime"; 
  const string END_TIME_OPTION        = "endtime";
  const string STAGING_AREA_OPTION    = "dir";     
  const string REDUCED_DATA_LOCATION_OPTION= "reduceddatalocation";
  const string LOGNAME_OPTION         = "logname";
  const string VERBOSE_OPTION         = "verbose";
  const string DBALIAS_OPTION         = "dbalias";
  const string WAIT_OPTION            = "wait";
  const string VERSION_OPTION         = "version";
  const string CONTENT_OPTION         = "content";
  const string DEFAULT_DB_ALIAS       = "ASTOREP";
  const string HELP_OPTION            = "help";

  // List of available options for the create package operation
  PackageConfig::OptionMap optionCreatePackageMap; 
  as::insert(optionCreatePackageMap)
    (PACKAGE_NAME_OPTION,
     PackageConfig::OptionStruct('p',"<identifier>",true,true,
				 "package identifier"))
    (MEDIA_TYPE_OPTION,
     PackageConfig::OptionStruct('m',"<media type>",true,true,
				 "type of media : "))
    (COPIES_NB_OPTION,
     PackageConfig::OptionStruct('n',"<number>",false,true,
				 "number of copies (1 as default)"))
    (ADDITIONAL_FILE_OPTION,
     PackageConfig::OptionStruct('f',"<file name>",false,true,
				 "additional file(s) that shall be included. "\
				 "For a remote file the syntax is hostName:filePath. "\
				 "Wildcards are supported as in bash for local files/directories, "\
				 "for remote files/directories only a trailing * is valid. Directories are scanned recursively"))
    (START_TIME_OPTION,
     PackageConfig::OptionStruct('s',"<YYYY-MM-DD[Thh:mm:ss]>",
				 true,true,"start time of the files"\
				 " retrieval"))
    (END_TIME_OPTION,
     PackageConfig::OptionStruct('e',"<YYYY-MM-DD[Thh:mm:ss]>",
				 false,true,"end time of the files"\
				 " retrieval (start time+one day as default)"))
    (STAGING_AREA_OPTION,
     PackageConfig::OptionStruct('d',"<dir>",true,true,
				 "staging area where the package is stored"))
    (REDUCED_DATA_LOCATION_OPTION,
     PackageConfig::OptionStruct('x',"<host1:/dir,host2:/dir...>",false,true,
				 "a comma separated list of remote locations containing reduced data"))
    (LOGNAME_OPTION,
     PackageConfig::OptionStruct('l',"<logfile>",false,true,
				 "name of the log file "\
				 "(standard output as default)"))
    (VERBOSE_OPTION,
     PackageConfig::OptionStruct('v',"<0,1 or 2>",false,true,
				 "verbose level (0 as default)"))
    (DBALIAS_OPTION,
     PackageConfig::OptionStruct('b',"<alias>",false,true, 
				 "database alias to retrieve"\
				 " database connection parameters "\
				 "(ASTOREP as default)"))
    (WAIT_OPTION,
     PackageConfig::OptionStruct('w',"<minutes>",false,true,
				 "sleep time in daemon mode "\
				 "(10 mn as default)"))
    (PackageConfig::NON_OPTIONAL_ARG,
     PackageConfig::OptionStruct(' ',"<run id(s)>",true,true,
				 "run(s) whose files shall be retrieved"));

  // List of available options for the add file operation
  PackageConfig::OptionMap optionAddPackageMap; 
  as::insert(optionAddPackageMap)
    (PACKAGE_NAME_OPTION,
     PackageConfig::OptionStruct('p',"<identifier>",true,true,
				 "package identifier"))
    (ADD_FILE_OPTION,
     PackageConfig::OptionStruct('a',"",true,false,
				 "add files flag"))
    (STAGING_AREA_OPTION,
     PackageConfig::OptionStruct('d',"<dir>",true,true,
				 "staging area where the package is stored"))
    (LOGNAME_OPTION,
     PackageConfig::OptionStruct('l',"<logfile>",false,true,
				 "name of the log file "\
				 "(standard output as default)"))
    (VERBOSE_OPTION,
     PackageConfig::OptionStruct('v',"<0,1 or 2>",false,true,
				 "verbose level (0 as default)"))
    (DBALIAS_OPTION,
     PackageConfig::OptionStruct('b',"<alias>",false,true, 
				 "database alias to retrieve"\
				 " database connection parameters " \
				 "(ASTOREP as default)"))
   (PackageConfig::NON_OPTIONAL_ARG,
     PackageConfig::OptionStruct(' ',"<file name(s)>",true,true,
				 "files or directories that shall be added "\
				 "to the package. For a remote file the "\
				 "syntax is hostName:filePath. Wildcards are supported "\
				 "as in bash for local files/directories, for remote "\
				 "files/directories only a trailing * is valid. "\
				 "Directories are scanned recursively"));

  // List of available options for the suspend file operation
  PackageConfig::OptionMap optionSuspendPackageMap; 
  as::insert(optionSuspendPackageMap)
    (PACKAGE_NAME_OPTION,
     PackageConfig::OptionStruct('p',"<identifier>",true,true,
				 "package identifier"))
    (SUSPEND_FILE_OPTION,
     PackageConfig::OptionStruct('u',"",true,false,
				 "suspend files flag"))
    (STAGING_AREA_OPTION,
     PackageConfig::OptionStruct('d',"<dir>",true,true,
				 "staging area where the package is stored"))
    (LOGNAME_OPTION,
     PackageConfig::OptionStruct('l',"<logfile>",false,true,
				 "name of the log file "\
				 "(standard output as default)"))
    (VERBOSE_OPTION,
     PackageConfig::OptionStruct('v',"<0,1 or 2>",false,true,
				 "verbose level (0 as default)"))
    (DBALIAS_OPTION,
     PackageConfig::OptionStruct('b',"<alias>",false,true, 
				 "database alias to retrieve"\
				 " database connection parameters "\
				 "(ASTOREP as default)"))
   (PackageConfig::NON_OPTIONAL_ARG,
     PackageConfig::OptionStruct(' ',"<file name(s)>",true,true,
				 "files or directories that shall be "\
				 "suspended from the package"));


  // List of available options for the restore file operation
  PackageConfig::OptionMap optionRestorePackageMap; 
  as::insert(optionRestorePackageMap)
    (PACKAGE_NAME_OPTION,
     PackageConfig::OptionStruct('p',"<identifier>",true,true,
				 "package identifier"))
    (RESTORE_FILE_OPTION,
     PackageConfig::OptionStruct('r',"",true,false,
				 "restore files flag"))
    (STAGING_AREA_OPTION,
     PackageConfig::OptionStruct('d',"<dir>",true,true,
				 "staging area where the package is stored"))
    (LOGNAME_OPTION,
     PackageConfig::OptionStruct('l',"<logfile>",false,true,
				 "name of the log file "\
				 "(standard output as default)"))
    (VERBOSE_OPTION,
     PackageConfig::OptionStruct('v',"<0,1 or 2>",false,true,
				 "verbose level (0 as default)"))
    (DBALIAS_OPTION,
     PackageConfig::OptionStruct('b',"<alias>",false,true, 
				 "database alias to retrieve"\
				 " database connection parameters "\
				 "(ASTOREP as default)"))

   (PackageConfig::NON_OPTIONAL_ARG,
     PackageConfig::OptionStruct(' ',"<file name(s)>",true,true,
				 "files or directories that shall be restored "\
				 "in the package"));

  // List of available options for the list content operation
  PackageConfig::OptionMap optionListContentPackageMap; 
  as::insert(optionListContentPackageMap)
    (PACKAGE_NAME_OPTION,
     PackageConfig::OptionStruct('p',"<identifier>",true,true,
				 "package identifier"))
    (CONTENT_OPTION,
     PackageConfig::OptionStruct('c',"",true,false,
				 "package content list flag"))
    (STAGING_AREA_OPTION,
     PackageConfig::OptionStruct('d',"<dir>",true,true,
				 "staging area where the package is stored"))
    (LOGNAME_OPTION,
     PackageConfig::OptionStruct('l',"<logfile>",false,true,
				 "name of the log file "\
				 "(standard output as default)"))
    (VERBOSE_OPTION,
     PackageConfig::OptionStruct('v',"<0,1 or 2>",false,true,
				 "verbose level (0 as default)"))
    (DBALIAS_OPTION,
     PackageConfig::OptionStruct('b',"<alias>",false,true, 
				 "database alias to retrieve"\
				 " database connection parameters "\
				 "(ASTOREP as default)"));

  // List of the whole set of available options
  PackageConfig::OptionMap allOptionMap;
  as::insert(allOptionMap)
    (HELP_OPTION,
     PackageConfig::OptionStruct('h',"",false,false,
				 "Print usage information and configuration "\
				 "file options."))
    (VERSION_OPTION,
     PackageConfig::OptionStruct(' ',"",false,false,"version number"));
  allOptionMap.insert(optionCreatePackageMap.begin(),
		      optionCreatePackageMap.end());
  allOptionMap.insert(optionAddPackageMap.begin(),optionAddPackageMap.end());
  allOptionMap.insert(optionSuspendPackageMap.begin(),
		      optionSuspendPackageMap.end());
  allOptionMap.insert(optionRestorePackageMap.begin(),
		      optionRestorePackageMap.end());
  allOptionMap.insert(optionListContentPackageMap.begin(),
		      optionListContentPackageMap.end());

  bool utcTimeFlag;
  int retStatus = 0;
  int verboseLevel;
  string logFile;
  string mainProgId;
  PackageConfig::StringValList fileList;


  DFS_TRACE("ObsProgPackage<ObservationService,StorageService>::configure");

  PackageConfig::StringValList progList;

  if (packageConfig_.initConfig(allOptionMap) > 0)
    {
      // logging facility configuration
      if (packageConfig_.getInt(VERBOSE_OPTION,verboseLevel))
	{
	  verboseLevel = 0;
	}
      if (packageConfig_.getString(LOGNAME_OPTION,logFile))
	{
	  logFile.clear();
	}
      if (packageConfig_.getString(DBALIAS_OPTION,connAlias_))
	{
	  connAlias_ = DEFAULT_DB_ALIAS;
	}
      if (loggingConfigure(verboseLevel,logFile))
	{
	  throw("Logging facility configuration failed");
	}

      // Display of the command line used to launch the program
      packageConfig_.displayCommandLine(); 

      // Gets the list of all available media type
      StorageService storageService(connAlias_);
      MediaType::ResultSet allMediaType;
      string listOfMediaType;
      if (storageService.findAllMediaType(allMediaType) > 0)
	{
	  
	  for (MediaType::pMediaType pmediaType = allMediaType.begin();
	       pmediaType != allMediaType.end();pmediaType++)
	    {
	      listOfMediaType += pmediaType->mediaType_ + " ";
	    }
	  optionCreatePackageMap[MEDIA_TYPE_OPTION].helpText_ += listOfMediaType;
	  allOptionMap[MEDIA_TYPE_OPTION].helpText_ += listOfMediaType;
	}
      else
	{
	  throw("No media type available");
	}

      // arguments given by the user belong to the list of available options
      // checks now which operation shall be performed according to 
      // the chosen options

      int optionFlag;
      if (!packageConfig_.getInt(VERSION_OPTION,optionFlag))
	{
	  // version display
	  cout << "Version : "<< VERSION << endl;
	  // only info display is performed, no other action is required
	  operationToBePerformed_ = NO_OP;
	} 
      else if (!packageConfig_.getInt(HELP_OPTION,optionFlag))
	{
	  // Help diplay to create a package
	  cout << endl;
	  cout << " * To create a package " << endl;
	  packageConfig_.displayHelp(optionCreatePackageMap);
	  cout << endl;
	  cout << endl;
	  cout << " * To add files to an existing package " << endl;
	  packageConfig_.displayHelp(optionAddPackageMap);	 
	  cout << endl;
	  cout << endl;
	  cout  << " * To suspend files from an existing package " << endl;
	  packageConfig_.displayHelp(optionSuspendPackageMap);	 
	  cout << endl;
	  cout << endl;
	  cout  << " * To restore files in an existing package " << endl;
	  packageConfig_.displayHelp(optionRestorePackageMap);	 
	  cout << endl;
	  cout << endl;
	  cout  << " * To list the content of a package " << endl;
	  packageConfig_.displayHelp(optionListContentPackageMap); 
	  cout << endl;
	  cout << endl;
	  cout << " * Miscellaneous " << endl;
	  cout << "--" << HELP_OPTION << " or " 
	       << "-"  << allOptionMap[HELP_OPTION].shortName_ << endl;
	  cout << "\t" << allOptionMap[HELP_OPTION].helpText_ << endl;
	  cout << "--" << VERSION_OPTION << endl;
	  cout << "\t" << allOptionMap[VERSION_OPTION].helpText_ << endl;
	  operationToBePerformed_ = NO_OP;
	} 
      else if (!packageConfig_.getString(PACKAGE_NAME_OPTION,packageName_) &&
	       !packageConfig_.getString(MEDIA_TYPE_OPTION,mediaType_) &&
	       !packageConfig_.getArgString(progList) &&
	       !packageConfig_.getTime(START_TIME_OPTION,
				       startTime_,
				       utcTimeFlag) &&
	       !packageConfig_.getString(STAGING_AREA_OPTION,stagingArea_))
	{  
          
          //package creation
	  operationToBePerformed_ = FETCH_DATA;
    
	  // Staging area validity checking
	  if (!(fs::exists(stagingArea_) && fs::is_directory(stagingArea_)))
	    {
	      std::string message = stagingArea_ +  " does not exist";
	      cout << message << endl;
	      DFS_ERROR("%s",message.c_str());
	      retStatus = -1;
	    }

	  // media validity checking
          DPPackerDef::TokenCharSeparator dirSep(" ");
	  DPPackerDef::Tokenizer tok = DPPackerDef::Tokenizer(listOfMediaType,dirSep);
	  DPPackerDef::Tokenizer::iterator beg = tok.begin();
	  while (beg != tok.end())
	    {
	      if (mediaType_ == *beg)
		break;
	      beg++;
	    }
	  if (beg == tok.end())
	    {
	      // the media type is not valid
	      std::string message =  "Media type no correct." \
		" Please choose among : " + listOfMediaType ;
	      cout << message << endl;
	      DFS_ERROR("%s",message.c_str());
	      retStatus = -1;
	    }

          // extract reduced data location
          string tempLocation;
          if (!packageConfig_.getString(REDUCED_DATA_LOCATION_OPTION, tempLocation)) {
             boost::algorithm::split(reducedFilesLocations_, tempLocation, boost::algorithm::is_any_of(",") ); 
          }


	  string progCId = 
	    ObsProgFactory<ObservationService,StorageService>::GenObsProgCId;
	  for (PackageConfig::pconstStringValue pval= progList.begin();
	       pval != progList.end(); pval++)
	    {
	      DFS_DEBUG("Prog %s has been registered",pval->c_str());
	      ObsProgStorSer* curProg = 
		ObsProgFactory<ObservationService,StorageService>
		::getObsProg(progCId,connAlias_,*pval);
	      progSet_.push_back(curProg);
	    }

	  const double H_18 = 0.75;
	  const double H_MINUS_6 = -0.25; // minus 6 hours
	  const double H_12   = 0.5;     // 12 hours 
	  if (!utcTimeFlag) //no utc time indicated
	    {
	      // the start time is set at 18h00 of the indicated date
	      startTime_ += H_18;
	      // a refining of the start and end time will be performed
	      // to retrieve calibration files
	      startTimeOffset_ = H_MINUS_6;
	      endTimeOffset_   = H_12;
	    }
	  else
	    {
	      // no time refining for calibration files will be perfomed
	      startTimeOffset_ = 0;
	      endTimeOffset_   = 0;
	    }

	  // Checks  if a file retrieval was  already performed      
	  Media availablePackage;
	  MediaFile::ResultSet previousFileSet;
	  if (storageService.findMediaFiles(packageName_,
					    availablePackage,
					    previousFileSet) > 0)
	    {
	      DFS_INFO("Some files have already been retrieved for %s",
		       packageName_.c_str());
	    }
	  else
	    {
	      DFS_INFO("No files previously retrieved for %s",
		       packageName_.c_str());
	    }
	  const double ONE_DAY = 1.;
	  if (packageConfig_.getTime(END_TIME_OPTION,endTime_,utcTimeFlag))
	    { // the end time was not given
	      // the end time is set one day after the start
	      endTime_ = startTime_ + ONE_DAY;
	      DFS_WARN("No end time, set by default to start date + one day");
	    }
	  else
	    {
	      if (!utcTimeFlag)
		{
		  //no time is given, the end time will be 
		  // the following day at 18.00
		  const double DAY_AFTER_AT_18H = 1.75;
		  endTime_ += DAY_AFTER_AT_18H;
		  endTimeOffset_   = H_12;
		}
	      else
		{
		  endTimeOffset_   = 0;
		}
	    }
          DFS_DEBUG("Start time : %f , End Time : %f",startTime_,endTime_);
	  if (packageConfig_.getInt(COPIES_NB_OPTION,numCopies_) ||
	      numCopies_ < 0)
	    {
	      DFS_WARN("Number of copies set to default value 1");
	      numCopies_ = 1;
	    }
	  PackageConfig::StringValList fileList;
	  if (!packageConfig_.getString(ADDITIONAL_FILE_OPTION,fileList))
	    {
	      const string DISK_SEPARATOR = ":";
	      string diskName;
	      string fileName;
	      for (PackageConfig::pconstStringValue pval= fileList.begin();
		   pval != fileList.end(); pval++)
		{
   		  unsigned int separatorIndex = pval->find(DISK_SEPARATOR);
		  if (separatorIndex != string::npos)
		    {
		      diskName = pval->substr(0,separatorIndex);
		      // a disk name was given, the file is a remote one
		      fileName = pval->substr(separatorIndex+1,
					      pval->length()-separatorIndex
					      + DISK_SEPARATOR.length()-1);

	      
		      insertRemoteDirectory(diskName, fileName, previousFileSet);
		    }
		  else
		    {
		      // The disk name was not given, the file is a local one
		      fileName = *pval;
		      // the file or directory is a local one
		      // Checks if it is a directory
		      insertLocalDirectory(fs::path(fileName,fs::native),
					   previousFileSet);
		    }
		}
	    }
	  if (packageConfig_.getInt(WAIT_OPTION,wait_) ||
	      wait_ < 0)
	    {
	      const int DEFAULT_PERIOD = 600; // in second
	      wait_ = DEFAULT_PERIOD;
	      DFS_WARN("Wait period set to default : %d s",
		       DEFAULT_PERIOD);
	    }
	  else 
	    {
	      // conversion in second
	      wait_ *= 60;
	    }
	  if (!retStatus)
	    {
	      DFS_INFO("Configuration successful to fetch data");
	    }
	  else 
	    {
	      DFS_ERROR("Configuration failed to fetch data");
	    }
	}  
      else if (!packageConfig_.getString(PACKAGE_NAME_OPTION,packageName_) &&
	       !(packageConfig_.getInt(ADD_FILE_OPTION,optionFlag)) &&
	       !packageConfig_.getString(STAGING_AREA_OPTION,stagingArea_) &&
	       !packageConfig_.getArgString(fileList))
	{
          // files adding into a package
	  operationToBePerformed_ = ADD_FILE;

	  MediaFile::ResultSet mediaFileSet;
	  Media availablePackage;
	  if (storageService.findMedia(packageName_,
				       availablePackage) > 0)
	    {
	      // Gets the list of files to be added and checks 
	      // that they are not already available in the package
	      const string DISK_SEPARATOR = ":";
	      string diskName;
	      string fileName;
	      for (PackageConfig::pconstStringValue pval= fileList.begin();
		   pval != fileList.end(); pval++)
		{
		  unsigned int separatorIndex = pval->find(DISK_SEPARATOR);
		  if (separatorIndex != string::npos)
		    {
		      diskName = pval->substr(0,separatorIndex);
		      // a disk name was given, the file is a remote one
		      fileName = pval->substr(separatorIndex+1,
					      pval->length()-separatorIndex
					      + DISK_SEPARATOR.length()-1);
	
		      insertRemoteDirectory(diskName, fileName, mediaFileSet);
		    }
		  else
		    {
		      // The disk name was not given, the file or directory 
		      // is a local one
		      fileName = *pval;
		      // the file is a local one
		      // Checks if it is a directory
		      insertLocalDirectory(fs::path(fileName,fs::native),
					   mediaFileSet);
		    }
		}
	    }
	  else
	    {
	      DFS_ERROR("package %s does not exist",packageName_.c_str());
	      retStatus = -1;
	    }
	  if (!retStatus)
	    {
	      DFS_INFO("Configuration successful to add files");
	    }
	  else 
	    {
	      DFS_ERROR("Configuration failed to add files");
	    }
	}
      else if (!packageConfig_.getString(PACKAGE_NAME_OPTION,packageName_) &&
	       !packageConfig_.getInt(SUSPEND_FILE_OPTION,optionFlag) &&
	       !packageConfig_.getString(STAGING_AREA_OPTION,stagingArea_) &&
	       !packageConfig_.getArgString(fileList)) 
	{
	  // suspends files from a package
	  operationToBePerformed_ = SUSPEND_FILE;

	  MediaFile::ResultSet mediaFileSet;
	  Media availablePackage;
	  if (storageService.findMediaFiles(packageName_,
					    availablePackage,
					    mediaFileSet) > 0)
	    {
	      packageId_ = availablePackage.mediaRequestId_;
	      for (PackageConfig::pconstStringValue pval = fileList.begin();
		   pval != fileList.end(); pval++)
		{
		  findFilesToHandle(fs::path(pval->c_str(),fs::native),
				    mediaFileSet);
		}
	      if (dataFileSet_.empty())
		retStatus = -1;
	    }
	  else
	    {
	      DFS_ERROR("package %s does not exist",packageName_.c_str());
	      retStatus = -1;
	    }
	  if (!retStatus)
	    {
	      DFS_INFO("Configuration successful to suspend files");
	    }
	  else 
	    {
	      DFS_ERROR("Configuration failed to suspend files");
	    }
	}
      else if (!packageConfig_.getString(PACKAGE_NAME_OPTION,packageName_) &&
	       !packageConfig_.getInt(RESTORE_FILE_OPTION,optionFlag) &&
	       !packageConfig_.getString(STAGING_AREA_OPTION,stagingArea_) &&
	       !packageConfig_.getArgString(fileList)) 
	{
	  // suspends files from a package
	  operationToBePerformed_ = RESTORE_FILE;

	  MediaFile::ResultSet mediaFileSet;
	  Media availablePackage;
	  if (storageService.findMediaFiles(packageName_,
					    availablePackage,
					    mediaFileSet) > 0)
	    {
	      packageId_ = availablePackage.mediaRequestId_;
	      for (PackageConfig::pconstStringValue pval = fileList.begin();
		   pval != fileList.end(); pval++)
		{
		  findFilesToHandle(fs::path(pval->c_str(),fs::native),
				    mediaFileSet);
		}
	      if (dataFileSet_.empty())
		retStatus = -1;
	    }
	  else
	    {
	      DFS_ERROR("package %s does not exist",packageName_.c_str());
	      retStatus = -1;
	    }
	  if (!retStatus)
	    {
	      DFS_INFO("Configuration successful to restore files");
	    }
	  else 
	    {
	      DFS_ERROR("Configuration failed to restore files");
	    }
	}
      else if (!packageConfig_.getString(PACKAGE_NAME_OPTION,packageName_) &&
	       !packageConfig_.getInt(CONTENT_OPTION,optionFlag) &&
	       !packageConfig_.getString(STAGING_AREA_OPTION,stagingArea_))
	{
	  // suspends files from a package
	  operationToBePerformed_ = LIST_CONTENT;

	  MediaFile::ResultSet mediaFileSet;
	  Media availablePackage;
	  if (storageService.findMediaFiles(packageName_,
					    availablePackage,
					    mediaFileSet) > 0)
	    {
	      packageId_ = availablePackage.mediaRequestId_;
	    }
	  else
	    {
	      DFS_ERROR("package %s does not exist",packageName_.c_str());
	      retStatus = -1;
	    }
	  if (!retStatus)
	    {
	      DFS_INFO("Configuration successful to list files");
	    }
	  else 
	    {
	      DFS_ERROR("Configuration failed to list files");
	    }
	}
      else
	{
	  // Arguments do no match any available configuration

	  // switch off logging facility
	  loggingConfigure(-1,"");
	  string nonOptionalArg = PackageConfig::NON_OPTIONAL_ARG;

	  if ((!packageConfig_.getTime(START_TIME_OPTION,
				       startTime_,
				       utcTimeFlag) ||
	       !packageConfig_.getString(MEDIA_TYPE_OPTION,mediaType_)) &&
	      packageConfig_.initConfig(optionCreatePackageMap) > 0)
	    {
	      // Help diplay to create a package
	      // to create a package, the package name, the media type
	      // the run id(s), the start time and the staging area options
	      // are mandatory
	      if (packageConfig_.getString(PACKAGE_NAME_OPTION,packageName_))
		cout << " --" << PACKAGE_NAME_OPTION << " is missing" << endl;
	      if (packageConfig_.getString(MEDIA_TYPE_OPTION,mediaType_))
		cout << "--" << MEDIA_TYPE_OPTION << " is missing" << endl;
	      if (packageConfig_.getArgString(progList))
		cout <<  optionCreatePackageMap[nonOptionalArg].argument_ 
		     << " is missing " << endl;
	      if (packageConfig_.getTime(START_TIME_OPTION,
					 startTime_,
					 utcTimeFlag))
		cout << "--" << START_TIME_OPTION << " is missing " << endl;
	      if (packageConfig_.getString(STAGING_AREA_OPTION,stagingArea_))
		cout << "--" << STAGING_AREA_OPTION << " is missing" << endl;
	      cout << "* To create a package " << endl;
	      packageConfig_.displayHelp(optionCreatePackageMap);
	      cout << endl;
	    }
	  else if (!(packageConfig_.getInt(ADD_FILE_OPTION,optionFlag)) &&
		   packageConfig_.initConfig(optionAddPackageMap) > 0)
	    {
	      // Help display to add files to an existing package
	      // to add files to an existing package, the package name, 
	      // file names and staging area options are mandatory
	      // but no non optional arg should be given

	      if (packageConfig_.getString(PACKAGE_NAME_OPTION,packageName_))
		cout << "--" << PACKAGE_NAME_OPTION << " is missing" << endl;
	      if (packageConfig_.getArgString(fileList))
		cout <<  optionAddPackageMap[nonOptionalArg].argument_ 
		     << " is missing " << endl;
	      if (packageConfig_.getString(STAGING_AREA_OPTION,stagingArea_))
		cout << "--" << STAGING_AREA_OPTION << " is missing " << endl;
	      cout  << "* To add files to an existing package " << endl;
	      packageConfig_.displayHelp(optionAddPackageMap);	 
	      cout << endl;
	    }
	  else if (!packageConfig_.getInt(SUSPEND_FILE_OPTION,optionFlag) &&
		   packageConfig_.initConfig(optionSuspendPackageMap) > 0)
	    {
	      // Help display to suspend files from an existing package
	      // to suspend files from an existing package, the package name, 
	      // file names and staging area options are mandatory
	      if (packageConfig_.getString(PACKAGE_NAME_OPTION,packageName_))
		cout << "--" << PACKAGE_NAME_OPTION << " is missing" << endl;
	      if (packageConfig_.getArgString(fileList))
		cout <<  optionSuspendPackageMap[nonOptionalArg].argument_ 
		     << " is missing " << endl;
	      if (packageConfig_.getString(STAGING_AREA_OPTION,stagingArea_))
		cout << "--" << STAGING_AREA_OPTION << " is missing " << endl;
	      cout  << "* To suspend files from an existing package " << endl;
	      packageConfig_.displayHelp(optionSuspendPackageMap);	 
	    }
	  else if (!packageConfig_.getInt(RESTORE_FILE_OPTION,optionFlag) &&
		   packageConfig_.initConfig(optionRestorePackageMap) > 0)
	    {
	      // Help display to restore files in an existing package
	      // to restore files in an existing package, the package name, 
	      // file names and staging area options are mandatory
	      if (packageConfig_.getString(PACKAGE_NAME_OPTION,packageName_))
		cout << "--" << PACKAGE_NAME_OPTION << " is missing" << endl;
	      if (packageConfig_.getArgString(fileList))
		cout <<  optionSuspendPackageMap[nonOptionalArg].argument_ 
		     << " is missing " << endl;
	      if (packageConfig_.getString(STAGING_AREA_OPTION,stagingArea_))
		cout << "--" << STAGING_AREA_OPTION << " is missing " << endl;
	      cout  << "* To restore files in an existing package " << endl;
	      packageConfig_.displayHelp(optionRestorePackageMap);	 
	    }
	  else if (!packageConfig_.getInt(CONTENT_OPTION,optionFlag) &&
		   packageConfig_.initConfig(optionListContentPackageMap) > 0)
	    {

	      if (packageConfig_.getString(PACKAGE_NAME_OPTION,packageName_))
		cout << "--" << PACKAGE_NAME_OPTION << " is missing" << endl;

	      if (packageConfig_.getString(STAGING_AREA_OPTION,stagingArea_))
		cout << "--" << STAGING_AREA_OPTION << " is missing " << endl;
	      cout  << "* To list the content of a package " << endl;
	      packageConfig_.displayHelp(optionListContentPackageMap);	 
	    }
	  else
	    {
	      // Invalid arguments or no argument given
	      // Complete help diplay 
	      cout << endl;
	      cout << "* To create a package " << endl;
	      packageConfig_.displayHelp(optionCreatePackageMap);
	      cout << endl;
	      cout << endl;
	      cout  << "* To add files to an existing package " << endl;
	      packageConfig_.displayHelp(optionAddPackageMap);	 
	      cout << endl;
	      cout << endl;
	      cout  << "* To suspend files from an existing package " << endl;
	      packageConfig_.displayHelp(optionSuspendPackageMap);	 
	      cout << endl;
	      cout << endl;
	      cout  << "* To restore files in an existing package " << endl;
	      packageConfig_.displayHelp(optionRestorePackageMap);	 
	      cout << endl;
	      cout << endl;
	      cout  << "* To list the content of a  package " << endl;
	      packageConfig_.displayHelp(optionListContentPackageMap);	 
	      cout << endl;
	      cout << endl;
	      cout << "* Miscellaneous " << endl;
	      cout << "--" << HELP_OPTION << " or " 
		   << "-"  << allOptionMap[HELP_OPTION].shortName_ << endl;
	      cout << "\t" << allOptionMap[HELP_OPTION].helpText_ << endl;
	      cout << "--" << VERSION_OPTION << endl;
	      cout << "\t" << allOptionMap[VERSION_OPTION].helpText_ << endl;

	    }
	  retStatus = -1;
	} 
    }
  else
    {
      // Invalid arguments or no argument given
      cout << endl;
      cout << "* To create a package " << endl;
      packageConfig_.displayHelp(optionCreatePackageMap);
      cout << endl;
      cout << endl;

      cout  << "* To add files to an existing package " << endl;
      packageConfig_.displayHelp(optionAddPackageMap);	 
      cout << endl;
      cout << endl;

      cout  << "* To suspend files from an existing package " << endl;
      packageConfig_.displayHelp(optionSuspendPackageMap);	 
      cout << endl;
      cout << endl;

      cout  << "* To restore files in an existing package " << endl;
      packageConfig_.displayHelp(optionRestorePackageMap);	 
      cout << endl;
      cout << endl;

      cout  << "* To list the content of a package " << endl;
      packageConfig_.displayHelp(optionListContentPackageMap);	 
      cout << endl;
      cout << endl;

      cout << "* Miscellaneous " << endl;
      cout << "--" << HELP_OPTION << " or " 
	   << "-"  << allOptionMap[HELP_OPTION].shortName_ << endl;
      cout << "\t" << allOptionMap[HELP_OPTION].helpText_ << endl;
      cout << "--" << VERSION_OPTION << endl;
      cout << "\t" << allOptionMap[VERSION_OPTION].helpText_ << endl;
      retStatus = -1;
    }
  return retStatus;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// retrieves package data or adds new files to it according to its configuration
//------------------------------------------------------------------------------
template <class ObservationService,class StorageService>
int
ObsProgPackage<ObservationService,StorageService>::
execute()
{
  int retStatus = 0;

  DFS_TRACE("ObsProgPackage<ObservationService,StorageService>::execute");
  switch(operationToBePerformed_)
    {

    case FETCH_DATA :
      retStatus = fetchData();
      if (!retStatus)
	DFS_INFO("fetch data on package %s completed", packageName_.c_str());
      else
	DFS_INFO("fetch data on package %s failed", packageName_.c_str());
      break;

    case ADD_FILE :
      retStatus = addFile();
      if (!retStatus)
	DFS_INFO("Add files to package %s completed", packageName_.c_str());
      else
	DFS_INFO("Add files to package %s failed", packageName_.c_str());
      break;

    case SUSPEND_FILE :
      retStatus = suspendFile();
      if (!retStatus)
	DFS_INFO("Suspend file from package %s completed", 
		 packageName_.c_str());
      else
	DFS_INFO("Suspend file from package %s failed", 
		 packageName_.c_str());
      break;

    case RESTORE_FILE :
      retStatus = restoreFile();
      if (!retStatus)
	DFS_INFO("Restore file in package %s completed", 
		 packageName_.c_str());
      else
	DFS_INFO("Restore file in package %s failed", 
		 packageName_.c_str());
      break;

    case LIST_CONTENT :
      retStatus = listContent();
      if (!retStatus)
	DFS_INFO("List content of package %s completed", 
		 packageName_.c_str());
      else
	DFS_INFO("List content of package %s failed", 
		 packageName_.c_str());
      break;

    case NO_OP :
      break;
    default :
      break;

    }
  return retStatus;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Stores the content of a package in the staging area and updates
// the database before the current ongoing operation is completed.
//------------------------------------------------------------------------------
template <class ObservationService,class StorageService>
int
ObsProgPackage<ObservationService,StorageService>::
terminate()
{
  int retStatus = 0;

  retStatus = storePackage();

  // the ingestion of the package content is completed    
  StorageService storageService(connAlias_);
  retStatus |= storageService.closeMediaInsert(mediaRequest_);

  if (retStatus == -1)
    {
      DFS_ERROR("Terminate for package %s failed",packageName_.c_str());
    }
  return retStatus;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// writes the content of the package in the staging area and updates the 
// media request data repository to keep a track of it along with its content
//------------------------------------------------------------------------------
template <class ObservationService,class StorageService>
int
ObsProgPackage<ObservationService,StorageService>::
storePackage()
{
  int retStatus = 0;

  DFS_TRACE("ObsProgPackage<ObservationService,StorageService>::storePackage");
      
  // package directory building
  stringstream mediaIdStream;
  mediaIdStream << mediaRequest_.mediaRequestId_;
  string packageDir = stagingArea_ + "/" + mediaIdStream.str() + 
    "." + packageName_;

  // programs copy
  typename ObsProgStorSer::pObsProg pobsProg;
  for (pobsProg = progSet_.begin();pobsProg != progSet_.end();pobsProg++)
    {
      if ((*pobsProg)->storeData(mediaRequest_.host_,packageDir))
	retStatus = -1;
    }

  // Copy of the additional files
  int nbcopiedAdditionalFile = 0;
  for (RawDataProduct::pRawDataProduct pdataFile = dataFileSet_.begin();
       pdataFile != dataFileSet_.end();pdataFile++)
    {
      if (!DPPackerCopy::copyFile(mediaRequest_.host_,packageDir,pdataFile))
	{
	  // update of the file name by adding its path inside
	  // the staging area
	  const_cast<string&>(pdataFile->magDiskFile_.fname_) = 
	    pdataFile->dpCat_ + "/" + pdataFile->magDiskFile_.fname_;
	  nbcopiedAdditionalFile++;
	}
    }
  if ((int)dataFileSet_.size() != nbcopiedAdditionalFile)
    {
      DFS_ERROR("Only a part of the additional files have been copied");
      retStatus = -1;
    }
  // The database is updated with the content of the package
  StorageService storageService(connAlias_);
  retStatus |= updatePackageDB(mediaRequest_);

  if (!retStatus)
    {
      DFS_INFO("package %s storage successful",packageName_.c_str());
    }
  else
    {
      DFS_ERROR("package %s storage failed",packageName_.c_str());
    }
  return retStatus;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// fetches all the observation program files according to
// the package configuration, updates the media request data repository and 
// copies all the files in the staging area.
//------------------------------------------------------------------------------
template <class ObservationService,class StorageService>
int
ObsProgPackage<ObservationService,StorageService>::
fetchData()
{
  bool overEndTime = false;
  int retStatus = 0; 
  int nbPreviousFiles;
  double startTime = 0.;
  double endTime = 0.;
  Media availablePackage;
  MediaFile::ResultSet previousFileSet;

  startTime = startTime_;
  endTime = endTime_;

  DFS_TRACE("ObsProgPackage<ObservationService,StorageService>::fetchData");

  if (startTime > endTime) 
    {
      overEndTime = true;
      DFS_ERROR("Start time beyond end time");
      retStatus = -1;
    }

  // creation of a media request and startup of the files retrieval
  struct utsname unameInfo;
  uname(&unameInfo);
  string localHostName = unameInfo.nodename;
  mediaRequest_ = Media(packageName_,mediaType_,numCopies_,localHostName);
  StorageService storageService(connAlias_);
  if (!storageService.insertMedia(mediaRequest_))
    {
      while (!overEndTime)
	{

	  // files retrieval for each each program in the package
	  typename ObsProgStorSer::pObsProg pobsProg;
	  for (pobsProg = progSet_.begin(); 
	       pobsProg != progSet_.end(); pobsProg++)
	    {
	      // Gets the list of files already stored in the package
	      // to avoid any redundancy
	      nbPreviousFiles = storageService.findMediaFiles(packageName_,
							      availablePackage,
							      previousFileSet);
	      if (nbPreviousFiles==0)
		{
		  DFS_INFO("No files previously retrieved");
		}
	      else if (nbPreviousFiles < 0)
		{
		  DFS_ERROR("Find previous files failed for %s",
			    packageName_.c_str());
		}
	      if ((*pobsProg)->fetchData(startTime,
					 startTimeOffset_,
					 endTime,
					 endTimeOffset_,
					 previousFileSet,
					 reducedFilesLocations_))
		{
		  retStatus = -1;
		}
	    
	      // copy in the staging area
	      if (storePackage())
		{
		  retStatus = -1;
		}
	    }
	  // gets the current utc time to compare it to the end time
	  TimeStamp currentTime;
	  if ((double)currentTime < endTime)
	      sleep(wait_);
	  else 
	    overEndTime = true;
	}
      // the ingestion of the package content is completed    
      retStatus |= storageService.closeMediaInsert(mediaRequest_);
    }
  else
    {
      DFS_ERROR("insertion of media request associated to %s failed",
		packageName_.c_str());
      retStatus = -1;
    }
  cout << "fetch data on package " << packageName_ << " completed" << endl;
  return retStatus;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// adds a local or remote file to an existing package and stores 
// it in its staging area.
//------------------------------------------------------------------------------
template <class ObservationService,class StorageService>
int 
ObsProgPackage<ObservationService,StorageService>::
addFile() 
{
  int retStatus = 0;
  FileCopyHandler fileCopyHandler;

  DFS_TRACE("ObsProgPackage<ObservationService,StorageService>::addFile");

  StorageService storageService(connAlias_);

  Media packageData; // package features in the data repository
  MediaFile::ResultSet packageFileSet;  //  package files in the data repository

  if (storageService.findMedia(packageName_,
			       packageData) > 0)
    {
      // Gets the name of the host where the package is stored
      string packageHostName = packageData.host_;

      // Gets the target file directory
      // The target file will be located in the package directory of 
      // the staging area.
      stringstream packageIdStr;
      packageIdStr << packageData.mediaRequestId_;
      string targetDirectory = stagingArea_ + "/" + packageIdStr.str() + "." 
	+ packageName_;

      int nbCopiedFiles =0;
      for (RawDataProduct::pRawDataProduct pdataFile = dataFileSet_.begin();
	   pdataFile != dataFileSet_.end();pdataFile++)
	{
	  if (!DPPackerCopy::copyFile(packageHostName,
				      targetDirectory,
				      pdataFile))
	    {
	      const_cast<string&>(pdataFile->magDiskFile_.fname_) = 
		pdataFile->dpCat_ + "/" + pdataFile->magDiskFile_.fname_;
	      nbCopiedFiles++;
	    }
	}
      if (nbCopiedFiles > 0)
	{
	  DFS_INFO("%d files on %d have been copied",
		   nbCopiedFiles,dataFileSet_.size());
	  // The database is updated with the content of the package
	  retStatus = updatePackageDB(packageData);
	}
      else
	{
	  DFS_ERROR("No file has been copied");
	  retStatus = -1;
	}
    }
  else
    {
      DFS_ERROR("Package %s does not exist",packageName_.c_str());
      retStatus = -1;
    }    
		       
  cout << "Add Files to package " << packageName_ << " completed" << endl;

  return retStatus;
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Suspends a file  from a package by updating its state in the database
//------------------------------------------------------------------------------
template <class ObservationService,class StorageService>
int 
ObsProgPackage<ObservationService,StorageService>::
suspendFile() 
{
  int retStatus = 0;
  StorageService storageService(connAlias_); 

  DFS_TRACE("ObsProgPackage<ObservationService,StorageService>::suspendFile");

  // Gets the file directory
  // The file  to be suspended shall be located in the package directory in
  // the staging area.
  stringstream packageIdStr;
  packageIdStr << packageId_;
  string targetDirectory = stagingArea_ + "/" + packageIdStr.str() + "." 
    + packageName_;

  int nbSuspendedFiles = 0;
  for (RawDataProduct::pRawDataProduct pdataFile = dataFileSet_.begin();
       pdataFile != dataFileSet_.end();pdataFile++)
    {
      string fileToBeSuspended = 
	targetDirectory + "/" + pdataFile->magDiskFile_.fname_;

      if (fs::exists(fs::path(fileToBeSuspended,fs::native)))
	{
	  if ( storageService.suspendFile(packageName_,
					  pdataFile->dpId_)>0)

	    {
	      DFS_INFO("%s file successfully suspended",
		       fileToBeSuspended.c_str());
	      nbSuspendedFiles++;
	    }
	  else 
	    {
	      DFS_ERROR("%s file not suspended",fileToBeSuspended.c_str());
	    }
	}
      else
	{
	  DFS_ERROR("File %s does not exist on local host",
		    fileToBeSuspended.c_str());
	}
    }
  if (nbSuspendedFiles > 0)
    {
      DFS_INFO("%d files on %d have been suspended",
	       nbSuspendedFiles,dataFileSet_.size());
    }
  else
    {
      DFS_ERROR("No file has been suspended");
      retStatus = -1;
    }		       
  cout << "Suspend Files from package " << packageName_ << " completed" << endl;
  return retStatus;
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Restores a file in a package by updating its state in the database
//------------------------------------------------------------------------------
template <class ObservationService,class StorageService>
int 
ObsProgPackage<ObservationService,StorageService>::
restoreFile() 
{
  int retStatus = 0;
  StorageService storageService(connAlias_); 

  DFS_TRACE("ObsProgPackage<ObservationService,StorageService>::restoreFile");

  // Gets the file directory
  // The file  to be restored shall be located in the package directory in
  // the staging area.
  stringstream packageIdStr;
  packageIdStr << packageId_;
  string targetDirectory = stagingArea_ + "/" + packageIdStr.str() + "." 
    + packageName_;

  int nbRestoredFiles = 0;
  for (RawDataProduct::pRawDataProduct pdataFile = dataFileSet_.begin();
       pdataFile != dataFileSet_.end();pdataFile++)
    {
      string fileToBeRestored = 
	targetDirectory + "/" + pdataFile->magDiskFile_.fname_;

      if (fs::exists(fs::path(fileToBeRestored,fs::native)))
	{
	  if (storageService.restoreFile(packageName_,
					 pdataFile->dpId_)>0)

	    {
	      DFS_INFO("%s file successfully restored",
		       fileToBeRestored.c_str());
	      nbRestoredFiles++;
	    }
	  else 
	    {
	      DFS_ERROR("%s file not restored",fileToBeRestored.c_str());
	    }
	}
      else
	{
	  DFS_ERROR("File %s does not exist on local host",
		    fileToBeRestored.c_str());
	}
    }
  if (nbRestoredFiles > 0)
    {
      DFS_INFO("%d files on %d have been restored",
	       nbRestoredFiles,dataFileSet_.size());
    }
  else
    {
      DFS_ERROR("No file has been restored");
      retStatus = -1;
    }		       
  cout << "Restore Files from package " << packageName_ << " completed" << endl;
  return retStatus;
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Lists the content of a package
//------------------------------------------------------------------------------
template <class ObservationService,class StorageService>
int 
ObsProgPackage<ObservationService,StorageService>::
listContent() 
{
  int retStatus = 0;


  DFS_TRACE("ObsProgPackage<ObservationService,StorageService>::listContent");

  // Gets the file content
  StorageService storageService(connAlias_);
  Media availablePackage;
  MediaFile::ResultSet packageFileSet;
  storageService.findMediaFiles(packageName_,
				availablePackage,
				packageFileSet);

  // Gets the package directory
  stringstream packageIdStr;
  packageIdStr << packageId_;
  string packageDirectory = stagingArea_ + "/" + packageIdStr.str() + "." 
    + packageName_;

  string completeFileName; // file name with the complete path
  string baseFileName; // file name without the complete path
  string fileStatus;
  bool availability;
  MediaFile::pconstMediaFile pconstmediaFile;
  const int FILE_COLUMN_WIDTH = 59;
  const int STATUS_COLUMN_WIDTH = 8;
  const int AVAILABILITY_COLUMN_WIDTH = 13;
  cout << endl;
  cout << setw(FILE_COLUMN_WIDTH + STATUS_COLUMN_WIDTH + 
	       AVAILABILITY_COLUMN_WIDTH) << setfill('*') << "*" <<  endl;
  cout << setw(FILE_COLUMN_WIDTH) << setfill(' ') << left << "File" 
       << setw(STATUS_COLUMN_WIDTH) << left << "Status" 
       << setw(AVAILABILITY_COLUMN_WIDTH) << left << "Available" << endl;
  cout << setw(FILE_COLUMN_WIDTH + STATUS_COLUMN_WIDTH + 
	       AVAILABILITY_COLUMN_WIDTH) << setfill('*') << "*" <<  endl;
  cout << setfill(' ');
  for (pconstmediaFile = packageFileSet.begin();
       pconstmediaFile != packageFileSet.end();pconstmediaFile++)
    {

      baseFileName = DPPackerDef::basename(pconstmediaFile->fileName_);
      fileStatus = pconstmediaFile->status_;
      completeFileName = packageDirectory + "/" + pconstmediaFile->fileName_;
      availability = fs::exists(fs::path(completeFileName,fs::native));
      cout << setw(FILE_COLUMN_WIDTH) << left << baseFileName 
           << setw(STATUS_COLUMN_WIDTH) << left << fileStatus 
	   << setw(AVAILABILITY_COLUMN_WIDTH) << left <<  availability << endl;
    }
  cout << "List content from package " << packageName_ << " completed" << endl;
  return retStatus;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  Inserts inside the media request data repository the content of the package
//------------------------------------------------------------------------------
template <class ObservationService,class StorageService>
int
ObsProgPackage<ObservationService,StorageService>::
updatePackageDB(Media& rmedia) 
{
  int retStatus = 0;  

  DFS_TRACE("ObsProgPackage<ObservationService,StorageService>"\
            "::updatePackageDB");

  typename ObsProgStorSer::pObsProg pobsProg;
  for (pobsProg = progSet_.begin(); pobsProg != progSet_.end();pobsProg++)
    {
      if ((*pobsProg)->updateProgDB(rmedia))
	{
	  retStatus = -1;
	}
    }
  if (!dataFileSet_.empty())
    {
      StorageService storageService(connAlias_);
      string fileName;
      MediaFile::ResultSet mediaFileSet;
      for (RawDataProduct::pRawDataProduct pdataFile = dataFileSet_.begin();
	   pdataFile != dataFileSet_.end(); pdataFile++)
	{

	  if (pdataFile->magDiskFile_.status_ == DPPackerDef::FILE_RETRIEVED)
	    {
	      fileName = pdataFile->magDiskFile_.fname_;
	      mediaFileSet.push_back(MediaFile(pdataFile->dpId_,
					       fileName,
					       pdataFile->magDiskFile_.fsize_,
					       pdataFile->magDiskFile_.fsize_,
					       DPPackerDef::NO_COMPRESSION_FLAG,
					       pdataFile->magDiskFile_.status_));
	    }
	}
      
      if (!mediaFileSet.empty() &&
	  !storageService.insertMediaFiles(rmedia,
					   mediaFileSet))
	{
	  DFS_INFO("Files insertion succeeded for additional files");
	  dataFileSet_.clear();
	}
      else
	{
	  DFS_ERROR("Files insertion failed for additional files");
	  retStatus = -1;
	}
    } 
  if (!retStatus)
    {
      DFS_INFO("Database update suceeded for package %s",packageName_.c_str());
    }
  else
    {
      DFS_ERROR("Database update failed for package %s",packageName_.c_str());
    }
  return retStatus;
}


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Inserts a file or the content of a local directory inside the set of 
// additional files to be stored in the package if not already available
//------------------------------------------------------------------------------
template <class ObservationService,class StorageService>
int
ObsProgPackage<ObservationService,StorageService>::
insertLocalDirectory(const fs::path& rcurrentPath,
		     const MediaFile::ResultSet& rmediaFileSet)
{
  int retStatus = 0;

  if (fs::exists(rcurrentPath))
    {
      if (fs::is_directory(rcurrentPath))
	{
	  // The current path is a directory one
	  // Parsing of its content
	  for(fs::directory_iterator currentEntry(rcurrentPath), endDir;
	      currentEntry != endDir;++currentEntry) 
	    {
	      retStatus = insertLocalDirectory(*currentEntry,
					       rmediaFileSet);
	    }
	}
      else
	{
	  // The current path is a file one
	  // The file is inserted inside the package.
	  string currentFile = rcurrentPath.string();

	  // takes the file name and removes the path
	  string fileName;
	  DPPackerDef::TokenCharSeparator dirSep("/");
	  DPPackerDef::Tokenizer tok(currentFile,dirSep);
	  DPPackerDef::Tokenizer::iterator beg;
	  beg = tok.begin();
	  while (beg != tok.end())
	    fileName = *(beg++);

	  // checks if the file is not already available 
	  MediaFile::pconstMediaFile pconstmediaFile;
	  for (pconstmediaFile = rmediaFileSet.begin();
	       pconstmediaFile != rmediaFileSet.end();pconstmediaFile++)
	    {
	      if (pconstmediaFile->fileName_.find(fileName) != string::npos)
		break;
	    }
	  if (pconstmediaFile == rmediaFileSet.end())
	    {
	      // the file has not been inserted yet
	      string category = DPPackerDef::ADDITIONAL_FILES_CAT;
	      double mjdObs = DPPackerDef::MJD_OBS_ZERO;
	      dataFileSet_.insert(RawDataProduct(fileName,
						 category,
						 mjdObs,
						 "",
						 currentFile));
	    }
	  else
	    {
	      DFS_ERROR("File %s already in the package",currentFile.c_str());
	      retStatus = -1;
	    }
	}
    }
  else
    {
      DFS_ERROR("%s does not exist",rcurrentPath.string().c_str());
      retStatus = -1;
    }
  return retStatus;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Inserts a file or the content of a local directory inside the set of 
// additional files to be stored in the package if not already available
//------------------------------------------------------------------------------
template <class ObservationService,class StorageService>
int
ObsProgPackage<ObservationService,StorageService>::
insertRemoteDirectory(const string host, const string path, 
		const MediaFile::ResultSet& rmediaFileSet) {

	RemoteFileListRetriever *fr = new RemoteFileListRetriever();
	vector<string> files = fr->getFilenames(host,path);

	for (vector<string>::iterator file = files.begin(); file < files.end(); file++) {

		// get the file id by removing the path from the file name
		string fileId;
		DPPackerDef::TokenCharSeparator dirSep("/");
		DPPackerDef::Tokenizer tok = DPPackerDef::Tokenizer(*file,dirSep);
		DPPackerDef::Tokenizer::iterator beg = tok.begin();
		while (beg != tok.end())
			fileId = *(beg++);
		MediaFile::pMediaFile pmediaFile;

		// checks if the file is not already available 
		MediaFile::pconstMediaFile pconstmediaFile;
		for (pconstmediaFile = rmediaFileSet.begin();
			pconstmediaFile != rmediaFileSet.end();pconstmediaFile++)
		{
		  if (pconstmediaFile->fileName_.find(*file) != string::npos)
		    break;
		}
		if (pconstmediaFile == rmediaFileSet.end()) 
		{
			// the file has not been inserted yet
		      
			// the file is a remote one
			string category = DPPackerDef::ADDITIONAL_FILES_CAT;
			double mjdObs = DPPackerDef::MJD_OBS_ZERO;
			dataFileSet_.insert(RawDataProduct(fileId,
						     category,
						     mjdObs,
						     host,
						     *file));
		}
		else
		{
			DFS_ERROR("file %s already in the package", file->c_str());
		}
	}
}


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Finds the set of files to be handled inside a package from a file name
// or directory
//------------------------------------------------------------------------------
template <class ObservationService,class StorageService>
int
ObsProgPackage<ObservationService,StorageService>::
findFilesToHandle(const fs::path& rcurrentPath,
		  const MediaFile::ResultSet& rmediaFileSet)
{
  int retStatus = 0;

  if (fs::exists(rcurrentPath) && fs::is_directory(rcurrentPath))
    {
      // The current path is a directory one
      // Parsing of its content
      for(fs::directory_iterator currentEntry(rcurrentPath), endDir;
	  currentEntry != endDir;++currentEntry) 
	{
	  retStatus = findFilesToHandle(*currentEntry,
					rmediaFileSet);
	}
    }
  else
    {
      // The current path is a file one
      string fileCategory; // category = additional or others

      // The file is inserted inside the package.
      string currentFile = rcurrentPath.string();
      // takes the file name and removes the path if it is not a manual file
      string fileName;
      if (currentFile.find(DPPackerDef::ADDITIONAL_FILES_CAT)!= string::npos){
	// that's a file added manually
	fileCategory = DPPackerDef::ADDITIONAL_FILES_CAT;
	fileName = currentFile;
      }
      else {
	fileName = DPPackerDef::basename(currentFile);
      }
      // checks if the file is  available in the media files
      MediaFile::pconstMediaFile pconstmediaFile;
      string currentMediaFile;
      for (pconstmediaFile = rmediaFileSet.begin();
	   pconstmediaFile != rmediaFileSet.end();pconstmediaFile++)
	{
	  currentMediaFile = pconstmediaFile->fileName_;
	  if (currentMediaFile.find(fileName) != string::npos) {
	    if (fileCategory == DPPackerDef::ADDITIONAL_FILES_CAT)
	      break;
	    else if (currentMediaFile.find(DPPackerDef::ADDITIONAL_FILES_CAT)==
		     string::npos)
	      break;
	  }
	}
      if (pconstmediaFile != rmediaFileSet.end())
	{
	  // the file is existing and is included in the list of files 
	  // to be handled
	  double mjdObs = DPPackerDef::MJD_OBS_ZERO;
	  string fileId;
	  if (fileCategory != DPPackerDef::ADDITIONAL_FILES_CAT)
	    {
	      // that's not an additional file, the file id is the one
	      // found in the table
	      fileId = pconstmediaFile->fileId_;
	    }
	  else
	    {
	      // that's an additional file, the file id is the file name
	      // without the path
	      fileId = DPPackerDef::basename(fileName);
	    }
	  dataFileSet_.insert(RawDataProduct(fileId,
					     fileCategory,
					     mjdObs,
					     "",
					     pconstmediaFile->fileName_));
	}
      else
	{
	  DFS_ERROR("File %s does not exist in the package",currentFile.c_str());
	  retStatus = -1;
	}
    }
  return retStatus;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// configures the system facilities by setting the verbose level and
// the output file
//------------------------------------------------------------------------------
template <class ObservationService,class StorageService>
int
ObsProgPackage<ObservationService,StorageService>::
loggingConfigure(int verboseLevel,
		 string loggingFile)
{
  int retStatus = 0;

  // Logging facility setup
  DfsLogConfig logConfig;
  const int NO_VERBOSE_MODE_LEVEL = -1;
  const string NO_VERBOSE_MODE = "~DEBUG|~INFO|~ERROR";
  const int ERROR_VERBOSE_MODE_LEVEL = 0;
  const string ERROR_VERBOSE_MODE = "~DEBUG|~INFO|ERROR";
  const int INFO_MODE_LEVEL = 1;
  const string INFO_MODE = "~DEBUG|INFO|ERROR";
  const int DEBUG_MODE_LEVEL = 2;
  const string DEBUG_MODE = "DEBUG|INFO|ERROR";

  string verboseMode;
  switch (verboseLevel)
    {
    case NO_VERBOSE_MODE_LEVEL : 
      // no message is taken in account
      verboseMode = NO_VERBOSE_MODE;
      break;
    case ERROR_VERBOSE_MODE_LEVEL : 
      // only error messages are taken in account
      verboseMode = ERROR_VERBOSE_MODE;
      break;
    case INFO_MODE_LEVEL :
      // only error and info  messages are taken in account
      verboseMode = INFO_MODE;
      break;
    case DEBUG_MODE_LEVEL :
      /// only debug, info and error messages are taken in account
      verboseMode = DEBUG_MODE;
      break;
    default :
      verboseMode = INFO_MODE;
    }    
  
  logConfig.setPriority(verboseMode);
  
  if (!loggingFile.empty())
    {
      ofstream configFileStream;
      configFileStream.open(loggingFile.c_str(),ios::app);
      if (configFileStream.is_open())
	{
	  logConfig.setCtrlFlags("OSTREAM");
	  logConfig.setFileName(loggingFile);
	  configFileStream.close();
	}
      else
	{
	  cout << "Could not open " << loggingFile << endl;
	  retStatus = -1;
	}
    }
  else
    {
      logConfig.setCtrlFlags("STDOUT");
    }

  if (logConfig.configure() == -1)
    {
      cout << "Could not apply logging configuration";
      retStatus = -1;
    }
  if ((verboseLevel < NO_VERBOSE_MODE_LEVEL) || 
      (verboseLevel > DEBUG_MODE_LEVEL))
    {
      // an error message is issued but no error code will be returned
      DFS_ERROR("Verbose level non correct");
      DFS_WARN("Reset verbose level to 0");
    }

  return retStatus;
}
#endif // OBS_PROG_PACKAGE_H
