// -*-c++-*-

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Christophe Moins- ESO/SDD/DFS
// SUBSYSTEM: arcsys 
//
// $Revision: 1.2 $
// $Log: PANL2NightObs.h,v $
// Revision 1.2  2007/10/25 09:05:38  cmoins
// adding of the Request_ID field
//
// Revision 1.1  2007/10/12 12:31:24  cmoins
// adding of PANL2Night table DAO
//
// 
//------------------------------------------------------------------------------

#ifndef PANL2_NIGHT_OBS_H
#define PANL2_NIGHT_OBS_H

#include <string>
#include <vector>

/// Implementation of a set of observation night conditions at paranal
struct PANL2NightObs
{

  // Default container where constraint sets will be stored
  typedef std::vector<PANL2NightObs> Container;
 
  /// Default constructor
  PANL2NightObs()
  {}

  /// Default destructor
  ~PANL2NightObs()
  {}

  /// Constructor setting the sky transparency related data members
  PANL2NightObs(std::string skyTransparency,
		int transparency):
    skyTransparency_(skyTransparency),
    transparency_(transparency)
  {}

  /// request id identifier 
  std::string requestId_;
  /// observation block identifier
  std::string obId_;
  /// sky transparency possible values : PHO, CLR, THN, THK 
  std::string skyTransparency_;
  /// sky transparency checked condition : 0 Ok, 1 Almost, 2 No.
  int transparency_;

};
#endif /* PANL2_NIGHT_OBS_H */

