// -*-c++-*-

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Christophe Moins- ESO/SDD/DFS
// SUBSYSTEM: arcsys 
//
// $Revision: 1.4 $
// $Log: RawDataProduct.h,v $
// Revision 1.4  2007/04/26 12:51:57  cmoins
// doxygen comments modification
//
// Revision 1.3  2007/04/25 15:12:24  cmoins
// made compliant to doxygen format
//
// Revision 1.2  2007/03/09 17:00:07  cmoins
// mapping to observations..data_product Table
//
// Revision 1.1  2007/03/06 10:38:58  cmoins
// renaming of ObsDataFile in RawDataProduct
// 
//------------------------------------------------------------------------------

#ifndef RAW_DATA_PRODUCT_H
#define RAW_DATA_PRODUCT_H

#include <set>
#include <string>
#include "MagDisk.h"
#include "MagDiskFile.h"

using namespace std;

/// Implementation of  a raw data product file
/**
 *  Stores informations about raw data product files created during observations
 *  Among other informations, stores the file id of a raw product,
 *  the observation program and block it belongs to, its category (science or
 *  calibration), its observation time.
 */
struct RawDataProduct
{
  /// Set of RawDataProduct
  typedef set<RawDataProduct> ResultSet;  
  typedef ResultSet::iterator pRawDataProduct; 

  /// Default Constructor
  RawDataProduct() 
  {}

  /// Constructor using a file identifier
  /**
   *  For the need of the == operator, to retrieve an element
   *  in a set of RawDataProduct
   */
  RawDataProduct(string dpId):
    dpId_(dpId),
    magDiskFile_(dpId)
  {}

  /// Constructor setting the identifier, category, time and magnetic disk data
  RawDataProduct(string dpId,
                 string dpCat,
		 double mjdObs,
		 string diskName,
		 string fname):
    dpId_(dpId),
    dpCat_(dpCat),
    mjdObs_(mjdObs),
    magDiskFile_(diskName,fname)
  {}

  
  /// Raw data product identifier
  string dpId_;   
  /// Observation block it belongs to
  int obId_;      
  string telId_;
  /// Id of the instruments it is associated to
  string insId_;  
  /// id of the program it belongs to
  string progId_; 
  string dpClass_;
  /// Category of the product (science,calibration)
  string dpCat_;  
  string dpType_;
  string dpTech_;
  string lightPath_;
  struct timeval expStart_;
  /// observation time
  double mjdObs_; 
  double expTime_;
  int tplSeqno_;
  int tplExpno_;
  string detId_;
  int detExpid_;
  string qualFlag_;
  int ra_;
  int dec_;
  string object_;
  string insMode_;
  string obsName_;
  int naxis1_;
  int naxis2_;
  double telAirmStart_;
  string origFile_;
  string detChip1Id_;
  struct timeval relDate_;
  string publicf_;
  string filterPath_;
  string grisPath_;
  string gratPath_;
  string slitPath_;
  double detDit_;
  int detNdit_;
  int detChopCycles_;
  /// data of the magnetic disk file
  MagDiskFile magDiskFile_; 

  /// Operator < used for the set container
  bool operator < (const RawDataProduct& rrawDataProduct) const
  {
    return (dpId_ < rrawDataProduct.dpId_);
  }

  /// Operator == used for the set container
  bool operator == (const RawDataProduct& rrawDataProduct) const
  {
    return (dpId_ < rrawDataProduct.dpId_);
  }
};
#endif /* RAW_DATA_PRODUCT_H */

