// -*-c++-*-

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Jens Knudstrup - ESO/DMD/DFS
// SUBSYSTEM: DFS Archive System
//
// $Name: arcsys-1_2_8 $
// $Revision: 1.10 $
// $Log: RepReqMsg.h,v $
// Revision 1.10  2007/05/03 15:32:07  jknudstr
// Intermediate check-in while developing the SA->PA Archive Replicator.
//
// Revision 1.9  2007/01/11 15:35:10  jknudstr
// -
//
// Revision 1.8  2006/08/08 15:29:22  jknudstr
// -
//
//-----------------------------------------------------------------------------

#ifndef REPREQMSG_H
#define REPREQMSG_H


// External packages includes
#include <ace/Message_Block.h>

// Local includes
#include <ArcSysEx.h>
#include <ReplicationRequest.h>


/// An instance of this class stores information in connection with one 
/// Replication Request.
class RepReqMsg: public ACE_Message_Block
{
  public:
    /// Report status of the Replication Request.
    typedef enum {
	NotReported = 0,
	Reported    = 1,
	Notified    = 2
    } ReportStatus;

    /// Constructor method.
    RepReqMsg(ReplicationRequest& repReq);

    /// Destructor method.
    ~RepReqMsg(void);

    /// Clone the ReplicationRequest object into the local instance.
    RepReqMsg&  copy(ReplicationRequest&  repReq);

    /// Return reference to the internal ReplicationRequest object.
    ReplicationRequest& repReq(void);

    /// Set the report status of the object.
    RepReqMsg& setReportStatus(const ReportStatus  repStat);
    
    /// Get the report status of the object.
    ReportStatus getReportStatus(void);

    /// Set the Request ID.
    RepReqMsg& setRequestId(const string& reqId);
    
    /// Get the  Request ID.
    ReportStatus getRequestId(void);

  private:
    ReplicationRequest  repReq_;       // The Replication Request object.
    ReportStatus        reportStatus_; // The status of the request.
};


#endif // REPREQMSG_H

// EOF

