// -*-c++-*-

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Christophe Moins- ESO/SDD/DFS
// SUBSYSTEM: arcsys 
//
// $Revision: 1.1 $
// $Log: Seeing.h,v $
// Revision 1.1  2007/09/21 15:32:12  cmoins
// seeing value implementation
//
//------------------------------------------------------------------------------

#ifndef SEEING_H
#define SEEING_H

#include <string>
#include <vector>

/// Implementation of a seeing value measured from a seeing monitor
struct Seeing
{

  // Default container where seeing values will be stored
  typedef std::vector<Seeing> Container;
 
  /// Default constructor
  Seeing()
  {}

  /// Default destructor
  ~Seeing()
  {}


  /// Constructor setting all the data member of an instance
  Seeing(int startDate,
	 int interval,
	 int fwhm,
	 int ra,
	 int dec,
	 int airmass,
	 int tau,
	 int rfl,
	 int tet):
    startDate_(startDate),
    interval_(interval),
    fwhm_(fwhm),
    ra_(ra),
    dec_(dec),
    airmass_(airmass),
    tau_(tau),
    rfl_(rfl),
    tet_(tet)
  {}

  /// Datetime of measurement
  int startDate_;
  /// time interval between two measurements
  int interval_;
  /// observatory site seeing
  int fwhm_;
  /// right ascension at which the site seeing is measured
  int ra_;
  /// declination at which the site seeing is measured
  int dec_;
  /// airmass at which the site seeing is measured
  int airmass_;

  int tau_;
  int rfl_;
  int tet_;

};
#endif /* SEEING_H */

