// -*-c++-*-

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Christophe Moins- ESO/DMD/DFS
// SUBSYSTEM: arcsys - Archive Systems
//
// $Revision: 1.2 $
// $Log: SeeingDAOSyb.h,v $
// Revision 1.2  2007/10/17 08:07:08  cmoins
// findSeeings replaced by findByDate
//
// Revision 1.1  2007/09/21 15:32:49  cmoins
// seeing object DAO class implementation
//
// 
//------------------------------------------------------------------------------

#ifndef SEEING_DAO_SYB_H
#define SEEING_DAO_SYB_H

#include "Seeing.h" 

class ServiceConnectionSyb;

/// Sybase Data Access Object to Seeing objects
class SeeingDAOSyb
{
public:
 
  /// \brief default Constructor
  SeeingDAOSyb() 
  {}

  /// \brief Constructor
  /// \param seeingTable  Seeing table
  SeeingDAOSyb(std::string seeingTable);
		   

  /// \brief Retrieves a set of seeings on a given time range
  /**
   * \param fromMjdDate       mjd start time of the retrieval
   * \param toMjdDate         mjd end time of the retrieval
   * \param rseeings          seeing values retrieved on the time range
   * \param rconnectionHandle database connection handle
   * \return the number of retrieved seeings, or -1 in case of error
   */
  int findByDate(double fromMjdDate,
		 double toMjdDate,
		 Seeing::Container& rseeings,
		 const ServiceConnectionSyb& rconnectionHandle) const;
  
private:

  /// Field Names
  static const std::string START_DATE_FIELD; 
  static const std::string INTERVAL_FIELD; 
  static const std::string FWHM_FIELD; 
  static const std::string RA_FIELD; 
  static const std::string DEC_FIELD; 
  static const std::string AIR_MASS_FIELD; 
  static const std::string TAU_FIELD; 
  static const std::string RFL_FIELD; 
  static const std::string TET_DISTANCE_FIELD; 

  /// table storing seeing values
  std::string seeingTable_;
};

#endif // SEEING_DAO_SYB_H
