// -*-c++-*-

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
// 
// PROJECT:   VLT Data Flow System
// AUTHOR:    Jens Knudstrup - ESO/DMD/DFS
// SUBSYSTEM: DFS Archive System
//
// $Name: arcsys-1_2_8 $
// $Revision: 1.16 $
// $Log: TlIngStatusTask.h,v $
// Revision 1.16  2007/12/06 08:59:10  cmoins
// adding of TestTlIngStatusTask as a friend class
//
// Revision 1.15  2007/01/11 15:35:10  jknudstr
// -
//
// Revision 1.14  2006/12/20 15:20:43  jknudstr
// -
//
// Revision 1.13  2006/10/30 08:56:51  jknudstr
// -
//
// Revision 1.12  2006/08/08 15:29:22  jknudstr
// -
//
//-----------------------------------------------------------------------------

#ifndef TLINGSTATUSTASK_H
#define TLINGSTATUSTASK_H


// Local includes
#include <GenTask.h>


/// Class that implements the TL Ingestion Status Handling Task.
/// This type of task should only run in one instance.
class TlIngStatusTask: public GenTask
{
  public:

#if defined UNIT_TEST
    friend class TestTlIngStatusTask;
#endif

    /// Constructor registering common instance of the Replication Agent Class
    /// and the output message queue to use for this task.
    TlIngStatusTask(RepAgent&     repAgent,
		    MsgQueue&     msgOutputQueue);

    /// Destructor method.
    ~TlIngStatusTask(void);

    /// ACE server loop.
    int svc(void);

    /// Method implementing the business logic of the initialization.
    void initBusiness(void);

    /// Method implementing the business logic of this server process.
    void srvBusiness(void);

    /// Return pointer to RepReqMsg, derived from associated instance of 
    /// ProTrieveArchiveRequest.
    RepReqMsg* getRepReqMsg(ProTrieveArchiveRequest&  archReq);

    /// Restore the previous state from the DB information.
    void restore();

  private: 
    ProTrieve           proTrieve_;
    string              trgDiskId_;
};


#endif // TLINGSTATUSTASK_H

// EOF

