//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//COPYRIGHT (c) 1996 European Southern Observatory
//LICENSE
//
//PROJECT:   VLT On-line Data Flow
//AUTHOR:    <Michael Wiedmer - ESO/DMD/DPG>
//SUBSYSTEM: <Core>
//
//$Revision: 1.1 $
//$Log: cor_assert_handler.h,v $
//Revision 1.1  2004/09/06 14:35:21  cguirao
//*** empty log message ***
//
//Revision 1.1  2004/06/23 12:47:09  cguirao
//First check in
//
//Revision 1.2  1998/06/30 08:49:54  dataflow
//Removing $NAME
//
//Revision 1.1  1997/04/10 08:05:44  dfadmin
//Initial revision
//
//Revision 1.4  1996/09/23 09:14:52  mwiedmer
//updated documentation
//
//Revision 1.3  1996/09/09 13:03:49  mwiedmer
//standardised
//
//Revision 1.2  1996/09/03 14:41:28  mwiedmer
//added RCS Name tag
//
//Revision 1.1  1996/09/02 11:04:06  dfadmin
//First release
//
//Revision 1.3  1996/09/02 09:45:36  mwiedmer
//first release
//
//Revision 1.2  1996/09/02 08:25:50  mwiedmer
//*** empty log message ***
//
//-----------------------------------------------------------------------------

#ifndef COR_ASSERT_HANDLER_H
#define COR_ASSERT_HANDLER_H

// includes from this library
// includes from other ESO libraries
// includes from other libraries
// system includes


// typedef, extern, enums

//=============================================================================
// Assertion utilities 
//
typedef void (* CorAssertFunc )();
typedef void (* CorAssertFuncArgs )(
  const char* filename,
  int line_number,
  const char* text
);

class CorAssertHandler;

CorAssertHandler corSetAssertHandler(CorAssertFunc);
CorAssertHandler corSetAssertHandler(CorAssertFuncArgs);
CorAssertHandler corCancelAssertHandler();

extern CorAssertHandler corAssertHandler;


//=============================================================================
// Assertion handling class.
// Private.  Please ignore.

/// Assertion checking mechanism
class CorAssertHandler {
/**
  {\footnotesize VLT On-line Data Flow, Subsystem: core} \\
  {\footnotesize $Id: cor_assert_handler.h,v 1.1 2004/09/06 14:35:21 cguirao Exp $ } \\

  {\large SYNOPSIS} \\
  \begin{verbatim}
    #include <cor_assert_handler.h> 

    CorAssertHandler handler;
  \end{verbatim}

  {\large DESCRIPTION} \\
  You can write your own assertion handling function, and tell the assertion
  system to call it whenever an assertion is violated.  The function must
  take one of two forms:
  
  1. No arguments\\
  \begin{verbatim}
    void <handler>();
  \end{verbatim}
  If an assertion fires, an error message is printed to standard output, 
  then the application handler is called.
 
  2. Arguments\\
  \begin{verbatim}
    void <handler>(const char* file, int line, const char* explanation);
  \end{verbatim}
  If an assertion fires, the application handler is called.  The first
  argument is the name of the file in which the assertion appears; the
  second argument is the line number of the assertion within the file;
  the third argument is the 'explanation' which appears in the assert
  macro.
 
  Tell the assertion system to use your handler via the call \\
  \begin{verbatim}
    corAssertHandler.set(<handler>);
  \end{verbatim}
 
  If you want to save the handler previously used, do this: \\
  \begin{verbatim}
    CorAssertHandler old = corAssertHandler.set(<handler>);
  \end{verbatim}
  after which 'old' can be subsequently used as an argument to 
  corAssertHandler.set() if desired.
 
  To resume default behaviour: \\
  \begin{verbatim}
     corAssertHandler.cancel();
  \end{verbatim}
  or \\
  \begin{verbatim}
     CorAssertHandler old = corAssertHandler.cancel();
  \end{verbatim}
 
  If no handler is set, your program will abort.

  {\large SEE ALSO} \\
  \Ref{COR_ASSERT}
*/

public:

  CorAssertHandler();

  CorAssertHandler set(
    CorAssertFunc
  );

  CorAssertHandler set(
    CorAssertFuncArgs
  );

  CorAssertHandler cancel();

  void call(
    const char* fname,
    int line,
    const char* explanation
  ) const;

private:

  CorAssertHandler(
    CorAssertFunc,
    CorAssertFuncArgs
  );

  CorAssertFunc h;

  CorAssertFuncArgs ha;
};


// inline functions
#include <cor_assert_handler.icc>

#endif

// Set mode for emacs.
// Local Variables:
// mode:C++
// End:
