// -*-c++-*-

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// COPYRIGHT (c) European Southern Observatory
// LICENSE:   ESO internal use only
//
// PROJECT:   VLT Data Flow System
// AUTHOR:    C.Moins - ESO/SDD/DFS
// SUBSYSTEM: DFS Archive System
//
// $Name: arcsys-1_2_8 $
// $Revision: 1.3 $
// $Log : dataAccessDef.h
//
//-----------------------------------------------------------------------------

#ifndef DATA_ACCESS_DEF_H
#define DATA_ACCESS_DEF_H

using namespace std;

/// namespace containing a set of common constants and types for dataAccess 
namespace dataAccessDef
{
  /// constant used when a string field is unitialized
  const string NO_INIT_STRING_FIELD = "NO_INIT";

  /// not valid media id value
  const int NOT_VALID_MEDIA_ID = -1;

  /// number of days between 01/01/1980 and mjd0
  const double MJD_1980 = 44239.;

  /** 
   * Translates MJD value into Unix time since 1980 (number of seconds since 
   * 1st January 1980 00:00:00 UTC)
   * \param mjd mjd date to convert
   */
  inline const long MjdToUnixTime80(double mjd) 
  {
    return (long)(((mjd - MJD_1980)*86400.)+0.5);
  }

}
#endif // dataAccessDef
