package org.eso.ohs.core.dbb.client;

import java.util.HashMap;

import org.eso.ohs.core.dbb.sql.DbbSqlChunkGroupMember;
import org.eso.ohs.core.dbb.xml.DbbXMLException;
import org.eso.ohs.core.utilities.MsgManager;


public class ChunkGroupRegistry extends Registry{
	
	public ChunkGroupRegistry(){
		registry = new HashMap<String, Object>();
	}
	
	/**
	 * Retrieve a DBB element by id.
	 * 
	 * @param id   Key to retrieve the chunk group member. If the input XML document
	 *             contains for instance an element like:
	 *             <pre>
	 *               &lt;<DbbCheckBox chunk="chunk20" label="Type"/> /&gt;
	 *             </pre>
	 *             this method allows to retrieve the corresponding
	 *             <code>DbbCheckBox</code> using key <em>chunk20</em>.
	 * 
	 * @return The widget registered with the input id, if any;
	 *         <tt>null</tt> otherwise.
	 */
	public DbbSqlChunkGroupMember getChunkGroup(String id ){
		/* Design by Contract */
		 if ((id == null))
			 	throw new IllegalArgumentException(MsgManager.errPrecondition("id attribute is null"));
		 /* DbC */
		return (DbbSqlChunkGroupMember) registry.get(id);
	}
	
	/**
	 * Insert a new DbbWidget in the widget registry
	 * @param key A string used to retrieve the object
	 * @param chunkGroup the DbbSqlChunkGroupMember to insert
	 * @throws DbbXMLException
	 */
	public void putChunkGroup( String key, DbbSqlChunkGroupMember chunkGroup ) throws DbbXMLException {
		 /* Design by Contract */
		 if ((key == null))
			 	throw new IllegalArgumentException(MsgManager.errPrecondition("key attribute is null"));
		 if ((chunkGroup == null))
			 	throw new IllegalArgumentException(MsgManager.errPrecondition("chunkGroup attribute is null"));
		/* DbC */ 
		if( registry.containsKey(key ) == true )
				throw new DbbXMLException(MsgManager.errmsg( "duplicate key: " + key ));
	            
		registry.put( key, chunkGroup );
	}//putElement
}
