package org.eso.ohs.core.dbb.client;

import java.awt.Cursor;
import java.awt.event.ActionEvent;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;

import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.utilities.GTreeNode;
import org.eso.ohs.core.utilities.MsgManager;
import org.eso.ohs.core.utilities.OHSConfig;

/**
 * The goal to this class is to manage loading and saving 
 * of the OHSConfig part related to
 * 1) widget displayed
 * 2) widget selected.<p>
 * This class replace many of the functionalities offered by SaveConfigAction.
 * @author G. Donino
 */
public class DbbViewConfigurationManager {

    public static final String cvsID_ =
        "$Id: DbbViewConfigurationManager.java,v 1.7 2006/04/26 09:26:44 tcanavan Exp $";

    /**
     * An OHSConfig default menu setting child: if "" the menu name is
     * in the main menu.
     */
    private static String DEF_FOLDER_KEY = "defaultMenuFolder";

    /**
     * An OHSConfig default menu setting child: the menu name
     */
    private static String DEF_NAME_KEY = "defaultMenuName";

    /**
     * An OHSConfig menu child: the configuration can't be overridden
     */
    private static String READ_ONLY = "readOnly";

    /**
     * An OHSConfig menu child: all the row widgets configuration are under it
     */
    private static String ROWS = "rows";

    /**
     * An OHSConfig menu child: all the column widgets configuration are under it
     */
    private static String COLS = "cols";

    /**
     * An OHSConfig menu child: all the sort widgets configuration are under it
     */
    private static String SORT = "sort";

    /**
     * An OHSConfig menu child: if "" the menu name is
     * in the main menu.
     */
    private static String MENU_ITEM_FOLDER = "menuRoot";

    /**
     * An OHSConfig menu child:  the menu name
     */
    private static String MENU_ITEM = "menuName";

    /**
     * paf node separator
     */
    private static String NODE_SEPARATOR = ".";

    /**
     * empty menu
     */
    private static String EMPTY_NAME = "";
    
    private static final String INVARIANT_MESSAGE = "The attribute menu is null. No Menu had beed registered to this DbbViewConfigurationManager";

    private boolean isSortHided = false;
    
    private JFrame parentFrame;
    private DbbView view;
    private String menuRoot;
    private String menuFolderKeyValue = EMPTY_NAME;
    private String helperConfFile;
//    private boolean singleMenu = false;
    private JMenu menu = null;
    private MenuMap menuMap = null;
//    private String requestDefault;
//    private final String DEFAULT = "default";
//    private final String DEFAULT_REQUEST = "Save As Default";
   

    private static Logger stdlog_ = Logger.getLogger(DbbViewConfigurationManager.class);

    /**
     * @param parentFrame the frame
     * @param menu the menu in which config is added
     * @param rowPanel The DbbWiew filter panel
     * @param colPanel The DbbWiew show panel
     * @param sortPanel The DbbWiew sorting panel 
     * @param menuRoot The menu path
     * @param helperConfFile the configuration file
     * @param name the action name is used as menu item name
     * @param singleMenu foldering of menu actions
     */
    public DbbViewConfigurationManager(
        JFrame parentFrame,
        DbbView view,
        String menuRoot,
        String helperConfFile) {

        this.parentFrame = parentFrame;
        this.view = view;
        this.menuRoot = menuRoot;
        this.helperConfFile = helperConfFile;
     //   singleMenu_ = singleMenu;
        menuMap = new MenuMap();
      //  requestDefault_ = name;
    }

    /**
     * The MenuMap container object
     * @return menuMap the menu item list of configuring widgets action.
     */
    public MenuMap getMenuMap() {
        return menuMap;
    }

   
    /**
     * Saves a configuration option under Options menu
     * into an helper file.
     * If a menu with this name is found and is not
     * in read only mode
     * (the menu has a child with name "readOnly"),
     * the configuration will be overridden
     * @param menuName the menuName to be saved
     * @return the operation went fine
     */
    public boolean addMenuToOHSConfig(String menuName) {
        boolean result = false;
        boolean okToAdd = false;
        OHSConfig stdConfig = OHSConfig.getInstance();
        Cursor currentCursor = parentFrame.getCursor();
        parentFrame.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        GTreeNode menu = menuAlreadyIn(menuName);
        //se il menu e' nullo significa che non e' alreadyIn()
        if (menu != null) {
        	//il menu e' nullo quindi esiste e lo voglio rimuovere??
        	
            // the menu you want to remove is read only
            if (menu.getChild(READ_ONLY) != null) {
                JOptionPane.showMessageDialog(
                    parentFrame,
                    "A configuration having this  \""
                        + menuName
                        + "\" name is already stored \n"
                        + "and is in read only mode: "
                        + " cannot override the configuration",
                    "Warning",
                    JOptionPane.WARNING_MESSAGE);
            } else {
                int overwrite =
                    JOptionPane.showConfirmDialog(
                        parentFrame,
                        "A configuration having this  \""
                            + menuName
                            + "\" name is already stored."
                            + "\nDo you want to overwrite it?.",
                        "Confirmation",
                        JOptionPane.YES_NO_OPTION);

                // if you want to replace, first remove it
                if (overwrite == JOptionPane.YES_OPTION) {
                    removeMenu(menu);
                    stdConfig.removeSubTree(menu.getPathName());
                    okToAdd = true;
                }
            }
        } else {
        //	il menu non c'e' quindi setto il flag per l'inserimento.
            okToAdd = true;
        }

        // adds the menu to the configuration
        if (okToAdd) {
//        	Ora faccio l'inserimento.
            int newIndexMenu = 0;

            // finds the first id not used
            while (stdConfig.getNode(menuRoot + NODE_SEPARATOR + newIndexMenu)
                != null) {
                newIndexMenu++;
            };

            // save the new configuration
            String menuRoot = this.menuRoot + NODE_SEPARATOR + newIndexMenu;
            stdConfig.setValue(menuRoot + NODE_SEPARATOR + MENU_ITEM, menuName);
            if (!menuFolderKeyValue.trim().equals(EMPTY_NAME)) {
                stdConfig.setValue(
                    menuRoot + NODE_SEPARATOR + MENU_ITEM_FOLDER,
                    menuFolderKeyValue);
            }
            
            DbbPanel rowPanel = getRowsPanel();
            if (rowPanel != null){
            	
            	appendWidgetConf(menuRoot, rowPanel, ROWS);
            }
            DbbPanel colPanel = getColumnsPanel();
            if (colPanel != null)
                appendWidgetConf(menuRoot, colPanel, COLS);
            
            DbbPanel sortPanel = getSortPanel();
            if (sortPanel != null)
                appendWidgetConf(menuRoot, sortPanel, SORT);

            // update menu popup window and savew configuration
            if (updateMenu(menuRoot) != null) {
                result = true;
            }
        }

        parentFrame.setCursor(currentCursor);
        return result;
    }

    /**
     * Deletes a menu from the OHSconfig structure and from the menu
     * item list
     * @param menuNode the node to be removed
     */
    private void removeMenu(GTreeNode menuNode) {
        GTreeNode menuNameNode = menuNode.getChild(MENU_ITEM);
        String menuName = menuNameNode.getValue();

        GTreeNode menuFolderNode = menuNode.getChild(MENU_ITEM_FOLDER);

        // if menu is in folder
        if (menuFolderNode != null
            && !menuFolderNode.getValue().trim().equals(EMPTY_NAME)) {
            String menuFolder = menuFolderNode.getValue();

            // remove menu from map
            JMenu menuFolderMenu = menuMap.getMenuFolder(menuFolder);
            JMenuItem menuItem = menuMap.removeMenuItem(menuFolder, menuName);

            //remove menu from folder in gui
            if (menuItem != null) {
                menuFolderMenu.remove(menuItem);
            }
        } else {
            // the menu is in main menu popup window,
            // remove the item from map and from menu
            JMenuItem menuItem = menuMap.removeMenuItem(menuName);
            if (menuItem != null) {
            	assert (menu != null): MsgManager.errInvariant(INVARIANT_MESSAGE);
                menu.remove(menuItem);
            }
        }
    }

    /**
     * adds a menu configuration list from the OHSconfig structure
     * @param menuPath the starting path: the children are menu configuration
     */
    public void addMenus(String menuPath) {
        OHSConfig config = OHSConfig.getInstance();
        GTreeNode node = config.getNode(menuPath);
        if (node != null) {
            for (Enumeration<GTreeNode> e = node.getChildren(); e.hasMoreElements();) {
                GTreeNode nextMenu = e.nextElement();
                String nextMenuPath = nextMenu.getPathName();
               //  ConfigWidgetValuesAction action = 
                updateMenu(nextMenuPath);
            }
        }
    }

    /**
     * apply this menu setting.
     *
     * @param menuDefaultKey the starting path:
     * the children are menu folder (defaultMenuFolder)  and name (defaultMenuName)
     */
    public void applyDefault(String menuDefaultKey) {
        ActionSuperclass defAction = null;
        OHSConfig config = OHSConfig.getInstance();
        String nameFolder =
            config.getValue(menuDefaultKey + NODE_SEPARATOR + DEF_FOLDER_KEY);
        String nameMenu =
            config.getValue(menuDefaultKey + NODE_SEPARATOR + DEF_NAME_KEY);

        if (!nameFolder.equals(EMPTY_NAME)) {
            defAction =
                (ActionSuperclass)menuMap.getActionMenu(nameFolder, nameMenu);
        } else {
            defAction = (ActionSuperclass)menuMap.getActionMenu(nameMenu);
        }
        if (defAction != null) {
            defAction.actionPerformedImpl(new ActionEvent(defAction, 0, EMPTY_NAME));
        }
    }

    /**
     * adds a menu item to the menu passed in the constructor
     * @param menuPath is the menu path to the OHSConfig that stores the
     *                  widget configurations.
     */
    private LoadViewAction updateMenu(String menuPath) {
    	assert (menuPath != null): MsgManager.errPrecondition("menuPath is null");
    	assert (menuPath.length() != 0): MsgManager.errPrecondition("menuPath is empty");
    	LoadViewAction result = null;
    	
        String menuName = null;
        String menuFolder = EMPTY_NAME;
        JMenuItem menuItemValue = null;
        OHSConfig config = OHSConfig.getInstance();
        GTreeNode menuItem = config.getNode(menuPath);
        GTreeNode colWidgets = null;
        GTreeNode rowWidgets = null;
        GTreeNode sortWidgets = null;
        GTreeNode menuNameNode = null;
        
        assert (menu != null): MsgManager.errInvariant(INVARIANT_MESSAGE);
        
        if (menuItem != null) {
            colWidgets = menuItem.getChild(COLS);
            rowWidgets = menuItem.getChild(ROWS);
            if (!isSortHided)
            	sortWidgets = menuItem.getChild(SORT);
            menuNameNode = menuItem.getChild(MENU_ITEM);
        }
        
        
  
        
   //     if (menuNameNode != null) {
        assert (menuNameNode != null): MsgManager.errPrecondition("menuNameNode is null");
        
            menuName = menuNameNode.getValue();
            result = new LoadViewAction(parentFrame, menuName, colWidgets, rowWidgets, sortWidgets, view);
            GTreeNode menuFolderNode = menuItem.getChild(MENU_ITEM_FOLDER);
            
            // search if folder in
            if (menuFolderNode != null)
            	menuFolder = menuFolderNode.getValue();
            
            if (!menuFolder.trim().equals(EMPTY_NAME)) {
                JMenu menuFolderMenu = menuMap.getMenuFolder(menuFolder);

                // the menu folder didn't exist, create it
                if (menuFolderMenu == null) {
                    stdlog_.debug("create menu folder: " + menuFolder);
                    menuFolderMenu = new JMenu(menuFolder);
                    menuMap.addMenuFolder(menuFolder, menu.add(menuFolderMenu));
                }

                // add menu to folder
                menuItemValue = menuFolderMenu.add(result);
                menuMap.addMenuItem(menuFolder, menuName, menuItemValue, result);
            } else {
                // add menu to main menu
                menuItemValue = menu.add(result);
                menuMap.addMenuItem(menuName, menuItemValue, result);
            }

   //     }
        assert (result!= null): MsgManager.errPostcondition("result is null");
        return result;
    }


    /**
     * append the widget configuration to the menuRoot
     * @param menuRoot menu path parent
     * @param dbbPanel panel to check
     * @param panelName name of the panel used in configuration
     */
    private void appendWidgetConf(
    		String menuRoot,
    		DbbPanel dbbPanel,
    		String panelName) {
           	OHSConfig stdConfig = OHSConfig.getInstance();
           	Vector<DbbWidget> rowList = dbbPanel.getWidgetList();
           	Vector<DbbWidget> rowActiveList = dbbPanel.getActiveWidgetList();
     
            // add widget configuration for the remaining widgets
            int ordIndex = rowActiveList.size();
            for (int j = 0; j < rowList.size(); j++) {
                DbbWidget widget = rowList.get(j);
                
                String widgetName = widget.getLabel().getText();
                String widgetValue = widget.get();
                if (rowActiveList.contains(widget))
                	widgetValue = widgetValue.replace('\'', ' ');
     
                String pathMenuWidget =
                    menuRoot
                        + NODE_SEPARATOR
                        + panelName
                        + NODE_SEPARATOR
                        + ordIndex
                        + NODE_SEPARATOR;
                
                stdConfig.setValue(pathMenuWidget + DbbPanel.NAME_KEY, widgetName);
                stdConfig.setValue(pathMenuWidget + DbbPanel.VALUE_KEY, widgetValue);
                stdConfig.setValue(pathMenuWidget + DbbPanel.ORDER_KEY, ordIndex);
                stdConfig.setValue(pathMenuWidget + DbbPanel.POSITION_KEY, j);
                ordIndex++;
            }
        }

    /**
     * Checks if the menu is already stored.
     * @param menuName menu to be stored
     * @return null if menu is not stored
     */
    private GTreeNode menuAlreadyIn(String menuName) {
        OHSConfig stdConfig = OHSConfig.getInstance();
        GTreeNode menusRootNode = stdConfig.getNode(menuRoot);
        String nextMenuName = EMPTY_NAME;
        String nextMenuNameFolder = EMPTY_NAME;
        if (menusRootNode != null) {
            for (Enumeration<GTreeNode> e = menusRootNode.getChildren(); e.hasMoreElements();) {

                GTreeNode nextMenu = e.nextElement();
                GTreeNode menuNameNode = nextMenu.getChild(MENU_ITEM);
                GTreeNode menuFolderNode = nextMenu.getChild(MENU_ITEM_FOLDER);
                if (menuNameNode != null) {
                    nextMenuName = menuNameNode.getValue();
                } else {
                    nextMenuName = EMPTY_NAME;
                }
                if (menuFolderNode != null) {
                    nextMenuNameFolder = menuFolderNode.getValue();
                } else {
                    nextMenuNameFolder = EMPTY_NAME;
                }
                if (nextMenuNameFolder.trim().equals(menuFolderKeyValue.trim())) {
                    if (!nextMenuName.equals(EMPTY_NAME)
                        && nextMenuName.equals(menuName)) {
                        return nextMenu;
                    }
                }
            }
        }
        return null;
    }
    

	public String getHelperConfFile() {
		return helperConfFile;
	}

	public DbbPanel getColumnsPanel() {
		return view.getColumnsPanel();
	}


	public DbbPanel getRowsPanel() {
		return view.getRowsPanel();
	}

	public DbbPanel getSortPanel() {
		return view.getSortPanel();
	}

	public DbbView getView() {
		return view;
	}

	public void saveConfig() {
		OHSConfig config = OHSConfig.getInstance();
        config.saveSubTreeToPafFile(helperConfFile, menuRoot);
	}

	public void setMenuFolderKeyValue(String menuFolderKeyValue) {
		this.menuFolderKeyValue = menuFolderKeyValue;
	}

	public JMenu getMenu() {
		return menu;
	}

	public void setMenu(JMenu menu) {
		this.menu = menu;
	}

	public boolean isSortHided() {
		return isSortHided;
	}

	public void setSortHided(boolean isSortHided) {
		this.isSortHided = isSortHided;
	}
	
}

