package org.eso.ohs.core.dbb.client;

import java.awt.Component;
import java.awt.Cursor;
import java.awt.event.ActionEvent;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Enumeration;

import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JTextField;

import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.widgets.SimpleInputDialog;
import org.eso.ohs.core.utilities.GTreeNode;
import org.eso.ohs.core.utilities.OHSConfig;

/**
 * Delete a configuration option under Options menu
 * previously saved in the helper file.
 * Once the deleting is performed, it override the configuration file
 * with the new configuration.
 * Usually it shares with the object SaveConfigAction a MenuMap
 * object.This is necessary for managing the menuItem lifecycle.
 * External keywords used: (for the meaning see the class)
 * SaveConfigAction.EMPTY_NAME
 * SaveConfigAction.MENU_ITEM
 * SaveConfigAction.MENU_ITEM_FOLDER
 * <br>
 * @see SaveConfigAction
 * @see MenuMap
 * <br>
 * @author D. Dorigo
 * @author cmavros
 * @version $Revision: 1.5 $
 */
public class DelConfigAction extends ActionSuperclass {

    public static final String cvsID_ =
        "$Id: DelConfigAction.java,v 1.5 2006/04/26 09:26:44 tcanavan Exp $";

    private JFrame parentFrame_;
    // private DbbPanel rowPanel_;
    // private DbbPanel colPanel_;
    private String menuFolderKeyValue_ = "";
    private boolean singleMenu_ = false;
    // private DbbPanel sortPanel_;
    private String menuRoot_;
    private String helperConfFile_;
    private MenuMap menuMap_ = null;
    private JMenu menu_ = null;
    private static Logger stdlog_ = Logger.getLogger(DelConfigAction.class);

    /**
     * @param parentFrame the frame
     * @param menu under which the menuItems are placed
     * @param menuRoot
     * @param helperConfFile the configuration file
     * @param name the action name is used as menu item name
     * @param singleMenu foldering of menu actions
     */
    public DelConfigAction(
        JFrame parentFrame,
        JMenu menu,
        String menuRoot,
        String helperConfFile,
        String name,
        boolean singleMenu) {
        super(parentFrame, name);
        parentFrame_ = parentFrame;
        menuRoot_ = menuRoot;
        singleMenu_ = singleMenu;
        menu_ = menu;
        menuMap_ = new MenuMap();
        helperConfFile_ = helperConfFile;
    }

    /**
     * action starting method
     * @param ev       user-initiated ActionEvent
     */
    public void actionPerformedImpl(ActionEvent ev) {
        Cursor currentCursor = parentFrame_.getCursor();
        parentFrame_.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        String menuName = getMenuName();
        if (menuName != null) {
            boolean delMenu = removeMenu(menuName);
            // subtree saved to paf file
            if (delMenu) {
                OHSConfig config = OHSConfig.getInstance();
                config.saveSubTreeToPafFile(helperConfFile_, menuRoot_);
            }
        }
        parentFrame_.setCursor(currentCursor);
    }

    /**
     * Deletes a menu from the OHSconfig structure and from the menu item list
     *
     * @param menuName the name (the folder name is in an attribute)
     * @return execution was successful
     */
    protected boolean removeMenu(String menuName) {
        boolean done = false;
        OHSConfig stdConfig = OHSConfig.getInstance();
        GTreeNode menuNode = stdConfig.getNode(menuRoot_);
        String menuFolder = SaveConfigAction.EMPTY_NAME;
        String menuNameValue = SaveConfigAction.EMPTY_NAME;
        GTreeNode nextMenu = null;
        stdlog_.debug("MENU to be deleted: " + menuName);
        String msg =
            "Are you sure you want to delete the configuration \""
                + menuName
                + "\" ?";

        // search all the nodes in the menu tree
        for (Enumeration<GTreeNode> e = menuNode.getChildren(); e.hasMoreElements();) {
            nextMenu = e.nextElement();

            GTreeNode menuNameNode = nextMenu.getChild(SaveConfigAction.MENU_ITEM);
            GTreeNode menuFolderNode =
                nextMenu.getChild(SaveConfigAction.MENU_ITEM_FOLDER);
            if (menuFolderNode != null) {
                stdlog_.debug("menuFolder \"" + menuFolder + "\"");
                menuFolder = menuFolderNode.getValue();
            } else {
                menuFolder = SaveConfigAction.EMPTY_NAME;
            }
            if (menuNameNode != null) {
                stdlog_.debug("menu name " + menuNameValue);
                menuNameValue = menuNameNode.getValue();
            }

            // if the menu is found
            if (menuFolder.trim().equals(menuFolderKeyValue_.trim())
                && menuNameValue.trim().equals(menuName.trim())) {

                // if menu is in folder
                if (!menuFolder.trim().equals(SaveConfigAction.EMPTY_NAME)) {
                    stdlog_.debug("menuFolder " + menuFolder);

                    // remove menu from map
                    JMenu menuFolderMenu = menuMap_.getMenuFolder(menuFolder);
                    int okToChange =
                        JOptionPane.showConfirmDialog(
                            null,
                            msg,
                            "Confirm delete ",
                            JOptionPane.YES_NO_OPTION);
                    if (okToChange == JOptionPane.YES_OPTION) {
                        JMenuItem menuItem =
                            menuMap_.removeMenuItem(menuFolder, menuName);
                        // remove menu from folder in gui
                        if (menuItem != null) {
                            menuFolderMenu.remove(menuItem);
                            done = true;
                        }
                        // remove folder if empty
                        if (menuFolderMenu.getItemCount() == 0) {
                            menu_.remove((Component)menuFolderMenu);
                            menuMap_.removeMenuFolder(menuFolderMenu.getText());
                            stdlog_.debug(
                                "removed the folder \"" + menuFolder + "\" menu ");
                        }
                    }
                } else {
                    // the menu is in main menu popup window,
                    // remove the item from map and from menu
                    JMenuItem menuItem = menuMap_.removeMenuItem(menuName);
                    if (menuItem != null) {
                        int okToChange =
                            JOptionPane.showConfirmDialog(
                                null,
                                msg,
                                "Confirm delete ",
                                JOptionPane.YES_NO_OPTION);
                        if (okToChange == JOptionPane.YES_OPTION) {
                            menu_.remove(menuItem);
                            done = true;
                        }
                    }
                }
                break;
            }
        }
        // drop OHSconfig menu info
        if (done) {
            stdConfig.removeSubTree(nextMenu.getPathName());
        } else {
            JOptionPane.showMessageDialog(
                null,
                "Menu " + menuName + " not found",
                "Warning ",
                JOptionPane.YES_NO_OPTION);
        }
        return done;
    }

    /**
     * The MenuMap should be the same used by SaveConfigAction object.
     *
     * @param menuMap the menu item list of configuring widgets action.
     */
    public void setMenuMap(MenuMap menuMap) {
        menuMap_ = menuMap;
    }

    /**
     * Request menu from user.
     * If the nothing is inputted return null.
     *
     * @return String The configuration menu name to be added
     */
    protected String getMenuName() {
        String menuName = null;
        if (!singleMenu_)
            menuName = getMenuViaDialog();
        else
            menuName = getSingleMenuViaDialog();

        // if not cancel op but menu empty
        if (menuName != null && menuName.length() == 0) {
            JOptionPane.showMessageDialog(
                parentFrame_,
                "The menu name is empty",
                "Warning",
                JOptionPane.WARNING_MESSAGE);
            menuName = null;
        }
        return menuName;
    }

    /**
     * Starts an menu dialog in which the client inserts the menu name
     */
    protected String getSingleMenuViaDialog() {
        menuFolderKeyValue_ = SaveConfigAction.EMPTY_NAME;
        JTextField newNameTF = new JTextField();
        // JTextField newRootNameTF = new JTextField();
        SimpleInputDialog dlg =
            new SimpleInputDialog(
                parentFrame_,
                "Delete Settings",
                "Enter details:\n",
                new String[] { "Settings Name:" },
                new Component[] { newNameTF });
        dlg.show();
        if (dlg.getUserChoice() == JOptionPane.OK_OPTION) {
            return newNameTF.getText();
        } else {
            return null; // Cancelled or closed window?
        }
    }

    /**
     * Writes the modified buffer (after the removing
     * of the option) to the helper config file.
     *
     * @param newFileConfigBuffer buffer to be writed.
     *                      If is null the file is not touch.
     * @return true if file is written successfully
     */
    protected boolean writeConfigFile(StringBuffer newFileConfigBuffer) {
        boolean result = true;
        try {
            if (newFileConfigBuffer != null) {
                PrintWriter out = new PrintWriter(new FileWriter(helperConfFile_));
                String body = newFileConfigBuffer.toString();
                out.println(body);
                out.close();
            } else {
                result = false;
            }
        } catch (IOException e) {
            stdlog_.error("File \"" + helperConfFile_ + "\" not writed", e);
            result = false;
        }
        return result;
    }

    /**
     * Starts an menu dialog in which the client inserts the menu name
     */
    public String getMenuViaDialog() {
        menuFolderKeyValue_ = SaveConfigAction.EMPTY_NAME;
        JTextField newNameTF = new JTextField();
        JTextField newRootNameTF = new JTextField();
        SimpleInputDialog dlg =
            new SimpleInputDialog(
                parentFrame_,
                "Delete Settings",
                "Enter details:\n",
                new String[] { "Folder Name <optional>:", "Settings Name:" },
                new Component[] { newRootNameTF, newNameTF });
        dlg.show();
        if (dlg.getUserChoice() == JOptionPane.OK_OPTION) {
            menuFolderKeyValue_ = newRootNameTF.getText();
            return newNameTF.getText();
        } else {
            return null; // Cancelled or closed window?
        }
    }
}

