package  org.eso.ohs.core.dbb.client;

import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Vector;

import javax.swing.JFrame;

import org.apache.log4j.Logger;
import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.utilities.GTreeNode;

/**
 * 
 * @author D. Dorigo
 * @author cmavros 24May02 Rearranged code to make methods more accessable to
 *          to the DefaultConfigAction().
 *
 */

public class LoadViewAction extends ActionSuperclass {

    private static final String rcsid = "$Id: LoadViewAction.java,v 1.3 2006/04/26 09:26:44 tcanavan Exp $";
 
    private final String NAME_KEY = "name";
    private final String POSITION_KEY = "position";
          
    private static Logger stdlog_ = 
	Logger.getLogger( LoadViewAction.class );
 	  GTreeNode colNames_; 
	 GTreeNode rowNames_;  
	 GTreeNode sortNames_; 
    DbbView view_;
    public LoadViewAction(JFrame viewProWin ,String name , 
				    GTreeNode colNames,
				    GTreeNode rowNames, 
				    GTreeNode sortNames ,
				    DbbView view) {
	     super(viewProWin, name );
	     colNames_  = colNames; 
	     rowNames_  = rowNames; 
	     sortNames_  = sortNames;
	     view_ = view;	     
	 }
	    
	 public final void actionPerformedImpl(final ActionEvent ev) {			  
		 Vector<DbbWidget> colWidgetsFull = view_.getColumnsPanelFull().getWidgetList();
		 Vector<DbbWidget> rowWidgetsFull = view_.getRowsPanelFull().getWidgetList();
		 Vector<DbbWidget> sortWidgetsFull = view_.getSortPanelFull().getWidgetList();
		 if(colNames_ != null ){
			 ArrayList<String> visibleColsList = gTreeToArrayList(colNames_, colWidgetsFull);
			 DbbPanel colPanel = view_.getColumnsPanelSubset(visibleColsList);
			 colPanel.setValueWidgets( colNames_);
			 view_.setColumnsPanel(colPanel, "North", true, 1, 1);
		 }
             if(rowNames_ != null ){
            	 ArrayList<String> visibleRowsList = gTreeToArrayList(rowNames_, rowWidgetsFull );
            	 DbbPanel rowPanel = view_.getRowsPanelSubset(visibleRowsList);
            	 rowPanel.setValueWidgets(rowNames_);
            	 view_.setWherePanel(rowPanel, "North", true, 1, 20);
             }
             if(sortNames_ != null ){
            	ArrayList<String> visibleSortList = gTreeToArrayList(sortNames_, sortWidgetsFull);
             	DbbPanel sortPanel = view_.getSortPanelSubset(visibleSortList);
             	sortPanel.setValueWidgets(sortNames_);
             	view_.setSortPanel(sortPanel, "West", true, 1, 1);
             }
	 }
	 
	 private ArrayList<String> gTreeToArrayList(GTreeNode treeNode, Vector<DbbWidget> widgetsListFull){
		ArrayList<String> result = new ArrayList<String>();
		HashMap<String, String> map = new HashMap<String, String>();
    	for (Enumeration<GTreeNode> e = treeNode.getChildren() ; e.hasMoreElements() ;){
    		GTreeNode nextWidget = e.nextElement();
			GTreeNode nameNode = nextWidget.getChild(NAME_KEY);
			GTreeNode positionNode = nextWidget.getChild(POSITION_KEY);
			String name = nameNode.getValue();
			//Older saves have no position informations!
			String position;
			if (positionNode == null)
				position = getDefaultPostion(name, widgetsListFull);
			else
				position = positionNode.getValue();
			map.put(position, name);
    	}
	    for (int i = 0; i < map.size(); i++){
	    	String keyPosition = String.valueOf(i);
	    	String value = map.get(keyPosition);
	    	if (value != null)
	    		result.add(value);
	    	else
	    		System.out.println();
	    }
	    return result;
	 }
	 
	 /**
	  * if the widget has no position, then get the position from the fullPanel ArrayList
	  * which preserve the sorting of the viewPro.xml file
	  */
	 private String getDefaultPostion(String name, Vector<DbbWidget> list) {
		String result = "-1";
		for (int i = 0; i < list.size(); i++){
			DbbWidget widget = list.get(i);
			String widgetName = widget.getLabel().getText();
			if (name.equals(widgetName)){
				result = String.valueOf(i);
				break;
			}
		}
		 
		return result;
	}

	private void printList(ArrayList list){
		 for (int i = 0; i < list.size(); i++){
			 stdlog_.debug(i+") " + list.get(i));
		 }
	 }
}

