package org.eso.ohs.core.dbb.client;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;

import org.eso.ohs.core.dbb.server.DbbQuery;
import org.eso.ohs.core.dbb.server.DbbQueryImpl;
import org.eso.ohs.core.utilities.ObjectIOException;

public class MemorySavingPagination extends Paginator {

	final private String ASC = " ASC";
	final private String DESC = " DESC";
	//It is an arrayList of rows (rows according to the DbbSQlResult Definition)
	PaginationCache cache = null;
	
	public MemorySavingPagination(SqlElements elements, String url, String userId, String password, String database) throws ObjectIOException, SQLException {
		super(elements, url, userId, password, database);
		sql = buildQuery();
		count = countItems();	
		cache = new PaginationCache();
		
	}

	private String buildQuery() {
		StringBuffer result = new StringBuffer();
		StringBuffer bufCount = new StringBuffer();
		
		ArrayList<Object> select = elements.getSelect();
		ArrayList<Object> from = elements.getFrom();
		ArrayList<Object> orderBy =  buildOrderBy(elements.getOrderBy(), select);
		ArrayList where = buildWhere (elements.getWhere(), orderBy, select);
		
		result.append("SELECT ");
		bufCount.append("SELECT count(*) ");
		
		for (int i = 0; i < select.size(); i++){
			result.append(select.get(i)).append(", ");
		}
		result.delete(result.length()-2, result.length()-1);
		
		result.append("FROM ");
		bufCount.append("FROM ");
		
		for (int i = 0; i < from.size(); i++){
			result.append(from.get(i)).append(", ");
			bufCount.append(from.get(i)).append(", ");
		}
		result.delete(result.length()-2, result.length()-1);
		bufCount.delete(bufCount.length()-2, bufCount.length()-1);
		
		result.append("WHERE ");
		bufCount.append("WHERE ");
		
		
		for (int i = 0; i < where.size(); i++){
			result.append(where.get(i)).append(", ");
			bufCount.append(where.get(i)).append(", ");
		}
		result.delete(result.length()-2, result.length()-1);
		bufCount.delete(bufCount.length()-2, bufCount.length()-1);
		sql = result.toString();
		sqlCount =  bufCount.toString();
		
		result.append("ORDER BY ");
		
		//Normal orderBy Clause
		
		StringBuffer tmp = new StringBuffer();
		for (int i = 0; i < orderBy.size(); i++){
			tmp.append(orderBy.get(i)).append(", ");
		}
		tmp.delete(tmp.length()-2, tmp.length()-1);
		this.orderBy = tmp.toString();
		
//		Normal orderByReversed Clause
		tmp = new StringBuffer();
		for (int i = 0; i < orderBy.size(); i++){
			String str = orderBy.get(i).toString();
			if ( str.indexOf("ASC") > 0){
				str = str.replaceAll("ASC", "DESC");
			}else{
				str = str.replaceAll("DESC", "ASC");
			}
			tmp.append(str).append(", ");
		}
		tmp.delete(tmp.length()-2, tmp.length()-1);
		this.orderByRev = tmp.toString();
		
		
		return result.toString();
	}

	private ArrayList buildWhere(ArrayList<Object> where, ArrayList<Object> orderBy, ArrayList<Object> select) {
		ArrayList result = new ArrayList();
		StringBuffer tmpClause = new StringBuffer();
		
		//we are assuming that the elements in select and orderBy are in the same order
		for (int i = 0; i < orderBy.size(); i++){
		
			
//			add all the fields that we have already used; ex: field1 = ?, field2 = ?,
			StringBuffer cClause = new StringBuffer("NOT (");
			
//			add the new field using > or < condition
			String str = (String)select.get(i);
			cClause.append(tmpClause.toString()).append(str);
			if (((String)orderBy.get(i)).endsWith(ASC)){
				cClause.append(" > ?)");
			}else{
				cClause.append(" < ?)");
			}
			//do not forget to use this field in the next conditions!!!
			tmpClause.append(str).append(" = ? AND ");	
			where.add(cClause.toString());
		}
		
		return result;
	}

	@SuppressWarnings("unchecked")
	private ArrayList<Object> buildOrderBy(ArrayList<Object> orderBy, ArrayList<Object> select) {
		ArrayList<String> tmp = new ArrayList<String>();
		for (int i = 0; i < select.size(); i++){
			String cSelect = (String) select.get(i);
			boolean found = false;
			for (int k = 0; k < orderBy.size() && found == false; k++){
				String cOrder = (String) orderBy.get(k); 
				if (cOrder.startsWith(cSelect)) found = true; 
			}
			if (found == false){
				tmp.add(cSelect + ASC);
			}
		}
		orderBy.addAll(tmp);
		
		
		return orderBy;
	}

	public DbbQuery moveTo(int pageNumber, int pageSize) throws SQLException {
		DbbQuery result = null;
		int totalPages = (count + (pageSize - 1))/pageSize;
		int startRow = 0;
		int endRow = 0;
		setReversed(false);
		String sql = null;
		
		//the page is in the first half od the result set?
		if (pageNumber > totalPages/2 + 1){
			pageNumber = totalPages + 1 - pageNumber;
			setReversed(true);
			sql = this.sql + orderByRev;
		}else{
			sql = this.sql + orderBy;
		}
		
		startRow = pageSize * (pageNumber - 1) + 1;
		endRow = startRow + pageSize - 1;
		//Statement stmt = con.createStatement();
		String setRowCount = "set rowcount " + endRow;
		System.out.println(setRowCount);
		stmt.execute(setRowCount);
		System.out.println(sql);
		PreparedStatement prepStmt = con.prepareStatement(sql);
		ResultSet queryResult = prepStmt.executeQuery();
		
		//ResultSet queryResult = db.execQuery(sql, stmt);
		
		result = new DbbQueryImpl(queryResult, session, startRow);
		
		return result;
	}

}
