package org.eso.ohs.core.dbb.client;

import java.util.ArrayList;

import org.eso.ohs.core.utilities.MsgManager;

public class PaginationCache {
 
	private ArrayList<ArrayList<Comparable>> cache = new ArrayList<ArrayList<Comparable>>();
	private Class[] dataType = null;
	
	
	public void addRow(Object[] data){
		if(data == null){ throw new IllegalArgumentException(MsgManager.errPrecondition("data is null"));}
		if(data.length == 0){ throw new IllegalArgumentException(MsgManager.errPrecondition("data is empty"));}
		ArrayList<Comparable> row = convertRow(data);
		cache.add(row);	
	}
	
	private ArrayList<Comparable> convertRow(Object[] data) {
		assert (data != null): MsgManager.errPrecondition("data is null");
		assert (data.length != 0): MsgManager.errPrecondition("data is empty");
		ArrayList<Comparable> result = new ArrayList<Comparable>();
		assert (hasDataType()): MsgManager.errInvariant("dataType is null");
		
		for (int i = 0; i < data.length; i++) {
			if (Integer.class.isAssignableFrom(dataType[i])){
				Integer value = (Integer) data[i];
				result.add(value);
			}else if (String.class.isAssignableFrom(dataType[i])){
				String value = (String) data[i];
				result.add(value);
			}
		}
		return result;
	}

	public boolean contains(Object[] data){
		if(data == null){ throw new IllegalArgumentException(MsgManager.errPrecondition("data is null"));}
		if(data.length == 0){ throw new IllegalArgumentException(MsgManager.errPrecondition("data is empty"));}
		
		boolean result = false;
		
		ArrayList<Comparable> lookFor = convertRow(data);
		result = cache.contains(lookFor);
		
		return result;
	}
	
	//it does not not work just like that.
	/*public int indexOf(Object[] data){
		if(data == null){ throw new IllegalArgumentException(MsgManager.errPrecondition("data is null"));}
		if(data.length == 0){ throw new IllegalArgumentException(MsgManager.errPrecondition("data is empty"));}
		int result = -1;
		
		assert (hasDataType()): MsgManager.errInvariant("dataType is null");
		result = cache.indexOf(data);
		
		return result;
	}*/
	
	//This is useless and almost impossible to achieve. 
	//We store elaborated data and not plain object
	/*public Object[] getDataObjectAt(int index){
		return (Object[]) cache.get(index);
	}*/
	
	public ArrayList getRowAt(int index){
		if (index < 0){ throw new IllegalArgumentException(MsgManager.errPrecondition("index must be >= 0"));}	
		return cache.get(index);
	}
	
	public ArrayList<ArrayList<Comparable>> getData(){
		return (ArrayList<ArrayList<Comparable>>) cache;
	}
	
	public boolean hasDataType(){
		boolean result = false;
		
		if (dataType != null){
			result = true;
		}
		
		return result;
	}

	public Class[] getDataType() {
		return dataType;
	}

	public void setDataType(Class[] dataType) {
		if(dataType == null){ throw new IllegalArgumentException(MsgManager.errPrecondition("dataType is null"));}
		if(dataType.length == 0){ throw new IllegalArgumentException(MsgManager.errPrecondition("dataType is empty"));}
	
		this.dataType = dataType;
	}
	
}
