package org.eso.ohs.core.dbb.htmlclient;

import org.eso.ohs.core.dbb.sql.DbbSqlChunk;

/**
 * This is an implementation of the DbbHtmlWidget abstract class,
 * defining a Check box widget.
 * <br>
 * @author amchavan, ESO 09-Dec-2003
 * @version $Revision: 1.2 $
 */

// $Id: DbbHtmlCheckBox.java,v 1.2 2006/03/07 10:29:04 gdonino Exp $

public class DbbHtmlCheckBox extends DbbHtmlWidget {


	public static final String cvsID_ =
		"$Id: DbbHtmlCheckBox.java,v 1.2 2006/03/07 10:29:04 gdonino Exp $";

	
	private boolean checked = false;
	/**
	 * Create a DbbHtmlCheckBox associated with a specific chunk.
	 *
	 * @param chunk DbbSqlChunk object.
	 */
	public DbbHtmlCheckBox( DbbSqlChunk chunk ) {
		this( chunk.getLabel(), chunk, false );
	}

	/**
	 * Create a DbbHtmlCheckBox, with a specific label, associated with a
	 * specific chunk
	 *
	 * @param labeltext Label text to be displayed.
	 * @param chunk     DbbSqlChunk object.
	 */
	public DbbHtmlCheckBox(String labeltext, DbbSqlChunk chunk, boolean checked) {
		super( labeltext, chunk );
		setDefault( "" );
		this.checked = checked;
	}
	

	/** Implementation of the abstract method */
	public String render( int indentLevel ) {
        String     name = getName(); 
		StringBuffer sb = new StringBuffer();
		
		if (checked){
			sb
			.append(indentSpaces(indentLevel))
			.append("<td align=\"right\"> ")
			.append(getLabel())
			.append(" </td>\n")
			.append(indentSpaces(indentLevel))
			.append("<td> ")
			.append("<input type=\"checkbox\" name=\"")
			.append( name )
			.append("\" checked> </td>\n");
			
		}else{
		sb
			.append(indentSpaces(indentLevel))
			.append("<td> ")
			.append(getLabel())
			.append(" </td>\n")
			.append(indentSpaces(indentLevel))
			.append("<td> ")
			.append("<input type=\"checkbox\" name=\"")
			.append( name )
			.append("\"> </td>\n");
		}
		
		return sb.toString();
	}

	/**
	 * Set the widget's value (implementation of the abstract method).
	 *
	 * @param val Can be "on", meaning <code>true</code>, 
     *            or an empty string, meaning <code>false</code>.
	 */
	public void set( String val ) {

        String      v = val.toLowerCase();
        DbbSqlChunk c = getSqlChunk();
        
        if( v.equals( "on" )) {
            c.setOperator( OP_EQUALS );
            c.setActive( true );
            v =  "true";
        } 
        else if( v.length() == 0 ) {
            c.setOperator( null );
            c.setActive( false );
            v = "false";
        } 
        else
            throw new RuntimeException( "Invalid value: '" + val + "'" );

        value_ = v;
        c.setValue( v );
	}
}
