/* Created on Dec 9, 2003 */

package org.eso.ohs.core.dbb.htmlclient;

import java.net.MalformedURLException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import javax.servlet.http.HttpServletRequest;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.client.BasicPaginationOpt;
import org.eso.ohs.core.dbb.client.DbbSqlPagingInfo;
import org.eso.ohs.core.dbb.client.Paginator;
import org.eso.ohs.core.dbb.server.DbbQuery;
import org.eso.ohs.core.dbb.server.DbbSqlEngine;
import org.eso.ohs.core.dbb.server.DbbSqlEngineImpl;
import org.eso.ohs.core.dbb.sql.DbbSqlChunk;
import org.eso.ohs.core.dbb.sql.DbbSqlQueryChunks;
import org.eso.ohs.core.dbb.sql.DbbSqlQueryResult;
import org.eso.ohs.core.utilities.MsgManager;

/**
 * TODO: javadoc
 * 
 * @author amchavan
 * @version $Revision: 1.6 $
 */

// $Id: DbbHtmlView.java,v 1.6 2006/04/26 09:26:44 tcanavan Exp $

public class DbbHtmlView {

	
	public static final String cvsID_ =
	"$Id: DbbHtmlView.java,v 1.6 2006/04/26 09:26:44 tcanavan Exp $";

	/** Width of the table borders */
	private static final int TABLEBORDER = 0;

	/** Number of displayed columns */
	private int cols = -1;
	
	/**  size of the resultsets */
	private int itemsCount = -1;

	/**
	 * DbbPanel for selecting which columns will be displayed
	 */
	protected DbbHtmlPanel columnsPanel_ = null;

	/** List of panels added "North" in the view. */
	private List<DbbHtmlPanel> north = null;

	/** List of panels added "South" in the view. */
	private List<DbbHtmlPanel> south = null;

	/** List of panels added "East" in the view. */
	private List<DbbHtmlPanel> east = null;

	/** List of panels added "West" in the view. */
	private List<DbbHtmlPanel> west = null;

	/**
	 * DbbHtmlPanel for the widgets that will generate the "ORDER BY" clauses
	 */
	protected DbbHtmlPanel sortPanel_ = null;
	
	/** SQL engine associated to this view. */
	private DbbSqlEngine sqlEngine_;

	/**
	 * DbbHtmlPanel for selecting which rows will be retrieved; i.e., the
	 * widgets that will generate the "WHERE" clauses.
	 */
	protected DbbHtmlPanel wherePanel_ = null;
	
	private DbbSqlPagingInfo sqlPagingInfo = new DbbSqlPagingInfo();
	
	private static Logger stdlog_ = Logger.getLogger(DbbHtmlView.class);
	
	private Paginator paginator = null;

	private boolean isReversed;
	/**
	 * Build an HTML database browser view
	 *
	 * @param sqlEngine Reference to the SQL engine which does all the work
	 */
	public DbbHtmlView( DbbSqlEngine sqlEngine ) {
		sqlEngine_ = sqlEngine;
		
		north = new ArrayList<DbbHtmlPanel>();
		south = new ArrayList<DbbHtmlPanel>();
		east  = new ArrayList<DbbHtmlPanel>();
		west  = new ArrayList<DbbHtmlPanel>();
	}

	/**
	 * @return The columns selection panel
	 */
	public DbbHtmlPanel getColumnsPanel() {
		return columnsPanel_;
	}

	/**
	 * @return The sort panel
	 */
	public DbbHtmlPanel getSortPanel() {
		return sortPanel_;
	}

	/**
	 * @return The rows selection panel
	 */
	public DbbHtmlPanel getWherePanel() {
		return wherePanel_;
	}

	/**
	 * Generate a fragment of HTML code rendering this view. The fragment is
	 * suitable to be included in the HTML rendering of an HTML form.
	 * <BR>
	 * The generated fragment is not indented.
	 * 
	 * @return An HTML fragment rendering this widget.
	 */
	public String render() {
		return render( 0 );
	}

	/**
	 * Generate a fragment of HTML code rendering this panel. The fragment is
	 * suitable to be included in the HTML rendering of an HTML form.
	 * 
	 * @param indentLevel
	 *            The level of indentation for the HTML fragment. Each level is
	 *            indented by INDENTSPACES spaces.
	 * 
	 * @return An HTML fragment rendering this widget.
	 * 
	 * @see DbbHtmlWidget#INDENTSPACES
	 */
	public String render( int indentLevel ) {
		
		
		String ind0 = DbbHtmlWidget.indentSpaces( indentLevel     );
		String ind1 = DbbHtmlWidget.indentSpaces( indentLevel + 1 );
		String ind2 = DbbHtmlWidget.indentSpaces( indentLevel + 2 );
		String ind3 = DbbHtmlWidget.indentSpaces( indentLevel + 3 );
		String ind4 = DbbHtmlWidget.indentSpaces( indentLevel + 4 );
		
		StringBuffer sb = new StringBuffer();
		Iterator<DbbHtmlPanel> panels = null;
		
		sb.append( ind0 )
			.append( "<table border=\"" ).append(TABLEBORDER).append("\">\n" );
		
		// North
		//---------------------------------------
		sb.append( ind1 ).append( "<tr> <!-- North -->\n" );		
		panels = north.iterator();
		while ( panels.hasNext()) {
			DbbHtmlPanel panel = panels.next();
			
			sb.append( ind2 ).append( "<td colspan=\"2\">\n" );
			sb.append( panel.render( indentLevel + 3 ));
			sb.append( ind2 ).append( "</td>\n" );
		}
		sb.append( ind1 ).append( "</tr> <!-- North -->\n" );
		
		
		sb.append( ind1 ).append( "<tr> <!-- East & West -->\n" );
		
		// West
		//---------------------------------------
		sb.append( ind2 ).append( "<td> <!-- West -->\n" );
		sb.append( ind3 )
			.append( "<table border=\"" ).append(TABLEBORDER).append("\">\n" );
		panels = west.iterator();
		while ( panels.hasNext()) {
			DbbHtmlPanel panel = panels.next();
			sb.append( ind4 ).append( "<tr><td>\n" );
			sb.append( panel.render(indentLevel + 5 ));
			sb.append( ind4 ).append( "</td></tr>\n" );
		}
		sb.append( ind3 ).append( "</table>\n" );
		sb.append( ind2 ).append( "</td> <!-- West -->\n" );

		// East
		//---------------------------------------
		sb.append( ind2 ).append( "<td> <!-- East -->\n" );
		sb.append( ind3 )
			.append( "<table border=\"" ).append(TABLEBORDER).append("\">\n" );
		panels = east.iterator();
		while ( panels.hasNext()) {
			DbbHtmlPanel panel = panels.next();
			sb.append( ind4 ).append( "<tr><td>\n" );
			sb.append( panel.render(indentLevel + 5 ));
			sb.append( ind4 ).append( "</td></tr>\n" );
		}
		sb.append( ind3 ).append( "</table>\n" );
		sb.append( ind2 ).append( "</td> <!-- East -->\n" );
		
		
		sb.append( ind1 ).append( "</tr> <!-- East & West -->\n" );	
		
		// South
		//---------------------------------------
		
		sb.append( ind1 ).append( "<tr> <!-- South -->\n" );		
		panels = south.iterator();
		while ( panels.hasNext()) {
			DbbHtmlPanel panel = panels.next();
			
			sb.append( ind2 ).append( "<td colspan=\"2\">\n" );
			sb.append( panel.render( indentLevel + 3 ));
			sb.append( ind2 ).append( "</td>\n" );
		}
		sb.append( ind1 ).append( "</tr> <!-- South -->\n" );
		
		
		sb.append( ind0 ).append( "</table>\n" );
		
		return sb.toString();
	}
	
	/**
	 * Set a column selection panel
	 *
	 * @param panel     DbbHtmlPanel defing a column selection panel
	 * @param position  Relative position in the view (NSEW)
	 */
	public void setColumnsPanel(DbbHtmlPanel panel, String position) {
		setColumnsPanel(panel, position, true, -1, -1);
	}

	/**
	 * Set a column selection panel
	 *
	 * @param panel     DbbHtmlPanel defing a column selection panel
	 * @param position  Relative position in the view (NSEW)
	 * @param visible   specifies if the panel is visible
	 */
	public void setColumnsPanel(DbbHtmlPanel panel, String position, boolean visible) {
		setColumnsPanel(panel, position, visible, -1, -1);
	}

	/**
	 * Set a column selection panel
	 *
	 * @param panel     DbbHtmlPanel defing a column selection panel
	 * @param position  Relative position in the view (NSEW)
	 * @param visible   specifies if the panel is visible
	 * @param x         Ignored
	 * @param y         Ignored
	 */
	public void setColumnsPanel( DbbHtmlPanel panel,
			                     String position,
								 boolean visible,
								 int x,
								 int y ) {
		columnsPanel_ = panel;
		addPanel( panel, position, visible );
	}
	
	
	/**
	 * Set a sort panel (delegated to define the sorting criteria)
	 * @param panel     DbbHtmlPanel defing a sort panel
	 * @param position  Relative position in the view (NSEW)
	 */
	public void setSortPanel(DbbHtmlPanel panel, String position) {
		setSortPanel(panel, position, true, -1, -1);
	}

	/**
	 * Set a sort panel (delegated to define the sorting criteria)
	 * @param panel     DbbHtmlPanel defing a sort panel
	 * @param position  Relative position in the view (NSEW)
	 * @param visible   specifies if the panel is visible
	 */
	public void setSortPanel(DbbHtmlPanel panel, String position, boolean visible) {
		setSortPanel(panel, position, visible, -1, -1);
	}

	/**
	 * Set a sort panel (delegated to define the sorting criteria)
	 * @param panel     DbbHtmlPanel defing a sort panel
	 * @param position  Relative position in the view (NSEW)
	 * @param visible   specifies if the panel is visible
	 * @param x         Ignored
	 * @param y         Ignored
	 */
	public void setSortPanel( DbbHtmlPanel panel,
							  String position,
							  boolean visible,
							  int x,
							  int y ) {
		sortPanel_ = panel;
		addPanel( panel, position, visible );
	}

	/**
	 * Set a row selection panel (delegated to define a where clause)
	 *
	 * @param panel     DbbHtmlPanel defing a row selection panel
	 * @param position  Relative position in the view (NSEW)
	 */
	public void setWherePanel(DbbHtmlPanel panel, String position) {
		setWherePanel(panel, position, true, -1, -1);
	}

	/**
	 * Set a row selection panel (delegated to define a where clause)
	 *
	 * @param panel     DbbHtmlPanel defing a row selection panel
	 * @param position  Relative position in the view (NSEW)
	 * @param visible   specifies if the panel is visible
	 */
	public void setWherePanel(DbbHtmlPanel panel, String position, boolean visible) {
		setWherePanel(panel, position, visible, -1, -1);
	}

	/**
	 * Set a row selection panel (delegated to define a where clause)
	 *
	 * @param panel     DbbHtmlPanel defing a row selection panel
	 * @param position  Relative position in the view (NSEW)
	 * @param visible   specifies if the panel is visible
	 * @param x         Ignored
	 * @param y         Ignored
	 */
	public void setWherePanel( DbbHtmlPanel panel,
			                   String position,
							   boolean visible,
							   int x,
							   int y ) {
		wherePanel_ = panel;
		addPanel( panel, position, visible );
	}

	/**
	 * @param panel     A DbbHtmlPanel to add to this view
	 * @param position  Relative position in the view (NSEW)
	 * @param visible   Flag: specifies if the panel is visible
	 */
	private void addPanel( DbbHtmlPanel panel, String position, boolean visible) {
		
		if( ! visible )
			return;
		String p = position.toUpperCase().substring(0,1);
		if(      p.equals( "N" ))
			north.add( panel );
		else if( p.equals( "S" ))
			south.add( panel );
		else if( p.equals( "E" ))
			east.add( panel );
		else if( p.equals( "W" ))
			west.add( panel );
		else
			throw new IllegalArgumentException(
				"position=\"" + position + "\", should be one of NSEW." );  
    }
    
    /**
     * Collect all the chunks from the widgets, pass them on to the
     * SQL engine, and have it return the SQLquery text.
     *
     * @return Sql statement
     */
    protected DbbSqlQueryChunks getQueryChunks( HttpServletRequest req ) {

        DbbSqlQueryChunks query_chunks = new DbbSqlQueryChunks();
        DbbSqlChunk t[];
        
        if (columnsPanel_ != null) {
            t = columnsPanel_.getActiveChunks( req );
            query_chunks.columns = t;
        }

        if (wherePanel_ != null) {
            t = wherePanel_.getActiveChunks( req );
            query_chunks.appendRows( t );
        }

        if (sortPanel_ != null) {
            t = sortPanel_.getActiveChunks( req );
            query_chunks.sort = t;
        }
        
        /* Send the chunks to the sql engine to build the query. */
        return query_chunks;
    }

    /**
     * Method prepareSqlStatementQuery creates the sqlString (positioningSqlString and 
     * fetchingSqlString) needed to create the prepared statements and then call the method
     * moveToPage(req, pageNumber, pageSize);
     * positioningSqlString and fetchingSqlString are class attributes
     * Used by DBIOThread
     * @return DbbQuery
     * @throws RemoteException
     * @throws SQLException
     * @throws RemoteException 
     * @throws MalformedURLException
     * @throws NotBoundException
     */
    public DbbQuery runQuery( HttpServletRequest req, int pageNumber, int pageSize )throws SQLException, RemoteException{	
    	if (req == null) throw new IllegalArgumentException("the request is null");
    	if (pageNumber < 0) throw new IllegalArgumentException("pageNumber must be => 0");
    	if (pageSize < 0) throw new IllegalArgumentException("pageSize muste be => 0");
    	 	
    	DbbQuery result = null;
    	DbbSqlQueryChunks queryChunks =  getQueryChunks( req );

    	/**
    	 * TODO: i explicitely cast a DbbSqlEngine (abstract) to a DbbSqlEngineImpl (implementation)
    	 * to evidence we are using some methods not present in the abstract class.
    	 * I think is not a good design and we should refactor to avoid this situation.
    	 * @throws SQLException 
    	 */
    	
    	DbbSqlEngineImpl sqlEngine = (DbbSqlEngineImpl) sqlEngine_;
    	
    	paginator = new BasicPaginationOpt(sqlEngine);
    	String positioningSqlString = sqlEngine.sqlBuildPositioningQuery(queryChunks, true);
    	sqlPagingInfo.setPositioningSqlString(positioningSqlString);
    	String fetchingSqlString = sqlEngine.sqlBuildFetchingQuery(queryChunks, false);
    	sqlPagingInfo.setFetchingSqlString(fetchingSqlString);
    	String countingSqlString = sqlEngine.sqlBuildCoutingQuery(queryChunks, false);
    	sqlPagingInfo.setCountingSqlString(countingSqlString);
    	((BasicPaginationOpt)paginator).setSqlCount(countingSqlString);
    //	int index = positioningSqlString.indexOf("ORDER BY");
    //	String str = positioningSqlString.substring(0, index);
    	((BasicPaginationOpt)paginator).setSqlQuery(positioningSqlString);
    	
    	result = moveToPage(req, pageNumber, pageSize);
    	setColumnsInformation(result);
    	
        return result;
    }
    
    
    /*
     * The method retrieve the column informations needed to set the proper values 
     * in the fetching sql prepared statement every time the user changes page
     * This method has to be called every time a new DBBquery is runned.
     */
    private void setColumnsInformation(DbbQuery result) throws RemoteException, SQLException  {
    	assert (result != null): MsgManager.errPrecondition("result is null");
    	
    	if(result.hasMoreRows())
			result.readRows();
		String[] columnsName = result.getResult().getColumnNames();
		sqlPagingInfo.setColumnsName(columnsName);
		int columnsCount = result.getResult().getColumns();
		sqlPagingInfo.setColumnsCount(columnsCount);
		Class[] columnsType = result.getResult().getColumnTypes();
		sqlPagingInfo.setColumnsType(columnsType);
	}

	/**
     * The method moveToPage first execute a count query and set the itemCount 
     * attribute in the request, then it call the method to retrieve one page of the resultset
     * method executeQuery()
     * @param req
     * @return
     * @throws SQLException
     */
    public DbbQuery moveToPage(HttpServletRequest req, int pageNumber, int pageSize )throws SQLException{
    	if (req == null) throw new IllegalArgumentException("the request is null");
    	if (pageNumber < 0) throw new IllegalArgumentException("pageNumber must be => 0");
    	if (pageSize < 0) throw new IllegalArgumentException("pageSize muste be => 0");
    	
    	DbbQuery result = null;
  
    		/**
    	 * TODO: i explicitely cast a DbbSqlEngine (abstract) to a DbbSqlEngineImpl (implementation)
    	 * to evidence we are using some methods not present in the abstract class.
    	 * I think is not a good design and we should refactor to avoid this situation.
    	 */
    	
    	DbbSqlEngineImpl sqlEngine = (DbbSqlEngineImpl) sqlEngine_;
    	
    	
    	sqlPagingInfo.setPageNumber(pageNumber);
    	sqlPagingInfo.setPageSize(pageSize);
    	
    //	result = sqlEngine.execPaginatedQuery(sqlPagingInfo);
    	result = paginator.moveTo(pageNumber, pageSize);
    	isReversed = paginator.isReversed();
    	try {
			setColumnsInformation(result);
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
    	return result;
    }
   
    public DbbPagingInformation getPagingInformation(HttpServletRequest req, int pageNumber, int pageSize )throws SQLException{
    	if (req == null) throw new IllegalArgumentException("the request is null");
    	if (pageNumber < 0) throw new IllegalArgumentException("pageNumber must be => 0");
    	if (pageSize < 0) throw new IllegalArgumentException("pageSize muste be => 0");
    	
    	DbbPagingInformation result = new DbbPagingInformation();
    
    	int count = ((DbbSqlEngineImpl)sqlEngine_).execSqlCount(sqlPagingInfo.getCountingSqlString());
  
    	result.setItemCount(count);
		int pageCount = count / pageSize;
		if ( (count % pageSize ) > 0)
			pageCount++;	
		result.setPageCount(pageCount);
		//redundant information
		result.setPageNumber(pageNumber);
		result.setPageSize(pageSize);
	
    	return result;
    }
    
    public void checkQuery(HttpServletRequest req){
    	String sql = "";
    	DbbQuery query = null;
    	DbbSqlQueryChunks queryChunks =  getQueryChunks( req );
    	
		try {
			sql = ((DbbSqlEngineImpl)sqlEngine_).sqlBuildPositioningQuery(queryChunks, false);
		
			query = ((DbbSqlEngineImpl)sqlEngine_).execSql(sql);
			query.readRows();
			query.setNumOfRowsToGet(10000);
			DbbSqlQueryResult dbbr = query.getResult();
			
			List colNames = Arrays.asList( dbbr.getColumnNames() );
			printRow(0,colNames);
		    Vector<Object>   data = dbbr.getDataVector();
		    Vector<List>   rows = new Vector<List>( data.size() );

		    for( int i = 0; i < data.size(); i++ ) {
		    	Object t[] = (Object[]) data.elementAt(i);
		        List row = Arrays.asList( t );
		        rows.add( row );
		        printRow(i, row);
		    }
		    
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
			
    }
    public void printRow(int a, List row){
    	System.out.print("\n " + a + ") \t");
    	for (int i = 0; i < row.size(); i++){
    		System.out.print(row.get(i) + "\t");
    	}
    }

	public int getItemsCount() {
		return itemsCount;
	}

	public void setItemsCount(int itemsCount) {
		this.itemsCount = itemsCount;
	}

	public boolean isReversed() {
		return isReversed;
	}

	public void setReversed(boolean isReversed) {
		this.isReversed = isReversed;
	}
}
