package org.eso.ohs.core.dbb.rmi;

// $Id: DbbRemoteQueryImpl.java,v 1.1 2004/02/17 11:16:24 tcanavan Exp $

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.sql.SQLException;

import org.eso.ohs.core.dbb.server.DbbQuery;
import org.eso.ohs.core.dbb.sql.DbbSqlQueryResult;

/**   
 * Implementation of the remote sql query.
 *
 * @author Bob Kemp
 */
public class DbbRemoteQueryImpl 
    extends UnicastRemoteObject
    implements DbbRemoteQuery {

    /** Sql query reference */
    private DbbQuery dbbQ_;


    //----------------------------------------
    // Constructor
    //----------------------------------------

    /**
     * Constructor for DbbRemoteQueryImpl
     * @param DbbQuery DbbQuery object
     * @exception RemoteException RMI Exception
     */
    public DbbRemoteQueryImpl(DbbQuery dbbQ)
	throws RemoteException {
	dbbQ_ = dbbQ;
    }


    //----------------------------------------
    // Public methods
    //----------------------------------------

    /**
     * Set the number of rows to be displayed per time. 
     * @param number of rows ro fetch
     * @exception RemoteException RMI Exception
     */
    public void setNumOfRowsToGet(int n)
	throws RemoteException {
	dbbQ_.setNumOfRowsToGet(n);
    }

    /** 
     * Get the ResultSet
     * @return The result set
     * @exception RemoteException RMI Exception
     */
    public DbbSqlQueryResult getResult()
	throws RemoteException {
	return dbbQ_.getResult();
    }

  
    /** 
     * Check if there is more rows in the result set 
     * for this query
     * @return true more rows in result set otherwase false
     * @exception RemoteException RMI Excpetion
     */
    public boolean hasMoreRows()
	throws RemoteException {
	return dbbQ_.hasMoreRows();
    }
    

    /** 
     * Read rows from the database   
     * @exception SQLException JDBC Exception
     * @exception RemoteException RMI Exception
     */
    public void readRows()
	throws SQLException, RemoteException {
	dbbQ_.readRows();
    }

    public DbbSqlQueryResult getEmptyResult() throws  RemoteException {

    opNotSupported();
    return null;
    
    }

    private void opNotSupported() throws RemoteException {
        throw new RemoteException("Operation not supported for this object");
        
    }
    
    public DbbSqlQueryResult appendRows(DbbSqlQueryResult queryResult) throws  RemoteException {

   
    opNotSupported();
    return queryResult;
    
    }
    /** 
     * Close the query
     * @exception SQLException JDBC Exception
     * @exception RemoteException RMI Exception
     */
    public void close()
	throws SQLException, RemoteException {
	dbbQ_.close();
    }
}



