package org.eso.ohs.core.dbb.sql;


import org.apache.log4j.Logger;
/** 
 * This class is a SqlChunkGroup which generates OR expression
 * based on the value of two SqlChunk members, which represent the values of a cycling range.
 * 
 * It is assumed the values of the chunks may be parsed into floating point
 * numbers.
 *
 *    ( fld <= XmaxchunkVal OR fld >= XminchunkVal)
 *
 * @author Nick Kornweibel for ESO
 */

public class CyclingOrChunk extends DbbSqlChunkGroup {

   /** Standard log channel */
   private static Logger stdlog_ =
       Logger.getLogger( CyclingOrChunk.class );

    public CyclingOrChunk(DbbSqlChunkGroupMember first,
            DbbSqlChunkGroupMember second) {
        addMember(first);
        addMember(second);
    }

    /**
     * See header for details on this expression.
     *
     * @return String containing the sql clause.
     */

    public String getExpressionClause() {
        String rc;
        DbbSqlChunk first = members_.elementAt(0);
        DbbSqlChunk second = members_.elementAt(1);
      
        String firstStr = null;
        String secondStr = null;

        // try and get a value for firstStr
        if (first.isActive()) {
	    
            firstStr = first.getValue();	    
            if ((firstStr != null) &&( firstStr.length() == 0)) {             
                firstStr = null;
            }
        }

        // try and get a value for secondStr
        if (second.isActive()) {
            secondStr = second.getValue();
            if (secondStr != null && secondStr.length() == 0) {
                secondStr = null;
            }
        }

        String firstStrExp = null;
        String secondStrExp = null;

        // create the expressions if we have good values.
        if (firstStr != null) {
            DbbSqlOperator op = first.getOperator();
            firstStrExp = "("+first.getColumnSpec()
                +" "+op.get()+" "+firstStr+")";
        }

        if (secondStr != null) {
            DbbSqlOperator op = second.getOperator();
            secondStrExp = "("+second.getColumnSpec()
                +" "+op.get()+" "+secondStr+")";
        }

        // only build the compound expression if we have two
        // sub expressions - otherwise simply return the
        // sub expression we have, or "" if there is nothing to return.

        if (firstStrExp != null && secondStrExp != null) {
                rc = "("+firstStrExp+" OR "+secondStrExp+")";
        } else if (firstStrExp != null) {
            rc = firstStrExp;
        } else if (secondStrExp != null) {
            rc = secondStrExp;
        } else {
            rc = "";
        }
        /*
        stdlog_.debug("CyclingOrChunk: firstStr: "+firstStr
            +" :firstStrExp: "+firstStrExp+" :minVal: "+minVal);
        stdlog_.debug("CyclingOrChunk: secondStr: "
            +secondStr+" :secondStrExp: "+secondStrExp+" :maxVal: "+maxVal);
        stdlog_.debug("CyclingOrChunk: rc: "+rc);
        */
        return rc;
    }
}
