package org.eso.ohs.core.gui.baseaction;

import javax.swing.JFrame;

import org.eso.ohs.core.gui.utilities.Utilities;
import org.eso.ohs.core.gui.widgets.OutOfMemoryReport;
import org.eso.ohs.core.gui.widgets.StoppableProgressDialog;


/**
 * This class extend IncrementalActionAutomaton class in order to support description change 
 * during the calculation
 * 
 * @author pnunes 08.30.2006
 *
 */
public class DescIncrementalActionAutomaton extends IncrementalActionAutomaton {
	
	private DescriptiveIncrementalAction action_;
	
	
	/**
     * Constructor.
     * 
     * @param action	Object that will perform the steps of a larger action
     * @param frm	Base frame to use for dialog boxes such as dialog_
     * @param msg	Message to display in the progress bar dialog (dialog_)
     */
    public DescIncrementalActionAutomaton(
    		DescriptiveIncrementalAction action,
        JFrame frm,
        String msg) {
        init(action, frm, msg);
    }

    protected void init(DescriptiveIncrementalAction action, JFrame frm, String msg) {
        baseFrame_ = frm;
        action_ = action;
        maximum_ = action.getNumStepsRemaining(0);

        if (maximum_ > 1) {
            dialog_ = new StoppableProgressDialog(baseFrame_, msg, 0, maximum_);

            int num_pauses = (int) (Math.sqrt(maximum_) + 0.5) * 2;

            if (num_pauses > 20) {
                num_pauses = 20;
            }
            num_steps_before_pausing_ =
                (maximum_ + num_pauses - 1) / num_pauses;
        } else {
            dialog_ = null;
            num_steps_before_pausing_ = 1000;
        }

        runLater();
    }
	
	
    protected void setProgressBarValue(int value) {
        if (dialog_ != null) {
            dialog_.setValue(value);

           	String status = action_.getStatus(counter_);
            dialog_.setMessage(status);
        }
    }
	
    /**
     * Method to manage each incremental step of a larger action.
     * It calls action_.performStep() to perform each step.
     */
    public void run() {
        counter_++;

        if (counter_ == -1) {
            action_.startOfAction();
            Utilities.waitCursor(baseFrame_);
            closeViews();
            showProgressBar();
            runLater();
        } else {
            int stepsRemaining = action_.getNumStepsRemaining(counter_);

            setProgressBarValue(maximum_ - stepsRemaining);

            if (!shouldStop() && stepsRemaining > 0) {
                try {
                    action_.performStep(counter_);
                            			
                    
                } catch (OutOfMemoryError e) {
                    e.printStackTrace();
                    OutOfMemoryReport.showMessage();
                    cleanUpAtEnd();
                    throw e;
                }
                runLater();
            } else {
                // All done: just cleanup and return
                new SlightPause(new Runnable() {
                    public void run() {
                        cleanUpAtEnd();
                    }
                }, 250);
                return;
            }
        }
    }

    public void cleanUpAtEnd() {
        //if (memento_ != null) {
        //MainWindow.getView().restoreViews(memento_);
        //	}
        //MainWindow.getView().setDisplaySuppressed(false);
        if (dialog_ != null) {
            dialog_.dispose();
        }
        Utilities.normalCursor(baseFrame_);
        action_.endOfAction();
    }

}
