package org.eso.ohs.core.gui.utilities;

import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;

/**
 * Class MenuUtils
 * Utility class generic menu operations
 * TODO Javadoc
 *
 * @version $Revision: 1.1 $
 * @author ddorigo, Jan 18, 2006
 */

// $Id: MenuUtils.java,v 1.1 2006/01/18 14:23:27 ddorigo Exp $
public class MenuUtils {

    /**
     * deletes all the submenus in sequence
     * @param m the top menu
     */
    public static void removeMenuItems(JMenu m) {
        for (int i = m.getItemCount() - 1; i >= 0; i--) {
            if (m.getItem(i) == null)
                continue;
            m.remove(m.getItem(i));
        }
    }

    /**
     * Adds a action to the named menu.
     * 
     * @param frame frame in which the menu is added
     * @param menuName      name of the menu
     * @param subMenu       subMenu to be added
     */
    public static JMenuItem addSubMenu(JFrame frame,String menuName, JMenuItem subMenu) {
        JMenu m = getMenu(frame,menuName);
        JMenuItem mnItem = m.add(subMenu);
        return mnItem;
    }
   
    /**
     * Retrieves or creates the menu with the given name in the menubar.
     * 
 * @param frame frame in which the menu is added
 * @param menuName name of the menu
 * @return JMenu the created menu
 */
public static JMenu getMenu(JFrame frame,String menuName) {
    if(frame == null) 
        throw new IllegalArgumentException("JFrame is not initialised");
    JMenuBar mb = frame.getJMenuBar();
    if (mb == null) {
        mb = new JMenuBar();
        frame.setJMenuBar(mb);
    }

    int numMenus = 0;
    JMenu m = null;
    boolean found = false;

    numMenus = mb.getMenuCount();

    for (int i = 0; i < numMenus; i++) {
        m = mb.getMenu(i);
        if (m.getText().equals(menuName)) {
            found = true;
            break;
        }
    }
    if (!found) {
        m = new JMenu(menuName);
        mb.add(m);
    }
    return m;
}

/**
 * Enables or disable a menu 
 * @param frame frame containg the menu
 * @param menuName the main menu name
 * @param enabled enable/disable the menu with true/false
 */
public static void setEnabled(JFrame frame, String menuName, boolean enabled) {
    MenuUtils.getMenu(frame,menuName).setEnabled(enabled);
}


}
