package org.eso.ohs.core.gui.widgets;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.FontMetrics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Vector;

import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.ListCellRenderer;
import javax.swing.ListSelectionModel;
import javax.swing.ScrollPaneConstants;
import javax.swing.border.Border;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.client.ArrayListTransferHandler;
import org.eso.ohs.core.dbb.client.DbbView;
import org.eso.ohs.core.gui.models.ValidationAddDualList;

public class DualListBox extends JDialog implements ActionListener {

	ArrayListTransferHandler arrayListHandler;

	private final String AUTOSORT = "AutoSort";
	private final String DRAG = "Drag & Drop";
	public final static int DRAG_NONE = 0;
	public final static int DRAG_SOURCE = 1;
	public final static int DRAG_DESTINATION = 2;
	public final static int DRAG_BOTH = 3;
	public final static boolean RUNTIME_SWITCHABLE = true;
	public final static boolean RUNTIME_NOT_SWITCHABLE = false;
	private final Color autoSortedColor = new Color(250, 250, 250);
	private final Color dragColor = Color.white;
	/**
	 * OptionPane status
	 */

	private int returnValue_ = JOptionPane.CLOSED_OPTION;

	/**
	 * label above source column
	 */

	private JLabel labelSourceList_ = new JLabel();

	/**
	 * label above source column
	 */

	private ValidationAddDualList addValidation_ = null;

	/**
	 * List to hold source data
	 */
	private JList source_;

	/**
	 * label above destination column
	 */
	private JLabel label2_ = new JLabel();

	/**
	 * List for destination data.
	 */

	private JList destination_;

	/**
	 * Source data model
	 */

	private DefaultListModel srcModel_;

	/**
	 * Destination data model
	 */

	private DefaultListModel destModel_;

	/**
	 * Add one element from source list
	 */

	private JButton addOne_ = null;

	/**
	 * Remove one from destination list
	 */

	private JButton removeOne_ = null;

	/**
	 * Button to commit changes
	 */

	private JButton ok_ = null;

	/**
	 * Button to cancel changes
	 */

	private JButton cancel_ = null;

	private boolean isDestSorted = true;

	private boolean isSourceSorted = true;

	private JFrame parentFrame_;
	
	private int type = 0;
	private boolean isRuntimeSwitchable = false;
	
	private Dimension windowPreferredSize = new Dimension(400,500);
	
	private ArrayList startingSelection = new ArrayList();
	
	/**
	 * Field stdlog_
	 */
	private static Logger stdlog_ = Logger.getLogger(DbbView.class);

	/**
	 * Create a DualListBox Dialog box
	 * 
	 * @param f
	 *            Frame of Dialog box
	 * @param srcModel
	 *            model for source data
	 * @param destModel
	 *            model for desination list
	 *@deprecated Use public DualListBox(JFrame f, DefaultListModel srcModel, DefaultListModel destModel, int type, boolean isSwitchable) 
	 */

	public DualListBox(JFrame f, DefaultListModel srcModel,
			DefaultListModel destModel) {
		super(f, "Export", true);
		srcModel_ = srcModel;
		destModel_ = destModel;
		parentFrame_ = f;
		init();
		ok_.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent arg0) {
				okDualAction();
			}
		});
		cancel_.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent arg0) {
				cancelAction();

			}

		});
		addListeners();
		pack();
	}
	/**
	 * 
	* @param f
	 *            Frame of Dialog box
	 * @param srcModel
	 *            model for source data
	 * @param destModel
	 *            model for desination list
	 * @param type
	 * 				Specify drag&drop capabilities for the JLists of the DualListBox. Use the constant static values: 
	 * 					DualListBox.DRAG_NONE
	 * 					DualListBox.DRAG_SOURCE
	 * 					DualListBox.DRAG_DESTINATION
	 * 					DualListBox.DRAG_BOTH
	 * 
	 * @param isSwitchable
	 * 				true if you want to switch AutoSort/Drag&Drop at run time.
	 */
	public DualListBox(JFrame f, DefaultListModel srcModel,
			DefaultListModel destModel, int type, boolean isSwitchable) {
		super(f, "Export", true);
		srcModel_ = srcModel;
		destModel_ = destModel;
		parentFrame_ = f;
		this.type = type;
		this.isRuntimeSwitchable = isSwitchable;
		init();
		
		ok_.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent arg0) {
				okDualAction();

			}

		});
		cancel_.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent arg0) {
				cancelAction();

			}

		});
		addListeners();
		pack();
	}

	/**
	 * Return Value used by OptionPane
	 * 
	 * @return returnValue
	 */

	public int getValue() {
		return returnValue_;
	}

	/**
	 * Set the source model for the source list
	 * 
	 * @param srcData
	 *            data to be copied into srcModel
	 */

	public void setSourceModel(Vector srcData) {
		for (int i = 0; i < srcData.size(); i++) {
			addSrcElement(srcData.elementAt(i));
		}
		if (isSourceSorted)
			sortListModel(srcModel_);
	}

	/**
	 * Set the source model for the source list
	 * 
	 * @param srcData
	 *            data to be copied into srcModel
	 */

	public void removeFromSourceList(Object[] srcRemoveList) {
		if (srcRemoveList != null) {
			for (int i = 0; i < srcRemoveList.length; i++) {
				srcModel_.removeElement(srcRemoveList[i]);
			}
		}
	}

	/**
	 * Set the source model for the source list
	 * 
	 * @param srcData
	 *            data to be copied into srcModel
	 */

	public void setDataRemovingDuplicationFromSource(Object[] srcList,
			Object[] destList) {
		setSourceModel(srcList);
		removeFromSourceList(destList);
		setDestModel(destList);
	}

	/**
	 * Set the source model for the source list
	 * 
	 * @param srcData
	 *            data to be copied into srcModel
	 */

	public void removeFromDestList(Object[] destRemoveList) {
		if (destRemoveList != null) {
			for (int i = 0; i < destRemoveList.length; i++) {
				destModel_.removeElement(destRemoveList[i]);
			}
		}
	}

	/**
	 * @param element
	 *            void
	 */
	private void addSrcElement(Object element) {
		srcModel_.addElement(element);
	}

	/**
	 * @param element
	 *            void
	 */
	private void addDestElement(Object element) {
		destModel_.addElement(element);
	}

	/**
	 * Set the source model for the source list
	 * 
	 * @param srcData
	 *            data to be copied into srcModel
	 */

	public void setSourceModel(Object[] srcData) {
		for (int i = 0; i < srcData.length; i++) {
			addSrcElement(srcData[i]);
		}
		if (isSourceSorted)
			sortListModel(srcModel_);
	}

	/**
	 * Set the destination model for the destination list
	 * 
	 * @param srcData
	 *            data to be copied into destModel
	 */

	public void setDestModel(Vector destData) {
		for (int i = 0; i < destData.size(); i++) {
			addDestElement(destData.elementAt(i));
		}
		if (isDestSorted)
			sortListModel(destModel_);
	}

	/**
	 * Set the destination model for the destination list
	 * 
	 * @param srcData
	 *            data to be copied into destModel
	 */

	public void setDestModel(Object[] destData) {
		for (int i = 0; i < destData.length; i++) {
			addDestElement(destData[i]);
		}
		if (isDestSorted)
			sortListModel(destModel_);
	}

	/**
	 * Return the destination model as an array of objects
	 * 
	 * @return destination model as an array of objects
	 */

	public Object[] getDestModel() {
		return destModel_.toArray();
	}

	/**
	 * Initialise the data models for the source and destination lists
	 * 
	 * @param srcData
	 *            Vector to initialise source list
	 * @param destData
	 *            Vector to initialise destination list
	 */

	public void setLists(Vector srcData, Vector destData) {
		srcModel_ = new DefaultListModel();
		for (int i = 0; i < srcData.size(); i++) {
			srcModel_.addElement(srcData.elementAt(i));
		}
		if (isSourceSorted)
			sortListModel(srcModel_);
		source_ = new JList(srcModel_);

		destModel_ = new DefaultListModel();
		for (int i = 0; i < destData.size(); i++) {
			destModel_.addElement(destData.elementAt(i));
		}
		if (isSourceSorted)
			sortListModel(destModel_);
		destination_ = new JList(destModel_);
	}

	/**
	 * Set the cell renders for the source and destinationl list
	 * 
	 * @param renderer
	 *            renderer for items in destination and source lists
	 */
	public void setCellRenderers(ListCellRenderer renderer) {
		source_.setCellRenderer(renderer);
		destination_.setCellRenderer(renderer);
	}

	/**
	 * Set the title for the source list
	 * 
	 * @param title
	 *            title for source list
	 */

	public void setSourceTitle(String title) {
		labelSourceList_.setText(title);
	}

	/**
	 * Set the tite for the destination list
	 * 
	 * @param title
	 *            for destination list
	 */

	public void setDestinationTitle(String title) {
		label2_.setText(title);
	}

	/**
	 * Inirialise the member variables and add gui components to the frame.
	 */
	private void init() {
		source_ = new JList(srcModel_);
		destination_ = new JList(destModel_);

		arrayListHandler = new ArrayListTransferHandler();
		source_.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		destination_.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		
		switch (type){
			case DRAG_NONE:
				source_.setBackground(autoSortedColor);
				destination_.setBackground(autoSortedColor);
				isRuntimeSwitchable = false;
			
			break;
			case DRAG_SOURCE:
			
				source_.setTransferHandler(arrayListHandler);
				source_.setDragEnabled(true);
				source_.setBackground(dragColor);
				destination_.setBackground(autoSortedColor);
			break;
			case DRAG_DESTINATION:
				
				source_.setBackground(autoSortedColor);
				destination_.setTransferHandler(arrayListHandler);
				destination_.setDragEnabled(true);
				destination_.setBackground(dragColor);
			
			break;
			case DRAG_BOTH:
				
				source_.setTransferHandler(arrayListHandler);
				source_.setDragEnabled(true);
				source_.setBackground(dragColor);
				destination_.setTransferHandler(arrayListHandler);
				destination_.setDragEnabled(true);
				destination_.setBackground(dragColor);
				
			break;
			default:
				
				source_.setBackground(autoSortedColor);
				destination_.setBackground(autoSortedColor);
				type = DRAG_NONE;
				isRuntimeSwitchable = false;
			
			break;
		}
		

		if (isSourceSorted)
			sortListModel(srcModel_);
		if (isDestSorted)
			sortListModel(destModel_);
		// source_.setModel(srcModel_);
		// destination_.setModel(destModel_);
		JPanel panel = new JPanel();
		JPanel panelContainer = new JPanel();
		panelContainer
				.setLayout(new BoxLayout(panelContainer, BoxLayout.Y_AXIS));
		panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
		JPanel sourceListPanel = new JPanel();
		sourceListPanel.setLayout(new BoxLayout(sourceListPanel,
				BoxLayout.Y_AXIS));
		JPanel buttonTransferPanel = new JPanel();
		buttonTransferPanel.setLayout(new BoxLayout(buttonTransferPanel,
				BoxLayout.Y_AXIS));
		JPanel destListPanel = new JPanel();
		destListPanel.setLayout(new BoxLayout(destListPanel, BoxLayout.Y_AXIS));

		JPanel buttonListPanel = new JPanel();
		buttonListPanel.setLayout(new BoxLayout(buttonListPanel,
				BoxLayout.X_AXIS));
		
		Dimension preferredListSize = new Dimension(200, 200);

		labelSourceList_.setText("Source list");

		final JScrollPane srcScrollPane = new JScrollPane(source_,
				ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
				ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		srcScrollPane.setPreferredSize(preferredListSize);
		sourceListPanel.add(labelSourceList_);
		sourceListPanel.add(srcScrollPane);
		
		addOne_ = new JButton(">");
		removeOne_ = new JButton("<");
		FontMetrics fm = addOne_.getFontMetrics(addOne_.getFont());
		
		int y = fm.getHeight() + 25;
		int x = fm.charWidth('>') + 40;
		
		Dimension preferredSize = new Dimension(x, y);

		
		buttonTransferPanel.add(addOne_);
		addOne_.setMaximumSize(preferredSize);
		addOne_.setMinimumSize(preferredSize);
		addOne_.setPreferredSize(preferredSize);

		
		buttonTransferPanel.add(removeOne_);
		removeOne_.setMaximumSize(preferredSize);
		removeOne_.setMinimumSize(preferredSize);
		removeOne_.setPreferredSize(preferredSize);

		Border border = addOne_.getBorder();
		Border margin = new EmptyBorder(2, 2, 2, 2);
		addOne_.setBorder(new CompoundBorder(margin, border));
		removeOne_.setBorder(new CompoundBorder(margin, border));

		addOne_.setAlignmentX(JButton.CENTER_ALIGNMENT);
		addOne_.setAlignmentY(JButton.CENTER_ALIGNMENT);
		removeOne_.setAlignmentX(JButton.CENTER_ALIGNMENT);
		removeOne_.setAlignmentY(JButton.CENTER_ALIGNMENT);
	
		
		label2_.setText("Destination list");
		destListPanel.add(label2_);

		

		final JScrollPane destScrollPane = new JScrollPane(destination_,
				ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
				ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		destScrollPane.setPreferredSize(preferredListSize);
		destListPanel.add(destScrollPane);

		ok_ = new JButton("OK");
		buttonListPanel.add(ok_);
		cancel_ = new JButton("Cancel");
		buttonListPanel.add(cancel_);
		
//		 Add the tadio button group "auto sort" "drag & drop"
		
		if (isRuntimeSwitchable){
			ButtonGroup group = new ButtonGroup();
			JRadioButton addButton;
			addButton = new JRadioButton(AUTOSORT, false);
			buttonListPanel.add(addButton);
			group.add(addButton);
			addButton.addActionListener(this);
			addButton = new JRadioButton(DRAG, true);
			buttonListPanel.add(addButton);
			group.add(addButton);
			addButton.addActionListener(this);
		}

		panel.add(sourceListPanel);
		panel.add(buttonTransferPanel);
		panel.add(destListPanel);
		panelContainer.add(panel);
		panelContainer.add(buttonListPanel);
		panelContainer.setPreferredSize(windowPreferredSize);
		setContentPane(panelContainer);
	}

	/**
	 * Add listeners for Dialog buttons
	 */

	protected void addListeners() {
		addOne_.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				addAction(event);
			}
		});
		removeOne_.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				removeAction(event);
			}
		});

	}

	/**
	 * Add listeners for Dialog buttons
	 */

	public void addOkActionListener(ActionListener okAction) {
		ok_.addActionListener(okAction);
	}

	/**
	 * Add listeners for Dialog buttons
	 */

	public void addCancelActionListener(ActionListener cancelAction) {
		cancel_.addActionListener(cancelAction);
	}

	/**
	 * Add one element from source list
	 * 
	 * @param e
	 *            Add one action event
	 */

	void addAction(ActionEvent e) {
		Object[] srcSelList = getSourceSelected();
		if (getAddValidation() != null) {
			srcSelList = getAddValidation().validateDataAction(srcSelList);
		}
		if (srcSelList.length > 0) {
			for (int i = 0; i < srcSelList.length; i++) {
				destModel_.addElement(srcSelList[i]);
				srcModel_.removeElement(srcSelList[i]);
			}
		}
		// sortLists();
		if (isSourceSorted)
			sortListModel(srcModel_);
		if (isDestSorted)
			sortListModel(destModel_);
		source_.clearSelection();
		destination_.clearSelection();
	}

	/**
	 * @return Object[]
	 */
	public Object[] getSourceSelected() {
		return source_.getSelectedValues();
	}

	/**
	 * sorts the list according to the natural sorting of the objects in the
	 * list void
	 */
	private void sortLists() {
		sortListModel(srcModel_);
		sortListModel(destModel_);
	}

	/**
	 * @param srcModel_
	 *            void
	 */
	@SuppressWarnings("unchecked")
	private void sortListModel(DefaultListModel srcModel) {
		Object[] elemList = srcModel.toArray();
		List list = new ArrayList();
		if (elemList != null) {
			for (int i = 0; i < elemList.length; i++) {
				list.add(elemList[i]);
			}
		}
		Collections.sort(list);
		srcModel.removeAllElements();
		for (int i = 0; i < list.size(); i++) {
			srcModel.addElement(list.get(i));
		}
	}

	/**
	 * Remove one action event
	 * 
	 * @param e
	 *            Remove one action event
	 */

	void removeAction(ActionEvent e) {
		Object[] destSelList = destination_.getSelectedValues();
		if (destSelList.length > 0) {
			for (int i = 0; i < destSelList.length; i++) {
				srcModel_.addElement(destSelList[i]);
				destModel_.removeElement(destSelList[i]);
			}
		}
		// sortLists();
		if (isSourceSorted)
			sortListModel(srcModel_);
		if (isDestSorted)
			sortListModel(destModel_);

		source_.clearSelection();
		destination_.clearSelection();
	}

	/**
	 * OK action event
	 * 
	 * @param e
	 *            OK action event
	 */

	public void okDualAction() {
		returnValue_ = JOptionPane.OK_OPTION;
		setVisible(false);
	}

	/**
	 * Cancel action event
	 * 
	 * @param e
	 *            Cancel action event
	 */

	void cancelAction() {
		returnValue_ = JOptionPane.CANCEL_OPTION;
		setVisible(false);
	}

	/**
	 * @param addValidation_
	 *            The addValidation_ to set.
	 */
	public void setAddValidation(ValidationAddDualList addValidation_) {
		this.addValidation_ = addValidation_;
	}

	/**
	 * @return Returns the addValidation_.
	 */
	private ValidationAddDualList getAddValidation() {
		return addValidation_;
	}

	/**
	 * 
	 * @return true if the DestinationList is alphabetically sorted
	 */
	public boolean isDestSorted() {
		return isDestSorted;
	}

	/**
	 * enable or disable alphabetical sorting on the Destination List
	 * 
	 * @param isDestSorted
	 */
	public void setDestSorted(boolean isDestSorted) {
		this.isDestSorted = isDestSorted;

		if (isDestSorted){ 
			destination_.setDragEnabled(false);
			stdlog_.debug("The destination list is now automatically sorted. You cannot change the order by drag&drop items");
			sortListModel(destModel_);
		}else{
			destination_.setDragEnabled(true);
			stdlog_.debug("Autosorting is disable. You can now change the order by drag&drop items");
		}
	}

	/**
	 * 
	 * @return true if the SourceList is alphabetically sorted
	 */
	public boolean isSourceSorted() {
		return isSourceSorted;
	}

	/**
	 * enable or disable alphabetical sorting on the Source List
	 * 
	 * @param isSourceSorted
	 */
	public void setSourceSorted(boolean isSourceSorted) {
		this.isSourceSorted = isSourceSorted;

		/*
		 * if (isDestSorted) destination_.setDragEnabled(true); else
		 * destination_.setDragEnabled(false);
		 */

		sortListModel(srcModel_);
	}

	public void actionPerformed(ActionEvent e) {

		String command = e.getActionCommand();
		if (command.equals(DRAG)){
			switch (type){
				case DRAG_NONE:
				
					break;
				case DRAG_SOURCE:
					source_.setBackground(Color.white);
					setSourceSorted(false);
					break;
				case DRAG_DESTINATION:
					destination_.setBackground(Color.white);
					setDestSorted(false);
					break;
				case DRAG_BOTH:
					source_.setBackground(Color.white);
					destination_.setBackground(Color.white);
					setSourceSorted(false);
					setDestSorted(false);
					break;
				default:
					
					break;
			}
			
		} else if (command.equals(AUTOSORT))
			switch (type){
			case DRAG_NONE:
			
				break;
			case DRAG_SOURCE:
				source_.setBackground(Color.lightGray);
				setSourceSorted(true);
				break;
			case DRAG_DESTINATION:
				destination_.setBackground(Color.lightGray);
				setDestSorted(true);
				break;
			case DRAG_BOTH:
				source_.setBackground(Color.lightGray);
				destination_.setBackground(Color.lightGray);
				setSourceSorted(true);
				setDestSorted(true);
				break;
			default:
				
				break;
		}
	}
	public Dimension getWindowPreferredSize() {
		return windowPreferredSize;
	}
	public void setWindowPreferredSize(Dimension windowPreferredSize) {
		this.windowPreferredSize = windowPreferredSize;
	}
}
