package org.eso.ohs.core.gui.widgets;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.HeadlessException;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.geom.AffineTransform;

import javax.swing.AbstractAction;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.JToolBar;

import org.eso.ohs.core.gui.widgets.imageviewer.FlipMode;
import org.eso.ohs.core.gui.widgets.imageviewer.ImageCanvas2D;
import org.eso.ohs.core.gui.widgets.imageviewer.LensPanel;
import org.eso.ohs.core.gui.widgets.imageviewer.PanZoom;

/**
 * @author tcanavan
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class JPEGViewer extends JFrame {

    // private static class 

    private JButton flipVBtn = new JButton();

    private JButton flipHBtn = new JButton();

    private JButton rotate90Btn = new JButton();

    private JButton rotate180Btn = new JButton();

    private JButton rotate270Btn = new JButton();

    private JButton rotateMinus90Btn = new JButton();

    private JButton rotateMinus180Btn = new JButton();

    private JButton rotateMinus270Btn = new JButton();

    private JButton rotateXDegrees = new JButton();
    
    private JButton lensBtn = new JButton();

    private JButton resetBtn = new JButton("Reset");

    private JButton cancelBtn = new JButton("Cancel");
    
    private JButton clearLensBtn = new JButton("Clear Lense");

    private JButton zoomBtn = new JButton();

    private JTextField degField = new JTextField(6);

    //private JAIImageCanvas imgCanvas = new JAIImageCanvas();

    private ImageCanvas2D[] imgCanvasList;

    private JLabel statusStr = new JLabel();

    private String[] titles;

    private JComponent comp;

    private Mediator mediator = new Mediator();

    private JList imgList = new JList();

    private static JPEGViewer jpegViewer;

    private int currentRow;

    private JTabbedPane tabPane = new JTabbedPane();

    private ImageCanvas2D currImgCanvas2D;
    
    private JFrame lensFrame = new JFrame();
    
    private PanZoom zoom;
    
    private LensPanel lensPanel;

    public static JPEGViewer getInstance() {
        if (jpegViewer == null) {
            jpegViewer = new JPEGViewer();
        }
        return jpegViewer;
    }

    /**
     * Constructor for JPEGViewer.
     * @throws HeadlessException
     */
    private JPEGViewer() throws HeadlessException {
        super();
        getContentPane().setLayout(new BorderLayout());
        JToolBar toolBar = new JToolBar();
        JPanel south = new JPanel(true);
        south.setLayout(new BoxLayout(south, BoxLayout.X_AXIS));
        south.add(cancelBtn);
        south.add(flipHBtn);
        south.add(flipVBtn);
        south.add(rotate90Btn);
        south.add(rotate180Btn);
        south.add(rotate270Btn);
        south.add(rotateMinus90Btn);
        south.add(rotateMinus180Btn);
        south.add(rotateMinus270Btn);
        south.add(rotateXDegrees);
        south.add(degField);
        south.add(resetBtn);
        south.add(lensBtn);
        //south.add(clearLensBtn);
        //south.add(zoomBtn);
        south.add(Box.createGlue());
        south.add(statusStr);

        cancelBtn.addActionListener(new CancelActionListener(mediator));
        flipVBtn.setAction(new FlipVActionListener(mediator));
        flipHBtn.setAction(new FlipHActionListener(mediator));
        rotate90Btn.setAction(new Rotate90ActionListener(mediator));
        rotate180Btn.setAction(new Rotate180ActionListener(mediator));
        rotate270Btn.setAction(new Rotate270ActionListener(mediator));
        rotateMinus90Btn.setAction(new RotateMinus90ActionListener(mediator));
        rotateMinus180Btn.setAction(new RotateMinus180ActionListener(mediator));
        rotateMinus270Btn.setAction(new RotateMinus270ActionListener(mediator));
        rotateXDegrees.setAction(new RotateXDegrees(mediator));
        //lensBtn.setAction(new LensActionListener(mediator));
        clearLensBtn.setAction(new ClearLensActionListener(mediator));

        resetBtn.setAction(new ResetActionListener(mediator));
        //statusStr.setText(status);
        getContentPane().add(south, BorderLayout.SOUTH);

    }

    /**
     * 
     * @param comp
     * @param images
     * @param titles
     */

    public void init(JComponent comp, Image[] images, String[] titles) {
        this.titles = titles;
        this.comp = comp;
        tabPane.removeAll();
        imgCanvasList = new ImageCanvas2D[images.length];
        ImageIcon[] imgIconList = new ImageIcon[images.length];
        for (int i = 0; i < images.length; i++) {
            imgIconList[i] =
                new ImageIcon(
                    images[i].getScaledInstance(150, -1, Image.SCALE_DEFAULT));
        }
        for (int i = 0; i < images.length; i++) {
            imgCanvasList[i] = new ImageCanvas2D();
            imgCanvasList[i].setAWTImage(images[i]);
            tabPane.addTab(titles[i],imgCanvasList[i]);
            zoom = new PanZoom(imgCanvasList[i]);
        }
        String status = "1 of " + images.length;
        statusStr.setText(status);
        getContentPane().add(tabPane, BorderLayout.CENTER);
        pack();
    }

    private class NextButtonActionListener extends AbstractAction {

        private Mediator mediator;

        public NextButtonActionListener(Mediator mediator) {
            super("Next");
            this.mediator = mediator;
        }

        public void actionPerformed(ActionEvent e) {
            mediator.forwardActionEvent(e);
        }

    }

    private class PrevButtonActionListener extends AbstractAction {

        private Mediator mediator;

        public PrevButtonActionListener(Mediator mediator) {
            super("Previous");
            this.mediator = mediator;
        }

        public void actionPerformed(ActionEvent e) {
            mediator.forwardActionEvent(e);
        }

    }
    
    private class LensActionListener extends AbstractAction {

        private Mediator mediator;

        public LensActionListener(Mediator mediator) {
            super("Lens");
            this.mediator = mediator;
        }

        public void actionPerformed(ActionEvent e) {
            mediator.forwardActionEvent(e);
        }

    }
    
    private class ClearLensActionListener extends AbstractAction {

        private Mediator mediator;

        public ClearLensActionListener(Mediator mediator) {
            super("Clear Lens");
            this.mediator = mediator;
        }

        public void actionPerformed(ActionEvent e) {
            mediator.forwardActionEvent(e);
        }

    }



    private class CancelActionListener extends AbstractAction {

        private Mediator mediator;

        public CancelActionListener(Mediator mediator) {
            super("Cancel");
            this.mediator = mediator;
        }

        public void actionPerformed(ActionEvent e) {
            mediator.forwardActionEvent(e);
        }

    }

    private class FlipHActionListener extends AbstractAction {

        private Mediator mediator;

        public FlipHActionListener(Mediator mediator) {
            super("Flip Horizontal");
            this.mediator = mediator;
        }

        public void actionPerformed(ActionEvent e) {
            mediator.forwardActionEvent(e);
        }

    }

    private class FlipVActionListener extends AbstractAction {

        private Mediator mediator;

        public FlipVActionListener(Mediator mediator) {
            super("Flip Vertical");
            this.mediator = mediator;
        }

        public void actionPerformed(ActionEvent e) {
            mediator.forwardActionEvent(e);
        }
    }

    private class ResetActionListener extends AbstractAction {
        private Mediator mediator;

        public ResetActionListener(Mediator mediator) {
            super("Reset");
            this.mediator = mediator;
        }

        public void actionPerformed(ActionEvent e) {
            mediator.forwardActionEvent(e);
        }
    }

    private class Rotate90ActionListener extends AbstractAction {
        private Mediator mediator;

        public Rotate90ActionListener(Mediator mediator) {
            super("+90");
            this.mediator = mediator;
        }

        public void actionPerformed(ActionEvent e) {
            mediator.forwardActionEvent(e);
        }
    }

    private class Rotate180ActionListener extends AbstractAction {
        private Mediator mediator;

        public Rotate180ActionListener(Mediator mediator) {
            super("+180");
            this.mediator = mediator;
        }

        public void actionPerformed(ActionEvent e) {
            mediator.forwardActionEvent(e);
        }
    }

    private class Rotate270ActionListener extends AbstractAction {
        private Mediator mediator;

        public Rotate270ActionListener(Mediator mediator) {
            super("+270");
            this.mediator = mediator;
        }

        public void actionPerformed(ActionEvent e) {
            mediator.forwardActionEvent(e);
        }
    }

    private class RotateMinus90ActionListener extends AbstractAction {
        private Mediator mediator;

        public RotateMinus90ActionListener(Mediator mediator) {
            super("-90");
            this.mediator = mediator;
        }

        public void actionPerformed(ActionEvent e) {
            mediator.forwardActionEvent(e);
        }
    }

    private class RotateMinus180ActionListener extends AbstractAction {
        private Mediator mediator;

        public RotateMinus180ActionListener(Mediator mediator) {
            super("-180");
            this.mediator = mediator;
        }

        public void actionPerformed(ActionEvent e) {
            mediator.forwardActionEvent(e);
        }
    }

    private class RotateMinus270ActionListener extends AbstractAction {
        private Mediator mediator;

        public RotateMinus270ActionListener(Mediator mediator) {
            super("-270");
            this.mediator = mediator;
        }

        public void actionPerformed(ActionEvent e) {
            mediator.forwardActionEvent(e);
        }
    }

    private class RotateXDegrees extends AbstractAction {
        private Mediator mediator;

        public RotateXDegrees(Mediator mediator) {
            super("XDeg");
            this.mediator = mediator;
        }

        public void actionPerformed(ActionEvent e) {
            mediator.forwardActionEvent(e);
        }
    }

    private class ZoomAction extends AbstractAction {
        private Mediator mediator;

        public ZoomAction(Mediator mediator) {
            super("Zoom");
            this.mediator = mediator;
        }

        public void actionPerformed(ActionEvent e) {
            mediator.forwardActionEvent(e);
        }
    }

    public void rotate(double ang, int index) {
        float angleInRad = (float) (ang * (Math.PI / 180.0F));

        Dimension dim = imgCanvasList[index].getImageSize();
        int wid = dim.width;
        int ht = dim.height;
        AffineTransform atx = imgCanvasList[index].getTransform();
        atx.rotate(angleInRad, wid / 2, ht / 2);
        imgCanvasList[index].applyTransform(atx);
    }

    /**
     * This class mediates between the various compoments in the widget.
     * All Events from the next and previous button are passed to the
     * Mediator class. It then updates the gui appropriately.
     * @author tcanavan
     *
     * To change this generated comment edit the template variable "typecomment":
     * Window>Preferences>Java>Templates.
     * To enable and disable the creation of type comments go to
     * Window>Preferences>Java>Code Generation.
     */

    private class Mediator {

        public void forwardActionEvent(ActionEvent ev) {
            if (ev.getActionCommand().equals("Cancel")) {
                processCancelEvent();
            }
            if (ev.getActionCommand().equals("+90")) {
                processRotateEvent(90);
            }

            if (ev.getActionCommand().equals("+180")) {
                processRotateEvent(180);
            }
            if (ev.getActionCommand().equals("+270")) {
                processRotateEvent(270);
            }
            if (ev.getActionCommand().equals("-90")) {
                processRotateEvent(-90);
            }

            if (ev.getActionCommand().equals("-180")) {
                processRotateEvent(-180);
            }
            if (ev.getActionCommand().equals("-270")) {
                processRotateEvent(-270);
            }
            
            if (ev.getActionCommand().equals("Lens")) {
                processLensEvent();
            }
            
            if (ev.getActionCommand().equals("Clear Lens")) {
                //processClearLensEvent();
            }



            if (ev.getActionCommand().equals("Flip Vertical")) {
                processFlipVerticalEvent();
            }

            if (ev.getActionCommand().equals("Flip Horizontal")) {
                processFlipHorizontalEvent();
            }

            if (ev.getActionCommand().equals("XDeg")) {
                int degrees = new Integer(degField.getText()).intValue();
                processRotateEvent(degrees);
            }

            if (ev.getActionCommand().equals("Reset")) {
                processResetEvent();
            }

        }
        /**
         * Method processLensEvent.
         */
        private void processLensEvent() {            
            if(lensFrame != null) { lensFrame.dispose();}
                lensFrame = new JFrame();
                lensFrame.addWindowListener(
                   new WindowAdapter(){
                      public void windowClosing(WindowEvent w){
                         zoom.setPanZoomOn(true);
                         lensPanel.setLensOn(false);
                      }
                   }
                );
                lensPanel = new LensPanel(imgCanvasList[tabPane.getSelectedIndex()]);
                lensFrame.getContentPane().add(lensPanel);
                lensFrame.pack();
             //}
             imgCanvasList[tabPane.getSelectedIndex()].paintImage();
             zoom.setPanZoomOn(false);
             lensFrame.show();
             lensPanel.setLensOn(true);

            
        }

        public AffineTransform createFlipTransform(
            int mode,
            int imageWid,
            int imageHt) {
            AffineTransform at = new AffineTransform();
            switch (mode) {
                case FlipMode.NORMAL :
                    break;
                case FlipMode.TOP_BOTTOM :
                    at =
                        new AffineTransform(
                            new double[] { 1.0, 0.0, 0.0, -1.0 });
                    at.translate(0.0, -imageHt);
                    break;
                case FlipMode.LEFT_RIGHT :
                    at =
                        new AffineTransform(
                            new double[] { -1.0, 0.0, 0.0, 1.0 });
                    at.translate(-imageWid, 0.0);
                    break;
                case FlipMode.TOP_BOTTOM_LEFT_RIGHT :
                    at =
                        new AffineTransform(
                            new double[] { -1.0, 0.0, 0.0, -1.0 });
                    at.translate(-imageWid, -imageHt);
                    break;
                default :
                    }
            return at;
        }

        public void flip(int mode, int index) {
            Dimension dim = imgCanvasList[index].getImageSize();
            int wid = dim.width;
            int ht = dim.height;
            AffineTransform flipTx = createFlipTransform(mode, wid, ht);
            AffineTransform atx = imgCanvasList[index].getTransform();
            atx.concatenate(flipTx);
            imgCanvasList[index].applyTransform(atx);
        }

        private void processFlipVerticalEvent() {
            flip(FlipMode.TOP_BOTTOM,tabPane.getSelectedIndex());
        }

        private void processRotateEvent(int degrees) {
            rotate(degrees,tabPane.getSelectedIndex());
        }

        private void processFlipHorizontalEvent() {
            flip(FlipMode.LEFT_RIGHT,tabPane.getSelectedIndex());
        }

        private void processResetEvent() {
            imgCanvasList[tabPane.getSelectedIndex()].reset();
        }
        
        

        private void processCancelEvent() {
            setVisible(false);
            //dispose();
        }

    }
}
