package org.eso.ohs.core.gui.widgets;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.lang.reflect.InvocationTargetException;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.SwingUtilities;

import org.apache.log4j.Logger;

/**
 * @author  pnunes
 * 
 * This class implements a dialog that displays the progress of multiple tasks 
 * 
 * Created on December 7, 2005, 2:20 PM
 */
public class MultipleProgressDialog extends JDialog implements ProgressModelListener , Runnable {
    

	private static Logger stdlog_ =
        Logger.getLogger(MultipleProgressDialog.class);
    private JLabel labelStatus;
    private JPanel panelStatus;
    private JProgressBar progressBarStatus;
    private ProgressModel [] data;
    private ProgressPanel [] panels;
    private int totalSteps;
	private JPanel panelDetails;
	private JButton buttonDetails;
	
	
    /** Creates new form MultipleProgressDialog */
    public MultipleProgressDialog(Frame parent,  ProgressModel [] initData) {
        super(parent,"Processing...",true);
        data = initData;
        initComponents(parent);
    }
    
    /** Creates new form MultipleProgressDialog */
    public MultipleProgressDialog(Dialog parent,  ProgressModel [] initData) {
        super(parent,"Processing...",true);
        data = initData;
        initComponents(parent);
    }
    

    private void initComponents(Component parent) {
    	                                                       
        getContentPane().setLayout(new GridBagLayout());   
        
        
        totalSteps = 0;        
        panelStatus = new JPanel();
        panelStatus.setLayout(new GridLayout());
        labelStatus = new JLabel();
        labelStatus.setText("Overall Progress:");        
        panelStatus.add(labelStatus);        
        
        
        progressBarStatus = new JProgressBar();
        progressBarStatus.setStringPainted(true);
        progressBarStatus.setForeground(Color.blue.darker()); 
        panelStatus.add(progressBarStatus);

        GridBagConstraints gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.insets = new Insets(5, 0, 5, 0);
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;            
        getContentPane().add(panelStatus, gridBagConstraints);
        
        
        
        buttonDetails = new JButton();
        buttonDetails.setText("Details >>");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.insets = new Insets(0, 5, 0, 5);
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        buttonDetails.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                buttonDetailsActionPerformed(evt);
            }

			
        });        
        getContentPane().add(buttonDetails, gridBagConstraints);
        
        panelDetails = new JPanel();
        panelDetails.setLayout(new GridBagLayout());    
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
                
              
        
        panels = new ProgressPanel[data.length];
        for (int i = 0; i < data.length; i++) {
        	
        	
        	ProgressModel model = data[i];        	
        	ProgressPanel panel = new ProgressPanel(model);        	        	
        	
        	model.addChangeListener(panel);
        	model.addChangeListener(this);
        	panels[i] = panel;
        	
        	gridBagConstraints = new GridBagConstraints();
        	           
            gridBagConstraints.insets = new Insets(2, 0, 2, 0);
            gridBagConstraints.gridx = 0;
            gridBagConstraints.gridy = i;
            panelDetails.add(panel, gridBagConstraints);                                    
		}

        panelDetails.setVisible(false);
        getContentPane().add(panelDetails, gridBagConstraints);
        
        progressBarStatus.setMinimum(1);
        progressBarStatus.setMaximum(totalSteps());
        	
        
        Point pt = parent.getLocation();
		Dimension sz = parent.getSize();
		pt.translate(sz.width/2, sz.height/2);
		setLocation(pt);
        pack();
    }
    
    private int totalSteps() {
    	int result = 0;
    	
    	for (int i = 0; i < data.length; i++) {
    		ProgressModel model = data[i];  
    		result += (model.getMaximum() - model.getMinimum());	
    	}
    	
    	return(result);
    }

   
    private void buttonDetailsActionPerformed(ActionEvent evt) {
		if( panelDetails.isVisible()) {
			panelDetails.setVisible(false);		
		}
		else {
			panelDetails.setVisible(true);			
		}
		
		this.pack();
		
	}
    

	public void modelChanged(ProgressModelEvent e)  {
				
		progressBarStatus.setMinimum(1);
        progressBarStatus.setMaximum(totalSteps());
		
		int value = progressBarStatus.getValue(); 
		progressBarStatus.setValue(value+1);							
				
		ProgressModel model = (ProgressModel) e.getSource();						
		
		String msg = model.getMsg();
		labelStatus.setText(msg);
		
		new SlightPause(this);
					
	}


	public void run() {
		this.show();
	}
	
	
	
	/** Utility class to execute the given Runnable object afer a delay */
    public static class SlightPause extends Thread {
        private Runnable runner_;
        private int millisecDelay_;

        public SlightPause(Runnable run, int delay) {
            millisecDelay_ = delay;
            runner_ = run;

            start();
        }

        public SlightPause(Runnable run) {
            this(run, 5);
        }

        public void run() {
            try {
                sleep(millisecDelay_); // Pause for Display to catch up.
            } catch (InterruptedException e) {
                // do nothing -- ignore
            }
            try {
				SwingUtilities.invokeAndWait(runner_);
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (InvocationTargetException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
        }
    }


    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
    	

    	ProgressModel [] models = new ProgressModelDefault[2];
    	models[0] = new ProgressModelDefault(0,10,"Loading OBs: ");
    	models[1] = new ProgressModelDefault(0,15,"Calculating visibilities: ");
    	
    	
    	MultipleProgressDialog dialog = new MultipleProgressDialog(new JFrame(),models);
    	
    	SwingUtilities.invokeLater(dialog);
    	
    	//Thread t = new Thread(dialog);
    	//t.start();

    	for (int i = 1; i <= 10; i++) {
    		try {
    			Thread.sleep(500);    			
    		} catch (InterruptedException e) {
    			e.printStackTrace();
    		}    		    		
    		models[0].setValue(i);
    		
    	}
    	
    	for (int i = 1; i <= 10; i++) {
    		try {
    			Thread.sleep(500);    			
    		} catch (InterruptedException e) {
    			// TODO Auto-generated catch block
    			e.printStackTrace();
    		}    		    		
    		models[1].setValue(i);
    		
    	}
    	try {
			Thread.sleep(3000);
		} catch (InterruptedException e) {

			e.printStackTrace();
		}         
        System.exit(0);
            

    }

	
    
    
}


