/*
 * Created by amchavan on Jul 14, 2005
 */
package org.eso.ohs.core.gui.widgets;
import java.util.Enumeration;
import java.util.Hashtable;

import javax.swing.JComponent;

import uic.widgets.UICWizard;


/**
 * Extends {@link uic.widgets.UICWizard} to add support for OHSWizardPage
 * and key/value pairs. 
 * <p/>
 * See {@linkplain http://uic.sourceforge.net/product/Wizard/view.phtml}
 * for more info.
 * 
 * @see OHSWizardPage
 * @version $Revision: 1.5 $
 * @author amchavan, Jul 14, 2005
 */

// $Id: OHSWizard.java,v 1.5 2006/04/26 09:26:44 tcanavan Exp $

public abstract class OHSWizard extends UICWizard {

    
    /** Where we keep our key/values pairs. */
    protected Hashtable<String, Object> values = new Hashtable<String, Object>();

    /** 
     * Where we keep track of the components that are registered
     * with us; they are indexed by a string (usually the 
     * component's name).
     */
    protected Hashtable<Object, String> widgetNames = new Hashtable<Object, String>();

    /**
     * Should never be called, use {@link #addPage(OHSWizardPage)} instead.
     * @see uic.widgets.UICWizard#addPage(javax.swing.JComponent)
     * @deprecated Use {@link #addPage(OHSWizardPage)} instead.
     */
    public void addPage( JComponent page ) {
        throw new IllegalArgumentException(
                    "Use addPage(OHSWizardPage) instead." );
    }
    
    /**
     * Substitutes 
     * {@link uic.widgets.UICWizard#addPage(javax.swing.JComponent)}
     * to enforsce usage of OHSWizardPages instead of generic
     * JComponents.
     * 
     * @param page The page to be added to the wizard.
     */
    public void addPage( OHSWizardPage page ) {
        super.addPage( (JComponent) page );
    }

    /**
     * In addition to the normal behaviour of this method, sets
     * the current page as verified if the current page is ready.
     * 
     * @see OHSWizardPage#isReady()
     * @see uic.widgets.UICWizard#enteringPage(javax.swing.JComponent, boolean)
     */
    protected void enteringPage( JComponent page, boolean arg1 ) {        
        OHSWizardPage wpage = (OHSWizardPage) page;
        setVerified( page, wpage.isReady() );
        super.enteringPage( page, arg1 );
    }
    
    /**
     * @return The value of the input key (possibly null). How
     *         keys/value pairs are entered is described with
     *         the <code>listenTo()</code> methods of the 
     *         OHSWizardPage class.
     *         
     * @see OHSWizardPage#listenTo(JComboBox)
     * @see OHSWizardPage#listenTo(JRadioButton)
     * @see OHSWizardPage#listenTo(JTextField)
     */
    public Object get( String key ) {
        return values.get( key );
    }
    
    /**
     * removes the selected key
     */
    public void remove( String key ) {
        values.remove( key );
    }
    
    

    /** 
     * @return The table of the components that are registered
     *         with us; they are indexed by a string (usually the 
     *         component's name).
     */         
    public Hashtable<Object, String> getWidgetNames() {
        return widgetNames;
    }
    
    /**
     * @return An enumeration of the keys in this wizard. How
     *         keys/value pairs are entered is described with
     *         the <code>listenTo()</code> methods of the 
     *         OHSWizardPage class.
     *         
     * @see OHSWizardPage#listenTo(JComboBox)
     * @see OHSWizardPage#listenTo(JRadioButton)
     * @see OHSWizardPage#listenTo(JTextField)
     */
    public Enumeration<String> keys() {
        return values.keys();
    }

    /**
     * This method gets called when the user terminates interacting
     * with a page of the wizard. THis method should not be called.
     * Call the leavingPage(OHSWizardPage, boolean) instead.
     * 
     * @param from      The OHSWizardPage we just left.
     * @param usingNext <code>true</code> if we left the page with
     *                  the <em>Next</em> button, <code>false</code>
     *                  otherwise.
     * 
     * @see OHSWizardPage#leaving()
     * @see uic.widgets.UICWizard#leavingPage(javax.swing.JComponent, boolean)
     */
    protected void leavingPage( JComponent from, boolean usingNext ) {
       OHSWizardPage fromPage = (OHSWizardPage) from;
       leavingPage(fromPage,usingNext);
    }
    
    /**
     * This method gets called when the user terminates interacting
     * with a page of the wizard. This implementation does nothing.
     * 
     * @param from      The OHSWizardPage we just left.
     * @param usingNext <code>true</code> if we left the page with
     *                  the <em>Next</em> button, <code>false</code>
     *                  otherwise.
     * 
     * @see OHSWizardPage#leaving()
     * @see uic.widgets.UICWizard#leavingPage(OHSWizardPage, boolean)
     */
    protected void leavingPage( OHSWizardPage from, boolean usingNext ) {
      
    }
    
    
    /**
     * This method gets called when the user terminates interacting
     * with the wizard. This implementation does nothing; subclasses
     * should add their own behaviour if needed.
     * 
     * @param successful Will be set to <code>true</code> if the
     *                   user clicked on the <em>Finish</em> button;
     *                   to <code>false</code> if the
     *                   user clicked on <em>Cancel</em>.
     */
    protected void leavingWizard( boolean successful ) {
        // no-op
    }
    
    /** 
     * Store a key/value pair in this wizard. 
     */ 
    public void put( String key, String value ) {
        values.put( key, value );
    }
    
    /*
     * Store a key/array pair in ths wizard
     */
    public void put( String key, String [] val) {
        values.put( key, val );
    }
    

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return this.getClass().getName();
    }
}
