package org.eso.ohs.core.gui.widgets;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.io.File;

import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.KeyStroke;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.FileIOUtils;

/**
 * This is a generic Text Display widget which allows you to Display,Print and
 * Save Text in a TextArea. It is widly used in OHS applications
 *  
 */

public class TextDisplayWidget extends JFrame {

    /** Save Report MenuItem */
    private static Logger stdlog_ = Logger.getLogger(TextDisplayWidget.class);

    protected JMenuItem saveReport_;

    /** Open Report MenuItem */
    private JMenuItem openReport_;

    /** Print Report Menu item */
    private JMenuItem printReport_;

    /** Text Are to display contents in */

    protected JTextArea textArea_ = new JTextArea();

    /** Close the widget down */
    private JMenuItem exit_;

    /** File menu */
    protected final JMenu fileMenu = new JMenu("File");

    /** Text Display widget constructor */

    public TextDisplayWidget(String title) {
        initialize(true, true, true);
        setTitle(title);
    }

    private static String previousPath_ = null;

    private String currentPath_ = null;

    /**
     * set the path user for saving the content if the String is not a valid
     * directory the value is not set
     * 
     * @param currentPath
     *            the directory
     */
    public void setCurrentPath(String currentPath) {
        File d = new File(currentPath_);
        if (d.isDirectory()) {
            currentPath_ = currentPath;
        } else {
            stdlog_.warn("Directory not found: " + currentPath);
        }
    }

    /**
     * set the path user for saving the content as the last used by the
     * application
     */
    public void setCurrentPathAsLast() {
        if (previousPath_ != null) {
            File d = new File(previousPath_);
            if (d.isDirectory()) {
                currentPath_ = previousPath_;
            } else {
                stdlog_.warn("Directory not found: " + previousPath_);
            }
        } else {
            stdlog_.warn("previous path not set");
        }
    }

    public void setWordWrappping(boolean enabled) {
        textArea_.setWrapStyleWord(enabled);
        textArea_.setLineWrap(enabled);
    }

    /**
     * Text Display widget
     * 
     * @param title
     *            of Widget
     * @param saveOption
     *            Add Save button
     * @param openOption
     *            Add Open button
     * @param printOption
     *            Add Print Button
     */

    public TextDisplayWidget(String title, boolean saveOption,
            boolean openOption, boolean printOption) {
        initialize(saveOption, openOption, printOption);
        setTitle(title);
    }

    /**
     * Deefault Constructor TextDisplayWdget
     */
    public TextDisplayWidget() {
        initialize(true, true, true);
    }

    /**
     * Initialise the Text Display Widget
     * 
     * @param saveOption
     * @param openOption
     * @param printOption
     */

    private void initialize(boolean saveOption, boolean openOption,
            boolean printOption) {
        JMenuBar mb = new JMenuBar();
        setVisible(false);
        this.setJMenuBar(mb);
        fileMenu.setMnemonic('F');
        mb.add(fileMenu);
        saveReport_ = new JMenuItem("Save", 'S');
        openReport_ = new JMenuItem("Open", 'E');
        printReport_ = new JMenuItem("Print", 'P');
        exit_ = new JMenuItem("Close", 'w');
        exit_.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_W,
                InputEvent.CTRL_MASK, false));
        
        saveReport_.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S,
                InputEvent.CTRL_MASK, false));
        
        openReport_.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_E,
                InputEvent.CTRL_MASK, false));
        printReport_.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_P,
                InputEvent.CTRL_MASK, false));


        //test which menu item must be added
        if (saveOption)
            fileMenu.add(saveReport_);
        if (openOption)
            fileMenu.add(openReport_);
        if (printOption)
            fileMenu.add(printReport_);

        fileMenu.add(exit_);
        this.getContentPane().add(
                new JScrollPane(textArea_ = new JTextArea(20, 40)));
        //show();
        pack();
        //addClosableWindow();

        createSaveActionListener();
        //saveReport_.addActionListener(new SaveActionListener());

        openReport_.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                JFileChooser chooser = new SwingFileChooser();
                int option = chooser.showOpenDialog(fileMenu);
                if (option == JFileChooser.APPROVE_OPTION) {
                    textArea_.setText("");
                    try {
                        textArea_.setText(FileIOUtils.fileToString(chooser
                                .getSelectedFile().getAbsolutePath()));
                        setTitle(chooser.getSelectedFile().getName());
                    } catch (Throwable t) {
                        textArea_.setText("Failed to read file contents.");
                    }
                }
            }
        });

        printReport_.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                Print.print((JFrame) textArea_.getTopLevelAncestor(), textArea_
                        .getText());

            }
        });

        exit_.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                dispose();
            }
        });
    }

    /**
     * Set the text area with string data
     * 
     * @param info
     *            String data
     */
    public void setTextArea(String info) {
        textArea_.append(info);
        textArea_.repaint();
        textArea_.setCaretPosition(0);
        pack();
    }

    /**
     * Clear text Area
     */
    public void clearTextArea() {
        textArea_.setText("");
    }

    /**
     * Create new Save action listener
     */

    public void createSaveActionListener() {
        saveReport_.addActionListener(new SaveActionListener());
    }

    /**
     * Set text for Save menu
     * 
     * @param s
     *            Set the save text
     */
    public void setSaveMenuText(String s) {
        saveReport_.setText(s);
    }

    /**
     * Set the open Text
     * 
     * @param s
     *            Set the open text
     */

    public void setOpenMenuText(String s) {
        openReport_.setText(s);
    }

    protected class SaveActionListener implements ActionListener {

        public void actionPerformed(ActionEvent ae) {
            JFileChooser chooser = null;

            if (currentPath_ != null) {
                File d = new File(currentPath_);
                if (d.exists())
                    chooser = new SwingFileChooser(currentPath_);
            } else
                chooser = new SwingFileChooser();

            int option = chooser.showSaveDialog(fileMenu);

            if (option == JFileChooser.APPROVE_OPTION) {
                try {
                    File f = new File(chooser.getSelectedFile()
                            .getAbsolutePath());
                    int returnValue = JOptionPane.OK_OPTION;
                    if (f.exists()) {
                        JOptionPane.showConfirmDialog(chooser,
                                "Do you wish to overwrite file " + f.getName()
                                        + " ?");
                    }
                    if (returnValue == JOptionPane.OK_OPTION) {
                        FileIOUtils.stringToFile(textArea_.getText(), chooser
                                .getSelectedFile().getAbsolutePath());
                        currentPath_ = chooser.getCurrentDirectory().toString();
                        previousPath_ = currentPath_;
                    }
                } catch (Throwable t) {
                    textArea_.setText("Failed to write file contents.");
                }
            }
        }
    }

}

