package org.eso.ohs.core.utilities;

import java.io.IOException;
import java.util.zip.CRC32;


/**
 * This class contains a utility methods check the CRC 
 * of a PAF file based on the crc32 Algorithm. PAF files 
 * contain the PAF.CHCKSUM keyword. 
 * If filled out the checksum is calculated on the rest of the file except
 * checksum. 
 * 
 * So 
 * PAF.CHCK.CHECKSUM "13313131313"
 * Remove    13313131313
 * Run CRC32 on the rest of the data. It shuuld equal the value between the quotes.
 *  
 * @author tcanavan
 *
 */
public class CRC32PAFFile {

 
    /**
     * Get a checksum from a PAF Fle. Delete the
     * value in  PAF.CHECKSUM and then compute the checksum.
     * @param filename
     * @return long
     * @throws IOException
     */
 
 
    public static long getCheckSum(String filename) throws IOException {
        StringBuffer buf = new StringBuffer(FileIOUtils.fileToString(filename));
        return getChecksum(buf);
        
    }
    
    /**
     * Compute the checksum on everytthing except the 
     * data between the PAF.CHCK.CHECKSUM quotes in the PAFFile. 
     * @param buf
     * @return
     */


    public static long getChecksum(StringBuffer buf) {
        int index = buf.indexOf("PAF.CHCK.CHECKSUM");
        int firstIndex = buf.indexOf("\"",index);
        int lastIndex = buf.indexOf("\"",firstIndex+1);
        buf.delete(firstIndex+1,lastIndex);
        CRC32 crc = new CRC32();
        crc.update(buf.toString().getBytes());
        return crc.getValue();       
    }
    
    /**
     * Check if a checksum is valid.
     * @param filename
     * @param checkSum
     * @return
     * @throws IOException
     */
    
    
    public static boolean validateChecksum(String filename,long checkSum) throws IOException {
        long validChecksum  = getCheckSum(filename);
        if(validChecksum == checkSum) {
            return true;
        }
        return false;
    }

}
