package org.eso.ohs.core.utilities;

/**
 * Default implementation of the GTreePath interface. This implementation
 * allows for the construction of a tree path through the addition of 
 * GTreeNodes.
 *
 * Please see GTreeModel for other details.
 *
 * @author Nick Kornweibel for ESO.
 */

import java.util.Vector;


public class DefaultGTreePath implements GTreePath {

    /**
     * Vector to store the tree nodes. The first element is the root node
     * of the tree path.
     */
    protected Vector<GTreeNode> treePath_ = new Vector<GTreeNode>();

    /**
     * Create a tree path with an empty path.
     */
    public DefaultGTreePath() {
    }

    /**
     * Create a tree path with the specified array as the path.
     */
    public DefaultGTreePath(String[] path) {
        setPath(path);
    }

    public DefaultGTreePath(GTreeNode[] path) {
        setPath(path);
    }

    /**
     * Overwrite this tree path with new data.
     */

    public void setPath(String[] path) {
        treePath_.removeAllElements();
        for (int i = 0; i < path.length; i++) {
            addNode(new DefaultGTreeNode(path[i]));
        }
    }

    public void setPath(GTreeNode[] path) {
        treePath_.removeAllElements();
        for (int i = 0; i < path.length; i++) {
            addNode(path[i]);
        }
    }

    /**
     * Add a node to the end of the current path.
     */
    public void addNode(GTreeNode n) {
        treePath_.addElement(n);
    }
 
    /**
     * Clear the path - making this tree path empty
     */
    public void clearPath() {
        treePath_.removeAllElements();
    }

    /**
     * @return true if this is an empty tree path.
     */
    public boolean isEmpty() {
        return treePath_.isEmpty();
    }

    /**
     * @return this path as an array of GTreeNode objects.
     * With element 0 being the root of the tree.
     * Note that the nodes are NOT joined in
     * a parent<->child relationship, simply an array of nodes designating
     * the path.
     */

    public GTreeNode[] getPath() {
        GTreeNode[] rc = new GTreeNode[treePath_.size()];
        for (int i = 0; i < treePath_.size(); i++) {
            rc[i] = treePath_.elementAt(i);
        }
        return rc;
    }
}
