package org.eso.ohs.core.utilities;

/**
 * A static class containing Key definitions for modifying the AppConfig data set.
 * This class can be added to by all OHS applications and is a central reference
 * for these keys.
 *
 * @author N. Kornweibel for ESO
 * <br>ChangeLog: cmavros 11/02/02 1512/P2PP
 * <br>ChangeLog: cmavros 19/02/02, 1640/OT  
 * <br>ChangeLog: cmavros 20JUNE02 [1652/P2PP]
 * <br>ChangeLog: cmavros 02JUL02 Add DefaultConfigReportKey keyword
 * <br>ChangeLog: cmavros 03JUL02 Add OPTIONS.FOLDER keyword
 *
 */

/**
 * @author pnunes
 *
 */
public class OHSKeys {

    /**
     * Reserved suffix used to indicate a key is a directory or file path
     */
    public static final String FolderKey = "FOLDER";

    /**
     * Key Names used in this configuration
     */
    public static final String HomeDirKey = "DIR.HOME";
    public static final String ConfigDirKey = "DIR.CONFIG";
    public static final String BaseDirKey = "DIR.BASE";

    public static final String DBDirectSubConfig = "DB";
    public static final String InstFolderKey = "INSTRUMENTS.FOLDER";
    public static final String ExpFolderKey = "IMPEX.FOLDER";
	public static final String FCFolderKey = "FC.FOLDER";
    public static final String MailHostKey = "MAIL.SMTP.HOST";
    public static final String P2PPSubmitKey = "P2PP.SUBMIT";
    public static final String CacheFolderKey = "CACHE.FOLDER";
    public static final String DumpFolderKey = "DUMP.DIRECTORY";
    public static final String ParamFolderKey = "PARAMFILE.FOLDER";
    public static final String PafFolderKey = "PAF.DIRECTORY";
    public static final String VersionKey = "VERSION";
    public static final String P2PPServerUrl = "P2PP.SERVER.URL";

    public static final String MaxNameLengthKey = "PERSISTENCE.NAME.LEN";
    public static final String MaxTextLengthKey = "PERSISTENCE.TEXT.LEN";
    public static final String BusObjConfigKey = "PERSISTENCE.OBJCFG";

//   this keyword is not used anywhere. i'll leave it commented just in case    
//    public static final String SuperUserKey = "USER.SUPERUSER.ENTITLED";
    
    public static final String UserOverrideKey = "USER.OVERRIDE";

    public static final String StartAppServerKey = "APPSERVER.START";
    public static final String AppServerNameKey = "APPSERVER.NAME";
    public static final String AppServerPortKey = "APPSERVER.PORT";
    public static final String AppServerDataPortKey = "APPSERVER.DATAPORT";
    public static final String AppServerLogKey = "APPSERVER.LOG";

    public static final String CcsActiveKey = "CCS.ACTIVE";
    public static final String CcsConnEnabledKey = "CCS.CONNECTION.ENABLED";
    public static final String CcsServerNameKey = "CCS.OHS.SERVER.NAME";
    public static final String MaskTrackerEnabledKey =
        "CCS.MASKTRACKER.ENABLED";
        

    public static final String ReadmeP2PPEnabledKey =
		"README.P2PP.ENABLED";

    public static final String FCNotesEnabledKey =
		"FC.NOTES.ENABLED";
    
    
    
    public static final String USGEnabledKey =
        "USD.ENABLED";

    public static final String ObsLongKey =
        "OBSERVATORY.LONG";

        
    public static final String ObsLatKey =
        "OBSERVATORY.LAT";
        
    public static final String FCScaledKey =
        "FC.SCALED.ENABLED";
        
        
    public static final String MaxFindingChartKey = "MAX.FINDING.CHARTS";
     
    public static final String CheckSumEnabled = "CHECKSUM.DISABLED";

    public static final String MaxFindingChartSize = "MAX.FINDING.CHARTSSIZE";


    public static final String DefaultPeriodKey = "PERIOD.DEFAULT";

    // Config base used for configuration reports.
    public static final String ConfigReportKey = "CONFIGREPORT";

    // Option Menu Keys 
    public static final String RepositoryConfigReportKey =
        "REPOSITORYBROWSER.CONFIG.CONFIGMENU";
    public static final String QueueViewConfigReportKey =
        "QUEUEVIEW.CONFIG.CONFIGMENU";
    public static final String DefaultConfigReportKey =
        "QUEUEVIEW.CONFIG.MENUDEFAULT.defaultMenuName";
    public static final String OptionFolderKey = "OPTIONS.FOLDER";

    public static final String EngModeCloningKey = "ENGINEERINGMODE.CLONING";
    public static final String OTRefreshQueueKey = "OT.REFRESHQUEUE.ENABLED";

    /**
     * Config keyword (base) for list of instruments for which 
     * VM OBs may be checked in to the repository.
     *     INSTRUMENT.VMCHECKINOK.1, INSTRUMENT.VMCHECKINOK.1, etc.
     * 
     * The set is read numerically and in order and the first key which is
     * missing or empty terminates the sequence. That is, in the following:
     * INSTRUMENTS.VMCHECKINOK.1 "VIMOS"
     * INSTRUMENTS.VMCHECKINOK.2 "FORS1"
     * INSTRUMENTS.VMCHECKINOK.4 "ISAAC"
     * Only 2 values: "VIMOS" and "FORS1" would be returned.
     */

    public static final String VMCheckInOk = "INSTRUMENTS.VMCHECKINOK";
    
    /**
     * Config keyword (base) for list of instruments for which 
     * VM OBs may be checked in to the repository.
     *     INSTRUMENT.VMCHECKINOK.1, INSTRUMENT.VMCHECKINOK.1, etc.
     * 
     * The set is read numerically and in order and the first key which is
     * missing or empty terminates the sequence. That is, in the following:
     * INSTRUMENTS.VMCHECKINOK.1 "VIMOS"
     * INSTRUMENTS.VMCHECKINOK.2 "FORS1"
     * INSTRUMENTS.VMCHECKINOK.4 "ISAAC"
     * Only 2 values: "VIMOS" and "FORS1" would be returned.
     */

    public static final String STOk = "INSTRUMENTS.SIDEREALTIME";

    

    /**
     * Part of the Multiple Instruments Package project:
     * If "y" then instrument package version is expected and default instrument
     * package location is not used.
     */
    public static final String IPVersioningKey = "INSTRUMENTS.IPVERSIONING";

    /**
     * Part of the Multiple Instruments Package project:
     * The first period for which IP Versioning is expected to be 
     * applied. Data associated with previous periods is not expected
     * to be associated with an IP version.
     */
    public static final String IPVersioningStartPeriodKey =
        "INSTRUMENTS.IPVERSIONING.STARTPERIOD";

    /**
     * Part of the Multiple Instruments Package project:
     * This FOLDER config setting determines where IPs are installed when
     * downloaded from the database by an application. The setting may be the
     * instruments folder, but is not required to be as in certain installations
     * the instruments folder is not writable and so IPs must be installed elsewhere.
     */
    public static final String IPVersioningIPCache =
        "INSTRUMENTS.IPVERSIONING.IPCACHE.FOLDER";

    /**
     * Part of the Multiple Instruments Package project:
     * This keyword sets OHS applications into a single IP version mode, where IPs
     * are returned are always the latest (highest) version, regardless of the version
     * that was required. This is used by OHS applications running in "Mountain Mode".
     */
    public static final String IPVersioningSingleIPModeEnabled =
        "INSTRUMENTS.IPVERSIONING.SINGLEIPMODE";

    /**
     * Part of the Multiple Instruments Package project:
     * This property, when enabled ('y') in conjunction with:
     *   -  single IP mode is NOT enabled 
     *   - IP cache folder is defined
     * indicates to the OHS application that missing IPs should be automatically
     * requested and downloaded from the database as required.
     * Note: that this is not a standard OHS property, and is supported currently only
     * by the AppServer and OT.
     */
    public static final String IPVersioningAutodownloadIP =
        "INSTRUMENTS.IPVERSIONING.IPCACHE.AUTODOWNLOAD";

    /**
     * Part of the Multiple Instruments Package project:
     * This property, when enabled ('y') will only permit checking-in of
     * OBs if the version of the IP used in the OB matches the required IP version
     * in the Observing Run. This config item has relevance at the database level,
     * and so will affect the following applications:
     * P2PP (in direct mode ONLY).
     * AppServer.
     */
    public static final String IPVersionsingEnsureCorrectIPVersionOnOBCheckin =
        "IPVERSIONING.CHECKIN.IPVERSIONCHECK";
        
    /**
     * Part of the Multiple Instrument Package Project:
     * This is a comma separated list of instruments that are known NOT to
     * exist. For this reason, they will not be requested from the AppServer or
     * downloaded from the IP Archive.
     */ 
    public static final String InstrumentsWithNoIP = "INSTRUMENTS.WITHNOIP";

    //Http configuration settings
    public static final String HttpProxyHostKey = "APPSERVER.HTTPPROXYHOST";
    public static final String HttpProxyPortKey = "APPSERVER.HTTPPROXYPORT";
    
    /**
     * The set of periods for which the appserver will check-in OBs/CBs - 
     * if the period of the OB/CB falls outside of this period, an exception is
     * thrown and the check-in fails.
     * If empty of non-existant, all periods are acceptable.
     */
    public static final String AcceptableCheckinPeriodSet = "APPSERVER.CHECKIN.PERIODS";

    /** Database name of USG phase2 database */
    public static final String USGDatabase = "USG.DATABASE";

    /** Directory for IPacMan's temporary files/directories */
    public static final String IPacManTmpFolderKey = "IPACMAN.TEMPDIR.FOLDER";
    
    
    

    /**
     * List of comma separated instruments that are ignored when generating
     * the missing IP report.
     * Valid separator characters include: ",", ";", " " and TAB.
     * Note that instrument names are case sensitive.
     */
    public static final String IPacManIgnoreIPList =
        "IPACMAN.MISSINGIPS.IGNORELIST";

    
    
    
    /**
     * Role names config parameters for the Engineering and Operations accounts 
     */
    public static final String engineeringRoleName = "ENGINEERING.ROLE.NAME";
    public static final String operationsRoleName = "OPERATIONS.ROLE.NAME";
    

    /**
     * 
     * The set of attributes of the Target and ConstraintSet objects that are
     * read from param files attached to templates. The keywords list the
     * parameter names that are read from any param file attached.
     * 
     * The Parameter names are numbered, to allow multiple param files to be
     * handled, for example, in the following definition, RA may be read from a
     * param file if it matches either of the parameters specified:
     * 
     * TARGET.RA.PARAM.1 "TEL.TARG.ALPHA"
     * TARGET.RA.PARAM.2 "TEL.TARG.RA"
     * 
     * In the above example, if TEL.TARG.ALPHA will be taken from the param
     * file (if present), before TEL.TARG.RA, due to the numbering of the
     * parameters: 1 and 2.
     * 
     * For example to read the SIM.PERF.AM parameter from a param file and
     * set it to the airmass of a ConstraintSet:
     * CONSTRAINTSET.AIRMASS.PARAM.1     "SIM.PERF.AM"
     */
    
    // ConstraintSet properties
    public static final String CSAirMassParam = "CONSTRAINTSET.AIRMASS.PARAM";
    public static final String CSFliParam     = "CONSTRAINTSET.FLI.PARAM";
    public static final String CSMoonParam    = "CONSTRAINTSET.MOON.PARAM";
    public static final String CSSeeingParam  = "CONSTRAINTSET.SEEING.PARAM";
    public static final String CSSkyParam     = "CONSTRAINTSET.SKY.PARAM";
    public static final String CSTelTempParam = "CONSTRAINTSET.TELTEMP.PARAM";
    public static final String CSVapourParam  = "CONSTRAINTSET.VAPOUR.PARAM";
    public static final String CSStrehlParam  = "CONSTRAINTSET.STREHLRATIO.PARAM";
    
    // Target properties
    public static final String TargetRAParam      = "TARGET.RA.PARAM"; 
    public static final String TargetPropRAParam  = "TARGET.PROPRA.PARAM";       
    public static final String TargetDecParam     = "TARGET.DEC.PARAM";
    public static final String TargetPropDecParam = "TARGET.PROPDEC.PARAM";
    public static final String TargetEquinoxParam = "TARGET.EQUINOX.PARAM";
    public static final String TargetEpochParam   = "TARGET.EPOCH.PARAM";
    public static final String TargetColorParam   = "TARGET.COLOR.PARAM";
    public static final String TargetMagParam     = "TARGET.MAG.PARAM";
    public static final String TargetNameParam    = "TARGET.NAME.PARAM";
    

    /**
     * P2PP site.cf setting. If set to "Y" will present the user with an "ok" dialog
     * when an OB (or CB) has been converted from one IP version to another. If not set
     * or set to "n", no dialog is presented.
     */
    public static final String ShowConversionDialog = "P2PP.IPVERSIONING.SHOWCONVERSION";
    
	/**
	 * Platform reserved keyword. A search for this keyword is not case sensitive
	 */
	public static final String platform = "PLATFORM";    
}
