package org.eso.ohs.core.utilities;


/**
 * A wrapper-exception for input/output exceptions from StorageManager
 * subclasses.  Because there are several different kinds of
 * StorageManager, each producing different kinds of exceptions it
 * is necessary to use a wrapper, so that the higher levels can know
 * exactly which kind of exception to catch for an I/O error.
 * There is also a second constructor for errors generated within
 * the storage manager, i.e.\ not due to another component throwing
 * an exception.
 * 
 * <p>The original exception (if any) is accessible using the
 * getException() method.
 * 
 * <p>
 * $Id: ObjectIOException.java,v 1.2 2006/03/20 14:09:29 ddorigo Exp $
 */

public class ObjectIOException extends Exception {

    String message = null;
    /**
       * Construct an exception that does <em>not</em> contain another
       * exception.
       * 
       * @param msg   explanatory message giving a reason for the exception.
       */
      public ObjectIOException(String msg) {
    super(msg);
      message = msg;
      }

      /**
       * Construct an exception that contains another exception.
       * The containing exception has the same message as the original.
       * 
       * @param exc     the original exception
       */
      public ObjectIOException(Throwable exc) {
    this(exc, exc.getMessage());
      }

      /**
       * Construct an exception that contains another exception.
       * The containing exception has the same message as the original.
       * 
       * @param exc     the original exception
       */
      public ObjectIOException(Throwable exc, String message) {
    super(exc);
      this.message = message;
      
      }

   public String getMessage() {
       if (message == null) {
           message = super.getMessage();
       }
       return message;
   }
    
}

