package org.eso.ohs.core.utilities.javamail;

// // The Imports

// // java util imports:
import java.io.File;
import java.util.Date;
import java.util.Properties;

import javax.activation.DataHandler;
import javax.activation.FileDataSource;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Multipart;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.AddressException;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMultipart;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.OHSConfig;
import org.eso.ohs.core.utilities.ObjectIOException;

/**
 * @author ddorigo
 * 
 * A mail management assistant for sending generic emails from a java
 * application. It makes use of the java mail api.
 * 
 *     $Id: MailManagement.java,v 1.9 2005/05/04 09:43:33 gdonino Exp $
 */
public class MailManagement {

	private InternetAddress recipientFROM_;
	private static final String rcsid =
		"$Id: MailManagement.java,v 1.9 2005/05/04 09:43:33 gdonino Exp $";
	protected static Logger stdlog_ =
		Logger.getLogger(MailManagement.class);

	private Properties props_;
    
    
    
    public MailManagement() {
        
    }
    

	/**
	 * Method MailManagement.
	 * 
	 *   initialize    the mail with the common properties  used for sending the
	 * mails. It uses the OHSConfig class for gettin the properties. Used prop:
	 * mail.smtp.host, mail.transport.protocol, mail.host, mail.user, mail.from,
	 * mail.debug.
	 * 
	 * @param smtpHost
	 * @param transportProtocol
	 * @param host
	 * @param user
	 * @param from
	 * @param debug
	 */

	public MailManagement(
		String smtpHost,
		String transportProtocol,
		String host,
		String user,
		String from,
		String debug) {

		setMailProperties(smtpHost, transportProtocol, host, user, from, debug);

	}

	/**
	 * Method setMailProperties.
	 * set the mail configuration properties.
	 * @param smtpHost
	 * @param transportProtocol
	 * @param host
	 * @param user
	 * @param from
	 * @param debug
	 */
	public void setMailProperties(
		String smtpHost,
		String transportProtocol,
		String host,
		String user,
		String from,
		String debug) {
		//Set the host smtp address
		props_ = new Properties();
		stdlog_.debug("smtpHost " + smtpHost);
		props_.put("mail.smtp.host", smtpHost);
		props_.put("mail.transport.protocol", transportProtocol);
		props_.put("mail.host", host);
		props_.put("mail.user", user);
		props_.put("mail.from", from);
		props_.put("mail.debug", debug);

		stdlog_.debug("mail props  " + props_);
	}
    
	/**
	 * Set the SmtpHost.
	 * @param smtpHost A string containing smtpHost
	 */
    public void setSmtpHost(String smtpHost) {
        props_ = new Properties();  
        props_.put("mail.smtp.host", smtpHost);        
    }
    
    

	/**
	 * Method MailManagement.
	 * 
	 *  initialize  the mail with the common properties  used for sending the
	 * mails. It uses the OHSConfig class for gettin the properties. 
	 *  
	 *  Used prop:
	 * mail.smtp.host, 
	 * mail.transport.protocol, 
	 * mail.host, 
	 * mail.user, 
	 * mail.from,
	 * mail.debug.
	 * 
	 * @param config Object containing mail properties.
	 * @throws ObjectIOException
	 * @throws AddressException
	 * @throws AddressException
	 */

	public MailManagement(OHSConfig config) throws ObjectIOException, AddressException   {
		
	    if (config == null
	            || config.getValue("mail.smtp.host")==null
	            || config.getValue("mail.transport.protocol")==null
	            || config.getValue("mail.host")==null
	            || config.getValue("mail.user")==null
	            || config.getValue("mail.from")==null
	            || config.getValue("mail.debug")==null ) {
	      throw new ObjectIOException("Configuration Object or data are null");
	    }
			setMailProperties(
				config.getValue("mail.smtp.host"),
				config.getValue("mail.transport.protocol"),
				config.getValue("mail.host"),
				config.getValue("mail.user"),
				config.getValue("mail.from"),
				config.getValue("mail.debug"));
	
		InternetAddress[] recipientListFrom = InternetAddress.parse(config.getValue("mail.from"));
       
        if (recipientListFrom.length > 0) {
		 recipientFROM_ =  recipientListFrom[0];
		
	    }
			
	}

	
	/**
	 * Prepare the message to be sent using the following parameters
	 * @param recipients 
	 * @param recipientCC
	 * @param subject
	 * @param message
	 * @return The message to be sent.
	 * @throws MessagingException
	 */
	public synchronized Message postTextPlainMail(
		String recipients,
		String recipientCC,
		String subject,
		String message)
		throws MessagingException {
		InternetAddress[] recipientArray = InternetAddress.parse(recipients);
		InternetAddress[] recipientsCC = InternetAddress.parse(recipientCC);
		Message msg =
			postTextPlainMail(recipientArray, recipientsCC, subject, message);
		return msg;
	}

	/**
	 * It sends the message.
	 * The method can be overridden in subclasses. That helps for testing
	 * @param msg
	 * @throws MessagingException
	 */
	protected void sendMsg(Message msg) throws MessagingException {
		stdlog_.debug(msg);
		Transport.send(msg);
	}


	
	/**
	 * Prepare the message to be sent using the following parameters
	 * @param subject
	 * @param message
	 * @return the message to be sent
	 * @throws MessagingException
	 */
	public synchronized Message postTextPlainMail(
		String subject,
		String message)
		throws MessagingException {

		// create some properties and get the default Session
		Session session = Session.getDefaultInstance(props_, null);

		// create a message
		Message msg = new MimeMessage(session);
	
		msg.setRecipient(Message.RecipientType.TO, recipientFROM_);	
		sendTextMsg(subject, message, msg);
		return msg;
	}
	
	
	/**
	 * Prepare the message to be sent using the following parameters
	 * @param recipients
	 * @param recipientsCC
	 * @param subject
	 * @param message
	 * @return the message to be sent
	 * @throws MessagingException
	 */
	public synchronized Message postTextPlainMail(
		InternetAddress recipients[],
		InternetAddress recipientsCC[],
		String subject,
		String message)
		throws MessagingException {

		// create some properties and get the default Session
		Session session = Session.getDefaultInstance(props_, null);

		// create a message
		Message msg = new MimeMessage(session);

		// checks the values
		if (recipients == null || recipientsCC == null) {
			throw new MessagingException("The recipient or the cc is null");
		}

		msg.setRecipients(Message.RecipientType.TO, recipients);

		msg.setRecipients(Message.RecipientType.CC, recipientsCC);

		sendTextMsg(subject, message, msg);
		return msg;
	}

	
	/**
	 * Prepare the message to be sent using the following parameters
	 * @param recipients
	 * @param recipientsCC
	 * @param recipientsBCC
	 * @param subject
	 * @param message
	 * @return The message to be sent.
	 * @throws MessagingException
	 */
	public synchronized Message postTextPlainMail(
		InternetAddress recipients[],
		InternetAddress recipientsCC[],
		InternetAddress recipientsBCC[],
		String subject,
		String message)
		throws MessagingException {

		// create some properties and get the default Session
		Session session = Session.getDefaultInstance(props_, null);

		// create a message
		Message msg = new MimeMessage(session);

		// checks the values
		if (recipients == null || recipientsCC == null) {
			throw new MessagingException("The recipient or the cc is null");
		}

		msg.setRecipients(Message.RecipientType.TO, recipients);
		msg.setRecipients(Message.RecipientType.CC, recipientsCC);
		msg.setRecipients(Message.RecipientType.BCC, recipientsBCC);

		sendTextMsg(subject, message, msg);
		return msg;
	}
	
	private void sendTextMsg(String subject, String message, Message msg)
		throws MessagingException {
		// Setting the Subject and Content Type
		msg.setSubject(subject);
		if (message == null ) {
			message = "";
		}
		msg.setContent(message, "text/plain");
		sendMsg(msg);
	}

	/**
	 * It sends a file as attachment to the recipient.
	 * @param recipient
	 * @param subject
	 * @param message
	 * @param fileName
	 * @throws MessagingException
	 */
	public synchronized void sendFile(
		String recipient,
		String subject,
		String message,
		File fileName) throws MessagingException
		 {
		stdlog_.info(
			"\n *** recipient *** \n"
				+ recipient
				+ "\n *** subject *** \n"
				+ subject
				+ "\n *** fileName *** \n"
				+ fileName);
		Session session = Session.getDefaultInstance(props_, null);
		// create a message
		MimeMessage msg = new MimeMessage(session);

		InternetAddress[] address = null;
		try {
			address = InternetAddress.parse(recipient);
		} catch (AddressException e) {
			stdlog_.error("cannot parse address list "+ recipient);
			stdlog_.error("",e);
			throw e;
		}
		msg.setRecipients(Message.RecipientType.TO, address);
		
		msg.setSubject(subject);

		// create and fill the first message part
		MimeBodyPart mbp1 = new MimeBodyPart();
		mbp1.setText(message);

		// create the second message part
		MimeBodyPart mbp2 = new MimeBodyPart();

		// attach the file to the message
		FileDataSource fds = new FileDataSource(fileName);
		mbp2.setDataHandler(new DataHandler(fds));
		mbp2.setFileName(fds.getName());

		// create the Multipart and its parts to it
		Multipart mp = new MimeMultipart();
		mp.addBodyPart(mbp1);
		mp.addBodyPart(mbp2);

		// add the Multipart to the message
		msg.setContent(mp);

		// set the Date: header
		msg.setSentDate(new Date());

		// send the message
		Transport.send(msg);

	}
}
