/* $Id: EsoUser.java,v 1.25 2006/04/26 09:26:49 tcanavan Exp $ */

package org.eso.ohs.dfs;

import java.util.Iterator;
import java.util.TreeSet;
import java.util.Vector;

import org.apache.commons.collections.CollectionUtils;

/**
 * Eso User business object class.
 * <br>
 * 
 * @author Carlo Boarotto, 10-Mar-2000
 */
public class EsoUser extends BusinessObject {

	/**
     * 
     */
    private static final long serialVersionUID = 1L;
    private static final int MAX_PASSWD = 8;
	public static final int MAX_SURNAME = 128;
	public static final int MAX_ITEM_NAME = 20;
	/** User's id. */
	private long tableId_ = -1;

	/** User's initials (item_name in the db table). */
	private String itemName_ = "";

	/** User's surname (may contain latex formattings). */
	private String surname_ = "";

	/** User's surname (may contain latex formattings). */
	private String email_ = "";

	/** User's delatex surname (no latex formattings). */
	private String surnameDL_ = "";

	/** User's password (P2PP access password). */
	private String password_ = "";

	/** Ignored flag. */
	private boolean ignored_ = false;


	/** user affiliation list */
	TreeSet<Affiliation> affiliationSet_;

	/** user roles list */
	private Vector<String> roles_;

	//----------------------------------------
	// Constructor
	//----------------------------------------

	/**
	 * Create a EsoUser and set its id.
	 */
	public EsoUser() {
		super();
		affiliationSet_ = new TreeSet<Affiliation>();
		roles_ = new Vector<String>();
	}

	//----------------------------------------
	// Public Methods
	//----------------------------------------

	/**
	 * Get value for surname_ property
	 *
	 * @return the current value of the property
	 */
	public String getItemName() {
		return itemName_;
	}

	/**
	 * Set new value for itemName_ property
	 *
	 * @param newValue the new value for the property
	 */
	public void setItemName(String newValue) {		
		int maxLength = MAX_ITEM_NAME;
		itemName_ = getSubString(newValue, maxLength);
		firePropertyChangeEvent();
	}


	/**
	 * Get value for surname_ property
	 *
	 * @return the current value of the property
	 */
	public String getSurname() {
		return surname_;
	}

	/**
	 * Set new value for surname_ property
	 *
	 * @param newValue the new value for the property
	 */
	public void setSurname(String newValue) {
		int maxLength = MAX_SURNAME;
		surname_ = getSubString(newValue, maxLength);
		firePropertyChangeEvent();
	}

	/**
	 * Get value for surnameDL_ property
	 *
	 * @return the current value of the property
	 */
	public String getSurnameDL() {
		return surnameDL_;
	}

	/**
	 * Set new value for surnameDL_ property
	 *
	 * @param newValue the new value for the property
	 */
	public void setSurnameDL(String newValue) {
		int maxLength = MAX_SURNAME;
				surnameDL_ = getSubString(newValue, maxLength);
		firePropertyChangeEvent();
	}

	/**
	 * Get value for password_ property
	 *
	 * @return the current value of the property
	 */
	public String getPassword() {
		return password_;
	}

	/**
	 * Set new value for password_ property
	 *
	 * @param newValue the new value for the property
	 */
	public void setPassword(String newValue) {
		int maxLength = MAX_PASSWD;
		password_ = getSubString(newValue, maxLength);
			firePropertyChangeEvent();
	}

	/**
	 * Get value for ignored_ property
	 *
	 * @return the current value of the property
	 */
	public boolean getIgnored() {
		return ignored_;
	}

	/**
	 * Set new value for ignored_ property
	 *
	 * @param newValue the new value for the property
	 */
	public void setIgnored(boolean newValue) {
		ignored_ = newValue;
		firePropertyChangeEvent();
	}

	/**
	 * Get value for tableId_ property
	 *
	 * @return the current value of the property
	 */
	public long getTableId() {
		return tableId_;
	}

	/**
	 * Set new value for tableId_ property
	 *
	 * @param newValue the new value for the property
	 */
	public void setTableId(long newValue) {
		tableId_ = newValue;
		firePropertyChangeEvent();
	}

	/**
	 * Get value for affiliation property
	 *
	 * @return the current value of the property
	 */
	public String[] getRoles() {
		String[] roles = (String[]) roles_.toArray(new String[roles_.size()]);
		return roles;
	}

	/**
	 * Get value for affiliation property
	 *
	 * @return the current value of the property
	 */
	public Affiliation getAffiliation(int cycle_id) {
		Affiliation affil = null;
		Iterator<Affiliation> cursor = affiliationSet_.iterator();
		while (cursor.hasNext()) {
			Affiliation currAffil = cursor.next();
			if (currAffil.getCycleId() == cycle_id) {
				affil = currAffil;
				break;
			}
		}
		return affil;
	}

	/**
	 * Get value for affiliation property
	 *
	 * @return the current value of the property
	 */
	public void setAffiliation(Affiliation affiliation) {
		affiliationSet_.add(affiliation);
	}
	/**
	* replace value for affiliation property
	*
	* @return the current value of the property
	*/
   public void replaceAffiliation(Affiliation affiliation) {
	affiliationSet_.remove(affiliation);
	   affiliationSet_.add(affiliation);
   }
	/**
	 * Get value for affiliation property
	 *
	 * @return  the last affiliation
	 */
	public Affiliation getLastAffiliation() {
		Affiliation affil = null;
		if (affiliationSet_.size() > 0) {
		   affil = affiliationSet_.last();
		}
		return affil;
	}

	/**
	 * Get value for affiliation property
	 *
	 * @return the current value of the property
	 */
	public Affiliation[] getAffiliationArray() {
		Affiliation[] affil =
			(Affiliation[]) affiliationSet_.toArray(
				new Affiliation[affiliationSet_.size()]);
		return affil;
	}


	
	/**
	    * Get  value for property. It gets from the current affiliation
	    *
	     */
	public String getCountryCode() {
		String country = null;
			country =
				affiliationSet_.last()
					.getCountryCode();
		return country;
	}

	/**
	 * Set new value for  property
	 *
	 * @param newValue the new value for the property
	 */
	public void setAffiliationArray(Affiliation[] newValue) {
		if (newValue != null) {
			affiliationSet_ = new TreeSet<Affiliation>();
			CollectionUtils.addAll(affiliationSet_, newValue);
		}
		firePropertyChangeEvent();
	}

	/**
	 * Set new value for  property
	 *
	 * @param newValue the new value for the property
	 */
	public void setRoles(String[] newValue) {
		if (newValue != null) {
			roles_ = new Vector<String>();
			for (int i = 0; i < newValue.length; i++) {
				roles_.addElement(newValue[i]);
			}
		}
		firePropertyChangeEvent();
	}

	public Object accept(BusinessVisitor v, Object arg) {
		return null;
	}
	
	private String getSubString(String value,int maxLength) {
		if (value == null) {
				value = "";
			}
	
			if (value.length() <= maxLength) {
				maxLength = value.length();
			}
			return value.substring(0, maxLength);
	}
	/**
	 * Check if a given eso user object is equal to the current one
	 * just comparing the database ids.  
	 */
	public boolean equals(Object obj) {

		boolean ret = false;

		try {
			EsoUser user = (EsoUser) obj;

			ret = (user.getId() == this.getId());
		} catch (Exception e) {
			ret = false;
			// debug
			e.printStackTrace();
		}

		return ret;
	}
    /**
     * @see org.eso.ohs.dfs.StorableObject#postProecess()
     */
    public void postProecess() {
    }

	public void setEmail(String email_) {
		this.email_ = email_;
	}

	public String getEmail() {
		return email_;
	}

}
