package org.eso.ohs.dfs;

import java.math.BigDecimal;

public class Seeing {
    public static final float MAX_SEEING = 2.0f;

    public static final float MIN_SEEING = 0.2f;

    BigDecimal seeing_ = null;

    public Seeing(double seeing) {
        validateSeeing(seeing);
        seeing_ = new BigDecimal(seeing);
    }

    /**
     * must be between 0.2 and 2.0
     * 
     * @param seeing
     *            void
     */
    private void validateSeeing(double seeing) {
        if (seeing < MIN_SEEING || seeing > MAX_SEEING) {
            throw new IllegalArgumentException(
                    "seeing/image quality not in range 0.2 to 2.0");
        }
    }

    /**
     * must be between 0.2 and 2.0
     * 
     * @param seeing
     *            void
     */
    public static boolean isValidSeeing(String seeing) {
        boolean result = true;
        try {
            BigDecimal seeingDecimal = new BigDecimal(seeing);
            if (seeingDecimal.doubleValue() < MIN_SEEING
                    || seeingDecimal.doubleValue() > MAX_SEEING) {
                result = false;
            }
        } catch (NumberFormatException e) {
            result = false;
        }
        return result;
    }

    public Seeing(String seeing) {
        seeing_ = new BigDecimal(seeing);
        validateSeeing(seeing_.doubleValue());
    }

    public Seeing(BigDecimal seeing) {
        seeing_ = seeing;
        if (seeing == null) {
            throw new IllegalArgumentException("seeing is null");
        }
        validateSeeing(seeing_.doubleValue());
    }

    public double doubleValue() {
        return seeing_.doubleValue();
    }

    /**
     * @return BigDecimal No duplication: the member is passed by reference (it
     *         is immutable).
     */
    public BigDecimal get() {
        return seeing_;
    }
}
