package org.eso.ohs.dfs;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.StringTokenizer;

import org.eso.ohs.core.utilities.MsgManager;
import org.eso.ohs.core.utilities.TextUtils;

public class TableRuleBO {

	/**
	 * @param args
	 */
	//
	private ArrayList<HashMap<String, String>> values = new ArrayList<HashMap<String, String>>();
	private ArrayList<String> columnList = new ArrayList<String>();
    private static String ROW_START_SEP = "(";
    private static String ROW_END_SEP = ")";
    private static String  ROW_SEP = ROW_END_SEP+ROW_START_SEP;
    private static String COL_SEP = ";";
    
        
    public TableRuleBO (String[] columnList, String strTable) {
        if (columnList == null || columnList.length == 0||
                strTable == null || strTable.length() == 0) {
            throw new IllegalArgumentException ("   One parameter is null or empty: ColumnList is "+columnList+" strTable is "+strTable);
        }   
        for (int i = 0; i < columnList.length; i++){
            addKey(columnList[i]);   
        }
        
        /*
         * let's parse the result string!
         * We expect:
         * 1) the string to have the format 
         *      (row1Item1,row1Item2,...)(row2Item1,row2Item2,...) etc.
         * 2) for every row, the number of 
         *      items should match the number of items in the columList 
         *      array
         * 3) the items are positional:
         *      if columnList = {"code", "description")
         *      then we expect that the first item of every row is the code
         *      while the second is the description.
         */
        //lets normalize the string by cutting the first ( and the last )
        strTable = strTable.substring(1, strTable.length()-1);
        //lets tokenize it
        StringTokenizer rowTokenizer = new StringTokenizer(strTable, ROW_SEP);
        //lets populate every row using the tokens
        while (rowTokenizer.hasMoreTokens()){
            String rowToken = rowTokenizer.nextToken();
            StringTokenizer itemTokenizer = new StringTokenizer(rowToken, COL_SEP);
            //It's better to check with and if statement instead of using an assertion
            if (itemTokenizer.countTokens() != columnList.length) throw new IllegalArgumentException(MsgManager.errInvariant(columnList.length + " were expected instead of " + itemTokenizer.countTokens()));
            
            HashMap<String, String> map = new HashMap<String, String>();
            for (int i = 0; i < columnList.length; i++){    
                String itemToken = itemTokenizer.nextToken();
                map.put(columnList[i], itemToken);
            }   
            addValueRow(map);
        }
    }
    
    
    
    public String toString() {
        int numRows = rowCount();
        String[] rowArray = new String[numRows];
        for ( int i = 0; i < numRows; i++) {
            HashMap row = getValueRow(i);
             Collection cellValues = row.values(); 
             int numCells = cellValues.size();           
           String[] stringArray = new String[numCells];
           Iterator cursor = cellValues.iterator();
           int j = 0;
           while (cursor.hasNext()) {
               stringArray[j] = (String) cursor.next();
               j++;
           }
           
            String rowString = TextUtils.arrayToString(stringArray,COL_SEP,true);
            rowArray[i] = rowString;
        }
        String tableString = ROW_START_SEP+
        TextUtils.arrayToString(rowArray,ROW_SEP,true)+ROW_END_SEP;
        
        return tableString;
        
        
    }
    
	public TableRuleBO() {
    }
    public void addValueRow(HashMap<String, String> map){
		values.add(map);
	}
	
	public void addKey(String key){
		columnList.add(key);
	}
	
	public void setValues(ArrayList<HashMap<String, String>> list){
		values = list;
	}

	public ArrayList<String> getColumnList() {
		return columnList;
	}

	public ArrayList<HashMap<String, String>> getValues() {
		return values;
	}
	
	public HashMap getValueRow(int i){
		return values.get(i);
	}
	
	public int rowCount(){
		return values.size();
	}
	
}
