package org.eso.ohs.dfs;

import java.text.SimpleDateFormat;
import java.util.Date;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.Convert;

/**
 * 
 * 
 * <p>
 * $Id: TimeInterval.java,v 1.30 2005/09/07 07:44:15 tcanavan Exp $
 * <br>
 * @author Bob Kemp
 */
public class TimeInterval extends SimpleStorableObject {

   /** Standard log channel */
   private static Logger stdlog_ =
       Logger.getLogger( TimeInterval.class );

    /** Version id for object serialization */
    static final long serialVersionUID = -5182388435966734083L;

    /** this is the limit that must not be exceeded for both, startTime and
     *  endTime because it is stored in the DB as a signed int analogous to
     *  UNIX time_t.
     */
    public static final long MAX_TIME_T = 2147483647L;

    /** Start time of interval during which the observation could be done */
    private long startTime;

    /** End time of interval during which the observation could be done */
    private long endTime;

    /**
     * Users preference, if several time intervals are given.
     * Only for absolute time intervals.
     */
    private int preference;

    /** only for relative time intervals. */
    private int indexId;

    /** only for relative time intervals. */
    private int obId;
    
    
    
    /* Time interval type */
    private String type;
    
    public static String STANDARD_TI_TYPE = "T";
    
    public static String SIDEREAL_TI_TYPE = "S";


    /**
     * Number of MilliSeconds in a second
     * This should be extracted out into one class with all these
     * constants in them
     */

    public static final int MILLISECS_PER_SEC = 1000; 

    
    /**
     * DefaultConstructor for Time Intervals
     *
     */


    public TimeInterval() {
    }


    /**
     * Copy Constructor for TimeIntervals
     * @param ti Refernce to TimeInterval class to be copied
     */

    public TimeInterval(TimeInterval ti) {
	startTime   = ti.startTime;
	endTime     = ti.endTime;
	preference  = ti.preference;
	indexId    = ti.indexId;
	obId    = ti.obId;
	type = ti.type;
    }


    /**
     * Returns the string used to refer to this type of BusinessObject
     * in messages to the user.
     *
     * @return user-oriented name
     */
    public static String getDisplayName() {
	return "TimeInt";
    }   

    /**
     * Returns the filename suffix for files containing this type of object.
     *
     * @return suffix for filename when stored in local filesystem
     */
    public static String getSuffix() {
	return "ERROR";
    }   

    /**
     * Generate report date for Time Intervals
     * @return Report string for Time Intervals
     *
     */


    public String toReportString() {
         StringBuffer buf = new StringBuffer();
         SimpleDateFormat timeFormat = 
	     new SimpleDateFormat( Convert.ISO_DATE_FORMAT );
         Date dt;
         dt = new Date(startTime*MILLISECS_PER_SEC);
         String minTime = timeFormat.format(dt);
         dt = new Date(endTime*MILLISECS_PER_SEC);     
         String maxTime = timeFormat.format(dt);
         buf.append(minTime + " " + maxTime);
         return buf.toString();
    }

/**
     * Generate report date for Time Intervals
     * @return Report string for Time Intervals
     *
     */


    public String toReportStringSidereal() {
         StringBuffer buf = new StringBuffer();
         SimpleDateFormat timeFormat = 
	     new SimpleDateFormat( Convert.ST_TIME_FORMAT);
         Date dt;
         dt = new Date(startTime*MILLISECS_PER_SEC);
         String minTime = timeFormat.format(dt);
         dt = new Date(endTime*MILLISECS_PER_SEC);     
         String maxTime = timeFormat.format(dt);
         buf.append(minTime + " " + maxTime);
         return buf.toString();
    }



    /**
     * Returns a String describing the errors in this object's state,
     * or the empty string if it is okay.
     * 
     * @return a user-oriented description of the errors.
     */
    public String verify() {
        if(startTime < 0 || endTime < 0 || startTime > endTime){
            return "Invalid time interval";
        }else if((long)startTime > MAX_TIME_T 
              || (long)endTime   > MAX_TIME_T){
	    stdlog_.debug("CLASS TimeInterval - verify: INTERVAL BORDER(S) OUT OF RANGE"); 
	    stdlog_.debug("CLASS TimeInterval - verify: startTime = "+startTime);
	    stdlog_.debug("CLASS TimeInterval - verify: endTime = "+endTime);
            return "Interval border(s) out of Range";
        }

        return "";
    }
    /**
     * @see org.eso.ohs.dfs.StorableObject#postProecess()
     */
    public void postProecess() {
    }
    
    /**
     * Compares two <code>TimeInterval</code> objects, comparing <code>startTime</code> and <code>endTime</code> properties
     * @param other
     * @return boolean
     */
    public boolean equals(TimeInterval other) {
        
       if ( (startTime == other.startTime) && (this.endTime == other.endTime) ) {
            return (true);
        }
        else
            return(false);
        
    }

    /**
     * @return Returns the endTime.
     */
    public long getEndTime() {
        return endTime;
    }
    /**
     * @param endTime The endTime to set.
     */
    public void setEndTime(long endTime) {
        this.endTime = endTime;
    }
    /**
     * @return Returns the nextObId.
     */
    public int getObId() {
        return obId;
    }
    /**
     * @param nextObId The nextObId to set.
     */
    public void setObId(int obId) {
        this.obId = obId;
    }
    /**
     * @return Returns the preference.
     */
    public int getPreference() {
        return preference;
    }
    /**
     * @param preference The preference to set.
     */
    public void setPreference(int preference) {
        this.preference = preference;
    }
    /**
     * @return Returns the prevObId.
     */
    public int getIndexId() {
        return indexId;
    }
    /**
     * @param prevObId The prevObId to set.
     */
    public void setIndexId(int indexId) {
        this.indexId = indexId;
    }
    /**
     * @return Returns the startTime.
     */
    public long getStartTime() {
        return startTime;
    }
    /**
     * @param startTime The startTime to set.
     */
    public void setStartTime(long startTime) {
        this.startTime = startTime;
    }
    
    
	public String getType() {
		return type;
	}
	public void setType(String type) {
		this.type = type;
	}
}

