package org.eso.ohs.dfs;

/**
 * Class WindInfo keeps the wind speed information the wind coming from north
 * will have 0, coming from south (-180 or 180)
 * 
 * TODO Javadoc
 * 
 * @version $Revision: 1.4 $
 * @author ddorigo, Feb 21, 2006
 */

// $Id: WindInfo.java,v 1.4 2006/03/01 16:50:54 ddorigo Exp $
public class WindInfo {

    private int windAngle = 0;

    private boolean windOverThreshold;

    /**
     * The default maximum value of the wind (m/s)
     */
    static final int DEFAULT_WIND_THRESHOLD = 12;

    public static final int DEFAULT_WIND_ANGLE = 0;
    public static final int MAX_ANGLE = 180;
    public static final int MIN_ANGLE = -180;
    public static final int ANGLE_CONVERSION = 360;

    public static final int UPPER_AZ_LIMIT = 270; 

    public static final int LOWER_AZ_LIMIT = 90; 

    public WindInfo(int windAngle) { 
        this.windAngle = windAngle;
    }

    /**
     * @param windOverThreshold
     *            The wind spped passing the threshold
     */
    public void setWindOverThreshold(boolean windOverThreshold) {
        this.windOverThreshold = windOverThreshold;
    }

    /**
     * @return Returns the current wind speed status
     */
    public boolean isWindOverThreshold() {
        return windOverThreshold;
    }

    /**
     * 
     * @param windAngle
     *            The wind angle from the north.
     * 
     */
    public void setWindAngle(int windAngle) {
        if (validWind(windAngle)) {
            throw new IllegalArgumentException("");
        }
        this.windAngle = windAngle;
    }

    /**
     * @return Returns the current wind angle.
     */
    public int getWindAngle() {
        return windAngle;
    }

    public static boolean isValidWind(String windText) {
        boolean result = true;
        try {
            int windAngle = Integer.parseInt(windText);
            result = validWind( windAngle);
        } catch (NumberFormatException e) {
            result = false;
        }
        return result;
    }

    /**
     * @param result
     * @param windAngle
     * @return boolean
     */
    private static boolean validWind( int windAngle) {
        boolean result = true;    
        if (windAngle < MIN_ANGLE || windAngle > MAX_ANGLE ) {
            result = false;
        }
        return result;
    }

}
