package org.eso.ohs.dfs.docviewdatatrans;

import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.ohs.core.docview.datatrans.ObjAdaptor;
import org.eso.ohs.dfs.TimeInterval;

/**
 * A transducer to convert set/get of String values into set/get
 * on a long integer property.  The target property's name is given
 * to the constructor.
 *
 * <p>
 * For example, users will call it's
 * setValue/getValue methods.  Those calls will be translated into
 * calls to setX/getX on the object given to the constructor.
 *
 * TimeInterval = int ":" int
 * TimeIntervalArray = TimeInterval ";" TimeInterval
 * 
 * <p>
 * $Id: TimeIntervalsAdaptor.java,v 1.6 2006/04/26 09:26:49 tcanavan Exp $
 */

public class TimeIntervalsAdaptor extends ObjAdaptor {
	
	private static Logger stdlog_ = Logger.getLogger(TimeIntervalsAdaptor.class);


    private static final TimeInterval[] tiArray_ = new TimeInterval[0];

    /**
     * Construct a transducer.
     */
    public TimeIntervalsAdaptor() {
        super(tiArray_.getClass());
    }

    /**
     * Construct a transducer to convert set/get of String values into
     * setX/getX on an int property of <code>object</code>.
     */
    public TimeIntervalsAdaptor(Object obj, String fieldName) {
        super(tiArray_.getClass(), obj, fieldName);
    }

    /**
     * Converts data from a String to a meta-level equivalent of the
     * actual property's type, i.e. to a java.lang.Long
     * representing a long int.  Used with reflection to set
     * the new property value.
     *
     * @param value the new value of the property.
     */
    public Object toMetaLevelObject(String value) {
        TimeInterval[] tis = parseTimeIntervalArrayString(value);
        return tis;
    }

    public static TimeInterval[] parseTimeIntervalArrayString(String newValue) {
        Vector<TimeInterval> intervals = new Vector<TimeInterval>();
        String timeVal;
        int idx = -1;
        int oldIdx = 0;
        stdlog_.debug("New Value = " + newValue);
        for (;;) {
            idx = newValue.indexOf(';', oldIdx);

            if (idx < 0) {
                break;
            }

            timeVal = newValue.substring(oldIdx, idx);
            intervals.addElement(parseTimeIntervalString(timeVal));
            oldIdx = idx + 1;
        }

        TimeInterval[] result = new TimeInterval[intervals.size()];
        intervals.copyInto(result);
        for (int i = 0; i < result.length; i++)
            result[i].setPreference( i + 1 );
        return result;
    }

    public static TimeInterval parseTimeIntervalString(String timeStr) {
        int sepIdx = timeStr.indexOf(':');

        if (sepIdx < 0) {
            throw new IllegalArgumentException("bad TimeInterval value");
        }

        String startTimeStr = timeStr.substring(0, sepIdx);
        String endTimeStr = timeStr.substring(sepIdx + 1);

        TimeInterval ti = new TimeInterval();
        ti.setStartTime(new Long(startTimeStr).longValue());
        ti.setEndTime(new Long(endTimeStr).longValue());

        return ti;
    }

    /**
     * Converts data to a String from the meta-level equivalent of the
     * actual property's type, i.e. from a java.lang.Long
     * to a long int.  Used with reflection to get
     * the current property value.
     *
     * @param value	value of the property.
     */
    public String fromMetaLevelObject(Object obj) {
        if (obj == null) {
            return "";
        }

        TimeInterval[] timeInts = (TimeInterval[]) obj;

        StringBuffer sb = new StringBuffer();

        for (int j = 0; j < timeInts.length; j++) {
            TimeInterval ti = timeInts[j];

            sb.append(ti.getStartTime());
            sb.append(':');
            sb.append(ti.getEndTime());
            sb.append(';');
        }

        return sb.toString();
    }

}
