package org.eso.ohs.evm;

/**
 * Interface for registering to receive messages from the ScriptingBox
 * info/warning/error message calls.
 * Scripts calling the various message reporting and clearing methods 
 * will result in call-backs to all registered EVMMessageListeners
 *
 * @author Nick Kornweibel
 */

public interface EVMMessageListener {

    /**
     * Called when a script sends an information message
     * @param the message
     */

    public void infoMessage(String msg);

    /**
     * Called when a script sends a warning message
     * @param the message
     */

    public void warningMessage(String msg);

    /**
     * Called when a script sends an error message
     * @param the message
     */

    public void errorMessage(String msg);

    /**
     * Called when a script adds a message to the report header
     * @param the message to be added to the report header.
     */

    public void headerMessage(String msg);

    /**
     * Called when a script adds a message to the report footer
     * @param the message to be added to the report footer
     */

    public void footerMessage(String msg);

    /**
     * Indicates the scripts wants to clear previously reported errors
     */

    public void errorsWereCleared();

    /**
     * Indicates the scripts wants to clear previously reported warnings
     */

    public void warningsWereCleared();
}

