package org.eso.ohs.evm;

/**
 * The EVMScriptFinder is used by the EVMExecutor to locate scripts to be
 * executed. The interface is to be implemented by the integrating application
 * and used to locate scripts. 
 *
 * @author Nick Kornweibel
 */

import java.io.File;
import java.util.Vector;

import org.eso.ohs.dfs.CalibrationBlock;



public interface EVMScriptFinder {

    /**
     * Called to locate the scripts that are to be executed with the OB.
     * The OB passed may by used to find scripts based
     * on data in the OB - instrument scpeific script execution, for example.
     *
     * @param the OB that is to be verified by the returned scripts
     * @return Vector of File objects indicating the script files to be
     * executed
     */

    public Vector<File> findScriptsEVM(CalibrationBlock ob);
    
    /**
     * Create a list of scripts that need to be executed to 
     * calculate the exposure/ececution time.
     * @param ob
     * @return Vector
     */
    public Vector<File> findScriptsExecTime(CalibrationBlock ob);
}
