package org.eso.ohs.instruments;

import java.util.StringTokenizer;

import org.apache.log4j.Logger;

/**
 * Instancaites a paramater class that represents a rectangle. 
 * The string that contains the rectangle values is checked for its validity 
 * <p>
 * $Id: IntRectParameter.java,v 1.9 2004/04/08 12:09:52 tcanavan Exp $
 * @author Tim Canavan
 */


public class IntRectParameter extends Parameter {

    static final long serialVersionUID = -4995135451799652547L;

   /** Standard log channel */
   private static Logger stdlog_ =
       Logger.getLogger( IntRectParameter.class );


    /**
     * Constructt an IntRectParameter
     */

    public IntRectParameter(ParamMetaData pMeta) {
        super(pMeta);
    }   

    /**
     * Return tootip text containig the range values if
     * they have been entered
     * @return tooltip text
     */

    public String getToolTipText() {
        String range = getRange();
        if(range != null)
            return "Enter 4 integer values within range " + getRange();
        else 
            return "Enter 4 integer values";
    }

    public char getDBTypeCode() {
        return 'R';
    }


    /**
     * This method verifies that a string is a particular rectangle
     * @param value String repesenting a particular rectangle
     * @return OK,NaN,Number of items,Range of allows values.
     */


    public boolean verifyValue(String value) {
        Integer xmin;Integer ymin;
        Integer xmax;Integer ymax;
        Integer xlow;Integer ylow;
        Integer xhigh;Integer yhigh;
        Integer numTokens;
        if(value == null || value.length() == 0) {
	    return false;
        }
        StringTokenizer valueToken = new StringTokenizer(value);
        numTokens = new Integer(valueToken.countTokens());        
        if(numTokens.intValue() != 4)
            return false;
        String range = getRange();
        if(range == null || range.length() <= 0)
            return false;
        StringTokenizer rangeToken = new StringTokenizer(range);
        numTokens = new Integer(rangeToken.countTokens());
        if(numTokens.intValue()!=4) {
            stdlog_.debug("Too few paramaters in tsf file for paramater" + getParameterName());
            return false;
        }
        try {
            xmin = new Integer(rangeToken.nextToken());
            ymin = new Integer(rangeToken.nextToken());
            xmax = new Integer(rangeToken.nextToken());
            ymax = new Integer(rangeToken.nextToken());

            xlow = new Integer(valueToken.nextToken());
            ylow = new Integer(valueToken.nextToken());
            xhigh = new Integer(valueToken.nextToken());
            yhigh = new Integer(valueToken.nextToken());
        }
        catch(NumberFormatException e) {
            stdlog_.debug("Invald integer " + e.getMessage());
            return false;
        }

        if (xlow.intValue() > xhigh.intValue()  || ylow.intValue() > yhigh.intValue()  ||
	    xlow.intValue() < xmin.intValue() || ylow.intValue() < ymin.intValue() ||
            xhigh.intValue() > xmax.intValue() || yhigh.intValue() > ymax.intValue() ) 
	    return false;
        return true;
    }

   
}

