/* $Id: ParamMetaData.java,v 1.17 2006/04/26 09:26:49 tcanavan Exp $ */

package org.eso.ohs.instruments;

import java.io.Serializable;

public class ParamMetaData implements Serializable {

    /** Version id for object serialization */
    static final long serialVersionUID = -6384701931650553429L;

    /**
     * Paramater name.
     */
    private String parameterName_;

    /**
     * Parameter type number,integer etc..
     */
    private String type_;

    /**
     * Valid range of values for numeric paramaters
     */
    private String range_;

    /**
     * Default values to be displaed 
     */
    private String defaultValue_;
   
    /**
     * Label describing th paramater
     */
    private String label_;

    /**
     * Comment help associated with the parameter
     */
    private String miniHelp_;

    /**
     * Specify that the parameter cannot change from
     * the value it has in the config file
     */
    private boolean isFixed_;


    private boolean hide_ = false;

    private boolean hasHiddenKeyword_ = false;

    private String value_;        
     
    public ParamMetaData(String parameterName) {
       parameterName_ = parameterName;
       isFixed_ = false;
    }


   public static boolean validParamKeyword(String paramKeyword) {
	  if(paramKeyword != null && paramKeyword.length() > 0) {
	       if(paramKeyword.length() > 3) {
                    String number = paramKeyword.substring(3,paramKeyword.length());
                    try {
                        new Integer(number);
                    }
                    catch(NumberFormatException e) {                     
                         return false;
                    }
                    paramKeyword = paramKeyword.substring(0,3);
                    if(TemplateSignatureConstants.firstKeywords.containsKey(paramKeyword)) {
                        Boolean indexed = TemplateSignatureConstants.firstKeywords.get(paramKeyword);
                        return indexed.booleanValue();
		    }
	       }
	       else {
                   if(TemplateSignatureConstants.firstKeywords.containsKey(paramKeyword)) {
                        Boolean indexed = TemplateSignatureConstants.firstKeywords.get(paramKeyword);
                        return indexed.booleanValue();
                   }
	       }
          }
          return false;
     }

    public boolean isDisplay() {
        if(isHidden())
            return false;
        for(int i=0;i<ExclParameter.tgParam.length;i++) {
            if(getParameterName().equals(ExclParameter.tgParam[i])
               || isFixed()) {
                return false;
            }
        }
        return true;
    }

    // 2.9.2 Patch for LaSilla instrument
    // ohs-2_9-branch
    public boolean samePrefix(String compareString) {
        String firstKeyword = parameterName_.substring(0,parameterName_.indexOf("."));
        if(Parameter.validParamKeyword(firstKeyword)) {
             if(parameterName_.matches(compareString + "?")) {
                 return true;
             }
        }
        return false;
   }


    public void setHiddenKeyword(boolean hasHiddenKeyword) {
         hasHiddenKeyword_ = hasHiddenKeyword;
    }


    public boolean hasHiddenKeyword() {
         return hasHiddenKeyword_;
    } 


  /**
     * Paramater name
     * @return paramaterName_ name of paramater
     */


     public String getParameterName() {return parameterName_;}

     

    /**
     * Type of Paramater integer = 'I'
     * @return type_ integer = 'I'
     */

    public String getType() {return type_;}

    /**
     * Range of allowed values
     * @return range_ allowed values -99.99..+99.99
     */

    public String getRange() {return range_;}

    /**
     * Initial value of paramater
     * @return defaultValue_ the defaule setting
     */
    public String getDefaultValue() {return defaultValue_;}

    /**
     * Label describing the paramater
     * @return label_ describing the paramater_  
     */
    public String getLabel() {return label_;}

    /**
     * Minhelp comment for paramater
     * @return miniHelp_ comment for paramater_  
     */
    public String getMiniHelp() {return miniHelp_;}

    /**
     * Is fixed value for parameter
     * @return isFixed_ whether this value should be able to change  
     */
    public boolean isFixed() {return isFixed_;}

    public String getValue() {return value_;}

    public void setHidden(boolean hide) {hide_ = hide;}

    public boolean isHidden() { return hide_;}


  
    /**
     * Type of Paramater integer
     * @param type String = 'Integer'
     */
    public void setType(String type)  { type_ = type;}


    /**
     * Range of allowed values
     * @param range_ allowed values -99.99..+99.99
     */

    public void setRange(String range) {range_ = range;}
  
    /**
     * Initial value of paramater
     * @param defaultValue the defaule setting
     */
    public void setDefault(String defaultValue) {defaultValue_ = defaultValue;}

   
    public void setValue(String value) {value_ = value;}


    /**
     * Label describing the paramater
     * @return label describing the paramater_  
     */
    
    public void setLabel(String label) {label_ = label;}

    /**
     * Minhelp comment for paramater
     * @param miniHelp comment for miniHelp__  
     */
    public void setMiniHelp(String miniHelp) { miniHelp_ =  miniHelp;}

   /**
     * Set value so parameter cannot be changed after parameter 
     * has been initialized from the config file
     * @param isFixed comment for isFixed_
     */

    /**
     * Fixed parameter or parameter that can be editied
     */

    public void setFixed(boolean isFixed) {isFixed_ = isFixed;}

    public String toString() {
        StringBuffer sb = new StringBuffer();        
        sb.append("Class Parameter\n");
        sb.append("Parameter Name=" + getParameterName() + "\n");
        sb.append("Range  =" + getRange() + "\n");
        sb.append("Default Value=" + getDefaultValue() + "\n");
        sb.append("Value=" + value_ + "\n");        
        sb.append("Label =" + getLabel() + "\n");
        sb.append("Label =" + getType() + "\n");
	sb.append("Minihelp =" + getMiniHelp() + "\n");       
        return sb.toString();
    }


}
