package org.eso.ohs.persistence;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.AccountInfo;

/**
 * This class manages the support for the username - userId mapping
 * 
 * @author pnunes
 *
 */
public class AccountInfoManager {

	
	private StorageManager remoteStrMgr;	
	private String cacheDirectory;	
	private static Logger LOG = Logger.getLogger(AccountInfoManager.class);
	

	public AccountInfoManager(StorageManager remoteStrMgr, String cacheDirectory) {
		super();
		this.remoteStrMgr = remoteStrMgr;
		this.cacheDirectory = cacheDirectory;
	}


	/**
	 * Checks if in the local cache there is the info file for this user
	 * 
	 * @param username
	 * @return
	 * @throws IOException
	 */
	public boolean checkLocalAccountInfo(String username) throws IOException {
		File file = new File(cacheDirectory + "/" + username + ".map");
		if (file.exists()) {
			
			try {
				ObjectInputStream inputStream = new ObjectInputStream(
						new BufferedInputStream(new FileInputStream(file)));

				AccountInfo map = (AccountInfo) inputStream.readObject();

				if (map.getAccountId() > 0
						&& map.getUsername().equals(username)) {
					return (true);
				}

			} catch (ClassNotFoundException e) {
				LOG.error(e);
				return(false);
			}
		} 
		
		return(false);
	}

	/**
	 * gets the user id from local cache
	 * 
	 * @param username
	 * @return an AccountInfo object or null if not found
	 */
	public AccountInfo getAccountInfoFromLocalCache(String username) {
//		File cacheDir = new File(cacheDirectory);
//		cacheDir.list()
		
		File file = new File(cacheDirectory + "/" + username + ".map");
		if (file.exists()) {
			try {
				ObjectInputStream inputStream = new ObjectInputStream(
						new BufferedInputStream(new FileInputStream(file)));

				AccountInfo map = (AccountInfo) inputStream.readObject();
				return (map);

			} catch (ClassNotFoundException e) {
				LOG.debug(e);
				return (null);
			} catch (FileNotFoundException e) {
				LOG.debug(e);
				return (null);
			} catch (IOException e) {
				LOG.debug(e);
				return (null);
			}
		} 
		
		return(null);
	}

	/**
	 * Retrieves the user Id from the database
	 * 
	 * 
	 * @param username
	 * @param password
	 * @return null if the account doesn't exist 
	 * @throws ObjectIOException
	 */
	public AccountInfo getAccountInfoFromCentralDB(String username, String password) throws ObjectIOException {
		return(remoteStrMgr.getAccountInfo(username, password));
	}
	
	/**
	 * Retrieves the user Id from the database
	 * 
	 * 
	 * @param username
	 * @param password
	 * @return null if the account doesn't exist 
	 * @throws ObjectIOException
	 */
	public AccountInfo getAccountInfoFromCentralDB(int userId, String password) throws ObjectIOException {
		return(remoteStrMgr.getAccountInfo(userId, password));
	}
	
	
	/**
	 * @param username
	 * @param password
	 * @param id
	 * @throws ObjectIOException
	 */
	public void writeLocalAccountInfo(AccountInfo info) throws ObjectIOException {
		File file = new File(cacheDirectory+"/"+ info.getUsername() +".map");
    	ObjectOutputStream out;
		try {
			out = new ObjectOutputStream(
					new BufferedOutputStream(new FileOutputStream(file)));
								
			out.writeObject(info);
			out.flush();
			out.close();
		} catch (FileNotFoundException e) {
			LOG.error(e);
			throw new ObjectIOException(e);
		} catch (IOException e) {
			LOG.error(e);
			throw new ObjectIOException(e);
		}		
	}	
	

	/**
	 * @param info
	 * @throws ObjectIOException
	 */
	public void deleteLocalAccountInfo(String username) throws ObjectIOException {		
    	
			File file = new File(cacheDirectory+"/"+ username +".map");
			if (!file.delete()) {
				LOG.error("Couldn't delete file " + file.getAbsolutePath());
				throw new ObjectIOException("Couldn't delete file " + file.getAbsolutePath());
			}
				
	}
	
	
}
