package org.eso.ohs.persistence;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Vector;

import org.eso.ohs.core.utilities.PafData;
import org.eso.ohs.core.utilities.PafFile;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.dfs.Readme;
import org.eso.ohs.instruments.ReadmeAttribute;

/**
 * Class ExportReadme
 *
 * TODO Javadoc
 *
 * @version $Revision: 1.5 $
 * @author tcanavan, Nov 24, 2004
 */

// $Id: ImportReadme.java,v 1.5 2006/04/26 09:36:38 ddorigo Exp $
public class ImportReadme {

    /**
     * Attributes of business objects which fail verification
     */

    private Vector<String> invalidAttributes_ = new Vector<String>();

    private Vector<String> missingAttributes_ = new Vector<String>();

    private Vector<String> emptyDataAttributes_ = new Vector<String>();

    private Vector<String> truncDataAttributes_ = new Vector<String>();

    public void importReadme(ObservingRun obsRun, PafFile pafFile)
        throws FileNotFoundException, IOException {
        Readme readme = obsRun.getReadme();
        ReadmeAttribute[] readmeAttributes = readme.getRedmeAttributes();
        PafData[] pafData = pafFile.getIdentifierList();
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < pafData.length; i++) {
            String pafdata = pafData[i].getValue();
            String id = pafData[i].getIdentifier();
            boolean found = false;
            for (int j = 0; j < readmeAttributes.length; j++) {
                String defaultValue =
                    readmeAttributes[j]
                        .getReadmeConfig()
                        .getMetaData()
                        .getDefaultValue();
                if (readmeAttributes[j].getName().equals(id)) {
                    found = true;
                    readmeAttributes[j].setValue(pafdata);
                    String verifyStr = readmeAttributes[j].verify();
                    if (verifyStr.equals(ReadmeAttribute.VERIFY_EMPTY)) {
                        emptyDataAttributes_.addElement(
                            readmeAttributes[j].getName());
                        readmeAttributes[j].setValue(defaultValue);
                    } else if (
                        verifyStr.equals(ReadmeAttribute.VERIFY_INVALID)) {
                        invalidAttributes_.addElement(
                            readmeAttributes[j].getName());
                        readmeAttributes[j].setValue(defaultValue);
                    } else if (
                        verifyStr.equals(ReadmeAttribute.VERIFY_TRUNCATED)) {
                        truncDataAttributes_.addElement(
                            readmeAttributes[j].getName());
                        readmeAttributes[j].setValue(
                            pafdata.substring(
                                0,
                                readmeAttributes[j]
                                    .getReadmeConfig()
                                    .getMetaData()
                                    .getMaxLength()
                                    - 1));
                    }
                    break;
                }
            }
            if (!found) {
                missingAttributes_.addElement(id);
            }
        }
        obsRun.setReadme(readme);
    }

    /**
     * Imported file generated warnings
     * @param return has warnings
    */

    public boolean hasWarning() {
        if (emptyDataAttributes_.size() > 0
            || invalidAttributes_.size() > 0
            || truncDataAttributes_.size() > 0
            || missingAttributes_.size() > 0)
            return true;
        return false;
    }

    public String warningReport() {
        StringBuffer buf = new StringBuffer();
        if (missingAttributes_.size() > 0) {
            buf.append("Invalid readme keywords \n");
            for (int i = 0; i < missingAttributes_.size(); i++) {
                buf.append(
                    missingAttributes_.elementAt(i) + " invalid keyword \n");
            }
        }

        if (invalidAttributes_.size() > 0) {
            buf.append("Data Invalid for the following keywords \n");
            for (int i = 0; i < invalidAttributes_.size(); i++) {
                buf.append(invalidAttributes_.elementAt(i) + "\n");
            }
        }
        if (emptyDataAttributes_.size() > 0) {
            buf.append("Data Missing for the following keywords \n");
            for (int i = 0; i < emptyDataAttributes_.size(); i++) {
                buf.append(emptyDataAttributes_.elementAt(i) + "\n");
            }
        }

        if (truncDataAttributes_.size() > 0) {
            buf.append("Data Truncated for the  following keywords \n");
            for (int i = 0; i < truncDataAttributes_.size(); i++) {
                buf.append(truncDataAttributes_.elementAt(i) + "\n");
            }
        }
        return buf.toString();
    }

}
