package org.eso.ohs.persistence.dao;

import org.eso.ohs.dfs.account.Account;
import org.eso.ohs.dfs.account.Profile;
import org.springframework.dao.DataAccessException;

/**
 * AccountDAO interface.   
 * Class EsoUserDAO
 *
 * TODO Javadoc
 *
 * @version $Revision: 1.11 $
 * @author tcanavan, Aug 11, 2005
 */

// $Id: AccountDAO.java,v 1.11 2006/05/03 04:40:29 tcanavan Exp $
 


public interface AccountDAO {
    
    /**
     * Find an account based upon the internal database id. This just retutns 
     * the account object and not to any of its composite objects.  
     * @param id
     * @return Account business object
     * @throws DataAccessException
     */
     public Account find(final Integer id) throws DataAccessException ;
     
     /**
      * Find an account based on username. This just returns the account 
      * object and does not refer to any of its composite objects.
      * @param userName
      * @return Account business object
      * @throws DataAccessException
      */
     
     public Account find(String userName)throws DataAccessException ;
     
     
     /**
      * Find the account object and its list of associated profiles 
      * based upon account name
      * @param userName
      * @return
      * @throws DataAccessException
      */
     
     public Account findProfiles(String userName)throws DataAccessException ;
     
     /**
      * Return only the default profile for a given user
      * 
      * @param userName
      * @return
      * @throws DataAccessException
      */
     
     public Profile findDefaultProfile(String userName) throws DataAccessException;
     
     
     /**
      * Update a given account profile
      * @param accProfile
      * @throws DataAccessException
      */
     
     
     public void updateProfile(Profile accProfile) throws DataAccessException;
     
     /**
      * Create a new account with at least one profile
      * @param account
      * @throws DataAccessException
      */
     public void create(Account account) throws DataAccessException ;
     
     /**
      * Updatev the account object.
      * @param account
      * @throws DataAccessException
      */
     public void updateAccount(Account account) throws DataAccessException;
     
     /**
      * Change the password for a given account.
      * @param account
      * @throws DataAccessException
      */
     
     public void changePassword(Account account) throws DataAccessException;
     
     /**
      * Delete an account from the database
      * 
      * @param account
      * @throws DataAccessException
      */
     
     public void delete(Account account) throws DataAccessException;
     
     /**
      * Check if an account exists.
      * @param userName
      * @return
      * @throws DataAccessException
      */
     
     public boolean accountExists(String userName) throws DataAccessException;
     
     
     
     /**
      * 
      * @param userName
      * @return
      * @throws DataAccessException
      */
     
     
     public Account getAccountRoles(final String userName) throws DataAccessException;
     
} 
