/**
 * 
 */
package org.eso.ohs.persistence.dao;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.eso.ohs.core.utilities.MsgManager;
import org.eso.ohs.dfs.phase1.CycleProperties;
import org.springframework.dao.DataAccessException;
import org.springframework.orm.hibernate3.HibernateTemplate;

/**
 * @author gdonino
 */
public class CyclePropertiesDAOHibernate implements CyclePropertiesDAO {

	
	private HibernateTemplate hibernateTemplate;


	/**
	 * @return Returns the hibernateTemplate.
	 */
	public HibernateTemplate getHibernateTemplate() {
		return hibernateTemplate;
	}

	/**
	 * @param hibernateTemplate The hibernateTemplate to set.
	 */
	public void setHibernateTemplate(HibernateTemplate hibernateTemplate) {
		this.hibernateTemplate = hibernateTemplate;
	}

	/**
	 * 
	 */
	public List getCycleList() throws DataAccessException {
		
		List result = new ArrayList();
		
		
		result = hibernateTemplate.findByNamedQuery("getCycleList");
		
		return result;	
	}
	
	
	/* (non-Javadoc)
	 * @see org.eso.ohs.persistence.dao.GtoObservationsDAO#find(org.eso.ohs.dfs.phase1.GtoObservations)
	 */
	public List getCyclePropertiesName(String cycleCode) throws DataAccessException {
		if (cycleCode == null) throw new IllegalArgumentException(MsgManager.errPrecondition("cycleCode is null"));
		
		List result = new ArrayList();	
		
		Object [] paramArray = new Object[1];
		paramArray[0] = cycleCode;
			
		result = hibernateTemplate.findByNamedQuery("getAttributesNameForCycle", paramArray);
			
		return result;
	}
	
	public String getCycleProperty(String cycleCode, String propertyName ) throws DataAccessException {
		if (cycleCode == null) throw new IllegalArgumentException(MsgManager.errPrecondition("cycleCode is null"));
		if (propertyName == null) throw new IllegalArgumentException(MsgManager.errPrecondition("propertyName is null"));
		
		String result = "";	
		
		Object [] paramArray = new Object[2];
		paramArray[0] = cycleCode;
		paramArray[1] = propertyName;
			
		
		List l = hibernateTemplate.findByNamedQuery("getAttributeValue", paramArray);
		if (l.size() == 1){
			result = (String) l.get(0);
		}	
		return result;
	}
	
	public HashMap<String, String> getAttributesForCycle(String cycleCode){
		if (cycleCode == null) throw new IllegalArgumentException(MsgManager.errPrecondition("cycleCode is null"));
		
		HashMap<String, String> result = new HashMap<String, String>();
		
		List attributesNameList = getCyclePropertiesName(cycleCode); 
		Iterator i = attributesNameList.iterator();
		while (i.hasNext()){
			String propName = (String) i.next();
			String propValue = getCycleProperty(cycleCode, propName);
			result.put(propName, propValue);
		}	
		return result;
	}

	
	@SuppressWarnings("unchecked")
	public List<Integer> getNormalCyclesList() throws DataAccessException {

		List<Integer> result = new ArrayList<Integer>();
		
		result = hibernateTemplate.findByNamedQuery("getNormalCycleList");
		return result;	
	}

	@SuppressWarnings("unchecked")
	public List<Integer> getDDTCyclesList() throws DataAccessException {

		List<Integer> result = new ArrayList<Integer>();

		result = hibernateTemplate.findByNamedQuery("getDDTCycleList");
		
		
		return result;	
	}


	public void create(CycleProperties cycleProperties)
	throws DataAccessException {

      hibernateTemplate.saveOrUpdate(cycleProperties);
	}

	public HashMap<String, String> getAttributesForCycle(Integer cycleID) throws DataAccessException {
		if (cycleID == null) throw new IllegalArgumentException(MsgManager.errPrecondition("cycleID is null"));
		
		HashMap<String, String> result = new HashMap<String, String>();
		
		List attributesNameList = getCyclePropertiesName(cycleID); 
		Iterator i = attributesNameList.iterator();
		while (i.hasNext()){
			String propName = (String) i.next();
			String propValue = getCycleProperty(cycleID, propName);
			result.put(propName, propValue);
		}	
		return result;
	}
	
	public String getCycleProperty(Integer cycleID, String propertyName ) throws DataAccessException {
		if (cycleID == null) throw new IllegalArgumentException(MsgManager.errPrecondition("cycleID is null"));
		if (propertyName == null) throw new IllegalArgumentException(MsgManager.errPrecondition("propertyName is null"));
		
		String result = "";	
		
		Object [] paramArray = new Object[2];
		paramArray[0] = cycleID;
		paramArray[1] = propertyName;
			
		
		List l = hibernateTemplate.findByNamedQuery("getAttributeValue", paramArray);
		if (l.size() == 1){
			result = (String) l.get(0);
		}	
		return result;
	}

	public List getCyclePropertiesName(Integer cycleID) throws DataAccessException {
		if (cycleID == null) throw new IllegalArgumentException(MsgManager.errPrecondition("cycleID is null"));
		List result = new ArrayList();	
		
		Object [] paramArray = new Object[1];
		paramArray[0] = cycleID;
			
		result = hibernateTemplate.findByNamedQuery("getAttributesNameForCycle", paramArray);
			
		return result;
	}

	public Integer getNextIDForPeriod(String stringPeriod) {
		if (stringPeriod == null) throw new IllegalArgumentException(MsgManager.errPrecondition("stringPeriod is null"));
		if (stringPeriod.length() == 0) throw new IllegalArgumentException(MsgManager.errPrecondition("stringPeriod is empty"));
		
		Integer result = null;	
		Integer fromID = Integer.valueOf(stringPeriod + "00");
		Integer toID = new Integer(fromID.intValue() + 100 - 1);
		Object [] paramArray = new Object[2];
		paramArray[0] = fromID;
		paramArray[1] = toID;
			
		List l = hibernateTemplate.findByNamedQuery("getMaxIDforPeriod", paramArray);
		if (l.get(0) != null){
			result = new Integer(((Integer) l.get(0)).intValue() + 1);
		}else
			result = fromID;
		
		return result;
	}

	
}
