package org.eso.ohs.persistence.dao;

import java.util.List;

import org.eso.ohs.dfs.phase1.GtoManagers;
import org.eso.ohs.dfs.phase1.InstrumentConfig;
import org.springframework.dao.DataAccessException;

/**
 * @author fsogni
 * This interface is the main interface to the gto_managers table in the opc70 database
 * This table represents the db interface for the gto_managers business object.  
 */


public interface GtoManagersDAO {
	
	/**
	 * Find a gto manager based upon
	 * the Id
	 * @param gtoObservation
	 * @return
	 * @throws DataAccessException
	 */
     
	public GtoManagers find(GtoManagers gtoManager) throws DataAccessException;
	
	
	/**
	 * Find a gto manager based upon
	 * the predefined attributes. Returns these attributes and the database id.
	 * @param gtoObservation
	 * @return
	 * @throws DataAccessException
	 */
     
	public GtoManagers findByProperties(GtoManagers gtoManager) throws DataAccessException;
	
	/**
	 * Find a gto manager based upon the user id.
	 * @param gtoObservation
	 * @return
	 * @throws DataAccessException
	 */
     
	public GtoManagers findByUserId(String userId, String cycleId) throws DataAccessException;
	

	

	/**
     * Check if a gto manager exists.
     * @param userName
     * @return
     * @throws DataAccessException
     */
    
    public boolean gtoManagerExists(String userName, String cycleId) throws DataAccessException;
	
	
	/**
	 * Find a gto observation based upon a given instrument configuration. 
	 * Returns an observation object
	 * @param gtoManager
	 * @return
	 * @throws DataAccessException
	 */
     
	public List find(InstrumentConfig insConfig) throws DataAccessException;
	
	/**
	 * Return all the GTO managers in the table
	 * @param String defining order by clause
	 * @throws DataAccessException
	 */
	public List getAll() throws DataAccessException;
	
	/**
	 * Create a gto manager
	 * @param gtoManager
	 * @throws DataAccessException
	 */
	
    public void create(GtoManagers gtoManager) throws DataAccessException;
    
    /**
     * Update a gto manager. This expects the database
     * id to be initialised. 
     * @param gtoManager
     * @throws DataAccessException
     */
    
    public void update(GtoManagers gtoManager) throws DataAccessException;
    
    /**
     * Delete a gto manager from the database. Only possible 
     * if it does not refer to a gto conflict (entry in gto_conflicts) 
     * @param gtoManager
     * @throws DataAccessException
     */
    
    public void delete(GtoManagers gtoManager) throws DataAccessException;
	
}
