/* $Id: DbaseHandlerCycle.java,v 1.21.4.1 2007/10/24 12:17:01 pnunes Exp $ */

package org.eso.ohs.persistence.dbase.phase1;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Set;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.TextUtils;
import org.eso.ohs.dfs.Cycle;
import org.eso.ohs.dfs.StorableObject;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.DBdataHelper;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.StorageManager.UnpackedStorableObject;
import org.eso.ohs.persistence.dbase.DbaseIOHandler;

/**
 * Sql handler to retrieve, to get and to update Cycle objects on the database.
 * <br>
 * 
 * @see Cycle
 * @author Carlo Boarotto, ESO 21-Mar-2000
 */
public class DbaseHandlerCycle extends DbaseIOHandler {

    public static int PROPOSAL_SUBMISSION_PHASE = 1;

    /**
     * Managed class
     * 
     * @return Cycle class
     */
    public Class getManagedClass() {
        return Cycle.class;
    }

    /** Standard log channel */
    private static Logger stdlog_ = Logger.getLogger(DbaseHandlerCycle.class);

    // private Hashtable telescopesInstruments_ = null;

    // cycle properties from the cycles table
    public static final String CYCLE_LABEL = "label";

    public static final String CYCLE_ID = "id";

    public static final String PERIOD = "period";

    public static final String DESCRIPTION = "description";

    public static final String STEP_ID = "step_id";

    // cycles properties from the cycle_prop table
    public static final String CYCLE_PROP_ID = "cycle_id";

    public static final String CYCLE_PROP_KEY = "cycle_key";

    public static final String CYCLE_PROP_VALUES = "cycle_values";

    public static final String COUNTRIES = "countries";

    public static final String RUN_CODES = "runCodeValues";

    public static final String ALT_RUN_CODES = "altRunCodeValues";

    public static final String PERIOD_LIST = "periodList";

    public static final String OBS_MODE_VALUES = "obsModeValues";

    public static final String MOON_LIST = "moonList";

    public static final String SEEING_VALUES = "seeingValues";

    public static final String PROPRIETARY_TIME = "proprietaryTimes";

    public static final String SKY_TRANSPARENCY_VALUES = "skyTransparencyValues";

    public static final String SUB_CATEGORY_CODES = "subCategoryCodes";

    public static final String EQUINOX = "equinox";

    public static final String VALID_MONTHS = "validMonths";

    public static final String SPECIAL_INSTRUMENTS = "specialInstruments";

    public static final String INTERF_INSTRUMENTS = "interf_instruments";

    public static final String BASELINES_ROOTNAME = "baselines.";

    public static final String LAMBDACOUPLES_ROOTNAME = "lambdacouples.";

    public static final String MOON_REQUIREMENTS = "tableMoonRequirements";

    public static final String TRANSPARENCY_REQUIREMENTS = "tableTransparencyRequirements";

    public static final String NATIONS = "tableNations";

    // Properties for the instrument/telescopes
    public static final String OFFERED_ID = "cycle_id";

    public static final String OFFERED_INSTRUMENT = "instrument_code";

    public static final String OFFERED_TELESCOPE = "telescope_code";

    public static final String INSTRUMENTS_CODE = "code";

    public static final String INSTRUMENTS_TEL = "tel";

    public static final String INSTRUMENTS_ID = "id";

    public static final String TELESCOPES_CODE = "code";

    public static final String TELESCOPES_ITEM_NAME = "item_name";

    // private String[] propertyList_ =
    // {COUNTRIES,RUN_CODES,ALT_RUN_CODES,OBS_MODE_VALUES,MOON_LIST,
    // SEEING_VALUES,SKY_TRANSPARENCY_VALUES,SUB_CATEGORY_CODES,EQUINOX,SPECIAL_INSTRUMENTS,INTERF_INSTRUMENTS};

    // cycles properties from the opc

    /** Sql select statement to retrieve cycle properties from the cycle table */

    protected static String cycle_fields = null;

    protected static String cycle_select_sql = null;

    /** Sql select statement to retrieve Cycle entries. */
    protected static String cycle_sql = null;

    /**
     * Sql select statement to retrieve cycle properties from the cycle_prop
     * table
     */

    protected static String cycle_properties_fields = null;

    protected static String cycle_properties_select_sql = null;

    /** Sql select statement to retrieve Cycle entries. */
    protected static String cycle_properties_sql = null;

    /**
     * Private constructor: it is a singleton class.
     */
    public DbaseHandlerCycle() {
        super();
        cycle_fields = Phase1SelectStmt.cyclesTable.get() + "." + CYCLE_LABEL
                + "," + Phase1SelectStmt.cyclesTable.get() + "." + CYCLE_ID
                + "," + Phase1SelectStmt.cyclesTable.get() + "." + PERIOD + ","
                + Phase1SelectStmt.cyclesTable.get() + "." + DESCRIPTION + ","
                + Phase1SelectStmt.cyclesTable.get() + "." + STEP_ID;

        cycle_select_sql = "SELECT  " + cycle_fields;
        cycle_sql = cycle_select_sql + " FROM "
                + Phase1SelectStmt.cyclesTable.get();
        cycle_properties_fields = Phase1SelectStmt.cyclePropertiesTable.get()
                + "." + CYCLE_PROP_ID + ","
                + Phase1SelectStmt.cyclePropertiesTable.get() + "."
                + CYCLE_PROP_KEY + ","
                + Phase1SelectStmt.cyclePropertiesTable.get() + "."
                + CYCLE_PROP_VALUES;

        cycle_properties_select_sql = "SELECT  " + cycle_properties_fields;
        cycle_properties_sql = cycle_properties_select_sql + " FROM "
                + Phase1SelectStmt.cyclePropertiesTable.get();
    }

    /**
     * Read opcReferee data
     * 
     * @param sql
     *            statement
     * @param Object
     *            id to be read
     * @exception SQLException
     *                an error occurred during the dbase operation
     * @exception ObjectNotFoundException
     *                the requested object was not found in the database
     * @exception ObjectIOException
     *                IO Eror occured while reading from the database
     */

    public UnpackedStorableObject read(Statement stmt, long objId)
            throws ObjectIOException, ObjectNotFoundException, SQLException {

        Cycle cycle = (Cycle) newInstance();
        // long cycleId = Config.getCfg().uniqueToTableId(objId);
        long cycleId = objId;

        // Retrieve cycle properties from table cycle
        String cyclesSql = cycle_sql + " WHERE " + CYCLE_ID + " = " + cycleId;
        ResultSet cyclesSqlRs = getResultSet(stmt, cyclesSql);
        cyclesSqlRs.next();
        cycle.setId(Config.getCfg().tableToUniqueId(objId, Cycle.class));
        cycle.setCycleId(cyclesSqlRs.getInt(CYCLE_ID));
        setCyclesFields(cycle, cyclesSqlRs);
        cyclesSqlRs.close();

        retrieveCycleProperties(stmt, cycle);
        retrieveTelescopesInstruments(stmt, cycle);
        UnpackedStorableObject uso = new UnpackedStorableObject(cycle);
        return uso;
    }

    /**
     * @throws ObjectNotFoundException
     * @throws SQLException
     * @throws ObjectIOException
     * 
     */
    private void retrieveCycleProperties(Statement stmt, Cycle cycle)
            throws SQLException, ObjectNotFoundException, ObjectIOException {

        String cyclePropertiesSql = cycle_properties_sql + " WHERE "
                + CYCLE_PROP_ID + " = " + cycle.getCycleId();
        ResultSet cyclePropRs = getResultSet(stmt, cyclePropertiesSql);
        Hashtable<String, String> cyclePropList = new Hashtable<String, String>();
        while (cyclePropRs.next()) {

            String key = cyclePropRs.getString(CYCLE_PROP_KEY);
            String values = cyclePropRs.getString(CYCLE_PROP_VALUES);
            if (key.startsWith(BASELINES_ROOTNAME)) {
                String baselineInstr = key.substring(BASELINES_ROOTNAME
                        .length());
                String[] valueList = TextUtils.stringToStringArray(values, " ",
                        true);
                cycle.setBaselines(baselineInstr, valueList);
            } else if (key.startsWith(LAMBDACOUPLES_ROOTNAME)) {
                String lambdaInstr = key.substring(LAMBDACOUPLES_ROOTNAME
                        .length());
                String[] valueList = TextUtils.stringToStringArray(values, " ",
                        true);
                cycle.setLambdacouples(lambdaInstr, valueList);
            } else {
                cyclePropList.put(key, values);
            }

        }
        cyclePropRs.close();
        String[] valueList = TextUtils.stringToStringArray(
                cyclePropList.get(ALT_RUN_CODES), " ", true);
        cycle.setAltcodeValues(valueList);
        valueList = TextUtils.stringToStringArray(cyclePropList
                .get(COUNTRIES), " ", true);
        cycle.setCountries(valueList);
        valueList = TextUtils.stringToStringArray(cyclePropList
                .get(RUN_CODES), " ", true);
        cycle.setRuncodeValues(valueList);
        Integer[] valueIntList = TextUtils.stringToIntObjArray(
                cyclePropList.get(PERIOD_LIST), " ");
        cycle.setPeriodList(valueIntList);
        valueList = TextUtils.stringToStringArray(cyclePropList
                .get(OBS_MODE_VALUES), " ", true);
        cycle.setObsModeValues(valueList);
        valueList = TextUtils.stringToStringArray(cyclePropList
                .get(MOON_LIST), " ", true);
        cycle.setMoonList(valueList);
        valueList = TextUtils.stringToStringArray(cyclePropList
                .get(SEEING_VALUES), " ", true);
        cycle.setSeeingValues(valueList);
        valueList = TextUtils.stringToStringArray(cyclePropList
                .get(PROPRIETARY_TIME), " ", true);
        cycle.setProprietaryTimes(valueList);
        valueList = TextUtils.stringToStringArray(cyclePropList
                .get(SKY_TRANSPARENCY_VALUES), " ", true);
        cycle.setSkyTransparencyValues(valueList);
        valueList = TextUtils.stringToStringArray(cyclePropList
                .get(SUB_CATEGORY_CODES), " ", true);
        cycle.setSubCategoryCodes(valueList);
        valueList = TextUtils.stringToStringArray(cyclePropList
                .get(EQUINOX), " ", true);
        cycle.setEquinox(valueList);
        valueList = TextUtils.stringToStringArray(cyclePropList
                .get(VALID_MONTHS), " ", true);
        cycle.setValidMonths(valueList);
        valueList = TextUtils.stringToStringArray(cyclePropList
                .get(SPECIAL_INSTRUMENTS), " ", true);
        cycle.setSpecialInstruments(valueList);
        valueList = TextUtils.stringToStringArray(cyclePropList
                .get(INTERF_INSTRUMENTS), " ", true);
        cycle.setInterferometricInstruments(valueList);

        cycle
                .setMoonRequirements(cyclePropList
                        .get(MOON_REQUIREMENTS));
        cycle.setTransparencyRequirements(cyclePropList
                .get(TRANSPARENCY_REQUIREMENTS));
        cycle.setNations(cyclePropList.get(NATIONS));

    }

    private void retrieveTelescopesInstruments(Statement stmt, Cycle cycle)
            throws SQLException, ObjectNotFoundException {

        String offeredTable = Phase1SelectStmt.offeredTable.get();
        String instrumentColumn = offeredTable + "." + OFFERED_INSTRUMENT;
        String telescopeColumn = offeredTable + "." + OFFERED_TELESCOPE;

        Hashtable<String, Vector<String>> telescopesInstruments = new Hashtable<String, Vector<String>>();
        String instrumentsTelescopesSql = "SELECT " + instrumentColumn + ","
                + telescopeColumn + " FROM " + offeredTable + " WHERE "
                + offeredTable + "." + OFFERED_ID + " = " + cycle.getCycleId();
        // + " ORDER BY " + telescopeColumn;

        stdlog_.debug("Telescopes: " + instrumentsTelescopesSql);

        ResultSet instTelescopesRs = getResultSet(stmt,
                instrumentsTelescopesSql);
        while (instTelescopesRs.next()) {

            String telescope = instTelescopesRs
                    .getString(OFFERED_TELESCOPE)
                    .trim();
            String instrument = instTelescopesRs
                    .getString(OFFERED_INSTRUMENT)
                    .trim();
            stdlog_.debug("Telescope in the vector: " + telescope);
            stdlog_.debug("Inserting the instruments in the vector: "
                    + instrument);
            Vector<String> instrumentVector = retrieveInstrumentVector(
                    telescopesInstruments, telescope);
            instrumentVector.add(instrument);
        }

        Hashtable<String, String[]> telInstrArrayTable = new Hashtable<String, String[]>();
        Set<String> telSet = telescopesInstruments.keySet();
        Iterator<String> cursorKey = telSet.iterator();
        String[] telList = new String[telSet.size()];
        int j = 0;
        while (cursorKey.hasNext()) {
            telList[j] = cursorKey.next();
            j++;
        }
        for (int i = 0; i < telList.length; i++) {
            Vector instVector = telescopesInstruments.get(telList[i]);
            String[] instArray = (String[]) instVector
                    .toArray(new String[instVector.size()]);
            telInstrArrayTable.put(telList[i], instArray);
        }
        cycle.setTelescopesInstrumentsTable(telInstrArrayTable);
        instTelescopesRs.close();
    }

    /**
     * @param telescopesInstruments
     * @param telescope
     * @return Vector
     */
    private Vector<String> retrieveInstrumentVector(Hashtable<String, Vector<String>> telescopesInstruments,
            String telescope) {
        Vector<String> instrumentVector = null;
        if (telescopesInstruments.containsKey(telescope)) {
            instrumentVector = telescopesInstruments.get(telescope);
        } else {
            instrumentVector = new Vector<String>();
            telescopesInstruments.put(telescope, instrumentVector);
        }
        return instrumentVector;
    }

    /**
     * @param stmt
     * @param cycleSql
     * @return
     * @throws SQLException
     * @throws ObjectNotFoundException
     */
    private ResultSet getResultSet(Statement stmt, String cycleSql)
            throws SQLException, ObjectNotFoundException {
        stdlog_.debug(cycleSql);

        boolean isThereAnObject = stmt.execute(cycleSql);

        if (isThereAnObject) {
            ResultSet rs = stmt.executeQuery(cycleSql);
            return rs;
        } else {
            throw new ObjectNotFoundException("Object not found in database");
        }

    }

    /**
     * Read data
     * 
     * @param sql
     *            statement
     * @param Object
     *            id to be read
     * @exception SQLException
     *                an error occurred during the dbase operation
     * @exception ObjectNotFoundException
     *                the requested object was not found in the database
     * @exception ObjectIOException
     *                IO Eror occured while reading from the database
     */

    public UnpackedStorableObject[] read(Statement stmt, String whereClause,
            String otherTables)
            throws ObjectIOException, ObjectNotFoundException, SQLException {

        UnpackedStorableObject[] cycle_array = null;
        Cycle cycle = null;
        if (otherTables == null) {
            otherTables = "";
        }
        String sql = cycle_sql + otherTables + " WHERE " + whereClause;

        stdlog_.info(sql);
        ResultSet rs = stmt.executeQuery(sql);

        try {
            Vector<Cycle> cycle_vector = new Vector<Cycle>();

            while (rs.next()) {

                cycle = (Cycle) newInstance();
                // New element
                cycle_vector.addElement(cycle);

                // Set only when attaching the object as a new
                // element
                setCyclesFields(cycle, rs);
                cycle.setId(Config.getCfg().tableToUniqueId(
                        rs.getInt(CYCLE_ID), Cycle.class));
                cycle.setCycleId(rs.getInt(CYCLE_ID));
            }

            rs.close();

            cycle_array = new UnpackedStorableObject[cycle_vector.size()];
            // add the affiliation to the user and insert them in array
            for (int i = 0; i < cycle_vector.size(); i++) {
                cycle = cycle_vector.elementAt(i);
                retrieveCycleProperties(stmt, cycle);
                retrieveTelescopesInstruments(stmt, cycle);
                cycle_array[i] = new UnpackedStorableObject(cycle);
            }
        } catch (SQLException e) {
            e.printStackTrace();
        }

        return cycle_array;
    }

    /**
     * Throws an ObjectIOException -- the phase 1 database cannot be changed
     * from within P2PP.
     * 
     * @param stmt
     *            JDBC Statement for contacting the dbase
     * @param id
     *            per-table id of the object to be written.
     * @param obj
     *            an array: the first element is the BO to be stored.
     * 
     * @exception ObjectIOException
     *                operation is not permitted exception
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public void write(Statement stmt, long id, UnpackedStorableObject obj)
            throws ObjectIOException {
        throw new ObjectIOException("no such method implemented");
    }

    /**
     * Throws an ObjectIOException -- the phase 1 database cannot be changed
     * from within P2PP.
     * 
     * @param stmt
     *            JDBC Statement for contacting the dbase
     * @param id
     *            per-table id of the object to be written.
     * @param obj
     *            an array: the first element is the BO to be stored.
     * 
     * @exception ObjectIOException
     *                operation is not permitted exception
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public void update(Statement stmt, long id, UnpackedStorableObject obj)
            throws ObjectIOException {
    }

    // Update both rated and comments tables.
    public void update(Statement stmt, StorableObject obj)
            throws ObjectIOException, SQLException {

        Cycle cycle = (Cycle) obj;

        // long id = Config.getCfg().uniqueToTableId(cycle.getId());

        updateCycleFields(stmt, cycle);
        updateCycleProperties(stmt, cycle);
        updateTelescopesInstruments(stmt, cycle);
    }

    /**
     * @param stmt
     * @param cycle
     * @throws SQLException
     */
    private void updateCycleFields(Statement stmt, Cycle cycle)
            throws SQLException {
        StringBuffer sb = new StringBuffer();
        sb.append(CYCLE_LABEL);
        sb.append(" = \'");
        sb.append(cycle.getLabel());
        sb.append("\',");
        sb.append(PERIOD);
        sb.append(" = ");
        sb.append(cycle.getPeriod());
        sb.append(",");
        sb.append(DESCRIPTION);
        sb.append(" = \'");
        sb.append(cycle.getDescription());
        sb.append("\',");
        sb.append(STEP_ID);
        sb.append(" = ");
        sb.append(cycle.getStepId());

        String sql = Phase1SelectStmt.beginTransaction + " \n " + "UPDATE  "
                + Phase1SelectStmt.cyclesTable.get() + " SET " + sb.toString()
                + " WHERE " + CYCLE_ID + " = "
                + Config.getCfg().uniqueToTableId(cycle.getId());

        stdlog_.debug(sql);
        stmt.executeUpdate(sql);
    }

    private void updateCycleProperties(Statement stmt, Cycle cycle)
            throws SQLException {

        long cycleId = cycle.getCycleId();

        StringBuffer sb = new StringBuffer();
        sb.append(Phase1SelectStmt.beginTransaction);
        sb.append("\n");

        String nationsTable = cycle.getNations().toString();
        updateCyclePropStmt(cycleId, sb, NATIONS, nationsTable);

        String moonTable = cycle.getMoonrequirements().toString();
        updateCyclePropStmt(cycleId, sb, MOON_REQUIREMENTS, moonTable);

        String skyTable = cycle.getTransparencyRequirements().toString();
        updateCyclePropStmt(cycleId, sb, TRANSPARENCY_REQUIREMENTS, skyTable);

        String[] baselineInstrList = cycle.getBaselineInstruments();
        for (int i = 0; i < baselineInstrList.length; i++) {
            String currentInstr = baselineInstrList[i];
            String baselineValues = TextUtils.arrayToString(cycle
                    .getBaselines(currentInstr), " ", true);
            updateCyclePropStmt(cycleId, sb, BASELINES_ROOTNAME + currentInstr,
                    baselineValues);

        }
        String[] lambdaInstrList = cycle.getLambdacouplesInstruments();
        for (int i = 0; i < lambdaInstrList.length; i++) {
            String currentInstr = lambdaInstrList[i];
            String lambdaValues = TextUtils.arrayToString(cycle
                    .getLambdacouples(currentInstr), " ", true);
            updateCyclePropStmt(cycleId, sb, LAMBDACOUPLES_ROOTNAME
                    + currentInstr, lambdaValues);

        }
        String countryValues = TextUtils.arrayToString(cycle.getCountries(),
                " ", true);
        updateCyclePropStmt(cycleId, sb, COUNTRIES, countryValues);

        String runValues = TextUtils.arrayToString(cycle.getRuncodeValues(),
                " ", true);
        updateCyclePropStmt(cycleId, sb, RUN_CODES, runValues);

        String runAltValues = TextUtils.arrayToString(cycle.getAltcodeValues(),
                " ", true);
        updateCyclePropStmt(cycleId, sb, ALT_RUN_CODES, runAltValues);

        String modeValues = TextUtils.arrayToString(cycle.getObsModeValues(),
                " ", true);
        updateCyclePropStmt(cycleId, sb, OBS_MODE_VALUES, modeValues);

        String periodValues = TextUtils.arrayToString(cycle
                .getPeriodListString(), " ", true);
        updateCyclePropStmt(cycleId, sb, PERIOD_LIST, periodValues);

        String moonValues = TextUtils.arrayToString(cycle.getMoonList(), " ",
                true);
        updateCyclePropStmt(cycleId, sb, MOON_LIST, moonValues);

        String seeingValues = TextUtils.arrayToString(cycle.getSeeingValues(),
                " ", true);
        updateCyclePropStmt(cycleId, sb, SEEING_VALUES, seeingValues);

        String skyValues = TextUtils.arrayToString(cycle
                .getSkyTransparencyValues(), " ", true);
        updateCyclePropStmt(cycleId, sb, SKY_TRANSPARENCY_VALUES, skyValues);

        String categoryValues = TextUtils.arrayToString(cycle
                .getSubCategoryCodes(), " ", true);
        updateCyclePropStmt(cycleId, sb, SUB_CATEGORY_CODES, categoryValues);

        String equinoxValues = TextUtils.arrayToString(cycle.getEquinox(), " ",
                true);
        updateCyclePropStmt(cycleId, sb, EQUINOX, equinoxValues);

        String monthsValues = TextUtils.arrayToString(cycle.getValidMonths(),
                " ", true);
        updateCyclePropStmt(cycleId, sb, VALID_MONTHS, monthsValues);

        String specialInstValues = TextUtils.arrayToString(cycle
                .getSpecialInstruments(), " ", true);
        updateCyclePropStmt(cycleId, sb, SPECIAL_INSTRUMENTS, specialInstValues);

        String intInstrValues = TextUtils.arrayToString(cycle
                .getInterferometricInstruments(), " ", true);
        updateCyclePropStmt(cycleId, sb, INTERF_INSTRUMENTS, intInstrValues);

        String propTimeValues = TextUtils.arrayToString(cycle
                .getProprietaryTimes(), " ", true);
        updateCyclePropStmt(cycleId, sb, PROPRIETARY_TIME, propTimeValues);

        String sql = sb.toString();
        stdlog_.debug(sql);
        stmt.executeUpdate(sql);

    }

    /**
     * @param cycleId
     * @param sb
     * @param countryValues
     *            void
     */
    private void updateCyclePropStmt(long cycleId, StringBuffer sb, String key,
            String values) {
        sb.append("\n UPDATE ");
        sb.append(Phase1SelectStmt.cyclePropertiesTable.get());
        sb.append(" SET ");
        sb.append(CYCLE_PROP_VALUES);
        sb.append(" = '");
        sb.append(values);
        sb.append("' WHERE ");
        sb.append(CYCLE_PROP_ID);
        sb.append(" = ");
        sb.append(cycleId);
        sb.append(" AND ");
        sb.append(CYCLE_PROP_KEY);
        sb.append(" = '");
        sb.append(key);
        sb.append("'");
        sb.append("\n");
    }

    /**
     * @param stmt
     * @param cycle
     * @throws SQLException
     * @throws ObjectIOException
     */
    private void updateTelescopesInstruments(Statement stmt, Cycle cycle)
            throws SQLException, ObjectIOException {

        // get the DB telInstruments couples
        Vector<TelInstCouple> telInstrCoupleDBlist = retrievTelInstr(stmt, cycle);
        Vector<TelInstCouple> telInstrCoupleCyclelist = new Vector<TelInstCouple>();
        String[] telArray = cycle.getAllTelescopes();
        for (int i = 0; i < telArray.length; i++) {
            String currTel = telArray[i];
            String[] instList = cycle.getInstruments(currTel);
            for (int j = 0; j < instList.length; j++) {
                String currInstr = instList[j];
                TelInstCouple telInst = new TelInstCouple(currTel, currInstr);
                // build a cycle telInstr couple
                telInstrCoupleCyclelist.add(telInst);
            }
        }

        StringBuffer query = new StringBuffer();
        String[] instrArray = cycle.getAllInstruments();
        boolean runQuery = false;

        // check if new instruments to be added
        for (int i = 0; i < instrArray.length; i++) {
            String currCycleObjInstr = instrArray[i];

            // check if instrument of cycle obj is in offered table (DB)
            TelInstCouple telInstrDB = getInstrTelCouple(telInstrCoupleDBlist,
                    currCycleObjInstr);
            if (telInstrDB != null) { // instrument in DB, either update the
                                        // offered table, either don't do
                                        // anything
                TelInstCouple telInstrCycleObj = getInstrTelCouple(
                        telInstrCoupleCyclelist, currCycleObjInstr);

                // the telescope is different. Let's update the offered table
                if (!telInstrCycleObj.getTel().equals(telInstrDB.getTel())) {
                    runQuery = true;
                    query.append("\nUPDATE ");
                    query.append(Phase1SelectStmt.offeredTable.get());
                    query.append(" SET ");
                    query.append(Phase1SelectStmt.offeredTable.get());
                    query.append(".");
                    query.append(OFFERED_TELESCOPE);
                    query.append("='");
                    query.append(telInstrCycleObj.getTel());
                    query.append("'");
                    query.append("\n WHERE ");
                    query.append(Phase1SelectStmt.offeredTable.get());
                    query.append(".");
                    query.append(OFFERED_ID);
                    query.append("=");
                    query.append(cycle.getCycleId());
                    query.append(" AND ");
                    query.append(Phase1SelectStmt.offeredTable.get());
                    query.append(".");
                    query.append(OFFERED_INSTRUMENT);
                    query.append("='");
                    query.append(telInstrCycleObj.getInst());
                    query.append("' \n ");
                }

            } else { // instrument not in offered table: let's add it
                TelInstCouple telInstrCycleObj = getInstrTelCouple(
                        telInstrCoupleCyclelist, currCycleObjInstr);
                runQuery = true;
                query.append("\nINSERT INTO ");
                query.append(Phase1SelectStmt.offeredTable.get());
                query.append("(");
                query.append(OFFERED_ID);
                query.append(",");
                query.append(OFFERED_TELESCOPE);
                query.append(",");
                query.append(OFFERED_INSTRUMENT);
                query.append(")");
                query.append("\n VALUES(");
                query.append(cycle.getCycleId());
                query.append(",'");
                query.append(telInstrCycleObj.getTel());
                query.append("','");
                query.append(telInstrCycleObj.getInst());
                query.append("') \n ");
            }
        }

        // instrument to be deleted if instrument in DB not in cycle obj
        for (int i = 0; i < telInstrCoupleDBlist.size(); i++) {
            TelInstCouple currTelInstrDB = telInstrCoupleDBlist
                    .get(i);

            // return null if instrument NOT in cycle obj
            TelInstCouple telInstrCycleObj = getInstrTelCouple(
                    telInstrCoupleCyclelist, currTelInstrDB.getInst());

            // cycle doesn't contain the instr in DB. check if it is
            // possible to delete
            if (telInstrCycleObj == null) {
                if (isTelInstrRemovable(stmt, cycle, currTelInstrDB.getInst(),
                        currTelInstrDB.getTel())) {
                    runQuery = true;
                    query.append("\nDELETE ");
                    query.append(Phase1SelectStmt.offeredTable.get());
                    query.append(" \n WHERE ").append(
                            Phase1SelectStmt.offeredTable.get()).append(
                            ".cycle_id =").append(cycle.getCycleId()).append(
                            "\nAND ").append(
                            Phase1SelectStmt.offeredTable.get()).append(
                            ".instrument_code ='").append(
                            currTelInstrDB.getInst()).append("'\nAND ").append(
                            Phase1SelectStmt.offeredTable.get()).append(
                            ".telescope_code ='").append(
                            currTelInstrDB.getTel()).append("' ");
                } else {
                    stdlog_.error("Requested DELETE of instrument "
                            + currTelInstrDB.getInst() + " from telescope "
                            + currTelInstrDB.getTel() + " not possible."
                            + "\nThe instrument has gto information for cycle "
                            + cycle.getLabel()
                            + ".\n Please request the change to the DBO.");
                }
            }

        }
        if (runQuery) {
            stdlog_.debug("SQL inst insert query:" + query);

            stmt.executeUpdate(query.toString());
        } else {
            stdlog_.debug("SQL instrument NO update");
        }
    }

    private static TelInstCouple getInstrTelCouple(Vector<TelInstCouple> telInstrCouplelist,
            String currInstr) {
        TelInstCouple resultCouple = null;
        for (int i = 0; i < telInstrCouplelist.size(); i++) {
            TelInstCouple currCouple = telInstrCouplelist
                    .get(i);
            if (currCouple.getInst().equals(currInstr)) {
                resultCouple = currCouple;
                break;
            }
        }
        return resultCouple;
    }

    /**
     * @param stmt
     * @param cycle
     * @throws SQLException
     * @throws ObjectIOException
     */
    public static String checkTelescopesInstrumentsUpdate(Cycle cycle)
            throws ObjectIOException {
        StringBuffer msg = new StringBuffer();
        // get the DB telInstruments couples
        Vector<TelInstCouple> telInstrCoupleDBlist;
        try {
            telInstrCoupleDBlist = retrieveTelInstr(cycle);

            Vector<TelInstCouple> telInstrCoupleCyclelist = new Vector<TelInstCouple>();
            String[] telArray = cycle.getAllTelescopes();

            for (int i = 0; i < telArray.length; i++) {
                String currTel = telArray[i];
                String[] instList = cycle.getInstruments(currTel);
                for (int j = 0; j < instList.length; j++) {
                    String currInstr = instList[j];
                    TelInstCouple telInst = new TelInstCouple(currTel,
                            currInstr);
                    // build a cycle telInstr couple
                    telInstrCoupleCyclelist.add(telInst);
                }
            }

            // instrument to be deleted if instrument in DB not in cycle obj
            for (int i = 0; i < telInstrCoupleDBlist.size(); i++) {
                TelInstCouple currTelInstrDB = telInstrCoupleDBlist
                        .get(i);

                // return null if instrument NOT in cycle obj
                TelInstCouple telInstrCycleObj = getInstrTelCouple(
                        telInstrCoupleCyclelist, currTelInstrDB.getInst());

                // cycle doesn't contain the instr in DB. check if it is
                // possible to delete
                if (telInstrCycleObj == null) {
                    if (!isTelInstrRemovable(cycle, currTelInstrDB.getInst(),
                            currTelInstrDB.getTel())) {
                        msg.append("\nRequested DELETE of instrument ")
                        .append(currTelInstrDB.getInst()).
                        append(" from telescope ").
                        append(currTelInstrDB.getTel()).
                        append(" not possible.")
                        .append("\nThe instrument has gto information for cycle ")
                        .append(cycle.getLabel());
                        stdlog_
                                .warn("CANNOT DELETE  instrument "
                                        + currTelInstrDB.getInst()
                                        + " from telescope "
                                        + currTelInstrDB.getTel());
                    }
                }
            }
        } catch (SQLException e) {
            throw new ObjectIOException(e);
        }
        return msg.toString();
    }

    /**
     * @param stmt
     * @param cycle
     * @throws SQLException
     *             void
     */
    private Vector<TelInstCouple> retrievTelInstr(Statement stmt, Cycle cycle)
            throws SQLException {
        Vector<TelInstCouple> telInstCouple = new Vector<TelInstCouple>();
        ResultSet rs = stmt.executeQuery(" SELECT " + OFFERED_INSTRUMENT + ", "
                + OFFERED_TELESCOPE + " \nFROM "
                + Phase1SelectStmt.offeredTable.get() + " \n WHERE cycle_id ="
                + cycle.getCycleId());

        while (rs.next()) {
            TelInstCouple telInstr = new TelInstCouple(rs
                    .getString(OFFERED_TELESCOPE), rs
                    .getString(OFFERED_INSTRUMENT));
            telInstCouple.add(telInstr);

        }
        return telInstCouple;
    }

    /**
     * @param stmt
     * @param cycle
     * @throws ObjectIOException
     * @throws SQLException
     *             void
     */
    private static Vector<TelInstCouple> retrieveTelInstr(Cycle cycle)
            throws SQLException, ObjectIOException {
        Vector<TelInstCouple> telInstCouple = new Vector<TelInstCouple>();

        String[][] telInstrArray = DBdataHelper
                .getInstance()
                .readDoubleStringArray(
                        " SELECT " + OFFERED_INSTRUMENT + ", "
                                + OFFERED_TELESCOPE + " \nFROM "
                                + Phase1SelectStmt.offeredTable.get()
                                + " \n WHERE cycle_id =" + cycle.getCycleId());
        for (int i = 0; i < telInstrArray.length; i++) {
            String instr = telInstrArray[i][0];
            String tel = telInstrArray[i][1];
            TelInstCouple telInstr = new TelInstCouple(tel, instr);
            telInstCouple.add(telInstr);

        }
        return telInstCouple;
    }

    /**
     * @param stmt
     * @param cycle
     * @throws ObjectIOException
     * @throws SQLException
     * @throws SQLException
     *             void
     */
    private boolean isTelInstrRemovable(Statement stmt, Cycle cycle,
            String instrument, String telescope)
            throws SQLException, ObjectIOException {
        boolean result = false;
        String selInstrGto = instrGto(cycle, instrument, telescope);

        String selInstrConfigGto = instrConfGto(cycle, instrument, telescope);

        int rowInGtoInstrTable = DBdataHelper.getInstance().readIntValue(
                selInstrGto, stmt);
        int rowInGtoInstrConfTable = DBdataHelper.getInstance().readIntValue(
                selInstrConfigGto, stmt);

        if (rowInGtoInstrConfTable == 0 && rowInGtoInstrTable == 0) {
            result = true;
        }
        return result;
    }

    /**
     * @param stmt
     * @param cycle
     * @throws ObjectIOException
     * @throws SQLException
     * @throws SQLException
     *             void
     */
    public static boolean isTelInstrRemovable(Cycle cycle, String instrument,
            String telescope) throws SQLException, ObjectIOException {
        boolean result = false;
        String selInstrGto = instrGto(cycle, instrument, telescope);

        String selInstrConfigGto = instrConfGto(cycle, instrument, telescope);

        int rowInGtoInstrTable = DBdataHelper.getInstance().readIntValue(
                selInstrGto);
        int rowInGtoInstrConfTable = DBdataHelper.getInstance().readIntValue(
                selInstrConfigGto);

        if (rowInGtoInstrConfTable == 0 && rowInGtoInstrTable == 0) {
            result = true;
        }
        return result;
    }

    /**
     * @param cycle
     * @param instrument
     * @param telescope
     * @return String
     */
    private static String instrConfGto(Cycle cycle, String instrument,
            String telescope) {
        String selInstrConfigGto = " SELECT count(*) \nFROM "
                + Phase1SelectStmt.offeredTable.get() + ","
                + Phase1SelectStmt.gtoInstrConfigTable.get() + " \n WHERE "
                + Phase1SelectStmt.gtoInstrConfigTable.get() + ".instrument = "
                + Phase1SelectStmt.offeredTable.get() + ".instrument_code "
                + " \nAND " 
                + Phase1SelectStmt.gtoInstrConfigTable.get() + ".cycle_id = "
                + Phase1SelectStmt.offeredTable.get() + ".cycle_id "
                + " \nAND " + Phase1SelectStmt.offeredTable.get()
                + ".cycle_id =" + cycle.getCycleId() + "\nAND "
                + Phase1SelectStmt.offeredTable.get() + ".instrument_code ='"
                + instrument + "'\nAND " + Phase1SelectStmt.offeredTable.get()
                + ".telescope_code ='" + telescope + "' ";
        return selInstrConfigGto;
    }

    /**
     * @param cycle
     * @param instrument
     * @param telescope
     * @return String
     */
    private static String instrGto(Cycle cycle, String instrument,
            String telescope) {
        String selInstrGto = " SELECT count(*) \nFROM "
                + Phase1SelectStmt.offeredTable.get() + ","
                + Phase1SelectStmt.gtoInstrTable.get() + " \n WHERE "
                + Phase1SelectStmt.gtoInstrTable.get() + ".instrument = "
                + Phase1SelectStmt.offeredTable.get() + ".instrument_code "
                + "\nAND " + Phase1SelectStmt.gtoInstrTable.get() + ".cycle_id = "
                + Phase1SelectStmt.offeredTable.get() + ".cycle_id "
                + "\nAND " + Phase1SelectStmt.offeredTable.get()
                + ".cycle_id =" + cycle.getCycleId() + "\nAND "
                + Phase1SelectStmt.offeredTable.get() + ".instrument_code ='"
                + instrument + "'\nAND " + Phase1SelectStmt.offeredTable.get()
                + ".telescope_code ='" + telescope + "' ";
        return selInstrGto;
    }

    /**
     * Throws an ObjectIOException -- the phase 1 database cannot be changed
     * from within P2PP.
     * 
     * @param stmt
     *            JDBC Statement for contacting the dbase
     * @param id
     *            per-table id of the object to be written.
     * 
     * @exception ObjectIOException
     *                operation is not permitted exception
     */
    public void delete(Statement stmt, long id) throws ObjectIOException {

    }

    /**
     * Set the user fields from the result set. Only for avoiding duplicated
     * code
     * 
     */

    protected void setCyclesFields(Cycle cycle, ResultSet rs)
            throws SQLException {

        cycle.setLabel(rs.getString(CYCLE_LABEL));
        cycle.setPeriod(rs.getInt(PERIOD));
        cycle.setDescription(rs.getString(DESCRIPTION));
        cycle.setStepId(rs.getInt(STEP_ID));
    }

}