/* $Id: DbaseHandlerEsoUser.java,v 1.2 2006/04/26 09:36:38 ddorigo Exp $ */

package org.eso.ohs.persistence.dbase.phase1;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.LinkedHashSet;
import java.util.Set;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.TextUtils;
import org.eso.ohs.dfs.Affiliation;
import org.eso.ohs.dfs.BusinessVisitor;
import org.eso.ohs.dfs.EsoUser;
import org.eso.ohs.dfs.StorableObject;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.StorageManager.UnpackedStorableObject;
import org.eso.ohs.persistence.dbase.DbaseIOHandler;

/**
 * Sql handler to retrieve, to get and to updateEsoUser objects
 * on the database
 * <br>
 * $Id: DbaseHandlerEsoUser.java,v 1.2 2006/04/26 09:36:38 ddorigo Exp $
 * @see EsoUser
 * @author Carlo Boarotto, ESO 21-Mar-2000
 */
public class DbaseHandlerEsoUser extends DbaseIOHandler {

	/**
	 * Field stdlog_
	 */
	private static Logger stdlog_ =
		Logger.getLogger(DbaseHandlerEsoUser.class);

	/**
	 * Managed class
	 * @return EsoUser class
	 */
	public Class getManagedClass() {
		return EsoUser.class;
	}

	/** A reference to the singleton DbaseHandlerEsoUser instance. */
	private static DbaseHandlerEsoUser singleton_ = null;

	/** eso_users column names */
	public static final String ID = "id";
	/**
	 * Field ITEM_NAME
	 */
	public static final String ITEM_NAME = "item_name";
	/**
	 * Field SURNAME
	 */
	public static final String SURNAME = "surname";
	/**
	 * Field SURNAME_DL
	 */
	public static final String SURNAME_DL = "surname_dl";
	/**
	 * Field IGNORED
	 */
	public static final String IGNORED = "ignored";
	/**
	 * Field PASSWD
	 */
	public static final String PASSWD = "item_password";

	/** eso_users obsolete (and not null) column names */
	public static final String REF_ACRONYM = "referee_acronym";
	/**
	 * Field EMAIL
	 */
	public static final String EMAIL = "email";
	/**
	 * Field PANEL
	 */
	public static final String PANEL = "panel";
	/**
	 * Field CHAIR
	 */
	public static final String CHAIR = "chair";

	/** user_affiliations column names */
	public static final String USER_ID = "user_id";
	/**
	 * Field PERIOD
	 */
	public static final String PERIOD = "period";
	/**
	 * Field CYCLE_ID
	 */
	public static final String CYCLE_ID = "cycle_id";
	/**
	 * Field CYCLE_LABEL
	 */
	public static final String CYCLE_LABEL = "label";
	/**
	 * Field COUNTRY
	 */
	public static final String COUNTRY = "country_code";
	/**
	 * Field AFFILIATION
	 */
	public static final String AFFILIATION = "affiliation";
	/**
	 * Field ACCESSED_LETTER
	 */
	public static final String ACCESSED_LETTER = "accessed_letter";

	/** web roles columns name */
	public static final String ROLE = "letters_role";
	/**
	 * Field LOGIN
	 */
	public static final String LOGIN = "login";

	/** Sql select statement to retireve EsoUser entry. other fileds could be added in subclass */
	protected static final String user_fields =
		Phase1SelectStmt.usersTable.get()
			+ "."
			+ ID
			+ ","
			+ Phase1SelectStmt.usersTable.get()
			+ "."
			+ ITEM_NAME
			+ ","
			+ Phase1SelectStmt.usersTable.get()
			+ "."
			+ SURNAME
			+ ","
			+ Phase1SelectStmt.usersTable.get()
			+ "."
			+ SURNAME_DL
			+ ","
			+ Phase1SelectStmt.usersTable.get()
			+ "."
			+ PASSWD
			+ ","
			+ Phase1SelectStmt.usersTable.get()
			+ "."
			+ IGNORED
			+ ","
			+ Phase1SelectStmt.usersTable.get()
			+ "."
			+ EMAIL;

	/**
	 * Field user_obsolete_fields
	 */
	protected final static String user_obsolete_fields =
		Phase1SelectStmt.usersTable.get()
			+ "."
			+ REF_ACRONYM
			+ ","
			+ Phase1SelectStmt.usersTable.get()
			+ "."
			+ PANEL
			+ ","
			+ Phase1SelectStmt.usersTable.get()
			+ "."
			+ CHAIR;

	/** Sql select statement to retireve EsoUser entry. other fields could be added in subclass */
	protected final static String user_select_sql = "SELECT DISTINCT " + user_fields;

	/** Sql select statement to retireve EsoUser entry. */
	protected final static String user_sql =
		user_select_sql + "\n FROM " + Phase1SelectStmt.usersTable.get() + " ";

	/**
	 * Field roles_fields
	 */
	protected static final String roles_fields =
		Phase1SelectStmt.esoUsersRolesTable.get()
			+ "."
			+ LOGIN
			+ ","
			+ Phase1SelectStmt.esoUsersRolesTable.get()
			+ "."
			+ ROLE;

	/**
	 * Field aff_fields
	 */
	protected final static String aff_fields =
		Phase1SelectStmt.affiliationsTable.get()
			+ "."
			+ USER_ID
			+ ","
			+ Phase1SelectStmt.affiliationsTable.get()
			+ "."
			+ CYCLE_ID
			+ ","
			+ Phase1SelectStmt.affiliationsTable.get()
			+ "."
			+ ACCESSED_LETTER
			+ ","
			+ Phase1SelectStmt.affiliationsTable.get()
			+ "."
			+ PERIOD
			+ ","
			+ Phase1SelectStmt.affiliationsTable.get()
			+ "."
			+ COUNTRY
			+ ","
			+ Phase1SelectStmt.affiliationsTable.get()
			+ "."
			+ AFFILIATION;

	/**
	 * Field aff_obj_fields
	 */
	protected final String aff_obj_fields =
		aff_fields
			+ ","
			+ Phase1SelectStmt.cyclesTable.get()
			+ "."
			+ CYCLE_LABEL;

	/** Sql select statement to retireve EsoUser entry. */
	protected final String aff_sql =
		"SELECT DISTINCT "
			+ aff_obj_fields
			+ " FROM "
			+ Phase1SelectStmt.affiliationsTable.get()
			+ ", "
			+ Phase1SelectStmt.cyclesTable.get()
			+ " ";

	/** Sql select statement to retireve EsoUser entry. */
	protected final String role_sql =
		"SELECT DISTINCT "+roles_fields+" FROM "
			+ Phase1SelectStmt.esoUsersRolesTable.get()
			+ " ";

    private static boolean loadAffiliationAndRoles_ = true;



	/** ctor */
	public DbaseHandlerEsoUser() {
		super();
	}

	/**
	  * Read user data
	  * @param stmt statement
	 * @param objId id to be read
	 * @return UnpackedStorableObject
	 * @exception ObjectIOException IO Eror occured while reading from the database
	  * @exception ObjectNotFoundException the requested object was not found in the database
	  * @exception SQLException an error occurred during the dbase operation
	  */
	public UnpackedStorableObject read(Statement stmt, long objId)
		throws ObjectIOException, ObjectNotFoundException, SQLException {

		EsoUser user = (EsoUser) newInstance();
		UnpackedStorableObject uso = new UnpackedStorableObject(user);
		String sql = user_sql + " WHERE " + ID + " = " + objId;
		stdlog_.debug(sql);
		ResultSet rs = stmt.executeQuery(sql);

		if (!rs.next()) { // Only one with this id !!
			throw new ObjectNotFoundException("Object not found in database");
		}

		user.setId(Config.getCfg().tableToUniqueId(objId, EsoUser.class));
		setUserFields(user, rs);
		rs.close();
		// load the affiliation for the user
		if (isLoadAffiliationAndRoles()){
		     loadAffiliationsAndRoles(stmt, user, objId);
		}
		return new UnpackedStorableObject(user);
	}

	/**
	 * Read data objects are unique)
	 * @param stmt statement
	 * @param whereClause where condition for retreiving the object
	 * @param otherTables String table condition
	 * @return UnpackedStorableObject[]
	 * @exception ObjectIOException IO Eror occured while reading from the database
	 * @exception ObjectNotFoundException the requested object was not found in the database
	 * @exception SQLException an error occurred during the dbase operation
	 */
	public UnpackedStorableObject[] read(
		Statement stmt,
		String whereClause,
		String otherTables)
		throws ObjectIOException, ObjectNotFoundException, SQLException {

		if (otherTables == null) {
			otherTables = "";
		}

		UnpackedStorableObject[] user_array = null;
		EsoUser user = null;
		String sql = buildSqlQuery(whereClause, otherTables);
		stdlog_.debug(sql);
		ResultSet rs = stmt.executeQuery(sql);

		try {
			Set<EsoUser> users = new LinkedHashSet<EsoUser>();
			while (rs.next()) {
				user = (EsoUser) newInstance();

				long userId = rs.getInt(ID);
				// Set only when attaching the object as a new
				// element
				user.setId(
					Config.getCfg().tableToUniqueId(userId, EsoUser.class));
				setUserFields(user, rs);
				// New element
				users.add(user);
			}
			rs.close();

			EsoUser[] userList =
				(EsoUser[]) users.toArray(new EsoUser[users.size()]);
			user_array = new UnpackedStorableObject[userList.length];
			// add the affiliation to the user and insert them in array
			for (int i = 0; i < user_array.length; i++) {
				user = userList[i];
				if (isLoadAffiliationAndRoles()){
				  
				loadAffiliationsAndRoles(
					stmt,
					user,
					Config.getCfg().uniqueToTableId(user.getId()));
				}
				user_array[i] = new UnpackedStorableObject(user);
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return user_array;
	}

	public static String buildSqlQuery(String whereClause, String otherTables) {
		return user_sql + otherTables + "\n WHERE " + whereClause;
	}

	/**
	 * Set the user fields from the result set.
	 * Only for avoiding duplicated code
	 *
	 * @param user EsoUser
	 * @param rs ResultSet
	 * @throws SQLException
	 */
	protected void setUserFields(EsoUser user, ResultSet rs)
		throws SQLException {
		user.setItemName(rs.getString(ITEM_NAME));
		user.setSurname(rs.getString(SURNAME));
		user.setSurnameDL(rs.getString(SURNAME_DL));
		user.setPassword(rs.getString(PASSWD).trim());
		user.setTableId(rs.getInt(ID));
		user.setIgnored(rs.getBoolean(IGNORED));
	}

	/**
	  * Read readAffiliations data
	  * @param stmt statement
	 * @param user id to be read
	  * @param userId long
	 * @exception ObjectIOException IO Eror occured while reading from the database
	  * @exception ObjectNotFoundException the requested object was not found in the database
	  * @exception SQLException an error occurred during the dbase operation
	  */
	protected void loadAffiliationsAndRoles(Statement stmt, EsoUser user, long userId)
		throws ObjectIOException, ObjectNotFoundException, SQLException {

		Vector<String> roles = new Vector<String>();
		String roles_sql =
			role_sql
				+ " WHERE "
				+ LOGIN
				+ " = '"
				+ userId
				+ "'";
		stdlog_.debug(roles_sql);
		ResultSet rsRoles = stmt.executeQuery(roles_sql);
		
		while (rsRoles.next()) {
			roles.addElement(rsRoles.getString(ROLE));
		}
		String[] rolesArray = (String[]) roles.toArray(new String[roles.size()]);
		user.setRoles(rolesArray);
		rsRoles.close();
		
		Vector<Affiliation> affiliation = new Vector<Affiliation>();
		
		Affiliation[] affArray = null;
		String affiliation_sql =
			aff_sql
				+ " WHERE "
				+ USER_ID
				+ " = "
				+ userId
				+ " AND "
				+ Phase1SelectStmt.affiliationsTable.get()
				+ "."
				+ CYCLE_ID
				+ " = "
				+ Phase1SelectStmt.cyclesTable.get()
				+ ".id "
				+ " ORDER BY "
				+ CYCLE_ID;
		stdlog_.debug(affiliation_sql);
		ResultSet rs = stmt.executeQuery(affiliation_sql);

		// get the affiliation list reusing the statement
		Affiliation el = null;
		while (rs.next()) {
			el =
				new Affiliation(
					rs.getLong(USER_ID),
					rs.getInt(PERIOD),
					rs.getInt(CYCLE_ID),
					rs.getString(COUNTRY),
					rs.getString(AFFILIATION),
					rs.getLong(ACCESSED_LETTER));
			el.setCycleLabel(rs.getString(CYCLE_LABEL));
			affiliation.addElement(el);
		}
		if (affiliation.size() > 0) { // if 0 inconsistency of the DB
			// assign last affiliation to the user
			affArray = new Affiliation[affiliation.size()];
			affiliation.copyInto(affArray);
		}
		rs.close();

		user.setAffiliationArray(affArray);
	}

	/**
	 * Throws an ObjectIOException -- the phase 1 database cannot be
	 * changed from within P2PP.
	 *
	 * @param stmt  JDBC Statement for contacting the dbase
	 * @param id    per-table id of the object to be written.
	 * @param obj   an array: the first element is the BO to be stored.
	 *
	 * @exception ObjectIOException     operation is not permitted exception
	 *
	 * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
	 * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
	 */
	public void write(Statement stmt, long id, UnpackedStorableObject obj)
		throws ObjectIOException {

		EsoUser user = (EsoUser) obj.getObject();

		String sqlText = buildInsertStmt(id, user);

		try {
			stmt.executeUpdate(sqlText);
		} catch (SQLException e) {
			stdlog_.error("INSERT error", e);
			throw new ObjectIOException("Error executing write: " + sqlText);
		}
	}

	public static String buildInsertStmt(long id, EsoUser user) {
		StringBuffer sb = new StringBuffer();
		sb.append(id);
		sb.append(",\'");
		sb.append(TextUtils.quoteSqlString(user.getItemName()));
		sb.append("\',\'");
		sb.append(TextUtils.quoteSqlString(user.getSurname()));
		sb.append("\',\'");
		sb.append(TextUtils.quoteSqlString(user.getSurnameDL()));
		sb.append("\',\'");
		sb.append(TextUtils.quoteSqlString(user.getPassword()));
		sb.append("\',");
		sb.append(user.getIgnored() == true ? 1 : 0);
		sb.append(",\'");
		sb.append(TextUtils.quoteSqlString(user.getEmail()));
		sb.append("\',\'\', \'\',0 ");

		StringBuffer affilStmt = insertAffiliationsStmt(user, id);
		StringBuffer rolesStmt = updateRolesStmt(user, id);
		String sqlText =
			Phase1SelectStmt.beginTransaction
				+ " \n "
				+ "INSERT INTO "
				+ Phase1SelectStmt.usersTable.get()
				+ " ("
				+ user_fields
				+ ", "
				+ user_obsolete_fields
				+ " ) "
				+ " VALUES ("
				+ sb.toString()
				+ " ) "
				+ affilStmt.toString()
				+ rolesStmt.toString();
		stdlog_.debug(sqlText);
		return sqlText;
	}

	/**
	 * Method insertAffiliationsStmt
	 * @param user EsoUser
	 * @param userId long
	 * @return StringBuffer
	 */
	private static StringBuffer insertAffiliationsStmt(
		EsoUser user,
		long userId) {
		StringBuffer affilStmt = new StringBuffer();
		Affiliation[] affiliation = user.getAffiliationArray();
		for (int i = 0; i < affiliation.length; i++) {
			String accessedLetter = null;
			if (affiliation[i].getAccessedLetter() > 0) {
				accessedLetter =
					String.valueOf(affiliation[i].getAccessedLetter());
			}
			affilStmt
				.append("\n ")
				.append(" INSERT INTO ")
				.append(Phase1SelectStmt.affiliationsTable.get())
				.append(" (")
				.append(aff_fields)
				.append(" ) ")
				.append(" VALUES (")
				.append(userId)
				.append(",")
				.append(affiliation[i].getCycleId())
				.append(",")
				.append(accessedLetter)
				.append(",")
				.append(affiliation[i].getPeriod())
				.append(",\'")
				.append(
					TextUtils.quoteSqlString(affiliation[i].getCountryCode()))
				.append("',\'")
				.append(
					TextUtils.quoteSqlString(affiliation[i].getInstitution()))
				.append("\'")
				.append(" ) ");
		}
		return affilStmt;
	}

	/**
	 * Throws an ObjectIOException -- the phase 1 database cannot be
	 * changed from within P2PP.
	 *
	 * @param stmt  JDBC Statement for contacting the dbase
	 * @param id    per-table id of the object to be written.
	 * @param obj   an array: the first element is the BO to be stored.
	 *
	 * @exception ObjectIOException     operation is not permitted exception
	 *
	 * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
	 * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
	 */
	public void update(Statement stmt, long id, UnpackedStorableObject obj)
		throws ObjectIOException {

		EsoUser user = (EsoUser) obj.getObject();
		update(stmt, user);
	}

	/**
		 * Throws an ObjectIOException -- the phase 1 database cannot be
		 * changed from within P2PP.
		 *
		 * @param stmt  JDBC Statement for contacting the dbase
		 * @param obj   an array: the first element is the BO to be stored.
		 *
		 * @exception ObjectIOException     operation is not permitted exception
		 *
		 * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
		 * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
		 */
	public void update(Statement stmt, StorableObject obj)
		throws ObjectIOException {

		EsoUser user = (EsoUser) obj;
		String sqlText = buildUpdateStmt(user);

		try {
			stmt.executeUpdate(sqlText);
		} catch (SQLException e) {
			stdlog_.error("INSERT error", e);
			throw new ObjectIOException("Error executing write: " + sqlText);
		}
	}

	public static String buildUpdateStmt(EsoUser user) {
		long userId = Config.getCfg().uniqueToTableId(user.getId());
		StringBuffer sb = new StringBuffer();
		sb.append(ITEM_NAME);
		sb.append(" = \'");
		sb.append(TextUtils.quoteSqlString(user.getItemName()));
		sb.append("\',");
		sb.append(SURNAME);
		sb.append(" = \'");
		sb.append(TextUtils.quoteSqlString(user.getSurname()));
		sb.append("\',");
		sb.append(SURNAME_DL);
		sb.append(" = \'");
		sb.append(TextUtils.quoteSqlString(user.getSurnameDL()));
		sb.append("\',");
		sb.append(PASSWD);
		sb.append(" = \'");
		sb.append(TextUtils.quoteSqlString(user.getPassword()));
		sb.append("\',");
		sb.append(EMAIL);
		sb.append(" = \'");
		sb.append(TextUtils.quoteSqlString(user.getEmail()));
		sb.append("\',");
		sb.append(IGNORED);
		sb.append(" = ");
		sb.append(user.getIgnored() == true ? 1 : 0);
		sb.append(" ");

		StringBuffer affilStmt = updateAffilStmt(user, userId);
		StringBuffer roleStmt = updateRolesStmt(user, userId);

		String sqlText =
			Phase1SelectStmt.beginTransaction
				+ " \n "
				+ "UPDATE "
				+ Phase1SelectStmt.usersTable.get()
				+ " SET "
				+ sb.toString()
				+ " WHERE id = "
				+ userId
				+ affilStmt.toString()
				+ roleStmt.toString();
		stdlog_.debug(sqlText);
		return sqlText;
	}

	/**
	 * Method updateAffilStmt
	 * @param user EsoUser
	 * @param userId long
	 * @return StringBuffer
	 */
	private static StringBuffer updateAffilStmt(EsoUser user, long userId) {
		Affiliation[] affiliation = user.getAffiliationArray();

		StringBuffer affilStmt = new StringBuffer();
		affilStmt.append(" \n DELETE FROM ");
		affilStmt.append(Phase1SelectStmt.affiliationsTable.get());
		affilStmt.append(" WHERE user_id = ");
		affilStmt.append(userId);
		for (int i = 0; i < affiliation.length; i++) {
			String accessedLetter = "null";
			if (affiliation[i].getAccessedLetter() > 0) {
				accessedLetter =
					String.valueOf(affiliation[i].getAccessedLetter());
			}
			affilStmt
				.append(" \n ")
				.append(" INSERT INTO ")
				.append(Phase1SelectStmt.affiliationsTable.get())
				.append(" (")
				.append(aff_fields)
				.append(" ) ")
				.append(" VALUES (")
				.append(userId)
				.append(",")
				.append(affiliation[i].getCycleId())
				.append(",")
				.append(accessedLetter)
				.append(",")
				.append(affiliation[i].getPeriod())
				.append(",\'")
				.append(
					TextUtils.quoteSqlString(affiliation[i].getCountryCode()))
				.append("',\'")
				.append(
					TextUtils.quoteSqlString(affiliation[i].getInstitution()))
				.append("\'")
				.append(" ) ");
		}
		return affilStmt;
	}

	/**
	 * Method updateRolesStmt
	 * @param user EsoUser
	 * @param userId long
	 * @return StringBuffer
	 */
	private static StringBuffer updateRolesStmt(EsoUser user, long userId) {
		String[] roles = user.getRoles();
        
		StringBuffer roleStmt = new StringBuffer();
		for (int i = 0; i < roles.length; i++) {
			roleStmt
				.append(" \n DELETE FROM ")
				.append(Phase1SelectStmt.esoUsersRolesTable.get())
				.append(" WHERE login = \'")
				.append(userId)
				.append("\' AND letters_role = \'")
				.append(roles[i])
				.append("\'");
		}
		for (int i = 0; i < roles.length; i++) {
			roleStmt.append(" \n ");
			roleStmt.append(" INSERT INTO ").append(
				Phase1SelectStmt.esoUsersRolesTable.get());
			roleStmt
				.append(" (")
				.append(roles_fields)
				.append(" ) ")
				.append(" VALUES (\'")
				.append(String.valueOf(userId))
				.append("',\'")
				.append(TextUtils.quoteSqlString(roles[i]))
				.append("\')");
		}
		return roleStmt;
	}

	/**
	 * Throws an ObjectIOException -- the phase 1 database cannot be
	 * changed from within P2PP.
	 *
	 * @param stmt  JDBC Statement for contacting the dbase
	 * @param id    per-table id of the object to be written.
	 *
	 * @exception ObjectIOException     operation is not permitted exception
	 */
	public void delete(Statement stmt, long id) throws ObjectIOException {
		throw new ObjectIOException("Operation not supported for this object");
	}

	/**
	 * Method accept
	 * @param v BusinessVisitor
	 * @param arg Object
	 * @return Object
	 */
	public Object accept(BusinessVisitor v, Object arg) {
		return null;
	}

	/**
	 * Return the one and only instance of the DbaseHandlerEsoUser.
	 *
	 * @return Singleton DbaseHandlerEsoUser instance.
	 */
	public static DbaseHandlerEsoUser getDbaseHandlerEsoUser() {
		if (singleton_ == null) {
			singleton_ = new DbaseHandlerEsoUser();
		}
		return singleton_;
	}

    /**
     * @param loadAffiliationAndRoles_ The loadAffiliationAndRoles_ to set.
     */
    public static void setLoadAffiliationAndRoles(boolean loadAffiliationAndRoles) {
       loadAffiliationAndRoles_ = loadAffiliationAndRoles;
    }

    /**
     * @return Returns the loadAffiliationAndRoles_.
     */
    public static boolean isLoadAffiliationAndRoles() {
        return loadAffiliationAndRoles_;
    }
}
