/* $Id: DbaseHandlerObsProgramme.java,v 1.5 2006/04/26 09:36:38 ddorigo Exp $ */

package org.eso.ohs.persistence.dbase.phase1;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.Convert;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.TextUtils;
import org.eso.ohs.dfs.BusinessVisitor;
import org.eso.ohs.dfs.ObsProgramme;
import org.eso.ohs.dfs.StorableObject;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.OpticonType;
import org.eso.ohs.persistence.RrmType;
import org.eso.ohs.persistence.XmmType;
import org.eso.ohs.persistence.StorageManager.UnpackedStorableObject;
import org.eso.ohs.persistence.dbase.DbaseIOHandler;

/**
 * Sql handler to retrieve, to get and to update EsoUser objects
 * on the database.  
 * <br>
 *
 * @see ObsProgramme
 */
public class DbaseHandlerObsProgramme extends DbaseIOHandler {

	public static final String BASIC_FLAG = "00000000000000000";
	private static Logger stdlog_ =
		Logger.getLogger(DbaseHandlerObsProgramme.class);

	/**
	  * Managed class
	  * @return EsoUser class
	  */
	public Class getManagedClass() {
		return ObsProgramme.class;
	}

	/** A reference to the singleton DbaseHandlerObsProgramme instance. */
	private static DbaseHandlerObsProgramme singleton_ = null;

	/** flag related to XMM */
	public static final int XMM_FLAG_POS = 0;
	/** programme column names */
	public static final String FLAGS = "flags";
	public static final String CYCLE_LABEL = "label"; // from cycles table
	public static final String CYCLE_ID = "cycle_id";
	public static final String PERIOD = "period";
	public static final String PANEL = "panel";
	public static final String SUBPANEL = "subpanel";
	public static final String OPC_SUBPANEL = "opc_subpanel";
	public static final String NUMBER = "number";
	public static final String ID = "id";
	public static final String TITLE = "title";
	public static final String ATTACHMENTS = "attachments";
	public static final String PROPOSALID = "proposalID";
	public static final String PROPOSALCODE = "proposal_code";
	public static final String POST_ADDRESS = "post_address";
	public static final String EMAIL_ADDRESS = "email_address";
	public static final String INDATE = "date_received";
	public static final String PROCDATE = "date_processed";
	public static final String PRINTED = "printed";
	public static final String IGNORED = "ignored";
	public static final String OBSRUN = "obs_runs_num";
	public static final String KEYPROG = "key_prog";
	public static final String GTO = "gto";

	/** Select  statement to retireve ObsProgramme entry. */
	protected final static String prog_sql =
		"SELECT DISTINCT "
			+ Phase1SelectStmt.programmesTable.get()
			+ "."
			+ ID
			+ ","
			+ Phase1SelectStmt.programmesTable.get()
			+ "."
			+ CYCLE_ID
			+ ","
			+ Phase1SelectStmt.programmesTable.get()
			+ "."
			+ PERIOD
			+ ","
			+ Phase1SelectStmt.programmesTable.get()
			+ "."
			+ PANEL
			+ ","
			+ Phase1SelectStmt.programmesTable.get()
			+ "."
			+ SUBPANEL
			+ ","
			+ Phase1SelectStmt.programmesTable.get()
			+ "."
			+ OPC_SUBPANEL
			+ ","
			+ Phase1SelectStmt.programmesTable.get()
			+ "."
			+ NUMBER
			+ ","
			+ Phase1SelectStmt.programmesTable.get()
			+ "."
			+ TITLE
			+ ","
			+ Phase1SelectStmt.programmesTable.get()
			+ "."
			+ ATTACHMENTS
			+ ","
			+ Phase1SelectStmt.programmesTable.get()
			+ "."
			+ PROPOSALID
			+ ","
			+ Phase1SelectStmt.programmesTable.get()
			+ "."
			+ PROPOSALCODE
			+ ","
			+ Phase1SelectStmt.programmesTable.get()
			+ "."
			+ POST_ADDRESS
			+ ","
			+ Phase1SelectStmt.programmesTable.get()
			+ "."
			+ FLAGS
			+ ","
			+ Phase1SelectStmt.programmesTable.get()
			+ "."
			+ EMAIL_ADDRESS
			+ ","
			+ Phase1SelectStmt.programmesTable.get()
			+ "."
			+ INDATE
			+ ","
			+ Phase1SelectStmt.programmesTable.get()
			+ "."
			+ PROCDATE
			+ ","
			+ Phase1SelectStmt.programmesTable.get()
			+ "."
			+ PRINTED
			+ ","
			+ Phase1SelectStmt.programmesTable.get()
			+ "."
			+ IGNORED
			+ ","
			+ Phase1SelectStmt.programmesTable.get()
			+ "."
			+ OBSRUN
			+ ","
			+ Phase1SelectStmt.programmesTable.get()
			+ "."
			+ KEYPROG
			+ ","
			+ Phase1SelectStmt.programmesTable.get()
			+ "."
			+ GTO
			+ ","
			+ Phase1SelectStmt.cyclesTable.get()
			+ "."
			+ CYCLE_LABEL
			+ " FROM "
			+ Phase1SelectStmt.programmesTable.get()
			+ ", "
			+ Phase1SelectStmt.cyclesTable.get()
			+ " ";

	protected final static String cycle_join =
		Phase1SelectStmt.programmesTable.get()
			+ "."
			+ CYCLE_ID
			+ " = "
			+ Phase1SelectStmt.cyclesTable.get()
			+ ".id ";

	/** 
	 */
	public DbaseHandlerObsProgramme() {
		super();
	}

	/**
	  * Read  user data
	  * @param sql statement
	  * @param Object id to be read
	  * @exception SQLException
	  *		    an error occurred during the dbase operation
	  * @exception ObjectNotFoundException
	  *		    the requested object was not found in the database   
	  * @exception ObjectIOException IO Eror occured while reading from the database
	  */

	public UnpackedStorableObject read(Statement stmt, long objId)
		throws ObjectIOException, ObjectNotFoundException, SQLException {

		stdlog_.debug("read method: id " + objId);
		ObsProgramme prog = (ObsProgramme) newInstance();

		UnpackedStorableObject uso = new UnpackedStorableObject(prog);
		String sql =
			prog_sql
				+ " WHERE "
				+ Phase1SelectStmt.programmesTable.get()
				+ "."
				+ ID
				+ " = "
				+ objId
				+ " AND    "
				+ cycle_join;
		stdlog_.debug("sql statement: " + sql);

		stdlog_.debug(sql);
		ResultSet rs = stmt.executeQuery(sql);

		if (!rs.next()) { // Only one with this id !!
			throw new ObjectNotFoundException("Object not found in database");
		}

		prog.setId(Config.getCfg().tableToUniqueId(objId, ObsProgramme.class));
		setProgFields(prog, rs);
		rs.close();
		return new UnpackedStorableObject(prog);
	}

	/**
	  * Read  data
	  * @param sql statement
	  * @param whereClause where condition for retreiving the object
	  * @exception SQLException
	  *		    an error occurred during the dbase operation
	  * @exception ObjectNotFoundException
	  *		    the requested object was not found in the database   
	  * @exception ObjectIOException IO Eror occured while reading from the database
	  */

	public UnpackedStorableObject[] read(
		Statement stmt,
		String whereClause,
		String otherTables)
		throws ObjectIOException, ObjectNotFoundException, SQLException {

		if (otherTables == null) {
			otherTables = "";
		}
		UnpackedStorableObject[] prog_array = null;
		ObsProgramme prog = null;
		String sql =
			prog_sql
				+ otherTables
				+ " WHERE "
				+ cycle_join
				+ " AND "
				+ whereClause;

		stdlog_.debug(sql);
		ResultSet rs = stmt.executeQuery(sql);

		try {
			Vector<ObsProgramme> prog_vector = new Vector<ObsProgramme>();

			while (rs.next()) {

				prog = (ObsProgramme) newInstance();
				// New element
				prog_vector.addElement(prog);

				// Set only when attaching the object as a new
				// element
				prog.setId(
					Config.getCfg().tableToUniqueId(
						rs.getInt(ID),
						ObsProgramme.class));
				stdlog_.debug("prog retrieved:" + rs.getInt(ID));

				setProgFields(prog, rs);
			}
			rs.close();

			prog_array = new UnpackedStorableObject[prog_vector.size()];
			for (int i = 0; i < prog_vector.size(); i++) {
				prog = prog_vector.elementAt(i);
				prog_array[i] = new UnpackedStorableObject(prog);
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}

		return prog_array;
	}

	/**
	 * Set the prog fields from the result set.
	 * Only for avoiding duplicated code
	 *
	 */
	protected void setProgFields(ObsProgramme prog, ResultSet rs)
		throws SQLException {
		prog.setPeriod(rs.getInt(PERIOD));
		prog.setTableId(rs.getInt(ID));
		prog.setCycle(rs.getInt(CYCLE_ID));
		prog.setCycleLabel(rs.getString(CYCLE_LABEL));
		prog.setPanel(rs.getString(PANEL).charAt(0));
		prog.setSubpanel(rs.getString(SUBPANEL));
		prog.setOpcSubpanel(rs.getInt(OPC_SUBPANEL));
		prog.setNumber(rs.getInt(NUMBER));
		prog.setTitle(rs.getString(TITLE));
		prog.setAttachments(rs.getInt(ATTACHMENTS));
		prog.setProposalID(rs.getString(PROPOSALID));
		prog.setProposalCode(rs.getString(PROPOSALCODE));
		prog.setPostAddr(rs.getString(POST_ADDRESS));
		prog.setEmailAddr(rs.getString(EMAIL_ADDRESS));
		String flags = rs.getString(FLAGS);
		checkOpticon(prog, flags);
		checkXmm(prog, flags);
		checkRrm(prog, flags);
		
		prog.setInDate(Convert.starcatDateToDate(rs.getLong(INDATE)));
		prog.setProcDate(Convert.starcatDateToDate(rs.getLong(PROCDATE)));
		prog.setIgnored(rs.getBoolean(IGNORED));
		prog.setPrinted(rs.getInt(PRINTED));
		prog.setObsRunsNum(rs.getInt(OBSRUN));
		prog.setKeyprog(rs.getInt(KEYPROG));
		prog.setGto(rs.getInt(GTO));

	}

	/**
	 * @param prog
	 * @param flags
	 */
	private void checkOpticon(ObsProgramme prog, String flags) {
		if (flags != null
			&& flags.length() > OpticonType.OPTICON_FLAG_POS
			&& flags.charAt(OpticonType.OPTICON_FLAG_POS)
				== OpticonType.OPTICON_FLAG) {
			prog.setOpticon(true);
		} else {
			prog.setOpticon(false);
		}
	}

	
	/**
	 * @param prog
	 * @param flags
	 */
	private void checkXmm(ObsProgramme prog, String flags) {
		if (flags != null
			&& flags.length() > XmmType.XMM_FLAG_POS
			&& flags.charAt(XmmType.XMM_FLAG_POS)
				== XmmType.XMM_FLAG) {
			prog.setXmm(true);
		} else {
			prog.setXmm(false);
		}
	}
	
	/**
	 * @param prog
	 * @param flags
	 */
	private void checkRrm(ObsProgramme prog, String flags) {
		if (flags != null
			&& flags.length() > RrmType.RRM_FLAG_POS
			&& flags.charAt(RrmType.RRM_FLAG_POS)
				== RrmType.RRM_FLAG) {
			prog.setRrm(true);
		} else {
			prog.setRrm(false);
		}
	}
	
	
	/**
	 * Throws an ObjectIOException -- the phase 1 database cannot be
	 * changed from within P2PP.
	 * 
	 * @param stmt  JDBC Statement for contacting the dbase
	 * @param id    per-table id of the object to be written.
	 * @param obj   an array: the first element is the BO to be stored.
	 * 
	 * @exception ObjectIOException	operation is not permitted exception
	 * 
	 * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
	 * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
	 */
	public void write(Statement stmt, long id, UnpackedStorableObject obj)
		throws ObjectIOException {

		throw new ObjectIOException("Operation not supported for this object");

	}

	/**
	* 
	 * @param stmt  JDBC Statement for contacting the dbase
	 * @param id    per-table id of the object to be written.
	 * @param obj    the BO to be stored.
	 * 
	 * @exception ObjectIOException	operation is not permitted exception
	 * 
	 * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
	 * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
	 */
	public void update(Statement stmt, long id, UnpackedStorableObject objSpec)
		throws ObjectIOException, SQLException {
		stdlog_.debug("update object");
		StorableObject obj = objSpec.getObject();
		update(stmt, obj);
		stdlog_.debug("update object done");

	}

	/**
	
	 * 
	 * @param stmt  JDBC Statement for contacting the dbase
	 * @param id    per-table id of the object to be written.
	 * @param obj   an array: the first element is the BO to be stored.
	 * 
	 * @exception ObjectIOException	operation is not permitted exception
	 * 
	 * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
	 * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
	 */
	public void update(Statement stmt, StorableObject obj)
		throws ObjectIOException, SQLException {

		ObsProgramme op = (ObsProgramme) obj;
		StringBuffer flags = new StringBuffer(BASIC_FLAG);
		
		stdlog_.debug("Initial flags:           " + flags);
		if (op.isOpticon()) {
			flags.replace(OpticonType.OPTICON_FLAG_POS,OpticonType.OPTICON_FLAG_POS + 1, "1");
		}
		stdlog_.debug("Flags after opticon set: " + flags);
		
		if (op.isXmm()) {
			flags.replace(XmmType.XMM_FLAG_POS,XmmType.XMM_FLAG_POS + 1, "1");
		}
		stdlog_.debug("Flags after xmm set:     " + flags);
		
		
		if (op.isRrm()) {
			flags.replace(RrmType.RRM_FLAG_POS,RrmType.RRM_FLAG_POS + 1, "1");
		}
		stdlog_.debug("Flags after rrm set:     " + flags);
		
		
		String sql =
			"UPDATE "
				+ Phase1SelectStmt.programmesTable.get()
				+ " "
				+ "SET "
				+ PERIOD
				+ " = "
				+ op.getPeriod()
				+ ", "
				+ CYCLE_ID
				+ " = "
				+ op.getCycle()
				+ ", "
				+ PANEL
				+ " = '"
				+ op.getPanel()
				+ "', "
				+ SUBPANEL
				+ " = '"
				+ op.getSubpanel()
				+ "', "
				+ OPC_SUBPANEL
				+ " = "
				+ op.getOpcSubpanel()
				+ ", "
				+ PROPOSALID
				+ " = '"
				+ op.getProposalID()
				+ "', "
				+ PROPOSALCODE
				+ " = '"
				+ op.getProposalCode()
				+ "', "
				+ EMAIL_ADDRESS
				+ " = '"
				+ TextUtils.quoteSqlString(op.getEmailAddr())
				+ "', "
				+ FLAGS
				+ " = '"
				+ TextUtils.quoteSqlString(flags.toString())
				+ "', "
				+ TITLE
				+ " = '"
				+ TextUtils.quoteSqlString(op.getTitle())
				+ "', "
				+ IGNORED
				+ " = "
				+ (op.getIgnored() ? 1 : 0)
				+ ", "
				+ PRINTED
				+ " = "
				+ op.getPrinted()
				+ ", "
				+ KEYPROG
				+ " = "
				+ op.getKeyprog()
				+ ", "
				+ GTO
				+ " = "
				+ op.getGto()
				+ " "
				+ "WHERE "
				+ ID
				+ " = "
				+ Config.getCfg().uniqueToTableId(op.getId())
				+ "\n\n";
		stdlog_.debug(sql);
		stmt.executeUpdate(sql);

	}

	/**
	 * Throws an ObjectIOException -- the phase 1 database cannot be
	 * changed from within P2PP.
	 * 
	 * @param stmt  JDBC Statement for contacting the dbase
	 * @param id    per-table id of the object to be written.
	 * 
	 * @exception ObjectIOException	operation is not permitted exception
	 */
	public void delete(Statement stmt, long id) throws ObjectIOException {
		throw new ObjectIOException("Operation not supported for this object");
	}

	public Object accept(BusinessVisitor v, Object arg) {
		return null;
	}

	/**
	   * Return the one and only instance of the DbaseHandlerObsProgramme.
	   * 
	   * @return Singleton DbaseHandlerObsProgramme instance.
	   */
	public static DbaseHandlerObsProgramme getDbaseHandlerObsProgramme() {
		if (singleton_ == null) {
			singleton_ = new DbaseHandlerObsProgramme();
		}
		return singleton_;
	}
}
