/* $Id: DbaseHandlerOpcReferee.java,v 1.2 2006/04/26 09:36:38 ddorigo Exp $ */

package org.eso.ohs.persistence.dbase.phase1;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.TextUtils;
import org.eso.ohs.dfs.OpcReferee;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.StorageManager.UnpackedStorableObject;
import org.eso.ohs.persistence.dbase.DbaseIOHandler;



/**
 * Sql handler to retrieve, to get and to update OpcReferee objects
 * on the database.  
 * <br>
 *
 * @see OpcReferee
 * @author Carlo Boarotto, ESO 21-Mar-2000 
 */
public class DbaseHandlerOpcReferee  extends DbaseIOHandler {
 
  private final static int MAX_PHASE1_CYCLES = 100000;


    /**
     * Managed class
     * @return OpcReferee class
     */
    public Class getManagedClass() {
	return OpcReferee.class;
    }

    public static int cycleIndexFromId(long id) {
	return (int) (id % MAX_PHASE1_CYCLES);
    }

    public static long userIdFromId(long id) {
	return id / MAX_PHASE1_CYCLES;
    }

    public static long idForPhase1Referee(long userId, int cycleId) {
	return userId*MAX_PHASE1_CYCLES + cycleId;
    }

    DbaseHandlerEsoUser esoUserHandler = null;
 
   /** Standard log channel */ 
    private static Logger stdlog_ = 
        Logger.getLogger( DbaseHandlerOpcReferee.class );

    /** A reference to the singleton OpcRefereeSql instance. */
    private static DbaseHandlerOpcReferee singleton_ = new DbaseHandlerOpcReferee();


    public static final String CYCLE_LABEL =  "label"; // from cycles table
    public  static final String CYCLE_ID = "cycle_id";
    public  static final String USER_ID = "user_id";
    public  static final String PANEL = "panel";
    public static final String EMAIL = "email";
    public static final String CHAIR = "chair";
    public static final String OPC_MEMBER = "opc_member";
    public static final String SUBPANEL = "opc_subpanel";
    public static final String ACRONYM = "acronym";



    /** Sql select statement to retrieve OpcReferee entries. */
   

    protected final static String ref_fields = 
      Phase1SelectStmt.refereesTable.get()+"."+CYCLE_ID + "," +
      Phase1SelectStmt.refereesTable.get()+"."+ USER_ID + "," +
      Phase1SelectStmt.refereesTable.get()+"."+ PANEL + "," + 
      Phase1SelectStmt.refereesTable.get()+"."+SUBPANEL + "," +
      Phase1SelectStmt.refereesTable.get()+"."+ EMAIL + ","  +
      Phase1SelectStmt.refereesTable.get()+"."+ ACRONYM +"," + 
      Phase1SelectStmt.refereesTable.get()+"."+OPC_MEMBER + "," +
      Phase1SelectStmt.refereesTable.get()+"."+ CHAIR;

  protected final static String ref_select_sql =  "SELECT DISTINCT " 
  + ref_fields+ "," + Phase1SelectStmt.cyclesTable.get()+"."+ CYCLE_LABEL;
 

	    protected final static String cycle_join = Phase1SelectStmt.refereesTable.get() + "."+CYCLE_ID + " = " +
	    Phase1SelectStmt.cyclesTable.get() + ".id ";
   /** Sql select statement to retrieve OpcReferee entries. */
    protected final static String referee_sql = ref_select_sql +
    " FROM " + Phase1SelectStmt.refereesTable.get()+ ", " +
    Phase1SelectStmt.cyclesTable.get()+ " "; 
  
    /** 
     * Private constructor: it is a singleton class.
     */
     public DbaseHandlerOpcReferee () {
	super();
	esoUserHandler = new DbaseHandlerEsoUser();
    }






 /**
     * Read  opcReferee data
     * @param sql statement
     * @param Object id to be read
     * @exception SQLException
     *		    an error occurred during the dbase operation
     * @exception ObjectNotFoundException
     *		    the requested object was not found in the database   
     * @exception ObjectIOException IO Eror occured while reading from the database
     */

    public UnpackedStorableObject read(Statement stmt,long objId)
			throws ObjectIOException, ObjectNotFoundException,
				SQLException {
 	OpcReferee ref_user = (OpcReferee)newInstance();

	UnpackedStorableObject uso = new UnpackedStorableObject(ref_user);
	String sql = referee_sql + " WHERE "+ CYCLE_ID + " = " + cycleIndexFromId(objId) +
	    " AND    "+ cycle_join +
	    " AND    "+ USER_ID + "  = " + userIdFromId(objId);
	  stdlog_.debug("read referee: "+sql);

    stdlog_.debug(sql);
        ResultSet rs = stmt.executeQuery(sql);

	if ( ! rs.next()) { // Only one with this id !!
	    throw new ObjectNotFoundException("Object not found in database");
	}

	ref_user.setId(objId);
	setRefFields(ref_user, rs);
	rs.close();
	
	return new UnpackedStorableObject(ref_user);
    }


   /**
     * Read  data
     * @param sql statement
     * @param Object id to be read
     * @exception SQLException
     *		    an error occurred during the dbase operation
     * @exception ObjectNotFoundException
     *		    the requested object was not found in the database   
     * @exception ObjectIOException IO Eror occured while reading from the database
     */


    public UnpackedStorableObject[] read(Statement stmt,String whereClause, String otherTables)
			throws ObjectIOException, ObjectNotFoundException,
				SQLException {


	UnpackedStorableObject[] user_array = null;
	OpcReferee ref_user = null;
	if (otherTables == null) {
	    otherTables = "";
	}
	String sql = referee_sql  + otherTables+ " WHERE "+ cycle_join +" AND "+ whereClause;
	stdlog_.debug(sql);
	ResultSet rs = stmt.executeQuery(sql);

	try {
	    Vector<OpcReferee> user_vector = new Vector<OpcReferee>();      
	
  	    while (rs.next()) {

		
		ref_user = (OpcReferee)newInstance();
		    // New element
		    user_vector.addElement(ref_user);

		    // Set only when attaching the object as a new
		    // element
		    setRefFields(ref_user, rs);
		  
		long refId = idForPhase1Referee(rs.getLong(USER_ID),rs.getInt(CYCLE_ID));
		
		ref_user.setId(Config.getCfg().tableToUniqueId(refId, OpcReferee.class));
		  }  	   
	    rs.close();
	    
	    user_array =  new UnpackedStorableObject[user_vector.size()];
	    // add the affiliation to the user and insert them in array
	    for (int i = 0 ; i< user_vector.size(); i++) {
		ref_user = 	user_vector.elementAt(i);
		user_array[i] = new UnpackedStorableObject(ref_user);
	    }
	}
	catch (SQLException e) {
	    e.printStackTrace();
	}
	
	return user_array;
    }



    /**
     * Throws an ObjectIOException -- the phase 1 database cannot be
     * changed from within P2PP.
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be written.
     * @param obj   an array: the first element is the BO to be stored.
     * 
     * @exception ObjectIOException	operation is not permitted exception
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public void write(Statement stmt, long id, UnpackedStorableObject obj)
			throws ObjectIOException {


	OpcReferee user  = (OpcReferee)obj.getObject();

	StringBuffer sb = new StringBuffer();

	sb.append(cycleIndexFromId(id));
	sb.append(",");
	sb.append(userIdFromId(id));
	sb.append(",\'");
	sb.append(TextUtils.quoteSqlString(user.getPanel()));
	sb.append("\',");
 	sb.append(user.getOpcSubpanel());
	sb.append(",\'");
 	sb.append(TextUtils.quoteSqlString(user.getEmail()));
 	sb.append("\',\'");
 	sb.append(TextUtils.quoteSqlString(user.getRefereeAcronym()));
 	sb.append("\',");
 	sb.append(user.getOpcMember()==true ? 1: 0);
 	sb.append(",");
 	sb.append(user.getChair()==true ? 1: 0);
	sb.append(" ");

 
	String sqlText = Phase1SelectStmt.beginTransaction+ " \n " +
	     "INSERT INTO " + Phase1SelectStmt.refereesTable.get()
	     + " ("+ ref_fields +" ) "
	    + " VALUES ("+ sb.toString()+ " ) "; 

	stdlog_.debug (sqlText);
	try {

	    stmt.executeUpdate(sqlText);
	} catch (SQLException e) {
	    stdlog_.error("update error",e);
	    throw new ObjectIOException("Error executing write: "+ sqlText );
	}
	
    }


    /**
     * Throws an ObjectIOException -- the phase 1 database cannot be
     * changed from within P2PP.
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be written.
     * @param obj   an array: the first element is the BO to be stored.
     * 
     * @exception ObjectIOException	operation is not permitted exception
     * 
     * @see org.eso.ohs.persistence.ObjectManager#disassembleReferences
     * @see org.eso.ohs.persistence.ObjectManager#reassembleReferences
     */
    public void update(Statement stmt, long id, UnpackedStorableObject obj)
			throws ObjectIOException {

	OpcReferee user  = (OpcReferee)obj.getObject();

	StringBuffer sb = new StringBuffer();
	sb.append(PANEL);
	sb.append(" = \'");
 	sb.append(TextUtils.quoteSqlString(user.getPanel()));
	sb.append("\',");
	sb.append(SUBPANEL);
	sb.append(" = ");
 	sb.append(user.getOpcSubpanel());
	sb.append(",");
 	sb.append(EMAIL);
	sb.append(" = \'");
 	sb.append(TextUtils.quoteSqlString(user.getEmail()));
 	sb.append("\',");
 	sb.append(ACRONYM);
	sb.append(" = \'");
 	sb.append(TextUtils.quoteSqlString(user.getRefereeAcronym()));
 	sb.append("\',");
 	sb.append(OPC_MEMBER);
	sb.append(" = ");
 	sb.append(user.getOpcMember()==true ? 1: 0);
 	sb.append(",");
 	sb.append(CHAIR);
	sb.append(" = ");
 	sb.append(user.getChair()==true ? 1: 0);
	sb.append(" ");

 
	String sqlText = Phase1SelectStmt.beginTransaction+ " \n " +
	    "UPDATE  " + Phase1SelectStmt.refereesTable.get()
	    + " SET "+ sb.toString()
	    + " WHERE "+ CYCLE_ID + " = " + cycleIndexFromId(id)
	    + " AND    "+ USER_ID + "  = " + userIdFromId(id); 

	stdlog_.debug(sqlText);
	try {

	    stmt.executeUpdate(sqlText);
	} catch (SQLException e) {
	    stdlog_.error("update error",e);
	    throw new ObjectIOException("Error executing write: "+ sqlText );
	}

    }

    /**
     * Throws an ObjectIOException -- the phase 1 database cannot be
     * changed from within P2PP.
     * 
     * @param stmt  JDBC Statement for contacting the dbase
     * @param id    per-table id of the object to be written.
     * 
     * @exception ObjectIOException	operation is not permitted exception
     */
    public void delete(Statement stmt, long id)
			throws ObjectIOException {
   
	String sqlText = Phase1SelectStmt.beginTransaction+ " \n " +
	    " \n DELETE  FROM " + Phase1SelectStmt.refereesTable.get()
	    + " WHERE "+ CYCLE_ID + " = " + cycleIndexFromId(id)
	    + " AND    "+ USER_ID + "  = " + userIdFromId(id); 

	stdlog_.debug (sqlText);
	try {

	    stmt.executeUpdate(sqlText);
	} catch (SQLException e) {
	    stdlog_.error("update error",e);
	    throw new ObjectIOException("Error executing write: "+ sqlText );
	}
    }


/**
 * Set the user fields from the result set.
 * Only for avoiding duplicated code
 *
 */ 
    protected void setRefFields (OpcReferee ref_user ,ResultSet rs) throws SQLException {
  
	ref_user.setOpcSubpanel(rs.getInt(SUBPANEL));
	ref_user.setPanel(rs.getString(PANEL));
	ref_user.setEmail(rs.getString(EMAIL));
	ref_user.setRefereeAcronym(rs.getString(ACRONYM));
	ref_user.setChair(rs.getBoolean(CHAIR));
	ref_user.setOpcMember(rs.getBoolean(OPC_MEMBER));
	ref_user.setUserId(rs.getLong(USER_ID));
	ref_user.setCycleId(rs.getInt(CYCLE_ID));
	ref_user.setCycleLabel( rs.getString(CYCLE_LABEL)); 
    }




}
