package org.eso.ohs.persistence.dbase.phase1;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.ConnectionPool;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.SqlUtils;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.DBdataHelper;

/**
 * I/O class for IPs stored in the database
 * <p>
 * 
 * @version $Revision: 1.4.4.1 $
 * @author Ron J Lobo <br>
 * @see org.eso.ohs.persistence.dbase.phase2.DbaseMgr
 * @see org.eso.ohs.persistence.ObjectManager
 */
public class EsoUsersDBIO {

    /** CVS ID string */
    public static final String cvsID_ = "$Id: EsoUsersDBIO.java,v 1.4.4.1 2007/10/24 12:31:50 pnunes Exp $";

    // Standard log channel
    private static final Logger stdlog_ = Logger.getLogger(EsoUsersDBIO.class);

    // databases
    private static final String obrepDbName = Config.getCfg().getDbName(
            ObservationBlock.getSuffix());

    private static final String opc70DbName = Config.getCfg().getDbName(
            ObservingRun.getSuffix());

    // private static final String phase2DbName =
    // AppConfig.getAppConfig().getValue(OHSKeys.USGDatabase);

    // tables
    private static final String OP = opc70DbName + "..obs_programmes";

    private static final String PRO = opc70DbName + "..proposed";

    private static final String EU = opc70DbName + "..eso_users";

    private static final String UA = opc70DbName + "..user_affiliations";

    private static final String CYC = opc70DbName + "..cycles";

    private static final String REF = opc70DbName + "..referees";

    private static final String EUR = opc70DbName + "..eso_users_roles";

    // private static final String RTD = opc70DbName + "..rated";
    // private static final String COM = opc70DbName + "..comments";
    private static final String OB = obrepDbName + "..obs_blocks";

    private static final String lsep = System.getProperty("line.separator");

    /**
     * @return set of countries
     */
    public static String[] getCountries() throws SQLException {
        // stdlog_.debug("EsoUsersDBIO.getCountries()");
        return getUniqueValues("country_code", UA, true);
    }

    /**
     * @return set of cycles
     */
    public static String[] getCycles() throws SQLException {
        // stdlog_.debug("EsoUsersDBIO.getCycles()");
        return getUniqueValues("label", CYC, false);
    }

    private static String[] getUniqueValues(String column, String table,
            boolean ascending) throws SQLException {

        // stdlog_.debug("EsoUsersDBIO.getUniqueValues()");
        Connection con = null;
        Statement stmt = null;
        try {
            Config cfg = Config.getCfg();
            con = ConnectionPool.getPool().acquireConnection(
                    cfg.getUrl(ObservationBlock.getSuffix()),
                    cfg.getDbUserName(), cfg.getDbPassword());
            stmt = con.createStatement();

            String sql = "select distinct " + column + " from " + table
                    + " order by " + column + (ascending ? "" : " desc");
            stdlog_.debug(sql);

            ResultSet rs = stmt.executeQuery(sql);
            Vector<String> vec = new Vector<String>();
            while (rs.next()) {
                String value = rs.getString(column);
                vec.addElement(value);
            }
            rs.close();
            stmt.close();
            ConnectionPool.getPool().releaseConnection(con);

            String[] stringArray = new String[vec.size()];
            for (int i = 0; i < vec.size(); i++) {
                stringArray[i] = vec.elementAt(i);
            }

            return stringArray;

        } catch (SQLException e) {
            stdlog_.error(e);
            if (con != null) {
                stmt.close();
                ConnectionPool.getPool().releaseConnection(con);
            }
            throw e;
        }
    }

    /**
     * returns a list of Primary Investigators
     * 
     * @return list of PIs
     */
    public static int[] getUniquePIs() throws SQLException {

        // stdlog_.debug("EsoUsersDBIO.getUniquePIs()");
        Connection con = null;
        Statement stmt = null;
        try {
            Config cfg = Config.getCfg();
            con = ConnectionPool.getPool().acquireConnection(
                    cfg.getUrl(ObservationBlock.getSuffix()),
                    cfg.getDbUserName(), cfg.getDbPassword());
            stmt = con.createStatement();

            String sql = "select count (distinct user_id) from " + PRO
                    + " where pi_flag=0";
            stdlog_.info(sql);
            int[] intArray = null;
            ResultSet rs = stmt.executeQuery(sql);
            if (rs.next()) {
                int count = rs.getInt(1);
                stdlog_.debug("#PIs=" + count);
                rs.close();

                sql = "select distinct user_id from " + PRO
                        + " where pi_flag=0 order by user_id";
                stdlog_.info(sql);

                rs = stmt.executeQuery(sql);
                intArray = new int[count];
                int i = 0;
                while (rs.next()) {
                    int value = rs.getInt("user_id");
                    intArray[i++] = value;
                }
            }
            rs.close();
            stmt.close();
            ConnectionPool.getPool().releaseConnection(con);
            return intArray;

        } catch (SQLException e) {
            stdlog_.error(e);
            if (con != null) {
                stmt.close();
                ConnectionPool.getPool().releaseConnection(con);
            }
            throw e;
        }
    }

    /**
     * returns basic data for a given userID and cycle
     * 
     * @param userID
     *            userID of investigator
     * @param cycle
     *            cycle of interest
     * @return Investigator
     */
    public static Investigator getBasicInvestigatorInfo(int userID, String cycle)
            throws SQLException {

        // stdlog_.debug("EsoUsersDBIO.getBasicInvestigatorInfo");
        Connection con = null;
        Statement stmt = null;
        try {
            Config cfg = Config.getCfg();
            con = ConnectionPool.getPool().acquireConnection(
                    cfg.getUrl(ObservationBlock.getSuffix()),
                    cfg.getDbUserName(), cfg.getDbPassword());
            stmt = con.createStatement();

            String sql = "select distinct " + EU + ".surname, " + EU
                    + ".surname_dl, " + EU + ".item_name, " + EU + ".ignored, "
                    + UA + ".affiliation, " + UA + ".country_code, " + UA
                    + ".period" + " from " + EU + ", " + UA + ", " + CYC
                    + " where " + EU + ".id=" + userID + " and " + CYC
                    + ".label='" + cycle + "' and " + EU + ".id=" + UA
                    + ".user_id and " + CYC + ".id=" + UA + ".cycle_id";
            stdlog_.info(sql);

            ResultSet rs = stmt.executeQuery(sql);
            Investigator invInfo = new Investigator();
            if (rs.next()) {
                UserAffiliation ua = new UserAffiliation();
                ua.setUserId(userID);
                ua.setCountry(rs.getString("country_code"));
                ua.setCycle(cycle);
                ua.setPeriod(rs.getInt("period"));
                ua.setAffiliation(rs.getString("affiliation"));

                invInfo.affils_ = new Vector<UserAffiliation>();
                invInfo.affils_.addElement(ua);
                invInfo.setUserId(userID);
                invInfo.setInitials(rs.getString("item_name"));
                invInfo.setSurnameDL(rs.getString("surname_dl"));
                invInfo.setSurname(rs.getString("surname"));
                invInfo.setIgnored(rs.getInt("ignored"));
            } else {
                String s = "User " + userID + " does not exist in the database";
                throw new SQLException(s);
            }
            rs.close();
            stmt.close();
            ConnectionPool.getPool().releaseConnection(con);

            return invInfo;

        } catch (SQLException e) {
            stdlog_.error(e);
            if (con != null) {
                stmt.close();
                ConnectionPool.getPool().releaseConnection(con);
            }
            throw e;
        }
    }

    /**
     * returns a list of proposals associated with an investigator
     * 
     * @param userID
     *            userID of investigator
     * @param cycle
     *            cycle of interest
     * @param primary
     *            primary investigator or co-investigator
     * @return Vector of Proposals
     */
    public static Vector<Proposal> getInvestigatorInfo(int userID, String cycle,
            boolean primary) throws SQLException {

        // stdlog_.debug("EsoUsersDBIO.getInvestigatorInfo");
        Connection con = null;
        Statement stmt = null;
        try {
            Config cfg = Config.getCfg();
            con = ConnectionPool.getPool().acquireConnection(
                    cfg.getUrl(ObservationBlock.getSuffix()),
                    cfg.getDbUserName(), cfg.getDbPassword());
            stmt = con.createStatement();

            String sql = "select distinct " + OP + ".proposalID, " + OP
                    + ".title" + " from " + OP + ", " + PRO + ", " + EU + ", "
                    + CYC + " where " + EU + ".id=" + userID + " and " + CYC
                    + ".label='" + cycle + "' and " + EU + ".id=" + PRO
                    + ".user_id and " + OP + ".id=" + PRO
                    + ".programme_id and " + CYC + ".id=" + OP
                    + ".cycle_id and " + OP + ".ignored=0 and " + PRO
                    + ".pi_flag" + (primary ? "=0" : "!=0") + " order by " + OP
                    + ".proposalID";
            stdlog_.info(sql);

            ResultSet rs = stmt.executeQuery(sql);
            Vector<Proposal> invVec = new Vector<Proposal>();
            while (rs.next()) {
                Proposal prop = new Proposal();
                prop.proposalID_ = rs.getString("proposalID");
                prop.title_ = rs.getString("title");
                invVec.addElement(prop);
            }
            rs.close();
            stmt.close();
            ConnectionPool.getPool().releaseConnection(con);

            return invVec;

        } catch (SQLException e) {
            stdlog_.error(e);
            if (con != null) {
                stmt.close();
                ConnectionPool.getPool().releaseConnection(con);
            }
            throw e;
        }
    }

    /**
     * returns a list user_affiliations for a given investigator
     * 
     * @param userID
     *            userID
     * @return Vector of user_affiliations
     */
    public static Vector<UserAffiliation> getUserAffils(int userID) throws SQLException {

        // stdlog_.debug("EsoUsersDBIO.getUserAffils");
        Connection con = null;
        Statement stmt = null;
        try {
            Config cfg = Config.getCfg();
            con = ConnectionPool.getPool().acquireConnection(
                    cfg.getUrl(ObservationBlock.getSuffix()),
                    cfg.getDbUserName(), cfg.getDbPassword());
            stmt = con.createStatement();

            String sql = "select country_code, label, affiliation, " + UA
                    + ".period from " + UA + ", " + CYC + " where user_id="
                    + userID + " and " + UA + ".cycle_id=" + CYC + ".id"
                    + " order by label";
            stdlog_.info(sql);

            ResultSet rs = stmt.executeQuery(sql);
            Vector<UserAffiliation> uaVec = new Vector<UserAffiliation>();
            while (rs.next()) {
                UserAffiliation ua = new UserAffiliation();
                ua.setUserId(userID);
                ua.setCountry(rs.getString("country_code"));
                ua.setCycle(rs.getString("label"));
                ua.setPeriod(rs.getInt("period"));
                ua.setAffiliation(rs.getString("affiliation"));
                uaVec.addElement(ua);
            }
            rs.close();
            stmt.close();
            ConnectionPool.getPool().releaseConnection(con);

            return uaVec;

        } catch (SQLException e) {
            stdlog_.error(e);
            if (con != null) {
                stmt.close();
                ConnectionPool.getPool().releaseConnection(con);
            }
            throw e;
        }
    }

    public static StringBuffer mergeUsers(Investigator keepUser,
            int[] deleteUsers, Vector deleteUAs)
            throws SQLException, ObjectIOException {

        // stdlog_.debug("EsoUsersDBIO.mergeUsers()");
        StringBuffer sb = new StringBuffer();
        sb.append("keep eso_user: id=" + keepUser.getUserId() + lsep);
        int[] progList = DBdataHelper.getInstance().readIntArray(
                "SELECT programme_id from " + PRO + " \n where user_id = "
                        + keepUser.getUserId());
        Connection con = null;
        Statement stmt = null;

        try {
            Config cfg = Config.getCfg();
            con = ConnectionPool.getPool().acquireConnection(
                    cfg.getUrl(ObservationBlock.getSuffix()),
                    cfg.getDbUserName(), cfg.getDbPassword());
            con.setAutoCommit(false);
            stmt = con.createStatement();

            // get log of eso_users and user_affiliations
            logEsoUsers(keepUser.getUserId(), sb, stmt);
            logEsoUsersRoles(keepUser.getUserId(), sb, stmt);
            logUserAffiliations(keepUser.getUserId(), sb, stmt);
            logProposed(keepUser.getUserId(), sb, stmt);
            for (int i = 0; i < deleteUsers.length; i++) {
                int userid = deleteUsers[i];
                sb.append(lsep + "delete eso_user: id=" + userid + lsep);
                logEsoUsers(userid, sb, stmt);
                logEsoUsersRoles(userid, sb, stmt);
                logUserAffiliations(userid, sb, stmt);
                logProposed(userid, sb, stmt);
            }

            // create delStr

            String delStr = arrayIntToString(deleteUsers, "(", ")",
                    ", ");
            String allUsersStr = arrayIntToString(deleteUsers, "("+keepUser.getUserId()+", ", ")",
            ", ");

            String delStrEUR =  arrayIntToString(deleteUsers, "('", "')",
            "', '");
         
   

            // update tables
            for (int i = 0; i < deleteUAs.size(); i++) {
                UserAffiliation ua = (UserAffiliation) deleteUAs.elementAt(i);
                doStatement(con, "delete from " + UA + " where user_id="
                        + ua.getUserId() + " and cycle_id=(select id from "
                        + CYC + " where label='" + ua.getCycle() + "')");
            }
            for  (int j = 0; j < progList.length; j++) {
                int piMin = -1;
                String sqlPi = "select pi=min(pi_flag) from " + PRO + 
                " where user_id in "+allUsersStr+ " and programme_id = "+progList[j];
                stdlog_.info(sqlPi);
     
                ResultSet rs = stmt.executeQuery(sqlPi);
                if(rs.next()) {
                  piMin = rs.getInt(1); 
                }
                rs.close();
            for  (int i = 0; i < deleteUsers.length; i++) {     
                    doStatement(con, "delete from " + PRO + " where user_id = "
                            + deleteUsers[i]+ " and programme_id = "+progList[j]);
                    doStatement(con, "update " + PRO + " set pi_flag="
                            + piMin + " where user_id = " + keepUser.getUserId() + " and programme_id = "+progList[j]);
                }
           
            }
            
            doStatement(con, "update " + PRO + " set user_id=" +
                    keepUser.getUserId() + " where user_id in " + delStr);
            doStatement(con, "update " + UA + " set user_id="
                    + keepUser.getUserId() + " where user_id in " + delStr);
            doStatement(con, "update " + EU + " set surname_dl="
                    + SqlUtils.sqlEscapedString(keepUser.getSurnameDL())
                    + ", item_name="
                    + SqlUtils.sqlEscapedString(keepUser.getInitials())
                    + " where id=" + keepUser.getUserId());
            doStatement(con, "delete from " + EU + " where id in " + delStr);
            doStatement(con, "delete from " + EUR + " where login in "
                    + delStrEUR);

            // all updates succeeded
            con.commit();
            // stdlog_.debug("mergeUsers: success");

        } catch (SQLException e) {
            tryRollback(con, e);
            throw e;

        } finally {
            if (con != null) {
                stmt.close();
                try {
                    con.setAutoCommit(true);
                } catch (SQLException ex) {
                    stdlog_.error("Error setAutoCommit(true):"
                            + ex.getMessage());
                }
                ConnectionPool.getPool().releaseConnection(con);
            }
        }
        return sb;
    }

    /**
     * @param deleteUsers
     * @param closeBracket
     * @param delStr
     * @param delStr2
     * @return String
     */
    private static String arrayIntToString(int[] deleteUsers, String start, String end,String sep) {
        StringBuffer delStr =new StringBuffer();
        delStr.append(start).append(deleteUsers[0]);
        for (int i = 1; i < deleteUsers.length; i++) {
            delStr.append(sep).append(deleteUsers[i]);
        }

        delStr.append(end);
        return delStr.toString();
    }

    private static void logEsoUsers(int userID, StringBuffer sb, Statement stmt)
            throws SQLException {
        String sql = "select * from " + EU + " where id=" + userID;
        stdlog_.info(sql);
        sb.append(sql + lsep);
        ResultSet rs = stmt.executeQuery(sql);
        while (rs.next()) {
            sb.append("table eso_user" + ": id=" + userID + ", item_name="
                    + rs.getString("item_name") + ", surname="
                    + rs.getString("surname") + ", surname_dl="
                    + rs.getString("surname_dl") + ", ignored="
                    + rs.getInt("ignored") + ", referee_acronym="
                    + rs.getString("referee_acronym") + ", email="
                    + rs.getString("email") + ", panel="
                    + rs.getString("panel") + ", chair=" + rs.getInt("chair")
                    + ", item_password=" + rs.getString("item_password")
                    + ", last_mod=" + rs.getInt("last_mod") + ", opc_subpanel="
                    + rs.getInt("opc_subpanel") + ", opc_member="
                    + rs.getInt("opc_member") + lsep);
        }
        rs.close();
    }

    private static void logEsoUsersRoles(int userID, StringBuffer sb,
            Statement stmt) throws SQLException {
        String sql = "select * from " + EUR + " where login='" + userID + "'";
        stdlog_.info(sql);
        sb.append(sql + lsep);
        ResultSet rs = stmt.executeQuery(sql);
        while (rs.next()) {
            sb.append("table eso_users_roles" + ": login="
                    + rs.getString("login") + ", letters_role="
                    + rs.getString("letters_role") + lsep);
        }
        rs.close();
    }
    
   

    private static void logUserAffiliations(int userID, StringBuffer sb,
            Statement stmt) throws SQLException {
        String sql = "select * from " + UA + " where user_id=" + userID;
        stdlog_.info(sql);
        sb.append(sql + lsep);
        ResultSet rs = stmt.executeQuery(sql);
        while (rs.next()) {
            sb.append("table user_affiliation" + ": user_id=" + userID
                    + ", period=" + rs.getInt("period") + ", country_code="
                    + rs.getString("country_code") + ", affiliation="
                    + rs.getString("affiliation") + ", cycle_id="
                    + rs.getInt("cycle_id") + ", accessed_letter="
                    + rs.getFloat("accessed_letter") + lsep);
        }
        rs.close();
    }

    private static void logProposed(int userID, StringBuffer sb, Statement stmt)
            throws SQLException {
        String sql = "select * from " + PRO + " where user_id=" + userID;
        stdlog_.info(sql);
        sb.append(sql + lsep);
        ResultSet rs = stmt.executeQuery(sql);
        while (rs.next()) {
            sb.append("table proposed" + ": user_id=" + userID
                    + ", programme_id=" + rs.getInt("programme_id")
                    + ", pi_flag=" + rs.getInt("pi_flag") + lsep);
        }
        rs.close();
    }

    private static void doStatement(Connection con, String sql)
            throws SQLException {
        stdlog_.info(sql);
        Statement stmt = con.createStatement();
        stmt.executeUpdate(sql);
        stmt.close();
    }

    private static void tryRollback(Connection con, Exception e) {
        stdlog_.error("caught exception: " + e.toString());
        try {
            stdlog_.debug("doing rollback ...");
            con.rollback();
            stdlog_.debug("rollback done");
        } catch (SQLException ex) {
            stdlog_.error("Error rolling back: ", ex);
        }
    }

    /**
     * find out whether a userID has obs_blocks
     * 
     * @param userID
     *            userID
     * @return boolean
     * @throws ObjectIOException
     */
    public static boolean hasObsBlocks(int userID)
            throws SQLException, ObjectIOException {

        boolean result = false;
        int numObsBlocks = DBdataHelper.getInstance().readIntValue(
                Phase1SelectStmt.countObsBlocks(userID));
        if (numObsBlocks > 0) {
            result = true;
        }
        return result;
    }

    /**
     * find out whether a user is a referee
     * 
     * @param userID
     *            userID
     * @return boolean
     * @throws ObjectIOException
     */
    public static boolean isReferee(int userID)
            throws SQLException, ObjectIOException {

        boolean result = false;
        int numReferee = DBdataHelper.getInstance().readIntValue(
                Phase1SelectStmt.countReferee(userID));
        if (numReferee > 0) {
            result = true;
        }
        return result;
    }
}

