package org.eso.ohs.persistence.dbase.phase2;

import java.io.IOException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.Media;
import org.eso.ohs.persistence.ObjectManager;
import org.eso.ohs.persistence.ObjectNotFoundException;

/**
 * I/O class for Obervation Block that belong to an execution sequence
 * of a Queue object
 * @author Giorgio Passera <DMD\ESO> <gpassera@eso.org>
 */

public class ExecutionSequenceDBIO {

   /** Standard log channel */ 
   private static Logger stdlog_ =
       Logger.getLogger( ExecutionSequenceDBIO.class );



    /**    
     * Read the exec_sequence table from the DBQ
     * @param stmt Sql statement
     * @param queueId the Id of a Queue
     * @exception SQLException
     *		    an error occurred during the dbase operation   
     * @exception IOException
     *            if an I/O error occurred while reading the object
     * @exception ObjectIOException IO Eror occured while reading from 
     *            the database
     *
     * @exception ObjectNotFoundException if the ObservationBlock is not found
     * @return a vector of Observation block objects
     */

    public static Vector<ObservationBlock> read(Statement stmt,
				               long queueId)
				throws ObjectIOException,
					IOException, SQLException ,
                                        ObjectNotFoundException {
	Vector<ObservationBlock> v = new Vector<ObservationBlock>();
	
	ResultSet rs = null;

	String sqlText = "SELECT * FROM exec_sequences"
		+ " WHERE sch_id = "  + queueId +
                " order by sequence" ;
	

	stdlog_.debug(sqlText);
	rs = stmt.executeQuery(sqlText);
        long ob_id;
        int  sequence;
       
        
       
	while (rs.next()) 
            {
	    ob_id = Config.getCfg().tableToUniqueId(rs.getInt("ob_id"),
		    ObservationBlock.class);
	    
	    sequence = rs.getInt("sequence");
          
	    ObjectManager objMgr = ObjectManager.getObjectManager();

            ObservationBlock ob = (ObservationBlock)objMgr.getBusObj(
					Media.DBASE,
					ob_id,
					ObservationBlock.class);	   
                
            v.addElement(ob);
            }                                
        rs.close();
        return v;        
        }
            
        


    /**    
     * Write data inot the exec_sequences table
     * @param stmt Sql statement
     * @param queueItemList a vector of QueueItem Objects to be written
     * @exception SQLException
     *		    an error occurred during the dbase operation   
     */

    public static void write(Statement stmt, long schId,
				Vector list)
					throws SQLException {
                                               
	StringBuffer sb = new StringBuffer();
	Config cfg = Config.getCfg();


	for (int i=0; i< list.size(); i++) 
           {
	   sb.setLength(0);    
           ObservationBlock crrOB = (ObservationBlock)list.elementAt(i);
	   sb.append(schId);
	   sb.append(",");
	   //sb.append(crrItem.getOB().getId());
	   sb.append(cfg.uniqueToTableId(crrOB.getId()));
	   sb.append(",");
	   sb.append(i);
           
           String sqlText = "INSERT INTO exec_sequences"
			 + " (sch_id, ob_id, sequence)"
			 + " VALUES (" + sb + ")";
	   stdlog_.debug(sqlText);
	   stmt.executeUpdate(sqlText);
           }
        }

	   

    /**
     * Update the exec sequence table
     * @param Sql statement
     * @param queueId the id of a Queue object
     * @param items a vector of QueueItem(s)
     * @exception SQLException
     *		    an error occurred during the dbase operation 
     */

    public static void update(Statement stmt, long queueId,
				 Vector items)
					throws SQLException {
                                                
	delete(stmt, queueId);
	write(stmt, queueId, items);
        
    }

    /**
     * Delete QueueItem(s) that belong to a Queue
     * @param Sql statement
     * @param queueId an Id for a Queue
     * 
     * @exception SQLException
     *		    an error occurred during the dbase operation   
     */

    public static void delete(Statement stmt, long queueId)
					throws SQLException {
                                             

	String sqlText = "DELETE FROM exec_sequences"
	    + " WHERE sch_id = " + queueId ;
	
	stdlog_.debug(sqlText);
	stmt.executeUpdate(sqlText);

	   
    }

}


