package org.eso.ohs.persistence.dbase.phase2;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.MaskQueue;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.Queue;
import org.eso.ohs.dfs.Summary;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.DirectoryNode;

/**
 * The storage manager class for files stored in the database.
 * A concrete subclass of StorageManager.
 * <p>
 * NOTE: we could not instantiate the Sybase driver in the normal way, using
 * Class.forName().  Instead it is necessary to follow the special instruction
 * in the Sybase JDBC driver installation instructions.  This is to update the
 * Java system properties.
 * <br>
 * @version $Revision: 1.2.4.1 $
 * @author Bob Kemp
 */
public class OTDbaseMgr extends DbaseMgr {

    public static final String cvsID_ =
        "$Id: OTDbaseMgr.java,v 1.2.4.1 2007/10/05 11:03:38 pnunes Exp $";

    /** Standard log channel */
    private static Logger stdlog_ = Logger.getLogger(OTDbaseMgr.class);

    private static String obrepDbName_ =
        Config.getCfg().getDbName(ObservationBlock.getSuffix());


    /**
     * Default constructor
     */
    public OTDbaseMgr() {
        super();
    }


    /**
     * Builds a list of Summary objects for a given directory.
     * These Summary objects contain the unique Id of the object
     * and the Class which contains details about the type of Object
     * that is being referenced be it an Observation Description,Observation
     * Block etc..
     *
     * @param node Directory from which you can create a list of objects
     * @exception ObjectIOException if there was an I/O error listing an object
     * @return Array of ObjectFileInfo
     */
    public Summary[] listObjects(DirectoryNode node) throws ObjectIOException {

        Vector<Summary> vec = new Vector<Summary>();
        Connection con = null;
        Statement stmt = null;

        try {
            con = acquireConnection();
            stmt = con.createStatement();
            stdlog_.debug("URL: " + con.getMetaData().getURL());

            String sqlText = null;
            Config cfg = Config.getCfg();

            if (cfg.isSuperUser()) {
                sqlText = "select " + obrepDbName_ + "..schedules.sch_id, " +
                    obrepDbName_+ "..schedules.item_name, " +
                    obrepDbName_ + "..schedules.tel_code, " +
                    obrepDbName_ + "..schedules.schedule_type, " +
                    "(SELECT count(*) from " +
                    obrepDbName_ + "..schedule_items where " +
                    obrepDbName_ + "..schedules.sch_id " + " = " +
                    obrepDbName_ + "..schedule_items.sch_id) AS num_items from " +
                    obrepDbName_ + "..schedules order by upper(item_name) ";
            } else {
                sqlText = "select " + obrepDbName_ + "..schedules.sch_id, " +
                    obrepDbName_ + "..schedules.item_name, " +
                    obrepDbName_ + "..schedules.tel_code, " +
                    obrepDbName_ + "..schedules.schedule_type, " +
                    "(SELECT count(*) from " +
                    obrepDbName_ + "..schedule_items where " +
                    obrepDbName_ + "..schedules.sch_id " + " = " +
                    obrepDbName_ + "..schedule_items.sch_id) AS num_items from " +
                    obrepDbName_ + "..schedules " +
                    "WHERE " + obrepDbName_ + "..schedules.user_id = " +
                    cfg.getAccountId() + " order by upper(item_name)";
           }
           stdlog_.debug(sqlText);

           Summary sum = null;
           ResultSet rs = stmt.executeQuery(sqlText);
           while (rs.next()) {
               sum = new Summary();
               Class c = Queue.class;
               String schType = rs.getString("schedule_type");
               if ("M".equals(schType))
                   c = MaskQueue.class;
               sum.setPropertyValue("QueueId",new Long(Config.getCfg().tableToUniqueId(rs.getInt("sch_id"),c)));
               sum.setPropertyValue("Name",rs.getString("item_name"));
               sum.setPropertyValue("Telescope",rs.getString("tel_code"));
               sum.setObjectClass(c);
               sum.setPropertyValue("Size",new Integer(rs.getInt("num_items")));
               vec.addElement(sum);
           }
           rs.close();

        } catch (SQLException e) {
            e.printStackTrace();
            throw new ObjectIOException(e);

        } finally {
            try {
                if (stmt != null) {
                    stmt.close();
                }
            } catch (SQLException e) {
                e.printStackTrace();
            } finally {
                if (con != null) {
                    releaseConnection(con);
                }
            }
        }

        Summary[] sums = new Summary[vec.size()];
        vec.copyInto(sums);
        return sums;
    }
}

