package org.eso.ohs.persistence.dbase.phase2;

import java.io.IOException;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.ConnectionPool;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.core.utilities.TextUtils;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.QueueItem;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.ObjectNotFoundException;

/**
 * I/O class for QueueItems (schedule_items) stored in the database.
 * 
 * @version $Revision: 1.5.4.1 $
 * @author Giorgio Passera <DMD\ESO> <gpassera@eso.org>
 */

public class QueueItemDBIO {

    /** Standard log channel */
    private static Logger stdlog_ = Logger.getLogger(QueueItemDBIO.class);

    private static final String rcsid = "$Id: QueueItemDBIO.java,v 1.5.4.1 2007/10/24 12:36:04 pnunes Exp $";

    private static String obrepDbName_ = Config.getCfg().getDbName(
            ObservationBlock.getSuffix());

    /**
     * Reads QueueItem(s) from database
     * 
     * @param stmt
     *            Sql statement
     * @param queueId
     *            the Id of a Queue
     * @exception SQLException
     *                an error occurred during the dbase operation
     * @exception IOException
     *                if an I/O error occurred while reading the object
     * @exception ObjectIOException
     *                IO Eror occured while reading from the database
     * 
     * @exception ObjectNotFoundException
     *                if the ObservationBlock is not found
     * @return a vector of QueueItem objects
     */

    public static QueueItem[] read(Statement stmt, long queueId)
            throws ObjectIOException, IOException, SQLException,
            ObjectNotFoundException {
        Vector<QueueItem> v = new Vector<QueueItem>();

        ResultSet rs = null;

        String sqlText = "SELECT " + obrepDbName_ + ".."
                + DbaseHandlerQueue.SCHEDULE_ITEMS + ".*," + obrepDbName_
                + "..obs_blocks.type FROM " + obrepDbName_ + ".."
                + DbaseHandlerQueue.SCHEDULE_ITEMS + "," + obrepDbName_
                + "..obs_blocks" + " WHERE " + obrepDbName_ + ".."
                + DbaseHandlerQueue.SCHEDULE_ITEMS + " .sch_id = " + queueId
                + " AND " + obrepDbName_ + ".."
                + DbaseHandlerQueue.SCHEDULE_ITEMS + " .ob_id = "
                + obrepDbName_ + "..obs_blocks.ob_id" + " order by "
                + obrepDbName_ + ".." + DbaseHandlerQueue.SCHEDULE_ITEMS
                + ".sequence";

        stdlog_.debug(sqlText);
        rs = stmt.executeQuery(sqlText);
        long ob_id;
        int sequence;
        int startTime;
        int endTime;
        String type;
        // stdlog_.debug ("size " + rs.getFetchSize());
        while (rs.next()) {
            ob_id = rs.getInt("ob_id");
            sequence = rs.getInt("sequence");
            startTime = rs.getInt("start_time");
            endTime = rs.getInt("end_time");
            type = rs.getString("type");
            QueueItem item = new QueueItem();
            item.setSequence(sequence);
            item.setStartTime(startTime);
            item.setEndTime(endTime);
            if (type.equals(CalibrationBlock.OBSBLK_TYPE))
                item.setObId(Config.getCfg().tableToUniqueId(ob_id,
                        ObservationBlock.class));
            else
                item.setObId(Config.getCfg().tableToUniqueId(ob_id,
                        CalibrationBlock.class));
            item.setType(type);
            v.addElement(item);
        }
        rs.close();
        QueueItem[] queueItems = new QueueItem[v.size()];
        v.copyInto(queueItems);
        return queueItems;
    }

    /**
     * Write a vector of QueueItem objects to the DB
     * 
     * @param stmt
     *            Sql statement
     * @param queueItemList
     *            a vector of QueueItem Objects to be written
     * @exception SQLException
     *                an error occurred during the dbase operation
     */

    public static void write(Statement stmt, long schId,
            QueueItem[] queueItemList) throws SQLException {

        StringBuffer sb = new StringBuffer();
        Config cfg = Config.getCfg();

        if (queueItemList != null && queueItemList.length > 0) {
            
            for (int i = 0; i < queueItemList.length; i++) {
            	StringBuffer sqlText = new StringBuffer();
                sb.setLength(0);
                QueueItem crrItem = queueItemList[i];
                sb
                        .append(schId)
                        .append(",")
                        .append(
                                Config.getCfg().uniqueToTableId(
                                        crrItem.getObId()))
                        .append(",")
                        .append(i)
                        .append(",'")
                        .append(TextUtils.quoteSqlString(crrItem.getRank()))
                        .append("',")
                        .append(crrItem.getStartTime())
                        .append(",")
                        .append(crrItem.getEndTime());

                sqlText
                        .append(" INSERT INTO ")
                        .append(obrepDbName_)
                        .append("..")
                        .append(DbaseHandlerQueue.SCHEDULE_ITEMS)
                        .append(" ( ")
                        .append(obrepDbName_)
                        .append("..")
                        .append(DbaseHandlerQueue.SCHEDULE_ITEMS)
                        .append(".sch_id, ")
                        .append(obrepDbName_)
                        .append("..")
                        .append(DbaseHandlerQueue.SCHEDULE_ITEMS)
                        .append(".ob_id, ")
                        .append(obrepDbName_)
                        .append("..")
                        .append(DbaseHandlerQueue.SCHEDULE_ITEMS)
                        .append(".sequence, ")
                        .append(obrepDbName_)
                        .append("..")
                        .append(DbaseHandlerQueue.SCHEDULE_ITEMS)
                        .append(".rank, ")
                        .append(obrepDbName_)
                        .append("..")
                        .append(DbaseHandlerQueue.SCHEDULE_ITEMS)
                        .append(".start_time, ")
                        .append(obrepDbName_)
                        .append("..")
                        .append(DbaseHandlerQueue.SCHEDULE_ITEMS)
                        .append(".end_time)")
                        .append(" VALUES (")
                        .append(sb.toString())
                        .append(")\n");
                stdlog_.info(sqlText.toString());
                stmt.addBatch(sqlText.toString());
            }
           
        }
        stmt.executeBatch();
    }

    /**
     * Update QueueItem(s)
     * 
     * @param Sql
     *            statement
     * @param queueId
     *            the id of a Queue object
     * @param items
     *            a vector of QueueItem(s)
     * @exception SQLException
     *                an error occurred during the dbase operation
     */

    public static void update(Statement stmt, long queueId, QueueItem[] items)
            throws SQLException {

        delete(stmt, queueId, "schedule_items");
        write(stmt, queueId, items);

    }

    /**
     * Delete QueueItem(s) that belong to a Queue
     * 
     * @param Sql
     *            statement
     * @param queueId
     *            an Id for a Queue
     * 
     * @exception SQLException
     *                an error occurred during the dbase operation
     */

    public static void delete(Statement stmt, long queueId, String table)
            throws SQLException {

        String sqlText = "DELETE FROM " + table + " WHERE " + table
                + ".sch_id = " + queueId;

        stdlog_.info(sqlText);
        stmt.executeUpdate(sqlText);
    }

    public static void removeQueueItems(long queueId, long[] obIds)
            throws SQLException {

        Config cfg = Config.getCfg();
        Connection con = ConnectionPool.getPool().acquireConnection(
                cfg.getUrl(ObservationBlock.getSuffix()), cfg.getDbUserName(),
                cfg.getDbPassword());
        Statement stmt = con.createStatement();
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < obIds.length; i++) {
            sb.append(cfg.uniqueToTableId(obIds[i]));
            if (i < obIds.length - 1)
                sb.append(",");
        }

        String sqlText = "DELETE from " + obrepDbName_ + ".."
                + DbaseHandlerQueue.TMP_SCHEDULE_ITEMS + " WHERE "
                + obrepDbName_ + ".." + DbaseHandlerQueue.TMP_SCHEDULE_ITEMS
                + ".ob_id = IN ( + sb.toString() + ) " + " AND " + obrepDbName_
                + ".." + DbaseHandlerQueue.TMP_SCHEDULE_ITEMS + ".sch_id = "
                + cfg.uniqueToTableId(queueId);
        stdlog_.info(sqlText);
        stmt.executeUpdate(sqlText);
    }

    /**
     * Reads QueueItem(s) from database
     * 
     * @param stmt
     *            Sql statement
     * @param queueId
     *            the Id of a Queue
     * @exception SQLException
     *                an error occurred during the dbase operation
     * @exception IOException
     *                if an I/O error occurred while reading the object
     * @exception ObjectIOException
     *                IO Eror occured while reading from the database
     * 
     * @exception ObjectNotFoundException
     *                if the ObservationBlock is not found
     * @return a vector of QueueItem objects
     */

    public static QueueItem[] readExecSeq(Statement stmt, long queueId)
            throws ObjectIOException, IOException, SQLException,
            ObjectNotFoundException {
        Vector<QueueItem> v = new Vector<QueueItem>();

        ResultSet rs = null;

        String sqlText = "SELECT " + obrepDbName_ + ".."
                + DbaseHandlerMaskQueue.EXEC_SEQ + ".*," + obrepDbName_
                + "..obs_blocks.type FROM " + obrepDbName_ + ".."
                + DbaseHandlerMaskQueue.EXEC_SEQ + "," + obrepDbName_
                + "..obs_blocks" + " WHERE " + obrepDbName_ + ".."
                + DbaseHandlerMaskQueue.EXEC_SEQ + " .sch_id = " + queueId
                + " AND " + obrepDbName_ + ".."
                + DbaseHandlerMaskQueue.EXEC_SEQ + " .ob_id = " + obrepDbName_
                + "..obs_blocks.ob_id" + " order by " + obrepDbName_ + ".."
                + DbaseHandlerMaskQueue.EXEC_SEQ + ".sequence";

        stdlog_.info(sqlText);
        rs = stmt.executeQuery(sqlText);
        long ob_id;
        int sequence;
        String type;

        // System.out.println ("size " + rs.getFetchSize());
        while (rs.next()) {
            ob_id = rs.getInt("ob_id");
            sequence = rs.getInt("sequence");
            type = rs.getString("type");
            QueueItem item = new QueueItem();
            item.setSequence(sequence);
            if (type.equals(CalibrationBlock.OBSBLK_TYPE))
                item.setObId(Config.getCfg().tableToUniqueId(ob_id,
                        ObservationBlock.class));
            else
                item.setObId(Config.getCfg().tableToUniqueId(ob_id,
                        CalibrationBlock.class));
            v.addElement(item);
        }
        rs.close();
        QueueItem[] queueItems = new QueueItem[v.size()];
        v.copyInto(queueItems);
        return queueItems;
    }

    /**
     * Write a vector of QueueItem objects to the DB
     * 
     * @param stmt
     *            Sql statement
     * @param queueItemList
     *            a vector of QueueItem Objects to be written
     * @exception SQLException
     *                an error occurred during the dbase operation
     */

    public static void writeExecSeq(Statement stmt, long schId,
            QueueItem[] queueItemList) throws SQLException {

        StringBuffer sb = new StringBuffer();
        Config cfg = Config.getCfg();

        if (queueItemList == null)
            return;

        for (int i = 0; i < queueItemList.length; i++) {
            sb.setLength(0);
            QueueItem crrItem = queueItemList[i];
            sb.append(schId);
            sb.append(",");
            // sb.append(crrItem.getOB().getId());
            sb.append(Config.getCfg().uniqueToTableId(crrItem.getObId()));
            sb.append(",");
            sb.append(i);

            StringBuffer sqlText = new StringBuffer()
                    .append("INSERT INTO ")
                    .append(obrepDbName_)
                    .append("..")
                    .append(DbaseHandlerMaskQueue.EXEC_SEQ)
                    .append(" ( ")
                    .append(obrepDbName_)
                    .append("..")
                    .append(DbaseHandlerMaskQueue.EXEC_SEQ)
                    .append(".sch_id, ")
                    .append(obrepDbName_)
                    .append("..")
                    .append(DbaseHandlerMaskQueue.EXEC_SEQ)
                    .append(".ob_id, ")
                    .append(obrepDbName_)
                    .append("..")
                    .append(DbaseHandlerMaskQueue.EXEC_SEQ)
                    .append(".sequence)")
                    .append(" VALUES (")
                    .append(sb)
                    .append(")");
            stdlog_.info(sqlText.toString());
            stmt.executeUpdate(sqlText.toString());
        }

    }
}
