package org.eso.ohs.persistence.dbase.phase2;

import java.io.IOException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.ohs.core.utilities.ConnectionPool;
import org.eso.ohs.core.utilities.OHSKeys;
import org.eso.ohs.core.utilities.ObjectIOException;
import org.eso.ohs.dfs.BusinessObject;
import org.eso.ohs.dfs.CalibrationBlock;
import org.eso.ohs.dfs.ConstraintSetDTO;
import org.eso.ohs.dfs.ObservationBlock;
import org.eso.ohs.dfs.ObservingRun;
import org.eso.ohs.dfs.USDReadmeDatails;
import org.eso.ohs.dfs.USGLog;
import org.eso.ohs.persistence.AppConfig;
import org.eso.ohs.persistence.Config;
import org.eso.ohs.persistence.ObjectNotFoundException;
import org.eso.ohs.persistence.dbase.DbaseIOHandler;
import org.jdom.JDOMException;

/**
 * Class ReadmeFileDBIO This class deals with reading and writing og the readme
 * file to the database. The readmefile is converted from the ReadmeAttribute
 * data structure to an XML file in the database. With each change that is made
 * to the system it records a history of changes that occurs. TODO Javadoc
 * 
 * @version $Revision: 1.4 $
 * @author tcanavan, Nov 8, 2004
 */

// $Id: USDFileDBIO.java,v 1.4 2006/07/07 09:44:57 tcanavan Exp $
public class USDFileDBIO {

    /** Standard log channel */
    private static Logger stdlog_ = Logger.getLogger(ReadmeFileDBIO.class);

    private static String obrepDbName = Config.getCfg().getDbName(
            ObservationBlock.getSuffix());

    private static String opcDbName = Config.getCfg().getDbName(
            ObservingRun.getSuffix());

    public static String usgFile = obrepDbName + "..usg_file";

    private static String CS = obrepDbName + "..constraint_sets";

    public static String usgSupportNotes = obrepDbName + "..usg_support_notes";

    public static String usgFileReplicate = obrepDbName
            + "..usg_file_replicate";

    public static String OB = obrepDbName + "..obs_blocks";

    private static String obs_runs = opcDbName + "..obs_runs";

    private static String proposed = opcDbName + "..proposed";

    private static String user_affialiations = opcDbName
            + "..user_affiliations";

    /**
     * Read readme file from the database.
     * 
     * @param conn
     * @param boId
     * @throws ObjectIOException
     * @throws SQLException
     * @throws IOException
     * @throws JDOMException
     * @throws IOException
     * @throws ObjectNotFoundException
     */

    public static USDReadmeDatails read(Connection conn, long runId)
            throws SQLException {
        setTextSize(conn);
        USDReadmeDatails details = new USDReadmeDatails();
        ResultSet rs;
        //long dbId = Config.getCfg().uniqueToTableId(runId);
        String sqlText = "SELECT " + usgFile + ".review_status, " + usgFile
                + ".special_remarks, " + usgFile + ".mts_abstract, " + usgFile
                + ".pre_imaging, " + usgFile + ".special_cal," + usgFile
                + ".time_criti," + usgFile + ".carryover_not_derived from "
                + usgFile + "," + ReadmeFileDBIO.README_FILE + " where "
                + usgFile + ".run_id = ? AND " + usgFile + ".run_id = "
                + ReadmeFileDBIO.README_FILE + ".run_id";
        System.out.println(sqlText);
        PreparedStatement pstmt = conn.prepareStatement(sqlText);
        pstmt.setLong(1, runId);
        rs = pstmt.executeQuery();
        boolean found = false;
        while (rs.next()) {
            String reviewStatus = rs.getString("review_status");
            details.setReviewStatus(rs.getString("review_status").trim());
            String specialRemarks = rs.getString("special_remarks");
            if (specialRemarks != null)
                details.setSpecialComments(specialRemarks.trim());
            String mtsAbstract = rs.getString("mts_abstract");
            if (mtsAbstract != null)
                details.setMtsAbstract(mtsAbstract.trim());
            details.setPreImaging(rs.getBoolean("pre_imaging"));
            details.setSpecialCal(rs.getBoolean("special_cal"));
            details.setTimeCritical(rs.getBoolean("time_criti"));
            details.setCarryOver(rs.getBoolean("carryover_not_derived"));
            found = true;
        }
        rs.close();
        pstmt.close();
        if (!found)
            throw new SQLException(
                    "USD file not available for this Observing Run");
        if (AppConfig.getAppConfig().getBoolean(OHSKeys.USGEnabledKey)) {
            sqlText = "SELECT " + usgSupportNotes + ".log_time,"
                    + usgSupportNotes + ".support_history" + " from "
                    + usgSupportNotes + " where " + usgSupportNotes
                    + ".run_id = ?";
            pstmt = conn.prepareStatement(sqlText);
            pstmt.setLong(1, runId);
            rs = pstmt.executeQuery();
            Vector<USGLog> vec = new Vector<USGLog>();
            while (rs.next()) {
                USGLog usgLog = new USGLog();
                usgLog.setLogMessage(rs.getString("support_history"));
                usgLog.setLogTime(rs.getLong("log_time"));
                vec.addElement(usgLog);
            }
            USGLog[] logArray = new USGLog[vec.size()];
            vec.copyInto(logArray);
            details.setUsgLog(logArray);
            details.setUsdStaffName(getUSDStaffName(conn, runId));
        }
        totalCalAndOB(conn, details, runId);
        details.setChilean(isChileanFlag(conn, runId));
        details.setCsDtos(getConstraintSet(conn, runId));
        return details;
    }

    /**
     * Read readme file from the database.
     * 
     * @param conn
     * @param boId
     * @throws ObjectIOException
     * @throws SQLException
     * @throws IOException
     * @throws JDOMException
     * @throws IOException
     * @throws ObjectNotFoundException
     */

    public static USDReadmeDatails readReplicate(Connection conn, long runId)
            throws SQLException {
        setTextSize(conn);
        USDReadmeDatails details = new USDReadmeDatails();
        ResultSet rs;
        //long dbId = Config.getCfg().uniqueToTableId(runId);
        String sqlText = "SELECT " + usgFile + ".review_status, " + usgFile
                + ".special_remarks, " + usgFile + ".mts_abstract, " + usgFile
                + ".pre_imaging, " + usgFile + ".special_cal," + usgFile
                + ".paranal_notes," + usgFile + ".time_criti," + usgFile
                + ".carryover_not_derived from " + usgFile + ","
                + ReadmeFileDBIO.README_REPLICATE + " where " + usgFile + ".run_id = ? AND "
                + usgFile + ".run_id = " + ReadmeFileDBIO.README_REPLICATE + ".run_id";
        PreparedStatement pstmt = conn.prepareStatement(sqlText);
        pstmt.setLong(1, runId);
        rs = pstmt.executeQuery();
        boolean found = false;
        while (rs.next()) {
            String reviewStatus = rs.getString("review_status");
            details.setReviewStatus(rs.getString("review_status").trim());
            String specialRemarks = rs.getString("special_remarks");
            if (specialRemarks != null)
                details.setSpecialComments(specialRemarks.trim());
            String mtsAbstract = rs.getString("mts_abstract");
            if (mtsAbstract != null)
                details.setMtsAbstract(mtsAbstract.trim());
            String notes = rs.getString("paranal_notes");
            if (notes != null)
                details.setObservatoryNotes(notes.trim());
            details.setPreImaging(rs.getBoolean("pre_imaging"));
            details.setSpecialCal(rs.getBoolean("special_cal"));
            details.setTimeCritical(rs.getBoolean("time_criti"));
            details.setCarryOver(rs.getBoolean("carryover_not_derived"));

            found = true;
        }
        rs.close();
        pstmt.close();
        totalCalAndOB(conn, details, runId);
        //details.setUsdStaffName(getUSDStaffName(conn, runId));
        details.setChilean(isChileanFlag(conn, runId));
        details.setCsDtos(getConstraintSet(conn, runId));
        if (!found)
            throw new SQLException(
                    "USD file not available for this Observing Run");
        return details;
    }

    public static void totalCalAndOB(Connection conn, USDReadmeDatails details,
            long dbId) throws SQLException {
        String sqlText = "SELECT count(*) from " + OB + " WHERE " + OB
                + ".run_id = ?" + " AND " + OB + ".type = ?";
        PreparedStatement pstmt = conn.prepareStatement(sqlText);
        pstmt.setLong(1, dbId);
        pstmt.setString(2, CalibrationBlock.CALBLK_TYPE);
        ResultSet rs = pstmt.executeQuery();
        int count = -1;
        while (rs.next()) {
            count = rs.getInt(1);
        }
        details.setTotalCBS(count);
        rs.close();
        pstmt.close();
        pstmt = conn.prepareStatement(sqlText);
        pstmt.setLong(1, dbId);
        pstmt.setString(2, CalibrationBlock.OBSBLK_TYPE);
        count = -1;
        rs = pstmt.executeQuery();
        while (rs.next()) {
            count = rs.getInt(1);
        }
        details.setTotalOBs(count);
        rs.close();
        pstmt.close();
    }
    

    /**
     * Update the observatory notes. This will be included after 
     * the mts abstract in the mountain view of the readme/usd file. 
     * @param observatoryNotes
     * @throws ObjectIOException
     */
    

    public static void updateObservatoryNotes(String observatoryNotes,long id) throws ObjectIOException {
        Connection con = null;
        Statement stmt = null;
        Config cfg = Config.getCfg();
        SQLException error = null;
        try {
            con = ConnectionPool.getPool().acquireConnection(
                    cfg.getUrl(ObservationBlock.getSuffix()),
                    cfg.getDbUserName(), cfg.getDbPassword());

            String sqlText = "UPDATE " + usgFile + " set " + usgFile
                    + ".paranal_notes = ?" + " where " + usgFile
                    + ".run_id = ?";
            PreparedStatement ps = con.prepareStatement(sqlText);
            ps.setString(1,observatoryNotes);
            ps.setLong(2,id);
            ps.execute();
            ps.close();
            stdlog_.debug(sqlText + " " + id + " " + observatoryNotes);
        } catch (SQLException e) {
            error = e;
        } finally {
            try {
                QueueManager.closeConnection(con, stmt, error);
            } catch (ObjectIOException e1) {
                throw new ObjectIOException(e1);
            }
        }
    }

    public static void write(Connection conn, USDReadmeDatails usdDetails,
            long runId) throws ObjectIOException {
        ObjectIOException error = null;
        try {
            long dbId = Config.getCfg().uniqueToTableId(runId);
            USGLog[] usgLog = usdDetails.getUsgLog();
            long[] logIds = new long[usgLog.length];
            for (int i = 0; i < logIds.length; i++) {
                logIds[i] = DbaseIOHandler.getDbId(conn);
            }
            setTextSize(conn);
            ResultSet rs;
            PreparedStatement pstmt;
            String sqlText = "UPDATE " + usgFile + " set " + usgFile
                    + ".review_status = ?, " + usgFile
                    + ".special_remarks = ? , " + usgFile
                    + ".mts_abstract = ? , " + usgFile + ".pre_imaging = ?, "
                    + usgFile + ".special_cal= ? ," + usgFile
                    + ".time_criti = ? ," + usgFile
                    + ".carryover_not_derived = ? " + " where " + usgFile
                    + ".run_id = ?";
            stdlog_.debug(sqlText);
            pstmt = conn.prepareStatement(sqlText);
            pstmt.setString(1, usdDetails.getReviewStatus());
            pstmt.setString(2, usdDetails.getSpecialComments());
            pstmt.setString(3, usdDetails.getMtsAbstract());
            pstmt.setBoolean(4, usdDetails.isPreImaging());
            pstmt.setBoolean(5, usdDetails.isSpecialCal());
            pstmt.setBoolean(6, usdDetails.isTimeCritical());
            pstmt.setBoolean(7, usdDetails.isCarryOver());
            pstmt.setLong(8, dbId);
            pstmt.execute();
            pstmt.close();
            sqlText = " DELETE from " + usgSupportNotes + " where "
                    + usgSupportNotes + ".run_id = ?";
            pstmt = conn.prepareStatement(sqlText);
            pstmt.setLong(1, dbId);
            pstmt.execute();
            pstmt.close();
            for (int i = 0; i < usgLog.length; i++) {
                sqlText = " INSERT INTO "
                        + usgSupportNotes
                        + " (run_id,usg_support_notes_id,support_history,log_time)"
                        + " VALUES (?,?,?,?)";
                pstmt = conn.prepareStatement(sqlText);
                pstmt.setLong(1, dbId);
                pstmt.setLong(2, logIds[i]);
                pstmt.setString(3, usgLog[i].getLogMessage());
                pstmt.setLong(4, usgLog[i].getLogTime());
                pstmt.execute();
                pstmt.close();
            }

        } catch (SQLException e) {
            try {
                conn.rollback();
            } catch (SQLException e1) {
                e1.printStackTrace();
            }
            error = new ObjectIOException(e.getMessage());
        } catch (ObjectIOException e) {
            error = e;
        }
        if (conn != null) {
            try {
                conn.setAutoCommit(true);
            } catch (SQLException e1) {
                e1.printStackTrace();
            }
            ConnectionPool.getPool().releaseConnection(conn);
        }
        if (error != null)
            throw error;
    }

    public static void write(USDReadmeDatails details, long runId)
            throws ObjectIOException {
        String suffix = BusinessObject.getSuffix(ObservationBlock.class);
        String url = Config.getCfg().getUrl(suffix);
        String user = Config.getCfg().getDbUserName();
        String pwd = Config.getCfg().getDbPassword();
        ObjectIOException error = null;
        stdlog_.debug("The url: \"" + url + "\" the user \"" + user + "\" ");
        Connection conn = null;
        try {
            conn = ConnectionPool.getPool().acquireConnection(url, user, pwd);
            write(conn, details, runId);

        } catch (SQLException e) {
            try {
                conn.rollback();
            } catch (SQLException e1) {
                e1.printStackTrace();
            }
            error = new ObjectIOException(e.getMessage());
        }
        if (conn != null) {
            try {
                conn.setAutoCommit(true);
            } catch (SQLException e1) {
                e1.printStackTrace();
            }
            ConnectionPool.getPool().releaseConnection(conn);
        }
        if (error != null)
            throw error;

    }

    public static USDReadmeDatails read(long runId) throws ObjectIOException {
        String suffix = BusinessObject.getSuffix(ObservationBlock.class);
        String url = Config.getCfg().getUrl(suffix);
        String user = Config.getCfg().getDbUserName();
        String pwd = Config.getCfg().getDbPassword();
        ObjectIOException error = null;
        stdlog_.debug("The url: \"" + url + "\" the user \"" + user + "\" ");
        Connection conn = null;
        USDReadmeDatails details = null;
        long dbId = Config.getCfg().uniqueToTableId(runId);
        try {
            conn = ConnectionPool.getPool().acquireConnection(url, user, pwd);
            details = read(conn, dbId);

        } catch (SQLException e) {
            try {
                conn.rollback();
            } catch (SQLException e1) {
                e1.printStackTrace();
            }
            error = new ObjectIOException(e.getMessage());
        }
        if (conn != null) {
            try {
                conn.setAutoCommit(true);
            } catch (SQLException e1) {
                e1.printStackTrace();
            }
            ConnectionPool.getPool().releaseConnection(conn);
        }
        if (error != null)
            throw error;
        return details;

    }

    public static ConstraintSetDTO[] getConstraintSet(Connection conn,
            long runId) throws SQLException {
        String sql = "select DISTINCT " + CS + ".seeing," + CS + ".airmass,"
                + CS + ".moon_distance," + CS + ".sky_transparency," + CS
                + ".fli," + "count(*) AS OBCOUNT" + " from " + OB + "," + CS
                + " where run_id = ? " + " AND " + OB + ".ob_id=" + CS
                + ".cs_id" + " group by " + CS + ".seeing," + CS + ".airmass,"
                + CS + ".moon_distance," + CS + ".sky_transparency," + CS
                + ".fli" + " order by OBCOUNT desc";
        stdlog_.debug(sql);
        PreparedStatement pstmt = conn.prepareStatement(sql);
        pstmt.setLong(1, runId);
        ResultSet rs = pstmt.executeQuery();
        Vector<ConstraintSetDTO> vec = new Vector<ConstraintSetDTO>();
        while (rs.next()) {
            ConstraintSetDTO cs = new ConstraintSetDTO();
            cs.setSeeing(rs.getFloat("seeing"));
            cs.setSkyTransparency(rs.getString("sky_transparency"));
            cs.setAirmass(rs.getFloat("airmass"));
            cs.setFractionalLunarIllumination(rs.getFloat("fli"));
            cs.setMoonAngularDistance(rs.getInt("moon_distance") / 3600000);
            cs.setCount(rs.getInt("OBCOUNT"));
            vec.addElement(cs);
        }
        ConstraintSetDTO[] csSet = new ConstraintSetDTO[vec.size()];
        vec.copyInto(csSet);
        return csSet;

    }

    public static String getUSDStaffName(Connection conn, long dbRunId)
            throws SQLException {

        String sql = "SELECT " + obrepDbName + "..usg_staff.firstname, "
                + obrepDbName + "..usg_staff.lastname " + " from "
                + obrepDbName + "..run_states," + obrepDbName + "..usg_staff"
                + " WHERE " + obrepDbName + "..run_states.run_id = ?" + " AND "
                + obrepDbName + "..run_states.cs_acronym" + " = " + obrepDbName
                + "..usg_staff.cs_acronym";
        stdlog_.debug(sql);
        PreparedStatement pstmt = conn.prepareStatement(sql);
        pstmt.setLong(1, dbRunId);
        ResultSet rs = pstmt.executeQuery();
        String firstname = null;
        String secondname = null;
        boolean found = false;
        while (rs.next()) {
            firstname = rs.getString(1);
            secondname = rs.getString(2);
            found = true;
        }
        rs.close();
        pstmt.close();
        if (found)
            return firstname + " " + secondname;
        return "N/A";
    }

    public static boolean isChileanFlag(Connection conn, long dbRunId)
            throws SQLException {
        String sqlText = " SELECT " + user_affialiations + ".country_code"
                + " FROM " + user_affialiations + "," + obs_runs + ","
                + proposed + " WHERE " + obs_runs + ".programme_id ="
                + proposed + ".programme_id" + " AND " + proposed
                + ".pi_flag=0" + " AND " + proposed + ".user_id ="
                + user_affialiations + ".user_id" + " AND " + obs_runs
                + ".id = ?";
        PreparedStatement pstmt = conn.prepareStatement(sqlText);
        pstmt.setLong(1, dbRunId);
        ResultSet rs = pstmt.executeQuery();
        String flag = null;
        while (rs.next()) {
            flag = rs.getString(1);
        }
        rs.close();
        pstmt.close();
        if (flag != null && flag.equals("RCH"))
            return true;
        return false;
    }

    private static void setTextSize(Connection conn) throws SQLException {
        Statement stmt = conn.createStatement();
        stmt.execute("set TEXTSIZE " + 1000000);
        stmt.close();
    }

    //    if (count > 0) {
    //        sqlText = "UPDATE " + usgFileReplicate + " set "
    //                + usgFileReplicate + ".review_status = ?, "
    //                + usgFileReplicate + ".special_remarks = ? , "
    //                + usgFileReplicate + ".mts_abstract = ? , "
    //                + usgFileReplicate + ".pre_imaging = ?, "
    //                + usgFileReplicate + ".special_cal= ? ," + USG_FILE
    //                + ".time_criti = ? ," + usgFileReplicate
    //                + ".carryover_not_derived = ? " + " where "
    //                + usgFileReplicate + ".run_id = ?";
    //        stdlog_.debug(sqlText);
    //        pstmt = conn.prepareStatement(sqlText);
    //        pstmt.setString(1, usdDetails.getReviewStatus());
    //        pstmt.setString(2, usdDetails.getSpecialComments());
    //        pstmt.setString(3, usdDetails.getMtsAbstract());
    //        pstmt.setBoolean(4, usdDetails.isPreImaging());
    //        pstmt.setBoolean(5, usdDetails.isSpecialCal());
    //        pstmt.setBoolean(6, usdDetails.isTimeCritical());
    //        pstmt.setBoolean(7, usdDetails.isCarryOver());
    //        pstmt.setLong(8, dbId);
    //        pstmt.execute();
    //        pstmt.close();
    //    } else {
    //        sqlText = "INSERT INTO " + usgFileReplicate
    //                + " (run_id,review_status,special_remarks, "
    //                + "mts_abstract,pre_imaging,special_cal,"
    //                + "time_criti,carryover_not_derived) VALUES"
    //                + " (?,?,?,?,?,?,?,?)";
    //        stdlog_.debug(sqlText);
    //        pstmt = conn.prepareStatement(sqlText);
    //        pstmt.setLong(1, dbId);
    //        pstmt.setString(2, usdDetails.getReviewStatus());
    //        pstmt.setString(3, usdDetails.getSpecialComments());
    //        pstmt.setString(4, usdDetails.getMtsAbstract());
    //        pstmt.setBoolean(5, usdDetails.isPreImaging());
    //        pstmt.setBoolean(6, usdDetails.isSpecialCal());
    //        pstmt.setBoolean(7, usdDetails.isTimeCritical());
    //        pstmt.setBoolean(8, usdDetails.isCarryOver());
    //        pstmt.execute();
    //        pstmt.close();
    //
    //    }

}

