package org.eso.ohs.phase2.apps.ipacman.actions;

import java.awt.event.ActionEvent;

import javax.swing.JOptionPane;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.client.DbbView;
import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.persistence.dbase.phase2.InstrumentPackageDBIO;
import org.eso.ohs.phase2.apps.ipacman.CheckObsRuns;
import org.eso.ohs.phase2.apps.ipacman.DbWin;

/**
 * Action class to delete a business object in the chosen directory.
 * <p>
 * $Id: DeleteIPAction.java,v 1.11 2005/03/30 09:56:13 tcanavan Exp $
 * @version $Revision: 1.11 $
 * @author Ron J Lobo
 */
public class DeleteIPAction extends ActionSuperclass {

    private static Logger stdlog_ = Logger.getLogger(DeleteIPAction.class);

    private DbbView view_;
    private DbWin dbWin_;


    /**
     * Constructs a delete action.
     *
     * @param device      medium containing the parent directory.
     * @param folderView  GUI view of (summary information in) the folder.
     */
    public DeleteIPAction(DbWin dbWin, DbbView view) {
        super(view, "Delete IP");
        dbWin_ = dbWin;
        view_ = view;
    }


    /**
     * Event handler for the delete  action.
     *
     * @param ev    user-initiated ActionEvent to delete an OD.
     */
    public void actionPerformedImpl(ActionEvent ae) {

        int rows[] = view_.getSelectedRows();
        if (rows.length != 1) {
            JOptionPane.showMessageDialog(view_.getTopLevelAncestor(),
                "Please select one IP to perform this operation",
                "Selection Error", JOptionPane.ERROR_MESSAGE);
            return;
        }

        String instr = view_.getValueAt(rows[0], 0).toString();
        String versionStr = view_.getValueAt(rows[0], 1).toString();
        float version = Float.valueOf(versionStr).floatValue();
        stdlog_.debug("instrument=" + instr + ", version=" + versionStr);

        int result = JOptionPane.showConfirmDialog(view_,
            "Are you sure you want to delete this\n" +
            "Instrument Packages from the database?",
            "Confirm Delete",
            JOptionPane.YES_NO_OPTION);

        if (result == JOptionPane.YES_OPTION) {
            if (InstrumentPackageDBIO.deleteIP(instr, version)) {
                JOptionPane.showMessageDialog(view_,
                    "The Instrument Package was successfully deleted \n" +
                    DbWin.click4ObsRuns, "Delete Successful",
                    JOptionPane.INFORMATION_MESSAGE);
                stdlog_.debug("IP deleted successfully");

                dbWin_.resetDbbView(true);

                try {
                    new CheckObsRuns(instr, version, null, view_);
                } catch (java.sql.SQLException e) {
                    stdlog_.error("DB error: " + e.getMessage());
                    JOptionPane.showMessageDialog(view_, DbWin.DBerror,
                        "Database Error", JOptionPane.ERROR_MESSAGE);
                }
            } else {
                String mesg = "Deletion of Instrument Package failed";
                stdlog_.error(mesg);
                JOptionPane.showMessageDialog(view_, mesg,
                    "Delete Failed", JOptionPane.ERROR_MESSAGE);
            }
        }
    }
}

