package org.eso.ohs.phase2.apps.ipacman.actions;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.IOException;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.KeyStroke;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.client.DbbView;
import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.widgets.ExtensionFileFilter;
import org.eso.ohs.core.gui.widgets.SwingFileChooser;
import org.eso.ohs.core.utilities.FileIOUtils;
import org.eso.ohs.instruments.ZipUtils;
import org.eso.ohs.instruments.ZipUtilsException;
import org.eso.ohs.persistence.dbase.phase2.InstrumentPackageDBIO;
import org.eso.ohs.phase2.apps.ipacman.DbWin;
import org.eso.ohs.phase2.apps.ipacman.DiffIPDialog;

/**
 * Action class to compare 2 IP packages and display differences in a JTree
 * <p>
 * $Id: DiffIPAction.java,v 1.17 2005/03/30 09:56:13 tcanavan Exp $
 * @version $Revision: 1.17 $
 * @author Ron J Lobo
 */
public class DiffIPAction extends ActionSuperclass {

    private static Logger stdlog_ = Logger.getLogger(DiffIPAction.class);

    private DbWin dbwin_;
    private DbbView view_;

    // gui components
    private JDialog diffIPdlg_;
    private JButton loadButton_[];
    private JTextField ipFileName_[];
    private JButton okButton_;
    private JButton cancelButton_;

    private boolean ipSelected_[];
    private boolean dbChoice_[];
    private String instr_[];
    private String version_[];


    /**
     * Constructor
     */
    public DiffIPAction(DbWin dbwin, DbbView view) {
        super((JComponent)view, "IP Diff");
        dbwin_ = dbwin;
        view_ = view;

        loadButton_ = new JButton[2];
        ipFileName_ = new JTextField[2];
        ipSelected_ = new boolean[2];
        dbChoice_ = new boolean[2];
        instr_ = new String[2];
        version_ = new String[2];
    }


    /**
     * Event handler for the DiffIPAction action
     *
     * @param ev User-initiated ActionEvent to add an IP
     */
    public void actionPerformedImpl(ActionEvent ae) {

        int rows[] = view_.getSelectedRows();
        stdlog_.debug("rows selected = " + rows.length);

        if (rows.length > 2) {
            errorMsg("Selection Error", "Too many rows selected\n" +
                "Please select 0, 1 or 2 rows for this operation");
            return;
        }

        // create the components
        if (diffIPdlg_ != null) {
            diffIPdlg_.dispose();
        }
        diffIPdlg_ = new JDialog(dbwin_, "Show Instrument Package Differences", true);
        JPanel ipPanel = new JPanel();
        ipPanel.setLayout(new BoxLayout(ipPanel, BoxLayout.Y_AXIS));
        ipPanel.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));

        for (int i=0; i<2; i++) {
            if (rows.length <= i) {
                dbChoice_[i] = ipSelected_[i] = false;
                ipPanel.add(createSelectIPPanel(i));
            } else {
                dbChoice_[i] = ipSelected_[i] = true;
                ipPanel.add(createShowChoicePanel(rows, i));
            }
        }

        diffIPdlg_.getContentPane().add(ipPanel, BorderLayout.CENTER);
        diffIPdlg_.getContentPane().add(createButtonPanel(), BorderLayout.SOUTH);

        ipPanel.registerKeyboardAction(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                cancelButton_.doClick(200);
            }},
            KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0),
            JComponent.WHEN_IN_FOCUSED_WINDOW
        );

        Dimension win = diffIPdlg_.getPreferredSize();
        Dimension s = Toolkit.getDefaultToolkit().getScreenSize();
        diffIPdlg_.setLocation((s.width-win.width)/3, (s.height-win.height)/3);
        diffIPdlg_.getRootPane().setDefaultButton(okButton_);
        diffIPdlg_.pack();
        diffIPdlg_.setVisible(true);

        // set the focus button
        if (rows.length == 2) {
            okButton_.requestFocusInWindow();
        } else {
            loadButton_[rows.length].requestFocusInWindow();
        }
    }


    private JPanel createSelectIPPanel(final int zipNum) {

        loadButton_[zipNum] = new JButton();
        loadButton_[zipNum].setText("Select IP" + (zipNum+1));
        loadButton_[zipNum].addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                loadIP(zipNum);
            }
        });

        ipFileName_[zipNum] = new JTextField();
        ipFileName_[zipNum].setColumns(60);
        ipFileName_[zipNum].setEditable(false);
        ipFileName_[zipNum].setFocusable(false);
        ipFileName_[zipNum].setBackground(Color.lightGray);

        JPanel p = new JPanel();
        p.add(loadButton_[zipNum]);
        p.add(ipFileName_[zipNum]);
        return p;
    }


    private JPanel createShowChoicePanel(int rows[], int panelNum) {
        instr_[panelNum] = view_.getValueAt(rows[panelNum], 0).toString();
        version_[panelNum] = view_.getValueAt(rows[panelNum], 1).toString();
        JLabel txt = new JLabel("   IP" + (panelNum+1) + ": Database entry  " +
            instr_[panelNum] + " v" + version_[panelNum] + "       ");
        JPanel p;
        if (dbChoice_[0] && dbChoice_[1]) {
            p = new JPanel();
        } else {
            p = new JPanel(new FlowLayout(FlowLayout.LEFT));
        }
        p.add(txt);
        return p;
    }


    private void loadIP(int zipNum) {
        stdlog_.debug("loadIP " + zipNum);

        SwingFileChooser fileChooser = new SwingFileChooser();
        fileChooser.setDialogTitle("Select Instrument Package:");
        fileChooser.setCurrentDirectory(DbWin.dir_);
        fileChooser.setFileFilter(new ExtensionFileFilter("zip", "Zip Files"));

        int response = fileChooser.showOpenDialog(diffIPdlg_);
        File zipFile = fileChooser.getSelectedFile();
        DbWin.dir_ = fileChooser.getCurrentDirectory();

        if (response != SwingFileChooser.APPROVE_OPTION || zipFile == null) {
            return;
        }

        String pathName = FileIOUtils.getCanonicalPath(zipFile);
        stdlog_.debug("got file: " + pathName);

        if (DbWin.checkIPfileName(zipFile.getName())) {
            ipSelected_[zipNum] = true;
            ipFileName_[zipNum].setText(pathName);
        } else {
            ipSelected_[zipNum] = false;
            ipFileName_[zipNum].setText("");
            errorMsg("Illegal Filename", DbWin.illegalFilename_);
        }

        okButton_.setEnabled(ipSelected_[0] && ipSelected_[1]);
    }


    /** Setup the button panel */
    private JPanel createButtonPanel() {

        okButton_ = new JButton();
        okButton_.setText("Ok");
        okButton_.setEnabled(ipSelected_[0] && ipSelected_[1]);
        okButton_.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                okBtnClick();
            }
        });

        cancelButton_ = new JButton();
        cancelButton_.setText("Cancel");
        cancelButton_.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                stdlog_.debug("cancelBtnClick");
                diffIPdlg_.dispose();
            }
        });

        JPanel buttonPanel = new JPanel();
        buttonPanel.add(okButton_);
        buttonPanel.add(cancelButton_);
        return buttonPanel;
    }


    private void okBtnClick() {

        stdlog_.debug("okBtnClick");
        diffIPdlg_.dispose();

        DiffIPDialog diffIPdlg = new DiffIPDialog(dbwin_, view_);
        view_.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

        // get the two files (from filesystem or from database)
        String files[] = new String[2];
        File tmpDir = ZipUtils.makeNewTmpDir();
        try {
            for (int i=0; i<2; i++) {
                if (dbChoice_[i]) {
                    float ver = Float.parseFloat(version_[i]);
                    files[i] = tmpDir + File.separator +
                        InstrumentPackageDBIO.getIP(
                            instr_[i], ver, tmpDir.toString(), false);
                } else {
                    files[i] = ipFileName_[i].getText();
                }
                stdlog_.debug("file " + i + " = " + files[i]);
            }

            diffIPdlg.doDiff(files, dbChoice_);

        } catch (IOException e) {
            getIPerror(e);
        } catch (java.sql.SQLException e) {
            getIPerror(e);
        } catch (ZipUtilsException e) {
            getIPerror(e);
        }

        // clean up tmp files & dirs
        for (int i=0; i<2; i++) {
            if (dbChoice_[i]) {
                stdlog_.debug("deleting: " + files[i]);
                (new File(files[i])).delete();
            }
        }
        stdlog_.debug("deleting: " + tmpDir);
        tmpDir.delete();

        view_.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    }


    private void getIPerror(Exception e) {
        stdlog_.error("Error in OkBtnClick()" + e.getMessage());
        errorMsg("error", "Error occured while comparing IPs\n" + e.getMessage());
    }


    /**
     * display an error message for the user
     *
     * @param title title of message dialog
     * @param mesg  message to display
     */
    private void errorMsg(String title, String mesg) {
        view_.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        final int errMsg = JOptionPane.ERROR_MESSAGE;
        JOptionPane.showMessageDialog(view_, mesg, title, errMsg);
        stdlog_.error(mesg);
    }
}

