package org.eso.ohs.phase2.apps.ipacman.actions;

import java.awt.Cursor;
import java.awt.event.ActionEvent;
import java.io.File;

import javax.swing.JOptionPane;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.client.DbbView;
import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.widgets.SwingFileChooser;
import org.eso.ohs.core.utilities.FileIOUtils;
import org.eso.ohs.persistence.dbase.phase2.InstrumentPackageDBIO;
import org.eso.ohs.phase2.apps.ipacman.DbWin;

/**
 * Action class to extract an IP from the database into the chosen directory
 * <p>
 * $Id: ExtractIPAction.java,v 1.13 2005/03/30 09:56:13 tcanavan Exp $
 * @version $Revision: 1.13 $
 * @author Ron J Lobo
 */
public class ExtractIPAction extends ActionSuperclass {

    /** Standard log channel */
    private static Logger stdlog_ = Logger.getLogger(ExtractIPAction.class);

    DbbView view_;


    /**
     * Constructs an extract action
     *
     * @param device      medium containing the parent directory
     * @param folderView  GUI view of (summary information in) the folder
     */
    public ExtractIPAction(DbbView view) {
        super(view, "Extract IP");
        view_ = view;
    }


    /**
     * Event handler for the create BusinessObject action
     *
     * @param ev       user-initiated ActionEvent to create an OD
     */
    public void actionPerformedImpl(ActionEvent ae) {

        // restrict user to one extract at a time
        int rows[] = view_.getSelectedRows();
        if (rows.length != 1) {
            err("Please select one IP to perform this operation", "Selection Error");
            return;
        }
        String instr = view_.getValueAt(rows[0], 0).toString();
        String versionStr = view_.getValueAt(rows[0], 1).toString();
        float version = Float.valueOf(versionStr).floatValue();
        stdlog_.debug("instrument=" + instr + ", version=" + versionStr);

        SwingFileChooser fileChooser = new SwingFileChooser();
        fileChooser.setFileSelectionMode(SwingFileChooser.DIRECTORIES_ONLY);
        fileChooser.setCurrentDirectory(DbWin.dir_);
        int response = fileChooser.showDialog(view_, "Select Directory");
        if (response != SwingFileChooser.APPROVE_OPTION) {
            return;
        }

        DbWin.dir_ = fileChooser.getCurrentDirectory();
        File zipFile = fileChooser.getSelectedFile();
        String path = FileIOUtils.getCanonicalPath(zipFile);
        stdlog_.debug("extract to: " + path);

        if (!zipFile.canWrite()) {
            err("No write permission for this directory\n"+path, "Can't write");
            return;
        }

        try {
            stdlog_.debug("read & unzip: " + instr + ", v" + version);
            view_.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
            InstrumentPackageDBIO.getIP(instr, version, path, false);
            String mesg = "Successful Extract for: " + instr + " v" + version;
            stdlog_.debug(mesg);
            JOptionPane.showMessageDialog(view_.getTopLevelAncestor(),
                mesg, "Success", JOptionPane.INFORMATION_MESSAGE);
        } catch (Exception e) {
            err("Error UnZipping file\n" + e.getMessage(), "UnZip error");
        }
        view_.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    }


    /**
     * convenience method to log an error and popup an error dialog
     */
    private void err(String mesg, String title) {
        stdlog_.debug(mesg);
        JOptionPane.showMessageDialog(view_.getTopLevelAncestor(),
            mesg, title, JOptionPane.ERROR_MESSAGE);
    }
}

