package org.eso.ohs.phase2.apps.ipacman.actions;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.sql.SQLException;
import java.util.Vector;

import javax.swing.JComponent;
import javax.swing.JOptionPane;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.client.DbbView;
import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.gui.widgets.TextDisplayWidget;
import org.eso.ohs.persistence.dbase.phase2.AffectedObsRun;
import org.eso.ohs.persistence.dbase.phase2.InstrumentPackageDBIO;
import org.eso.ohs.phase2.apps.ipacman.DbWin;

/**
 * Action class to show list of downloadable Obs Runs
 * that are missing their IPs
 * <p>
 * $Id: MissingIPsAction.java,v 1.13 2005/03/30 09:56:13 tcanavan Exp $
 * @version $Revision: 1.13 $
 * @author Ron J Lobo
 */
public class MissingIPsAction extends ActionSuperclass {

    private static Logger stdlog_ = Logger.getLogger(MissingIPsAction.class);

    private DbWin dbwin_;
    private DbbView view_;

    // names of instruments that are ignored in the missing IP report
    private Vector ipsToIgnore_;


    /**
     * Constructor
     */
    public MissingIPsAction(DbWin dbwin, DbbView view) {
        super((JComponent)view, "Missing IPs");
        dbwin_ = dbwin;
        view_ = view;
    }

    
    /**
     * Constructor
     */
    public MissingIPsAction(DbWin dbwin, DbbView view, Vector ipsToIgnore) {
        super((JComponent)view, "Missing IPs");
        dbwin_ = dbwin;
        view_ = view;
        ipsToIgnore_ = ipsToIgnore;
    }    


    /**
     * Event handler for the MissingIPsAction action
     *
     * @param ev User-initiated ActionEvent
     */
    public void actionPerformedImpl(ActionEvent ae) {

        // stdlog_.debug("MissingIPsAction.actionPerf");
        Vector missingIPs;
        Vector obsRunsMissingIPs;
        try {
            missingIPs = InstrumentPackageDBIO.getMissingIPs(true, ipsToIgnore_);
            obsRunsMissingIPs = InstrumentPackageDBIO.getMissingIPs(false, ipsToIgnore_);
        } catch (SQLException e) {
            JOptionPane.showMessageDialog(view_, DbWin.DBerror,
                "Database Error", JOptionPane.ERROR_MESSAGE);
            return;
        }

        int numMissingIPs = missingIPs.size();
        int numORMIPs = obsRunsMissingIPs.size();
        if (numMissingIPs == 0 && numORMIPs == 0) {
            String msg = "No Records Found";
            String instList = getIPIgnoreListAsString(6);
            if (instList.length() > 0) {
                msg = msg+"\n"+"Note: "+ipsToIgnore_.size()+" instruments were ignored.";
            }
            JOptionPane.showMessageDialog(view_,
                "No Missing IPs and no Observing Runs without IPs were found",
                msg, JOptionPane.INFORMATION_MESSAGE);
            return;
        }

        final int max = 500;
        String missingIPsStr = numMissingIPs + " IPs missing\n";
        String ORMIPsStr = numORMIPs + " Observing Runs with Missing IPs";

        if (numORMIPs > max) {
            ORMIPsStr += "\n(Displaying the first " + max + " records only) ";
            JOptionPane.showMessageDialog(view_, missingIPsStr + ORMIPsStr,
                "Truncating Output", JOptionPane.WARNING_MESSAGE);
        }

        StringBuffer report = new StringBuffer();
        String sp = "                              ";
        if (numMissingIPs > 0 && numORMIPs > 0) {
            report.append("\nSummary\n=======\n\n" + missingIPsStr + ORMIPsStr + "\n");
        }
        String instList = getIPIgnoreListAsString(6);
        if (instList.length() > 0) {
            report.append("\nNote: "+ipsToIgnore_.size()+" instruments were ignored.\n"
            +"The ignored instruments are listed at the end of the report.\n");
        }
        report.append("\n");
        if (numMissingIPs > 0) {
            report.append("\nMissing Instrument Packages\n" +
                "===========================\n\n" +
                "Instrument               IP Version\n" +
                "----------               ----------\n");
            for (int i=0; i < numMissingIPs; i++) {
                AffectedObsRun obsRun = (AffectedObsRun)missingIPs.elementAt(i);
                if (obsRun.ipVersion_ == 0 && i > 0) {
                    AffectedObsRun obsRunPrev = (AffectedObsRun)missingIPs.elementAt(i-1);
                    if (obsRun.instrument_.equals(obsRunPrev.instrument_) &&
                            obsRunPrev.ipVersion_ < 0) {
                        stdlog_.debug("ignoring duplicate " + obsRun.instrument_ + " v0.0");
                        continue;  // skip repetition
                    }
                }
                report.append((obsRun.instrument_ + sp).substring(0,25) +
                    (obsRun.ipVersion_ <= 0 ? "0.00\n" : obsRun.ipVersion_ + "\n"));
            }
            report.append("\n\n");
        }

        if (numORMIPs > 0) {
            report.append("\nDownloadable Observing Runs with Missing IPs\n" +
                "============================================\n\n" +
                "Observing Run      Instrument      Period      IP Version\n" +
                "-------------      ----------      ------      ----------\n");
            numORMIPs = numORMIPs > max ? max : numORMIPs;
            for (int i=0; i < numORMIPs; i++) {
                AffectedObsRun obsRun = (AffectedObsRun)obsRunsMissingIPs.elementAt(i);
                String rep = (obsRun.obsRunName_ + sp).substring(0,19) +
                    (obsRun.instrument_ + sp).substring(0,16) +
                    (obsRun.period_ + sp).substring(0,12) +
                    (obsRun.ipVersion_ <= 0 ? "0.00\n" : obsRun.ipVersion_ + "\n");
                report.append(rep);
            }
        }
        if (instList.length() > 0) {
            report.append("\nIgnored Instruments (read from site.cf)"
                         +"\n=======================================\n"+instList+"\n");
        }

        TextDisplayWidget txtDisplay = new TextDisplayWidget("Missing IPs Report");
        txtDisplay.clearTextArea();
        txtDisplay.setTextArea(report.toString());
        txtDisplay.setSize(new Dimension(450, 600));
        txtDisplay.setLocation(300, 200);
        txtDisplay.setVisible(true);
    }
    
    /**
     * @return String comma separated listing of the ipsToIgnore_ list
     * or "" if empty. 
     * @param instPerLine int The returned String will contain a "\n" after 
     * every instPerLine instrument.
     */
    private String getIPIgnoreListAsString(int instPerLine) {
        StringBuffer rc = new StringBuffer();
        if (ipsToIgnore_ != null && ipsToIgnore_.size() > 0) {
            for (int i = 0; i < ipsToIgnore_.size(); i++) {
                if (rc.length() > 0) {
                    rc.append(", ");
                }
                if ((i+1)%instPerLine == 0) {
                    rc.append("\n");
                }
                rc.append((String)ipsToIgnore_.elementAt(i));

            }
        }
        return rc.toString();
    }
}

