package org.eso.ohs.phase2.apps.ipacman.actions;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.sql.SQLException;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.KeyStroke;
import javax.swing.border.Border;

import org.apache.log4j.Logger;
import org.eso.ohs.core.dbb.client.DbbView;
import org.eso.ohs.core.gui.baseaction.ActionSuperclass;
import org.eso.ohs.core.utilities.Period;
import org.eso.ohs.persistence.dbase.phase2.InstrumentPackageDBIO;
import org.eso.ohs.phase2.apps.ipacman.CheckObsRuns;
import org.eso.ohs.phase2.apps.ipacman.DbWin;
import org.eso.ohs.phase2.apps.ipacman.IPACMAN;
import org.eso.ohs.phase2.apps.ipacman.IPacManException;

/**
 * Action class to enable selection of Observing Runs to view/edit
 * <p>
 * $Id: ViewObsRunsIPAction.java,v 1.13 2005/05/06 14:12:15 tcanavan Exp $
 * @version $Revision: 1.13 $
 * @author Ron J Lobo
 */
public class ViewObsRunsIPAction extends ActionSuperclass {

    private static Logger stdlog_ =
        Logger.getLogger(ViewObsRunsIPAction.class);

    private DbWin dbWin_;
    private DbbView view_;

    // gui components
    private JDialog dlg_;
    private JComboBox instrCB_;
    private JComboBox verCB_;
    private JList periodList_;
    private JButton okButton_;
    private JButton cancelButton_;

    private final String clearStr = "";
    private final String unassignedStr = "unassigned";

    private final float anyVersion = InstrumentPackageDBIO.anyVersion;
    private final float unassignedVersion = InstrumentPackageDBIO.unassignedVersion;


    /**
     * Constructor
     */
    public ViewObsRunsIPAction(DbWin dbwin, DbbView view) {
        super((JComponent)view, "Observing Runs");
        dbWin_ = dbwin;
        view_ = view;
    }


    /**
     * Event handler for the ViewObsRunsIPAction action
     *
     * @param ev User-initiated ActionEvent to add an IP
     */
    public void actionPerformedImpl(ActionEvent ae) {

        try {
            JPanel panel1 = createInstrAndVersionPanel();
            JPanel panel2 = createPeriodsPanel();
            JPanel panel3 = createButtonPanel();

            Border padding = BorderFactory.createEmptyBorder(5,5,0,5);
            panel1.setBorder(padding);
            panel2.setBorder(padding);

            panel1.registerKeyboardAction(new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    cancelButton_.doClick(200);
                }},
                KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0),
                JComponent.WHEN_IN_FOCUSED_WINDOW
            );

            // create the dialog
            if (dlg_ != null) {
                dlg_.dispose();
            }
            dlg_ = new JDialog(dbWin_, true);
            dlg_.getContentPane().add(panel1, BorderLayout.NORTH);
            dlg_.getContentPane().add(panel2, BorderLayout.CENTER);
            dlg_.getContentPane().add(panel3, BorderLayout.SOUTH);

            Dimension win = dlg_.getPreferredSize();
            Dimension scrn = Toolkit.getDefaultToolkit().getScreenSize();
            dlg_.setLocation((scrn.width-win.width)/3, (scrn.height-win.height)/3);
            dlg_.setTitle("View Observing Runs");
            dlg_.getRootPane().setDefaultButton(okButton_);
            dlg_.pack();
            dlg_.setVisible(true);
            okButton_.requestFocusInWindow();

        } catch (IPacManException e) {
            stdlog_.error("ViewObsRuns error");
        }
    }


    private JPanel createInstrAndVersionPanel() throws IPacManException {

        int rows[] = view_.getSelectedRows();
        // stdlog_.debug("rows selected = " + rows.length);

        // create the top part of the dialog
        try {
            String[] instruments = InstrumentPackageDBIO.getInstruments();
            instrCB_ = new JComboBox(instruments);
            if (rows.length > 0) {
                String instr = view_.getValueAt(rows[0], 0).toString();
                for (int i=0; i < instruments.length; i++) {
                    if (instr.equals(instruments[i])) {
                        instrCB_.setSelectedIndex(i);
                        break;
                    }
                }
            }
            instrCB_.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    try {
                        loadIPversions(clearStr);
                    } catch (IPacManException eIPM) {
                        stdlog_.error("Error setting version list");
                    }
                }
            });
        } catch (SQLException e) {
            String mesg = "Failed to read instrument list from database.\n";
            errorMsg("Database Error", mesg + DbWin.DBerror, e);
            throw new IPacManException(e);
        }

        verCB_ = new JComboBox();
        String version = clearStr;
        if (rows.length > 0) {
            version = view_.getValueAt(rows[0], 1).toString();
        }
        loadIPversions(version);

        JLabel instrLbl = new JLabel("Instrument");
        JLabel versionLbl = new JLabel("Version");

        JPanel panel = new JPanel();
        panel.setLayout(new GridLayout(0,2,0,5));
        panel.add(instrLbl);
        panel.add(instrCB_);
        panel.add(versionLbl);
        panel.add(verCB_);
        return panel;
    }


    private void loadIPversions(String selection) throws IPacManException {
        try {
            verCB_.removeAllItems();
            verCB_.addItem(clearStr);
            verCB_.addItem(unassignedStr);
            String instr = (String)instrCB_.getSelectedItem();
            Vector v = InstrumentPackageDBIO.getAllIPversions(instr);
            for (int i=0; i < v.size(); i++) {
                String s = v.elementAt(i) + "00";
                int pos = s.indexOf('.');
                s = s.substring(0, pos+3);
                verCB_.addItem(s);
                if (selection.equals(s)) {
                    verCB_.setSelectedItem(s);
                }
            }
        } catch (SQLException e) {
            String mesg = "Failed to read IP versions from database.\n";
            errorMsg("Database Error", mesg + DbWin.DBerror, e);
            throw new IPacManException(e);
        }
    }


    private JPanel createPeriodsPanel() {

        JLabel periodLbl = new JLabel("Period   ");
        int lastPeriod = Period.getCurrentPeriod() + Period.ADDITIONAL;
        String[] periodArr = new String[lastPeriod-IPACMAN.firstMipsPeriod+2];
        periodArr[0] = ""+IPACMAN.EngObsRunsPeriod;
        for (int start=1, i=IPACMAN.firstMipsPeriod; i<=lastPeriod; i++) {
            periodArr[start++] = "" + i;
        }
        periodList_ = new JList(periodArr);
        JScrollPane periodSP = new JScrollPane(periodList_);
        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());
        panel.add(periodLbl, BorderLayout.WEST);
        panel.add(periodSP, BorderLayout.CENTER);
        return panel;
    }


    private JPanel createButtonPanel() {

        okButton_ = new JButton();
        okButton_.setText(" Ok ");
        okButton_.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                okBtnClick();
            }
        });

        cancelButton_ = new JButton();
        cancelButton_.setText("Cancel");
        cancelButton_.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                dlg_.dispose();
            }
        });

        JPanel buttonPanel = new JPanel();
        buttonPanel.add(okButton_);
        buttonPanel.add(cancelButton_);
        return buttonPanel;
    }


    private void okBtnClick() {

        String instr = (String)instrCB_.getSelectedItem();
        String version = (String)verCB_.getSelectedItem();
        // stdlog_.debug("version=" + version);
        float verF;
        if (version.equals(clearStr)) {
            verF = anyVersion;
        } else if (version.equals(unassignedStr)) {
            verF = unassignedVersion;
        } else {
            verF = Float.parseFloat(version);
        }
        // stdlog_.debug("verF=" + verF);

        Object[] newPeriodsA = periodList_.getSelectedValues();
        // stdlog_.debug("#newPeriods=" + newPeriodsA.length);
        int[] periods = null;
        if (newPeriodsA.length > 0) {
            periods = new int[newPeriodsA.length];
            for (int i=0; i < newPeriodsA.length; i++) {
                String s = (String)newPeriodsA[i];
                // stdlog_.debug(" add: " + s);
                periods[i] = Integer.parseInt(s);
            }
        }

        try {
            dlg_.dispose();
            new CheckObsRuns(instr, verF, periods, view_);
        } catch (SQLException e) {
            stdlog_.error("DB error: " + e.getMessage());
            errorMsg("Database Error", DbWin.DBerror, e);
        }
    }


    /**
     * display an error message for the user
     *
     * @param title title of message dialog
     * @param mesg  message to display
     */
    private void errorMsg(String title, String mesg, Exception e) {
        JOptionPane.showMessageDialog(dlg_, mesg, title, JOptionPane.ERROR_MESSAGE);
        stdlog_.error(mesg);
        stdlog_.error(e.getMessage());
    }
}

